//LoadRegionFromFile.cpp - Body of associated class.

#include "LoadRegionFromFile.h"
#include <stdlib.h>
#include <string.h>
#include <fstream.h>

/* =======================================================================================
	LoadRegionFromFile is used to load test data about all target containment regions which
	exist within the given ocean region.

	This software was developed as part of a grant by ONR 311, Office of Naval Research, 
	800 North Quincy Street, Arlington, Virginia 22217, for development of an "Application 
	of Volume Modeling Techniques to Dynamic Containment Regions for Naval Applications".
	Grant was awarded to Arizona State University, PRISM Laboratory, Department of Computer 
	Science and Engineering, Dr. Greg Nielson and Gary Graf.


	Author: Gary Graf, PRISM Laboratory, Arizona State University.
	Version:  1.0, 10 Sep 2002

  Copyright 2002, all rights reserved.
  
	Notes:
=========================================================================================*/

	const int MAX_NAME_SIZE = 80;
	

/**************************************************************************************
---------------------------------------------------------------------------------------
		Constructors / Destructors
---------------------------------------------------------------------------------------
**************************************************************************************/

	/*
		Simple constructor.  The act of instantiation simultaneously loads the data from
			the file specified.

		@param fileName - A pointer to a character string which identifies the file to be 
				loaded into memory.
	*/
CLoadRegionFromFile::CLoadRegionFromFile(char* fileName) {

	char inString[MAX_NAME_SIZE];
	char *numPointsString;
	int tempI;
	char *numTrianglesString;
	sourceName = new char[MAX_NAME_SIZE];
	
	minLat = 361.0;
	maxLat = -361.0;
	minLon = 361.0;
	maxLon = -361.0;

		//Load data
	ifstream fin;
 	fin.open(fileName);

		//Read the name of the containment region
	fin.getline(sourceName, MAX_NAME_SIZE);


		//Read in the number of points
	fin.getline(inString, MAX_NAME_SIZE);
		//First call strips off the "points:" and the second retrieves the number of vertices
	numPointsString = strtok(inString, ":");
	numPointsString = strtok(NULL, ":");
		//	Convert the string to an integer to get the number of vertices
	numVerts = atoi(numPointsString);

	vertexList = new float[numVerts][3];	//Index 0 = lon, 1 = lat, 2 = depth
	int i;
	for(i = 0; i < numVerts; ++i){
		fin>>tempI;
		fin>>vertexList[i][1];
		fin>>vertexList[i][0];
		fin>>vertexList[i][2];
			//Find the extremes in the data set
		if(vertexList[i][1] < minLat)
			minLat = vertexList[i][1];
		if(vertexList[i][1] > maxLat)
			maxLat = vertexList[i][1];
		if(vertexList[i][0] < minLon)
			minLon = vertexList[i][0];
		if(vertexList[i][0] > maxLon)
			maxLon = vertexList[i][0];
	}	//end for

		//Strip of eol from last numeric line
	fin.getline(inString, MAX_NAME_SIZE);

		//Read in the number of triangles
	fin.getline(inString, MAX_NAME_SIZE);
		//First call strips off the "points:" and the second retrieves the number of vertices
	numTrianglesString = strtok(inString, ":");
	numTrianglesString = strtok(NULL, ":");
		//	Convert the string to an integer to get the number of vertices
	numTriangles = atoi(numTrianglesString);

	triangleList = new int[numTriangles][3];

	for(i = 0; i < numTriangles; ++i){
		fin>>triangleList[i][0];
		fin>>triangleList[i][1];
		fin>>triangleList[i][2];
	}	//end for

	fin.close();	//Done with light data
}	//end constructor LoadRegionFromFile()


	//Destructor
CLoadRegionFromFile::~CLoadRegionFromFile(){
}	//end destructor ~CLoadRegionFromFile()


//end class CLoadRegionFromFile()

