#ifndef CONTACT_EVENT_H
#define CONTACT_EVENT_H

// 2002, Virginia Tech UVAG Group //

#include<string>
#include "coord.h"
#include "ThreatType.h"


/*! ContactEvents are used to communicate position updates from the feeder to the main app */
struct ContactEvent {
  /// A ContactEven will hold either precise (point) data, or fromSensor (area) data
  union {
	PreciseContact precise;
	WeaponContact weapon;
	ContactFromSensor fromSensor;
        HypotheticalContact hypothetical;
 } coord;
  /// holds the name of the contact whose information is provided in this event
  std::string name;
  /// holds the name of the solution for the contact
  std::string solution;
  /// a number identifying the icon to use for this contact
  int iconID;
  /// type of contact (vehicle, weapon, sensor, etc)
  ContactType contactType;
  /// holds the type of contact and its threat classification
  ThreatClass threat;
  Category category;
  /*! packs the data contained in this object in a byte array, ready to be send to the main app. unpackFrom() performs the complementary action.
      It assumes the buffer is big enough to contain sizeof(ContactEvent)+2 bytes.
      Returns the number of bytes used in buffer. */
  unsigned int packTo(char *buffer) const;
  /*! unpacks the data  contained in a byte array, that was packed using packTo(),
      and updates all fields in this object according to the content of the byte array.
      It assumes the buffer is big enough to contain a marshalled representation of the object.
      This can be tricky to calculate, since the contact name length changes. */
  void unpackFrom(const char *buffer);
};




#endif
