#ifndef _FEEDER_CON_H
#define _FEEDER_CON_H

// 2002, Fernando Das-Neves, Virginia Tech UVAG Group //


#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include "ContactEvent.h"

/*! FeederConnection is the link between an application that sends events and
   the viz app that shows the events */
   class FeederConnection {
   private:
     /// A struct used internally to query the socket for incoming data
      struct pollfd pollSocket;
   protected:
      /// Unix socket used for connection
      int sockfd;
       /// buffer used to marshall & unmarshall data sent thru the socket
      char * marshallingBuffer;
      /// number of bytes in buffer pointed to by marshallingBuffer
      unsigned int bufferSize;
      /*! gets space for marshallingBuffer. If there was a buffer already
          allocated, it will be deleted.
          Returns the buffer size, or 0 is it couldn't allocate memory. */
      unsigned int allocateMarshallingBuffer();
      /*! If marshallingBuffer is not NULL, frees the buffer in marshallingBuffer. */
      void destroyMarshallingBuffer();
   public:
     static const char * SOCK_ADDRESS;
     static const int NO_CONNECTION = -1;
     static const int  FAILED  = -1;
      FeederConnection():sockfd(-1),marshallingBuffer(NULL),bufferSize(0) { }
      /// creates the Unix socket to send data. Returns the file descriptor of the socket, or -1 if there is an error
      int startFeeder();
      /// connect to the Unix socket to receive data.  Returns the file descriptor of the socket, or -1 if there is an error
      int connectToFeeder();
      /// closes the Unix socket. Returns 0 if success, or -1 if it failed (the socket was no longer available)
      int closeConnection();
      /// sends an event through the connection, and returns the number of bytes sent
      int sendEvent(const ContactEvent& evt) const;
      int getSocket() const { return sockfd; }
      /// returns true if the socket is opened and there is data awaiting
      bool isDataAvailable();
      /// reads an event from the connection, and returns the number of bytes read, or -1 if it was not possible to read an event
      int readEvent(ContactEvent& evt);
      /// sends a single byte from the connection, useful to receive quick condition flags. Returns true if byte was sent succesfully
      bool sendByte(unsigned char aByte) const;
      /// reads a single byte from the connection, useful to receive quick condition flags. Returns true if byte was read succesfully
      bool readByte(unsigned char& aByte);
      ~FeederConnection();
   };

#endif
