/// 2002, Virginia Tech UVAG Group ///

#include "IvColorScale.h"
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include "glRenderText.h"
#include <stdlib.h>

/************************** ColorScale class ******************************/

SO_NODE_SOURCE(IvColorScale);

void IvColorScale::initClass() {
  SO_NODE_INIT_CLASS(IvColorScale,SoGroup,"Group");
}

void IvColorScale::drawTicks(void* colorScale, SoAction*) {
    IvColorScale *self = (IvColorScale*)colorScale;
    if(self->ticks) {
      float xpos=self->origin[0]+self->width+SCALE_TO_TICK_SEPARATION;
      float ypos=self->origin[1]-6;
      // draw ticks' background
      glColor3f(self->tickBg[0],self->tickBg[1],self->tickBg[2]);
      for(int i=0; i<self->numticks;i++) {
        glRectf(xpos,ypos-2,xpos+self->tickWidth,ypos+11);
        ypos+=self->tickDistance;
      }
      ypos=self->origin[1]-6;
      // draw ticks
      glColor3f(self->tickColor[0],self->tickColor[1],self->tickColor[2]); // text
      for(int i=0; i<self->numticks;i++) {
        glRenderTextAt(xpos,ypos,self->ticks[i],GLUT_BITMAP_HELVETICA_10);
        ypos+=self->tickDistance;
      }
    }
}

IvColorScale::IvColorScale() {
  SO_NODE_CONSTRUCTOR(IvColorScale);
  init();
}

void IvColorScale::init() {
 origin[0] = origin[1] = 0.0f;
 ticks=NULL;
 numticks=0;
 tickColor[0] = tickColor[1] = tickColor[2] = 1.0f;
 tickBg[0] = tickBg[1] = tickBg[2] = 0.0f;
 tickWidth = 0;
 cmap = new SoBaseColor;
 this->addChild(cmap);
 // we need to draw ticks before scale, so we can draw the background too
 SoMaterialBinding *binding = new SoMaterialBinding;
 binding->value.setValue(SoMaterialBinding::PER_VERTEX_INDEXED);
 this->addChild(binding);
 points = new SoCoordinate3;
 this->addChild(points);
 scale = new SoIndexedTriangleStripSet;
 this->addChild(scale);
 SoCallback *glCallback = new SoCallback;
 glCallback->setCallback(IvColorScale::drawTicks,this);
 this->addChild(glCallback);
}

void IvColorScale::setRange(float newMin, float newMax,unsigned int howmanyTicks) {
    typedef char* pchar;
    int i, pixels;
    tickWidth = 0;
    min=newMin; max = newMax;
    if(ticks) {
     for(i=0;i<numticks;i++)
      free(ticks[i]);
    }
    free(ticks);
    if(howmanyTicks <2) numticks=2;
    else numticks=howmanyTicks;
    ticks = (unsigned char **)malloc(sizeof(char*)*numticks);
    float val=min, step=(max-min)/(numticks-1);
      for(i=0;i<numticks;i++) {
      ticks[i] = (unsigned char *)malloc(sizeof(char)*20);   
      sprintf((char*)ticks[i],"%g",val);
      pixels = glutBitmapLength(GLUT_BITMAP_HELVETICA_10,ticks[i]);
      if(pixels > tickWidth) tickWidth = pixels;
      val+=step;
    }    
    tickDistance =height/(float)(numticks-1);
}

void IvColorScale::setSize(float newWidth, float newHeight) {
  width=newWidth; height=newHeight;
}

void IvColorScale::setColormap(SoMFColor& acmap, float xleft, float ybottom) {
  origin[0]=xleft; origin[1]=ybottom;
  setColormap(acmap);
}

/* setFrom() keeps the info to update the colormap, but does not apply 
 the changes. The reason is that changing the colorscale may imply to
 change the geoset, and we do not want to do that unless we are sure
 we're in the APP stage. This way is more complicated to program, but
 makes possible to change the colormap at any time, without worring 
 for Performer to crash in MP mode.
*/
void IvColorScale::setColormap(SoMFColor& colorMap) {
  cmap->rgb = colorMap;
  reset();
}


// apply changes to the ColorScale
void IvColorScale::reset() {
    unsigned int numpts = 2*cmap->rgb.getNum(); 
    float cellheight = height/cmap->rgb.getNum();
    bool changeSize = (scale->coordIndex.getNum() != numpts+1);
    int i;
    if(changeSize) { // # cells in scale does not match # of colors      
        SbVec3f* pts;
        int32_t *pts_indices, *clr_indices;
        // allocate space for data, so we modify the node's fields only once
            pts = (SbVec3f*)malloc(sizeof(SbVec3f)*numpts);
            pts_indices = (int32_t*)malloc(sizeof(int32_t)*(numpts+1));
        clr_indices  = (int32_t*)malloc(sizeof(int32_t)*(numpts+1));
        // recalculate points of triangle strip
        float y= origin[1];
        for(i=0; i<numpts; i+=2) {
          pts[i].setValue(origin[0],y,0);
          pts[i+1].setValue(origin[0]+width,y,0);
          y+=cellheight;
        }
        // adjust coord indices
        for(i=0; i<numpts; i++) {
          pts_indices[i] = i;      
        }
        pts_indices[numpts] =  SO_END_STRIP_INDEX;
        // adjust color indices
        for(i=0; i<numpts; i+=2) {
          clr_indices[i] = clr_indices[i+1] = (i >> 1);
        }
        clr_indices[numpts] = SO_END_FACE_INDEX;
        scale->materialIndex.setValues(0,numpts+1,clr_indices);
        points->point.setValues(0,numpts,pts);
        scale->coordIndex.setValues(0,numpts+1,pts_indices);
        free(pts);
        free(pts_indices);
        free(clr_indices);
   }
}

IvColorScale::~IvColorScale() {
  if(ticks) free(ticks);
}
