#ifndef COLORSCALE_H
#define COLORSCALE_H


// 2002, Virginia Tech UVAG Group //


#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/actions/SoAction.h>

// next header is used to make IvColorScale a "nice" inventor subclass
#include <Inventor/nodes/SoSubNode.h>

/*! A ColorScale is an Inventor object used to visualize a color gradient. It is meant to be the child of
   an IvHUD node. It will draw a 2D color bar laying on the XY plane */
class IvColorScale:public SoGroup {
  SO_NODE_HEADER(IvColorScale);
  /// current color map
  SoBaseColor *cmap;
  /// coordinates of the triangle mesh that makes the color bar
  SoCoordinate3 *points;
  /// the triangle strip that makes the color bar
  SoIndexedTriangleStripSet *scale;
  /// origin of color scale
  float origin[2];
  /// size of the color scale
  float width,height;
  /// maximum and minimum scalar value associated to the color scale
  float min,max;
  /// keeps the string to write at every tick (mark) in the color scale
  unsigned char** ticks;
  /// how many tick marks to show next to the color bar
  unsigned int numticks;
  /// distance between ticks in the X axis
  float tickDistance;
  /// color used to draw the ticks (depth labels)
  float tickColor[3];
  /// color used to draw background for the color scale
  float tickBg[3];
  /// width in pixels of longest tick text
  float tickWidth;
  void reset();
  void init();
protected:
  /// number of pixels separating the ticks from the color bar
  enum {SCALE_TO_TICK_SEPARATION = 6 };
  static void drawTicks(void* colorScale, SoAction*);
  ~IvColorScale();
public:
  static void initClass();
  IvColorScale();
  void setLength(float aWidth) { width=aWidth; }
  void setHeight(float aHeight) { height=aHeight; }
  void setRange(float newMin, float newMax, unsigned int numberOfTicks=2);
  void setSize(float width,float height);
  void setPosition(float xleft, float ybottom) { origin[0]=xleft; origin[1]=ybottom; }
  /// tell the color scale what colors to show
  void setColormap(SoMFColor& colormap);
  void setColormap(SoMFColor& colormap, float xleft, float ybottom);
  void setTickColor(float r, float g, float b) { tickColor[0]=r; tickColor[0]=g; tickColor[0]=b; }
  void setTickBackground(float r, float g, float b) { tickBg[0]=r; tickBg[0]=g; tickBg[0]=b; }
};

#endif
