#include "IvContactLabeler2D.h"
#include <GL/gl.h>
#include <GL/glut.h>
#include "glRenderText.h"
#include <stdio.h>
#include "degree_conv.h"

// used to get the user head's current position and angles
#include <navmgr/NavManager.h>

   SO_NODE_SOURCE(IvContactLabeler2D)
   
   void IvContactLabeler2D::initClass() {
      SO_NODE_INIT_CLASS(IvContactLabeler2D,SoCallback,"Callback");
   
   }

   IvContactLabeler2D::IvContactLabeler2D():hud(NULL) {
      SO_NODE_CONSTRUCTOR(IvContactLabeler2D);
      SO_NODE_ADD_FIELD(transform, (1.0f, 0.0f, 0.0f, 0.0f,
         0.0f, 1.0f, 0.0f, 0.0f,
         0.0f, 0.0f, 1.0f, 0.0f,
         0.0f, 0.0f, 0.0f, 1.0f
         )); 
      bgColor[0]=bgColor[1]=bgColor[2]=0.0f;
      setCallback(IvContactLabeler2D::drawLabels,this);
   }

   IvContactLabeler2D::IvContactLabeler2D(DiverseHUD *aHUD):hud(aHUD) {
      SO_NODE_CONSTRUCTOR(IvContactLabeler2D);
      SO_NODE_ADD_FIELD(transform, (1.0f, 0.0f, 0.0f, 0.0f,
         0.0f, 1.0f, 0.0f, 0.0f,
         0.0f, 0.0f, 1.0f, 0.0f,
         0.0f, 0.0f, 0.0f, 1.0f
         )); 
      bgColor[0]=bgColor[1]=bgColor[2]=0.0f;
      setCallback(IvContactLabeler2D::drawLabels,this);
   }

   void IvContactLabeler2D::useHUD(DiverseHUD *aHUD) {
      hud = aHUD;
   }

/* drawLabels() will work under the assumption that 'contactList' 
    contains pair<ObservableTrack*,IvContactView*> */
   void IvContactLabeler2D::drawLabels(void* thisObject, SoAction* action) {
      if(action->isOfType(SoGLRenderAction::getClassTypeId())) {
         IvContactLabeler2D *self = (IvContactLabeler2D*)thisObject;
         SbVec3f centerPoint, contactPos;
         GLdouble screenPoint[3];
         dVec3 headPos,headOrientationVector;
         const SbMatrix& m = self->transform.getValue();

         // get user head's transformation matrix 
	 dMatrix navMatrix = NavManager::getMatrix();
         headPos=navMatrix.getTrans();
         navMatrix.setTrans(0,0,0);
         headOrientationVector=navMatrix*dVec3(0,0,-1); /* (0,0,-1) = for Inventor = looking forward, parallel to the floor */

         for(ContactList::iterator it = self->contacts.begin(); it != self->contacts.end(); ++it) {
            LabelData* labelData = ((LabelData*)it->second);
            // draw the label only if the IvContactView object being labeled is drawn too
            if(labelData->view->isAvailable() && labelData->view->isVisible()) {
               DiverseHUD *hud = self->hud;
               // get position of icon
               contactPos = labelData->view->getPosition().getValue();
               // apply transform to icon position and store it in centerPoint
               m.multVecMatrix(contactPos,centerPoint);
               // draw the label only if it appears in semicircle (90 degree angle) around the vector aiming ath the direction in which the user is currently looking
               if(!hud->isLocalCoordCulled(centerPoint[0],centerPoint[1],centerPoint[2])) {
                   // project transformed icon position to 2D screen coords
                   gluProject(centerPoint[0],centerPoint[1], centerPoint[2],hud->glModelViewMatrix,hud->glProjMatrix,hud->viewport,screenPoint,screenPoint+1,screenPoint+2);
                   /*** printf("label 3d pos = [%f, %f, %f], screen = [ %f ,%f]\n",centerPoint[0],centerPoint[1],centerPoint[2],screenPoint[0],screenPoint[1]); ***/
                   screenPoint[1]+=24;
                   if(screenPoint[1] > hud->viewport[3]) screenPoint[1] -= 32;
                   float len = labelData->labelLengthInPixels/2;
                   glColor3f(self->bgColor[0],self->bgColor[1],self->bgColor[2]);
                   glRectf(screenPoint[0]-len-2,screenPoint[1]-2,screenPoint[0]+len+2,screenPoint[1]+11);
                   glColor3f(labelData->labelColor[0],labelData->labelColor[1],labelData->labelColor[2]);
                   glRenderTextAt(screenPoint[0]-len,screenPoint[1],(const unsigned char*)((ObservableTrack*)it->first)->getName().c_str(),GLUT_BITMAP_HELVETICA_10);
               }
            }
         }
      }
   }

/* This is written under the assumption that contact labels do not change, 
   and so we can cache some of the data */
   void IvContactLabeler2D::displayLabelForContactView(IvContactView* acv, float labelRed, float labelGreen, float labelBlue) {
   /* associate the view's model with the view, so when get get an event from the model, 
     we know which view is associated to it */
      LabelData *labelData = new LabelData;
      labelData->view = acv;
   /* calculate the length in pixels of the contact name - since contact names
     don't change, we do this only once */
      labelData->labelLengthInPixels = glutBitmapLength(GLUT_BITMAP_HELVETICA_10, (const unsigned char*)acv->getModel()->getName().c_str());
      labelData->labelColor[0]=labelRed;
      labelData->labelColor[1]=labelGreen;
      labelData->labelColor[2]=labelBlue;
      contacts[acv->getModel()]=labelData;
   /* attach this object as observer of the IvContactView's model, so when the
     model is destroyed, we remove the ContactView from 
     the list of contacts to label */
      acv->getModel()->addView(this);
   }

/* This method is called every time something in the contact changes */
   void IvContactLabeler2D::update(void *info) {
   /* Since we are attached to a ContactView's model (see displayLabelForContactView() ), 
     we know 'info' is really of type TrackEvent* */
      if(((TrackEvent*)info)->eventType == TrackEvent::TRACK_ABOUT_TO_DIE ) {
      // remove the track from the list of track to label
         delete (LabelData*)contacts[((TrackEvent*)info)->track];
         contacts.erase(((TrackEvent*)info)->track);
      }
   }
