/// 2002, Fernando Das-Neves, Virginia Tech UVAG Group ///

#include <Inventor/misc/SoChildList.h>
#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/actions/SoGetBoundingBoxAction.h>
#include <Inventor/actions/SoGetMatrixAction.h>
#include <Inventor/actions/SoHandleEventAction.h>
#include <Inventor/actions/SoPickAction.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/nodes/SoPickStyle.h>
#include "IvHUD.h"
#include <GL/glut.h>


   SO_NODE_SOURCE(IvHUD);


   void IvHUD::initClass() {
      SO_NODE_INIT_CLASS(IvHUD,SoGroup,"Group");
   }

   void IvHUD::initInstance(SbViewportRegion* vp) {
   // make everything in the HUD (any children) unpickable
      SoPickStyle *pickStyle = new SoPickStyle;
      pickStyle->style = SoPickStyle::UNPICKABLE;
      this->addChild(pickStyle);
      if(vp == NULL)
         updateViewportFromGL();
      else {
         updateViewport(vp);
      }	
   }

   IvHUD::IvHUD(SbViewportRegion* vp):SoGroup(),hasBBox(false),isVisible(true) {
      SO_NODE_CONSTRUCTOR(IvHUD);
      initInstance(vp);
   }

   IvHUD::IvHUD(int numChildren, SbViewportRegion* vp):SoGroup(numChildren),hasBBox(false),isVisible(true) {
      SO_NODE_CONSTRUCTOR(IvHUD);
      initInstance(vp);
   }

   IvHUD::~IvHUD() {
   }


   void IvHUD::updateViewport(SbViewportRegion* vp) {
      SbVec2s vec= vp->getViewportOriginPixels();
      viewport[0]=vec[0]; viewport[1]=vec[1];
      vec= vp->getWindowSize();
      viewport[2]=vec[0]; viewport[3]=vec[1];
   }

   void IvHUD::updateViewportFromGL() {
      glGetIntegerv(GL_VIEWPORT,viewport);
   }

   void IvHUD::preDraw(int w, int h, bool& lights) {
      glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
      glLoadIdentity();
      glMatrixMode(GL_PROJECTION);
      glPushMatrix();
      glLoadIdentity();
      gluOrtho2D(0,w,0,h);
      glDisable(GL_DEPTH_TEST);
      if((lights = glIsEnabled(GL_LIGHTING)) == GL_TRUE)
         glDisable(GL_LIGHTING);
   
   }

   void IvHUD::postDraw(bool lights) {
      if(lights)
         glEnable(GL_LIGHTING);
      glEnable(GL_DEPTH_TEST);
      glMatrixMode(GL_PROJECTION);
      glPopMatrix();
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();
   }

   void IvHUD::getBoundingBox(SoGetBoundingBoxAction *action) {
      if(hasBBox)
         SoGroup::getBoundingBox(action);
      else {
         ((SoGetBoundingBoxAction *)action)->resetCenter();
      }
   }

   void IvHUD::GLRender(SoGLRenderAction *action) {
      if(isVisible) {
         bool lightState;
         GLint windowSize[4];
         /*  NOTE: We're forced to get the projection and modelview matrices
             here, before preDraw() is called, because preDraw will change them
             to be 2D orthographic */
         glGetDoublev(GL_PROJECTION_MATRIX,glProjMatrix);
         glGetDoublev(GL_MODELVIEW_MATRIX,glModelViewMatrix);
         glGetIntegerv(GL_VIEWPORT,windowSize);   	
         preDraw(windowSize[2],windowSize[3],lightState);
         children->traverse(action);
         postDraw(lightState);
      }
   }


