#include "IvHighlightSelection2D.h"
#include <Inventor/actions/SoGetBoundingBoxAction.h>
#include <Inventor/elements/SoCacheElement.h>
#include <GL/gl.h>
#include <GL/glu.h>

#ifdef __linux__
  #include <values.h>
#else
  #include <limits.h>
#endif





   void IvHighlightSelection2D::setSelection(SoPath* p) { 
      if(selectionPath) selectionPath->unref(); 
      selectionPath = p; 
      p->ref(); 
      updateBbox();
   }

   IvHighlightSelection2D::IvHighlightSelection2D(DiverseHUD* theHUD):selectionPath(NULL),lineThickness(4.0f),hud(theHUD),drawFlag(true) {
      boxColor[0] = 1.0f; boxColor[1] = boxColor[2] = 0.0f;
      setCallback(IvHighlightSelection2D::drawBbox,this);
   }


   void IvHighlightSelection2D::updateBbox() {
      if (selectionPath == NULL) 
          return;  
      // Compute the 3d bounding box of the passed path
      SbViewportRegion vp;
      vp.setViewport(hud->viewport[0],hud->viewport[1],hud->viewport[2],hud->viewport[3]);
      SoGetBoundingBoxAction bba(vp);   
      bba.apply(selectionPath);
      SbVec3f min, max;
      bba.getBoundingBox().getBounds(min, max);
      // check if center point of bounding box falls within the visible area, and don't render it if it is culled
      if(!hud->isLocalCoordCulled((min[0]+max[0])/2,(min[1]+max[1])/2,(min[2]+max[2])/2)) {
          GLdouble screenPoint[8][3];
          // Project points to screen coordinates
          gluProject(min[0], min[1], min[2],hud->glModelViewMatrix,hud->glProjMatrix,hud->viewport,screenPoint[0],screenPoint[0]+1,screenPoint[0]+2);
          //   printf("highlight 3d pos = [%f, %f, %f], screen = [ %f ,%f]\n",min[0],min[1],min[2],screenPoint[0][0],screenPoint[0][1]);	
          gluProject(min[0], min[1], max[2],hud->glModelViewMatrix,hud->glProjMatrix,hud->viewport,screenPoint[1],screenPoint[1]+1,screenPoint[1]+2);
          gluProject(min[0], max[1], min[2],hud->glModelViewMatrix,hud->glProjMatrix,hud->viewport,screenPoint[2],screenPoint[2]+1,screenPoint[2]+2);
          gluProject(min[0], max[1], max[2],hud->glModelViewMatrix,hud->glProjMatrix,hud->viewport,screenPoint[3],screenPoint[3]+1,screenPoint[3]+2);
          gluProject(max[0], min[1], min[2],hud->glModelViewMatrix,hud->glProjMatrix,hud->viewport,screenPoint[4],screenPoint[4]+1,screenPoint[4]+2);
          gluProject(max[0], min[1], max[2],hud->glModelViewMatrix,hud->glProjMatrix,hud->viewport,screenPoint[5],screenPoint[5]+1,screenPoint[5]+2);
          gluProject(max[0], max[1], min[2],hud->glModelViewMatrix,hud->glProjMatrix,hud->viewport,screenPoint[6],screenPoint[6]+1,screenPoint[6]+2);
          gluProject(max[0], max[1], max[2],hud->glModelViewMatrix,hud->glProjMatrix,hud->viewport,screenPoint[7],screenPoint[7]+1,screenPoint[7]+2);
          // Find the encompassing 2d box
          xmin = ymin = FLT_MAX; xmax = ymax = FLT_MIN;
          for (int i = 0; i < 8; i++) {
              if(xmin > screenPoint[i][0] ) xmin = screenPoint[i][0];
              else if(xmax < screenPoint[i][0]) xmax = screenPoint[i][0];
              if(ymin > screenPoint[i][1] ) ymin = screenPoint[i][1];
              else if(ymax < screenPoint[i][1]) ymax = screenPoint[i][1];
          }
      }
   }  

   void IvHighlightSelection2D::drawBbox(void* thisObject, SoAction* action) {
      if(action->isOfType(SoGLRenderAction::getClassTypeId())) {
         IvHighlightSelection2D *self = (IvHighlightSelection2D*)thisObject;
         if(self->selectionPath != NULL) {
            float oldWidth;
            self->updateBbox();
            if(self->drawFlag) {
               glColor3f(self->boxColor[0],self->boxColor[1],self->boxColor[2]);
               glGetFloatv(GL_LINE_WIDTH,&oldWidth);
               glLineWidth(self->lineThickness);
               glBegin(GL_LINE_LOOP);	  
               glVertex2f(self->xmin,self->ymin);
               glVertex2f(self->xmin,self->ymax);
               glVertex2f(self->xmax,self->ymax);
               glVertex2f(self->xmax,self->ymin);
               glEnd();
               glLineWidth(oldWidth);
            }
         }
      // invalidate rendering cache to force redraw of this node when render area changes
         SoCacheElement::invalidate(action->getState());
      }
   }
