#include <math.h>
#include <stdio.h>
#include "intersect.h"

static void convexHullOrder(float sr[][3],int numtot,float cr[3]);
static short between(float var, float a, float b);

/*finds the 2D intersection of convex shapes s1 and s2*/
void intersect2DConvex(float s1[][3], int p1,float s2[][3], int p2, float sr[][3], int *pr) {
  float c1[]={0,0};
  float c2[]={0,0};
  float cr[]={0,0};
  int i,j;
  //find the centroids
  for (i=0;i<p1;i++) {
    c1[0]+=s1[i][0];
    c1[1]+=s1[i][1];
  }
  c1[0]/=p1;
  c1[1]/=p1;
  for (i=0;i<p2;i++) {
    c2[0]+=s2[i][0];
    c2[1]+=s2[i][1];
  }
  c2[0]/=p2;
  c2[1]/=p2;
  //find the line equations
  float pt1[3], pt2[3];
  float *lines1=new float[p1*4];
  for (i=0;i<p1;i++) {
    pt1[0]=s1[i][0];
    pt1[1]=s1[i][1];
    pt1[2]=s1[i][2];
    j=(i+1)%p1;
    pt2[0]=s1[j][0];
    pt2[1]=s1[j][1];
    pt2[2]=s1[j][2];
    lines1[4*i+1]=pt1[0]-pt2[0];
    lines1[4*i+0]=pt2[1]-pt1[1];
    lines1[4*i+2]=-(lines1[4*i+0]*pt1[0]+lines1[4*i+1]*pt1[1]);
    lines1[4*i+3]=lines1[4*i+0]*c1[0]+lines1[4*i+1]*c1[1]+lines1[4*i+2];
  }
  float *lines2=new float[p2*4];
  for (i=0;i<p2;i++) {
    pt1[0]=s2[i][0];
    pt1[1]=s2[i][1];
    pt1[2]=s2[i][2];
    j=(i+1)%p2;
    pt2[0]=s2[j][0];
    pt2[1]=s2[j][1];
    pt2[2]=s2[j][2];
    lines2[4*i+1]=pt1[0]-pt2[0];
    lines2[4*i+0]=pt2[1]-pt1[1];
    lines2[4*i+2]=-(lines2[4*i+0]*pt1[0]+lines2[4*i+1]*pt1[1]);
    lines2[4*i+3]=lines2[4*i+0]*c2[0]+lines2[4*i+1]*c2[1]+lines2[4*i+2];
  }
  int numtot=0, num1=0;
  float sol;
  short inside;
  //find points that are inside the other
  for (i=0;i<p1;i++) {
    inside=1;
    for (j=0;j<p2;j++){
      sol=s1[i][0]*lines2[4*j+0]+s1[i][1]*lines2[4*j+1]+lines2[4*j+2];
      if (sol*lines2[4*j+3]<0) {inside=0;break;}
    }
    if (inside) {
      sr[numtot][0]=s1[i][0];
      sr[numtot][1]=s1[i][1];
      sr[numtot][2]=s1[i][2];
      numtot++;
      num1++;
    }
  }
  if (numtot==p1) {
    *pr=p1;
    return;
  }
  for (i=0;i<p2;i++) {
    inside=1;
    for (j=0;j<p1;j++){
      sol=s2[i][0]*lines1[4*j+0]+s2[i][1]*lines1[4*j+1]+lines1[4*j+2];
      if (sol*lines1[4*j+3]<0) {inside=0;break;}
    }
    if (inside) {
      sr[numtot][0]=s2[i][0];
      sr[numtot][1]=s2[i][1];
      sr[numtot][2]=s2[i][2];
      numtot++;
    }
  }
  if ((numtot==p2)&&(num1==0)) {
    *pr=p2;
    return;
  }
  float m1, m2, b1,b2, x,y;
  //find intersection points
  for (i=0;i<p1;i++) {
    m1=-(lines1[4*i+0]/lines1[4*i+1]);
    b1=-(lines1[4*i+2]/lines1[4*i+1]);
    for (j=0;j<p2;j++){
      m2=-(lines2[4*j+0]/lines2[4*j+1]);
      b2=-(lines2[4*j+2]/lines2[4*j+1]);
      if ((m1==m2)||((lines1[4*i+1]==0)&&(lines2[4*j+1]==0))) continue;
      if (lines1[4*i+1]==0) {
	x=s1[i][0];
      } else if (lines2[4*j+1]==0)
	x=s2[j][0];
      else
	x=(b2-b1)/(m1-m2);
      if (lines1[4*i+1]==0)
	y=m2*x+b2;
      else
	y=m1*x+b1;
      if (between(x,s1[i][0],s1[(i+1)%p1][0])) {
	if (between(x,s2[j][0],s2[(j+1)%p2][0])) {
	  if (between(y,s1[i][1],s1[(i+1)%p1][1])) {
	    if (between(y,s2[j][1],s2[(j+1)%p2][1])) {
	      sr[numtot][0]=x;
	      sr[numtot][1]=y;
	      numtot++;
	    }
	  }
	}
      }
    }
  }
  //remove duplicates
  for (i=0;i<numtot;i++){
    for (j=i+1;j<numtot;j++){
      if (sr[i][0]==sr[j][0])
	if (sr[i][1]==sr[j][1]) {
	  for (int k=j+1; k<numtot;k++) {
	    sr[k-1][0]=sr[k][0];
	    sr[k-1][1]=sr[k][1];
	    sr[k-1][2]=sr[k][2];
	  }
	  numtot--;
	  j--;
	}
    }
  }
  //find the result centroid
  for (i=0;i<numtot;i++) {
    cr[0]+=sr[i][0];
    cr[1]+=sr[i][1];
  }
  cr[0]/=numtot;
  cr[1]/=numtot;
  
  //order them into a shape
  convexHullOrder(sr,numtot,cr);
  *pr=numtot;
  delete[] lines1;
  delete[] lines2;
}


//takes a list of surface points and orders them into a convex hull
//ie in order of their angle from their centroid cr
void convexHullOrder(float sr[][3],int numtot,float cr[3]){
  float *angles = new float [numtot];
  int i,j;
  float tempA;
  float temp[3];
  for (i=0;i<numtot;i++){
    angles[i]=atan2(sr[i][1]-cr[1],sr[i][0]-cr[0]);
  }
  for (i=0;i<numtot;i++) {
    for (j=i+1;j<numtot;j++) {
      if (angles[i]>angles[j]) {
	tempA=angles[i];
	temp[0]=sr[i][0];
	temp[1]=sr[i][1];
	temp[2]=sr[i][2];
	angles[i]=angles[j];
	sr[i][0]=sr[j][0];
	sr[i][1]=sr[j][1];
	sr[i][2]=sr[j][2];
	angles[j]=tempA;
	sr[j][0]=temp[0];
	sr[j][1]=temp[1];
	sr[j][2]=temp[2];
      }
    }
  }
  delete[] angles;
}

//is var in between a and b?
short between(float var, float a, float b) {
  if (((var-a)>-0.001)&&((var-b)<0.001))
    return 1;
  if (((var-a)<0.001)&&((var-b)>-0.001))
    return 1;
  return 0;

}
