#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/actions/SoGetBoundingBoxAction.h>
#include <Inventor/SbViewportRegion.h>
#include <Inventor/SoInput.h>
#include <Inventor/SoDB.h>
#include <stdlib.h>


void printUsage() {
  printf("USAGE:\nivbounds filename\n");
}

void exitError(const char* errmsg) {
  printf("ERROR: %s.\n",errmsg);
  printUsage();
  exit(-1);
}


int main(int argc, char* argv[]) {
        SoDB::init();

        char *iv_infilename = NULL;

        SoSeparator *objectRoot;

        if(argc <=1 || (strcasecmp(argv[1],"--help")==0 || strcasecmp(argv[1],"-h")==0)) {
			printf("ivbounds reads an inventor file, and prints its bounding box.\n");
                        printUsage();
                        exit(-1);
	        } else {
                   iv_infilename = argv[1];
                }

        if(iv_infilename == NULL) {
          exitError("Filename of file to transform missing");
        }

        /* read inventor object */
        SoInput infile;
        if(infile.openFile(iv_infilename) == FALSE) {
          exitError("Can't open input inventor file");
        }

        objectRoot = SoDB::readAll(&infile);
        infile.closeFile();
        if(objectRoot == NULL) {
          exitError("Can't read input inventor file. Probably it is not an valid Inventor file.");
        }

        SoGetBoundingBoxAction bboxAction(SbViewportRegion(1,1));
	bboxAction.apply(objectRoot);

        printf("\nBounding Box of %s:\n",iv_infilename);
        SbBox3f bbox = bboxAction.getBoundingBox();
	printf("X: [%f, %f]\nY: [%f, %f]\nZ: [%f, %f]\n",
        	bbox.getMin()[0],bbox.getMax()[0],
                bbox.getMin()[1],bbox.getMax()[1],
                bbox.getMin()[2],bbox.getMax()[2]);
}
