#include <Inventor/Xt/SoXt.h>
#include <signal.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/IntrinsicP.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtRenderArea.h>
#include <Xm/Form.h>
#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/MainW.h>
#include <Inventor/nodes/SoLightModel.h>
#define USE_LIGHT
#ifdef USE_LIGHT
	#include <Inventor/nodes/SoDirectionalLight.h>
	#include <Inventor/nodes/SoEnvironment.h>
	#include <Inventor/nodes/SoPointLight.h>
#endif
#include <Inventor/nodes/SoMatrixTransform.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/nodes/SoScale.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoLOD.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/SoInput.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/actions/SoGetBoundingBoxAction.h>
#include <Inventor/nodes/SoCoordinate3.h>

// Used to add a texture on top of the map, currently the grid
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTextureCoordinatePlane.h>
#include <Inventor/nodes/SoTexture2Transform.h>

// Used to render the little map in the bezel
#include <Inventor/SoOffscreenRenderer.h>
#include <Inventor/nodes/SoOrthographicCamera.h>

// Used to keep map and icon movement coordinated
#include <Inventor/engines/SoCalculator.h>

// object-picking headers
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/SoPath.h>
#include <Inventor/SoPickedPoint.h>
#include <Inventor/actions/SoRayPickAction.h>

#include "VoidPtrList.h"

#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoPickStyle.h>

#ifdef __linux__
  #include <values.h>
#else
  #include <limits.h>
#endif

// next 2 only used to debug the minimap
#if DBG_SAVE_MAP_IMAGE_TO_DISK
  #include <Inventor/SoOutput.h>
#endif

/*Next line is to have a max size to clip the window -- a not very sophisticated
  way to deal with the "more than one window in the same display" case, like in a CAVE */
#include "maxDisplaySize.h"

/* X11 stuff + keyboard handling */
#include <X11/Intrinsic.h>
#include <X11/Xlib.h>
#define XK_LATIN1 // get Latin-1 (iso 8859-1) keysyms
#define XK_MISCELLANY // get keypad keysyms
#include <X11/keysymdef.h>

// TALOSS headers
#include "FeederConnection.h"
#include "FeederCommand.h"
#include "ContactEvent.h"
#include "timeval_ops.h"
#include "track.h"
#include "trackview.h"
#include "UiMediator.h"
#include "oid.h"
#include "TrackUiView.h"
#include "bezel/ui_area_sizes.h"
#include "DiverseHUD.h"
#include "IvColorScale.h"
#include "IvCompass.h"
#include "IvTiledBathymetry.h"
#include "IvHighlightSelection2D.h"
#include "IvTransformEngine.h"
#include "IvContactLabeler2D.h"
#include "NameToTrackMapping.h"
#include "ContactIconCollection3D.h"
#include "IvContactLabeler2D.h"
#include "degree_conv.h"
#include "bezel/ui_constants.h"
#include "degree_conv.h"

// DGL headers
#include <eventmgr/devents.h>
#include <dgl/dglcore/DGL.h>
#include <dgl/libdgl/cdgl.h>
#include <displaymgr/DisplayManager.h>
#include <displaymgr/DDisplay.h>
#include <navmgr/NavManager.h>
#include <dmath/dMatrix.h>
#include <dmath/dQuat.h>
#include <dmath/Rect.h>
#include <Producer/RenderSurface>
#include <displaymgr/DWindow.h>
#include <displaymgr/DisplayConfig.h>
#include <eventmgr/EventManager.h>

/* xwand headers - control X11 cursor with a wand - 
   only makes sense in a CAVE, idesk or something like that. */
#include <X11/cursorfont.h>
#include "xwand/invisibleCursor.h"
#include "xwand/XWandFakeMouse.h"



using namespace std;


// Object used to talk to the data feeder
static FeederConnection feeder;


// Object to talk to the GUI
static UiMediator *talkWithBezel = NULL;


// These are the colors used for contact labels
static const float OWNSHIP_LABEL_COLOR[3] = { 0.5   , 0.5,    1.0f };
static const float CONTACT_LABEL_COLOR[3] = { 1.0f,   0.5,     0.5 };
static const float WEAPON_LABEL_COLOR[3]  = { 0.6   , 1.0f,    0.6 };


/* This is the name of the file that contains the list of filenames for 3D icons. 
   The first icon in the list is icon #0, next is icon #1 and so forth. */
static const char* const CONTACT_ICONLIST_FILE = "contactIcon.list";

// if FAST_TRANSPARENCY is defined, then we use Inventor's faster but not-so-nice way way to render translucient object
//#define FAST_TRANSPARENCY

// maximum number of contact events sent by the feeder that we check for in one pass of the main loop
#define MAX_SIMULTANEOUS_CONTACT_EVENTS 3

/* Boundaries of the total area possibly covered by the tileset. This is NOT
   the area covered by the current tileset, but the area for which the
   lowest left tile will be tile #0. */
#define MAP_MIN_LAT  -90.0f
#define MAP_MIN_LON -180.0f
#define MAP_MAX_LAT   90.0f
#define MAP_MAX_LON  180.0f

// width and length of a map tile in degrees - all tiles are assumed to be of equal size, except the tiles at the edge of the map
#define TILE_SIZE_X 0.5f
#define TILE_SIZE_Y 0.5f

// minimum distance between onwship and an edge in the visible area of the map before we 'slide' the map to show more tiles
#define EDGE_THRESHOLD (TILE_SIZE_X/5.0f)

#define TILE_SCALING_TO_METERS_X DEGREES_TO_METERS(1.0f)
#define TILE_SCALING_TO_METERS_Y DEGREES_TO_METERS(1.0f)

// This is the exaggeration applied to map heights . Make it a dynamically changeable variable
float TILE_EXAGGERATION_Z= 16.0f;


// this is the deepest point we expect contacts (depth of AOU/wedges) in feet
#define BATHYMETRY_MAX_DEPTH -1000.0f
#define AOU_MAX_DEPTH -300.0f

#define VERYSHALLOW_THRESHOLD 400.0f
#define SHALLOW_THRESHOLD 1000.0f
#define MIDRANGE_THRESHOLD 8000.0f
#define DEEP_THRESHOLD 16000.0f

// This is the  default radius in X,Y of the map area to show around the ownship
#define DEFAULT_MAP_RADIUS_AROUND_X (TILE_SIZE_X*0.5f)
#define DEFAULT_MAP_RADIUS_AROUND_Y (TILE_SIZE_Y*0.5f)


/* give a more explicit name to xwand application events: */
// event 8 = print current nav position
#define APP_ACTION_IS_PRINT_CURRENT_POS XW_APP_EVENT_8
// event 9 = quit this program
#define APP_ACTION_IS_QUIT XW_APP_EVENT_9



/* This type defines the ways the visible area of bathymetry can be updated: 
   FREE_MAP_BROWSING = wherever the user wants to go,
   MAP_FOLLOWS_OWNSHIP = bathymetry changes to keep the onwship on the map
*/
   typedef enum { FREE_MAP_BROWSING, MAP_FOLLOWS_OWNSHIP } MapUpdatingMode;

// This is the height of the isometric camera over the minimap; this value tells how much of the minimap is visible
#define CAMERA_HEIGHT_IN_MINIMAP 18200.0f

//determins whether or not region growth is on
   bool GROW_REGIONS=false;

/* These are  global objects so that the vertical exaggeration slider can use them to change the Z scale,
   and the wireframe drawing mode can be turned on or off */
   static   SoDrawStyle *bathymetryStyle;

   NameToTrackMapping trackDirectory; // creates tracks, and keeps track of mapping from from track names to track objects



/// a struct used in subve.c++ to pass all selection objects to callbacks
   struct SelectionBoxes {
      /// currently selected target in render area
      IvHighlightSelection2D *targetSelection;
      /// currently selected weapon in render area
      IvHighlightSelection2D *weaponSelection;
   };


/*! This class is a queue of ContactEvents. We use it to keep and read events 
    generated in this program, instead of coming from an outside source. */
   class ContactEventQueue {
       bool eventAvailable;
       ContactEvent theEvent;
   public:
       void addEvent(const ContactEvent& newEvent) { theEvent = newEvent; eventAvailable=true; }
       void readEvent(ContactEvent& anEvent) { if(eventAvailable) { anEvent=theEvent; eventAvailable=false; } }
       ContactEventQueue():eventAvailable(false) { }
       bool isDataAvailable() const { return eventAvailable; }
   };


/*! This class slightly modifies the behavior of IvHighlightSelection2D in updateBbox(), 
    to handle the case where the desired higlighted object is marked 'invisible' */
   class IvContactViewHighlighter2D: public IvHighlightSelection2D {
   public:
     /// Creates a  selection object, that will work with a certain DiverseHUD's render area
      IvContactViewHighlighter2D(DiverseHUD* theHUD):IvHighlightSelection2D(theHUD) {
      }
      virtual void updateBbox();
   };

   void IvContactViewHighlighter2D::updateBbox() {
      IvContactView* cview = (IvContactView*)getSelectedObject();
      if(cview != NULL) {
         if(cview->isVisible()) {
            show();
            IvHighlightSelection2D::updateBbox();
         } 
         else {
            hide();
         }
      }
   }

   void saveSceneGraph(SoNode* rootNode,const char* prefixFilename=NULL) {
       const int MAX_LEN_FILENAME=256;
       char filename[MAX_LEN_FILENAME];
       SoWriteAction action;
       time_t now;
       struct tm *timeval;
       time(&now);
       timeval = gmtime(&now);
       if(prefixFilename==NULL)
           strftime(filename,MAX_LEN_FILENAME-1,"%Y-%m-%d_%H%M%S.iv",timeval);
       else {
           unsigned int lenprefix = strlen(prefixFilename);
           strncpy(filename,prefixFilename,lenprefix);
           strftime(filename+lenprefix,MAX_LEN_FILENAME-1-lenprefix,"%Y-%m-%d_%H%M%S.iv",timeval);
       }
       SoSeparator* falseRoot = new SoSeparator;
       falseRoot->ref();
       SoSeparator* camSep = new SoSeparator;
       falseRoot->addChild(camSep);
       camSep->setName("Camera_Aim");
       SoTranslation *moveCam = new SoTranslation;
       camSep->addChild(moveCam);
       dMatrix NavMatrix = NavManager::getMatrix();
       dVec3 vec = NavMatrix.getTrans();
       moveCam->translation.setValue(vec[0],vec[1],vec[2]);
       //vec = NavManager::getDirection();
       SoCoordinate3 *pts = new SoCoordinate3;
       camSep->addChild(pts);
       pts->point.set1Value(0,0,0,0);
       pts->point.set1Value(1,0,0,-1000);
       SoLineSet *line = new SoLineSet;
       camSep->addChild(line);
       line->numVertices.setValue(2);
       line->startIndex.setValue(0);
       falseRoot->addChild(rootNode);
       action.getOutput()->openFile(filename);
       action.getOutput()->setBinary(FALSE);
       action.apply(falseRoot);
       action.getOutput()->closeFile();
       falseRoot->removeChild(rootNode);
       falseRoot->unref();
   }


   void turnWindowBorderOffPreCallback() {
       vector<DDisplay*> dglDisplays = DisplayManager::getDisplays();
       vector<DWindow*> dglWindows = dglDisplays[0]->getWindows();
       dglWindows[0]->setBorder(false);
   }

   typedef 
      struct {
         SbViewportRegion viewport;
         SoNode * rootNode;
	 SoGLRenderAction *renderer;
         float cameraAspectRatio; // display's field of view, in radians
         unsigned int windowLeft,windowBottom;
      } RenderingData;

   /// This is a global variable that keeps all the data needed by the rendering callback
   RenderingData mainWindowRenderingData;


   SoSeparator* xxxroot;


   // This is the function that draws the scene - called by Diverse in dglFrame()
   void drawSceneGraphCallback() {
      glEnable(GL_DEPTH_TEST);
      glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
      glRotatef(90.0f,1.0f,0.0f,0.0f);     
      mainWindowRenderingData.renderer->setViewportRegion(mainWindowRenderingData.viewport);
      glViewport(mainWindowRenderingData.windowLeft,mainWindowRenderingData.windowBottom,mainWindowRenderingData.viewport.getWindowSize().getValue()[0],mainWindowRenderingData.viewport.getWindowSize().getValue()[1]);
      mainWindowRenderingData.renderer->apply(mainWindowRenderingData.rootNode);
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();
   }

   void changeColorscaleDepth(IvTiledBathymetry *b, IvColorScale* colorscale, float d) {
      b->setColorscaleDepth(0,FT_TO_M(d),64);
      colorscale->setRange(-d,0,9);
   }

   void setBathymetryToWireframe(SoDrawStyle* wireframe) {
      wireframe->style = SoDrawStyle::LINES;
   }

   /* This function is used to change the bathymetry to solid mode */
   void setBathymetryToSolid(SoDrawStyle* wireframe) {
      wireframe->style = SoDrawStyle::FILLED;
   }

   typedef enum { LATITUDE,LONGITUDE } GeoCoordinateType;
   void makeLatLonString(char *charArray, float latLonValue, GeoCoordinateType coordType) {
     char dir;
     if (latLonValue>=0) {
       dir=(coordType==LATITUDE?'N':'E');
     } else {
       latLonValue=-latLonValue;
       dir=(coordType==LATITUDE?'S':'W');
     }
     sprintf(charArray,"%1.2f%c\0\0",latLonValue,dir);     
   }

   /*this routine adds placement markers to the map*/
   SoGroup* createLocationMarkers(const SbBox3f& bbox) {
     register unsigned int i,j,xsteps,ysteps;
     SoText2 *labelObject;
     SoTranslation *trans;
     SoTransformSeparator *sep;
     SoLOD *details;
     SoGroup *detailGroup;
     const float steps[] = {0.166666667,0.333333333,0.5};
     const unsigned int numsteps = 3;
     SoGroup* markerGroup = new SoGroup;
     
     //make the outer markings
     SoFont *font = new SoFont();
     font->name.setValue("defaultFont");
     font->size.setValue(18);
     markerGroup->addChild(font);
     SoMaterial *mat = new SoMaterial;
     mat->diffuseColor.setValue(0,0,0);
     markerGroup->addChild(mat);

     sep = new SoTransformSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     trans->translation.setValue(bbox.getMin().getValue()[0],(bbox.getMin().getValue()[1]+bbox.getMax().getValue()[1])*0.5,20);
     labelObject = new SoText2;
     sep->addChild(labelObject);
     labelObject->justification = SoText2::RIGHT;

     sep = new SoTransformSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     labelObject = new SoText2;
     sep->addChild(labelObject);
     labelObject->justification = SoText2::LEFT;

     sep = new SoTransformSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     labelObject = new SoText2;
     sep->addChild(labelObject);
     labelObject->justification = SoText2::CENTER;

     sep = new SoTransformSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     labelObject = new SoText2;
     sep->addChild(labelObject);
     labelObject->justification = SoText2::CENTER;

     details=new SoLOD;
     markerGroup->addChild(details);
     details->center.setValue((bbox.getMin().getValue()[0]+bbox.getMax().getValue()[0])*0.5,(bbox.getMin().getValue()[1]+bbox.getMax().getValue()[1])*0.5,20);
     details->range.set1Value(0,10000);
     details->range.set1Value(1,80000);
     for (unsigned int stepIndex = 0; stepIndex<numsteps; stepIndex++) {
       float step = steps[stepIndex];
       detailGroup = new SoGroup;
       details->addChild(detailGroup);
       xsteps=int((bbox.getMax().getValue()[0]-bbox.getMin().getValue()[0])/step-0.1);
       ysteps=int((bbox.getMax().getValue()[1]-bbox.getMin().getValue()[1])/step-0.1);

       for (i=0;i<xsteps; i++) {
	     sep = new SoTransformSeparator;
	     detailGroup->addChild(sep);
	     trans = new SoTranslation;
	     sep->addChild(trans);     
	     labelObject = new SoText2;
	     sep->addChild(labelObject);
	     labelObject->justification = SoText2::CENTER;
       }

       for (j=0;j<ysteps; j++) {
	     sep = new SoTransformSeparator;
	     detailGroup->addChild(sep);
	     trans = new SoTranslation;
	     sep->addChild(trans);     
	     labelObject = new SoText2;
	     sep->addChild(labelObject);
	     labelObject->justification = SoText2::CENTER;
       }

       for (i=0; i<xsteps; i++) {
           for (j=0; j<ysteps; j++) {
	       sep = new SoTransformSeparator;
	       detailGroup->addChild(sep);
	       trans = new SoTranslation;
	       sep->addChild(trans);     
	       labelObject = new SoText2;
	       sep->addChild(labelObject);
	       labelObject->justification = SoText2::CENTER;
	       
	       sep = new SoTransformSeparator;
	       detailGroup->addChild(sep);
	       trans = new SoTranslation;
	       sep->addChild(trans);     
	       labelObject = new SoText2;
	       sep->addChild(labelObject);
	       labelObject->justification = SoText2::CENTER;
           }
       }
     }
     return markerGroup;
   }



/*this routine assign values to the  placement markers on the map*/
   void updateLocationMarkers(SoGroup* markerGroup, const SbBox3f& bbox) {
     register unsigned int i, j, xsteps, ysteps, childIndex;
     float x, y, step;
     const float steps[] = {0.166666667,0.333333333,0.5};
     const unsigned int numsteps = 3;
     char label[100];
     SoText2 *labelObject;
     SoTranslation *trans;
     SoTransformSeparator *sep;
     SoGroup *details, *detailGroup;

     if(markerGroup==NULL) return;

     sep = (SoTransformSeparator*)markerGroup->getChild(2);

     trans = (SoTranslation*)sep->getChild(0);

     trans->translation.setValue(bbox.getMin().getValue()[0],(bbox.getMin().getValue()[1]+bbox.getMax().getValue()[1])*0.5,20);

     labelObject =  (SoText2*)sep->getChild(1);

     sep->addChild(labelObject);

     makeLatLonString(label,bbox.getMin().getValue()[0],LONGITUDE);

     labelObject->string.setValue(label);

     sep = (SoTransformSeparator*)markerGroup->getChild(3);
     trans = (SoTranslation*)sep->getChild(0);
     trans->translation.setValue(bbox.getMax().getValue()[0],(bbox.getMax().getValue()[1]+bbox.getMin().getValue()[1])*0.5,20);
     labelObject =  (SoText2*)sep->getChild(1);
     makeLatLonString(label,bbox.getMax().getValue()[0],LONGITUDE);
     labelObject->string.setValue(label);

     sep = (SoTransformSeparator*)markerGroup->getChild(4);
     trans = (SoTranslation*)sep->getChild(0);
     trans->translation.setValue((bbox.getMin().getValue()[0]+bbox.getMax().getValue()[0])*0.5,bbox.getMin().getValue()[1],20);
     labelObject =  (SoText2*)sep->getChild(1);
     makeLatLonString(label,bbox.getMin().getValue()[1],LATITUDE);
     labelObject->string.setValue(label);

     sep = (SoTransformSeparator*)markerGroup->getChild(5);
     trans = (SoTranslation*)sep->getChild(0);
     trans->translation.setValue((bbox.getMin().getValue()[0]+bbox.getMax().getValue()[0])*0.5,bbox.getMax().getValue()[1],20);
     labelObject =  (SoText2*)sep->getChild(1);
     makeLatLonString(label,bbox.getMax().getValue()[1],LATITUDE);
     labelObject->string.setValue(label);

     details = (SoGroup*)markerGroup->getChild(6);
     for (int stepIndex = 0; stepIndex<numsteps; stepIndex++) {
       step = steps[stepIndex];
       childIndex=0;
       detailGroup = (SoGroup*)details->getChild(stepIndex);
       xsteps=int((bbox.getMax().getValue()[0]-bbox.getMin().getValue()[0])/step-0.1);
       ysteps=int((bbox.getMax().getValue()[1]-bbox.getMin().getValue()[1])/step-0.1);
       
       x=bbox.getMin().getValue()[0]+step;
       y=bbox.getMin().getValue()[1]; 
       for (i=0;i<xsteps; i++) {
	     sep = (SoTransformSeparator*)detailGroup->getChild(childIndex);
	     trans = (SoTranslation*)sep->getChild(0);            
	     trans->translation.setValue(x,y+step*0.5,20);
             labelObject = (SoText2*)sep->getChild(1);
	     makeLatLonString(label,i,LONGITUDE);
	     labelObject->string.setValue(label);
	     childIndex++;
             x+=step;
       }

       x=bbox.getMin().getValue()[0];    
       y=bbox.getMin().getValue()[1]+step;
       for (j=0; j<ysteps; j++) {
	     sep = (SoTransformSeparator*)detailGroup->getChild(childIndex);
	     trans = (SoTranslation*)sep->getChild(0);            
	     trans->translation.setValue(x+step*0.5,y,20);
             labelObject = (SoText2*)sep->getChild(1);
	     makeLatLonString(label,i,LATITUDE);
	     labelObject->string.setValue(label);
	     childIndex++;
             y+=step;
       }

       x=bbox.getMin().getValue()[0]+step;    
       y=bbox.getMin().getValue()[1]+step;
       for (i=0;i<xsteps; i++) {
           for (j=0; j<ysteps; j++) {
	     sep = (SoTransformSeparator*)detailGroup->getChild(childIndex);
	     trans = (SoTranslation*)sep->getChild(0);            
	     trans->translation.setValue(x,y+step*0.5,20);
             labelObject = (SoText2*)sep->getChild(1);
	     makeLatLonString(label,i,LATITUDE);
	     labelObject->string.setValue(label);
	     childIndex++;

	     sep = (SoTransformSeparator*)detailGroup->getChild(childIndex);
	     trans =  (SoTranslation*)sep->getChild(0);            
	     trans->translation.setValue(x+step*0.5,y,20);
             labelObject = (SoText2*)sep->getChild(1);
	     makeLatLonString(label,i,LONGITUDE);
	     labelObject->string.setValue(label);
	     childIndex++;
             y+=step;
          }
          x+=step;
       }
    }
  }

 
   // IvSeaSurface is a translucent rectangle that can be shown and hidden
   class IvSeaSurface: public SoSwitch {
      protected:
         ~IvSeaSurface() { };
      public:
         IvSeaSurface(float seaRed, float seaGreen, float seaBlue);
         /* if a viewport and a map are provided, then the bounding box is recalculated to encompass the map.
         Otherwise use the values already in bbox */
         void updateSurfaceBox(SbBox3f& bbox, const SbViewportRegion *vp=NULL,  SoNode* map=NULL);
         inline void hide() { whichChild.setValue(SO_SWITCH_NONE); }
         inline void show() { whichChild.setValue(SO_SWITCH_ALL);  }
         inline bool isVisible() const { return whichChild.getValue() != SO_SWITCH_NONE; }
         void setColor(float seaRed, float seaGreen, float seaBlue);
   };

   IvSeaSurface::IvSeaSurface(float seaRed, float seaGreen, float seaBlue):SoSwitch(4) {
      whichChild=SO_SWITCH_ALL;
      SoMaterialBinding *mbinding=new SoMaterialBinding;
      mbinding->value=SoMaterialBinding::OVERALL;
      addChild(mbinding);
      SoMaterial *boxMat = new SoMaterial;
      addChild(boxMat);
      setColor(seaRed, seaGreen, seaBlue);
      SoCoordinate3 *boundcoord = new SoCoordinate3;
      boundcoord->point.setNum(5);
      addChild(boundcoord);
      SoFaceSet *boundary = new SoFaceSet;
      boundary->numVertices.setNum(1);
      boundary->numVertices.set1Value(0,5);
      addChild(boundary);
   }

   void IvSeaSurface::updateSurfaceBox(SbBox3f& bbox, const SbViewportRegion *vp,  SoNode* map) {
      /* if we're provided viewport and a map, then recalculate the bounding box around the map.
         Otherwise we use the values already in bbox */
      if (vp!=NULL && map!=NULL) {
         SoGetBoundingBoxAction bboxAction(*vp);
         bboxAction.apply(map);
         bbox = bboxAction.getBoundingBox();
      }
      SbVec3f *boundcoord = ((SoCoordinate3*)this->getChild(2))->point.startEditing();
      boundcoord[0].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[1].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[2].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[3].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[4].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      ((SoCoordinate3*)this->getChild(0))->point.finishEditing();
   }

   void IvSeaSurface::setColor(float r, float g, float b){
       SoMaterial *boxMat = (SoMaterial*)getChild(1);
       boxMat->ambientColor.set1Value(0,r,g,b);
       boxMat->diffuseColor.set1Value(0,r,g,b);
       boxMat->specularColor.set1Value(0,r,g,b);
       boxMat->emissiveColor.set1Value(0,r,g,b);
       boxMat->transparency.set1Value(0,0.8);
   }


   /* This function will update a box around an object created with a call to createBoundingBoxAround()
     to the current coordinates of the object's bounding box. */
   void updateBoundingBoxAround(SoGroup* map, SoGroup* mapBox, SbBox3f& bbox, const SbViewportRegion *vp=NULL) {
      printf("updateBoundingBoxAround:begin\n");
      if (vp) {
         SoGetBoundingBoxAction bboxAction =(*vp);
         bboxAction.apply(map);
         bbox = bboxAction.getBoundingBox();
      }
      // IMPORTANT: Getting boundcoord right depends on the bounding box geometry having been created with a call to createBoundingBoxAround()
      SbVec3f *boundcoord = ((SoCoordinate3*)mapBox->getChild(2))->point.startEditing();
      boundcoord[0].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[1].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[2].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[3].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[4].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
   
      boundcoord[5].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[6].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[7].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[8].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[9].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
   
      boundcoord[10].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[11].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
   
      boundcoord[12].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[13].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],0);
   
      boundcoord[14].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[15].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],0);
   
      boundcoord[16].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[17].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],0);
      ((SoCoordinate3*)mapBox->getChild(0))->point.finishEditing();
      cout << "bathymetry bbox min = [" << bbox.getMin().getValue()[0] << ',' << bbox.getMin().getValue()[1] << ',' << bbox.getMin().getValue()[2] << "],\n" << endl;
      cout << "                max = [" << bbox.getMax().getValue()[0] << ',' << bbox.getMax().getValue()[1] << ',' << bbox.getMax().getValue()[2] << "]\n" << endl;
   }

   /* This function will create an Inventor group that draws a box around an object's bounding box. */
   SoGroup* createBoundingBoxAround(SoGroup* parent, SoGroup* map, SbBox3f& bbox, const SbViewportRegion *vp) {
      printf("Entering createBoundingBoxAround()\n");
      SoGroup *boxroot = new SoGroup;
      parent->addChild(boxroot);
      boxroot->setName("map box");
      SoMaterialBinding *mbinding=new SoMaterialBinding;
      mbinding->value=SoMaterialBinding::OVERALL;
      boxroot->addChild(mbinding);
      SoBaseColor *color= new SoBaseColor;
      // make the box light blue
      color->rgb.setValue(0.0f,0.235f,1.0f);
      boxroot->addChild(color);
      SoCoordinate3 *boundcoord = new SoCoordinate3;
      boundcoord->point.setNum(18); // A pairs of 5 pts for the top and bottom, plus 4 pairs to join the corners
      boxroot->addChild(boundcoord);
      printf("Calling updateBoundingBoxAround()\n");
      updateBoundingBoxAround(map,boxroot,bbox,vp);
      printf("Back to createBoundingBoxAround()\n");
      SoLineSet *boundary = new SoLineSet;
      boundary->numVertices.setNum(6);
      boundary->numVertices.set1Value(0,5);
      boundary->numVertices.set1Value(1,5);
      boundary->numVertices.set1Value(2,2);
      boundary->numVertices.set1Value(3,2);
      boundary->numVertices.set1Value(4,2);
      boundary->numVertices.set1Value(5,2);
      boxroot->addChild(boundary);
      return boxroot;
   }

   // Called by the viewer when the slider dynamically changes Z scaling
   bool setVerticalExaggeration(SoScale *mapScaling,unsigned int val) {
      TILE_EXAGGERATION_Z=val;
     //change the exaggeration of the bathymetry
      mapScaling->scaleFactor.setValue(mapScaling->scaleFactor.getValue()[0],mapScaling->scaleFactor.getValue()[1],TILE_EXAGGERATION_Z);
     //change the scale of the tracks and contacts
      trackDirectory.applyScaleToTracks(mapScaling->scaleFactor.getValue()[0],mapScaling->scaleFactor.getValue()[1],mapScaling->scaleFactor.getValue()[2]);
     // we always accept the scaling factor sent, so we always return true
      return true;
   }


   // Called by the viewer when the user requested to toggle wireframe rendering on/off
   bool  toggleWireframeMode(bool drawAsWireframe) {
      if(drawAsWireframe)
         setBathymetryToWireframe(bathymetryStyle);
      else
         setBathymetryToSolid(bathymetryStyle);
    // we always accept request to toggle drawing style, so we return true
      return true;
   }



   SoGroup* createSceneGraph(SoGroup* root,
                     SoScale*& mapScaling,   						
                     SoTranslation*& mapCentering,
                     IvTiledBathymetry*& bathymetry,
                     SoDrawStyle*& bathymetryInWireframe,
                     SoSeparator*& ownshipIcon, SoSeparator*& ownshipMarker,
                     SoSeparator*& targetMarker,
                     SoSeparator*& weaponIcon,  SoSeparator*& weaponMarker,
                     SoSwitch*& colormapSwitch, SoSwitch *& textureSwitch,
                     SoTexture2Transform *& rangeRingCenter) {
      SoInput loader;
      SoGroup *group;
      SoSeparator *mapRoot;
      SoNode *node;

      /* add a light close to the horizon, lighting the map from the side,
         to add contrast to the map features. */
      SoDirectionalLight *spotLight = new SoDirectionalLight;
      spotLight->intensity.setValue(1.0);
      spotLight->direction.setValue(0.57735,0.57735,-0.57735);
      root->addChild(spotLight);
     
      mapRoot = new SoSeparator;
      root->addChild(mapRoot);

      mapScaling = new SoScale;
      // set scaling factor so that everything is in meters (bathymetry tiles are in degrees in X and Y, and in meters in Z), plus some vertical exaggeration
      mapScaling->scaleFactor.setValue(TILE_SCALING_TO_METERS_X,TILE_SCALING_TO_METERS_Y,TILE_EXAGGERATION_Z);
      mapRoot->addChild(mapScaling);   

      mapCentering = new SoTranslation;
      mapRoot->addChild(mapCentering);   
   
   // load colormaps
   
      colormapSwitch = new SoSwitch();
      mapRoot->addChild(colormapSwitch);
   
      loader.openFile("color.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
   // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
   /* Now remove parent node added by loader.
      IMPORTANT:This works only if the colormap was the only child,
      which we know is true because we made the colormaps */
      group->removeChild(node);
      group->unref();
   
      loader.openFile("sandscale.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
   // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
      group->removeChild(node);
      group->unref();
   
      loader.openFile("bluescale.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
      // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
      group->removeChild(node);
      group->unref();
   

      // set default colormap
      colormapSwitch->whichChild.setValue(0);

      // set style to be able to render bathymetry as wireframe when required
      bathymetryInWireframe = new SoDrawStyle;
      bathymetryInWireframe->style = SoDrawStyle::FILLED;
      mapRoot->addChild(bathymetryInWireframe);
   
      SoMaterialBinding *mapBinding = new SoMaterialBinding;
      mapBinding->value.setValue(SoMaterialBinding::PER_VERTEX_INDEXED);
      mapRoot->addChild(mapBinding);
   
      textureSwitch=new SoSwitch;
      mapRoot->addChild(textureSwitch);
   
      // add a square texture on top of the map; cheap way to overlap a grid
      SoGroup *gridSep=new SoGroup;
      SoTexture2 *bottomgrid = new SoTexture2;
      gridSep->addChild(bottomgrid);
      bottomgrid->filename.setValue("square.rgb");
      bottomgrid->wrapS=SoTexture2::REPEAT;
      bottomgrid->wrapT=SoTexture2::REPEAT;
      SoTextureCoordinatePlane *texPlane1 = new SoTextureCoordinatePlane;
      texPlane1->directionS.setValue(SbVec3f(12,0,0));
      texPlane1->directionT.setValue(SbVec3f(0,12,0));
      gridSep->addChild(texPlane1);
      textureSwitch->addChild(gridSep);
   
      // add a range ring  texture on top of the map
      SoGroup *ringSep=new SoGroup;
      SoTexture2 *bottomcircle = new SoTexture2;
      ringSep->addChild(bottomcircle);
      bottomcircle->filename.setValue("circles.rgb");
      bottomcircle->wrapS=SoTexture2::CLAMP;
      bottomcircle->wrapT=SoTexture2::CLAMP;
      SoTexture2Transform *scale = new SoTexture2Transform;
      rangeRingCenter = new SoTexture2Transform;
      rangeRingCenter->translation.setValue(0,0);
      scale->translation.setValue(0.5,0.5);
      //1 km range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=111km*20/512
      //scale->scaleFactor.setValue(4.3359375,4.3359375);
      //1 kyd range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=121.36kyd*20/512
      //scale->scaleFactor.setValue(4.740625,4.740625);
      //2 kyd range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=121.36kyd*20/512 /2
      scale->scaleFactor.setValue(2.3703125,2.3703125);
      ringSep->addChild(scale);
      ringSep->addChild(rangeRingCenter);
      SoTextureCoordinatePlane *texPlane2 = new SoTextureCoordinatePlane;
      //1 km range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=111km*20/512
      texPlane2->directionS.setValue(SbVec3f(1,0,0));
      texPlane2->directionT.setValue(SbVec3f(0,1,0));
      ringSep->addChild(texPlane2);
      textureSwitch->addChild(ringSep);
   
   // start with range rings
      textureSwitch->whichChild.setValue(1);
   
   // create tiled bathymetry object
      bathymetry = new IvTiledBathymetry();
      mapRoot->addChild(bathymetry);
   
   // load icon for ownship
      loader.openFile("goodsub.iv");
      ownshipIcon = SoDB::readAll(&loader);
      loader.closeFile();
   
   // load marker icon for ownship's track
      loader.openFile("goodsub_marker.iv");
      ownshipMarker = SoDB::readAll(&loader);
      loader.closeFile();
   
   
   // load marker icon for target contact's track
      loader.openFile("track_marker.iv");
      targetMarker = SoDB::readAll(&loader);
      loader.closeFile();
   
   
   // load icon for weapon
      loader.openFile("weapon.iv");
      weaponIcon = SoDB::readAll(&loader);
      loader.closeFile();
   
   // load marker icon for weapon's track
      loader.openFile("weapon_marker.iv");
      weaponMarker = SoDB::readAll(&loader);
      loader.closeFile();
   
      return mapRoot;
   } 

   DiverseHUD* createHUD(SoGroup* root, SbViewportRegion* viewport, SoMFColor& colormap,
                   IvColorScale*& colorscale , SelectionBoxes& selectionBoxes, IvContactLabeler2D*& contactLabeler, IvCompass*& compass) {
      DiverseHUD *hud = new DiverseHUD(viewport);
      root->addChild(hud);
      // makes the HUD objects not shiny
      SoLightModel *lm = new SoLightModel;
      lm->model.setValue(SoLightModel::BASE_COLOR);
      hud->addChild(lm);
      colorscale = new IvColorScale;
      colorscale->setSize(30,200);
      colorscale->setPosition(8,16);
      colorscale->setColormap(colormap);
      colorscale->setRange(-DEEP_THRESHOLD,0,9);
      hud->addChild(colorscale);
   
      selectionBoxes.targetSelection = new IvContactViewHighlighter2D(hud);
      selectionBoxes.targetSelection->setColor(CONTACT_LABEL_COLOR[0],CONTACT_LABEL_COLOR[1],CONTACT_LABEL_COLOR[2]);
      hud->addChild(selectionBoxes.targetSelection);

      selectionBoxes.weaponSelection = new IvContactViewHighlighter2D(hud);
      selectionBoxes.weaponSelection->setColor(WEAPON_LABEL_COLOR[0],WEAPON_LABEL_COLOR[1],WEAPON_LABEL_COLOR[2]);
      hud->addChild(selectionBoxes.weaponSelection);

      contactLabeler = new IvContactLabeler2D(hud);
      hud->addChild(contactLabeler);     
   
      compass = new IvCompass;
      compass->setSize(150,150);
      compass->setPosition(mainWindowRenderingData.viewport.getWindowSize().getValue()[0]-300,20);
      hud->addChild(compass);

      return hud;
   }


   /* This function calculates how far the camera should be from the scene to have the are in 'box' completely visible */
   void calculateDistanceToViewAll(float& neededDistance, float& newNearClipDistance, float& newFarClipDistance, 
               const SbBox3f& box, float slack = 0.0f, float cameraAspectRatio = 1.0f) {
      // calculate box diagonal, that's the maximum possible length of the object bouded by the box
      float x=box.getMax()[0]-box.getMin()[0];
      float y=box.getMax()[1]-box.getMin()[1];
      float z=box.getMax()[2]-box.getMin()[2];
      float radius=sqrt(x*x+y*y+z*z);
   
      neededDistance = radius / cameraAspectRatio;
   
      newNearClipDistance  = neededDistance - radius;
      if (newNearClipDistance < 0.0 )
         newNearClipDistance = 0;
      newFarClipDistance   = neededDistance + radius;
      printf("viewAll() distance = %f\n",neededDistance);
   }

   /* This is a helper function that does all the work of figuring where to put the scene so 'aNode' is visible, with an inclination of 'tiltAngle' on the X axis */
   void overviewObject(SoNode *aNode, const SbViewportRegion& vp, const float tiltAngle, const float cameraAspectRatio, SoTranslation* sceneCenter, const SoScale *scaling, SoRotation *rot, SoTranslation* sceneOffset,float& nearClip, float& farClip) {
      float distance;
      float xmin,ymin,zmin,xmax,ymax,zmax, center[3];
      /* Clear translations - since overviewObject() "pushes away" the object to make it visible,
         if we don't reset Z every time we call overviewObject(), it would get farther and farther away of the origin. */
      sceneCenter->translation.setValue(0,0,0);
      sceneOffset->translation.setValue(0,0,0);
      SoGetBoundingBoxAction bboxAction(vp);
      bboxAction.apply(aNode);
      bboxAction.getXfBoundingBox().getBounds(xmin,ymin,zmin,xmax,ymax,zmax);
      SbBox3f bbox = bboxAction.getBoundingBox();
      calculateDistanceToViewAll(distance,nearClip,farClip,bbox,1.0f,cameraAspectRatio);
      printf("viewAll() distance = %f\n",distance);
      center[0]=(xmin+xmax)/2;
      center[1]=(ymin+ymax)/2;
      center[2]=(zmin+zmax)/2;
      printf("overviewObject():\n\tnon-transformed map bbox min = [%f, %f, %f], max = [%f,%f,%f]\n",xmin,ymin,zmin,xmax,ymax,zmax);
      printf("overviewObject():\n\ttransformed map bbox min = [%f, %f, %f], max = [%f,%f,%f]\n",bbox.getMin()[0],bbox.getMin()[1],bbox.getMin()[2],bbox.getMax()[0],bbox.getMax()[1],bbox.getMax()[2]);
      rot->rotation.setValue(SbVec3f(1,0,0),tiltAngle);
      sceneCenter->translation.setValue(-center[0],-center[1],-center[2]);
      sceneOffset->translation.setValue(0,0,-distance-(bbox.getMin()[2]+bbox.getMax()[2])/2);
      /* IMPORTANT! note that here we're pivoting over the Y axis, even though we moved the scene away on Z.
           We have to do this because our drawing callback rotates the scene 90 degrees, so that Z becomes "front" instead of "up" */
	float T1, T2, T3, T4;
	rot->rotation.getValue().getValue(T1, T2, T3, T4);
   }


   /* Moves the scene so that the camera is pointing at the center of the bounding box of the Inventor object 'aNode', an the object is completely visible.
      The object will be visible from the top.
      Theis function is not completely generic, scene manipulation is dependent upong the shape of the scene built in main() */
   void setTopDownOverview(SoNode* aNode, const SbViewportRegion& vp, const float cameraAspectRatio,  SoTranslation* sceneCenter, const SoScale *scaling, SoRotation *rot, SoTranslation* sceneOffset) {
      float nearClip, farClip;
      // reset DGL navigator (if any) to put camera in a known position
      dMatrix Temp;
      Temp.makeIdentity();
      NavManager::setMatrix(Temp);
      overviewObject(aNode,vp,0.0f,cameraAspectRatio,sceneCenter,scaling,rot,sceneOffset,nearClip,farClip);
   }

   /* Moves the scene so that the camera is pointing at the center of the bounding box of the Inventor object 'aNode', an the object is completely visible.
      The scene tilted will be tilted 'mapTiltAngleInRadians'  on the X axis.
      Theis function is not completely generic, scene manipulation is dependent upong the shape of the scene built in main() */
   void setTopDownPerspectiveOverview(SoNode* aNode, float  mapTiltAngleInRadians, const SbViewportRegion& vp, const float cameraAspectRatio, SoTranslation * sceneCenter, const SoScale *scaling, SoRotation *rot, SoTranslation *sceneOffset) {
      float nearClip, farClip;
      printf("begin setTopDownPerspectiveOverview()\n");
      // reset DGL navigator (if any) to put camera in a known position
      dMatrix Temp;
      Temp.makeIdentity();
      NavManager::setMatrix(Temp);
      overviewObject(aNode,vp, mapTiltAngleInRadians,cameraAspectRatio,sceneCenter,scaling,rot,sceneOffset,nearClip,farClip);
      printf("end setTopDownPerspectiveOverview()\n");
   }
 
   /* Moves the scene so that the camera is pointing at the point 'centerPos', with the scene tilted 'mapTiltAngleInRadians'  on the X axis.
      Theis function is not completely generic, scene manipulation is dependent upong the shape of the scene built in main() */
   void setViewCenteredOn(const SbVec3f& centerPos, const SbViewportRegion& vp, const float cameraAspectRatio, SoTranslation *sceneCenter, float mapTiltAngleInRadians, SoRotation* mapRotation, SoTranslation *sceneOffset) {
   #define ENCLOSING_VIEW_X 5000.0f
   #define ENCLOSING_VIEW_Y 5000.0f
   #define ENCLOSING_VIEW_Z 250.0f
      float distance,nearClip,farClip;
      const float *xyz = centerPos.getValue();
      printf("setViewCenteredOn(%f,%f,%f)\n",xyz[0],xyz[1],xyz[2]);
      SbBox3f enclosingBox;
      enclosingBox.setBounds(xyz[0]-ENCLOSING_VIEW_X,xyz[1]-ENCLOSING_VIEW_Y,xyz[2]-ENCLOSING_VIEW_Z,
                           xyz[0]+ENCLOSING_VIEW_X,xyz[1]+ENCLOSING_VIEW_Y,xyz[2]+ENCLOSING_VIEW_Z);;
      calculateDistanceToViewAll(distance,nearClip,farClip,enclosingBox,cameraAspectRatio);
      sceneCenter->translation.setValue(-centerPos[0],-centerPos[1],-centerPos[2]);
      mapRotation->rotation.setValue(SbVec3f(1,0,0),mapTiltAngleInRadians);
      sceneOffset->translation.setValue(0,0,-distance);
      dMatrix Temp;
      Temp.makeIdentity();
      NavManager::setMatrix(Temp);
   }

   SbBox3f fillMapCenteredOn(IvTiledBathymetry* bathymetry, SbViewportRegion* renderArea, float x, float y) {
      bathymetry->fillVisibleAreaWithTiles(Rectangle2dFloat(x-DEFAULT_MAP_RADIUS_AROUND_X,y-DEFAULT_MAP_RADIUS_AROUND_Y, x+DEFAULT_MAP_RADIUS_AROUND_X,y+DEFAULT_MAP_RADIUS_AROUND_Y));
      SoGetBoundingBoxAction action(*renderArea);
      action.apply(bathymetry);
      return action.getBoundingBox();
   }

   void cleanUpAndFinish(int signal = 0) {
   // IMPORTANT! clean-up shared memory
      if(talkWithBezel != NULL) delete talkWithBezel;
   // close socket connection to feeder before finishing
      feeder.closeConnection();
      exit(1);
   }


/*** feeder functions ****/

   void stopFeeder() {
      feeder.sendByte(FeederCommand::STOP_FEEDER);
   }

   void resumeFeeder() {
      feeder.sendByte(FeederCommand::RESUME_FEEDER);
   }

   void tellFeederToDie() {
      feeder.sendByte(FeederCommand::QUIT);
   }

   // Pick only if it's one of the contact icons. Called by a SoSelection node
   SoPath *pickFilter(void * selectionObject, const SoPickedPoint *pick)
   {
   // Find out which child of selection got picked
      SoNode *n;
      SoPath *path = pick->getPath();
      int i =0;
      while (i < path->getLength()) {
         n = path->getNode(i);
         if (n->isOfType(IvContactView::getClassTypeId())) // is it a contact view?
            break;
         i++;
      }
   	// was a IvContactView picked?
      if(i != path->getLength()) {
         // we found an IvContactView; discard the path after it, we don't care about the rest
         path->truncate(i+1);
         SelectionBoxes *selectionBoxes = (SelectionBoxes*)selectionObject;
         DiscreteTrack *track = dynamic_cast<DiscreteTrack*>( ((IvContactView*)(n))->getModel() );
         switch(track->getContactType()) {
            case CONTACT_IS_WEAPON:
               if(talkWithBezel->selectedWeapon != BAD_OID)
                  ((IvHighlightSelection2D*)(selectionBoxes->weaponSelection))->deselect();
               ((IvHighlightSelection2D*)(selectionBoxes->weaponSelection))->setSelection(path);
               talkWithBezel->selectedWeapon = track;
               talkWithBezel->askBezel(BezelCommand::SELECT_CONTACT,OID::fromObject(track));
               break;
            case CONTACT_IS_VEHICLE:
            case CONTACT_FROM_SENSOR:
               if(talkWithBezel->selectedTarget != BAD_OID)
                  ((IvHighlightSelection2D*)(selectionBoxes->targetSelection))->deselect();
               ((IvHighlightSelection2D*)(selectionBoxes->targetSelection))->setSelection(path);
               talkWithBezel->selectedTarget = track;
               talkWithBezel->askBezel(BezelCommand::SELECT_CONTACT,OID::fromObject(track));
         }
       	 // resend the track's current position to the views, so the UI gets the current position of the new selection
         track->changed(TrackEvent::RESENDING_TRACK_POS);
         // we need to return the path to the SoSelection object for the SoSelection to force a repaint - strange...
         return path;
      }
   /* fool the SoSelection into thinking it failed to pick; we don't need it to actually
      keep any path info in the SoSelection, since we're keeping that information
      by ourselves in the selectionObject */
      return NULL;
   }


   SoPath* makePath(SoNode* root, SoNode* finalNode) {
      SoSearchAction nodeFinder;
      nodeFinder.setNode(finalNode);
      nodeFinder.setInterest(SoSearchAction::FIRST);
      nodeFinder.apply(root);
      SoPath* pathToNode = nodeFinder.getPath();
      pathToNode->ref();
      return pathToNode;
   }

   bool connectToFeeder(FeederConnection& feeder) {
      unsigned int attempts = 0;
      #define MAX_ATTEMPTS_TO_CONNECT 5
      while(attempts < MAX_ATTEMPTS_TO_CONNECT && feeder.connectToFeeder() < 0) {
         attempts++;
         usleep(500000); // wait for 1/2 second
      }
      return attempts != MAX_ATTEMPTS_TO_CONNECT;
   }


   SoSeparator* get3dIconFor(const ContactIconCollection3D& icons, Category vehicleID) {
      SoSeparator *icon;
      if(vehicleID >= icons.getSize()) {
         return (SoSeparator*)icons.getDefaultIcon()->copy();
      }
      else {
         icon = (SoSeparator*)icons[(unsigned int)vehicleID];
         if(icon != NULL)
            return (SoSeparator*)icon->copy();
         else
            return (SoSeparator*)icons.getDefaultIcon()->copy();
      }
   }

   /* This function must setup the 2D interface, also called "the bezel". Since till
      now the UI is a separate program, this function executes the bezel program */
   void setup2dUserInterface(const char* xdisplay) {	   
       static char* environ[] = { "DISPLAY=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx", NULL }; 
       static char* argv[] = { "bezel", NULL };
       if(fork()==0) { // we're the child
            if(xdisplay==NULL) {
                xdisplay=getenv("DISPLAY");
	        if(xdisplay==NULL)
		    xdisplay=":0.0";
            }
	    strcpy(environ[0]+8,xdisplay); // put value of DISPLAY env. var. immediately after DISPLAY= in environ[0]
            printf("bezel will appear on DISPLAY %s\n",xdisplay);
	    if(execve("./bezel/bezel",argv,environ) == -1) {
              printf("ERROR: Can't find bezel program\n");
	      exit(-1);
            }
       }
   } 

   void printHelp(const char* thisProgramName) {
     printf("USAGE: %s [-w | --xwand] [-a | --angle <pitch>][-h | --help]\n",thisProgramName);
     printf("\t-w, --xwand\t Tells this program it should xwand to control cursor in a VR display.\n");
     printf("\t-a, --angle <pitch>\t Map pitch. Inclination in degrees to show the map.\n");
     printf("\t-h, --help\t Prints this help and exits.\n");
   }

   void setWindowSize(unsigned int left, unsigned int bottom, unsigned int width, unsigned int height) {
       vector<DDisplay*> dglDisplays = DisplayManager::getDisplays();
       vector<DWindow*> dglWindows = dglDisplays[0]->getWindows();
       dglWindows[0]->setWindowGeometry(Rect(left,bottom,width,height));
   }

   
   // Finds the X Display named 'xdisplayName' among the displays being used by Diverse
   Display* findXDisplayInDglNamed(const char* xdisplayName) {
       Display* dpy = NULL;
       vector<DDisplay*> dglDisplays = DisplayManager::getDisplays();
       // try to find which a dgl display with the same string
       for(vector<DDisplay*>:: iterator iter = dglDisplays.begin(); iter !=  dglDisplays.end(); ++iter) {
          vector<DWindow*> dglWindows = (*iter)->getWindows();
          Producer::RenderSurface *mySurface = (Producer::RenderSurface*)dglWindows[0]->getRenderSurface();
          if(mySurface != NULL) {
              dpy = mySurface->getDisplay();
              if(strcmp(xdisplayName,DisplayString(dpy)) ) {
                   break;
              }
          }
       }
       return dpy;
   }


   // Returns the X Display and X Window of the first display, as defined by the currently-used display DSO
   void getX11OutputDisplayAndWindow(Display*& dpy, Window& window) {
         vector<DDisplay*> dglDisplays = DisplayManager::getDisplays();
        //Get the dgl window data out
	vector<DWindow*> dglWindows = dglDisplays[0]->getWindows();
        Producer::RenderSurface* mySurface = (Producer::RenderSurface*)dglWindows[0]->getRenderSurface();
	if(mySurface == NULL) {
	    dpy = NULL;
	    window = NULL;
        } else {
            dpy = mySurface->getDisplay();
            window = mySurface->getWindow();
        }
    }


    // this function is used mainly during debugging
    void printCurrentHeadPosition() {                     
        printf("------------------------ CURRENT HEAD TRACKER -------------------------\n");
        dVec3 vec3;
	dMatrix navmatrix=  NavManager::getMatrix();
	vec3=navmatrix.getTrans();
        printf("Current Position: XYZ=[%f,%f,%f],", vec3[0],vec3[1],vec3[2]);
        DTrackerEvent *evt=EventManager::getTrackerState("headtracker");
        vec3=evt->getDirection();                     
        printf(" HPR=[%f,%f,%f]\n",vec3[0],vec3[1],vec3[2]);
        printf("-----------------------------------------------------------------------\n");

    }

//////////////////////////////////////////////////////
//////////                MAIN             ///////////
//////////////////////////////////////////////////////

   int main(int argc, char**argv) {      
      int i; // just a generic loop var
      float mapTiltAngle=-45.0f; // the inclination angle for the map - here's in degrees, but throughout the program it will be in radians (see further below)

      bool seaSurfaceIsVisible=false; // this flags memorizes the current visibility state of the sea surface
      bool useXwand=false;
      bool xwandIsActive=false;
      Cursor wandCursor;
      Cursor invisibleCursor; // this cursor (a blank one) is only used if we have to use xwand
      Display *renderingDisplay, *xwandDisplay; // if we're going to use xwand, this holds which X11 Display to use
      Window xwandWindow;      // window where the xwand will control the cursor (only if xwand is used)
      XWandFakeMouse fakeMouse; // if we're going to use xwand, this object IS the xwand
      unsigned int xwandInitialPosX,xwandInitialPosY; // initial position of  xwand pointer

      SoXt::init(argv[0]);
   
      // take a look at the program arguments
      for(i=1; i<argc; i++) {
          if (strcmp(argv[i],"-h")==0 || strcmp(argv[i],"--help")==0) {
            printHelp(argv[0]);
            return EXIT_SUCCESS;
          } else if (strcmp(argv[i],"-w")==0 || strcmp(argv[i],"--xwand")==0) {
	       useXwand=true;
          } else if (strcmp(argv[i],"-a")==0 || strcmp(argv[i],"--angle")==0) {
               char *lastdigit;
	       i++;
               // do we have a valid number in degrees for the map tilting angle?
	       if(i>=argc || ((mapTiltAngle=strtod(argv[i],&lastdigit))==0 && lastdigit==argv[i])) {
	           printHelp(argv[0]);
                   return EXIT_FAILURE;
               }
               if(mapTiltAngle < -360.0f)
	             mapTiltAngle=-360.0f;
               else if(mapTiltAngle > 360.0f)
	             mapTiltAngle=360.0f;
          } else printf("%s: unknown argument '%s', ignoring it...\n",argv[0],argv[i]);
	  i++;
      }

      // from here on the mapTiltAngle must  ALWAYS be in radians
      mapTiltAngle=DEGREES_TO_RADIANS(mapTiltAngle);


      // Initialize DGL - It reads the config file, sets up navigators and renderers, etc      
      fflush(stdout);
      printf("Calling dglInit...\n");      
      fflush(stdout);
      dglInit(argc,argv);
      printf("Calling dglStart...\n");      
      fflush(stdout);
      dglStart(); 

      /* Set the moving (translation) speed. The number " 0.045 degrees" is purely based on experience,
         but since it is in degrees, it scales with the map; change the degrees-to-meters conversion,
         and you will also adjust the moving speed here automatically. */
      NavManager::setSpeedFactor(DEGREES_TO_METERS(0.045));

      // get DGL renderer
      fflush(stdout);
      printf("getting display list...\n");
      vector<DDisplay*> displayList = DisplayManager::getDisplays();
      printf("getting display #1...\n");
      DDisplay* dglDisplay = displayList[0];
      printf("Display = %s\n",dglDisplay->getDisplayName().c_str());
      displayList.clear();

      if (!dglDisplay) {
         printf("ERROR: Could not find a valid DGL display object. Exiting.\n");
         return EXIT_FAILURE;
      }

      getX11OutputDisplayAndWindow(renderingDisplay,xwandWindow);
      if (renderingDisplay == NULL) {
           printf("ERROR: Unable to get Window IDs from DGL output DSO. Exiting.\n");
          return EXIT_FAILURE;
      } 

      // calculate area left for the rendering window, after we substract what's taken by the bezel
      mainWindowRenderingData.windowLeft = 0;
      mainWindowRenderingData.windowBottom = BOTTOM_AREA_HEIGHT;

      mainWindowRenderingData.viewport.setWindowSize(min(MAX_DISPLAY_WIDTH,WidthOfScreen(DefaultScreenOfDisplay(renderingDisplay)))-RIGHT_AREA_WIDTH,
                                               min(MAX_DISPLAY_HEIGHT,HeightOfScreen(DefaultScreenOfDisplay(renderingDisplay)))-TOP_AREA_HEIGHT-BOTTOM_AREA_HEIGHT);

      setWindowSize(mainWindowRenderingData.windowLeft,
          mainWindowRenderingData.windowBottom,mainWindowRenderingData.viewport.getWindowSize().getValue()[0],mainWindowRenderingData.viewport.getWindowSize().getValue()[1]);


      dglPreConfigCallback( turnWindowBorderOffPreCallback ); 
      dglDisplayCallback( drawSceneGraphCallback );

      // if xwand is needed, then initialize it and create a pixmap for an invisible cursor
      if(useXwand) {
	if(!fakeMouse.initFromFile(XWandFakeMouse::XWandConfigDefaultFilename)) {
           printf("ERROR: Couldn't initialized xwand. Couldn't read XWand config file %s, XWand is DISABLED now.\n.",XWandFakeMouse::XWandConfigDefaultFilename);
           useXwand=false;
        } else {
	    fakeMouse.xwandData.printXWandDataTo(stdout);
            xwandDisplay=findXDisplayInDglNamed(fakeMouse.xwandData.display);
            if(xwandDisplay!=NULL) {
               /* if the xwand must appear in a display different from the one used for rendering window,
                  then cursor changes will affect the root window of that display. Otherwise the cursor
                  changes will affect the rendering window */
               if(xwandDisplay!=renderingDisplay) {
                   xwandWindow = DefaultRootWindow(xwandDisplay);
               }
 	       XColor whitePixel;
               whitePixel.pixel = WhitePixel(xwandDisplay,DefaultScreen(xwandDisplay));
               // create a pointer to use when xwand is on
               wandCursor = XCreateFontCursor(xwandDisplay,XC_arrow);
               // create one blank (invisible) cursor to fake hiding it when xwand is off
               Pixmap invisibleCursorPixmap = XCreateBitmapFromData(xwandDisplay,xwandWindow,invisibleCursorBits, INVISIBLECURSOR_WIDTH, INVISIBLECURSOR_HEIGHT);
               Pixmap invisibleCursorMask   = XCreateBitmapFromData(xwandDisplay,xwandWindow,invisibleCursorMaskBits, INVISIBLECURSOR_WIDTH, INVISIBLECURSOR_HEIGHT);
               invisibleCursor = XCreatePixmapCursor(xwandDisplay,invisibleCursorPixmap,invisibleCursorMask,&whitePixel,&whitePixel,0,0);
               XFreePixmap(xwandDisplay,invisibleCursorPixmap);
               XFreePixmap(xwandDisplay,invisibleCursorMask);
               printf("Using XWand, it will appear on display %s .\n",fakeMouse.xwandData.display);
	       // calculate where to show xwand pointer initially, when the xwand is on
               xwandInitialPosX = min(MAX_DISPLAY_WIDTH,WidthOfScreen(DefaultScreenOfDisplay(xwandDisplay)))/2;
               xwandInitialPosY = HeightOfScreen(DefaultScreenOfDisplay(xwandDisplay))/2;
            } else {
               printf("** WARNING! ** The display %s to be used by xwand is not any of the displays in which we will render - Check %s.\n\tXWand is disabled now.\n",fakeMouse.xwandData.display,XWandFakeMouse::XWandConfigDefaultFilename);
               useXwand=false;
            }
        }
      }
 
      // setup bezel (the 2D inteface around the rendering window) on the same display the rendering window is going to appear
      setup2dUserInterface(DisplayString(renderingDisplay));

   // Set signal handlers, to ensure cleaning up after anything goes wrong
      struct sigaction signalHandling;
      signalHandling.sa_handler = cleanUpAndFinish;
      signalHandling.sa_flags = 0;
      sigaction(SIGINT,&signalHandling,NULL);
      sigaction(SIGQUIT,&signalHandling,NULL);
      sigaction(SIGTERM,&signalHandling,NULL);


   /* create object that will communitate with bezel through shared memory,
      the object in itself is in shared memory */
      talkWithBezel = new(ObjectInSharedMemory::CREATE_WITH_KEY,UI_SHMEM_ID) UiMediator;
   
   // Establish connnection with contact data source
      cout << " Waiting for connection from Contact Data Feeder...\n";
      cout.flush();
      if(!connectToFeeder(feeder)) {
         cerr << endl << "Could not connect to data source. Exiting." << endl;
         cleanUpAndFinish();
      }

      cout << "feeder connected." << endl;
      cout.flush();
   
   /* Here initialize Inventor Types that are not built-in. This needs to
      be done after SoDB::init() is called, which is done by SoXt::init() */
      DiverseHUD::initClass();
      IvContactView::initClass();
      IvContactLabeler2D::initClass();
      IvColorScale::initClass();
      IvCompass::initClass();
      IvSwitchEngine::initClass();
      IvTransformEngine::initClass();
      IvTiledBathymetry::initClass();
   
   /* create root node for scene graph - root of scene graph must be a
      SoSelection if we want to be able to pick objects by clicking;
      otherwise we could just use a SoSeparator here */
      SoSelection *sceneGraphRoot = new SoSelection;
      sceneGraphRoot->ref();
 
      // these node names are not necessary, but they make a scene graph dump a lot easier to follow
      sceneGraphRoot->setName("sceneGraphRoot");   

      SoMaterial *m = new SoMaterial;
      m->shininess = 50;
      sceneGraphRoot->addChild(m);   
    

#ifdef DEBUG_DRAW_WORLD_AXIS
   /* This adds the 3 axes around (0,0,0), just to scene where the scene is.
      Useful only for debugging transformations, has not use otherwise. */
      SoMaterial *axisMaterial = new SoMaterial;
      axisMaterial->diffuseColor.setValue(1,0,0);
      sceneGraphRoot->addChild(axisMaterial);
      SoInput loader;
      loader.openFile("axis.iv");
      SoGroup *axisXYZ = SoDB::readAll(&loader);
      loader.closeFile();
      sceneGraphRoot->addChild(axisXYZ);
#endif



      // add an orthographic camera to render the minimap
      SoTransformSeparator *sceneSeparator = new SoTransformSeparator;
      sceneGraphRoot->addChild(sceneSeparator);
   
      sceneSeparator->setName("sceneSeparator");   

      SoSwitch *cameraSwitch = new SoSwitch;
      /* since the minimap follows the ownship, attach the camera
         to the same coordinate system of the ownship's icon */
      sceneSeparator->addChild(cameraSwitch);
   
      SoOrthographicCamera *topDownCamera = new SoOrthographicCamera;
      // 20k range
      topDownCamera->height.setValue(CAMERA_HEIGHT_IN_MINIMAP); // set height of camera view (works somewhat like distance)
      topDownCamera->nearDistance.setValue(0.0f); // be sure everything will be visible in the map
      topDownCamera->farDistance.setValue(FLT_MAX);
      topDownCamera->viewportMapping.setValue(SoCamera::ADJUST_CAMERA); // camera adjusts to viewport aspect ratio
      SbViewportRegion topDownViewport(MAP_IMAGE_X,MAP_IMAGE_Y); // size of top-down map image, must be set in the render area
      cameraSwitch->addChild(topDownCamera);
      #define TOPDOWN_SCENE_CAMERA_INDEX 0 // just a useful name for the child index of the orthographic camera
   
      // Here we turn all cameras off, but it's not really important, since we render with the camera controlled by DGL, not by Inventor
      cameraSwitch->whichChild.setValue(SO_SWITCH_NONE);

      // now setup scene graph
      SoSeparator *ownshipIcon, *ownshipTrackIcon,
         *contactIcon, *contactTrackIcon,
         *weaponIcon,  *weaponTrackIcon;
      SoSwitch     *colormapSwitch;
      SoGroup *mapRoot, *rootMapBox, *mapLocations;
      DiverseHUD *hud;
      IvColorScale *colorscale;
      SelectionBoxes selectionBoxes;
      IvContactLabeler2D* contactLabeler;
      SbBox3f mapBBox;
      SoScale *mapScaling;
      SoRotation *sceneRotation;
      SoTranslation *centerSceneTranslation, *sceneOffset, *iconSliding;
      IvTiledBathymetry *bathymetry;
      IvSeaSurface *seaSurface;
      IvCompass *compass;       

      // We use as a cheap way to project range rings over the map, they're always centered under the ownship
      SoSwitch * textureSwitch;
      SoTexture2Transform *rangeRingCenter;

      // This translation affects everything on screen, except the HUD
      sceneOffset = new SoTranslation;
      sceneSeparator->addChild(sceneOffset); 

      sceneOffset->setName("sceneOffset");

      // This rotation affects everything on the screen, except the HUD
      sceneRotation = new SoRotation;
      sceneSeparator->addChild(sceneRotation);

      // This is the collection of 3D icons we use to show ships, planes, etc
      ContactIconCollection3D iconCollection;
      // Load the collection of 3D icons
      if(iconCollection.load3dIconsFrom(CONTACT_ICONLIST_FILE) == ContactIconCollection3D::CANT_FIND_FILE) {
         printf("WARNING! Couldn't find icon list file '%s', all targets will look the same.\n",CONTACT_ICONLIST_FILE);
      }
      else if(iconCollection.getSize() == 0) {
        // for some reason, the icons weren't loaded
         printf("WARNING! '%s' doesn't have any valid find icon filename, all targets will look the same.\n",CONTACT_ICONLIST_FILE);
      }
      iconCollection.setDefaultIconTo(iconCollection.getSize()-1);    

      SoDirectionalLight *alight = new SoDirectionalLight;
      sceneGraphRoot->addChild(alight);

      mapRoot = createSceneGraph(sceneSeparator,
                           mapScaling,
                           centerSceneTranslation,
                           bathymetry,
                           bathymetryStyle,
                           ownshipIcon,ownshipTrackIcon,
                           contactTrackIcon,
                           weaponIcon,weaponTrackIcon,
                           colormapSwitch,textureSwitch,
                           rangeRingCenter);

      mapRoot->setName("mapRoot");
      mainWindowRenderingData.viewport.setViewport(0,0, mainWindowRenderingData.viewport.getWindowSize().getValue()[0], mainWindowRenderingData.viewport.getWindowSize().getValue()[1]);
      mainWindowRenderingData.rootNode = sceneGraphRoot;
      mainWindowRenderingData.renderer = new SoGLRenderAction(mainWindowRenderingData.viewport);
      #ifndef FAST_TRANSPARENCY
        mainWindowRenderingData.renderer->setTransparencyType(SoGLRenderAction::DELAYED_BLEND);
      #endif

      // Set the near and far clipping plane in all viewports so we don't have to fight with them later.
      DisplayManager::setNearClip(0.1); // a value much smaller than this produces weird problems in the Z-buffer
      DisplayManager::setFarClip(FLT_MAX);

      // Get field of view - we need it to calculate how far to put the scene graph to make it all 
      CameraProps* myCam = dglDisplay->getConfig()->windows[0]->viewports[0]->camera;
      mainWindowRenderingData.cameraAspectRatio = ((float)mainWindowRenderingData.viewport.getWindowSize().getValue()[1])/mainWindowRenderingData.viewport.getWindowSize().getValue()[0];
      printf("aspect ratio = %f\n",mainWindowRenderingData.cameraAspectRatio);
   

      // initialize bathymetry
      bathymetry->initInstance(Rectangle2dFloat(MAP_MIN_LON,MAP_MIN_LAT,MAP_MAX_LON,MAP_MAX_LAT), Point2dFloat(TILE_SIZE_X,TILE_SIZE_Y), Point2dUint(2 ,2), 0, 0, 1 , 1, "TILES/tile_");
      bathymetry->useFastBoundingBox(true,FT_TO_M(BATHYMETRY_MAX_DEPTH),0.0f);

      // create iconRoot, the root node of all icons for targets
      SoGroup *iconRoot = new SoSeparator;
      sceneSeparator->addChild(iconRoot);

      iconRoot->setName("iconRoot");

      // iconSliding will have the transformation to apply to both positions and points of all 3d icons
      iconSliding = new SoTranslation;

      iconSliding->setName("iconSliding");

      /* Here we make the icon's translation a function of the map's scaling and translation.
        The important thing here is that we multiply the scaling and translation in sceneOffset to make
        the translation in iconSliding, but we don't touch the scaling factor for icons, so the icons
        don't change shape or size, even if the map changes scale or size. */
      SoCalculator *iconCoordCalculator = new SoCalculator;
      iconCoordCalculator->A.connectFrom(&mapScaling->scaleFactor);
      iconCoordCalculator->B.connectFrom(&centerSceneTranslation->translation);
      iconCoordCalculator->expression.set1Value(0,"oB=vec3f(A[0]*B[0],A[1]*B[1],A[2]*B[2])");
      iconSliding->translation.connectFrom(&iconCoordCalculator->oB);
      iconRoot->addChild(iconSliding);
   
      hud = createHUD(sceneGraphRoot, &mainWindowRenderingData.viewport,
                     ((SoBaseColor*)colormapSwitch->getChild(0))->rgb,colorscale,
                     selectionBoxes, contactLabeler,compass);
  
      /* The next 2 engines (iconTransforms and sceneAllTransforms) solve the problem of 
         how to apply all the transformations affecting icons to the icon labels.
         Since the icon labels are drawn in 2D, any transformations accumulated
         when drawing the 3D icons are lost when we change to orthographic coordinates.
         What we do here is to use an Inventor engine to connect the transformations
         affecting 3D icons to the object that labels the icons, so that when
         one changes, the contactLabeler is notified and can apply the same
         transform to the label positions. */
      IvTransformEngine *iconTransforms = new IvTransformEngine,
                        *sceneAllTransforms = new IvTransformEngine;
      iconTransforms->translation.connectFrom(&iconSliding->translation);
      iconTransforms->rotation.connectFrom(&sceneRotation->rotation);
      sceneAllTransforms->inputMatrix.connectFrom(&iconTransforms->transfMatrix);
      sceneAllTransforms->translation.connectFrom(&sceneOffset->translation);
      contactLabeler->transform.connectFrom(&sceneAllTransforms->transfMatrix);

      // default color scale to SHALLOW
      changeColorscaleDepth(bathymetry,colorscale,SHALLOW_THRESHOLD);
   

      /* Initialize picking policy and callbacks. */
      // here we do all the setup (callbacks, etc) needed for object picking to work
      sceneGraphRoot->policy = SoSelection::SINGLE;
      // filter wich objects are picked
      sceneGraphRoot->setPickFilterCallback(pickFilter,&selectionBoxes);
   
      // setup off-screen renderer for top-down image (minimap) of the bathymetry
      SoOffscreenRenderer *imageRenderer = new SoOffscreenRenderer(topDownViewport);
      imageRenderer->setComponents(SoOffscreenRenderer::RGB);
   
   /* Get a SoPath from the scene graph root to the iconRoot node. We'll use this path
      when a contact needs to be highlighted. We could build this path by hand since
      we build the scene graph, but searching for the node after building
      the static part of the scene graph has the advantage that we can change
      the shape of the scene graph without changing how this path is obtained.
      Besides, we do this only once. */
      SoPath* pathToIconRoot = makePath(sceneGraphRoot,iconRoot);

   // Flag used to tell when we need to render a new minimap
      bool updateMiniMap = false;
      bool updateCamera = false;
   
   // Flag used to tell when we need to recenter the minimap
      bool recenterMiniMap = false;
   
   // This is a flag that tells how to update the bathymetry: following the ownship, or wherever the user wants to go
      MapUpdatingMode mapUpdatingMode = MAP_FOLLOWS_OWNSHIP;
   
   // Flag used to tell when we need to recenter the rangeRings
      bool showRings = true;
      	
      Timeline timeline; // object that manages all tracks
      // variables to create, keep track and display ships
      ContactEvent contactEvent;
      ObservableTrack *track, *ownshipTrack;
      IvContactView * contact;
      TrackTraceDecorator *trackTrace;
      IvTrackView *traceView;
      TrackUiView *trackUiView;
      CoordAndTime *coordAndTime;
      IvPreciseContactView *ownshipIconView;
      // This object is used to change the color of 3D icons when the threat class of an icon changes
      Iv3dIconColorizer iconColorizer;

      // this is a local contact event queue; right now we only use to to deal with Hypothetical Contacts created by the user
      ContactEventQueue localEventQueue; 
      bool newContactEventAvailable; // true if a new event is available from the feeder or from the local contact event queue

      int xtiles=0, ytiles=0; // how many map tiles to slide in X and Y when we get close to the edge of the map's visible area   
   
      int bytes_read; // number of bytes read from the feeder when a new contact data is available
      int contactEventsRead = 0; //  number of contact events received in one pass of the contact event loop

      // enable depth testing before starting to render, otherwise Inventor rendering doesn't do depth testing correctly
      glEnable(GL_DEPTH_TEST);
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);

      bool quitRenderingNow = false;     
      while(!talkWithBezel->readyToQuit() && !quitRenderingNow && dglIsRunning()) {
         contactEventsRead = 0;
         // process contact events
         while(contactEventsRead <= MAX_SIMULTANEOUS_CONTACT_EVENTS && (localEventQueue.isDataAvailable() || feeder.isDataAvailable())) {

             newContactEventAvailable=false;
             if(localEventQueue.isDataAvailable()) { // check for events from the local queue
                 localEventQueue.readEvent(contactEvent);
	         newContactEventAvailable=true;
             } else if((bytes_read = feeder.readEvent(contactEvent)) != -1) { // check for events from the feeder queue
                 /* check that the socket is still there, and that there actually was something to read, 
                    otherwise it may just be that the socket is down at the server side */            
	         if(bytes_read != 0) { 
	             newContactEventAvailable=true;
                 } else {
                     // something went wrong with the socket connection, close the socket
                     feeder.closeConnection();
                 }
             }
             if(newContactEventAvailable) { // if true, the variable 'contactEvent' holds the event data
                 contactEventsRead++;
                  track = trackDirectory.findTrackForContact(contactEvent.name,contactEvent.solution);
                  if(track == NULL) { // we don't have a track object associated to the track name given
                     // Create new track, associate a 3D icon to it, etc etc
                     track = trackDirectory.addTrackForContact(contactEvent.name,contactEvent.solution,contactEvent.contactType);
                     if(contactEvent.contactType == CONTACT_FROM_SENSOR) {
                        // create an AOU with the depth of the bathymetry
		        if (contactEvent.category==SUR) {
                        // create an AOU with the depth of the bathymetry
			  contact = new IvAOUView(-10,0);
			} else {
			  contact = new IvAOUView(- AOU_MAX_DEPTH,0);
                        }
		     } else if(contactEvent.contactType == HYPOTHETICAL_CONTACT) {
                        // create an AOU with the same depth of the bathymetry
                        contact = new IvHypotheticalView(&trackDirectory);
		     } else if(contactEvent.contactType == CONTACT_IS_WEAPON) {
                        contact = new IvWPNView();
		     } else {
                        contact = new IvPreciseContactView;
                     }                     
                  
                     contact->setModel(track,OBSERVER_IS_CONTACTVIEW);
                     contact->scaleBy(mapScaling->scaleFactor.getValue()[0],mapScaling->scaleFactor.getValue()[1],mapScaling->scaleFactor.getValue()[2]);
                     iconRoot->addChild(contact);
                  
                  // add a track trace object, to show where the contact has been in the past
                     trackTrace = new TrackTraceDecorator(track);
                  // create the track view, that shows the track in the scene
                     traceView = new IvTrackView();
                     traceView->setMinMarkerDistanceSeparation(0.05);
                  // the track view depends on the updates from the track trace
                     traceView->setModel(trackTrace);
                  // tell the track directory to talk to the the track through the track trace
                     trackDirectory.replaceExistingTrackForContact(contactEvent.name,contactEvent.solution,trackTrace);
                     traceView->scaleBy(mapScaling->scaleFactor.getValue()[0],mapScaling->scaleFactor.getValue()[1],mapScaling->scaleFactor.getValue()[2]);
                     if(contactEvent.contactType != HYPOTHETICAL_CONTACT)
		       iconRoot->addChild(traceView);
		     else
		       traceView->ref();
	             contact->show();
  	             traceView->show();
                     /* put trace track (with contact track inside) into timeline -
                     track is added to timeline before adding a new step, so the timeline can "see" the event */
                     timeline.addTrack(trackTrace);
                     /* Here we decide which type of icon to use for the contact,
                     add the new step to the corresponding track,
                     and we add an UI view to the track. */
                     switch(contactEvent.contactType) {
                        case CONTACT_IS_OWNSHIP:
                           ownshipIconView = (IvPreciseContactView*)contact;
                           ownshipTrack = track;
			   track->setThreatClass(FRIENDLY);
                           ownshipIconView->setMarker(ownshipIcon);
                           traceView->setTraceColor(Iv3dIconColorizer::OWNSHIP_RGBCOLOR[0],Iv3dIconColorizer::OWNSHIP_RGBCOLOR[1],Iv3dIconColorizer::OWNSHIP_RGBCOLOR[2]);
                           traceView->setMarker(ownshipTrackIcon);
                           trackUiView = new OwnshipTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,OWNSHIP_LABEL_COLOR[0],OWNSHIP_LABEL_COLOR[1],OWNSHIP_LABEL_COLOR[2]);
			   contactEvent.coord.precise.xyz[2]=-contactEvent.coord.precise.xyz[2];
                           timeline.appendStepToTrack(trackTrace,new PreciseContact(contactEvent.coord.precise));
                           // since this is the first time we see our sub, we fill the bathymetry with our sub in the center
                           mapBBox=fillMapCenteredOn(bathymetry,&mainWindowRenderingData.viewport,contactEvent.coord.precise.xyz[0],contactEvent.coord.precise.xyz[1]);
                           rootMapBox=createBoundingBoxAround(mapRoot,bathymetry,mapBBox,&mainWindowRenderingData.viewport);
                           seaSurface=new IvSeaSurface(0.0f,0.23f,0.8f);
                           mapRoot->addChild(seaSurface);
                           seaSurface->updateSurfaceBox(mapBBox);
                           seaSurface->hide();
			   seaSurfaceIsVisible=false;
                           setTopDownPerspectiveOverview(mapRoot,mapTiltAngle,mainWindowRenderingData.viewport,mainWindowRenderingData.cameraAspectRatio,centerSceneTranslation,mapScaling,sceneRotation,sceneOffset);
                           mapLocations=createLocationMarkers(mapBBox);
/* --> PROBLEM HERE          mapRoot->addChild(mapLocations);   
                           updateLocationMarkers(mapLocations,mapBBox); */
                           recenterMiniMap=true;
                           updateMiniMap=true;
                           break;
                        case CONTACT_IS_VEHICLE:
  			   track->setThreatClass(contactEvent.threat);
			   // assign a color to contact based on threat level. It will be updated automatically
                           contact->setMarkerColorUsing(&iconColorizer);
                           // assign an icon to the contact, based on contact classification
                           assert(iconCollection.getDefaultIcon() != NULL);
                           contactIcon = get3dIconFor(iconCollection,contactEvent.category);
                           ((IvPreciseContactView*)contact)->setMarker(contactIcon);
                           traceView->setTraceColorUsing(&iconColorizer);
                           traceView->setMarker(contactTrackIcon);
                           trackUiView = new VehicleTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,CONTACT_LABEL_COLOR[0],CONTACT_LABEL_COLOR[1],CONTACT_LABEL_COLOR[2]);
			    contactEvent.coord.precise.xyz[2]=-contactEvent.coord.precise.xyz[2];
                           timeline.appendStepToTrack(trackTrace,new PreciseContact(contactEvent.coord.precise));
                           break;
                        case CONTACT_IS_WEAPON:
                          contactIcon = get3dIconFor(iconCollection,WEAPON);
                           contact->setMarkerColorUsing(&iconColorizer);
                           ((IvWPNView*)contact)->setMarker(contactIcon);
                           traceView->setMarker(weaponTrackIcon);
                           traceView->setTraceColor(Iv3dIconColorizer::WEAPON_RGBCOLOR[0],Iv3dIconColorizer::WEAPON_RGBCOLOR[1],Iv3dIconColorizer::WEAPON_RGBCOLOR[2]);
                           trackUiView = new WeaponTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,WEAPON_LABEL_COLOR[0],WEAPON_LABEL_COLOR[1],WEAPON_LABEL_COLOR[2]);
                           contactEvent.coord.weapon.xyz[2]=-contactEvent.coord.weapon.xyz[2];
                           timeline.appendStepToTrack(trackTrace,new WeaponContact(contactEvent.coord.weapon));
                           break;
                        case CONTACT_FROM_SENSOR:
			  track->setThreatClass(contactEvent.threat);
			   // assign a color to contact based on threat level. It will be updated automatically
                           contact->setMarkerColorUsing(&iconColorizer);
                           traceView->setTraceColorUsing(&iconColorizer);
                           traceView->setMarker(contactTrackIcon->copy());
                           trackUiView = new SensorTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,CONTACT_LABEL_COLOR[0],CONTACT_LABEL_COLOR[1],CONTACT_LABEL_COLOR[2]);
                           // invert Z, depth is sent as a positive value
                           contactEvent.coord.fromSensor.xyz[2]=-contactEvent.coord.fromSensor.xyz[2];
                           timeline.appendStepToTrack(trackTrace,new ContactFromSensor(contactEvent.coord.fromSensor));
                           break;
                        case HYPOTHETICAL_CONTACT:
			   track->setThreatClass(contactEvent.threat);
	                   // assign a color to contact based on threat level. It will be updated automatically
                           contact->setMarkerColorUsing(&iconColorizer);
                           traceView->setTraceColorUsing(&iconColorizer);
                           traceView->setMarker(contactTrackIcon->copy());
                           trackUiView = new SensorTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,CONTACT_LABEL_COLOR[0],CONTACT_LABEL_COLOR[1],CONTACT_LABEL_COLOR[2]);
                           timeline.appendStepToTrack(trackTrace,new HypotheticalContact(contactEvent.coord.hypothetical));
                           break;
                     }
                     trackUiView->setModel(track);                 
                     track = trackTrace;
                  }
                  else { // update an already existing track  
                     switch(contactEvent.contactType) {
                        case CONTACT_IS_OWNSHIP:
                        // now update ownship's position
                           contactEvent.coord.precise.xyz[2]=-contactEvent.coord.precise.xyz[2];
                           timeline.appendStepToTrack(track, new PreciseContact(contactEvent.coord.precise));
                           break;
                     
                        case CONTACT_FROM_SENSOR:
                           contactEvent.coord.fromSensor.xyz[2]=-contactEvent.coord.fromSensor.xyz[2];
                           timeline.appendStepToTrack(track, new ContactFromSensor(contactEvent.coord.fromSensor));
                           if(contactEvent.threat != track->getThreatClass()) {
                              track->setThreatClass(contactEvent.threat);
                           }
                           break;
                        case HYPOTHETICAL_CONTACT:
                           timeline.appendStepToTrack(track, new HypotheticalContact(contactEvent.coord.hypothetical));
                           if(contactEvent.threat != track->getThreatClass()) {
                              track->setThreatClass(contactEvent.threat);
                           }
                        case CONTACT_IS_WEAPON:
                            contactEvent.coord.weapon.xyz[2]=-contactEvent.coord.weapon.xyz[2];
                           timeline.appendStepToTrack(track, new WeaponContact(contactEvent.coord.weapon));
			   break;
                        default:
                           contactEvent.coord.precise.xyz[2]=-contactEvent.coord.precise.xyz[2];
                           timeline.appendStepToTrack(track, new PreciseContact(contactEvent.coord.precise));
                           if(contactEvent.threat != track->getThreatClass()) {
                              track->setThreatClass(contactEvent.threat);
                           }
                     }
                  }
               
                  // if timeline is stopped, don't try to update the minimap
                  if(!timeline.isStopped()) {
                     updateMiniMap = true;
                     if(contactEvent.contactType == CONTACT_IS_OWNSHIP)
                        recenterMiniMap = true;
                  }
              }    

         }
         /* process UI pending command -- these commands don't come from the 3D view, but from the 2D user interface */
         if(talkWithBezel->isCommandAvailable()) {
            switch(talkWithBezel->command) {
               case MainAppCommand::STEP_FORWARD_IN_TIME:
                  timeline.setTimeDirection(FORWARD_IN_TIME);
                  timeline.playNextEvent();
                  updateMiniMap = true;
                  recenterMiniMap = true;
                  break;
               case MainAppCommand::STEP_BACK_IN_TIME:
                  timeline.setTimeDirection(BACKWARD_IN_TIME);
                  timeline.playNextEvent();
                  updateMiniMap = true;
                  recenterMiniMap = true;
                  break;
               case MainAppCommand::STOP_MOVEMENT:
                  timeline.stop();
                  break;
               case MainAppCommand::RESUME_MOVEMENT:
                  timeline.resume();
                  break;
               case MainAppCommand::SET_COLOR_MAP:
                  colormapSwitch->whichChild.setValue(0);
                  colorscale->setColormap(((SoBaseColor*)colormapSwitch->getChild(0))->rgb);
                  break;
               case MainAppCommand::SET_SAND_MAP:
                  colormapSwitch->whichChild.setValue(1);
                  colorscale->setColormap(((SoBaseColor*)colormapSwitch->getChild(1))->rgb);
                  break;
               case MainAppCommand::SET_BLUE_MAP:
                  colormapSwitch->whichChild.setValue(2);
                  colorscale->setColormap(((SoBaseColor*)colormapSwitch->getChild(2))->rgb);
                  break;
               case MainAppCommand::SET_CAMERA:
                  switch(talkWithBezel->commandArg.whereToPositionCamera) {
                     case MainAppCommand::TOP_DOWN_CAMERA:
                        setTopDownOverview(mapRoot,mainWindowRenderingData.viewport,mainWindowRenderingData.cameraAspectRatio,centerSceneTranslation,mapScaling,sceneRotation,sceneOffset);
                        break;
                     case MainAppCommand::TOP_DOWN_PERSPECTIVE_CAMERA:
                        setTopDownPerspectiveOverview(mapRoot,mapTiltAngle,mainWindowRenderingData.viewport,mainWindowRenderingData.cameraAspectRatio,centerSceneTranslation,mapScaling,sceneRotation,sceneOffset);
                        break;
                     case MainAppCommand::CAMERA_CENTERED_ON:
                        track = (ObservableTrack*)OID::toObject(talkWithBezel->commandArg.contactID);
                        if(track) {
                           float *pos = track->getCurrentCoordAndTime()->xyz;
                           setViewCenteredOn(SbVec3f(pos[0],pos[1],pos[2]),mainWindowRenderingData.viewport,mainWindowRenderingData.cameraAspectRatio,centerSceneTranslation,mapTiltAngle,sceneRotation,sceneOffset);
                        }
                        break;
                     case MainAppCommand::TOP_DOWN_CAMERA_CENTERED_ON:
                        track = (ObservableTrack*)OID::toObject(talkWithBezel->commandArg.contactID);
                        if(track) {
                           float *pos = track->getCurrentCoordAndTime()->xyz;
                           setViewCenteredOn(SbVec3f(pos[0],pos[1],pos[2]),mainWindowRenderingData.viewport,mainWindowRenderingData.cameraAspectRatio,centerSceneTranslation,0,sceneRotation,sceneOffset);
                        }
                        break;
                  }
                  break;

	        case MainAppCommand::SURFACE_ON:
		  seaSurface->updateSurfaceBox(mapBBox,&mainWindowRenderingData.viewport,bathymetry);
                  seaSurface->show();
                  seaSurfaceIsVisible=true;
		  break;

		case MainAppCommand::SURFACE_OFF:
                  seaSurface->hide();
                  seaSurfaceIsVisible=false;
		  break;

               case MainAppCommand::SCROLL_VAL:
                  updateCamera = true;
                  updateMiniMap = true;
                  recenterMiniMap = true;
                  break;

               case MainAppCommand::SELECT_TARGET:
                  selectionBoxes.targetSelection->deselect();
                  // the UI have stored the object id of the target to select in talkWithBezel->selectedWeapon
                  if(talkWithBezel->selectedTarget != BAD_OID) {
                     track = (ObservableTrack*)OID::toObject(talkWithBezel->selectedTarget);
                  // resend the track's current position to the views, so the UI gets the current position of the new selection
                     track->changed(TrackEvent::RESENDING_TRACK_POS);
                  // create a path from the scene graph root to the Inventor node used to show the target position. To do this:
                  // step 1) clone the path up to and before the Inventor node that represents the track;
                     SoPath *pathToView = pathToIconRoot->copy();
                  // step 2) Append the the Inventor node that represents the track to the path
                     pathToView->append((IvContactView*)track->findObserverWithMask(OBSERVER_IS_CONTACTVIEW,NULL));
                  // step 3) select the object by using the path
                     selectionBoxes.targetSelection->setSelection(pathToView);
                  }
                  break;
               case MainAppCommand::SELECT_WEAPON:
                  selectionBoxes.weaponSelection->deselect();
                  // the UI have stored the object id of the weapon to select in talkWithBezel->selectedWeapon
                  if(talkWithBezel->selectedWeapon != BAD_OID) {
                     track = (ObservableTrack*)OID::toObject(talkWithBezel->selectedWeapon);
                  // resend the track's current position to the views, so the UI gets the current position of the new selection
                     track->changed(TrackEvent::RESENDING_TRACK_POS);
                  // create a path from the scene graph root to the Inventor node used to show the weapon position. To do this:
                  // step 1) clone the path up to and before the Inventor node that represents the track;
                     SoPath *pathToView = pathToIconRoot->copy();
                  // step 2) Append the the Inventor node that represents the track to the path
                     pathToView->append((IvContactView*)track->findObserverWithMask(OBSERVER_IS_CONTACTVIEW,NULL));
                  // step 3) select the object by using the path
                     selectionBoxes.weaponSelection->setSelection(pathToView);
                  }
                  break;
               case MainAppCommand::STOP_FEEDER:
                  stopFeeder();
                  break;
               case MainAppCommand::RESUME_FEEDER:
                  resumeFeeder();
                  break;
               case MainAppCommand::SHOW_INTERSECTING:
                  IvContactView::setShowIntersecting(true);
                  break;
               case MainAppCommand::HIDE_INTERSECTING:
                  IvContactView::setShowIntersecting(false);
                  break;
               case MainAppCommand::NAV_GRID:
                  textureSwitch->whichChild.setValue(0);
                  showRings=false;
                  break;
	       case MainAppCommand::GROWREGION:
		  trackDirectory.makeTracksGrow(talkWithBezel->growRegion.numSolution,talkWithBezel->growRegion.solutionList);
		  break;
		case MainAppCommand::REDUCEREGION:
		  trackDirectory.makeTracksReduce(talkWithBezel->reduceRegion.numSolution,talkWithBezel->reduceRegion.solutionList);
		  break;
               case MainAppCommand::HYPOTHESIS:
                  break;
               case MainAppCommand::LON_LAT:
                  bathymetry->fillVisibleAreaWithTiles(Rectangle2dFloat(talkWithBezel->lonLat.lon_nw,talkWithBezel->lonLat.lat_se,talkWithBezel->lonLat.lon_se,talkWithBezel->lonLat.lat_nw ));
                  seaSurface->updateSurfaceBox(mapBBox,&mainWindowRenderingData.viewport,bathymetry);
                  updateLocationMarkers(mapLocations,mapBBox);
		  break;
               case MainAppCommand::HYPO_CONTACTS:
                 /* the user requested the creation of a hypothetical contact, so we create a contact event 
                    for it, and we queue the event. Then is the contact event loop's problem to deal with it. */
		 contactEvent.name="HYPOTH";
		 contactEvent.solution=talkWithBezel->hypoContact.regionName;
		 contactEvent.contactType=HYPOTHETICAL_CONTACT;
		 contactEvent.coord.hypothetical.numContacts=talkWithBezel->hypoContact.numContact;
		 contactEvent.coord.hypothetical.time.tv_sec=1;
		 contactEvent.coord.hypothetical.time.tv_usec=0;
                 localEventQueue.addEvent(contactEvent);
		 for (int z=0;z<talkWithBezel->hypoContact.numContact;z++)
  	               strcpy(contactEvent.coord.hypothetical.contactList[z],talkWithBezel->hypoContact.contactList[z]);
                  break;
               case MainAppCommand::SOLUTIONS:
		 trackDirectory.makeTracksVisible(talkWithBezel->solution.numSolution,talkWithBezel->solution.solutionList);
                  break;
               case MainAppCommand::RANGE_RINGS:
                  if (ownshipTrack != NULL) {
                     coordAndTime = ownshipTrack->getCurrentCoordAndTime();
                     rangeRingCenter->translation.setValue(-coordAndTime->xyz[0],-coordAndTime->xyz[1]);
                  }
                  textureSwitch->whichChild.setValue(1);
                  showRings=true;
                  break;
               case MainAppCommand::VERYSHALLOW:
                  changeColorscaleDepth(bathymetry,colorscale,VERYSHALLOW_THRESHOLD);
                  break;
               case MainAppCommand::SHALLOW:
                  changeColorscaleDepth(bathymetry,colorscale,SHALLOW_THRESHOLD);
                  break;
               case MainAppCommand::MIDRANGE:
                  changeColorscaleDepth(bathymetry,colorscale,MIDRANGE_THRESHOLD);
                  break;
               case MainAppCommand::DEEP:
                  changeColorscaleDepth(bathymetry,colorscale,DEEP_THRESHOLD);
                  break;
               case MainAppCommand::SNAPSHOT:
                  SoWriteAction *a=new SoWriteAction;
                  a->getOutput()->openFile("scene.iv");
                  a->getOutput()->setBinary(FALSE);
                  a->apply(sceneGraphRoot);
                  a->getOutput()->closeFile();
                  break;
            }
         }
         talkWithBezel->signalCommandDone();

         /* If the bathymetry must follow the ownship, then check if ownship got too close to the edge.
            If that is the case, then 'slide' the map visible area to show more of the bathymetry. */
         if(ownshipTrack!=NULL && mapUpdatingMode == MAP_FOLLOWS_OWNSHIP) {
             // check if the ownship is too close to the map edge; if so => slide map
            coordAndTime = ownshipTrack->getCurrentCoordAndTime();
            if (coordAndTime->xyz[0]-mapBBox.getMin()[0] < EDGE_THRESHOLD) {
               xtiles+=-1;
            } 
            else if (mapBBox.getMax()[0]-coordAndTime->xyz[0] < EDGE_THRESHOLD) {
               xtiles+=1;
            }
            if (coordAndTime->xyz[1]-mapBBox.getMin()[1] < EDGE_THRESHOLD) {
               ytiles+=-1;
            } 
            else if (mapBBox.getMax()[1]-coordAndTime->xyz[1] < EDGE_THRESHOLD) {
               ytiles+=1;
            }
         }  
      // shift the map to show new tiles, if needed       
         if(xtiles !=0 || ytiles !=0) {
            float sliding[3]; 
          // move map so the area stays in the same position on the screen
            centerSceneTranslation->translation.getValue().getValue(sliding[0],sliding[1],sliding[2]);
            sliding[0]-=TILE_SIZE_X*xtiles;
            sliding[1]-=TILE_SIZE_Y*ytiles;
            centerSceneTranslation->translation.setValue(sliding[0],sliding[1],sliding[2]);
          // shift map to get new tiles
            bathymetry->shiftMapBy(xtiles,ytiles); 
         
         // update the box around the map with the new area covered by the new tiles
            updateBoundingBoxAround(bathymetry, rootMapBox, mapBBox, &mainWindowRenderingData.viewport);
            if(seaSurface->isVisible())
                seaSurface->updateSurfaceBox(mapBBox);
	    updateLocationMarkers(mapLocations,mapBBox);
            updateMiniMap=true;
            recenterMiniMap=true;
            xtiles=ytiles=0;
         }
      
        //try to keep range rings around ownship
         if (ownshipTrack!=NULL && showRings) {
            coordAndTime = ownshipTrack->getCurrentCoordAndTime();
            rangeRingCenter->translation.setValue(-coordAndTime->xyz[0],-coordAndTime->xyz[1]);
         }

         // update compass orientation to match scene and camera rotations
         if(!xwandIsActive) {
             SbRotation compassRotation(NavManager::getMatrix().getQuat().asdVec4()._v);
	     compassRotation.invert();
	     compassRotation*=sceneRotation->rotation.getValue();
             compass->rotation=compassRotation;
         }

         // if xwand is activated, then check wand buttons
         if(useXwand) {
	     switch(fakeMouse.performButtonAction()) {
                 case XW_TRIGGER:  // user pressed the TRIGGER button; start/stop xwand
                     XBell(xwandDisplay,100);
                     if(!xwandIsActive) {                      
                          // xwand on: change to 2D navigation 
                          NavManager::stopNavigation();
		          // center cursor on screen
			  XWarpPointer(xwandDisplay,None,DefaultRootWindow(xwandDisplay),
				xwandInitialPosX,xwandInitialPosY,
                                0,0,xwandInitialPosX,xwandInitialPosY);
                          XDefineCursor(xwandDisplay,xwandWindow,wandCursor);
                          xwandIsActive=true; 
                          break;
                      } else {
                          // disable xwand and re-enable 3D navigation
                          XDefineCursor(xwandDisplay,xwandWindow,invisibleCursor);
                          XWarpPointer(xwandDisplay,None,DefaultRootWindow(xwandDisplay),
				0,0,
                                0,0,0,0);
                          NavManager::startNavigation();
                          xwandIsActive=false;
                      }
		      break;
                 case APP_ACTION_IS_PRINT_CURRENT_POS: 
                      XBell(xwandDisplay,100);
                      printCurrentHeadPosition();
                      break;   
                 case APP_ACTION_IS_QUIT: // Quit rendering loop & bye-bye!
                      XBell(xwandDisplay,100);
                      quitRenderingNow = true;                     
                      break;   
             }
             if(xwandIsActive)         
                fakeMouse.updateScreenPointerWithJoyStick();        
         }
	        
         // render new minimap, and send it to bezel for display
         if(ownshipIconView!=NULL && updateMiniMap) {
             // something has moved, so we re-center top-down map camera
             // on ownship if ownship has moved, and then we render a new minimap
             if(recenterMiniMap) {
                 SbVec3f cameraAim,ownshipLocation = ownshipIconView->getPosition().getValue();
                 // before rendering the minimap, be sure to put the camera perpendicular to the map and facing it
                 sceneAllTransforms->outputMatrix.multVecMatrix(SbVec3f(ownshipLocation[0],ownshipLocation[1],ownshipLocation[2]),ownshipLocation);
                 topDownCamera->position.setValue(ownshipLocation);
                 sceneRotation->rotation.getValue().multVec(SbVec3f(0,0,-1),cameraAim);
                 cameraAim+=ownshipLocation;
                 topDownCamera->pointAt(cameraAim);
               
                 if(updateCamera){
                     // initial height = +/-20K (18200) to +/-2K (1800), accounts for m to kyds conversion
                     topDownCamera->height.setValue(CAMERA_HEIGHT_IN_MINIMAP - talkWithBezel->scroll_val.scroll_val*((0.9*CAMERA_HEIGHT_IN_MINIMAP)));
                     updateCamera = false;
                 }
                 recenterMiniMap = false;
             }

             dglSync(); // we need to be sure that DGL is not rendering while we're here, otherwise the camera change may also affect the main view - This is a problem with Inventor

             // set orthographic top-down camera as the active camera for minimap rendering
             cameraSwitch->whichChild.setValue(TOPDOWN_SCENE_CAMERA_INDEX);
 
             // turn off HUD, so it doesn't show up in  minimap
             hud->hide();
             // turn off sea surface, so it doesn't obscure what's beneath
	     seaSurface->hide();
             imageRenderer->render(sceneSeparator);
             // send minimap image to bezel
             talkWithBezel->updateMap(imageRenderer->getBuffer());
            
             #if DBG_SAVE_MAP_IMAGE_TO_DISK
                  // saves in SGI RGB image format
                  char filename[128];
                  sprintf(filename,"i%g_%g_%g.rgb",contactEvent.coord.xyz[0],contactEvent.coord.xyz[1],contactEvent.coord.xyz[2]);
                  FILE *fp = fopen(filename,"wb");
                  imageRenderer->writeToRGB(fp);
                  fclose(fp);
                  cout << "subve debug: image saved in file '" << filename << "'\n"; cout.flush();
             #endif
            
             // make HUD visible again
             hud->show();
             // make sea surface visible again, it it was visible before
             if(seaSurfaceIsVisible) seaSurface->show();
             // turn off orthographic camera and reset rendering viewport
             cameraSwitch->whichChild.setValue(SO_SWITCH_NONE);

             // clear updating flag
             updateMiniMap = false;
         } else {
              dglSync();
         }

         //  render scene and get input device data
         dglFrame();

      }


      // ...now we can exit.
      XUndefineCursor(xwandDisplay,xwandWindow);
      tellFeederToDie();
      if(quitRenderingNow) { // quitting was initiated by the renderer => tell bezel to die
          talkWithBezel->askBezel(BezelCommand::QUIT);
      }
      pathToIconRoot->unref();
      sceneGraphRoot->unref();
      delete mainWindowRenderingData.renderer;
      cleanUpAndFinish();
   
      return EXIT_SUCCESS;
   }
	
