#include "timeval_ops.h"
#include <math.h>

/* math.h in gcc 2.9.X has an error, and doesn't define trunc(); for all the
   others, it won't hurt. */
extern "C" {
  extern double trunc(double);
}


double timevalToDouble(const timeval& time) {
  return ((double)time.tv_sec) + ((double)time.tv_usec)/1000000.0;
}

timeval doubleToTimeval(double dbltime) {
  timeval time;
  double secs = trunc(dbltime);
  time.tv_sec = (time_t)secs;
  time.tv_usec = (time_t)((dbltime-secs)*1000000);
  return time;
}

void operator-=(timeval& t1, const timeval& t2) {
 t1.tv_usec-=t2.tv_usec;
 if(t1.tv_usec < 0) {
   t1.tv_usec+=1000000;
   t1.tv_sec=t1.tv_sec-1;
 } else  if(t1.tv_usec >= 1000000) {
   t1.tv_usec-=1000000;
   t1.tv_sec=t1.tv_sec+1;
 }
 t1.tv_sec-=t2.tv_sec;
}


void operator+=(timeval& t1, const timeval& t2) {
 t1.tv_usec+=t2.tv_usec;
 if(t1.tv_usec >= 1000000) {
   t1.tv_usec-=1000000;
   t1.tv_sec=t1.tv_sec+1;
 } else if(t1.tv_usec < 0) {
   t1.tv_usec+=1000000;
   t1.tv_sec=t1.tv_sec-1;
 }
 t1.tv_sec+=t2.tv_sec;
}

timeval operator-(const timeval& t1, const timeval& t2) {
 timeval result = t1;
 result-=t2;
 return result;
}

timeval operator+(const timeval& t1, const timeval& t2) {
 timeval result = t1;
 result+=t2;
 return result;
}

void operator*=(timeval& t1, float factor) {
 t1.tv_usec*=factor;
 t1.tv_sec*=factor;
 t1.tv_sec+=t1.tv_usec/1000000;
 t1.tv_usec%=1000000;
}

void operator/=(timeval&t1, float factor) {
  t1.tv_sec/=factor;
  t1.tv_usec/=factor;
}

/*istream& operator>> (istream& is, CoordAndTime& pt) {
  is >> pt.time >> pt.xyz[0] >> pt.xyz[1] >> pt.xyz[2] >> pt.hpr[0] >> pt.hpr[1] >> pt.hpr[2];
  pt.xyz[2]*=-1;
  return is;
}*/

ostream& operator<<(ostream& os,timeval& tm) {
  os << tm.tv_sec << ':' << tm.tv_usec;
  return os;
}
