#include "FakeMouse.h"

FakeMouse::FakeMouse():dpy(NULL),moving_wnd(0),resizing_wnd(0),drawing_line(0) {
}

FakeMouse::FakeMouse(char *display) {
  moving_wnd = 0;
  resizing_wnd = 0;
  drawing_line = 0;
  if ((dpy = XOpenDisplay(display)) == NULL) {
    fprintf(stderr,
	    "XWAND ERROR: XOpenDisplay(\"%s\"): unable to open display: %s\n",
	    display, display);
  }
}


void FakeMouse::useDisplay(const char* newDpy) {
  if(dpy !=NULL)
	XCloseDisplay(dpy);
  if ((dpy = XOpenDisplay(newDpy)) == NULL) {
    fprintf(stderr,
	    "XWAND ERROR: XOpenDisplay(\"%s\"): unable to open display: %s\n",
	    newDpy, newDpy);
  }
  else {
    printf("FakeMouse using Display %s\n",newDpy);
  }
}

void FakeMouse::update(){
  last_x = real_x;
  last_y = real_y;
  XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &current_wnd,
		&real_x, &real_y, &wnd_x, &wnd_y, &mask);

  //if currently moving a window... do so
  if(drawing_line==1){
    fprintf(stderr, "Drawing Line\n");
  }

  //if the mouse is in a window get it's attributes
  if(current_wnd){
    XGetWindowAttributes(dpy, current_wnd, &attrib);
  }

}

void FakeMouse::press(int b){
    XW_send_fake_Xbutton(dpy,b,True);  /* press button */
}

void FakeMouse::release(int b){
    XW_send_fake_Xbutton(dpy,b,False);  /* release button */
}

void FakeMouse::click(int b){
  press(b);
  release(b);
}

void FakeMouse::doubleClick(int b){
  click(b);
  click(b);
}

void FakeMouse::setPosition(int x, int y){
  XWarpPointer(dpy, None, (DefaultRootWindow(dpy)), 0,0,0,0, x, y);
  XFlush(dpy);
}

void FakeMouse::addPosition(int x, int y){
  setPosition(real_x+x, real_y+y);
}

void FakeMouse::raiseWindow(){
  if(current_wnd){
    XRaiseWindow(dpy, current_wnd);
    XGetWindowAttributes(dpy, current_wnd, &attrib);
  }
}

void FakeMouse::lowerWindow(){
  if(current_wnd){
    XLowerWindow(dpy, current_wnd);
    XGetWindowAttributes(dpy, current_wnd, &attrib);
  }
}

void FakeMouse::startResizeWindow(){
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F8),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),0,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F8),0,CurrentTime);
}

void FakeMouse::stopResizeWindow(){
  click(1);
}

void FakeMouse::startMoveWindow(){
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F7),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),0,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F7),0,CurrentTime);
}

void FakeMouse::stopMoveWindow(){
  click(1);
}

void FakeMouse::startDrawLine(){
  drawing_line = 1;
}

void FakeMouse::stopDrawLine(){
  drawing_line = 0;
}

void FakeMouse::destroyWindow(){
  if(current_wnd){
    XDestroyWindow(dpy, current_wnd);
  }
}

void FakeMouse::minimizeWindow(){
    fprintf(stderr, "MinimizeWindow!!!\n");
    if(current_wnd){
        int n = XIconifyWindow(dpy, current_wnd, 0);
        //int n = XWindow(dpy, current_wnd, 0);
        fprintf(stderr, "current window %d\n", n);
        XGetWindowAttributes(dpy, current_wnd, &attrib);
    }
}

void FakeMouse::maximizeWindow(){
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F10),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),0,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F10),0,CurrentTime);
}

void FakeMouse::restoreWindow(){
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F5),1,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_Alt_L),0,CurrentTime);
  XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, XK_F5),0,CurrentTime);
}

void FakeMouse::xtestext1_correct_mouse(Display *dpy){
  Window root, w;
  int rx, ry, wx, wy;
  unsigned int m;
  
  XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &w,
		&rx, &ry, &wx, &wy, &m);
}


void FakeMouse::XW_send_fake_Xbutton(Display *dpy, int button, Bool press)
{
  XTestFakeButtonEvent(dpy,button,press,CurrentTime); /* button press now */
  xtestext1_correct_mouse(dpy);
}

void FakeMouse::get_pointer_position(){
  XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &current_wnd,
		&real_x, &real_y, &wnd_x, &wnd_y, &mask);
}

void FakeMouse::getMousePosition(int& x, int& y) {
  get_pointer_position();
  x = real_x;
  y = real_y;
}

void FakeMouse::doEvent(int event, bool change, char* command){
    int keysym;
    
    switch (event) {
        case FM_BUTTON1:
            if(change){
                press(1);
            }
            else {
                release(1);
            }
            break;
        case FM_BUTTON2:
            if(change){
                press(2);
            }
            else {
                release(2);
            }
            break;
        case FM_BUTTON3:
            if(change){
                press(3);
            }
            else {
                release(3);
            }
            break;
        case FM_RAISE_WINDOW:
            if(change){
                //nothing
            }
            else {
                raiseWindow();
            }
            break;
        case FM_LOWER_WINDOW:
            if(change){
                //nothing
            }
            else {
                lowerWindow();
            }
            break;
        case FM_MOVE_WINDOW:
            if(change){
                startMoveWindow();
            }
            else {
                stopMoveWindow();
            }
            break;
        case FM_RESIZE_WINDOW:
            if(change){
                startResizeWindow();
            }
            else {
                stopResizeWindow();
            }
            break;
        case FM_MAXIMIZE_WINDOW:
            if(change){
                //nothing
            }
            else {
                maximizeWindow();
            }
            break;
        case FM_MINIMIZE_WINDOW:
            if(change){
                //nothing
            }
            else {
                minimizeWindow();
            }
            break;
        case FM_RESTORE_WINDOW:
            if(change){
                //nothing
            }
            else {
                restoreWindow();
            }
            break;
        case FM_DESTROY_WINDOW:
            if(change){
                //nothing
            }
            else {
                destroyWindow();
            }
            break;
        case FM_KEYPRESS:
            //get the keysym
            fprintf(stderr, "doing keypress %s\n", command);
            
            keysym = XStringToKeysym(command);
            if(keysym == NoSymbol){
                fprintf(stderr, "didn't find a symbol :::%s:::\n", command);
                break;
            }
            if(change==1){
                XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, keysym),1,CurrentTime);
            }
            else {
                XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, keysym),0,CurrentTime);
            }
            break;
        case FM_SYSTEM:
            if(change==1){
                //nothing
            }
            else {
                system(command);
            }
            break;
    }
    XFlush(dpy);
}

void FakeMouse::keyPress(int status, int keysym)
{
    XTestFakeKeyEvent(dpy, XKeysymToKeycode(dpy, keysym), status, CurrentTime);
    XFlush(dpy);
}
