#ifndef _FAKE_MOUSE_H_
#define _FAKE_MOUSE_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xlib.h>
#include <X11/extensions/XTest.h>
#include <X11/extensions/XInput.h>
#include <X11/keysym.h>


class FakeMouse {
 public:
  FakeMouse(); // If you use the default constructor, you have to call useDisplay() too
  FakeMouse(char *display);
  void useDisplay(const char *newDpy);

  void doEvent(int event, bool change, char* command);
  void keyPress(int status, int keysym);
  
  void update();
  void press(int b);
  void release(int b);
  void click(int b);
  void doubleClick(int b);

  void raiseWindow();
  void lowerWindow();
  void startMoveWindow();
  void stopMoveWindow();
  void startDrawLine();
  void stopDrawLine();
  void startResizeWindow();
  void stopResizeWindow();
  void destroyWindow();
  void minimizeWindow();
  void maximizeWindow();
  void restoreWindow();

  void setPosition(int x, int y);
  void addPosition(int x, int y);

  void getMousePosition(int& x, int& y);
  Display* getDisplay(){return dpy;};
  void get_pointer_position();

  private:
    enum {
     FM_BUTTON1,
     FM_BUTTON2,
     FM_BUTTON3,
     FM_RAISE_WINDOW,
     FM_LOWER_WINDOW,
     FM_MOVE_WINDOW,
     FM_RESIZE_WINDOW,
     FM_MAXIMIZE_WINDOW,
     FM_MINIMIZE_WINDOW,
     FM_RESTORE_WINDOW,
     FM_DESTROY_WINDOW,
     FM_KEYPRESS,
     FM_SYSTEM
  };

  static void xtestext1_correct_mouse(Display *dpy);
  void XW_send_fake_Xbutton(Display *dpy, int button, Bool press);


  Display *dpy;
  Window root;
  Window current_wnd;
  XWindowAttributes attrib;
  int real_x, real_y, last_x, last_y, wnd_x, wnd_y;
  unsigned int mask;

  int moving_wnd, resizing_wnd, drawing_line;
};

#endif

