#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include "xwandparser.h"

//This file holds functions that parse the config file
//Basically a switch on the first char of the file and hten
//fill a struct up with data and send it back


//Wrapping function for parsing the xwand
//This basically allows us to print out debug information if
//we need to
bool XWandData::readXWandConfigFile(const char* filename, const bool verbose)
{         
        bool fileFound =xWandParser(filename);
        if (fileFound) {
            if (verbose) {
		printXWandDataTo(stdout);
	    }
        }
	//Return the data the program gathered
	return fileFound;
}

void XWandData::printXWandDataTo(FILE *output) {
     register int i;
     fprintf(output,"XWand Settings:\n");
     fprintf(output,"===============\n");
     fprintf(output,"X Display where pointer will be controlled by xwand = %s\n",display);
     fprintf(output,"Number of Buttons: %d\n", numButtons);
     fprintf(output,"Button Actions:\n");
     for (i=0; i < numButtons; i++) {
          fprintf(output,"%i : ",i);
          switch(buttons[i]) {
              case XW_NONE:
			fprintf(output,"NONE (no action)\n");
			break;
              case XW_TRIGGER:
			fprintf(output,"TRIGGER (starts/stops xwand)\n");
			break;
              case XW_LEFTBUTTON:
			fprintf(output,"LEFTBUTTON (left button click)\n");
			break;
              case XW_RIGHTBUTTON:
			fprintf(output,"RIGHTBUTTON (right button click)\n");
			break;
              case XW_RUN_APP:
			fprintf(output,"RUN_APP (starts a program)\n");
			break;
              case XW_APP_EVENT_0:
			fprintf(output,"APP_EVENT_0 (program-refined action)\n");
                        break;
              case XW_APP_EVENT_1:
			fprintf(output,"APP_EVENT_1 (program-refined action)\n");
                        break;
              case XW_APP_EVENT_2:
			fprintf(output,"APP_EVENT_2 (program-refined action)\n");
                        break;
              case XW_APP_EVENT_3:
			fprintf(output,"APP_EVENT_3 (program-refined action)\n");
                        break;
              case XW_APP_EVENT_4:
			fprintf(output,"APP_EVENT_4 (program-refined action)\n");
                        break;
              case XW_APP_EVENT_5:
			fprintf(output,"APP_EVENT_5 (program-refined action)\n");
                        break;
              case XW_APP_EVENT_6:
			fprintf(output,"APP_EVENT_6 (program-refined action)\n");
                        break;
              case XW_APP_EVENT_7:
			fprintf(output,"APP_EVENT_7 (program-refined action)\n");
                        break;
              case XW_APP_EVENT_8:
			fprintf(output,"APP_EVENT_8 (program-refined action)\n");
                        break;
              case XW_APP_EVENT_9:
			fprintf(output,"APP_EVENT_9 (program-refined action)\n");
                        break;
              default:
			fprintf(output,"action #%i - this shouldn't have happened!\n",buttons[i]);
			break;
         }
     }
     if(app!=NULL) {
         if(numArgs==1)
           fprintf(output,"Program to start: %s, with no arguments\n", app[0]);
         else {
           fprintf(output,"Program to start: %s  with %i arguments: ", app[0], numArgs-1);
           for(i=1; i<numArgs; i++)
                fprintf(output,"%s ",app[i]);
           fprintf(output,"\n");
         }
     }
     fprintf(output,"-----------------------------------------------------------\n");
}


void XWandData::allocButtons(int maxNumButtons) {
	if (buttons != NULL) {
		delete[] buttons;
        }
	buttons = new XWandButtonActions[maxNumButtons];
        numButtons = maxNumButtons;
	for(unsigned int i=0;i<numButtons;i++) buttons[i]=XW_NONE;
}

char* XWandData::nextToken(char *pos) {
  while(*pos && !isspace(*pos)) pos++;
  while(*pos && isspace(*pos)) pos++;
  return pos;
}

void XWandData::assignButtons(char *textline) {
        if (numButtons == 0)
                allocButtons(10);
        else
		allocButtons(numButtons);
	char* action = strtok(textline, " ");
	numActions = 0;
        while(action && numActions < numButtons) {
          // cheap way to check string: compare 2nd letter
          switch(toupper(action[1])) {
             case 'R': // TRIGGER
	   	buttons[numActions]=XW_TRIGGER;
		break;
             case 'E': // LEFTBUTTON
		buttons[numActions]=XW_LEFTBUTTON;
		break;
             case 'I': // RIGHTBUTTON
		buttons[numActions]=XW_RIGHTBUTTON;
		break;
             case 'U': // RUN_APP
		buttons[numActions]=XW_RUN_APP;
		break;
             case 'O': // NONE
		buttons[numActions]=XW_NONE;
		break;
             case 'P':  // APP_EVENT_0, APP_EVENT_1... APP_EVENT_9
                if(strlen(action) > 10) {
                   if(action[10] >= '0' && action[10] <= '9') {
                       buttons[numActions]=XWandButtonActions(int(XW_APP_EVENT_0)+action[10]-'0');
                       break;
                   }
                }
	     default:
		fprintf(stderr,"ERROR in xwand.config: Action %s for button %i is not valid.\n\tValid Actions are: {trigger, leftbutton,rightbutton, run_app, none}\n.",action,numActions+1);
           }
 	   action = strtok(NULL, " ");
           numActions++; // if action is not valid, still go to next button assignment
        }
}

void XWandData::getAppArgs(char* textline) {
   typedef char* TCharPtr;
   // we handle up to 9 arguments to the app
   app = new TCharPtr[10];
   char *arg = strtok(textline," ");
   numArgs = 0;
   while(arg && numArgs < 10) {
     app[numArgs] = strdup(arg);
     numArgs++;
     arg = strtok(NULL," ");
   }
} 

//XWand parsing function, handles getting the data out of the xwand configuration file
bool XWandData::xWandParser(const char* filename) {
        const int MAX_LEN_LINE =  256;
	FILE* inputFile;
	if (filename == NULL)
		inputFile = fopen("xwand.config", "r");
	else
		inputFile = fopen(filename, "r");
	if (inputFile!=NULL) {
          unsigned int linecounter=1;
	  char line[MAX_LEN_LINE], *startline, *endline;
          fgets(line, MAX_LEN_LINE, inputFile);
          while (!feof(inputFile)) {
                startline=line;
		while(*startline && isspace(*startline)) startline++;
                if(*startline=='\0')
		        continue;
		// remove trailing newline, if present
                endline=startline+1;
                while(*endline) ++endline;
                --endline;
                if(*endline=='\n') *endline='\0';
                // now parse commands
		switch(toupper(*startline)) {
			case '#': // comment line, ignore it
			{
				break;
			}
			case 'A': // ACTIONS
			{
				assignButtons(nextToken(startline));
				break;
			}
			case 'D': // DISPLAY
			{
				display = strdup(nextToken(startline));
				break;
			}
                        case 'N': // NUMBUTTONS
                        {
                                allocButtons(atoi(nextToken(startline)));
				break;
                        }
                        case 'R': // RUN_APP
				getAppArgs(nextToken(startline));
			all:
			{
				printf("Unrecognized command in %s, line number %i.\n\tIgnoring line and continuing...\n",filename,linecounter);
				break;
			}
		}
		//Read in the next line
		fgets(line, MAX_LEN_LINE, inputFile);
                linecounter++;
	    }
            fclose(inputFile);
            return true;
       } else  {
         return false;
       }   
}

XWandData::~XWandData() {
  if(app) {
     for(int i=0;i<numArgs;i++)
	free(app[i]);
     delete[] app;
  }
  delete[] buttons;
  free(display);
}
