#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <fstream>
#include <stdlib.h>
#include <ctype.h>
#include "ContactIconCollection3D.h"

   ContactIconCollection3D::ContactIconCollection3D(unsigned int numberOfIcons):size(numberOfIcons) {
      icons = (SoNode**)malloc(numberOfIcons*sizeof(SoNode*));
      size = numberOfIcons;
   }

   void ContactIconCollection3D::setSize(unsigned int numberOfIcons) {
      icons = (SoNode**)realloc(icons,numberOfIcons*sizeof(SoNode*));
      for(unsigned int i = size; i<numberOfIcons; i++) {
         icons[i]=NULL;
      }
      size = numberOfIcons;
   }


   SoNode* ContactIconCollection3D::operator[](unsigned int iconNo) const {
      if (iconNo>=size || icons[iconNo]==NULL)
         if (index_default_icon<size)
            return icons[index_default_icon];
         else
            return NULL;
      else
         return icons[iconNo];
   }

   ContactIconCollection3D::FILE_RESULT ContactIconCollection3D::load3dIconsFrom(const char* const filename, bool ownership) {
      unsigned int i;
      IconList iconList;
      SoNode* icon;
      char line[256], *beginName, *endName;
      SoInput loader;
      std::ifstream fin(filename);
      if (fin.fail()) {
         return CANT_FIND_FILE;
      }
      IamTheOwner=ownership;
      if (icons) {
         for(i=0;i<size;i++)
            if(icons[i]!=NULL) icons[i]->unref();
         free(icons);
         size = 0;
         index_default_icon = 1;
      }
      while(fin) {
         fin.getline(line,256,'\n');
         beginName = line;
         while(*beginName && isspace(*beginName)) ++beginName;
         endName=beginName;
         while(*endName) ++endName;
         while(endName != beginName && isspace(*endName)) --endName;
         if(*endName) ++endName;
         *endName = '\0';
         if(*beginName != '\0' && *beginName != '#') {
            if(*beginName != '?' &&  strcasecmp(beginName,"null")) {
               loader.openFile(beginName);
               icon = SoDB::readAll(&loader);
               if(icon != NULL) {
                  icon->ref();
               }
               iconList.push_back(icon);
               loader.closeFile();
            }
            else {
               iconList.push_back(NULL);
            }
         }
      }
      size = iconList.size();
      icons = (SoNode**)malloc(size*sizeof(SoNode*));
      i=0;
      for (IconList::iterator it = iconList.begin(); it != iconList.end(); ++it) {
         icons[i]=(SoNode*)*it;
         ++i;
      }
      dontKeepADefaultIcon();
      return NO_ERROR;
   }

   ContactIconCollection3D::~ContactIconCollection3D() {
      if(IamTheOwner) {
         for(unsigned int i=0;i<size;i++)
            if(icons[i]!=NULL) icons[i]->unref();
      }
      if(size!=0) free(icons);
   }
   void ContactIconCollection3D::setDefaultIconTo(unsigned int index) {
      if (index >= size) {
        /* We need to show something up when we're asked to show a 3D icon,
           so here we make one up right here */
         SoSeparator *defaultIcon = new SoSeparator;
         SoBaseColor *color = new SoBaseColor;
         defaultIcon->addChild(color);
         color->rgb.setValue(0.5,0.5,0.5);
         defaultIcon->addChild(new SoCube);
         if(size != 0) {
            icons = (SoNode**)realloc(icons,(size+1)*sizeof(SoNode*));
         } 
         else {
            icons = (SoNode**)malloc(sizeof(SoNode*));
         }
         icons[size]=defaultIcon;
         index_default_icon = size;
         ++size;
      } 
      else {
         index_default_icon = index;
      }
   }

   void ContactIconCollection3D::dontKeepADefaultIcon() {
   	/* reset index of default icon to be the size of the collection, 
   	   this means for us there is no default icon, since the valid indices go from 0 to size-1 */
      index_default_icon = size; 
   }
