#include "DiverseHUD.h"


SO_NODE_SOURCE(DiverseHUD);


void DiverseHUD::initClass() {
      IvHUD::initClass();
      SO_NODE_INIT_CLASS(DiverseHUD,IvHUD,"IvHUD");
}


DiverseHUD::DiverseHUD(SbViewportRegion* vp):IvHUD(vp),headDataIsOld(true) { 
      SO_NODE_CONSTRUCTOR(DiverseHUD);
}


// Constructor that takes approximate number of children as argument.
DiverseHUD::DiverseHUD(int numChildren, SbViewportRegion* vp):IvHUD(numChildren,vp),headDataIsOld(true) {
      SO_NODE_CONSTRUCTOR(DiverseHUD);
}

bool DiverseHUD::isWorldCoordCulled(float worldPointXcoord, float worldPointYcoord, float worldPointZcoord) const {
    if(headDataIsOld) {
      // get user head's transformation matrix 
      dMatrix navMatrix = NavManager::getMatrix();
      headPos=navMatrix.getTrans();
      navMatrix.setTrans(0,0,0);
      headOrientationVector=navMatrix*dVec3(0,0,-1); /* (0,0,-1) = for Inventor = looking forward, parallel to the floor */
      headDataIsOld=false;
    }
    worldPointXcoord-=headPos[0];
    worldPointYcoord-=headPos[1];
    worldPointZcoord-=headPos[2];
    // cull if the angle between the head orientation and the point appears outside of a semicircle (90 degree angle) around the vector aiming ath the direction in which the user is currently looking
    return (worldPointXcoord*headOrientationVector[0]+worldPointYcoord*headOrientationVector[1]+worldPointZcoord*headOrientationVector[2]<0);   
}

bool DiverseHUD::isLocalCoordCulled(float localPointXcoord, float localPointYcoord, float localPointZcoord) const {
    float worldPointXcoord, worldPointYcoord, worldPointZcoord;
    // transform point from local coordinate system to world coordinate system
    worldPointXcoord=localPointXcoord*glModelViewMatrix[0]+localPointYcoord*glModelViewMatrix[4]+localPointZcoord*glModelViewMatrix[8]+glModelViewMatrix[12];
    worldPointYcoord=localPointXcoord*glModelViewMatrix[1]+localPointYcoord*glModelViewMatrix[5]+localPointZcoord*glModelViewMatrix[9]+glModelViewMatrix[13];
    worldPointZcoord=localPointXcoord*glModelViewMatrix[2]+localPointYcoord*glModelViewMatrix[6]+localPointZcoord*glModelViewMatrix[10]+glModelViewMatrix[14];
    return isWorldCoordCulled(worldPointXcoord,worldPointYcoord,worldPointZcoord);
}

void DiverseHUD::GLRender(SoGLRenderAction *action) {
    headDataIsOld=true;
    IvHUD::GLRender(action);
}
