/// 2002, Virginia Tech UVAG Group ///

#include "IvCompass.h"
#include <GL/gl.h>
#include <GL/glut.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include "glRenderText.h"
#include <stdlib.h>

/************************** IvCompass class ******************************/

   void convertQuaternionToMatrix(const SbRotation& q, GLfloat *matrix) {
       float w,x,y,z;
       q.getValue(w,x,y,z);
       GLfloat xsqr=2*x*x,ysqr=2*y*y,zsqr=2*z*z;
       GLfloat xy=2*x*y,wz=2*w*z,xz=2*x*z,wy=2*w*y;
       GLfloat yz=2*y*z,wx=2*w*x;
       matrix[0]=1-ysqr-zsqr;
       matrix[1]=xy-wz;
       matrix[2]=xz+wy;
       matrix[4]=xy+wz;
       matrix[5]=1-xsqr-zsqr;
       matrix[6]=yz-wx;
       matrix[8]=xz-wy;
       matrix[9]=yz+wx;
       matrix[10]=1-xsqr-ysqr;
       matrix[3]=matrix[7]=matrix[11]=matrix[12]=matrix[13]=matrix[14]=0.0f;
       matrix[15]=1.0f;	  	
   }

   SO_NODE_SOURCE(IvCompass);

   void IvCompass::initClass() {
      SO_NODE_INIT_CLASS(IvCompass,SoNode,"Node");
   }

	
   void IvCompass::GLRender(SoGLRenderAction*) {
      GLfloat orientationMatrix[16];
      float xpos=this->origin[0]+this->width*0.5;
      float ypos=this->origin[1]+this->height*0.5;
      GLboolean lights;
      lights = glIsEnabled(GL_LIGHTING);
      if(!lights) glEnable(GL_LIGHTING);
      glShadeModel(GL_SMOOTH);
      float proj[16]={2/this->width,0,0,0, 0,2/this->height,0,0, 0,0,(this->width+1)/(this->width+1-1),-(this->width+1)/(this->width+1-1), 0,0,1,0};
      float orth[16]={1,0,0,0, 0,1,0,0, 0,0,0,0, 0,0,0,1};
      glPushMatrix();
      glTranslatef(xpos,ypos,0);
      glScalef(8500,8500,0);
      glMultMatrixf(orth);
      glMultMatrixf(proj);
      glTranslatef(0,0,-this->width*0.75-1);
      // rotate the compass accordint to the value in the orientation field
      glRotatef(90,0,1,0);
      convertQuaternionToMatrix(rotation.getValue(),orientationMatrix);
      glMultMatrixf(orientationMatrix);
      glRotatef(-90,0,1,0);
      GLfloat color[] = {1,1,1,1};
      glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,color);
      // draw the ring
      glutSolidTorus(this->height*0.02,this->height*0.3,20,200);
   	//draw the crosshairs
      GLUquadricObj* quad=gluNewQuadric();
      gluQuadricDrawStyle(quad,GLU_FILL);
      glPushMatrix();
      glRotatef(-90,1,0,0);
      gluCylinder(quad,this->height*0.02,this->height*0.02,this->height*0.29,20,1);
      glPopMatrix();
      glPushMatrix();
      glRotatef(90,1,0,0);
      gluCylinder(quad,this->height*0.02,this->height*0.02,this->height*0.45,20,1);
      glPopMatrix();
      glPushMatrix();
      glRotatef(-90,0,1,0);
      gluCylinder(quad,this->height*0.02,this->height*0.02,this->height*0.29,10,1);
      glPopMatrix();
      glPushMatrix();
      glRotatef(90,0,1,0);
      gluCylinder(quad,this->height*0.02,this->height*0.02,this->height*0.29,10,1);
      glPopMatrix();
      gluDeleteQuadric(quad);
   
      // draw the arrow aiming north (a cone, actually)
      glPushMatrix();
      glTranslatef(0,-this->height*0.43,0);
      glRotatef(90,1,0,0);
      glutSolidCone(this->height*0.04,this->height*0.07,10,10);
      glPopMatrix();
      glDisable(GL_LIGHTING);
      float black[]={0,0,0,0};
      glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,black);
      // label north with a red "N"
      glColor3f(1,0,0);
      static const unsigned char *north=(const unsigned char*)"N";
      glRenderTextAt(-3,-this->height*0.5,north,GLUT_BITMAP_TIMES_ROMAN_10);
      glPopMatrix();
      if (lights == GL_TRUE)
         glEnable(GL_LIGHTING);
      else
         glDisable(GL_LIGHTING);
   }

   IvCompass::IvCompass() {
      SO_NODE_CONSTRUCTOR(IvCompass);
      SO_NODE_ADD_FIELD(rotation,(1,0,0,0));
      init();
  }

   void IvCompass::init() {
      origin[0] = origin[1] = 0.0f;
   }

   void IvCompass::setSize(float newWidth, float newHeight) {
      width=newWidth; height=newHeight;
   }

   IvCompass::~IvCompass() {
   
   }
