#include "NameToTrackMapping.h"
#include "trackview.h"

//this routine goes through the tracks, finds their IvContactView and IvTrackView and applies a new scale to them
   void NameToTrackMapping::applyScaleToTracks(float x, float y, float z) {
      MapType::const_iterator item = dict.begin();
      while (item!=dict.end()) {
         ObservableTrack *cur=item->second;
         if (cur!=NULL) {
         //is this a Track Trace decorator?
            TrackTraceDecorator *d=static_cast<TrackTraceDecorator *>(cur);
            IvTrackView *t;
            IvContactView *c;
            if (d!=NULL) {
            // get the IvTrackView object that observes this track
               cur=d->getTrack();
               t=static_cast<IvTrackView *>(d->findObserverWithMask(OBSERVER_IS_TRACKVIEW, NULL));
               if (t!=NULL) {
                  t->scaleBy(x,y,z);
               }
            }
         //get the IvContactView that observes this track
            c=static_cast<IvContactView*>(cur->findObserverWithMask(OBSERVER_IS_CONTACTVIEW, NULL));
            if (c!=NULL) {
               c->scaleBy(x,y,z);
            }
         }
         item++;
      }
   }


   ObservableTrack *NameToTrackMapping::findTrackForContact(const string& trackName, const string & solutionName) const {
      return findTrackForContact(trackName+solutionName);
   }

   ObservableTrack *NameToTrackMapping::findTrackForContact(const string& fullname) const {
      MapType::const_iterator item = dict.find(fullname);
      if(item == dict.end())
         return NULL;
      return item->second;
   }

   ObservableTrack *NameToTrackMapping::addTrackForContact(const string& trackName, const string & solutionName ,const ContactType contactType) {
      string fullname=trackName+solutionName;
      DiscreteTrack *newTrack = new DiscreteTrack;
      dict[fullname] = newTrack;
      newTrack->setName(trackName);
       newTrack->setSolution(solutionName);
     newTrack->setContactType(contactType);
      return newTrack;
   }

   void NameToTrackMapping::removeTrackForContact(const string& trackName, const string & solutionName) {
      string fullname=trackName+solutionName;
      dict.erase(fullname);
   }

   void NameToTrackMapping::replaceExistingTrackForContact(const string& trackName, const string & solutionName, ObservableTrack* anotherTrack) {
      string fullname=trackName+solutionName;
   #ifndef NDEBUG
      assert(dict.find(fullname) != dict.end());
   #endif
      dict[fullname]=anotherTrack;
   }

   void NameToTrackMapping::makeTracksVisible(int num, char solutions[][64]) {
      bool vis=false;
      int i=0;
      MapType::const_iterator item = dict.begin();
      while (item!=dict.end()) {
         ObservableTrack *cur=item->second;
	 const char *namec=item->first.c_str();
	 vis=false;
	 for (int i=0;i<num;i++) {
	   if (!strcmp(namec,solutions[i])) {
	     vis=true;
	   }
	 }
	 if (!strcmp(cur->getSolution().c_str(),"")) {
	   vis=true;
	 }
         if (cur!=NULL) {
         //is this a Track Trace decorator?
            TrackTraceDecorator *d=static_cast<TrackTraceDecorator *>(cur);
            IvTrackView *t;
            IvContactView *c;
            if (d!=NULL) {
            // get the IvTrackView object that observes this track
               cur=d->getTrack();
               t=static_cast<IvTrackView *>(d->findObserverWithMask(OBSERVER_IS_TRACKVIEW, NULL));
               if (t!=NULL) {
		 if (vis) {
		   t->show();
		 }
		 else {
		   t->hide();
		 }
               }
            }
         //get the IvContactView that observes this track
            c=static_cast<IvContactView*>(cur->findObserverWithMask(OBSERVER_IS_CONTACTVIEW, NULL));
            if (c!=NULL) {
		 if (vis) {
		   c->show();
		 }
		 else {
		   c->hide();
		 }
            }
         }
         item++;
      }
   }

   void NameToTrackMapping::makeTracksGrow(int num, char solutions[][64]) {
      bool vis=false;
      int i=0;
      MapType::const_iterator item = dict.begin();
      while (item!=dict.end()) {
         ObservableTrack *cur=item->second;
	 const char *namec=item->first.c_str();
	 vis=false;
	 for (int i=0;i<num;i++) {
	   if (!strcmp(namec,solutions[i])) {
	     vis=true;
	   }
	 }
         if (cur!=NULL) {
	   TrackTraceDecorator *d=static_cast<TrackTraceDecorator *>(cur);
	   IvContactView *c;
	   if (d!=NULL) {
	     // get the IvTrackView object that observes this track
	     cur=d->getTrack();
	   }
	   //get the IvContactView that observes this track
	   c=static_cast<IvContactView*>(cur->findObserverWithMask(OBSERVER_IS_CONTACTVIEW, NULL));
	   if (c!=NULL) {
	     c->setGrow(vis);
	   }
         }
         item++;
      }
   }

   void NameToTrackMapping::makeTracksReduce(int num, char solutions[][64]) {
      bool vis=false;
      int i=0;
      MapType::const_iterator item = dict.begin();
      while (item!=dict.end()) {
         ObservableTrack *cur=item->second;
	 const char *namec=item->first.c_str();
	 vis=false;
	 for (int i=0;i<num;i++) {
	   if (!strcmp(namec,solutions[i])) {
	     vis=true;
	   }
	 }
         if (cur!=NULL) {
	   TrackTraceDecorator *d=static_cast<TrackTraceDecorator *>(cur);
	   IvContactView *c;
	   if (d!=NULL) {
	     // get the IvTrackView object that observes this track
	     cur=d->getTrack();
	   }
	   //get the IvContactView that observes this track
	   c=static_cast<IvContactView*>(cur->findObserverWithMask(OBSERVER_IS_CONTACTVIEW, NULL));
	   if (c!=NULL) {
	     c->setSelfIntersect(vis);
	   }
         }
         item++;
      }
   }
