#include "TrackUiView.h"
#include "ContactEvent.h"


#include <stdio.h>

/************ TrackUiView ************/

   void TrackUiView::copyContactDataToMediator() {
   //We're about to change the data, be sure the UI has read the last thing we sent
      while(!ui.contactDataWasRead());
   // give the track's (our model) address as the contactID
      ui.actionData.contactID = OID::fromObject(track);
   }


   void TrackUiView::setModel(Observable* anObservableTrack) {
      if(track)
         track->removeView(this);
      anObservableTrack->addView(this);
      track = static_cast<ObservableTrack*>(anObservableTrack);
      notifyOfNewContact();
   }

   void TrackUiView::notifyOfNewContact() {
      copyContactDataToMediator();
      // new contacs also give their names when they are "new"
      strncpy(ui.actionData.name,track->getName().c_str(),UiMediator::MAX_CONTACTNAME_LENGTH-1);
      strncpy(ui.actionData.solution,track->getSolution().c_str(),UiMediator::MAX_CONTACTNAME_LENGTH-1);
      ui.signalContactDataAvailable(BezelCommand::NEW_CONTACT);
      newContactNotified =true;
   }

/* please note this method doesn't fill the contact pos, angle etc. 
   It is the subclass' responsability to fill the correct fields in 
   the ui object, depending on the contact data being precise or from a sensor. */
   void TrackUiView::update(void *aTrackEvent) {
      if(((TrackEvent*)aTrackEvent)->eventType & TrackEvent::POS_CHANGED) {
         if(!newContactNotified)
            notifyOfNewContact();
         else {
           copyContactDataToMediator();
	   ui.signalContactDataAvailable(BezelCommand::NEW_CONTACT_POSITION);
         }
      } 
      else if(((TrackEvent*)aTrackEvent)->eventType & TrackEvent::TRACK_ABOUT_TO_DIE) {
         copyContactDataToMediator();
         ui.signalContactDataAvailable(BezelCommand::REMOVE_CONTACT);
      } else if(((TrackEvent*)aTrackEvent)->eventType & TrackEvent::RESENDING_TRACK_POS) {
         copyContactDataToMediator();
         ui.signalContactDataAvailable(BezelCommand::RESENDING_CONTACT_POSITION);           
      }
   }



   TrackUiView::~TrackUiView() {
   // spin lock till we're clear to write new data by the mediator
      while(!ui.contactDataWasRead());
      strncpy(ui.actionData.name,track->getName().c_str(),UiMediator::MAX_CONTACTNAME_LENGTH-1);
      strncpy(ui.actionData.solution,track->getSolution().c_str(),UiMediator::MAX_CONTACTNAME_LENGTH-1);
      ui.signalContactDataAvailable(BezelCommand::REMOVE_CONTACT);
      if(track) track->removeView(this);
   }


/************ DiscreteTrackUiView ************/

void DiscreteTrackUiView::update(void *aTrackEvent) {
  if(((TrackEvent*)aTrackEvent)->eventType & TrackEvent::RESENDING_TRACK_POS) {
         copyContactDataToMediator();
         ui.signalContactDataAvailable(BezelCommand::RESENDING_CONTACT_POSITION);           
  } else if(((TrackEvent*)aTrackEvent)->eventType & (DiscreteTrackEvent::PAST_END_OF_TRACK | DiscreteTrackEvent::PAST_BEGINNING_OF_TRACK)) {
         copyContactDataToMediator();
         ui.signalContactDataAvailable(BezelCommand::REMOVE_CONTACT);
         newContactNotified = false;
  } else TrackUiView::update(aTrackEvent);
}

/************ OnwshipTrackUiView ************/

   void OwnshipTrackUiView::copyContactDataToMediator() {
      TrackUiView::copyContactDataToMediator();
      ui.actionData.precise = *(PreciseContact*)track->getCurrentCoordAndTime();
      ui.actionData.contactType = CONTACT_IS_OWNSHIP;
   }

/************ VehicleTrackUiView ************/


   void VehicleTrackUiView::copyContactDataToMediator() {
      TrackUiView::copyContactDataToMediator();
      ui.actionData.precise = *(PreciseContact*)track->getCurrentCoordAndTime();
      ui.actionData.precise.time -= startTime;
      ui.actionData.contactType = CONTACT_IS_VEHICLE;
   }

   void VehicleTrackUiView::notifyOfNewContact() {
      CoordAndTime *coord = track->getCurrentCoordAndTime();
      // check if the contact has a valid position; if so, then propagate the change
      if(coord != NULL) { 
        ui.actionData.precise = *(PreciseContact*)coord;
        startTime = ui.actionData.precise.time;
        TrackUiView::notifyOfNewContact();
      }
   }

   void VehicleTrackUiView::update(void *aTrackEvent) {
      if(((TrackEvent*)aTrackEvent)->eventType  & (TrackEvent::POS_CHANGED|TrackEvent::RESENDING_TRACK_POS)) {
       /* only propagate the change if this is a new vehicle, or if 
          this is the currently selected target (shared between Vehicles and Sensor data) */
         if(!newContactNotified || ui.selectedTarget == OID::fromObject(track)) {
            DiscreteTrackUiView::update(aTrackEvent);
         }
      }  
      else DiscreteTrackUiView::update(aTrackEvent);
   }


/************** SensorTrackUiView ************/


   void SensorTrackUiView::copyContactDataToMediator() {
      TrackUiView::copyContactDataToMediator();
      ui.actionData.fromSensor=*(ContactFromSensor*)track->getCurrentCoordAndTime();
      ui.actionData.fromSensor.time -= startTime;
      ui.actionData.contactType = CONTACT_FROM_SENSOR;
   }

   void SensorTrackUiView::notifyOfNewContact() {
      CoordAndTime *coord = track->getCurrentCoordAndTime();
      // check if the contact has a valid position; if so, then propagate the change
      if(coord != NULL) {
       ui.actionData.fromSensor=*(ContactFromSensor *)coord;
        startTime = ui.actionData.fromSensor.time;
        TrackUiView::notifyOfNewContact();
      }
   }

   void SensorTrackUiView::update(void *aTrackEvent) {
      if(((TrackEvent*)aTrackEvent)->eventType & (TrackEvent::POS_CHANGED|TrackEvent::RESENDING_TRACK_POS)) {
       /* only propagate the change if this sensor is new, or if this is the currently selected
          target (shared between Vehicles and Sensor data) */
         if(!newContactNotified || ui.selectedTarget == OID::fromObject(track)) {
            DiscreteTrackUiView::update(aTrackEvent);
         } 
      } 
      else DiscreteTrackUiView::update(aTrackEvent);
   }

/************** HypotheticalTrackUiView ************/


   void HypotheticalTrackUiView::copyContactDataToMediator() {
      TrackUiView::copyContactDataToMediator();
      ui.actionData.hypothetical=*(HypotheticalContact*)track->getCurrentCoordAndTime();
      ui.actionData.hypothetical.time -= startTime;
      ui.actionData.contactType = HYPOTHETICAL_CONTACT;
   }

   void HypotheticalTrackUiView::notifyOfNewContact() {
      CoordAndTime *coord = track->getCurrentCoordAndTime();
      // check if the contact has a valid position; if so, then propagate the change
      if(coord != NULL) {
       ui.actionData.hypothetical=*(HypotheticalContact *)coord;
        startTime = ui.actionData.hypothetical.time;
        TrackUiView::notifyOfNewContact();
      }
   }

   void HypotheticalTrackUiView::update(void *aTrackEvent) {
     if(((TrackEvent*)aTrackEvent)->eventType & (TrackEvent::POS_CHANGED|TrackEvent::RESENDING_TRACK_POS)) {
       /* only propagate the change if this hypothetical is new, or if this is the currently selected
          target */
       if(!newContactNotified || ui.selectedTarget == OID::fromObject(track)) {
	 DiscreteTrackUiView::update(aTrackEvent);
       } 
     } 
     else DiscreteTrackUiView::update(aTrackEvent);
   }



/************ WeaponTrackUiView ************/

   void WeaponTrackUiView::copyContactDataToMediator() {
      TrackUiView::copyContactDataToMediator();
      ui.actionData.weapon = *(WeaponContact*)track->getCurrentCoordAndTime();
      ui.actionData.weapon.time -= startTime;
      ui.actionData.contactType = CONTACT_IS_WEAPON;
   }

   void WeaponTrackUiView::notifyOfNewContact() {
      CoordAndTime *coord = track->getCurrentCoordAndTime();
      // check if the contact has a valid position; if so, then propagate the change
      if(coord != NULL) {
        ui.actionData.weapon = *(WeaponContact*)coord;
        startTime = ui.actionData.weapon.time;
        TrackUiView::notifyOfNewContact();
      }
   }

   void WeaponTrackUiView::update(void *aTrackEvent) {
      if(((TrackEvent*)aTrackEvent)->eventType & (TrackEvent::POS_CHANGED|TrackEvent::RESENDING_TRACK_POS)) {
       /* only propagate the change 
          if this weapon is new, or if this is the currently selected weapon. */
         if(!newContactNotified || ui.selectedWeapon == OID::fromObject(track)) {
            DiscreteTrackUiView::update(aTrackEvent);
         }
      }	
      else DiscreteTrackUiView::update(aTrackEvent);
   }
