#include <jni.h>

#include <signal.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtRenderArea.h>
//#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include "SoXtExaminerViewer.h"
#include <Inventor/Xt/viewers/SoXtWalkViewer.h>
#include <X11/IntrinsicP.h>
#include <Xm/Form.h>
#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/MainW.h>
#include <Inventor/nodes/SoLightModel.h>
#define USE_LIGHT
#ifdef USE_LIGHT
	#include <Inventor/nodes/SoDirectionalLight.h>
	#include <Inventor/nodes/SoEnvironment.h>
#endif
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoLOD.h>
#include <Inventor/SoInput.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/actions/SoGetBoundingBoxAction.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/actions/SoWriteAction.h>

// Used to add a texture on top of the map, currently the grid
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTextureCoordinatePlane.h>
#include <Inventor/nodes/SoTexture2Transform.h>

// Used to render the little map in the bezel
#include<Inventor/SoOffscreenRenderer.h>
#include<Inventor/nodes/SoOrthographicCamera.h>


#include "VoidPtrList.h"
#include <map>

#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoPickStyle.h>

#ifdef __linux__
  #include <values.h>
#else
  #include <limits.h>
#endif

// next 2 only used to debug the minimap
#if DBG_SAVE_MAP_IMAGE_TO_DISK
  #include <Inventor/SoOutput.h>
#endif

#include <assert.h>

#include <X11/Intrinsic.h>
#include <X11/Xlib.h>
#define XK_LATIN1 // get Latin-1 (iso 8859-1) keysyms
#define XK_MISCELLANY // get keypad keysyms
#include <X11/keysymdef.h>

#include "FeederConnection.h"
#include "FeederCommand.h"
#include "ContactEvent.h"
#include "timeval_ops.h"
#include "track.h"
#include "trackview.h"
#include "UiMediator.h"
#include "oid.h"
#include "TrackUiView.h"
#include "bezel/ui_area_sizes.h"
#include "IvHUD.h"
#include "IvColorScale.h"
#include "IvCompass.h"
#include "IvTiledBathymetry.h"
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/SoPath.h>
#include <Inventor/SoPickedPoint.h>
#include <Inventor/actions/SoRayPickAction.h>
#include "IvHighlightSelection2D.h"
#include "IvTransformEngine.h"
#include "IvContactLabeler2D.h"
#include "NameToTrackMapping.h"
#include "ContactIconCollection3D.h"
#include "IvContactLabeler2D.h"
#include "degree_conv.h"

#include "bezel/ui_constants.h"

   using namespace std;

   ObservableTrack *ownshipTrack=NULL;

   static SoXtFullViewer *myRenderArea = NULL;
   static Widget shell = NULL;
   static bool useWalkViewer = FALSE;
   static FeederConnection feeder;
   static UiMediator *talkWithBezel = NULL;
   bool waterSurface = false;

   // These are the colors used for contact labels
   static const float OWNSHIP_LABEL_COLOR[3] = { 0.5   , 0.5,     1.0f};
   static const float CONTACT_LABEL_COLOR[3] = { 1.0f,   0.5,     0.5 };
   static const float WEAPON_LABEL_COLOR[3]  = { 0.6   , 1.0f,    0.6 };

   /* This is the name of the file that contains the list of filenames for 3D icons. First
      icon in the list is icon #0, next is icon #1 and so forth. */
   static const char* const CONTACT_ICONLIST_FILE = "contactIcon.list";

// if FAST_TRANSPARENCY is defined, then we use Inventor's faster but not-so-nice way way to render translucient object
//#define FAST_TRANSPARENCY

// maximum number of contact events sent by the feeder we check for in one pass
#define MAX_SIMULTANEOUS_CONTACT_EVENTS 3

/* Boundaries of the total area possibly covered by the tileset. This is NOT
   the area covered by the current tileset, but the area for which the
   lowest left tile will be tile #0. */
#define MAP_MIN_LAT  -90.0f
#define MAP_MIN_LON -180.0f
#define MAP_MAX_LAT   90.0f
#define MAP_MAX_LON  180.0f

// width and length of a map tile in degrees - all tiles are assumed to be of equal size, except the tiles at the edge of the map
#define TILE_SIZE_X 0.5f
#define TILE_SIZE_Y 0.5f

// minimum distance between onwship and an edge in the visible area of the map before we 'slide' the map to show more tiles
#define EDGE_THRESHOLD (TILE_SIZE_X/5.0f)

//#define TILE_SCALING_TO_METERS_X DEGREES_TO_METERS(TILE_SIZE_X)
//#define TILE_SCALING_TO_METERS_Y DEGREES_TO_METERS(TILE_SIZE_Y)
#define TILE_SCALING_TO_METERS_X DEGREES_TO_METERS(1)
#define TILE_SCALING_TO_METERS_Y DEGREES_TO_METERS(1)
//#define TILE_EXAGGERATION_Z 1.0f
//Make it a dynamically changeable variable
   float TILE_EXAGGERATION_Z= 1.0f;


// this is the deepest point we expect contacts (depth of AOU/wedges) in feet
//#define BATHYMETRY_MAX_DEPTH -1000.0f
#define BATHYMETRY_MAX_DEPTH -1000.0f
#define AOU_MAX_DEPTH -300.0f

#define VERYSHALLOW_THRESHOLD 400.0f
#define SHALLOW_THRESHOLD 1000.0f
#define MIDRANGE_THRESHOLD 8000.0f
#define DEEP_THRESHOLD 16000.0f

// This is the radius in X,Y of the map area to show around the ownship
#define DEFAULT_MAP_RADIUS_AROUND_X (TILE_SIZE_X*0.5f)
#define DEFAULT_MAP_RADIUS_AROUND_Y (TILE_SIZE_Y*0.5f)

// This type defines the ways the bathymetry can be updated: wherever the user wants to go, or following the onwship
   typedef enum { FREE_MAP_BROWSING, MAP_FOLLOWS_OWNSHIP } MapUpdatingMode;

// This is the height of the isometric camera over the minimap; this value tells how much of the minimap is visible
#define CAMERA_HEIGHT_IN_MINIMAP 36400.0f

//enumerated type for how a camera should follow an object if it is following
   typedef enum {
     FOLLOW_SIDEVIEW,
     FOLLOW_TOPDOWN,
     FOLLOW_FIRSTPERSON,
     FOLLOW_CROSSVIEW,
     FOLLOW_WINGMAN
   } FOLLOW_STYLE;

/* These are  global objects so that the vertical exaggeration slider can use them to change the Z scale,
   and the wireframe drawing mode can be turned on or off */
   static   SoScale *exaggeration; //this is the scale that adjusts the vertical exaggeration of the bathymetry
   static   SoDrawStyle *bathymetryStyle;

   NameToTrackMapping trackDirectory; // creates tracks, and maps from track

/// a struct used in subve.c++ to pass all selection objects to callbacks
   struct SelectionBoxes {
      /// currently selected target in render area
      IvHighlightSelection2D *targetSelection;
      /// currently selected weapon in render area
      IvHighlightSelection2D *weaponSelection;
   };

   class IvContactViewHighlighter2D: public IvHighlightSelection2D {
   public:
     /// Creates a  selection object, that will work with a certain IvHUD's render area and 'aCamera'.
      IvContactViewHighlighter2D(IvHUD* theHUD, SoCamera* aCamera):IvHighlightSelection2D(theHUD,aCamera) {
      }
      virtual void updateBbox();
   };

   void IvContactViewHighlighter2D::updateBbox() {
      IvContactView* cview = (IvContactView*)getSelectedObject();
      if(cview != NULL) {
         if(cview->isVisible()) {
            show();
            IvHighlightSelection2D::updateBbox();
         }
         else {
            hide();
         }
      }
   }

   void changeColorscaleDepth(IvTiledBathymetry *b, IvColorScale* colorscale, float d) {
      b->setColorscaleDepth(0,FT_TO_M(d),64);
      colorscale->setRange(-d,0,9);
   }

   void setBathymetryToWireframe(SoDrawStyle* wireframe) {
      wireframe->style = SoDrawStyle::LINES;
   }

   /* This function is used to change the bathymetry to solid mode */
   void setBathymetryToSolid(SoDrawStyle* wireframe) {
      wireframe->style = SoDrawStyle::FILLED;
   }

   void makeLLString(char *res, float LL, bool isLat) {
     char dir='N';
     if (LL>=0) {
       if (isLat) {
	 dir='N';
       } else {
	 dir='E';
       }
     } else {
       LL=-LL;
       if (isLat) {
	 dir='S';
       } else {
	 dir='W';
       }
     }
     sprintf(res,"%1.2f%c\0\0",LL,dir);     
   }
/*this routine adds placement markers to the map*/
   void updateLocationMarkers(SoGroup* markerGroup, SbBox3f bbox) {
     float i,j;
     markerGroup->removeAllChildren();
     char str[100];
     SoText2 *cur;
     SoTranslation *trans;
     SoSeparator *sep;

     //make the outer markings
     SoFont *font = new SoFont();
     font->name.setValue("defaultFont");
     font->size.setValue(18);
     markerGroup->addChild(font);
     SoMaterial *mat = new SoMaterial;
     mat->diffuseColor.setValue(0,0,0);
     markerGroup->addChild(mat);

     sep = new SoSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     trans->translation.setValue(bbox.getMin().getValue()[0],(bbox.getMin().getValue()[1]+bbox.getMax().getValue()[1])*0.5,20);
     cur = new SoText2;
     sep->addChild(cur);
     makeLLString(str,bbox.getMin().getValue()[0],false);
     cur->string.setValue(str);
     cur->justification = SoText2::RIGHT;

     sep = new SoSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     trans->translation.setValue(bbox.getMax().getValue()[0],(bbox.getMax().getValue()[1]+bbox.getMin().getValue()[1])*0.5,20);
     cur = new SoText2;
     sep->addChild(cur);
     makeLLString(str,bbox.getMax().getValue()[0],false);
     cur->string.setValue(str);
     cur->justification = SoText2::LEFT;

     sep = new SoSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     trans->translation.setValue((bbox.getMin().getValue()[0]+bbox.getMax().getValue()[0])*0.5,bbox.getMin().getValue()[1],20);
     cur = new SoText2;
     sep->addChild(cur);
     makeLLString(str,bbox.getMin().getValue()[1],true);
     cur->string.set1Value(1,str);
     cur->justification = SoText2::CENTER;

     sep = new SoSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     trans->translation.setValue((bbox.getMin().getValue()[0]+bbox.getMax().getValue()[0])*0.5,bbox.getMax().getValue()[1],20);
     cur = new SoText2;
     sep->addChild(cur);
     makeLLString(str,bbox.getMax().getValue()[1],true);
     cur->string.setValue(str);
     cur->justification = SoText2::CENTER;

     SoLOD *details=new SoLOD;
     markerGroup->addChild(details);
     details->center.setValue((bbox.getMin().getValue()[0]+bbox.getMax().getValue()[0])*0.5,(bbox.getMin().getValue()[1]+bbox.getMax().getValue()[1])*0.5,20);
     details->range.set1Value(0,10000);
     details->range.set1Value(1,80000);
     SoGroup *detailGroup;
     float steps[] = {0.166666667,0.333333333,0.5};
     int numsteps = 3;
     for (int stepi = 0;stepi<numsteps;stepi++) {
       float step = steps[stepi];
       detailGroup = new SoGroup;
       details->addChild(detailGroup);
       for (i=bbox.getMin().getValue()[0];i<bbox.getMax().getValue()[0]-step*0.1;i+=step) {
	 for (j=bbox.getMin().getValue()[1];j<bbox.getMax().getValue()[1]-step*0.1;j+=step) {
	   if (i>bbox.getMin().getValue()[0]) {
	     sep = new SoSeparator;
	     detailGroup->addChild(sep);
	     trans = new SoTranslation;
	     sep->addChild(trans);
	     trans->translation.setValue(i,j+step*0.5,20);
	     cur = new SoText2;
	     sep->addChild(cur);
	     makeLLString(str,i,false);
	     cur->string.setValue(str);
	     cur->justification = SoText2::CENTER;
	   }
	   if (j>bbox.getMin().getValue()[1]) {
	     sep = new SoSeparator;
	     detailGroup->addChild(sep);
	     trans = new SoTranslation;
	     sep->addChild(trans);
	     trans->translation.setValue(i+step*0.5,j,20);
	     cur = new SoText2;
	     sep->addChild(cur);
	     makeLLString(str,j,true);
	     cur->string.setValue(str);
	     cur->justification = SoText2::CENTER;
	   }
	 }
       }
     }
   }
 void updateBoundingBoxAround1(SoNode* aNode, SoGroup* topBBox, const SbViewportRegion & vp, SbBox3f& bbox, bool trustBBox, bool surf) {
    if (!trustBBox) {
         SoGetBoundingBoxAction bboxAction =(vp);
         bboxAction.apply(aNode);
         bbox = bboxAction.getBoundingBox();
      }
      // IMPORTANT: Getting boundcoord right depends on the bounding box geometry having been created with a call to createBoundingBoxAround()
      SbVec3f *boundcoord = ((SoCoordinate3*)topBBox->getChild(2))->point.startEditing();
      if (surf){
     	 boundcoord[0].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
     	 boundcoord[1].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],0);
     	 boundcoord[2].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],0);
      	 boundcoord[3].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],0);
      	 boundcoord[4].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      }
      else{
        boundcoord[0].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
	boundcoord[1].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
	boundcoord[2].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
	boundcoord[3].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
 	boundcoord[4].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      }


      ((SoCoordinate3*)topBBox->getChild(0))->point.finishEditing();
      cout << "bathymetry bbox min = [" << bbox.getMin().getValue()[0] << ',' << bbox.getMin().getValue()[1] << ',' << bbox.getMin().getValue()[2] << "]," << endl;
      cout << "                max = [" << bbox.getMax().getValue()[0] << ',' << bbox.getMax().getValue()[1] << ',' << bbox.getMax().getValue()[2] << "]" << endl;

 }
   /* This function will update a box around an object created with a call to createBoundingBoxAround()
     to the current coordinates of the object's bounding box. */
   void updateBoundingBoxAround(SoNode* aNode, SoGroup* topBBox, const SbViewportRegion & vp, SbBox3f& bbox, bool trustBBox) {
      if (!trustBBox) {
         SoGetBoundingBoxAction bboxAction =(vp);
         bboxAction.apply(aNode);
         bbox = bboxAction.getBoundingBox();
      }
      // IMPORTANT: Getting boundcoord right depends on the bounding box geometry having been created with a call to createBoundingBoxAround()
      SbVec3f *boundcoord = ((SoCoordinate3*)topBBox->getChild(2))->point.startEditing();
      boundcoord[0].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[1].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[2].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[3].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[4].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);

      boundcoord[5].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[6].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[7].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[8].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[9].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);

      boundcoord[10].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[11].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[12].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[13].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[14].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);

      boundcoord[15].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[16].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[17].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[18].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[19].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);


      ((SoCoordinate3*)topBBox->getChild(0))->point.finishEditing();
      cout << "bathymetry bbox min = [" << bbox.getMin().getValue()[0] << ',' << bbox.getMin().getValue()[1] << ',' << bbox.getMin().getValue()[2] << "]," << endl;
      cout << "                max = [" << bbox.getMax().getValue()[0] << ',' << bbox.getMax().getValue()[1] << ',' << bbox.getMax().getValue()[2] << "]" << endl;
   }

   void setColor(SoMaterial* boxMat, float r, float g, float b, float tran){
     boxMat->ambientColor.set1Value(0,r,g,b);
     boxMat->diffuseColor.set1Value(0,r,g,b);
     boxMat->specularColor.set1Value(0,r,g,b);
     boxMat->emissiveColor.set1Value(0,r,g,b);
     boxMat->transparency.set1Value(0,tran);
}

   /* This function will create an inventor object drawing a box around an object's bounding box. */
   SoGroup* createBoundingBoxAround(SoNode* aNode, SoGroup* parent, const SbViewportRegion & vp, SbBox3f& bbox, bool trustBBox) {
      SoGroup *boxroot = new SoGroup;
      parent->addChild(boxroot);

      SoMaterialBinding *mbinding=new SoMaterialBinding;
      mbinding->value=SoMaterialBinding::OVERALL;

      boxroot->addChild(mbinding);
      SoMaterial *boxMat = new SoMaterial;
      setColor(boxMat, 0.0f,0.235f,1.0f, 0.5f);
      boxroot->addChild(boxMat);
      SoCoordinate3 *boundcoord = new SoCoordinate3;
      boundcoord->point.setNum(20); // A pairs of 5 pts for the 4 sides
      boxroot->addChild(boundcoord);
      updateBoundingBoxAround(aNode,boxroot,vp,bbox,trustBBox);

      SoFaceSet *boundary = new SoFaceSet;
      boundary->numVertices.setNum(4);
      boundary->numVertices.set1Value(0,5);
      boundary->numVertices.set1Value(1,5);
      boundary->numVertices.set1Value(2,5);
      boundary->numVertices.set1Value(3,5);
      boxroot->addChild(boundary);

      return boxroot;
   }

   SoGroup* createSurface(SoNode* aNode, SoGroup* parent, const SbViewportRegion & vp, SbBox3f& bbox, bool trustBBox) {
      SoGroup *boxroot = new SoGroup;
      parent->addChild(boxroot);

      SoMaterialBinding *mbinding=new SoMaterialBinding;
      mbinding->value=SoMaterialBinding::OVERALL;

      boxroot->addChild(mbinding);
      SoMaterial *boxMat = new SoMaterial;
      setColor(boxMat, 0.0f,0.235f,1.0f, 0.8f);
      boxroot->addChild(boxMat);
      SoCoordinate3 *boundcoord = new SoCoordinate3;
      boundcoord->point.setNum(5);
      boxroot->addChild(boundcoord);
      updateBoundingBoxAround1(aNode,boxroot,vp,bbox,trustBBox, waterSurface);

      SoFaceSet *boundary = new SoFaceSet;
      boundary->numVertices.setNum(1);
      boundary->numVertices.set1Value(0,5);
      boxroot->addChild(boundary);

      return boxroot;
   }

//called by the viewer when the slider dynamically changes Z scaling
   bool setVerticalExaggeration(unsigned int val){
      TILE_EXAGGERATION_Z=val;
     //change the exaggeration of the bathymetry
      exaggeration->scaleFactor.setValue(exaggeration->scaleFactor.getValue()[0],exaggeration->scaleFactor.getValue()[1],TILE_EXAGGERATION_Z);
     //change the scale of the tracks and contacts
      trackDirectory.applyScaleToTracks(exaggeration->scaleFactor.getValue()[0],exaggeration->scaleFactor.getValue()[1],exaggeration->scaleFactor.getValue()[2]);
     // we always accept the scaling factor sent, so we always return true
      return true;
   }

// called by the viewer when the user requested to toggle wireframe rendering on/off
   bool  toggleWireframeMode(bool drawAsWireframe) {
      if(drawAsWireframe)
         setBathymetryToWireframe(bathymetryStyle);
      else
         setBathymetryToSolid(bathymetryStyle);
    // we always accept request to toggle drawing style, so we return true
      return true;
   }



   SoGroup* createSceneGraph(SoGroup* root,
   SoTranslation*& mapSliding,
   SoScale*& mapScaling,
   SoSeparator*& mapLocations,
   IvTiledBathymetry*& bathymetry,
   SoDrawStyle*& bathymetryInWireframe,
   SoSeparator*& ownshipIcon, SoSeparator*& ownshipMarker,
   SoSeparator*& targetMarker,
   SoSeparator*& weaponIcon,  SoSeparator*& weaponMarker,
   SoSwitch*& colormapSwitch, SoSwitch *& textureSwitch,
   SoTexture2Transform *& rangeRingCenter) {

      SoGroup *group, *mapRoot;
      SoNode *node;
      #ifdef USE_LIGHT
      SoDirectionalLight *light = new SoDirectionalLight;
      light->intensity.setValue(1.0);
     //light->color.setValue(0.5,0.5,0);
      root->addChild(light);
      SoMaterial *myMaterial = new SoMaterial;
      myMaterial->diffuseColor.setValue(1,0,0);
      root->addChild(myMaterial);
      #endif

      mapRoot = new SoSeparator;
      root->addChild(mapRoot);

      mapScaling = new SoScale;
      mapRoot->addChild(mapScaling);

      mapSliding = new SoTranslation;
      mapRoot->addChild(mapSliding);

      mapLocations = new SoSeparator;
      mapRoot->addChild(mapLocations);

      SoInput loader;

   // load colormaps

      colormapSwitch = new SoSwitch();
      mapRoot->addChild(colormapSwitch);

      loader.openFile("color.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
   // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
   /* Now remove parent node added by loader.
    IMPORTANT:This works only if the colormap was the only child,
    which we know is true because we made the colormaps */
      group->removeChild(node);
      group->unref();

      loader.openFile("grayscale.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
   // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
      group->removeChild(node);
      group->unref();

      loader.openFile("bluescale.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
   // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
      group->removeChild(node);
      group->unref();


    loader.openFile("sandscale.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
   // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
      group->removeChild(node);
      group->unref();

   
   // set default colormap
      colormapSwitch->whichChild.setValue(3);
   
     // set style to be able to render bathymetry as wireframe when required
      bathymetryInWireframe = new SoDrawStyle;
      bathymetryInWireframe->style = SoDrawStyle::FILLED;
      mapRoot->addChild(bathymetryInWireframe);
   
      SoMaterialBinding *mapBinding = new SoMaterialBinding;
      mapBinding->value.setValue(SoMaterialBinding::PER_VERTEX_INDEXED);
      mapRoot->addChild(mapBinding);
   
      SoSeparator *mapTexRoot = new SoSeparator;
      mapRoot->addChild(mapTexRoot);

      textureSwitch=new SoSwitch;
      mapTexRoot->addChild(textureSwitch);
   
   // add a square texture on top of the map; cheap way to overlap a grid
      SoGroup *gridSep=new SoGroup;
      SoTexture2 *bottomgrid = new SoTexture2;
      gridSep->addChild(bottomgrid);
      bottomgrid->filename.setValue("square.rgb");
      bottomgrid->wrapS=SoTexture2::REPEAT;
      bottomgrid->wrapT=SoTexture2::REPEAT;
      SoTextureCoordinatePlane *texPlane1 = new SoTextureCoordinatePlane;
      texPlane1->directionS.setValue(SbVec3f(12,0,0));
      texPlane1->directionT.setValue(SbVec3f(0,12,0));
      gridSep->addChild(texPlane1);
      textureSwitch->addChild(gridSep);
   
   // add a range ring  texture on top of the map
      SoGroup *ringSep=new SoGroup;
      SoTexture2 *bottomcircle = new SoTexture2;
      ringSep->addChild(bottomcircle);
      bottomcircle->filename.setValue("circles.rgb");
      bottomcircle->wrapS=SoTexture2::CLAMP;
      bottomcircle->wrapT=SoTexture2::CLAMP;
      SoTexture2Transform *scale = new SoTexture2Transform;
      rangeRingCenter = new SoTexture2Transform;
      rangeRingCenter->translation.setValue(0,0);
      scale->translation.setValue(0.5,0.5);
      //1 km range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=111km*20/512
      //scale->scaleFactor.setValue(4.3359375,4.3359375);
      //1 kyd range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=121.36kyd*20/512
      //scale->scaleFactor.setValue(4.740625,4.740625);
      //2 kyd range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=121.36kyd*20/512 /2
      scale->scaleFactor.setValue(2.3703125,2.3703125);
      ringSep->addChild(scale);
      ringSep->addChild(rangeRingCenter);      
      SoTextureCoordinatePlane *texPlane2 = new SoTextureCoordinatePlane;
      //1 km range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=111km*20/512
      texPlane2->directionS.setValue(SbVec3f(1,0,0));
      texPlane2->directionT.setValue(SbVec3f(0,1,0));
      ringSep->addChild(texPlane2);
      textureSwitch->addChild(ringSep);
   
   // start with range rings
      textureSwitch->whichChild.setValue(1);
   
   // create tiled bathymetry object
      bathymetry = new IvTiledBathymetry();
      mapTexRoot->addChild(bathymetry);
      bathymetry->ref();
   
   // load icon for ownship
      loader.openFile("goodsub.iv");
      ownshipIcon = SoDB::readAll(&loader);
      loader.closeFile();
   
   // load marker icon for ownship's track
      loader.openFile("goodsub_marker.iv");
      ownshipMarker = SoDB::readAll(&loader);
      loader.closeFile();
   
   
   // load marker icon for target contact's track
      loader.openFile("track_marker.iv");
      targetMarker = SoDB::readAll(&loader);
      loader.closeFile();
      targetMarker->ref();
   
   // load icon for weapon
      loader.openFile("weapon.iv");
      weaponIcon = SoDB::readAll(&loader);
      loader.closeFile();
   
   // load marker icon for weapon's track
      loader.openFile("weapon_marker.iv");
      weaponMarker = SoDB::readAll(&loader);
      loader.closeFile();
   
      return mapRoot;
   }

   IvHUD* createHUD(SoGroup* root, SoXtRenderArea *renderArea, SoCamera* sceneCamera, SoTranslation* iconSliding, SoMFColor& colormap,
		    IvColorScale*& colorscale ,IvCompass*& compass, SelectionBoxes& selectionBoxes, IvContactLabeler2D*& contactLabeler) {
      IvHUD *hud = new IvHUD;
      hud->setRenderArea(renderArea);
      root->addChild(hud);
      // force lights off
      SoLightModel *lm = new SoLightModel;
      lm->model.setValue(SoLightModel::BASE_COLOR);
      hud->addChild(lm);
      colorscale = new IvColorScale;
      colorscale->setSize(30,200);
      colorscale->setPosition(8,16);
      colorscale->setColormap(colormap);
      colorscale->setRange(-DEEP_THRESHOLD,0,9);
      //hud->addChild(colorscale);

      compass = new IvCompass;
      compass->setSize(150,150);
      compass->setPosition(750,20);
      //hud->addChild(compass);

      selectionBoxes.targetSelection = new IvContactViewHighlighter2D(hud,sceneCamera);
      selectionBoxes.targetSelection->setColor(CONTACT_LABEL_COLOR[0],CONTACT_LABEL_COLOR[1],CONTACT_LABEL_COLOR[2]);
      hud->addChild(selectionBoxes.targetSelection);
   
      selectionBoxes.weaponSelection = new IvContactViewHighlighter2D(hud,sceneCamera);
      selectionBoxes.weaponSelection->setColor(WEAPON_LABEL_COLOR[0],WEAPON_LABEL_COLOR[1],WEAPON_LABEL_COLOR[2]);
      hud->addChild(selectionBoxes.weaponSelection);
   
      contactLabeler = new IvContactLabeler2D;
      contactLabeler->useHUD(hud,sceneCamera);
      hud->addChild(contactLabeler);
      hud->addChild(colorscale);
      hud->addChild(compass);
   
      return hud;
   }


   void setCameraToTopDownOverview(SoNode* aNode, SoCamera* camera, const SbViewportRegion & vp) {
      camera->orientation.setValue(0,0,0,1);
      camera->viewAll(aNode,vp);
   }

   void setCameraToTopDownPerspectiveOverview(SoNode* aNode, SoCamera* camera, const SbViewportRegion & vp) {
      camera->orientation.setValue(0.33119,0.0133465,0.00298144,0.943464);
      camera->viewAll(aNode,vp);
   }



  void setCameraTopDownCenteredOn(float xyz[3],const SbVec3f& scaling, const SbVec3f& sliding, SoCamera *camera) {
      SbVec3f target;
      camera->orientation.setValue(0,0,0,1);
      target.setValue((xyz[0]+sliding[0])*scaling[0],(xyz[1]+sliding[1])*scaling[1]+sliding[1],(xyz[2]+sliding[2])*scaling[2]);
      camera->position.setValue(target[0],target[1],target[2]+40000);
      camera->pointAt(target);
   }

   void setCameraSideOnWingMan(float xyz[3],const SbVec3f& scaling, const SbVec3f& sliding, float hpr[3],SoCamera *camera) {
      SbVec3f target;
      target.setValue((xyz[0]+sliding[0])*scaling[0],(xyz[1]+sliding[1])*scaling[1]+sliding[1],(xyz[2]+sliding[2])*scaling[2]);
      camera->position.setValue(target[0],target[1],target[2]);
      SbVec3f axis;
      SbRotation rot;
      SbRotation rot2;
      axis.setValue(1,0,0);
      rot.setValue(axis,DEGREE_TO_RADIANS(90.0));
      axis.setValue(0,1,0);
      rot2.setValue(axis,DEGREE_TO_RADIANS(hpr[2]));
      rot*=rot2;
      axis.setValue(1,0,0);
      rot2.setValue(axis,DEGREE_TO_RADIANS(hpr[1]));
      rot*=rot2;
      axis.setValue(0,0,1);
      rot2.setValue(axis,-DEGREE_TO_RADIANS(hpr[0]));
      rot*=rot2;
      SbMatrix mat,imat;
      SbVec3f source, dest;
      rot.getValue(mat);
      imat=mat.inverse();
      source.setValue(1000,0,-40);
      imat.multMatrixVec(source,dest);
      camera->position.setValue(target[0]+dest[0],target[1]+dest[1],target[2]+dest[2]);
      axis.setValue(0,0,1);
      rot2.setValue(axis,DEGREE_TO_RADIANS(90.0));
      rot*=rot2;
      camera->orientation.setValue(rot);
   }
   void setCameraSideOnCrossView(float xyz[3],const SbVec3f& scaling, const SbVec3f& sliding, float hpr[3],SoCamera *camera) {
      SbVec3f target;
      target.setValue((xyz[0]+sliding[0])*scaling[0],(xyz[1]+sliding[1])*scaling[1]+sliding[1],(xyz[2]+sliding[2])*scaling[2]);
      camera->position.setValue(target[0],target[1],target[2]);
      SbVec3f axis;
      SbRotation rot;
      SbRotation rot2;
      axis.setValue(1,0,0);
      rot.setValue(axis,DEGREE_TO_RADIANS(90.0));
      axis.setValue(0,1,0);
      rot2.setValue(axis,DEGREE_TO_RADIANS(hpr[2]));
      rot*=rot2;
      axis.setValue(1,0,0);
      rot2.setValue(axis,DEGREE_TO_RADIANS(hpr[1]));
      rot*=rot2;
      axis.setValue(0,0,1);
      rot2.setValue(axis,-DEGREE_TO_RADIANS(hpr[0]));
      rot*=rot2;
      SbMatrix mat,imat;
      SbVec3f source, dest;
      rot.getValue(mat);
      imat=mat.inverse();
      source.setValue(0,0,500);
      imat.multMatrixVec(source,dest);
      camera->position.setValue(target[0]+dest[0],target[1]+dest[1],target[2]+dest[2]);
      camera->orientation.setValue(rot);
   }

   void setCameraCenteredOn(float xyz[3],const SbVec3f& scaling, const SbVec3f& sliding, SoCamera *camera) {
      SbVec3f target;
      target.setValue((xyz[0]+sliding[0])*scaling[0],(xyz[1]+sliding[1])*scaling[1]+sliding[1],(xyz[2]+sliding[2])*scaling[2]);
      camera->position.setValue(target[0],target[1]-6000,target[2]+4000);
      camera->pointAt(target);
   }

   void setCameraFirstPersonFrom(float xyz[3],const SbVec3f& scaling, const SbVec3f& sliding, float hpr[3],SoCamera *camera) {
      SbVec3f target;
      target.setValue((xyz[0]+sliding[0])*scaling[0],(xyz[1]+sliding[1])*scaling[1]+sliding[1],(xyz[2]+sliding[2])*scaling[2]);
      camera->position.setValue(target[0],target[1],target[2]);
      SbVec3f axis;
      SbRotation rot;
      SbRotation rot2;
      axis.setValue(1,0,0);
      rot.setValue(axis,DEGREE_TO_RADIANS(90.0));
      axis.setValue(0,1,0);
      rot2.setValue(axis,DEGREE_TO_RADIANS(hpr[2]));
      rot*=rot2;
      axis.setValue(1,0,0);
      rot2.setValue(axis,DEGREE_TO_RADIANS(hpr[1]));
      rot*=rot2;
      axis.setValue(0,0,1);
      rot2.setValue(axis,-DEGREE_TO_RADIANS(hpr[0]));
      rot*=rot2;
      SbMatrix mat,imat;
      SbVec3f source, dest;
      rot.getValue(mat);
      imat=mat.inverse();
      source.setValue(0,0,-40);
      imat.multMatrixVec(source,dest);
      camera->position.setValue(target[0]+dest[0],target[1]+dest[1],target[2]+dest[2]);
      camera->orientation.setValue(rot);
   }


   SbBox3f fillMapCenteredOn(IvTiledBathymetry* bathymetry, SoXtRenderArea* renderArea, float x, float y) {
      bathymetry->fillVisibleAreaWithTiles(Rectangle2dFloat(x-DEFAULT_MAP_RADIUS_AROUND_X,y-DEFAULT_MAP_RADIUS_AROUND_Y, x+DEFAULT_MAP_RADIUS_AROUND_X,y+DEFAULT_MAP_RADIUS_AROUND_Y));
      SoGetBoundingBoxAction action(renderArea->getViewportRegion());
      action.apply(bathymetry);
      return action.getBoundingBox();
   }

///////////////////////////////////////////////////////////////////
// Create the viewer and lay it out so that it is attached
// to the bottom of the (optional) menu bar
//
   SoXtFullViewer *buildAndLayoutViewer(Widget parent, Widget menuWidget, bool walk) {
      Arg args[12];
      int n = 0;
      SoXtFullViewer *viewer = NULL;

      //if (!walk)
      viewer = new SoXtExaminerViewer(parent,NULL,TRUE,SoXtFullViewer::BUILD_DECORATION);
	 //else
         //viewer = new SoXtWalkViewer(parent);

      viewer->setViewing(TRUE);   // default to VIEW


   #ifndef FAST_TRANSPARENCY
      viewer->setTransparencyType(SoGLRenderAction::DELAYED_BLEND);
   #endif

   #ifdef USE_LIGHT
      viewer->setHeadlight(false);
   #endif

      if(menuWidget != NULL) {
         XtSetArg(args[n], XmNtopAttachment,    XmATTACH_WIDGET); n++;
         XtSetArg(args[n], XmNtopWidget,        menuWidget); n++;
      }
      XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
      XtSetArg(args[n], XmNleftAttachment,   XmATTACH_FORM); n++;
      XtSetArg(args[n], XmNrightAttachment,  XmATTACH_FORM); n++;
      XtSetValues(viewer->getWidget(), args, n);

      // set callbacks for special funtions: change Z scaling, and changing drawing mode
      viewer->setVerticalExaggerationAppCB(setVerticalExaggeration);
      viewer->setToggleWireframeStyleAppCB(toggleWireframeMode,false);

      return viewer;
   }


   void setNoBorder( Display *dpy, Window w, unsigned long decorations )
   {
      PropMotifWmHints motif_hints;
      Atom prop, proptype;

   /* setup the decoration property */
      motif_hints.flags = MWM_HINTS_DECORATIONS;
      motif_hints.decorations = decorations;
   
   /* get the atom for the property */
      prop = XInternAtom( dpy, "_MOTIF_WM_HINTS", True );
      if (!prop) {
      /* something went wrong! */
         return;
      }
   
   /* not sure this is correct, seems to work, XA_WM_HINTS didn't work */
      proptype = prop;
   
      XChangeProperty( dpy, w,           /* display, window */
         prop, proptype,                 /* property, type */
         32,                             /* format: 32-bit datums */
         PropModeReplace,                /* mode */
         (unsigned char *) &motif_hints, /* data */
         PROP_MOTIF_WM_HINTS_ELEMENTS    /* nelements */
         );
   }

/* specify the values for the Window Size Hints we want to enforce:*/
   void setWindowPosAndSize(Display *dpy, Window xwin, int x, int y, int w, int h) {
      XSizeHints hint; /* used to fix window size */
      hint.x = x;
      hint.y = y;
      hint.width  = w;
      hint.height = h;
      hint.win_gravity = StaticGravity;
      hint.flags  = PPosition | PSize | PWinGravity;
      XSetWMNormalHints (dpy, xwin, &hint);
   }

   Widget createXmWindow(Display *dpy, Widget shell, const char* title,
   short xorig, short yorig, short xsize, short ysize, 
   bool canChangeSize, bool hasBorder)   {
      Window xwin;                                    /* handle to the GL window */
      Atom atomName, atomType;                         /* used to make            */
      MotifWmHints motif_hints;                        /* window non-resizeable   */
      XSetWindowAttributes xswa;
   
    /* For some reason, setting the size hint is not enough, we also
    need to set the top widget properties */
      XtVaSetValues(shell,XmNx, xorig, XmNy, yorig, NULL);
      XtVaSetValues(shell, XmNwidth, xsize, XmNheight, ysize, NULL);
   
    /* create the window */
      xswa.colormap = DefaultColormap(dpy,DefaultScreen(dpy));
      xswa.border_pixel = 0;
      xswa.event_mask = StructureNotifyMask  | ButtonPressMask | ButtonReleaseMask
         | ExposureMask | KeyPressMask; /* express interest in events */
   
      XtCreateWindow(shell,InputOutput,DefaultVisual(dpy,DefaultScreen(dpy)),
         CWBorderPixel|CWColormap|CWEventMask, &xswa);
      XSync(dpy,False);
      xwin = XtWindow(shell);
   
    /* create motif widget for main window, the one that is visible */
      Widget main_window = XtCreateManagedWidget("mainWindow",xmMainWindowWidgetClass,shell,NULL,0);
   
   /* define string that will show up in the window title bar (and icon) */
      if(title != NULL & hasBorder)
         XStoreName(dpy, xwin, title);
   
   /* the following, culminating with XChangeProperty, enables us to remove
    *  the "[re]size" menu item from the default window menu, the "maximize"
    *  control button on the window frame, and the resize handle cursors
    *  that are normally available when the cursor moves into the proximity
    *  of any of the window's 4 corners.
    */
      if(!hasBorder || !canChangeSize) {
         if (!hasBorder)
            motif_hints.flags = MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS;
         else
            motif_hints.flags = MWM_HINTS_FUNCTIONS;
      /* remove the decorations from the function list we don't want to have */
         motif_hints.functions = MWM_FUNC_ALL | MWM_FUNC_RESIZE | MWM_FUNC_MAXIMIZE;
         motif_hints.decorations = 0; /* Mask value to specify no decorations */
         motif_hints.input_mode = 0;
      
      /* get the atom for the property */
         atomName = XInternAtom( dpy, "_MOTIF_WM_HINTS", True );
         if (atomName) {
         /* not sure this is correct, seems to work, XA_WM_HINTS didn't work */
            atomType = atomName;
            XChangeProperty( dpy, xwin,                         /* display, window */
               atomName, atomType,                 /* property, type */
               32,                             /* format: 32-bit datums */
               PropModeReplace,                /* mode */
               (unsigned char *) &motif_hints, /* data */
               PROP_MOTIF_WM_HINTS_ELEMENTS    /* nelements */
               );
         }
      }
   
   /* specify the values for the Window Size Hints we want to enforce */
      setWindowPosAndSize(dpy,xwin,xorig,yorig,xsize,ysize);
      return main_window;
   }

   void showWindow(Widget topWidget) {
      XtRealizeWidget(topWidget);
      XmUpdateDisplay(topWidget);  // better than XSync() because it redraws too
   }

   void cleanUpAndFinish(int signal = 0) {
   // IMPORTANT! clean-up shared memory
      if(talkWithBezel != NULL) delete talkWithBezel;
   // close socket connection to feeder before finishing
      feeder.closeConnection();
      //(*jvm)->DestroyJavaVM(jvm);
      exit(1);
   }


/*** feeder functions ****/

   void stopFeeder() {
      feeder.sendByte(FeederCommand::STOP_FEEDER);
   }

   void resumeFeeder() {
      feeder.sendByte(FeederCommand::RESUME_FEEDER);
   }

   void tellFeederToDie() {
      feeder.sendByte(FeederCommand::QUIT);
   }

/* Pick only if it's one of the contact icons.
   Called by a SoSelection node. */
   SoPath *pickFilter(void * selectionObject, const SoPickedPoint *pick)
   {
   // Find out which child of selection got picked
      SoNode *n;
      SoPath *path = pick->getPath();
      int i =0;
      while (i < path->getLength()) {
         n = path->getNode(i);
         if (n->isOfType(IvContactView::getClassTypeId())) // is it a contact view?
            break;
         i++;
      }
   	// was a IvContactView picked?
      if(i != path->getLength()) {
         // we found an IvContactView; discard the path after it, we don't care about the rest
         path->truncate(i+1);
         SelectionBoxes *selectionBoxes = (SelectionBoxes*)selectionObject;
         DiscreteTrack *track = static_cast<DiscreteTrack*>( ((IvContactView*)(n))->getModel() );
         switch(track->getContactType()) {
            case CONTACT_IS_WEAPON:
	      /*
		if(talkWithBezel->selectedWeapon != BAD_OID)
                  ((IvHighlightSelection2D*)(selectionBoxes->weaponSelection))->deselect();
               ((IvHighlightSelection2D*)(selectionBoxes->weaponSelection))->setSelection(path);
	      */
               talkWithBezel->selectedWeapon = track;
               talkWithBezel->askBezel(BezelCommand::SELECT_CONTACT,OID::fromObject(track));
               break;
            case CONTACT_IS_VEHICLE:
            case CONTACT_FROM_SENSOR:
	      /*
               if(talkWithBezel->selectedTarget != BAD_OID)
                  ((IvHighlightSelection2D*)(selectionBoxes->targetSelection))->deselect();
               ((IvHighlightSelection2D*)(selectionBoxes->targetSelection))->setSelection(path);
	      */
	       talkWithBezel->selectedTarget = track;
	       talkWithBezel->askBezel(BezelCommand::SELECT_CONTACT,OID::fromObject(track));
         }
       	 // resend the track's current position to the views, so the UI gets the current position of the new selection
         track->changed(TrackEvent::RESENDING_TRACK_POS);
         // we need to return the path to the SoSelection object for the SoSelection to force a repaint - strange...
         return path;
      }
   /* fool the SoSelection into thinking it failed to pick; we don't need it to actually
      keep any path info in the SoSelection, since we're keeping that information
      by ourselves in the selectionObject */
      return NULL;
   }


   SoPath* makePath(SoNode* root, SoNode* finalNode) {
      SoSearchAction nodeFinder;
      nodeFinder.setNode(finalNode);
      nodeFinder.setInterest(SoSearchAction::FIRST);
      nodeFinder.apply(root);
      SoPath* pathToNode = nodeFinder.getPath();
      pathToNode->ref();
      return pathToNode;
   }

   bool connectToFeeder(FeederConnection& feeder) {
      unsigned int attempts = 0;
     #define MAX_ATTEMPTS_TO_CONNECT 5
      while(attempts < MAX_ATTEMPTS_TO_CONNECT && feeder.connectToFeeder() < 0) {
         attempts++;
         usleep(500000);
      }
      return attempts != MAX_ATTEMPTS_TO_CONNECT;
   }


   SoSeparator* get3dIconFor(const ContactIconCollection3D& icons, Category vehicleID) {
   
      SoSeparator *icon;
      if(vehicleID >= icons.getSize()) {
         return (SoSeparator*)icons.getDefaultIcon()->copy();
      } 
      else {
         icon = (SoSeparator*)icons[(unsigned int)vehicleID];
         if(icon != NULL)
            return (SoSeparator*)icon->copy();
         else
            return (SoSeparator*)icons.getDefaultIcon()->copy();
      }
   }



/////////////////////////////////////////////////////
//////////                MAIN             //////////
/////////////////////////////////////////////////////

   int main(int argc, char**argv) {

   // First thing is to set signal handlers, to ensure cleaning up before finishing
      struct sigaction signalHandling;
      signalHandling.sa_handler = cleanUpAndFinish;
      signalHandling.sa_flags = 0;
      sigaction(SIGINT,&signalHandling,NULL);
      sigaction(SIGQUIT,&signalHandling,NULL);
      sigaction(SIGTERM,&signalHandling,NULL);


   /* create object that will communitate with bezel through shared memory,
      the object in itself is in shared memory */
      talkWithBezel = new(ObjectInSharedMemory::CREATE_WITH_KEY,UI_SHMEM_ID) UiMediator;

   // Establish connnection with contact data source
      cout << " Waiting for connection from Feeder...";
      cout.flush();
      if(!connectToFeeder(feeder)) {
         cerr << endl << "Could not connect to data source. Exiting." << endl;
         cleanUpAndFinish();
      }

      cout << "feeder connected." << endl;
      cout.flush();

      XtAppContext appContext;

   // Initialize Xt
      shell = XtAppInitialize(
         &appContext,                // context will be returned
         "SubVE",                 // application class
         NULL, 0,                    // options
         (int *) &argc, argv,        // command line args in/out
         NULL,                       // fallback_resources
         NULL, 0);

      /* get display - we can't use SoXt::getDisplay() yet */
      Display** dpys; Cardinal num;
      XtGetDisplays(appContext,&dpys, &num);
      Display * display = *dpys;
      XtFree((char*)dpys);



   /* build the main window - this is done FIRST (before initialzing
     Inventor and reading) to make the application startup feel
     faster. */

   /* get screen size */
      Widget main_window = createXmWindow(display, shell, NULL, 0,TOP_AREA_HEIGHT,
         WidthOfScreen(DefaultScreenOfDisplay(display))-RIGHT_AREA_WIDTH,
         HeightOfScreen(DefaultScreenOfDisplay(display))-TOP_AREA_HEIGHT-BOTTOM_AREA_HEIGHT,
         false, false);
   /* The following 2 lines are useful if we ever need to add more widgets, besides the viewer:
      Widget form = XtCreateWidget("Form", xmFormWidgetClass, main_window, NULL, 0);
      XtManageChild(form); */

      showWindow(shell);

   // Initialize Inventor
      SoXt::init(shell);

   /* Here initialize Inventor Types that are not built-in. This needs to
      be done after SoDB::init() is called, which is done by SoXt::init() */
      IvHUD::initClass();
      IvContactView::initClass();
      IvContactLabeler2D::initClass();
      IvColorScale::initClass();
      IvCompass::initClass();
      IvSwitchEngine::initClass();
      IvTransformEngine::initClass();
      IvTiledBathymetry::initClass();


   /* create root node for scene graph - root of scene graph must be a
      SoSelection if we want to be able to pick objects by clicking;
      otherwise we could just use a SoSeparator here */
      SoSelection *root = new SoSelection;

   // add a camera so the viewer does not add another one
      SoSwitch *cameraSwitch = new SoSwitch;
      root->addChild(cameraSwitch);
      SoPerspectiveCamera *navCamera = new SoPerspectiveCamera;
      navCamera->nearDistance.setValue(0.0f); // be sure everything will be visible in the map
      navCamera->farDistance.setValue(FLT_MAX);
      cameraSwitch->addChild(navCamera);
   // add an orthographic camera for the map view
      SoOrthographicCamera *topDownCamera = new SoOrthographicCamera;
   // 20k range
      topDownCamera->height.setValue(CAMERA_HEIGHT_IN_MINIMAP); // set height of camera view (works somewhat like distance)
      topDownCamera->nearDistance.setValue(0.0f); // be sure everything will be visible in the map
      topDownCamera->farDistance.setValue(FLT_MAX);
      topDownCamera->viewportMapping.setValue(SoCamera::ADJUST_CAMERA); // camera adjusts to viewport aspect ratio
      SbViewportRegion topDownViewport(MAP_IMAGE_X,MAP_IMAGE_Y); // size of top-down map image, must be set in the render area
      cameraSwitch->addChild(topDownCamera);

   // IMPORTANT: Set the switch to the default camera, otherwise the camera switch would say "use all cameras"
      cameraSwitch->whichChild.setValue(0);


   // Create and lay out the viewer
      myRenderArea = buildAndLayoutViewer(main_window, NULL, useWalkViewer);
   //  myRenderArea->setPopupMenuEnabled(FALSE);

   // set the background color to blue.
      //myRenderArea->setBackgroundColor(SbColor(0.0,0.7,1.0));
      myRenderArea->setBackgroundColor(SbColor(0.5,0.5,0.5));

   /* show the render area now, even if we didn't set up the scene graph yet,
     so that we can show something on screen while we build the scene graph */
      myRenderArea->show();
      XSync(XtDisplay(shell), False);

   // get render area position and size inside main window (needed for pick testing)
      short renderAreaXorig, renderAreaYorig, renderAreaXsize,renderAreaYsize;
      XtVaGetValues(myRenderArea->getRenderAreaWidget(),XmNx, &renderAreaXorig, XmNy, &renderAreaYorig, NULL);
      XtVaGetValues(myRenderArea->getRenderAreaWidget(), XmNwidth, &renderAreaXsize, XmNheight, &renderAreaYsize, NULL);


   // now setup scene graph
      SoSeparator *ownshipIcon, *ownshipTrackIcon,
         *contactIcon, *contactTrackIcon,
         *weaponIcon,  *weaponTrackIcon;
      SoSwitch     *colormapSwitch;
      SoGroup *mapRoot, *rootMapBox, *rootSurfBox;
      IvHUD *hud;
      IvColorScale *colorscale;
      IvCompass *compass;
      SelectionBoxes selectionBoxes;
      IvContactLabeler2D* contactLabeler;
      SbBox3f mapBBox;
      SoTranslation *mapSliding, *iconSliding;
      SoSeparator *mapLocations;
      IvTiledBathymetry *bathymetry;

      SoSwitch * textureSwitch;
      SoTexture2Transform * rangeRingCenter;

   // This is the collection of 3D icons we use to show ships, planes, etc
      ContactIconCollection3D iconCollection;
   // Load the collection of 3D icons
      if(iconCollection.load3dIconsFrom(CONTACT_ICONLIST_FILE) == ContactIconCollection3D::CANT_FIND_FILE) {
         printf("WARNING! Couldn't find icon list file '%s', all targets will look the same.\n",CONTACT_ICONLIST_FILE);
      }
      else if(iconCollection.getSize() == 0) {
        // for some reason, the icons weren't loaded
         printf("WARNING! Couldn't find any icon filename in file '%s', all targets will look the same.\n",CONTACT_ICONLIST_FILE);
      }
      iconCollection.setDefaultIconTo(iconCollection.getSize()-1);


      mapRoot = createSceneGraph(root,
         mapSliding,
         exaggeration,
	 mapLocations,
         bathymetry,
         bathymetryStyle,
         ownshipIcon,ownshipTrackIcon,
         contactTrackIcon,
         weaponIcon,weaponTrackIcon,
         colormapSwitch,textureSwitch,
         rangeRingCenter);

      // set scaling factor so that everything is in meters (bathymetry tiles are in degrees in X and Y, and in meters in Z), plus some vertical exaggeration
      exaggeration->scaleFactor.setValue(TILE_SCALING_TO_METERS_X,TILE_SCALING_TO_METERS_Y,TILE_EXAGGERATION_Z);
      bathymetry->initInstance(Rectangle2dFloat(MAP_MIN_LON,MAP_MIN_LAT,MAP_MAX_LON,MAP_MAX_LAT), Point2dFloat(TILE_SIZE_X,TILE_SIZE_Y), Point2dUint(2 ,2), 0, 0, 1 , 1, "TILES/tile_");
      bathymetry->useFastBoundingBox(true,FT_TO_M(BATHYMETRY_MAX_DEPTH),0.0f);

      SoGroup *iconRoot = new SoGroup;
      root->addChild(iconRoot);
      iconSliding = new SoTranslation;
      iconRoot->addChild(iconSliding);

      hud = createHUD(root,myRenderArea,navCamera,iconSliding,
         ((SoBaseColor*)colormapSwitch->getChild(3))->rgb,colorscale,
      compass,selectionBoxes, contactLabeler);
      compass->setCamera(navCamera);
      /* This engine solves the problem of how to apply the transformations
         affecting icons to the icon labels. Since the icon labels are
         drawn in 2D, any transformation accumulated when drawing the 3D icons is lost.
         What we do here is to use an Inventor engine to connect the transformations
         affecting 3D icons to the object that labels the icons, so that when
         one changes, the contactLabeler is notified and can apply the same
         transform to the label positions. */
      IvTransformEngine *mapPosTransforms = new IvTransformEngine;
      mapPosTransforms->translation.connectFrom(&iconSliding->translation);
      contactLabeler->transform.connectFrom(&mapPosTransforms->transfMatrix);

   // default scale to SHALLOW
      changeColorscaleDepth(bathymetry,colorscale,SHALLOW_THRESHOLD);

   // connect scene graph to render area, and make sure that the scene is visible
      myRenderArea->setSceneGraph(root);

      myRenderArea->getCamera()->orientation.setValue(0.33119,0.0133465,0.00298144,0.943464);
      myRenderArea->saveHomePosition();

      myRenderArea->getCamera()->viewAll(root,myRenderArea->getViewportRegion());

   /********************************************************/

   // make render area visible
      myRenderArea->show();
      SoXt::show(shell);

   // initialize picking policy and callbacks.
   // here we do all the setup (callbacks, etc) needed for object picking to work
      root->policy = SoSelection::SINGLE;
   /* filter wich objects are picked */
      root->setPickFilterCallback(pickFilter,&selectionBoxes);
      myRenderArea->redrawOnSelectionChange(root);

   // setup off-screen renderer for top-down image (minimap) of the bathymetry
      SoOffscreenRenderer *imageRenderer = new SoOffscreenRenderer(myRenderArea->getGLRenderAction());
      imageRenderer->setComponents(SoOffscreenRenderer::RGB);

   /* Get a SoPath from the scene graph root to the iconRoot node. We'll use this path
      when a contact needs to be highlighted. We could build this path by hand since
      we build the scene graph, but searching for the node after building
      the static part of the scene graph has the advantage that we can change
      the shape of the scene graph without changing how this path is obtained.
      Besides, we do this only once. */
      SoPath* pathToMapRoot = makePath(root,iconRoot);

   // Flag used to tell when we need to render a new minimap
      bool updateMiniMap = false;
      bool updateCamera = false;

   // Flag used to tell when we need to recenter the minimap
      bool recenterMiniMap = false;

   // Flags saying whether the camera should follow an object
   // And whether to do it top down or angle view
      bool followObjectMode = false;
      FOLLOW_STYLE followStyle = FOLLOW_SIDEVIEW;

   // This is a flag that tells how to update the bathymetry: following the ownship, or wherever the user wants to go
      MapUpdatingMode mapUpdatingMode = MAP_FOLLOWS_OWNSHIP;

   // Flag used to tell when we need to recenter the rangeRings
      bool showRings = true;

   // get keycode translations - they change from one keyboard to another
      const KeyCode kc_Escape = XKeysymToKeycode(display,XK_Escape);
      const KeyCode kc_c = XKeysymToKeycode(display,XK_C);
      const KeyCode kc_h = XKeysymToKeycode(display,XK_H);
      const KeyCode kc_g = XKeysymToKeycode(display,XK_G);
      const KeyCode kc_l = XKeysymToKeycode(display,XK_L);
      const KeyCode kc_s = XKeysymToKeycode(display,XK_S);
      const KeyCode kc_r = XKeysymToKeycode(display,XK_R);
      const KeyCode kc_kp_plus = XKeysymToKeycode(display,XK_KP_Add);
      const KeyCode kc_kp_minus = XKeysymToKeycode(display,XK_KP_Subtract);
      const KeyCode kc_up = XKeysymToKeycode(display,XK_Up);
      const KeyCode kc_down = XKeysymToKeycode(display,XK_Down);
      const KeyCode kc_left = XKeysymToKeycode(display,XK_Left);
      const KeyCode kc_right = XKeysymToKeycode(display,XK_Right);

      Timeline timeline; // object that manages all tracks


      /**** main loop ****/
      if (SoXt::getTopLevelWidget == NULL) {
      #ifndef NDEBUG
         SoDebugError::post("SubVE main loop:",
            "exiting -- SoXt::init never called");
      #endif
         exit(1);
      }

      // variables to create, keep track and display ships
      ContactEvent contactEvent;
      ContactEvent *intersect=NULL;
      ObservableTrack *track, *followTrack=NULL;
      IvContactView * contact;
      TrackTraceDecorator *trackTrace;
      IvTrackView *traceView;
      TrackUiView *trackUiView;

      CoordAndTime *coordAndTime;
   	// This object is used to change the color of 3D icons when the threat class changes
      Iv3dIconColorizer iconColorizer;


      int bytes_read; // number of bytes read when new contact data is available
      int contactEventsRead = 0; //  number of contact events received in one pass

      float sliding[3];
      int xtiles=0, ytiles=0;
      sliding[0]=sliding[1]=sliding[2]=0.0f;

















      JavaVMOption options[2];
      JavaVMInitArgs vm_args;
  JavaVM *jvm;
  JNIEnv *env;
  long result;
  jmethodID mid;
  jfieldID fid;
  jobject jobj;
  jclass cls;
  int i, asize;

  options[0].optionString = ".";
  options[1].optionString = "-Djava.compiler=NONE";

      vm_args.version = JNI_VERSION_1_4;
      //JNI_GetDefaultJavaVMInitArgs(&vm_args);
  vm_args.options = options;
  vm_args.nOptions = 2;
  vm_args.ignoreUnrecognized = JNI_TRUE;

  //result = JNI_CreateJavaVM(
  //           &jvm,(void **)&env, &vm_args);
  //if(result == JNI_ERR ) {
  //  printf("Error invoking the JVM");
  //  exit (-1);
  //}

  //cls = (env)->FindClass("TimeBearing");
  //if( cls == NULL ) {
  //  printf("can't find class ArrayHandler\n");
  //  exit (-1);
  //}
  //(env)->ExceptionClear();
  //mid=(env)->GetStaticMethodID(cls,"main","([Ljava/lang/String;)V");
  //mid=(*env)->GetMethodID(env, cls, "<init>", "()V");
  //jobj=(*env)->NewObject(env, cls, mid);
  //fid=(*env)->GetFieldID(env, cls, "arraySize", "I");
  //asize=(*env)->GetIntField(env, jobj, fid);

  //printf("size of array is %d",asize);
  //(*jvm)->DestroyJavaVM(jvm);
















      while(!talkWithBezel->readyToQuit()) {
         contactEventsRead = 0;
        // process contact events
         while((intersect)||(feeder.isDataAvailable() && contactEventsRead <= MAX_SIMULTANEOUS_CONTACT_EVENTS)) {
          // check that the socket is still there
 	    if (intersect) {
	      contactEvent=*intersect;
	      delete intersect;
	      intersect=NULL;
	      bytes_read=1;
	    } else {
	      bytes_read = feeder.readEvent(contactEvent);
	    }
	    if (bytes_read!=-1) {
            /* check that there actually was something to read, otherwise it may just
               be that the socket is down at the server side */
               if(bytes_read != 0) {
                  contactEventsRead++;
                  track = trackDirectory.findTrackForContact(contactEvent.name,contactEvent.solution);
                  if(track == NULL) { // we don't have a track object associated to the track name given
                     // Create new track, associate a 3D icon to it, etc etc
                     track = trackDirectory.addTrackForContact(contactEvent.name,contactEvent.solution,contactEvent.contactType);
                     if(contactEvent.contactType == CONTACT_FROM_SENSOR) {
		        if (contactEvent.category==SUR)
                        // create an AOU with the depth of the bathymetry
			  contact = new IvAOUView(10,0);
		        else if (contactEvent.category==AIR)
			  contact = new IvAOUView(0,AOU_MAX_DEPTH);
			else
			  contact = new IvAOUView(-AOU_MAX_DEPTH,0);
		     }
                     else if(contactEvent.contactType == HYPOTHETICAL_CONTACT)
                        // create an AOU with the depth of the bathymetry
                        contact = new IvHypotheticalView(&trackDirectory);
                     else if(contactEvent.contactType == CONTACT_IS_WEAPON){
		    	 contact = new IvWPNView();
			}
		     else
                        contact = new IvPreciseContactView;
                     contact->setModel(track,OBSERVER_IS_CONTACTVIEW);
                     contact->scaleBy(exaggeration->scaleFactor.getValue()[0],exaggeration->scaleFactor.getValue()[1],exaggeration->scaleFactor.getValue()[2]);
                     iconRoot->addChild(contact);

                  // add a track trace object, to show where the contact has been in the past
                     trackTrace = new TrackTraceDecorator(track);
                  // create the track view, that shows the track in the scene
                     traceView = new IvTrackView();
                     traceView->setMinMarkerDistanceSeparation(0.05);
                  // the track view depends on the updates from the track trace
                     traceView->setModel(trackTrace);
                  // tell the track directory to talk to the the track through the track trace
                     trackDirectory.replaceExistingTrackForContact(contactEvent.name,contactEvent.solution,trackTrace);
                     traceView->scaleBy(exaggeration->scaleFactor.getValue()[0],exaggeration->scaleFactor.getValue()[1],exaggeration->scaleFactor.getValue()[2]);
		     if(contactEvent.contactType != HYPOTHETICAL_CONTACT)
		       iconRoot->addChild(traceView);
		     else
		       traceView->ref();
		     if ((contactEvent.contactType != HYPOTHETICAL_CONTACT)&&(strcmp(track->getSolution().c_str(),"CCS")&&strcmp(track->getSolution().c_str(),""))) {
		       contact->hide();
		       traceView->hide();
		     } else {
		       contact->show();
		       traceView->show();
		     }
                  /* put trace track (with contact track inside) into timeline -
                  track is added to timeline before adding a new step, so the timeline can "see" the event */
                     timeline.addTrack(trackTrace);
                  /* Here we decide which type of icon to use for the contact,
                  add the new step to the corresponding track,
                     and we add an UI view to the track. */
                     switch(contactEvent.contactType) {
                        case CONTACT_IS_OWNSHIP:
                           ownshipTrack = track;
			   track->setThreatClass(FRIENDLY);
			   //track->setThreatClass(contactEvent.threat);
                           ((IvPreciseContactView*)contact)->setMarker(ownshipIcon);
                           traceView->setTraceColor(Iv3dIconColorizer::OWNSHIP_RGBCOLOR[0],Iv3dIconColorizer::OWNSHIP_RGBCOLOR[1],Iv3dIconColorizer::OWNSHIP_RGBCOLOR[2]);
                           traceView->setMarker(ownshipTrackIcon);
                           trackUiView = new OwnshipTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,OWNSHIP_LABEL_COLOR[0],OWNSHIP_LABEL_COLOR[1],OWNSHIP_LABEL_COLOR[2]);
                           //contactEvent.coord.precise.xyz[2]=-contactEvent.coord.precise.xyz[2];
                           timeline.appendStepToTrack(trackTrace,new PreciseContact(contactEvent.coord.precise));
                           if(mapUpdatingMode == MAP_FOLLOWS_OWNSHIP) {
                             // since this is the first time we see our sub, we fill the bathymetry with our sub in the center
                              mapBBox=fillMapCenteredOn(bathymetry,myRenderArea,contactEvent.coord.precise.xyz[0],contactEvent.coord.precise.xyz[1]);
                              rootMapBox=createBoundingBoxAround(bathymetry,mapRoot,myRenderArea->getViewportRegion(),mapBBox,true);
			      rootSurfBox=createSurface(bathymetry,mapRoot,myRenderArea->getViewportRegion(),mapBBox,true);
                              setCameraToTopDownPerspectiveOverview(root,myRenderArea->getCamera(),myRenderArea->getViewportRegion());
			      updateLocationMarkers(mapLocations,mapBBox);
                              recenterMiniMap=true;
                              updateMiniMap=true;
			   }
                           break;
                        case CONTACT_IS_VEHICLE:
			   track->setThreatClass(contactEvent.threat);
                           // assign a color to contact based on threat level. It will be updated automatically
                           contact->setMarkerColorUsing(&iconColorizer);
                           // assign an icon to the contact, based on contact classification
                           contactIcon = get3dIconFor(iconCollection,contactEvent.category);
                           ((IvPreciseContactView*)contact)->setMarker(contactIcon);
                           traceView->setTraceColorUsing(&iconColorizer);
                           traceView->setMarker(contactTrackIcon->copy());
                           trackUiView = new VehicleTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,CONTACT_LABEL_COLOR[0],CONTACT_LABEL_COLOR[1],CONTACT_LABEL_COLOR[2]);
                           //contactEvent.coord.precise.xyz[2]=-contactEvent.coord.precise.xyz[2];
                           timeline.appendStepToTrack(trackTrace,new PreciseContact(contactEvent.coord.precise));
                           break;
                        case CONTACT_IS_WEAPON:
                           contactIcon = get3dIconFor(iconCollection,WEAPON);
                           contact->setMarkerColorUsing(&iconColorizer);
                           ((IvWPNView*)contact)->setMarker(contactIcon);
                           traceView->setMarker(weaponTrackIcon);
                           traceView->setTraceColor(Iv3dIconColorizer::WEAPON_RGBCOLOR[0],Iv3dIconColorizer::WEAPON_RGBCOLOR[1],Iv3dIconColorizer::WEAPON_RGBCOLOR[2]);
                           trackUiView = new WeaponTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,WEAPON_LABEL_COLOR[0],WEAPON_LABEL_COLOR[1],WEAPON_LABEL_COLOR[2]);
                           //contactEvent.coord.weapon.xyz[2]=-contactEvent.coord.weapon.xyz[2];
                           timeline.appendStepToTrack(trackTrace,new WeaponContact(contactEvent.coord.weapon));
                          break;
                        case CONTACT_FROM_SENSOR:
			   track->setThreatClass(contactEvent.threat);
                           // assign a color to contact based on threat level. It will be updated automatically
                           contact->setMarkerColorUsing(&iconColorizer);
                           traceView->setTraceColorUsing(&iconColorizer);
                           traceView->setMarker(contactTrackIcon->copy());
                           trackUiView = new SensorTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,CONTACT_LABEL_COLOR[0],CONTACT_LABEL_COLOR[1],CONTACT_LABEL_COLOR[2]);
                           // invert Z, depth is sent as a positive value
                           //contactEvent.coord.fromSensor.xyz[2]=-contactEvent.coord.fromSensor.xyz[2];
                           timeline.appendStepToTrack(trackTrace,new ContactFromSensor(contactEvent.coord.fromSensor));
                           break;
                        case HYPOTHETICAL_CONTACT:
			   track->setThreatClass(contactEvent.threat);
                           // assign a color to contact based on threat level. It will be updated automatically
                           contact->setMarkerColorUsing(&iconColorizer);
                           traceView->setTraceColorUsing(&iconColorizer);
                           traceView->setMarker(contactTrackIcon->copy());
                           trackUiView = new HypotheticalTrackUiView(*talkWithBezel);
                           contactLabeler->displayLabelForContactView(contact,CONTACT_LABEL_COLOR[0],CONTACT_LABEL_COLOR[1],CONTACT_LABEL_COLOR[2]);
                           timeline.appendStepToTrack(trackTrace,new HypotheticalContact(contactEvent.coord.hypothetical));
                           break;
                     }
                     trackUiView->setModel(track);

                     track = trackTrace;
                  }
                  else { // update an already existing track
                     switch(contactEvent.contactType) {
                        case CONTACT_IS_OWNSHIP:
			  //contactEvent.coord.precise.xyz[2]=-contactEvent.coord.precise.xyz[2];
                           timeline.appendStepToTrack(track, new PreciseContact(contactEvent.coord.precise));
                           break;
                        case CONTACT_FROM_SENSOR:
			  //contactEvent.coord.fromSensor.xyz[2]=-contactEvent.coord.fromSensor.xyz[2];
                           timeline.appendStepToTrack(track, new ContactFromSensor(contactEvent.coord.fromSensor));
                           if(contactEvent.threat != track->getThreatClass()) {
                              track->setThreatClass(contactEvent.threat);
                           }
                           break;
                        case HYPOTHETICAL_CONTACT:
                           timeline.appendStepToTrack(track, new HypotheticalContact(contactEvent.coord.hypothetical));
                           if(contactEvent.threat != track->getThreatClass()) {
                              track->setThreatClass(contactEvent.threat);
                           }
			   break;
                        case CONTACT_IS_WEAPON:
			  //contactEvent.coord.weapon.xyz[2]=-contactEvent.coord.weapon.xyz[2];
                           timeline.appendStepToTrack(track, new WeaponContact(contactEvent.coord.weapon));
			   break;
                        default:
			  //contactEvent.coord.precise.xyz[2]=-contactEvent.coord.precise.xyz[2];
                           timeline.appendStepToTrack(track, new PreciseContact(contactEvent.coord.precise));
                           if(contactEvent.threat != track->getThreatClass()) {
                              track->setThreatClass(contactEvent.threat);
                           }
                     }
                  }
               
                  // if timeline is stopped, don't try to update the minimap
                  if(!timeline.isStopped()) {
                     updateMiniMap = true;
                     if(contactEvent.contactType == CONTACT_IS_OWNSHIP)
                        recenterMiniMap = true;
                  }
               }
            }
            else { // something went wrong, close the socket
               feeder.closeConnection();
            }
         }
      
      /* process UI pending command */
         if(talkWithBezel->isCommandAvailable())
            switch(talkWithBezel->command) {
            
               case MainAppCommand::STEP_FORWARD_IN_TIME:
                  timeline.setTimeDirection(FORWARD_IN_TIME);
                  timeline.playNextEvent();
                  updateMiniMap = true;
                  recenterMiniMap = true;
                  break;
               case MainAppCommand::STEP_BACK_IN_TIME:
                  timeline.setTimeDirection(BACKWARD_IN_TIME);
                  timeline.playNextEvent();
                  updateMiniMap = true;
                  recenterMiniMap = true;
                  break;
               case MainAppCommand::STOP_MOVEMENT:
                  timeline.stop();
                  break;
               case MainAppCommand::RESUME_MOVEMENT:
                  timeline.resume();
                  break;
               case MainAppCommand::SET_COLOR_MAP:
                  colormapSwitch->whichChild.setValue(0);
                  colorscale->setColormap(((SoBaseColor*)colormapSwitch->getChild(0))->rgb);
                  break;
               case MainAppCommand::SET_GREY_MAP:
                  colormapSwitch->whichChild.setValue(1);
                  colorscale->setColormap(((SoBaseColor*)colormapSwitch->getChild(1))->rgb);
                  break;
               case MainAppCommand::SET_BLUE_MAP:
                  colormapSwitch->whichChild.setValue(2);
                  colorscale->setColormap(((SoBaseColor*)colormapSwitch->getChild(2))->rgb);
                  break;
               case MainAppCommand::SET_SAND_MAP:
                  colormapSwitch->whichChild.setValue(3);
                  colorscale->setColormap(((SoBaseColor*)colormapSwitch->getChild(3))->rgb);
                  break;

               case MainAppCommand::SET_CAMERA:
                  switch(talkWithBezel->commandArg.whereToPositionCamera) {
                     case MainAppCommand::TOP_DOWN_CAMERA:
                        setCameraToTopDownOverview(mapRoot,myRenderArea->getCamera(),myRenderArea->getViewportRegion());
                        break;
                     case MainAppCommand::TOP_DOWN_PERSPECTIVE_CAMERA:
                        setCameraToTopDownPerspectiveOverview(mapRoot,myRenderArea->getCamera(),myRenderArea->getViewportRegion());
                        break;

                     case MainAppCommand::TOP_DOWN_CAMERA_CENTERED_ON:
                        track = (ObservableTrack*)OID::toObject(talkWithBezel->commandArg.contactID);
			if(track) {
			  followStyle=FOLLOW_TOPDOWN;
			  followTrack=track;
			  coordAndTime = followTrack->getCurrentCoordAndTime();
			  setCameraTopDownCenteredOn(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),myRenderArea->getCamera());
			}
			break;
		     case MainAppCommand::CROSS:
		       track = (ObservableTrack*)OID::toObject(talkWithBezel->commandArg.contactID);
                        if(track) {
			  followStyle=FOLLOW_CROSSVIEW;
			  followTrack=track;
			  coordAndTime = followTrack->getCurrentCoordAndTime();
			  setCameraSideOnCrossView(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),coordAndTime->hpr,myRenderArea->getCamera());
                        }
			break;
		     case MainAppCommand::WINGMAN:
		       track = (ObservableTrack*)OID::toObject(talkWithBezel->commandArg.contactID);
                        if(track) {
			  followStyle=FOLLOW_WINGMAN;
			  followTrack=track;
			  coordAndTime = followTrack->getCurrentCoordAndTime();
			  setCameraSideOnWingMan(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),coordAndTime->hpr,myRenderArea->getCamera());
                        }
			break;
                     case MainAppCommand::CAMERA_CENTERED_ON:
                        track = (ObservableTrack*)OID::toObject(talkWithBezel->commandArg.contactID);
                        if(track) {
			  followStyle=FOLLOW_SIDEVIEW;
			  followTrack=track;
			  coordAndTime = followTrack->getCurrentCoordAndTime();
			  setCameraCenteredOn(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),myRenderArea->getCamera());
                        }
			break;
                     case MainAppCommand::CAMERA_FIRST_PERSON_FROM:
                        track = (ObservableTrack*)OID::toObject(talkWithBezel->commandArg.contactID);
                        if(track) {
			  followStyle = FOLLOW_FIRSTPERSON;
			  followTrack=track;
			  coordAndTime = followTrack->getCurrentCoordAndTime();
			  setCameraFirstPersonFrom(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),coordAndTime->hpr,myRenderArea->getCamera());
                        }
                        break;
                  }
                  break;
               case MainAppCommand::FOLLOW_ON:
 		  followObjectMode=true;
		  break;
		case MainAppCommand::FOLLOW_OFF:
 		  followObjectMode=false;
		  break;
     		case MainAppCommand::SURFACE_ON:
 		  waterSurface=true;
		  updateBoundingBoxAround1(bathymetry,rootSurfBox,myRenderArea->getViewportRegion(),mapBBox,false, waterSurface);
		  break;
		case MainAppCommand::SURFACE_OFF:
 		  waterSurface =false;
		  updateBoundingBoxAround1(bathymetry,rootSurfBox,myRenderArea->getViewportRegion(),mapBBox,false, waterSurface);
		  break;

               case MainAppCommand::SCROLL_VAL:
                  updateCamera = true;
                  updateMiniMap = true;
                  recenterMiniMap = true;
                  break;
               case MainAppCommand::SELECT_TARGET:
                  selectionBoxes.targetSelection->deselect();
               // the UI have stored the object id of the target to select in talkWithBezel->selectedWeapon
                  if(talkWithBezel->selectedTarget != BAD_OID) {
                     track = (ObservableTrack*)OID::toObject(talkWithBezel->selectedTarget);
                  // resend the track's current position to the views, so the UI gets the current position of the new selection
                     track->changed(TrackEvent::RESENDING_TRACK_POS);
                  // create a path from the scene graph root to the Inventor node used to show the target position. To do this:
                  // step 1) clone the path up to and before the Inventor node that represents the track;
                     SoPath *pathToView = pathToMapRoot->copy();
                  // step 2) Append the the Inventor node that represents the track to the path
                     pathToView->append((IvContactView*)track->findObserverWithMask(OBSERVER_IS_CONTACTVIEW,NULL));
                  // step 3) select the object by using the path
                     selectionBoxes.targetSelection->setSelection(pathToView);
                  }
                  break;
               case MainAppCommand::SELECT_WEAPON:
                  selectionBoxes.weaponSelection->deselect();
               // the UI have stored the object id of the weapon to select in talkWithBezel->selectedWeapon
                  if(talkWithBezel->selectedWeapon != BAD_OID) {
                     track = (ObservableTrack*)OID::toObject(talkWithBezel->selectedWeapon);
                  // resend the track's current position to the views, so the UI gets the current position of the new selection
                     track->changed(TrackEvent::RESENDING_TRACK_POS);
                  // create a path from the scene graph root to the Inventor node used to show the weapon position. To do this:
                  // step 1) clone the path up to and before the Inventor node that represents the track;
                     SoPath *pathToView = pathToMapRoot->copy();
                  // step 2) Append the the Inventor node that represents the track to the path
                     pathToView->append((IvContactView*)track->findObserverWithMask(OBSERVER_IS_CONTACTVIEW,NULL));
                  // step 3) select the object by using the path
                     selectionBoxes.weaponSelection->setSelection(pathToView);
                  }
                  break;
               case MainAppCommand::STOP_FEEDER:
                  stopFeeder();
                  break;
               case MainAppCommand::RESUME_FEEDER:
                  resumeFeeder();
                  break;
               case MainAppCommand::SHOW_INTERSECTING:
                  IvContactView::setShowIntersecting(true);
                  break;
               case MainAppCommand::HIDE_INTERSECTING:
                  IvContactView::setShowIntersecting(false);
                  break;
               case MainAppCommand::NAV_GRID:
                  textureSwitch->whichChild.setValue(0);
                  showRings=false;
                  break;
		case MainAppCommand::GROWREGION:
		  trackDirectory.makeTracksGrow(talkWithBezel->growRegion.numSolution,talkWithBezel->growRegion.solutionList);
		  //printf("grow : %d\n",talkWithBezel->growRegion.numSolution);
		  //for (int z=0;z<talkWithBezel->growRegion.numSolution;z++)
		  //printf("%s\n",talkWithBezel->growRegion.solutionList[z]);
		  break;
		case MainAppCommand::REDUCEREGION:
		  trackDirectory.makeTracksReduce(talkWithBezel->reduceRegion.numSolution,talkWithBezel->reduceRegion.solutionList);
		  break;
		  
	        case MainAppCommand::TARGET_SPEED:
		  //cout<<"Target Speed = "<<talkWithBezel->targetSpeed.targetSpeed << endl;
                  if(talkWithBezel->selectedTarget != BAD_OID) {
		    track = (ObservableTrack*)OID::toObject(talkWithBezel->selectedTarget);
		    if (track) track->setMaxSpeed(talkWithBezel->targetSpeed.targetSpeed);
		  }
		  break;
		  
               case MainAppCommand::HYPOTHESIS:
               //cout << "Max Speed = " << talkWithBezel->hypothesis.maxSpeed <<endl;
               //cout << "Max Depth = " << talkWithBezel->hypothesis.maxDepth <<endl;
               //cout << "Min Depth = " << talkWithBezel->hypothesis.minDepth <<endl;
                  break;
               case MainAppCommand::LON_LAT:
                  bathymetry->fillVisibleAreaWithTiles(Rectangle2dFloat(talkWithBezel->lonLat.lon_nw,talkWithBezel->lonLat.lat_se,talkWithBezel->lonLat.lon_se,talkWithBezel->lonLat.lat_nw ));
                  updateBoundingBoxAround(bathymetry,rootMapBox,myRenderArea->getViewportRegion(),mapBBox,false);
		  updateBoundingBoxAround1(bathymetry,rootSurfBox,myRenderArea->getViewportRegion(),mapBBox,false, waterSurface);
		  updateLocationMarkers(mapLocations,mapBBox);
                  break;
               case MainAppCommand::HYPO_CONTACTS:
		 intersect=new ContactEvent;
		 intersect->name="HYPOTH";
		 intersect->solution=talkWithBezel->hypoContact.regionName;
		 //intersect->solution="USR";
		 intersect->contactType=HYPOTHETICAL_CONTACT;
		 intersect->coord.hypothetical.numContacts=talkWithBezel->hypoContact.numContact;
		 for (int z=0;z<talkWithBezel->hypoContact.numContact;z++)
		   strcpy(intersect->coord.hypothetical.contactList[z],talkWithBezel->hypoContact.contactList[z]);
		 intersect->coord.hypothetical.time.tv_sec=1;
		 break;
               case MainAppCommand::SOLUTIONS:
		  trackDirectory.makeTracksVisible(talkWithBezel->solution.numSolution,talkWithBezel->solution.solutionList);
                  break;
               case MainAppCommand::RANGE_RINGS:
                  if (ownshipTrack != NULL) {
                     coordAndTime = ownshipTrack->getCurrentCoordAndTime();
                     rangeRingCenter->translation.setValue(-coordAndTime->xyz[0],-coordAndTime->xyz[1]);
                  }
                  textureSwitch->whichChild.setValue(1);
                  showRings=true;
                  break;
               case MainAppCommand::VERYSHALLOW:
                  changeColorscaleDepth(bathymetry,colorscale,VERYSHALLOW_THRESHOLD);
                  break;
               case MainAppCommand::SHALLOW:
                  changeColorscaleDepth(bathymetry,colorscale,SHALLOW_THRESHOLD);
                  break;
               case MainAppCommand::MIDRANGE:
                  changeColorscaleDepth(bathymetry,colorscale,MIDRANGE_THRESHOLD);
                  break;
               case MainAppCommand::DEEP:
                  changeColorscaleDepth(bathymetry,colorscale,DEEP_THRESHOLD);
                  break;
               case MainAppCommand::SNAPSHOT:
                  SoWriteAction *a=new SoWriteAction;
                  a->getOutput()->openFile("scene.iv");
                  a->getOutput()->setBinary(FALSE);
                  a->apply(root);
                  a->getOutput()->closeFile();
                  break;
            }
         talkWithBezel->signalCommandDone();
      
        /* test if there is any new X event before calling SoXt::nextEvent,
           otherwise it would block there till the next X event arrives */
         XEvent event; // UI events
         XtInputMask pendingEvents = XtAppPending(appContext);
      	/* process all pending X events before continuing,
      	   this makes the app more responsive */
         while(pendingEvents != 0) {
            if(pendingEvents & XtIMXEvent) {
               // process X events
               SoXt::nextEvent(appContext, &event);
               switch(event.type) {
                  case KeyPress: // process key presses
                     if(((XKeyEvent*)&event)->keycode == kc_Escape) { // ESC = Quit =>
                        talkWithBezel->signalQuit(); // tell the bezel to quit
                     }
                     else  if(((XKeyEvent*)&event)->keycode == kc_s) { // s = stop movement of all tracks
		       SoXt::dispatchEvent((XEvent*)&event);
                     }
                     else if(((XKeyEvent*)&event)->keycode == kc_kp_plus) { // plus = go forward in time
                        timeline.setTimeDirection(FORWARD_IN_TIME);
                        timeline.playNextEvent();
                        updateMiniMap = true;
                        recenterMiniMap = true;
                     }
                     else if(((XKeyEvent*)&event)->keycode == kc_kp_minus) {
                        timeline.setTimeDirection(BACKWARD_IN_TIME); // minus = go back in time
                        timeline.playNextEvent();
                        updateMiniMap = true;
                        recenterMiniMap = true;
                     }
                     else if(((XKeyEvent*)&event)->keycode == kc_r) {
		        timeline.rewind();
			updateMiniMap = true;
			recenterMiniMap = true;
		     }
                     else if(((XKeyEvent*)&event)->keycode == kc_c) { // re-center map in ownship
		       xtiles=0; ytiles=0;
                       coordAndTime=ownshipTrack->getCurrentCoordAndTime();
                       mapBBox=fillMapCenteredOn(bathymetry,myRenderArea,coordAndTime->xyz[0],coordAndTime->xyz[1]);
                       updateBoundingBoxAround(bathymetry,rootMapBox,myRenderArea->getViewportRegion(),mapBBox,true);
		       updateBoundingBoxAround1(bathymetry,rootSurfBox,myRenderArea->getViewportRegion(),mapBBox,true, waterSurface);
		       updateLocationMarkers(mapLocations,mapBBox);
                       setCameraToTopDownPerspectiveOverview(mapRoot,myRenderArea->getCamera(),myRenderArea->getViewportRegion());
                       recenterMiniMap=true;
                       updateMiniMap=true;
		       mapUpdatingMode = MAP_FOLLOWS_OWNSHIP;
                     } else if(xtiles == 0 && ytiles ==0) {
		        if(((XKeyEvent*)&event)->keycode == kc_right) {
                          xtiles+=1;
			  sliding[0]-=TILE_SIZE_X;
		        } else if(((XKeyEvent*)&event)->keycode == kc_left) {
		          xtiles+=-1;
			  sliding[0]+=TILE_SIZE_X;
		        } else if(((XKeyEvent*)&event)->keycode == kc_down) {
                          ytiles+=-1;
			  sliding[1]+=TILE_SIZE_Y;
		        } else if(((XKeyEvent*)&event)->keycode == kc_up) {
		          ytiles+=1;
			  sliding[1]-=TILE_SIZE_Y;
		        }
			if(xtiles !=0 || ytiles != 0) mapUpdatingMode = FREE_MAP_BROWSING;
                     }
                     break;
               
               
                  default:
                     SoXt::dispatchEvent((XEvent*)&event);
               }
            }
            else {
               XtAppProcessEvent(appContext,XtIMTimer | XtIMSignal);
            }
            pendingEvents = XtAppPending(appContext);
         }	 

	 /* If the bathymetry must follow the ownship, then check if ownship got too close to the edge.
	    If that is the case, then slide the map to show more of the bathymetry. */
         if(ownshipTrack!=NULL && mapUpdatingMode == MAP_FOLLOWS_OWNSHIP) {
             // check if the ownship is too close to the map edge; if so => slide map
	     coordAndTime = ownshipTrack->getCurrentCoordAndTime();
	     if (coordAndTime->xyz[0]-mapBBox.getMin()[0] < EDGE_THRESHOLD) {
	       xtiles+=-1;
               //sliding[0]+=TILE_SIZE_X;
            } 
            else if (mapBBox.getMax()[0]-coordAndTime->xyz[0] < EDGE_THRESHOLD) {
               xtiles=1;
               //sliding[0]-=TILE_SIZE_X;
	     }
             if (coordAndTime->xyz[1]-mapBBox.getMin()[1] < EDGE_THRESHOLD) {
	       ytiles+=-1;
	       //sliding[1]+=TILE_SIZE_Y;
             } else if (mapBBox.getMax()[1]-coordAndTime->xyz[1] < EDGE_THRESHOLD) {
               ytiles+=1;
               //sliding[1]-=TILE_SIZE_Y;
            }
         }
      
          // shift the map to show new tiles, if needed       
         if(ownshipTrack!=NULL && (xtiles !=0 || ytiles !=0)) {
         // shift map to get new tiles
            bathymetry->shiftMapBy(xtiles,ytiles);
         // move map so the area stays in the same position on the screen
            mapSliding->translation.setValue(sliding);
	    iconSliding->translation.setValue(sliding[0]*exaggeration->scaleFactor.getValue()[0],sliding[1]*exaggeration->scaleFactor.getValue()[1],0);
	    // update the box around the map with the new area covered by the new tiles
	    updateBoundingBoxAround(bathymetry,rootMapBox,myRenderArea->getViewportRegion(),mapBBox,false);
	    updateBoundingBoxAround1(bathymetry,rootSurfBox,myRenderArea->getViewportRegion(),mapBBox,false, waterSurface);
	    updateLocationMarkers(mapLocations,mapBBox);
	    updateMiniMap=true;
	    recenterMiniMap=true;
	    xtiles=ytiles=0;
	  }

   	  //try to keep range rings around ownship
	  if ((ownshipTrack!=NULL)&&(showRings)) {
	     coordAndTime = ownshipTrack->getCurrentCoordAndTime();
             rangeRingCenter->translation.setValue(-coordAndTime->xyz[0],-coordAndTime->xyz[1]);
	  }
	  //update camera if it is in follow mode
	  if ((followObjectMode)&&(followTrack)) {
	    coordAndTime = followTrack->getCurrentCoordAndTime();
	    switch (followStyle) {
 	      case FOLLOW_SIDEVIEW:
		setCameraCenteredOn(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),myRenderArea->getCamera());
		break;
 	      case FOLLOW_TOPDOWN:
		setCameraTopDownCenteredOn(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),myRenderArea->getCamera());
		break;
 	      case FOLLOW_FIRSTPERSON:
		setCameraFirstPersonFrom(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),coordAndTime->hpr,myRenderArea->getCamera());
		break;
              case FOLLOW_WINGMAN:
	        setCameraSideOnWingMan(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),coordAndTime->hpr,myRenderArea->getCamera());
		break;
              case FOLLOW_CROSSVIEW:
	        setCameraSideOnCrossView(coordAndTime->xyz,exaggeration->scaleFactor.getValue(),mapSliding->translation.getValue(),coordAndTime->hpr,myRenderArea->getCamera());
		break;



	    }
	  }

          if(ownshipTrack!=NULL) {
            // if timeline is stopped, don't try to update the minimap
            if(updateMiniMap) {
              /* something has moved, so we re-center top-down map camera
                 on ownship if ownship has moved, and then we render a new minimap */
               if(recenterMiniMap) {
                  coordAndTime = ownshipTrack->getCurrentCoordAndTime();
                  const SbVec3f& scaling = exaggeration->scaleFactor.getValue();
                  const SbVec3f& trans = iconSliding->translation.getValue();
                  SbVec3f pos(coordAndTime->xyz[0]*scaling[0]+trans[0],coordAndTime->xyz[1]*scaling[1]+trans[1],coordAndTime->xyz[2]*scaling[2]+trans[2]);
                  topDownCamera->position.setValue(pos);
               
                  if(updateCamera){
                  // initial height = +/-20K (18200) to +/-2K (1800), accounts for m to kyds conversion
                     topDownCamera->height.setValue(CAMERA_HEIGHT_IN_MINIMAP - talkWithBezel->scroll_val.scroll_val*((0.9*CAMERA_HEIGHT_IN_MINIMAP)));
                     updateCamera = false;
                  }
                 // clear re-center minimap flag
                  recenterMiniMap = false;
               }
               // copy current viewport, then change it to image size
               SbViewportRegion vp = myRenderArea->getViewportRegion();
               myRenderArea->setViewportRegion(topDownViewport);
               // set orthographic top-down camera as the active camera for minimap rendering
               cameraSwitch->whichChild.setValue(1);
               // turn off HUD, so it doesn't show up in  minimap
               hud->hide();
               imageRenderer->render(root);
            
            /*******************************
                * send minimap image to bezel *
            *******************************/
               talkWithBezel->updateMap(imageRenderer->getBuffer());
               #if DBG_SAVE_MAP_IMAGE_TO_DISK
                 // saves in SGI RGB image format
                 char filename[128];
                 sprintf(filename,"i%g_%g_%g.rgb",contactEvent.coord.xyz[0],contactEvent.coord.xyz[1],contactEvent.coord.xyz[2]);
                 FILE *fp = fopen(filename,"wb");
                 imageRenderer->writeToRGB(fp);
                 fclose(fp);
                 cout << "subve debug: image saved in file '" << filename << "'\n"; cout.flush();
               #endif
            
               // make HUD visible again
               hud->show();
               // reset perspective camera for normal rendering
               cameraSwitch->whichChild.setValue(0);
               // restore viewport for normal viewing
               myRenderArea->setViewportRegion(vp);
               // clear updating flag
               updateMiniMap = false;
            }
         }
      }
      tellFeederToDie();
      pathToMapRoot->unref();
      root->unref();
      cleanUpAndFinish();
   }
