// New bezel 02/20/03

#include "jni.h"
#include "bezel.h"
#include "datetime.h"
#include <FL/Fl_Pixmap.H>
#include <FL/Fl_Image.H>
#include <FL/Fl_Menu.H>
#include <FL/fl_draw.H>
#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <signal.h>
#include <unistd.h>
#include <math.h>
#include <vector>
#include <map>

#include "ui_constants.h"
#include  "ui_area_sizes.h"
#include  "ui_app_defaults.h"	// for DEFAULT constants

#include "../UiMediator.h"
#include "../coord.h"
#include "../oid.h"
#include <X11/Xlib.h>

#include "../maxDisplaySize.h"

// object used to communicate with main app via shared memory
   static UiMediator *talkWithMainApp = NULL;

#define UI_NUMFIELD_MAX_LENGTH  24

   OID::Type ownshipID = BAD_OID;

   using namespace std;

   typedef vector<OID::Type> oidlist;

   oidlist solutionsChosen;
   oidlist defaultSolutions;
   oidlist growList;
   oidlist reduceList;

   char ownshipLONLabel[UI_NUMFIELD_MAX_LENGTH];
   char ownshipLATLabel[UI_NUMFIELD_MAX_LENGTH];
   int numSol=0;
   struct right_ui uiRight;
//struct bottom_ui uiBottom;
   static bool running = true;
   
// JAVA
   JavaVM *jvm;
   JNIEnv *env;
   jclass cls;


   

// little convenience string, that we use whenever we need to clear a text field
   static const char* const emptyString = " ";


// Toggles button functionality on and off, value set by command-line argument
   int buttonsOff = 1;

   

/***************************************************************************/
/*                    Callback function declarations                              */
/* Following block of function declarations are the callback functions that send */
/* command and data to subve.                                                      */
/***************************************************************************/
   static void Follow (Fl_Widget*);	   // to follow objects
   static void Pause (Fl_Widget*);	   // to pause the timeline
   static void Forward (Fl_Widget*);	   // to step forward in the timeline
   static void Backwards (Fl_Widget*);	   // to step backward in the timeline
   static void Snapshot (Fl_Widget*);	   // to take a snapshot of the current scene
   static void Toggle_target (Fl_Widget*); // toggle between showing/hiding the intersections
   static void ReduceRegion (Fl_Widget*); // toggle between reducing the region of the AOUS
   static void GrowRegion (Fl_Widget*); // toggle between showing/hiding the growth region of the AOUS
   static void NavGrid (Fl_Widget*);	   // to set the navigation grid on
   static void SolutionOnOff (Fl_Widget*);
   static void scroll_cb(Fl_Widget*);	   // to zoom in/out of the topdown display
   static void Sensor (void);

   // to selsct and display the target item from the target menu
   static void TargetMenuItemSelected(Fl_Widget* w);
      static void SolutionMenuItemSelected(Fl_Widget* w);

   // to selsct and display the weapon item from the Weapon menu
   static void WnpMenuItemSelected(Fl_Widget* w);

   // to selsct and display the missile item from the Missile menu
   static void MisMenuItemSelected(Fl_Widget* w);

   // to selsct and display the uuvitem from the uuv menu
   static void UuvMenuItemSelected(Fl_Widget* w);

   // to selsct and display the Sa item from the Sa menu
   static void SaMenuItemSelected(Fl_Widget* w);

   // to selsct and display the Ta item from the Ta menu
   static void TaMenuItemSelected(Fl_Widget* w);

   static void ChangeMapToColor (void);   // to change the bathymetry map to color
   static void ChangeMapToGrey (void);    // to change the bathymetry map to grey
   static void ChangeMapToBlue (void);    // to change the bathymetry map to blue
   static void ChangeMapToSand (void);    // to change the bathymetry map to blue
   static void ChangeMapToVeryShallow (void); // to change the bathmetry map to very shallow water
   static void ChangeMapToShallow (void); // to change the bathmetry map to shallow water
   static void ChangeMapToMidrange (void);
   static void ChangeMapToDeep (void);    // to change the bathmetry map to deep water

   static void Quit (void);               // to quit the program
   static void WindowKeyPressed (void);   // to listem for input from keyboard

   //This function is called to set camera to topdown overview
   static void SetCameraToTopDownOverview(void);

   //This function is called to set camera to wingman view
   static void SetCameraToWingManView(void);

   //This function is called to set camera to wingman view
   static void SetCameraToWingManViewO(void);

   //This function is called to set camera to wingman view
   static void SetCameraToWingManViewT(void);

   //This function is called to set camera to cross view
   static void SetCameraToCrossView(void);
   static void SetCameraToCrossViewO(void);
   static void SetCameraToCrossViewT(void);

   //This function is called to set camera to topdown perspective overview
   static void SetCameraToTopDownPerspectiveOverview(void);

   //This function is called to set camera to look down on ownship
   static void SetCameraToTopDownOwnshipview(void);

   //This function is called to set camera on ownship
   static void SetCameraToOwnship(void);

   //This function is called to set camera on current target
   static void SetCameraToCurrentTarget(void);

   //This function is called to set camera on current weapon
   static void SetCameraToCurrentWeapon(void);

   //This function is called to set camera to look out from current weapon
   static void SetCameraToCurrentWeaponFirstPerson(void);

   //This function is called to set camera to look out from ownship
   static void SetCameraToOwnshipFirstPerson(void);

   // feeder button callback function to start or stop the feeder.
   static void StartStopFeeder (Fl_Widget*);
   static void Surface(Fl_Widget*);

   /* popup window that prompts user for the desired target speed, and send it over to
      subve through share memory. */
   static void TargetSpeed(Fl_Widget* o);

   // to get the initial max speed, max/min depth of the contact
   static void Hypothesis(Fl_Widget* o);
   static void HypoContact(Fl_Widget*); // list of hypothecial contacts
   static void Solutions(Fl_Widget*);   // list of solutions for the contact
   static int Lon_Lat(Fl_Widget*);	 // to get the longitude and latitude of the starting pt
   bool CheckLonLat(const char*, double&, int); // to check the accurancy of the lon and lat
   void LonLatErrorBox();		// to display the correct format for lon and lat
   void IllegalRectBox();		// to check if the pairs of lon/lat make a legal rectangle
   void EmptyEntry();			// to display a error message for empty input strings.

  /*************************************/
  /* functions and subfunctions to create */
  /* right bezel			  */
  /*************************************/
   void ownship(int&);		// create the ownship display fields in the right bezel
   void contact(int&);		// create the contact display fields in the right bezel
   void limit(int);		// max and min range, bearning, and depth
   void time();			// output box for time
   void topDownMap();		// function to draw the topdown display map
   void sensor();		// function to display sensor information
   void taDisplay(int);		// function to display TA sensor information
   void saDisplay(int);		// function to display SA sensor information
   void trpDisplay(int);	// function to display TRP sensor information
   void uuvDisplay(int);	// function to display UUV sensor information
   void misDisplay(int);	// function to display MIS sensor information

/***********General Functions************************/
#define ABS(x) (((x)<0)?(-(x)):(x))  /* x = ABS(x) */
#define MIN(x,y) (((x)<(y))?(x):(y)) /* out = MIN(x,y) */
#define MAX(x,y) (((x)>(y))?(x):(y)) /* out = MAX(x,y) */
#define ROUND(x) ((int)((x)+0.5))    /* x = ROUND(x) */
#define FIX(x) ((int)((x)))          /* x = FIX(x) */
#define REM(x,y) (float)((x-y*(int)(x/y)))

#define FEET_TO_METERS(x) (x*0.3048)
#define METERS_TO_FEET(x) (x*3.2808)
#define FPS_TO_KTS(x) (x/1.687778)
#define PI M_PI


// fake speed values
#define FAKE_TARGET_SPEED 5.7
#define FAKE_WEAPON_SPEED 27.3
#define FAKE_OWNSHIP_SPEED 8.3
#define ZERO 0.0f
#define NINE 9999.9f

#define DEG2RAD( deg ) ( deg * ( M_PI / 180.0f ) )


/****************** C FUNCTIONS ************************/

bool isMultiSolutionContact(const char *name) {
  if (name) {
    if (strlen(name)>0) {
      if (name[0]!='T') {
	return true;
      }
    }
  }
  return false;
}


// Convert Decimal Lat/Lon to Conventional Notation:

void CON_LATLON(float degll, int dflag,char llstring[]) {
   char cdeg[3];
   char cmin[2];
   char csec[3];
   int degree,minute,second;
   float din,dminute;

  // Convert values from decimal degree to character strings:
   din = ABS(degll);
   degree = FIX(din);
   dminute = ((din - degree) * 60);
   minute = FIX(dminute);
   if (minute < 10)
     sprintf(cmin,"0%1d",minute);
   else
     sprintf(cmin,"%2d",minute);
   second = FIX((dminute - minute) * 60);
   if (second < 10)
     sprintf(csec,"0%1d",second);
   else
     sprintf(csec,"%2d",second);

   // Concatenate character strings for final display:
   if (dflag == 1)  {   // we have latitude
    if (degll > 0)        // we have North latitude
      sprintf(llstring,"%3d%2s%2sN",degree,cmin,csec);
    else
      sprintf(llstring,"%3d%2s%2sS",degree,cmin,csec);
   } else {        // we have longitude

    if (degll > 0)        // we have East longitude
      sprintf(llstring,"%3d%2s%2sE",degree,cmin,csec);
    else
      sprintf(llstring,"%3d%2s%2sW",degree,cmin,csec);
   }

   // Send the final string back to the calling routine:
  // return llstring;

}




// Convert Input time/date to sub DateTimeFormat:
void CON_DATETIME(float inDTF, char outstring[]) {
   float decyears=0,decdays=0,daysecs=0,dechours=0,decminutes=0,decseconds=0;
   int year=0,julday=0;
   int hour=0;
   int minute=0;
   int second=0;
   char HH[3];
   char MM[3];
   char SS[3];
   char YR[4];


  // Get number of years since 1970:
   decyears = inDTF/(3600*24*365.4);
   year = 1970 + FIX(decyears);
   sprintf(YR,"%4d",year);
   decdays = decyears - FIX(decyears);
   julday = FIX(decdays*365.4);
   daysecs = decdays*365.4 - julday;
   daysecs = daysecs*3600*24;
   hour = FIX(daysecs/3600);
   decseconds = (daysecs - hour*3600);
   minute = FIX(decseconds/60);
   second = FIX(decseconds - minute*60);


   // Load the character strings:
      if (hour < 10)
        sprintf(HH,"0%d",hour);
      else
        sprintf(HH,"%d",hour);

      if (minute < 10)
        sprintf(MM,"0%d",minute);
      else
        sprintf(MM,"%d",minute);

      if (second < 10)
        sprintf(SS,"0%d",second);
      else
        sprintf(SS,"%d",second);


 sprintf(outstring,"%c%c%c%c%c%c:%c%cZ %c%c%c%c%c",month[julday][0],month[julday][1],HH[0],HH[1],MM[0],MM[1],SS[0],SS[1],month[julday][2],month[julday][3],month[julday][4],YR[2],YR[3]);


   // Send the final string back to the calling routine:
  // return outstring;

}

// Convert Time Age into proper format:
void CON_AGE(float AgeSeconds, char subage[]) {

   int minute=0;
   int second=0;
   char MM[4];
   char SS[3];

  // Get Minutes and Second:
   minute = FIX(AgeSeconds/(60));
   second = FIX(AgeSeconds - minute*60);

   if (minute > 1000)
        return;


   // Load the character strings:
       if(minute < 10)
                sprintf(MM,"00%d",minute);

       if ((minute > 9) & (minute < 100))
                sprintf(MM,"0%d",minute);

       if(minute > 99)
                sprintf(MM,"%d",minute);

      if (second < 10)
        sprintf(SS,"0%d",second);
      else
        sprintf(SS,"%d",second);



      sprintf(subage,"%s:%s",MM,SS);

   // Send the final string back to the calling routine:
//   return subage;

}


// Calculate Range and Bearing:
float xship,yship,zship,hship;
float xtarget,ytarget,ztarget,htarget;
float range = 0;
float bearing = 0;
int conCount =0;

void range_bearing(float x1, float y1,float x2, float y2, float heading) {

    /* Declare Local Variables */
    float xrng,yrng,rnge,tbear,bear;

    /* Get range to target in local coordinate system */
    xrng = x2 - x1;
    yrng = y2 - y1;
    rnge = sqrt(xrng*xrng + yrng*yrng);
    rnge = rnge * 105.8 *6/10; //1058 meters per degree scaled for 42 degrees North Latitude

    /* Get the angle form the x-axis, theta, in local coordinate system.
       This is the true bearing from own vessel to target. */
    tbear = atan2(xrng,yrng) * 180/PI;

    /* Now subtract own vessel heading */
   // bear = tbear - heading;
    bear = tbear;
    if(bear < 0)
        bear = 360 + bear;

    /* return the values as pointers */
    range = rnge;
    bearing = bear;
}

/************************************************/

   // menu items for "color" menu on the top bezel
   Fl_Menu_Item menu_Map[] = {
   {"Water Depth  ", 0,  0, 0, 64, 0, 3, 14, 0},
   {"  400 FT", 0, (Fl_Callback*)ChangeMapToVeryShallow, 0, 0, 0, 3, 14, 0},
   {" 1000 FT", 0, (Fl_Callback*)ChangeMapToShallow, 0, 0, 0, 3, 14, 0},
   {" 8000 FT", 0, (Fl_Callback*)ChangeMapToMidrange, 0, 0, 0, 3, 14, 0},
   {"16000 FT", 0, (Fl_Callback*)ChangeMapToDeep, 0, 0, 0, 3, 14, 0},
   {0},
   {"Color", 0,(Fl_Callback*)ChangeMapToColor, 0, 0, 0, 3, 14, 0},
   {"Blue", 0, (Fl_Callback*)ChangeMapToBlue, 0, 0, 0, 3, 14, 0},
   {"Sand", 0, (Fl_Callback*)ChangeMapToSand, 0, 0, 0, 3, 14, 0},
   {0}
   };

   // menu items for "View" menu on the top bezel
   Fl_Menu_Item menu_Views[] = {
   {"Top-down Overview", 0,  (Fl_Callback*)SetCameraToTopDownOverview, 0, 0, 0, 3, 14, 0},
   {"Top-down Perspective Overview ", 0, (Fl_Callback*)SetCameraToTopDownPerspectiveOverview, 0, 0, 0, 3, 14, 0},
   {"Ownship Views  ", 0,  0, 0, 64, 0, 3, 14, 0},
   {"Cross, Ownship", 0, (Fl_Callback*)SetCameraToCrossViewO,  0, 0, 0, 3, 14, 0},
   {"First Person, Ownship",0, (Fl_Callback*)SetCameraToOwnshipFirstPerson, 0, 0, 0, 3, 14, 0},
   {"Perpective, centered on Ownship",0, (Fl_Callback*)SetCameraToOwnship, 0, 0, 0, 3, 14, 0},
   {"Top-down centered on Ownship ", 0, (Fl_Callback*)SetCameraToTopDownOwnshipview, 0, 0, 0, 3, 14, 0},
   {"WingMan, Ownship",0, (Fl_Callback*)SetCameraToWingManViewO, 0, 0, 0, 3, 14, 0},
   {0},
   {"Target Views  ", 0,  0, 0, 64, 0, 3, 14, 0},
   {"Cross, current Target", 0, (Fl_Callback*)SetCameraToCrossViewT,  0, 0, 0, 3, 14, 0},
   {"Perspective, centered on current Target", 0, (Fl_Callback*)SetCameraToCurrentTarget,  0, 0, 0, 3, 14, 0},
   {"WingMan, current Target",0, (Fl_Callback*)SetCameraToWingManViewT, 0, 0, 0, 3, 14, 0},
   {0},
   {"Weapon Views  ", 0,  0, 0, 64, 0, 3, 14, 0},
   {"Cross, current Weapon", 0, (Fl_Callback*)SetCameraToCrossView,  0, 0, 0, 3, 14, 0},
   {"FirstPerson, current Weapon", 0, (Fl_Callback*)SetCameraToCurrentWeaponFirstPerson,  0, 0, 0, 3, 14, 0},
   {"Perspective, centered on current Weapon", 0, (Fl_Callback*)SetCameraToCurrentWeapon,  0, 0, 0, 3, 14, 0},
   {"WingMan, current Weapon", 0, (Fl_Callback*)SetCameraToWingManView,  0, 0, 0, 3, 14, 0},
   {0},
   {0}
   };

   // menu items for "Simulation speed" menu on the top bezel
   Fl_Menu_Item menu_Sim[] = {
   {"1X", 0,  0, 0, 0, 0, 3, 14, 0},
   {"2X", 0,  0, 0, 0, 0, 3, 14, 0},
   {"4X", 0,  0, 0, 0, 0, 3, 14, 0},
   {"8X", 0,  0, 0, 0, 0, 3, 14, 0},
   {"16X", 0,  0, 0, 0, 0, 3, 14, 0},
   {0}
   };

   // menu items for "Manipulators" menu on the top bezel
   Fl_Menu_Item menu_Manipulators[] = {
   {"Mouse", 0,  0, 0, 0, 0, 3, 14, 0},
   {"Trackball", 0,  0, 0, 0, 0, 3, 14, 0},
   {"Flight Stick", 0,  0, 0, 0, 0, 3, 14, 0},
   {0}
   };

   // menu items for "Help" menu on the top bezel
   Fl_Menu_Item menu_Help[] = {
   {"Mouse", 0,  0, 0, 0, 0, 3, 14, 0},
   {"Flight Stick", 0,  0, 0, 0, 0, 3, 14, 0},
   {"Program Info.", 0,  0, 0, 0, 0, 3, 14, 0},
   {0}
   };



/**************CREATE TOP MENU SCREEN***********************/
   Fl_Window* create_top(void) {
      { Fl_Window* o = new Fl_Window(min(MAX_DISPLAY_WIDTH,Fl::w())-280, TOP_AREA_HEIGHT);
         o->position (-10,0);
         o->when(FL_WHEN_RELEASE);
         o->callback((Fl_Callback*)WindowKeyPressed);
         {
      //  Fl_Box *o = new Fl_Box(400,0,min(MAX_DISPLAY_WIDTH,Fl::w()),40,"TALOSS (3D Advanced Localization Observation Submarine System)");


        Fl_Box *o = new Fl_Box(300,0,min(MAX_DISPLAY_WIDTH,Fl::w()),40,"Immersive DIRECTION of RELATIVE MOTION Demonstration");
            o->box(FL_BORDER_FRAME);
            o->labelfont(FL_HELVETICA_BOLD);
            o->labelcolor(FL_BLUE);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }

	  /********* TALOSS Icon **************************/
  	static Fl_Pixmap talossPixmap(taloss);
         { Fl_Box* o = new Fl_Box(945,1, 52,38);
               o->box(FL_BORDER_FRAME);
               o->color(55);
               o->labelcolor(7);
               talossPixmap.label(o);
          }

  	/********* SVPL Icon **************************/
  	static Fl_Pixmap svplPixmap(svpl);
         { Fl_Box* o = new Fl_Box(3,3, 96,32);
               o->box(FL_BORDER_FRAME);
               o->color(55);
               o->labelcolor(7);
               svplPixmap.label(o);
          }


        { Fl_Button* o = new Fl_Button(0, TOP_AREA_HEIGHT-40, 140, 40,"Quit");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->callback((Fl_Callback*)Quit);
         //o->callback(Toggle, (long*)TOGGLE_QUIT);
         }
         { Fl_Menu_Button* o = new Fl_Menu_Button(140, TOP_AREA_HEIGHT-40,140, 40, "Map Options    ");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->menu(menu_Map);
         }
         { Fl_Menu_Button* o = new Fl_Menu_Button(280, TOP_AREA_HEIGHT-40, 140, 40, "Views  ");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->menu(menu_Views);
         }
         { Fl_Menu_Button* o = new Fl_Menu_Button(420, TOP_AREA_HEIGHT-40, 140, 40, "Sim. Speed  ");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->menu(menu_Sim);
         }
         { Fl_Menu_Button* o = new Fl_Menu_Button(560, TOP_AREA_HEIGHT-40, 140, 40, "Manipulators  ");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->menu(menu_Manipulators);
         }
         { Fl_Menu_Button* o = new Fl_Menu_Button(700, TOP_AREA_HEIGHT-40, 150, 40, "Stereo");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
         }
         { Fl_Menu_Button* o = new Fl_Menu_Button(850, TOP_AREA_HEIGHT-40, 150, 40, "Help");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->menu(menu_Help);
         }

         o->clear_border();
         o->show();
         o->end();
         return o;
      }
   }

   // loading icons to fltk
   static Fl_Pixmap upPixmap(up);
   static Fl_Pixmap downPixmap(Down);
   static Fl_Pixmap upDownPixmap(updown);
   static Fl_Pixmap taPixmap(ta);
   static Fl_Pixmap spherePixmap(sphere);
   static Fl_Pixmap wpnPixmap(torp);
   static Fl_Pixmap uuvPixmap(uuv);
   static Fl_Pixmap tomahawkPixmap(tomahawk);

/******************CREATE RIGHT Display SCREEN*****************************/
   Fl_Window *create_right() {
      {
      int pos = 78;
        Fl_Double_Window* o = uiRight.rightWin = new Fl_Double_Window(RIGHT_AREA_WIDTH, Fl::h());
         o->position (min(MAX_DISPLAY_WIDTH,Fl::w())-RIGHT_AREA_WIDTH,Fl::h()-TOP_AREA_HEIGHT-BOTTOM_AREA_HEIGHT);
         o->color(DEFAULT_BACKGROUND_COLOR);
         o->align(FL_ALIGN_CENTER);
         o->when(FL_WHEN_RELEASE);
         o->callback((Fl_Callback*)WindowKeyPressed);

	 { Fl_Box* o = new Fl_Box(0, 0, 280, 1024);
            o->box(FL_BORDER_FRAME);
            o->color(55);
            o->labeltype(FL_NO_LABEL);
         }

	 // Time keeper
	 time();

	 // create ownship
	 ownship(pos);

	 // create contacts
	 contact(pos);

	 // sensor information
	 sensor();

	 // topdown mini map
	topDownMap();

         { Fl_Slider* o = new Fl_Slider(50, 985, 200, 20, "ZOOM");
            o->type(5);
            o->box(FL_FLAT_BOX);
            //o->color(10);
            o->color(55);
            o->selection_color(1);
             o->labelfont(1);
            o->labelsize(10);
             o->labelcolor(7);
             o->callback((Fl_Callback*)scroll_cb);
         }

	 o->clear_border();
         o->show();
         o->end();
         return o;
      }
 }

/* Time - this function is to create a display box to display time */
void time(){
	{ Fl_Box* o = new Fl_Box(10, -15, 150, 100,"DTG");
		o->labelfont(1);
		o->labelcolor(7);
		o->labelsize(20);
		o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
	}

	uiRight.ownshipTime = new Fl_Output(70, 15, 200,40, "");
	uiRight.ownshipTime->box(FL_FLAT_BOX);
	uiRight.ownshipTime->color(DEFAULT_BACKGROUND_COLOR);
	uiRight.ownshipTime->textcolor(7);
	uiRight.ownshipTime->textfont(9);
	uiRight.ownshipTime->textsize(DEFAULT_TEXT_SIZE);
	uiRight.ownshipTime->align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
}

/* topDownMap - this function is to setup the space to display the
   top down map.  It also creates the axis with the labels */
void topDownMap(){
         uiRight.mapBox = new Fl_Box(55, 760, MAP_IMAGE_X, MAP_IMAGE_Y);
         uiRight.mapBox->box(FL_BORDER_FRAME);

          { Fl_Box* o = new Fl_Box(110, 740, 35, 15, "OVERHEAD VIEW");
            o->labelfont(1);
            o->labelcolor(7);
	    o->align(FL_ALIGN_CENTER);
         }

           uiRight.text1 = new Fl_Output(12,760,40,20,"   ");
           uiRight.text1->box(FL_FLAT_BOX);
           uiRight.text1->color(DEFAULT_BACKGROUND_COLOR);
           uiRight.text1->textfont(1);
           uiRight.text1->textcolor(7);
           uiRight.text1->value("20");			// initial label
           uiRight.text1->align(FL_ALIGN_BOTTOM);

         { Fl_Box* o = new Fl_Box(15, 860, 30, 15, "KYDS");
            o->labelfont(1);
            o->labelcolor(7);
         }

	   uiRight.text2 = new Fl_Output(10,960,45,20,"   ");
           uiRight.text2->box(FL_FLAT_BOX);
           uiRight.text2->color(DEFAULT_BACKGROUND_COLOR);
           uiRight.text2->textfont(1);
           uiRight.text2->textcolor(7);
           uiRight.text2->value("20");			// initial label
           uiRight.text2->align(FL_ALIGN_BOTTOM);

         { Fl_Box* o = new Fl_Box(140, 960, 35, 15, "KYDS");
            o->labelfont(1);
            o->labelcolor(7);
         }

           uiRight.text4 = new Fl_Output(230,960,40,20,"   ");
           uiRight.text4->box(FL_FLAT_BOX);
           uiRight.text4->color(DEFAULT_BACKGROUND_COLOR);
           uiRight.text4->textfont(1);
           uiRight.text4->textcolor(7);
           uiRight.text4->value("20");			// initial label
           uiRight.text4->align(FL_ALIGN_BOTTOM);
}

/* Ownship --  this function is to setup the display box to diplay the
   ownship information such as course, speed, drift, longitude/latitude
   etc...  */
void ownship(int &pos){
         { Fl_Box* o = new Fl_Box(5, pos, 270, 173, "OWNSHIP");
            o->color(55);
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_TOP_RIGHT|FL_ALIGN_INSIDE);
         }
	 pos+=27;
	{ Fl_Box* o = new Fl_Box(10, pos, 110, 20, "COURSE");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.ownshipCourse = new Fl_Output(110, pos, 100, 20, "deg");
         uiRight.ownshipCourse->box(FL_FLAT_BOX);
	 uiRight.ownshipCourse->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.ownshipCourse->textcolor(7);
	 uiRight.ownshipCourse->textfont(9);
         uiRight.ownshipCourse->textsize(DEFAULT_TEXT_SIZE);
         uiRight.ownshipCourse->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 { Fl_Box* o = new Fl_Box(220, pos, 50, 20, "DEG");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
	pos+=20;
	 { Fl_Box* o = new Fl_Box(10, pos, 110, 20, "SPEED");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.ownshipSpeed = new Fl_Output(110, pos, 100, 20, "kts");
         uiRight.ownshipSpeed->box(FL_FLAT_BOX);
         uiRight.ownshipSpeed->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.ownshipSpeed->textcolor(7);
	 uiRight.ownshipSpeed->textfont(9);
         uiRight.ownshipSpeed->textsize(DEFAULT_TEXT_SIZE);
         uiRight.ownshipSpeed->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	  { Fl_Box* o = new Fl_Box(220, pos, 50, 20, "KTS");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
	  }
	 pos+=20;
         { Fl_Box* o = new Fl_Box(10, pos, 110, 20, "DEPTH");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.ownshipDepth = new Fl_Output(110, pos, 100, 20, "ft");
         uiRight.ownshipDepth->box(FL_FLAT_BOX);
	 uiRight.ownshipDepth->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.ownshipDepth->textcolor(7);
	 uiRight.ownshipDepth->textfont(9);
         uiRight.ownshipDepth->textsize(DEFAULT_TEXT_SIZE);
	 uiRight.ownshipDepth->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 { Fl_Box* o = new Fl_Box(220, pos, 50, 20, "FT");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }

	 pos +=20;
         { Fl_Box* o = new Fl_Box(10, pos, 110, 20, "SET");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.ownshipSet = new Fl_Output(110, pos, 100, 20, "deg");
         uiRight.ownshipSet->box(FL_FLAT_BOX);
  	 uiRight.ownshipSet->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.ownshipSet->textcolor(7);
	 uiRight.ownshipSet->textfont(9);
         uiRight.ownshipSet->textsize(DEFAULT_TEXT_SIZE);
         uiRight.ownshipSet->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 { Fl_Box* o = new Fl_Box(220, pos, 50, 20, "DEG");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
	 pos +=20;
         { Fl_Box* o = new Fl_Box(10, pos, 110, 20, "DRIFT");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.ownshipDrift = new Fl_Output(110, pos, 100, 20, "kts");
         uiRight.ownshipDrift->box(FL_FLAT_BOX);
	 uiRight.ownshipDrift->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.ownshipDrift->textcolor(7);
	 uiRight.ownshipDrift->textfont(9);
         uiRight.ownshipDrift->textsize(DEFAULT_TEXT_SIZE);
         uiRight.ownshipDrift->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 { Fl_Box* o = new Fl_Box(220, pos, 50, 20, "KTS");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
	 pos += 20;
         { Fl_Box* o = new Fl_Box(10, pos, 110, 20, "LAT");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.ownshipLAT = new Fl_Output(110, pos, 100, 20, ownshipLATLabel);
         uiRight.ownshipLAT->box(FL_FLAT_BOX);
	 uiRight.ownshipLAT->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.ownshipLAT->textcolor(7);
	 uiRight.ownshipLAT->textfont(9);
         uiRight.ownshipLAT->textsize(DEFAULT_TEXT_SIZE);
         uiRight.ownshipLAT->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 pos+=20;
         { Fl_Box* o = new Fl_Box(10, pos, 110, 20, "LON");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.ownshipLON = new Fl_Output(110, pos, 100, 20, ownshipLONLabel);
         uiRight.ownshipLON->box(FL_FLAT_BOX);
	 uiRight.ownshipLON->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.ownshipLON->textcolor(7);
	 uiRight.ownshipLON->textfont(9);
         uiRight.ownshipLON->textsize(DEFAULT_TEXT_SIZE);
         uiRight.ownshipLON->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);
}

/* Contacts --  this function is to setup the display box to diplay the
   contact information such as Age, Max/min bearing, Max/min range,
   course, speed, Max/min depth, Longituide, latitude, etc... */
void contact(int& pos){
	 pos+=35;
         { Fl_Box* o = new Fl_Box(5, pos, 270, 255, "CONTACT");
            o->color(55);
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_TOP_RIGHT|FL_ALIGN_INSIDE);
         }
	 pos-=5;
         { Fl_Box* o = uiRight.dcontact_icon = new Fl_Box(10, pos, 37, 35);
            o->box(FL_BORDER_FRAME);
            o->color(55);
            o->labelcolor(7);
            downPixmap.label(o);
	    o->show();
         }
         { Fl_Box* o = uiRight.ucontact_icon = new Fl_Box(10, pos, 37, 35);
            o->box(FL_BORDER_FRAME);
            o->color(55);
            o->labelcolor(7);
            upPixmap.label(o);
            o->hide();
         }
	 { Fl_Box* o = uiRight.udcontact_icon = new Fl_Box(10, pos, 37, 35);
            o->box(FL_BORDER_FRAME);
            o->color(55);
            o->labelcolor(7);
            upDownPixmap.label(o);
            o->hide();
         }
         uiRight.targetMenu = new Fl_Menu_Button(55, pos+5, 130, 30, "Contacts");
         uiRight.targetMenu->box(FL_EMBOSSED_BOX);
         uiRight.targetMenu->color(42);
         uiRight.targetMenu->labelfont(1);
         uiRight.targetMenu->labelcolor(7);
         uiRight.targetMenu->labelsize(20);
	uiRight.targetMenu->callback(TargetMenuItemSelected);

	pos+=40;
	uiRight.solutionMenu = new Fl_Menu_Button(55, pos, 130, 30, "Solutions ");
         uiRight.solutionMenu->box(FL_EMBOSSED_BOX);
         uiRight.solutionMenu->color(42);
         uiRight.solutionMenu->labelfont(1);
         uiRight.solutionMenu->labelcolor(7);
         uiRight.solutionMenu->labelsize(20);
	uiRight.solutionMenu->callback(SolutionMenuItemSelected);



        { Fl_Button* o = new Fl_Button(191, pos, 85, 30, "");
            if(buttonsOff == 0)
            {
               o->type(1);
               o->label("Limit ON");
             }
             else
             {
               o->type();
               o->set();
             }
               o->labeltype(FL_EMBOSSED_LABEL);
               o->labelfont(1);
               o->callback((Fl_Callback*)SolutionOnOff);
              }

 

/*	

	{ Fl_Button* o = new Fl_Button(191, pos, 85, 30, "Limit OFF");
	    o->type(1);
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
	    o->callback((Fl_Callback*)SolutionOnOff);
         }
*/
	pos+=40;
	 { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "AGE");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.targetAge = new Fl_Output(110, pos, 100, OUTPUT_BOX, "min");
         uiRight.targetAge->box(FL_FLAT_BOX);
  	 uiRight.targetAge->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.targetAge->textcolor(7);
	 uiRight.targetAge->textfont(9);
         uiRight.targetAge->textsize(DEFAULT_TEXT_SIZE);
         uiRight.targetAge->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

  	{ Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "MIN");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }

	 pos+=OUTPUT_BOX;
         { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "BEARING");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.targetBearing = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
         uiRight.targetBearing->box(FL_FLAT_BOX);
	 uiRight.targetBearing->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.targetBearing->textcolor(7);
	 uiRight.targetBearing->textfont(9);
         uiRight.targetBearing->textsize(DEFAULT_TEXT_SIZE);
         uiRight.targetBearing->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
	 pos +=OUTPUT_BOX;
         { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "RANGE");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.targetRange = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kyds");
         uiRight.targetRange->box(FL_FLAT_BOX);
	 uiRight.targetRange->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.targetRange->textcolor(7);
	 uiRight.targetRange->textfont(9);
         uiRight.targetRange->textsize(DEFAULT_TEXT_SIZE);
         uiRight.targetRange->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KYDS");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }

	 pos+=OUTPUT_BOX;
         { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "COURSE");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.targetCourse = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
         uiRight.targetCourse->box(FL_FLAT_BOX);
	 uiRight.targetCourse->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.targetCourse->textcolor(7);
	 uiRight.targetCourse->textfont(9);
         uiRight.targetCourse->textsize(DEFAULT_TEXT_SIZE);
         uiRight.targetCourse->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
	 pos+=OUTPUT_BOX;
	 { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "SPEED");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.targetSpeed = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kts");
         uiRight.targetSpeed->box(FL_FLAT_BOX);
	 uiRight.targetSpeed->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.targetSpeed->textcolor(7);
	 uiRight.targetSpeed->textfont(9);
         uiRight.targetSpeed->textsize(DEFAULT_TEXT_SIZE);
         uiRight.targetSpeed->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KTS");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }

	 pos+=OUTPUT_BOX;
         { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "DEPTH");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.targetDepth = new Fl_Output(110, pos, 100, OUTPUT_BOX, "ft");
         uiRight.targetDepth->box(FL_FLAT_BOX);
	 uiRight.targetDepth->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.targetDepth->textcolor(7);
	 uiRight.targetDepth->textfont(9);
         uiRight.targetDepth->textsize(DEFAULT_TEXT_SIZE);
         uiRight.targetDepth->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "FT");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
 	 pos+=OUTPUT_BOX;
         { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "LAT");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }
         uiRight.targetLAT = new Fl_Output(110, pos, 100, OUTPUT_BOX, "N");
         uiRight.targetLAT->box(FL_FLAT_BOX);
	 uiRight.targetLAT->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.targetLAT->textcolor(7);
	 uiRight.targetLAT->textfont(9);
         uiRight.targetLAT->textsize(DEFAULT_TEXT_SIZE);
         uiRight.targetLAT->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 pos+=OUTPUT_BOX;
         { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "LON");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
         }

         uiRight.targetLON = new Fl_Output(110, pos, 100, OUTPUT_BOX, "W");
         uiRight.targetLON->box(FL_FLAT_BOX);
	 uiRight.targetLON->color(DEFAULT_BACKGROUND_COLOR);
	 uiRight.targetLON->textcolor(7);
	 uiRight.targetLON->textfont(9);
         uiRight.targetLON->textsize(DEFAULT_TEXT_SIZE);
         uiRight.targetLON->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	 // function to display max/min range, bearing, and depth
	 limit(495);
}

/* sensor --  this function is to setup the display box to diplay the
   weapon information.  It contains functionalities such as displaying
   multiple weapon information of one type of weapons as well as displaying
   information for different type of weapons such as UUV, MIS, TA, SA etc... */
void sensor(){
int pos = 515;
    { Fl_Group* o = uiRight.sensor_group = new Fl_Group(5, pos-10, 270, 235);
	int pos = 515;
	{ Fl_Box* o = new Fl_Box(5, pos, 270, 230, "SENSOR");
	    o->color(55);
	    o->labelfont(1);
	    o->labelcolor(7);
            o->align(FL_ALIGN_TOP_RIGHT|FL_ALIGN_INSIDE);
         }
         { Fl_Button* o = new Fl_Button(170, pos, 35, 30, "@+32>");
            o->color(42);
            o->labeltype(FL_SYMBOL_LABEL);
            o->labelfont(1);
            o->labelsize(20);
            o->callback((Fl_Callback*)Sensor);
         }

       { Fl_Box* o = uiRight.senno = new Fl_Box(70, pos, 65, 30, "TRP");
            o->box(FL_EMBOSSED_BOX);
            o->color(42);
         //o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->labelcolor(7);
            o->labelsize(20);
         }

	 taDisplay(pos);
	 saDisplay(pos);
	 uuvDisplay(pos);
	  trpDisplay(pos);
	  misDisplay(pos);


         if(buttonsOff == 0)
	   o->show();
         else
           o->hide();
           o->end();

	  
    }
}

/* taDisplay -- This function setup the display box to display the
	TA weapon information such as age, cb, crb, crr, crs, etc... */
void taDisplay(int pos){
         { Fl_Group* o = uiRight.ta_group = new Fl_Group(5, pos-10, 270, 235);

	    { Fl_Box* o = new Fl_Box(0, pos+10, 270,280, "TA");
	       o->color(55);
	       o->labelfont(1);
	       o->labelcolor(7);
               o->align(FL_ALIGN_TOP_RIGHT|FL_ALIGN_INSIDE);
            }
	    { Fl_Box* o = uiRight.ta_pixmap = new Fl_Box(10, pos-5, 37, 35);
               o->box(FL_BORDER_FRAME);
               o->color(55);
               o->labelcolor(7);
               taPixmap.label(o);
            }
	    uiRight.taMenu = new Fl_Menu_Button(60, pos, 100, 30, "TA");
            uiRight.taMenu->box(FL_EMBOSSED_BOX);
            uiRight.taMenu->color(42);
          //uiRight.taMenu->labeltype(FL_EMBOSSED_LABEL);
            uiRight.taMenu->labelfont(1);
            uiRight.taMenu->labelcolor(7);
            uiRight.taMenu->labelsize(20);
	    uiRight.taMenu->callback(TaMenuItemSelected);

	    pos +=35;
            { Fl_Box* o = new Fl_Box(10, pos, 210, OUTPUT_BOX, "AGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.taAge = new Fl_Output(110, pos, 100, OUTPUT_BOX, "min");
            uiRight.taAge->box(FL_FLAT_BOX);
	    uiRight.taAge->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.taAge->textcolor(7);
	    uiRight.taAge->textfont(9);
	    uiRight.taAge->textsize(DEFAULT_TEXT_SIZE);
            uiRight.taAge->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	   { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "MIN");
            o->labelfont(1);
            o->labelcolor(7);
            o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
           }

	   pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 210, OUTPUT_BOX, "CB");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.taCB = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
            uiRight.taCB->box(FL_FLAT_BOX);
	    uiRight.taCB->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.taCB->textcolor(7);
	    uiRight.taCB->textfont(9);
	    uiRight.taCB->textsize(DEFAULT_TEXT_SIZE);
            uiRight.taCB->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

            { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 210, OUTPUT_BOX, "CRB");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.taCRB = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
            uiRight.taCRB->box(FL_FLAT_BOX);
	    uiRight.taCRB->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.taCRB->textcolor(7);
	    uiRight.taCRB->textfont(9);
	    uiRight.taCRB->textsize(DEFAULT_TEXT_SIZE);
            uiRight.taCRB->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 210, OUTPUT_BOX, "CRR");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.taCRR = new Fl_Output(110, pos, 100, OUTPUT_BOX, "yds");
            uiRight.taCRR->box(FL_FLAT_BOX);
	    uiRight.taCRR->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.taCRR->textcolor(7);
	    uiRight.taCRR->textfont(9);
            uiRight.taCRR->textsize(DEFAULT_TEXT_SIZE);
            uiRight.taCRR->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "YDS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
	    { Fl_Box* o = new Fl_Box(10, pos, 210, OUTPUT_BOX, "CRS");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.taCRS = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
            uiRight.taCRS->box(FL_FLAT_BOX);
	    uiRight.taCRS->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.taCRS->textcolor(7);
	    uiRight.taCRS->textfont(9);
            uiRight.taCRS->textsize(DEFAULT_TEXT_SIZE);
            uiRight.taCRS->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 210, OUTPUT_BOX, "SPD");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.taSPD = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kts");
            uiRight.taSPD->box(FL_FLAT_BOX);
	    uiRight.taSPD->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.taSPD->textcolor(7);
	    uiRight.taSPD->textfont(9);
            uiRight.taSPD->textsize(DEFAULT_TEXT_SIZE);
            uiRight.taSPD->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KTS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 210, OUTPUT_BOX, "DP");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.taDP = new Fl_Output(110, pos, 100, OUTPUT_BOX, "ft");
            uiRight.taDP->box(FL_FLAT_BOX);
	    uiRight.taDP->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.taDP->textcolor(7);
	    uiRight.taDP->textfont(9);
            uiRight.taDP->textsize(DEFAULT_TEXT_SIZE);
            uiRight.taDP->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "FT");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
	    { Fl_Box* o = new Fl_Box(10, pos, 210, OUTPUT_BOX, "LAT");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.taLAT = new Fl_Output(110, pos, 100, OUTPUT_BOX, "N");
            uiRight.taLAT->box(FL_FLAT_BOX);
	    uiRight.taLAT->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.taLAT->textcolor(7);
	    uiRight.taLAT->textfont(9);
            uiRight.taLAT->textsize(DEFAULT_TEXT_SIZE);
            uiRight.taLAT->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 210, OUTPUT_BOX, "LON");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.taLON = new Fl_Output(110, pos, 100, OUTPUT_BOX, "W");
            uiRight.taLON->box(FL_FLAT_BOX);
	    uiRight.taLON->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.taLON->textcolor(7);
	    uiRight.taLON->textfont(9);
            uiRight.taLON->textsize(DEFAULT_TEXT_SIZE);
            uiRight.taLON->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    o->hide();
            o->end();
         }
}

/* SA group  -- This function setup the display box to display the
	SA weapon information such as age, sb, rng, -sr, +sr, etc... */
void saDisplay(int pos)
{
         { Fl_Group* o = uiRight.sa_group = new Fl_Group(5, pos-10, 270, 235);

	    { Fl_Box* o = new Fl_Box(0, pos+10, 270,280, "SA");
	       o->color(55);
	       o->labelfont(1);
	       o->labelcolor(7);
               o->align(FL_ALIGN_TOP_RIGHT|FL_ALIGN_INSIDE);
            }
            { Fl_Box* o = new Fl_Box(10, pos-5, 37, 35);
               o->box(FL_BORDER_FRAME);
               o->color(55);
               o->labelcolor(7);
               spherePixmap.label(o);
            }
	    uiRight.saMenu = new Fl_Menu_Button(60, pos, 100, 30, "SA");
            uiRight.saMenu->box(FL_EMBOSSED_BOX);
            uiRight.saMenu->color(42);
          //uiRight.saMenu->labeltype(FL_EMBOSSED_LABEL);
            uiRight.saMenu->labelfont(1);
            uiRight.saMenu->labelcolor(7);
            uiRight.saMenu->labelsize(20);
	    uiRight.saMenu->callback(SaMenuItemSelected);

	    pos+=35;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "AGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.saAge = new Fl_Output(110, pos, 100, OUTPUT_BOX, "min");
            uiRight.saAge->box(FL_FLAT_BOX);
	    uiRight.saAge->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.saAge->textcolor(7);
	    uiRight.saAge->textfont(9);
            uiRight.saAge->textsize(DEFAULT_TEXT_SIZE);
            uiRight.saAge->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "MIN");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
             { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "SB");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.saSB = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
            uiRight.saSB->box(FL_FLAT_BOX);
	    uiRight.saSB->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.saSB->textcolor(7);
	    uiRight.saSB->textfont(9);
            uiRight.saSB->textsize(DEFAULT_TEXT_SIZE);
            uiRight.saSB->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "RNG");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.saRNG = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kyds");
            uiRight.saRNG->box(FL_FLAT_BOX);
	    uiRight.saRNG->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.saRNG->textcolor(7);
	    uiRight.saRNG->textfont(9);
            uiRight.saRNG->textsize(DEFAULT_TEXT_SIZE);
            uiRight.saRNG->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

            { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KYDS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "-SR");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.saSR1 = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kyds");
            uiRight.saSR1->box(FL_FLAT_BOX);
	    uiRight.saSR1->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.saSR1->textcolor(7);
	    uiRight.saSR1->textfont(9);
            uiRight.saSR1->textsize(DEFAULT_TEXT_SIZE);
            uiRight.saSR1->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KYDS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "+SR");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
	    uiRight.saSR2 = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kyds");
            uiRight.saSR2->box(FL_FLAT_BOX);
	    uiRight.saSR2->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.saSR2->textcolor(7);
	    uiRight.saSR2->textfont(9);
            uiRight.saSR2->textsize(DEFAULT_TEXT_SIZE);
            uiRight.saSR2->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KYDS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    o->hide();
            o->end();
         }
}
/*UUV group  -- This function is used to setup the display box to display the
    UUV weapon information such as age, bearing, range, course, speed, depth,
    Longitude, latitude, etc... */
void uuvDisplay(int pos)
{
         { Fl_Group* o = uiRight.uuv_group = new Fl_Group(5, pos-10, 270, 235);

	    { Fl_Box* o = new Fl_Box(0, pos+10, 270,280, "UUV");
	       o->color(55);
	       o->labelfont(1);
	       o->labelcolor(7);
               o->align(FL_ALIGN_TOP_RIGHT|FL_ALIGN_INSIDE);
            }

           { Fl_Box* o = new Fl_Box(10, pos-5, 37, 35);
               o->box(FL_BORDER_FRAME);
               o->color(55);
               o->labelcolor(7);
               uuvPixmap.label(o);
            }

            uiRight.uuvMenu = new Fl_Menu_Button(60, pos, 100, 30, "UUV");
            uiRight.uuvMenu->box(FL_EMBOSSED_BOX);
            uiRight.uuvMenu->color(42);
          //uiRight.uuvMenu->labeltype(FL_EMBOSSED_LABEL);
            uiRight.uuvMenu->labelfont(1);
            uiRight.uuvMenu->labelcolor(7);
            uiRight.uuvMenu->labelsize(20);
	    uiRight.uuvMenu->callback(UuvMenuItemSelected);

	    pos+=35;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "AGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.uuvAge = new Fl_Output(110, pos, 100, OUTPUT_BOX, "min");
            uiRight.uuvAge->box(FL_FLAT_BOX);
	    uiRight.uuvAge->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.uuvAge->textcolor(7);
	    uiRight.uuvAge->textfont(9);
            uiRight.uuvAge->textsize(DEFAULT_TEXT_SIZE);
            uiRight.uuvAge->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "MIN");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "BEARING");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.uuvBearing = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
            uiRight.uuvBearing->box(FL_FLAT_BOX);
	    uiRight.uuvBearing->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.uuvBearing->textcolor(7);
	    uiRight.uuvBearing->textfont(9);
            uiRight.uuvBearing->textsize(DEFAULT_TEXT_SIZE);
            uiRight.uuvBearing->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "RANGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.uuvRange = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kyds");
            uiRight.uuvRange->box(FL_FLAT_BOX);
	    uiRight.uuvRange->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.uuvRange->textcolor(7);
	    uiRight.uuvRange->textfont(9);
            uiRight.uuvRange->textsize(DEFAULT_TEXT_SIZE);
            uiRight.uuvRange->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KYDS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "COURSE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.uuvCourse = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
            uiRight.uuvCourse->box(FL_FLAT_BOX);
	    uiRight.uuvCourse->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.uuvCourse->textcolor(7);
	    uiRight.uuvCourse->textfont(9);
            uiRight.uuvCourse->textsize(DEFAULT_TEXT_SIZE);
            uiRight.uuvCourse->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "SPEED");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.uuvSpeed = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kts");
            uiRight.uuvSpeed->box(FL_FLAT_BOX);
	    uiRight.uuvSpeed->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.uuvSpeed->textcolor(7);
	    uiRight.uuvSpeed->textfont(9);
            uiRight.uuvSpeed->textsize(DEFAULT_TEXT_SIZE);
            uiRight.uuvSpeed->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KTS");
             o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
	    { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "DEPTH");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.uuvDepth = new Fl_Output(110, pos, 100, OUTPUT_BOX, "ft");
            uiRight.uuvDepth->box(FL_FLAT_BOX);
	    uiRight.uuvDepth->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.uuvDepth->textcolor(7);
	    uiRight.uuvDepth->textfont(9);
            uiRight.uuvDepth->textsize(DEFAULT_TEXT_SIZE);
            uiRight.uuvDepth->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "FT");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "LAT");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.uuvLAT = new Fl_Output(110, pos, 100, OUTPUT_BOX, "N");
            uiRight.uuvLAT->box(FL_FLAT_BOX);
	    uiRight.uuvLAT->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.uuvLAT->textcolor(7);
	    uiRight.uuvLAT->textfont(9);
            uiRight.uuvLAT->textsize(DEFAULT_TEXT_SIZE);
            uiRight.uuvLAT->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "LON");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.uuvLON = new Fl_Output(110, pos, 100, OUTPUT_BOX, "W");
            uiRight.uuvLON->box(FL_FLAT_BOX);
	    uiRight.uuvLON->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.uuvLON->textcolor(7);
	    uiRight.uuvLON->textfont(9);
            uiRight.uuvLON->textsize(DEFAULT_TEXT_SIZE);
            uiRight.uuvLON->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "MODE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            uiRight.uuvMode = new Fl_Output(110, pos, 100, OUTPUT_BOX, "W");
            uiRight.uuvMode->box(FL_FLAT_BOX);
	    uiRight.uuvMode->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.uuvMode->textcolor(7);
	    uiRight.uuvMode->textfont(9);
            uiRight.uuvMode->textsize(DEFAULT_TEXT_SIZE);
            uiRight.uuvMode->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    o->hide();
            o->end();
         }
}


/* MIS group   -- This function is used to setup the display box to display the
    MIS weapon information such as age, bearing, range, course, speed, depth,
    Longitude, latitude, etc... */
void misDisplay(int pos){
         { Fl_Group* o = uiRight.mis_group = new Fl_Group(5, 505, 270, 235);

	    { Fl_Box* o = new Fl_Box(0, 525, 270,280, "MIS");
	       o->color(55);
	       o->labelfont(1);
	       o->labelcolor(7);
               o->align(FL_ALIGN_TOP_RIGHT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 510, 37, 35);
               o->box(FL_BORDER_FRAME);
               o->color(55);
               o->labelcolor(7);
               tomahawkPixmap.label(o);
            }

            uiRight.misMenu = new Fl_Menu_Button(60, 515, 100, 30, "MIS");
            uiRight.misMenu->box(FL_EMBOSSED_BOX);
            uiRight.misMenu->color(42);
          //uiRight.misMenu->labeltype(FL_EMBOSSED_LABEL);
            uiRight.misMenu->labelfont(1);
            uiRight.misMenu->labelcolor(7);
            uiRight.misMenu->labelsize(20);
	    uiRight.misMenu->callback(MisMenuItemSelected);

            { Fl_Box* o = new Fl_Box(10, 550, 110, OUTPUT_BOX, "AGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.misAge = new Fl_Output(110, 550, 100, OUTPUT_BOX, "min");
            uiRight.misAge->box(FL_FLAT_BOX);
	    uiRight.misAge->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.misAge->textcolor(7);
	    uiRight.misAge->textfont(9);
            uiRight.misAge->textsize(DEFAULT_TEXT_SIZE);
            uiRight.misAge->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 550, 50, OUTPUT_BOX, "MIN");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 570, 110, OUTPUT_BOX, "BEARING");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.misBearing = new Fl_Output(110, 570, 100, OUTPUT_BOX, "deg");
            uiRight.misBearing->box(FL_FLAT_BOX);
	    uiRight.misBearing->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.misBearing->textcolor(7);
	    uiRight.misBearing->textfont(9);
            uiRight.misBearing->textsize(DEFAULT_TEXT_SIZE);
            uiRight.misBearing->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 570, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 590, 110, OUTPUT_BOX, "RANGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.misRange = new Fl_Output(110, 590, 100, OUTPUT_BOX, "kyds");
            uiRight.misRange->box(FL_FLAT_BOX);
	    uiRight.misRange->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.misRange->textcolor(7);
	    uiRight.misRange->textfont(9);
            uiRight.misRange->textsize(DEFAULT_TEXT_SIZE);
            uiRight.misRange->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 590, 50, OUTPUT_BOX, "KYDS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 610, 110, OUTPUT_BOX, "COURSE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.misCourse = new Fl_Output(110, 610, 100, OUTPUT_BOX, "deg");
            uiRight.misCourse->box(FL_FLAT_BOX);
	    uiRight.misCourse->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.misCourse->textcolor(7);
	    uiRight.misCourse->textfont(9);
            uiRight.misCourse->textsize(DEFAULT_TEXT_SIZE);
            uiRight.misCourse->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 610, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 630, 110, OUTPUT_BOX, "SPEED");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.misSpeed = new Fl_Output(110, 630, 100, OUTPUT_BOX, "kts");
            uiRight.misSpeed->box(FL_FLAT_BOX);
	    uiRight.misSpeed->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.misSpeed->textcolor(7);
	    uiRight.misSpeed->textfont(9);
            uiRight.misSpeed->textsize(DEFAULT_TEXT_SIZE);
            uiRight.misSpeed->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 630, 50, OUTPUT_BOX, "KTS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 650, 110, OUTPUT_BOX, "DEPTH");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.misDepth = new Fl_Output(110, 650, 100, OUTPUT_BOX, "ft");
            uiRight.misDepth->box(FL_FLAT_BOX);
	    uiRight.misDepth->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.misDepth->textcolor(7);
	    uiRight.misDepth->textfont(9);
            uiRight.misDepth->textsize(DEFAULT_TEXT_SIZE);
            uiRight.misDepth->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 650, 50, OUTPUT_BOX, "FT");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 670, 110, OUTPUT_BOX, "LAT");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.misLAT = new Fl_Output(110, 670, 100, OUTPUT_BOX, "N");
            uiRight.misLAT->box(FL_FLAT_BOX);
	    uiRight.misLAT->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.misLAT->textcolor(7);
	    uiRight.misLAT->textfont(9);
            uiRight.misLAT->textsize(DEFAULT_TEXT_SIZE);
            uiRight.misLAT->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

            { Fl_Box* o = new Fl_Box(10, 690, 110, OUTPUT_BOX, "LON");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.misLON = new Fl_Output(110, 690, 100, OUTPUT_BOX, "W");
            uiRight.misLON->box(FL_FLAT_BOX);
	    uiRight.misLON->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.misLON->textcolor(7);
	    uiRight.misLON->textfont(9);
            uiRight.misLON->textsize(DEFAULT_TEXT_SIZE);
            uiRight.misLON->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

            { Fl_Box* o = new Fl_Box(10, 710, 110, OUTPUT_BOX, "MODE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            uiRight.misMode = new Fl_Output(110, 710, 100, OUTPUT_BOX, "W");
            uiRight.misMode->box(FL_FLAT_BOX);
	    uiRight.misMode->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.misMode->textcolor(7);
	    uiRight.misMode->textfont(9);
            uiRight.misMode->textsize(DEFAULT_TEXT_SIZE);
            uiRight.misMode->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);
            o->hide();
            o->end();
         }
}

 /*TRP group   -- This function is used to setup the display box to display the
    TRP weapon information such as age, bearing, range, course, speed, depth,
    Longitude, latitude, etc... */
void trpDisplay(int pos){
         { Fl_Group* o = uiRight.trp_group = new Fl_Group(5, 505, 270, 235);

        { Fl_Box* o = new Fl_Box(10, 510, 37, 35);
               o->box(FL_BORDER_FRAME);
               o->color(55);
               o->labelcolor(7);
               wpnPixmap.label(o);
            }

           uiRight.wnpMenu = new Fl_Menu_Button(60, 515, 100, 30, "TRP");
           uiRight.wnpMenu->box(FL_EMBOSSED_BOX);
           uiRight.wnpMenu->color(42);
           uiRight.wnpMenu->labelfont(1);
           uiRight.wnpMenu->labelcolor(7);
           uiRight.wnpMenu->labelsize(20);
	   uiRight.wnpMenu->callback(WnpMenuItemSelected);

            { Fl_Box* o = new Fl_Box(10, 550, 110, OUTPUT_BOX, "AGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.weaponAge = new Fl_Output(110, 550, 100, OUTPUT_BOX, "min");
            uiRight.weaponAge->box(FL_FLAT_BOX);
	    uiRight.weaponAge->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.weaponAge->textcolor(7);
	    uiRight.weaponAge->textfont(9);
            uiRight.weaponAge->textsize(DEFAULT_TEXT_SIZE);
            uiRight.weaponAge->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 550, 50, OUTPUT_BOX, "MIN");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 570, 110, OUTPUT_BOX, "BEARING");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.weaponBearing = new Fl_Output(110, 570, 100, OUTPUT_BOX, "deg");
            uiRight.weaponBearing->box(FL_FLAT_BOX);
	    uiRight.weaponBearing->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.weaponBearing->textcolor(7);
	    uiRight.weaponBearing->textfont(9);
            uiRight.weaponBearing->textsize(DEFAULT_TEXT_SIZE);
            uiRight.weaponBearing->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 570, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 590, 110, OUTPUT_BOX, "RANGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.weaponRange = new Fl_Output(110, 590, 100, OUTPUT_BOX, "kyds");
            uiRight.weaponRange->box(FL_FLAT_BOX);
	    uiRight.weaponRange->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.weaponRange->textcolor(7);
	    uiRight.weaponRange->textfont(9);
            uiRight.weaponRange->textsize(DEFAULT_TEXT_SIZE);
            uiRight.weaponRange->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 590, 50, OUTPUT_BOX, "KYDS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 610, 110, OUTPUT_BOX, "COURSE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.weaponCourse = new Fl_Output(110, 610, 100, OUTPUT_BOX, "deg");
            uiRight.weaponCourse->box(FL_FLAT_BOX);
	    uiRight.weaponCourse->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.weaponCourse->textcolor(7);
	    uiRight.weaponCourse->textfont(9);
            uiRight.weaponCourse->textsize(DEFAULT_TEXT_SIZE);
            uiRight.weaponCourse->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 610, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 630, 110, OUTPUT_BOX, "SPEED");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.weaponSpeed = new Fl_Output(110, 630, 100, OUTPUT_BOX, "kts");
            uiRight.weaponSpeed->box(FL_FLAT_BOX);
	    uiRight.weaponSpeed->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.weaponSpeed->textcolor(7);
	    uiRight.weaponSpeed->textfont(9);
            uiRight.weaponSpeed->textsize(DEFAULT_TEXT_SIZE);
            uiRight.weaponSpeed->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 630, 50, OUTPUT_BOX, "KTS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 650, 110, OUTPUT_BOX, "DEPTH");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.weaponDepth = new Fl_Output(110, 650, 100, OUTPUT_BOX, "ft");
            uiRight.weaponDepth->box(FL_FLAT_BOX);
	    uiRight.weaponDepth->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.weaponDepth->textcolor(7);
	    uiRight.weaponDepth->textfont(9);
            uiRight.weaponDepth->textsize(DEFAULT_TEXT_SIZE);
            uiRight.weaponDepth->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, 650, 50, OUTPUT_BOX, "FT");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            { Fl_Box* o = new Fl_Box(10, 670, 110, OUTPUT_BOX, "LAT");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.weaponLAT = new Fl_Output(110, 670, 100, OUTPUT_BOX, "N");
            uiRight.weaponLAT->box(FL_FLAT_BOX);
	    uiRight.weaponLAT->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.weaponLAT->textcolor(7);
	    uiRight.weaponLAT->textfont(9);
            uiRight.weaponLAT->textsize(DEFAULT_TEXT_SIZE);
            uiRight.weaponLAT->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

            { Fl_Box* o = new Fl_Box(10, 690, 110, OUTPUT_BOX, "LON");
	       o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.weaponLON = new Fl_Output(110, 690, 100, OUTPUT_BOX, "W");
            uiRight.weaponLON->box(FL_FLAT_BOX);
	    uiRight.weaponLON->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.weaponLON->textcolor(7);
	    uiRight.weaponLON->textfont(9);
            uiRight.weaponLON->textsize(DEFAULT_TEXT_SIZE);
            uiRight.weaponLON->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

            { Fl_Box* o = new Fl_Box(10, 710, 110, OUTPUT_BOX, "MODE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

          //  uiRight.weaponMode = new Fl_Output(110, 710, 100, OUTPUT_BOX, "W");
            uiRight.weaponMode = new Fl_Output(65, 710, 230, OUTPUT_BOX, "W");
            uiRight.weaponMode->box(FL_FLAT_BOX);
	    uiRight.weaponMode->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.weaponMode->textcolor(7);
	    uiRight.weaponMode->textfont(9);
            uiRight.weaponMode->textsize(DEFAULT_TEXT_SIZE);
            uiRight.weaponMode->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

            o->show();
            o->end();
         }
}

/* limit -- This function is used to setup the display box for diaplaying the
   max/min depth, bearing, and range information */
void limit(int pos){
         { Fl_Group* o = uiRight.limit_group = new Fl_Group(5, 505, 270, 235);
           { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "MAX RANGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.targetMaxRange = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kyds");
            uiRight.targetMaxRange->box(FL_FLAT_BOX);
	    uiRight.targetMaxRange->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.targetMaxRange->textcolor(7);
	    uiRight.targetMaxRange->textfont(9);
            uiRight.targetMaxRange->textsize(DEFAULT_TEXT_SIZE);
            uiRight.targetMaxRange->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KYDS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "MIN RANGE");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.targetMinRange = new Fl_Output(110, pos, 100, OUTPUT_BOX, "kyds");
            uiRight.targetMinRange->box(FL_FLAT_BOX);
	    uiRight.targetMinRange->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.targetMinRange->textcolor(7);
	    uiRight.targetMinRange->textfont(9);
            uiRight.targetMinRange->textsize(DEFAULT_TEXT_SIZE);
            uiRight.targetMinRange->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "KYDS");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "MAX BRNG");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.targetMaxBearing = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
            uiRight.targetMaxBearing->box(FL_FLAT_BOX);
	    uiRight.targetMaxBearing->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.targetMaxBearing->textcolor(7);
	    uiRight.targetMaxBearing->textfont(9);
            uiRight.targetMaxBearing->textsize(DEFAULT_TEXT_SIZE);
            uiRight.targetMaxBearing->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "MIN BRNG");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.targetMinBearing = new Fl_Output(110, pos, 100, OUTPUT_BOX, "deg");
            uiRight.targetMinBearing->box(FL_FLAT_BOX);
	    uiRight.targetMinBearing->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.targetMinBearing->textcolor(7);
	    uiRight.targetMinBearing->textfont(9);
            uiRight.targetMinBearing->textsize(DEFAULT_TEXT_SIZE);
            uiRight.targetMinBearing->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "DEG");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "MAX DEPTH");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.targetMaxDepth = new Fl_Output(110, pos, 100, OUTPUT_BOX, "ft");
            uiRight.targetMaxDepth->box(FL_FLAT_BOX);
	    uiRight.targetMaxDepth->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.targetMaxDepth->textcolor(7);
	    uiRight.targetMaxDepth->textfont(9);
            uiRight.targetMaxDepth->textsize(DEFAULT_TEXT_SIZE);
            uiRight.targetMaxDepth->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "FT");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

	    pos+=OUTPUT_BOX;
            { Fl_Box* o = new Fl_Box(10, pos, 110, OUTPUT_BOX, "MIN DEPTH");
               o->labelfont(1);
               o->labelcolor(7);
               o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }
            uiRight.targetMinDepth = new Fl_Output(110, pos, 100, OUTPUT_BOX, "ft");
            uiRight.targetMinDepth->box(FL_FLAT_BOX);
	    uiRight.targetMinDepth->color(DEFAULT_BACKGROUND_COLOR);
	    uiRight.targetMinDepth->textcolor(7);
	    uiRight.targetMinDepth->textfont(9);
            uiRight.targetMinDepth->textsize(DEFAULT_TEXT_SIZE);
            uiRight.targetMinDepth->align(FL_ALIGN_RIGHT|FL_ALIGN_INSIDE);

	    { Fl_Box* o = new Fl_Box(220, pos, 50, OUTPUT_BOX, "FT");
             o->labelfont(1);
             o->labelcolor(7);
             o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
            }

            o->hide();
            o->end();
         }
}

/* create_bottom -- this function is to create the botton button menu as part
     of the bezel.  It allows user to stop, pulse, or start the feeder; to
     allow users to step forward or backward in the time line to view the information;
     to change the griding on the map to navigation grid or range rings;
     It also is used to get information from user and send information
     to subve through the share memory. */
Fl_Window* create_bottom(void){
      { Fl_Window* o = new Fl_Window(min(MAX_DISPLAY_WIDTH,Fl::w())-280, BOTTOM_AREA_HEIGHT);
         o->position(0,(Fl::h()-BOTTOM_AREA_HEIGHT));
         o->clear_border();

         o->when(FL_WHEN_RELEASE);
         o->callback((Fl_Callback*)WindowKeyPressed);

         { Fl_Button* o = new Fl_Button(0, 0, 140, 30, "Follow OFF");
	    o->type(1);
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->labelcolor(34);
	    o->callback((Fl_Callback*)Follow);
         }
	 { Fl_Button* o = new Fl_Button(140, 0, 150, 30, "Hide Intersections");
	    o->type(1);
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
	    o->callback((Fl_Callback*)Toggle_target);
         }
	 /*
         { Fl_Button* o = new Fl_Button(290, 0, 140, 30, "Target Speed");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
	    o->callback((Fl_Callback*)TargetSpeed);
         }
	 */
         { Fl_Button* o = new Fl_Button(290, 0, 140, 30, "Reduce Region(s)");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
	    o->callback((Fl_Callback*)ReduceRegion);
         }
         { Fl_Button* o = new Fl_Button(430, 0, 140, 30, "Grow Region(s)");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
	    o->callback((Fl_Callback*)GrowRegion);
         }
         { Fl_Button* o = new Fl_Button(570, 0, 140, 30, "Hypo Contact");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
	    o->callback((Fl_Callback*)HypoContact);
         }
         { Fl_Button* o = new Fl_Button(710, 0, 140, 30, "Solutions");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
         o->callback((Fl_Callback*)Solutions);
         }
         { Fl_Button* o = new Fl_Button(850, 0, 150, 30, "Range Rings");
            o->type(1);
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->callback((Fl_Callback*)NavGrid);
         }
         { Fl_Button* o = new Fl_Button(0, 30, 140, 30, "LON/LAT");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
	     o->callback((Fl_Callback*)Lon_Lat);
         }
         { Fl_Button* o = new Fl_Button(140, 30, 150, 30, "FEEDER:ON");
            o->type(1);
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->set();
            o->callback((Fl_Callback*)StartStopFeeder);
         }
         { Fl_Button* o = new Fl_Button(290, 30, 140, 30, "SURFACE OFF");
            o->type(1);
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->callback((Fl_Callback*)Surface);
         }
         { Fl_Button* o = new Fl_Button(430, 30, 140, 30, "RUNNING");
            o->type(1);
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->set();
            o->callback((Fl_Callback*)Pause);
         }
         { Fl_Button* o = new Fl_Button(570, 30, 140, 30, "BACKWARDS");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->callback((Fl_Callback*)Backwards);
         }
         { Fl_Button* o = new Fl_Button(710, 30, 140, 30, "FORWARD");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
            o->callback((Fl_Callback*)Forward);
         }
         { Fl_Button* o = new Fl_Button(850, 30, 150, 30, "SNAPSHOT");
            o->labeltype(FL_EMBOSSED_LABEL);
            o->labelfont(1);
	    o->callback((Fl_Callback*)Snapshot);
         }
         o->show();
         o->end();
         return o;
      }
   }

/* create_all -- this function adds the top, right, and bottom display
   bezel into one group. */
   Fl_Group* create_all () {
      Fl_Group* w;
      w = new Fl_Group(0,0,800,600);
      w->add(*create_top());
      w->add(create_right());
      w->add(create_bottom());
      w->end();
      return w;
   }
/* Follow -- this is a callback function to toggle whether the camera follows an object*/
   void Follow (Fl_Widget* button) {
      if(((Fl_Button*)button)->value() == 1) {
         ((Fl_Button*)button)->label("Follow ON");
         talkWithMainApp->askMainApp(MainAppCommand::FOLLOW_ON);
      }
      else{
         ((Fl_Button*)button)->label("Follow OFF");
         talkWithMainApp->askMainApp(MainAppCommand::FOLLOW_OFF);
      }
      ((Fl_Button*)button)->redraw();
   }

/* Pause -- this is a callback function to pause or restart the subve*/
   void Pause (Fl_Widget* button) {
      if(((Fl_Button*)button)->value() == 1) {
         ((Fl_Button*)button)->label("RUNNING");
         talkWithMainApp->askMainApp(MainAppCommand::RESUME_MOVEMENT);
      }
      else{
         ((Fl_Button*)button)->label("PAUSED");
         talkWithMainApp->askMainApp(MainAppCommand::STOP_MOVEMENT);
      }
      ((Fl_Button*)button)->redraw();

   }

    /* StartStopFeeder -- this is a callback function to start or
       to stop the feeder*/
   void StartStopFeeder (Fl_Widget* button) {
      if(((Fl_Button*)button)->value() == 1) {
         ((Fl_Button*)button)->label("FEEDER:ON");
         talkWithMainApp->askMainApp(MainAppCommand::RESUME_FEEDER);
      }
      else{
         ((Fl_Button*)button)->label("FEEDER:OFF");
         talkWithMainApp->askMainApp(MainAppCommand::STOP_FEEDER);
      }
      ((Fl_Button*)button)->redraw();

   }

   void Surface (Fl_Widget* button) {
      if(((Fl_Button*)button)->value() == 1) {
         ((Fl_Button*)button)->label("SURFACE ON");
         talkWithMainApp->askMainApp(MainAppCommand::SURFACE_ON);
      }
      else{
         ((Fl_Button*)button)->label("SURFACE OFF");
         talkWithMainApp->askMainApp(MainAppCommand::SURFACE_OFF);
      }
      ((Fl_Button*)button)->redraw();

   }

   /* Backwards -- this is a callback function to step backward in the
      timeline*/
   void Backwards (Fl_Widget* o) {
      talkWithMainApp->askMainApp(MainAppCommand::STEP_BACK_IN_TIME);
   }

   /* Forward -- this is a callback function to step backward in the
      timeline*/
   void Forward (Fl_Widget* o) {
      talkWithMainApp->askMainApp(MainAppCommand::STEP_FORWARD_IN_TIME);
   }

   /* Snapshot -- this is a callback function to step backward in the
      timeline */
   void Snapshot (Fl_Widget* o) {
     talkWithMainApp->askMainApp(MainAppCommand::SNAPSHOT);
   }

   /* Toggle_target -- this is a callback function to hide or show the
   	intersection between two areas. */
   void Toggle_target (Fl_Widget* button) {
      if(((Fl_Button*)button)->value() == 1) {
         ((Fl_Button*)button)->label("Show Intersecting");
	 talkWithMainApp->askMainApp(MainAppCommand::HIDE_INTERSECTING);

      }
      else{
         ((Fl_Button*)button)->label("Hide Intersecting");
	 talkWithMainApp->askMainApp(MainAppCommand::SHOW_INTERSECTING);
      }
      ((Fl_Button*)button)->redraw();
   }


  /* NavGrid -- this is a callback function that toggles between
   	navigation grid and range rings. */
   void NavGrid (Fl_Widget* button) {
      if(((Fl_Button*)button)->value() == 1) {
         ((Fl_Button*)button)->label("Nav Grid");
	 talkWithMainApp->askMainApp(MainAppCommand::NAV_GRID);
      }
      else{
      ((Fl_Button*)button)->label("Range Rings");
	 talkWithMainApp->askMainApp(MainAppCommand::RANGE_RINGS);
      }
      ((Fl_Button*)button)->redraw();
   }

   /* SolutionOnOff -- this is a callback function that toggles
   	between turning the contact information limits on and off*/
   void SolutionOnOff (Fl_Widget* button) {
      if(((Fl_Button*)button)->value() == 1) {
         ((Fl_Button*)button)->label("Limit ON");
	uiRight.sensor_group->hide();
	uiRight.limit_group->show();
	 // bring up the box;
      }
      else{
      ((Fl_Button*)button)->label("Limit OFF");
      uiRight.limit_group->hide();
      uiRight.sensor_group->show();

      }
      ((Fl_Button*)button)->redraw();
   }

   /* scroll_cb -- this is a callback function to zoom into or out of the
   	top down display map acording to the scrool bar value.*/
   void scroll_cb(Fl_Widget* o) {
        char temp[6];
        float tval;
        talkWithMainApp->scroll_val.scroll_val= ((Fl_Valuator*)o)->value();
        talkWithMainApp->askMainApp(MainAppCommand::SCROLL_VAL);
        //printf("***** %g     \n", ((Fl_Valuator*)o)->value());
        tval = ((Fl_Valuator*)o)->value();	// 0..1 from scrollbar
        tval = ABS(tval - 1);			// 1..0
        tval = tval*18.0 + 2;			// 20..1
	// steve: make scale +/- 20 to 2  (3.5 -> 18.0)
        sprintf(temp,"%5d", (int)tval);
        uiRight.text1->value(temp);
        uiRight.text4->value(temp);
        sprintf(temp,"%5d",(int)(-tval));
        uiRight.text2->value(temp);
     //   uiRight.text3->value(temp);
        fflush(stdout);
   }

   /* Sensor -- this is a button callback function that scroll through the list of
   	weapon (SA, TA, TRP, UUV, etc...)information to be displayed. */
   void Sensor (void)
   {
      static int sensorno = 3;

    /* Hide all groups in the Sensor Panel Area */

      uiRight.ta_group->hide();
      uiRight.sa_group->hide();
      uiRight.trp_group->hide();
      uiRight.mis_group->hide();
      uiRight.uuv_group->hide();

      switch (sensorno) {
         case 0:
            uiRight.senno->label("TA");
            uiRight.ta_group->show();
            sensorno = 1;
            break;
         case 1:
            uiRight.senno->label("SA");
            uiRight.sa_group->show();
            sensorno = 2;
            break;
         case 2:
	    uiRight.senno->label("TRP");
            uiRight.trp_group->show();
            sensorno = 3;
            break;

	case 3:
            uiRight.senno->label("MIS");
            uiRight.mis_group->show();
            sensorno = 4;
            break;
	case 4:
            uiRight.senno->label("UUV");
            uiRight.uuv_group->show();
            sensorno = 0;
            break;
      }
   }

   /* ChangeMapToColor -- this is a callback function to change the
   	map color map.*/
   void ChangeMapToColor(void) {
      talkWithMainApp->askMainApp(MainAppCommand::SET_COLOR_MAP);
   }

   /* ChangeMapToGrey -- this is a callback function to change the
   	map grey map.*/
   void ChangeMapToGrey(void) {
        talkWithMainApp->askMainApp(MainAppCommand::SET_GREY_MAP);
   }

   /* ChangeMapToBlue -- this is a callback function to change the
   	map blue map.*/
   void ChangeMapToBlue(void) {
      talkWithMainApp->askMainApp(MainAppCommand::SET_BLUE_MAP);
   }


   /* ChangeMapToSand -- this is a callback function to change the
        map sand map.*/
   void ChangeMapToSand(void) {
      talkWithMainApp->askMainApp(MainAppCommand::SET_SAND_MAP);
   }



   /* ChangeMapToVeryShallow -- this is a callback function to change the
   	map to very shallow water map.*/
   void ChangeMapToVeryShallow(void) {
      talkWithMainApp->askMainApp(MainAppCommand::VERYSHALLOW);
   }

   /* ChangeMapToShallow- this is a callback function to change the
   	map to shallow water map.*/
   void ChangeMapToShallow(void) {
      talkWithMainApp->askMainApp(MainAppCommand::SHALLOW);
   }

   /* ChangeMapToMidrange -- this is a callback function to change the
   	map to midrange water map.*/
   void ChangeMapToMidrange(void) {
      talkWithMainApp->askMainApp(MainAppCommand::MIDRANGE);
   }

   /* ChangeMapToDeep -- this is a callback function to change the
   	map deep water map.*/
   void ChangeMapToDeep(void) {
      talkWithMainApp->askMainApp(MainAppCommand::DEEP);
   }

   /* Quit -- this is a callback function that sends information back
   	to subve to quit out of everything*/
   void Quit(void) {
      running = false;
   }

   /* WindowKeyPressed -- This is a callback function that listens to
      the keypresses and if escape key is press it call Quit to quit the program.*/
   void WindowKeyPressed(void) {
      if(Fl::event_key() == FL_Escape) {
         Quit();
      }
   }

   /* SetCameraToTopDownOverview -- this is a callback function to set
   	the camera to top of the scene.*/
   static void SetCameraToTopDownOverview(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::TOP_DOWN_CAMERA,NULL);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

   /* SetCameraToTopDownPerspectiveOverview -- this is a callback function to
   	set the camera to topdown perspective angle*/
   static void SetCameraToTopDownPerspectiveOverview(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::TOP_DOWN_PERSPECTIVE_CAMERA,NULL);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }
   /* SetCameraToTopDownOwnshipOverview -- this is a callback function to
   	to center on ownship from above*/
   static void SetCameraToTopDownOwnshipview(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::TOP_DOWN_CAMERA_CENTERED_ON,ownshipID);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

   /* SetCameraToOwnship -- this is a callback function that sets the camera
   	to center at ownship*/
   static void SetCameraToOwnship(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::CAMERA_CENTERED_ON,ownshipID);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

   /* SetCameraToOwnshipFirstPerson -- this is a callback function that sets the camera
   	looking out from ownship*/
   static void SetCameraToOwnshipFirstPerson(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::CAMERA_FIRST_PERSON_FROM,ownshipID);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

   /* SetCameraToCurrentTarget -- this is a callback function that sets the camera
   	to center at current target*/
   static void SetCameraToCurrentTarget(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::CAMERA_CENTERED_ON,talkWithMainApp->selectedTarget);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

   /* SetCameraToCurrentWeapon-- this is a callback function that sets the camera
   	to center at current weapon*/
   static void SetCameraToCurrentWeapon(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::CAMERA_CENTERED_ON,talkWithMainApp->selectedWeapon);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

   /* SetCameraToCurrentWeaponFirstPerson -- this is a callback function that sets the camera
   	looking out from current weapon*/
   static void SetCameraToCurrentWeaponFirstPerson(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::CAMERA_FIRST_PERSON_FROM,talkWithMainApp->selectedWeapon);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

   /* SetCameraToWingManView -- this is a callback function to set
   	the camera to top of the scene.*/
   static void SetCameraToWingManView(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::WINGMAN,talkWithMainApp->selectedWeapon);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

   /* SetCameraToWingManView -- this is a callback function to set
   	the camera to top of the scene.*/
   static void SetCameraToWingManViewO(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::WINGMAN,ownshipID);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }
      /* SetCameraToWingManView -- this is a callback function to set
   	the camera to top of the scene.*/
   static void SetCameraToWingManViewT(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::WINGMAN,talkWithMainApp->selectedTarget);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }
   /* SetCameraToCrossView -- this is a callback function to set
   	the camera to top of the scene.*/
   static void SetCameraToCrossView(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::CROSS,talkWithMainApp->selectedWeapon);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

     /* SetCameraToCrossView -- this is a callback function to set
   	the camera to top of the scene.*/
   static void SetCameraToCrossViewO(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::CROSS,ownshipID);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }
     /* SetCameraToCrossView -- this is a callback function to set
   	the camera to top of the scene.*/
   static void SetCameraToCrossViewT(void) {
      talkWithMainApp->setArgumentForCameraCommand(MainAppCommand::CROSS,talkWithMainApp->selectedTarget);
      talkWithMainApp->askMainApp(MainAppCommand::SET_CAMERA);
   }

    /* TargetSpeed -- this is a button callback function that display
    	a popup window to get the target speed from the user and send
	the speed information across the share memory to subve*/
/*
    void TargetSpeed(Fl_Widget* o){
	Fl_Window window(320,160, "Target Speed");
	Fl_Box box(70, 15, 200, 25, "Please Enter the Target Speed.");
  	box.labelsize(14);
	Fl_Input Target_Speed(70, 60, 200, 25, "");
  	Fl_Button cancel(40, 120, 80, 25, "cancel");
  	Fl_Return_Button ok(175, 120, 80, 25, "OK");
  	window.hotspot(&cancel);
  	window.end();
  	window.show();
	for (;;) {
    	   Fl::wait();
           Fl_Widget *o;
   	   while ((o = Fl::readqueue()))
      		if (o == &ok || o == &window){
		   const char* temp = Target_Speed.value();
		   if(temp!=NULL && temp[0]!='\0'){
		     talkWithMainApp->targetSpeed.targetSpeed = atoi(temp);
		     talkWithMainApp->askMainApp(MainAppCommand::TARGET_SPEED);
		     return;
		   }
		 }
		else if (o == &cancel || o == &window) return;
	}
  }
*/
  /* Hypothesis -- this is a button callback function that popups a
  	window to prompt the user for max speen, max/min depth */
   void Hypothesis(Fl_Widget* o){
	Fl_Window window(320,200, "Hypothesis");
	Fl_Box box(70, 15, 200, 25, "Please Enter the following values:");
  	box.labelsize(14);
	Fl_Input max_Speed(100, 50, 200, 25, "Max Speed:");
	Fl_Input max_Depth(100, 80, 200, 25, "Max Depth:");
	Fl_Input min_Depth(100, 110, 200, 25, "Min Depth:");
  	Fl_Button cancel(40, 160, 80, 25, "cancel");
  	Fl_Return_Button ok(175, 160, 80, 25, "OK");
  	window.hotspot(&cancel);
  	window.end();
  	window.show();
	for (;;) {
    	   Fl::wait();
           Fl_Widget *o;
   	   while ((o = Fl::readqueue()))
      		if (o == &ok || o == &window){
	           const char* maxS = max_Speed.value();
		   const char* maxD = max_Depth.value();
		   const char* minD = min_Depth.value();
		   if((maxS!=NULL && maxS[0]!='\0')&&(maxD!=NULL && maxD[0]!='\0')&&
		          (minD!=NULL && minD[0]!='\0')){
	              uiRight.targetSpeed->value(max_Speed.value(), strlen(maxS));
      	 	      uiRight.targetMaxDepth->value(max_Depth.value(), strlen(maxD));
     		      uiRight.targetMinDepth->value(min_Depth.value(), strlen(minD));
		      uiRight.targetSpeed->redraw();
		      uiRight.targetMaxDepth->redraw();
		      uiRight.targetMinDepth->redraw();
	              talkWithMainApp->hypothesis.maxSpeed = atof(maxS);
		      talkWithMainApp->hypothesis.maxDepth = atof(maxD);
		      talkWithMainApp->hypothesis.minDepth = atof(minD);
		      talkWithMainApp->askMainApp(MainAppCommand::HYPOTHESIS);
       		      return;
		   }
	         }
		else if (o == &cancel || o == &window) return;
	}
  }

  /* HypoContact -- This is a button callback function that popup
  	a window with a radio list of contacts that allows user to
	choose to display in the subve */
  /* this function also allows the user to input a name for the list of 
     contacts for easier recall of this data */

    void HypoContact(Fl_Widget* o){
    	const Fl_Menu_Item * items = uiRight.targetMenu->menu();
	int size = items->size();
	int winSize=220+numSol*20;
	Fl_Window window(320, winSize, "Solution List");
	Fl_Box box1(70, 15, 200, 25, "Please check the desired solution(s)");
  	box1.labelsize(14);
	int loc = 110;
	Fl_Check_Button *solution[64];

        // Composite display solution labels
        char SoLabel[64][30];
	Fl_Box *box[100];
	int count = 0;
	int sol = 0;
        int count2=0;
	
	Fl_Input* intersectionName;
	{ 
	  Fl_Input* o = intersectionName = new Fl_Input(70, 60, 200, 25, "Name of intersection:");
	  o->align(FL_ALIGN_TOP);
	}// this needs to be attached to data structure, and saved

	// to list the contacts and solutions
	while(size-1>0){
	  const char *label =items->label();
	  if (isMultiSolutionContact(label)){
	    box[sol] = new Fl_Box(20, loc, 200, 35, label ); sol++;
	    loc+=25;
	    const Fl_Menu_Item * sols = ((Fl_Menu_Button*)(items->user_data_))->menu();
	    int size2 = sols->size();
	    count2+=(size2-1);
	    char Label[64];
	    while(size2-1>0){
	      const char *solabel =sols->label();
	      strcpy(Label, label);
	      strcat(Label, solabel);
	      sprintf(SoLabel[count],"%s",Label);
	      solution[count] = new Fl_Check_Button(100, loc, 200, 25, solabel );
	      sols=sols->next();
	      count++;  size2--; loc+=20;
	    }
	  }
	  items = items->next(1);
	  size--;
	}
  	
  	Fl_Button cancel(40, loc+40, 80, 25, "cancel");
  	Fl_Return_Button ok(175, loc+40, 80, 25, "OK");
  	window.hotspot(&cancel);
  	window.end();
  	window.show();
	
	// to store the chosen contacts and/or solutions to an array to
	// be send across to subve through sharememory
	for (;;) {
	  Fl::wait();
	  Fl_Widget *o;
	  while ((o = Fl::readqueue()))
	    if (o == &ok || o == &window){
	      int num = 0;
	      count = 0;
	      while(count2>count){
		if(solution[count]->value() == 1){
		  strcpy(talkWithMainApp->hypoContact.contactList[num],SoLabel[count]);
		  num++;
		}
		
		count++;
	      }
	      
	      talkWithMainApp->hypoContact.numContact = num;
	      strcpy(talkWithMainApp->hypoContact.regionName,intersectionName->value());
	      talkWithMainApp->askMainApp(MainAppCommand::HYPO_CONTACTS);
	      return;
	    }
	    else if (o == &cancel || o == &window) return;
	}
    }


    void GrowRegion (Fl_Widget* button) {
     	const Fl_Menu_Item * items = uiRight.targetMenu->menu();
	int size = items->size();
	int winSize=160+numSol*20;
	Fl_Window window(320, winSize, "Grow Regions");
	Fl_Box box1(70, 15, 200, 25, "  Please check the contact(s) \nthat you would like to see grow.");
  	box1.labelsize(14);         
	int loc = 60;
	Fl_Check_Button *solution[64];
        // Composite display solution labels
        char SoLabel[64][30];
	oidlist oids;
	Fl_Box *box[100];
	int count = 0;
	int sol = 0;
        int count2=0;

	// to list the contacts and solutions
	while(size-1>0){
	   const char *label =items->label();
	   if (isMultiSolutionContact(label)){
		box[sol] = new Fl_Box(20, loc, 200, 35, label ); sol++;
		loc+=25;

		const Fl_Menu_Item * sols = ((Fl_Menu_Button*)(items->user_data_))->menu();
		int size2 = sols->size();
                count2+=(size2-1);
		char Label[64];
		while(size2-1>0){
		  const char *solabel =sols->label();
		  oids.push_back(sols->user_data_);
		  strcpy(Label, label);
		  strcat(Label, solabel);
		  sprintf(SoLabel[count],"%s",Label);
		  solution[count] = new Fl_Check_Button(100, loc, 200, 25, solabel );
		  for (int tm = 0; tm < growList.size(); tm++) {
		    if (sols->user_data_==growList[tm]) {
		      solution[count]->value(1);
		      break;
		    }
		  }
                  sols=sols->next();
		  count++;  size2--; loc+=20;
		}
  	   }
	   items = items->next(1);
	   size--;
	}
  	
  	Fl_Button cancel(40, loc+40, 80, 25, "cancel");
  	Fl_Return_Button ok(175, loc+40, 80, 25, "OK");
  	window.hotspot(&cancel);
  	window.end();
  	window.show();

	// to store the chosen contacts and/or solutions to an array to
	// be send across to subve through sharememory
	for (;;) {
	  Fl::wait();
	  Fl_Widget *o;
	  while ((o = Fl::readqueue()))
	    if (o == &ok || o == &window){
	      int num = 0;
	      count = 0;
	      growList.clear();
	      while(count2>count){
		if(solution[count]->value() == 1){
		  strcpy(talkWithMainApp->growRegion.solutionList[num],SoLabel[count]);
		  growList.push_back(oids[count]);
		  num++;
		}
		count++;
	      }
	      talkWithMainApp->growRegion.numSolution = num;
	      talkWithMainApp->askMainApp(MainAppCommand::GROWREGION);
	      return;
	    }
	    else if (o == &cancel || o == &window) return;
	}
    }     

    void ReduceRegion (Fl_Widget* button) {
     	const Fl_Menu_Item * items = uiRight.targetMenu->menu();
	int size = items->size();
	int winSize=160+numSol*20;
	Fl_Window window(320, winSize, "Reduce Regions");
	Fl_Box box1(70, 15, 200, 25, "  Please check the contact(s) \nthat you would like to see reduce.");
  	box1.labelsize(14);         
	int loc = 60;
	Fl_Check_Button *solution[64];
        // Composite display solution labels
        char SoLabel[64][30];
	oidlist oids;
	Fl_Box *box[100];
	int count = 0;
	int sol = 0;
        int count2=0;

	// to list the contacts and solutions
	while(size-1>0){
	   const char *label =items->label();
	   if (isMultiSolutionContact(label)){
		box[sol] = new Fl_Box(20, loc, 200, 35, label ); sol++;
		loc+=25;

		const Fl_Menu_Item * sols = ((Fl_Menu_Button*)(items->user_data_))->menu();
		int size2 = sols->size();
                count2+=(size2-1);
		char Label[64];
		while(size2-1>0){
		  const char *solabel =sols->label();
		  oids.push_back(sols->user_data_);
		  strcpy(Label, label);
		  strcat(Label, solabel);
		  sprintf(SoLabel[count],"%s",Label);
		  solution[count] = new Fl_Check_Button(100, loc, 200, 25, solabel );
		  for (int tm = 0; tm < reduceList.size(); tm++) {
		    if (sols->user_data_==reduceList[tm]) {
		      solution[count]->value(1);
		      break;
		    }
		  }
                  sols=sols->next();
		  count++;  size2--; loc+=20;
		}
  	   }
	   items = items->next(1);
	   size--;
	}
  	
  	Fl_Button cancel(40, loc+40, 80, 25, "cancel");
  	Fl_Return_Button ok(175, loc+40, 80, 25, "OK");
  	window.hotspot(&cancel);
  	window.end();
  	window.show();

	// to store the chosen contacts and/or solutions to an array to
	// be send across to subve through sharememory
	for (;;) {
	  Fl::wait();
	  Fl_Widget *o;
	  while ((o = Fl::readqueue()))
	    if (o == &ok || o == &window){
	      int num = 0;
	      count = 0;
	      reduceList.clear();
	      while(count2>count){
		if(solution[count]->value() == 1){
		  strcpy(talkWithMainApp->reduceRegion.solutionList[num],SoLabel[count]);
		  reduceList.push_back(oids[count]);
		  num++;
		}
		count++;
	      }
	      talkWithMainApp->reduceRegion.numSolution = num;
	      talkWithMainApp->askMainApp(MainAppCommand::REDUCEREGION);
	      return;
	    }
	    else if (o == &cancel || o == &window) return;
	}
    }     


  /* Solutions -- This is a button callback function that popup
  	a window with a radio list of contacts and solutions that
	allows user to 	choose to display in the subve */
  void Solutions(Fl_Widget* o){
    	const Fl_Menu_Item * items = uiRight.targetMenu->menu();
	int size = items->size();
//	int winSize=130+size*20;
	int winSize=160+numSol*20;
	Fl_Window window(320, winSize, "Solution List");
	Fl_Box box1(70, 15, 200, 25, "Please check the desired solution(s)");
  	box1.labelsize(14);
	int loc = 60;
	//Fl_Check_Button *solution[size];
	Fl_Check_Button *solution[64];
        // Composite display solution labels
        char SoLabel[64][30];
	Fl_Box *box[100];
	int count = 0;
	oidlist oids;
	int sol = 0;
        int count2=0;

	// to list the contacts and solutions
	while(size-1>0){
	  const char *label =items->label();
	  if (isMultiSolutionContact(label)){
	    box[sol] = new Fl_Box(20, loc, 200, 35, label ); sol++;
	    loc+=25;
	    
	    const Fl_Menu_Item * sols = ((Fl_Menu_Button*)(items->user_data_))->menu();
	    int size2 = sols->size();
	    count2+=(size2-1);
	    char Label[64];
	    while(size2-1>0){
	      const char *solabel =sols->label();
	      strcpy(Label, label);
	      strcat(Label, solabel);
	      sprintf(SoLabel[count],"%s",Label);
	      oids.push_back(sols->user_data_);
	      solution[count] = new Fl_Check_Button(100, loc, 200, 25, solabel );
	      int tm;
	      if ((strcmp(solabel, "CCS")==0)||(Label[0]=='H')) {
		bool defFound=false;
		for (tm = 0; tm < defaultSolutions.size(); tm++) {
		  if (sols->user_data_==defaultSolutions[tm]) {
		    defFound=true;
		    break;
		  }
		}
		if(!defFound){
		  solutionsChosen.push_back(sols->user_data_);
		  defaultSolutions.push_back(sols->user_data_);
		}
	      }
	      for (tm = 0; tm < solutionsChosen.size(); tm++) {
		if (sols->user_data_==solutionsChosen[tm]) {
		  solution[count]->value(1);
		  break;
		}
	      }
	      
	      sols=sols->next();
	      count++;  size2--; loc+=20;
	    }
	  }
	  items = items->next(1);
	  size--;
	}
	
  	Fl_Button cancel(40, loc+40, 80, 25, "cancel");
  	Fl_Return_Button ok(175, loc+40, 80, 25, "OK");
  	window.hotspot(&cancel);
  	window.end();
  	window.show();
	
	// to store the chosen contacts and/or solutions to an array to
	// be send across to subve through sharememory
	for (;;) {
	  Fl::wait();
	  Fl_Widget *o;
	  while ((o = Fl::readqueue()))
	    if (o == &ok || o == &window){
	      int num = 0;
	      count = 0;
	      solutionsChosen.clear();
	      while(count2>count){
		if(solution[count]->value() == 1){
		  strcpy(talkWithMainApp->solution.solutionList[num],SoLabel[count]);
		  solutionsChosen.push_back(oids[count]);
		  num++;
		}
		
		count++;
	      }
	      
	      talkWithMainApp->solution.numSolution = num;
	      talkWithMainApp->askMainApp(MainAppCommand::SOLUTIONS);
	      return;
	    }
	    else if (o == &cancel || o == &window) return;
	}
  }

  /* Lon_Lat -- this a callback function that prompts user for the starting
  	point of the subve display by entering the north-east
  	and the south-west point of the longitude and latitude */
   int Lon_Lat(Fl_Widget* o){
	double lon_se ='\0';
	double lon_nw ='\0';
	double lat_se ='\0';
	double lat_nw ='\0';
  	Fl_Window window(320,260, "Longitude/Latitude");
	Fl_Box box(70, 15, 200, 25, "Please enter Longitude/Latitude");
	Fl_Box box1(60, 35, 200, 25, "for the north-west and south-east corner.");
  	box.labelsize(12);
	box1.labelsize(12);
  	Fl_Input LON_NW(70, 60, 200, 25, "Lon_nw:");
  	Fl_Input LAT_NW(70, 100, 200, 25, "Lat_nw:");
	Fl_Input LON_SE(70, 140, 200, 25, "Lon_se:");
  	Fl_Input LAT_SE(70, 180, 200, 25, "Lat_se:");
  	Fl_Button cancel(40, 220, 80, 25, "cancel");
  	Fl_Return_Button ok(175, 220, 80, 25, "OK");
  	window.hotspot(&cancel);
  	window.end();
  	window.show();
  	for (;;) {
    	   Fl::wait();
           Fl_Widget *o;
   	   while ((o = Fl::readqueue())) {
      		if (o == &ok) {
		        bool flag1 = CheckLonLat(LON_NW.value(), lon_nw, 1);
			bool flag2 = CheckLonLat(LAT_NW.value(), lat_nw, 2);
			bool flag3 = CheckLonLat(LON_SE.value(), lon_se, 1);
			bool flag4 = CheckLonLat(LAT_SE.value(), lat_se, 2);
			if(flag1 && flag2 && flag3 && flag4){
				if (lon_nw ==0&& lat_nw == 0&&lon_se==0 && lat_se==0) IllegalRectBox();
				else if (lon_nw>=lon_se) IllegalRectBox();
				else if (lat_nw<=lat_se) IllegalRectBox();
				else {
					talkWithMainApp->lonLat.lon_nw=lon_nw;
					talkWithMainApp->lonLat.lat_nw=lat_nw;
					talkWithMainApp->lonLat.lon_se=lon_se;
					talkWithMainApp->lonLat.lat_se=lat_se;
					talkWithMainApp->askMainApp(MainAppCommand::LON_LAT);
					return 1;
				}
			}
			else LonLatErrorBox();

      		} else if (o == &cancel || o == &window) return 0;
	   }
	}
  }

  /* LonLatErrorBox -- displays the correct data format for longitude and latitude*/
void LonLatErrorBox(){
  	Fl_Window window(320,280, "ERROR!");
	Fl_Box box(60, 20, 200, 25, "Value you've entered is not correct!");
	Fl_Box box1(65, 40, 200, 25, "Please try again!");
	Fl_Box box2(60, 70, 200, 25, "Formats examples:");
	Fl_Box box5(60, 90, 200, 35, "(+ -)90");
	Fl_Box box6(60, 110, 200, 25, "(+ -)90.0");
	Fl_Box box7(60, 130, 200, 25, "(+ -)90 12");
	Fl_Box box8(60, 150, 200, 25, "(+ -)90 12 12");
	Fl_Box box3(60, 175, 200, 25, "Range (latitude):  (+ -)90");
	Fl_Box box4(60, 195, 200, 25, "Range (longitude):  (+ -)180");
	box.labelsize(12);
	box1.labelsize(12);
	box2.labelsize(12);
	box3.labelsize(12);
	box4.labelsize(12);
	box5.labelsize(12);
	box6.labelsize(12);
	box7.labelsize(12);
	box8.labelsize(12);

  	Fl_Return_Button ok(125, 230, 80, 25, "OK");
  	window.end();
  	window.show();
  	for (;;) {
    	   Fl::wait();
           Fl_Widget *o;
   	   while ((o = Fl::readqueue()))
      		if (o == &ok || o == &window) return;
	}
  }
   /* IllegalRectBox -- this function checks the user input of the longitude
  	and latitude data too see if the input is current.  that the northwest
	longitude is less than southeast longitude, and that NW lat is greater
	than the SE latitude, etc...*/
   void IllegalRectBox(){
  	Fl_Window window(370,180, "ERROR!");
	Fl_Box box(60, 20, 200, 25, "Value you've entered for the upper left corner coordinate");
	Fl_Box box1(60, 40, 200, 25, "and/or the lower right corner coordinate is not correct!");
	Fl_Box box2(60, 70, 200, 25, "1: Can not have all 0 coordinates.");
	Fl_Box box3(60, 90, 200, 25, "2: Lat_NW must be greater than Lat_SE.");
	Fl_Box box4(60, 110, 200, 25, "3: Lon_NW must be smaller than Lon_SE.");
	box.labelsize(12);
	box1.labelsize(12);
	box2.labelsize(12);
	box3.labelsize(12);
	box4.labelsize(12);

  	Fl_Return_Button ok(125, 140, 80, 25, "OK");
  	window.end();
  	window.show();
  	for (;;) {
    	   Fl::wait();
           Fl_Widget *o;
   	   while ((o = Fl::readqueue()))
      		if (o == &ok || o == &window) return;
	}
  }

  /* EmptyEntry -- this function create a popup window to tell user
   	to enter some data or to cancle*/
   void EmptyEntry(){
  	Fl_Window window(370,120, "ERROR!");
	Fl_Box box(60, 20, 200, 25, "Please enter the value(s)");
	Fl_Box box1(60, 40, 200, 25, "or 'cancle' to continue!");
	box.labelsize(12);
	box1.labelsize(12);
  	Fl_Return_Button ok(125, 80, 80, 25, "OK");
  	window.end();
  	window.show();
  	for (;;) {
    	   Fl::wait();
           Fl_Widget *o;
   	   while ((o = Fl::readqueue()))
      		if (o == &ok || o == &window) return;
	}
  }

  /* CheckLonLat -- this function checks for the format of the
  	longitude and latitude data-- (+ -)deg, min, and sec,
	or (+ -) min, sec... etc ... */
bool CheckLonLat(const char* lon, double& value, int check)
{
	char temp1[100], temp2[100], temp3[100];
	for (int k=0; k<100; k++){
		temp1[k] = '\0';
		temp2[k] = '\0';
		temp3[k] = '\0';
		}
	double t1= 0;
	double t2 = 0;
	double t3 = 0;
	int space =-1;
	int j =0;
	bool flag = true;
	for(int i =0; lon != NULL && lon[i] != '\0'; i++)
	{
		if(lon[i]=='.'|| lon[i]=='0' || lon[i]=='1'|| lon[i]=='2'
			  || lon[i]=='3' || lon[i]=='4' || lon[i]=='5'
			  || lon[i]=='6' || lon[i]=='7' || lon[i]=='8' || lon[i]=='9'){

			  if (space ==-1)space =0;
			  if(space ==0) temp1[j]=lon[i];
			  else if(space ==1) temp2[j] = lon[i];
		          else if(space == 2) temp3[j] =lon[i];
		          j++;

			  if (lon[0] == '-')flag = false;
		}
		else if((lon[i]=='+' || lon[i]=='-')&& space == -1){ temp1[j] = lon[i];j++;}
		else if(lon[i]==' '){
			while(lon[i] == ' '){
				i++;
				j = 0;
			}
			if (j==0) i--;
			if(i!=0) space++;
		}
		else return false;
	}

	switch(space){
	case 0:
		value = atof(temp1);
		break;
	case 1:
		t1 = atof(temp1);
		t2 = atof(temp2);
		if(flag) value = t1+ t2*(5.0/300);
		else value = t1- t2*(5.0/300);
		break;
	case 2:
		t1 = atof(temp1);
		t2 = atof(temp2);
		t3 = atof(temp3);
		if(flag) value = t1+(t2+t3*(5.0/300))*(5.0/300);
		else value = t1-(t2+t3*(5.0/300))*(5.0/300);
		break;
	default:
		value = 0;
		return false;
	}

	// lon
	if (check == 1){
		if(value <= 180 && value >= -180) return true;
		else return false;

	}
	// lat
	else{
		if(value <= 90 && value >= -90) return true;
		else return false;
	}

	return true;
}

/* updateOwnship -- this function updates the ownship data displayed
	in the right bezel to current data passed over from subve
	through share memory.  Those data includes course, speed, depth
	set, drift, etc...*/
   void updateOwnship(const PreciseContact& ownship) {
      char number[UI_NUMFIELD_MAX_LENGTH+1];
      char number2[9];
      char subdtf[16];
/******************************/
      xship = ownship.xyz[0];
      yship = ownship.xyz[1];
/******************************/

     snprintf(number,UI_NUMFIELD_MAX_LENGTH,"%-12.3f",ownship.xyz[0]);

     CON_LATLON(ownship.xyz[0],2, number2);
      uiRight.ownshipLON->value(number2,strlen(number2));

/******** Dummy Ownship values: ****************/
    sprintf(number,"%-12.3f",ZERO);
    uiRight.ownshipSet->value(number,strlen(number));
    uiRight.ownshipDrift->value(number,strlen(number));
    uiRight.ownshipSet->value(number,strlen(number));
    sprintf(number,"%-5.1f",FAKE_OWNSHIP_SPEED);
    uiRight.ownshipSpeed->value(number,strlen(number));
/***********************************************/


      CON_LATLON(ownship.xyz[1],1, number2);
      uiRight.ownshipLAT->value(number2,strlen(number2));

      sprintf(number,"%-4.0f",ABS(METERS_TO_FEET(ownship.xyz[2])));
      uiRight.ownshipDepth->value(number,strlen(number));

      sprintf(number,"%-3.0f",ABS(ownship.hpr[0]));
      uiRight.ownshipCourse->value(number,strlen(number));
      hship = atof(number);

      CON_DATETIME(ownship.time.tv_sec, subdtf);
      uiRight.ownshipTime->value(subdtf,strlen(subdtf));

      uiRight.ownshipLON->redraw();
      uiRight.ownshipLAT->redraw();
      uiRight.ownshipDepth->redraw();
      uiRight.ownshipCourse->redraw();
      uiRight.ownshipSet->redraw();
      uiRight.ownshipDrift->redraw();
      uiRight.ownshipSpeed->redraw();
      uiRight.ownshipTime->redraw();
   }

/* updateCurrentTarget -- this function updates the current precise contact
	target data displayed in the right bezel to current data passed
	over from subve through share memory.  Those data includes age,
	speed, depth bearing,	etc...*/
   void updateCurrentTarget(const PreciseContact& target) {
      char number[UI_NUMFIELD_MAX_LENGTH+1];
      char number2[9];
      //char subage[6];
      char subage[]={'\0','\0','\0','\0','\0','\0','\0'};

      CON_LATLON(target.xyz[0],2,number2);
      uiRight.targetLON->value(number2,strlen(number2));
      xtarget = target.xyz[0];

      CON_LATLON(target.xyz[1],1,number2);
      uiRight.targetLAT->value(number2,strlen(number2));
      ytarget = target.xyz[1];
      
      if (ABS(METERS_TO_FEET(target.xyz[2])) <10){
	uiRight.udcontact_icon->show();
	uiRight.dcontact_icon->hide();
      }
      else{
	uiRight.udcontact_icon->hide();
	uiRight.dcontact_icon->show();
      }

      sprintf(number,"%-4.0f",ABS(METERS_TO_FEET(target.xyz[2])));
      uiRight.targetDepth->value(number,strlen(number));

      sprintf(number,"%-3.0f",ABS(target.hpr[0]));
      uiRight.targetCourse->value(number,strlen(number));

      CON_AGE(target.time.tv_sec,subage);
      uiRight.targetAge->value(subage,strlen(subage));

      sprintf(number,"%-5.1f",FAKE_TARGET_SPEED);
      uiRight.targetSpeed->value(number,strlen(number));

     // Get Target Range and Bearing:
     range_bearing(xship, yship,target.xyz[0], target.xyz[1], hship);
     range = range * 1.936;

      sprintf(number,"%-6.1f",ABS(range));
      uiRight.targetRange->value(number,strlen(number));

      sprintf(number,"%-3.0f",ABS(bearing));
      strcat(number, "T");
      uiRight.targetBearing->value(number,strlen(number));

      uiRight.targetLON->redraw();
      uiRight.targetLAT->redraw();
      uiRight.targetDepth->redraw();
      uiRight.targetCourse->redraw();
      uiRight.targetAge->redraw();
      uiRight.targetRange->redraw();
      uiRight.targetBearing->redraw();
      uiRight.targetSpeed->redraw();

      // JAVA Info to Time/Bearing Plot:
       jmethodID mid;
       (env)->ExceptionClear();
       mid=(env)->GetStaticMethodID(cls,"addTB","(DDD)V");
       env->CallStaticVoidMethod(cls, mid,(double)target.time.tv_sec,bearing,(double)hship );
		   
   }


/* updateCurrentTarget -- this function updates the current sensor contact
	target data displayed in the right bezel to current data passed
	over from subve through share memory.  Those data includes age,
	speed, depth bearing,	etc...*/
   void updateCurrentTarget(const ContactFromSensor& target) {
      char number[UI_NUMFIELD_MAX_LENGTH+1];
      char number2[9];
      //char subage[6];
      char subage[]={'\0','\0','\0','\0','\0','\0','\0'};

      CON_LATLON(target.xyz[0],2,number2);
      uiRight.targetLON->value(number2,strlen(number2));

      CON_LATLON(target.xyz[1],1,number2);
      uiRight.targetLAT->value(number2,strlen(number2));

      // submerge or surface ship
      if (ABS(METERS_TO_FEET(target.xyz[2])) <10){
	uiRight.udcontact_icon->show();
	uiRight.dcontact_icon->hide();
      }
      else{
	uiRight.udcontact_icon->hide();
	uiRight.dcontact_icon->show();
      }
      
      
      sprintf(number,"%-4.0f",ABS(METERS_TO_FEET(target.xyz[2])));
      uiRight.targetDepth->value(number,strlen(number));

      sprintf(number,"%-3.0f",ABS(target.hpr[0]));
      uiRight.targetCourse->value(number,strlen(number));

      //sprintf(number,"%-5.1i",target.time.tv_sec);
      //uiRight.targetAge->value(number,strlen(number));

      CON_AGE(target.time.tv_sec,subage);
      uiRight.targetAge->value(subage,strlen(subage));


      sprintf(number,"%-5.1f",FAKE_TARGET_SPEED);
      uiRight.targetSpeed->value(number,strlen(number));

   // Get Target Range and Bearing:
     range_bearing(xship, yship,target.xyz[0], target.xyz[1], hship);
     range = range * 1.936;

      sprintf(number,"%-6.1f",ABS(range));
      uiRight.targetRange->value(number,strlen(number));

      sprintf(number,"%-3.0f",ABS(bearing));
      strcat(number, "T");
      uiRight.targetBearing->value(number,strlen(number));

      /* following lines of datas are all fake target datas.
      	please change it when the real datas are passed over
	from subve; or are calculate */
	sprintf(number,"%-12.3f",NINE);
      uiRight.targetMaxRange->value(number,strlen(number));
      uiRight.targetMinRange->value(number,strlen(number));
      uiRight.targetMaxBearing->value(number,strlen(number));
      uiRight.targetMinBearing->value(number,strlen(number));
      uiRight.targetMaxDepth->value(number,strlen(number));
      uiRight.targetMinDepth->value(number,strlen(number));

      uiRight.targetMaxRange->redraw();
      uiRight.targetMinRange->redraw();
      uiRight.targetMaxDepth->redraw();
      uiRight.targetMinDepth->redraw();
      uiRight.targetMaxBearing->redraw();
      uiRight.targetMinBearing->redraw();

      uiRight.targetLON->redraw();
      uiRight.targetLAT->redraw();
      uiRight.targetDepth->redraw();
      uiRight.targetCourse->redraw();
      uiRight.targetAge->redraw();
      uiRight.targetRange->redraw();
      uiRight.targetBearing->redraw();
      uiRight.targetSpeed->redraw();
   }


   /* clearCurrentTarget -- this function clear all the data fields in the
   	target display window at the right bezel*/
   void clearCurrentTarget() {
      uiRight.targetMaxRange->value(emptyString,1);
      uiRight.targetMinRange->value(emptyString,1);
      uiRight.targetMaxDepth->value(emptyString,1);
      uiRight.targetMinDepth->value(emptyString,1);
      uiRight.targetMaxBearing->value(emptyString,1);
      uiRight.targetMinBearing->value(emptyString,1);

      uiRight.targetLON->value(emptyString,1);
      uiRight.targetLAT->value(emptyString,1);
      uiRight.targetDepth->value(emptyString,1);
      uiRight.targetCourse->value(emptyString,1);
      uiRight.targetAge->value(emptyString,1);
      uiRight.targetRange->value(emptyString,1);
      uiRight.targetBearing->value(emptyString,1);
      uiRight.targetSpeed->value(emptyString,1);

      uiRight.targetMenu->label("Contacts");
      uiRight.solutionMenu->label("Solutions");

      uiRight.solutionMenu->redraw();
	uiRight.targetMenu->redraw();

      uiRight.targetMaxRange->redraw();
      uiRight.targetMinRange->redraw();
      uiRight.targetMaxDepth->redraw();
      uiRight.targetMinDepth->redraw();
      uiRight.targetMaxBearing->redraw();
      uiRight.targetMinBearing->redraw();

      uiRight.targetLON->redraw();
      uiRight.targetLAT->redraw();
      uiRight.targetDepth->redraw();
      uiRight.targetCourse->redraw();
      uiRight.targetAge->redraw();
      uiRight.targetRange->redraw();
      uiRight.targetBearing->redraw();
      uiRight.targetSpeed->redraw();
   }

   /* addNewTarget -- this function adds new precise target name to the target menu and store the
   	data so it can be display when it's chosen*/
   void addNewTarget(OID::Type contactID, const char* targetName, const PreciseContact& target) {
     numSol++;
     /* we store 'contactID' in the user_data field of the menu item,
	so when a menu item is selected, we can tell the main app
	what contact the action affects by passing its contactID as parameter. */
     int val = uiRight.targetMenu->add(targetName,(const char*)NULL,NULL,contactID);
     /* if there is no other target,
	then automatically select this one. */
     if (uiRight.targetMenu->size()==2) {
       uiRight.targetMenu->value(val);
       TargetMenuItemSelected(uiRight.targetMenu);
       updateCurrentTarget(target);
     }
   }

   /* addNewSol -- this function add the name and information regarding to
   	target solution to the contact list so it can be displyed when it's
	chosen by user.*/
   void addNewSol(OID::Type contactID, const char* SolName, const ContactFromSensor& Sol, Fl_Menu_Button* tmpMenu) {
     numSol++;
      /* we store 'contactID' in the user_data field of the menu item,
	 so when a menu item is selected, we can tell the main app
	 what contact the action affects by passing its contactID as parameter. */
     int val = tmpMenu->add(SolName,(const char*)NULL,NULL,contactID);
     
     /* if there is no other target,
	then automatically select this one. */
     if (tmpMenu->size() == 2) {
       tmpMenu->value(val);
     }
   }

   /* addNewTarget -- this function adds new sensor target name to the target menu and store the
   	data so it can be display when it's chosed*/
   void addNewTarget(OID::Type contactID, const char* targetName, const char* solName, const ContactFromSensor& target) {
     Fl_Menu_Button *tmpMenu;
     bool find = false;
     int pos=0;
     //   if(talkWithMainApp->selectedTarget != BAD_OID){
     if (uiRight.targetMenu->size()>0){
       const Fl_Menu_Item *items = uiRight.targetMenu->menu();
       while(items->label()!=NULL&&strcmp(items->label(),targetName)!=0) {
	 items=items->next(1);
	 pos++;
       }
       if(items->label()!=NULL&&strcmp(items->label(),targetName)==0){
	 tmpMenu  = (Fl_Menu_Button*)items->user_data_;
	 find = true;
       }
     }
     
     if (uiRight.targetMenu->size()<0 || !find){
       tmpMenu = new Fl_Menu_Button(55, 500, 142, 30, "Solutions ");
       tmpMenu->box(FL_EMBOSSED_BOX);
       tmpMenu->color(42);
       tmpMenu->labelfont(1);
       tmpMenu->labelcolor(7);
       tmpMenu->labelsize(20);
       tmpMenu->callback(SolutionMenuItemSelected);
       numSol++;
     }
     
     addNewSol(contactID, solName, target, tmpMenu);
     pos = uiRight.targetMenu->add(targetName,(const char*)NULL,NULL,tmpMenu);
     if (!find) {
       //pos = uiRight.targetMenu->add(targetName,(const char*)NULL,NULL,tmpMenu);
       if (uiRight.targetMenu->size()==2) {
	 uiRight.targetMenu->value(pos);
	 //TargetMenuItemSelected(uiRight.targetMenu);
       }
     }
     if (uiRight.targetMenu->value()==pos) {
       int solval=uiRight.solutionMenu->value();
       if (solval<tmpMenu->size()-1)
	 tmpMenu->value(solval);
       TargetMenuItemSelected(uiRight.targetMenu);
       if (tmpMenu->size()==2)
	 updateCurrentTarget(target);
     }
   }

   /* removeTarget -- this function removes the target from the target list */
   void removeTarget(OID::Type contactID) {
     if(contactID != BAD_OID) {
       bool foundContact=false;
       int temp = 0;
       int tval=0,sval=-1;
       Fl_Menu_Button* solMen2;
       const Fl_Menu_Item * items = uiRight.targetMenu->menu();
       while(temp<uiRight.targetMenu->size()-1) {
	 if(!isMultiSolutionContact(items->label())){
	   if (items->user_data_==contactID) {
	     tval=temp;
	     sval=-1;
	     foundContact=true;
	   }
	 }
	 else{
	   Fl_Menu_Button* solMen = (Fl_Menu_Button *)items->user_data_;
	   const Fl_Menu_Item * solItems = solMen->menu();
	   int temp2=0;
	   while(temp2<solMen->size()-1) {
	     if (solItems->user_data_==contactID) {
	       tval=temp;
	       sval=temp2;
	       solMen2=solMen;
	       foundContact=true;
	     }
	     temp2++;
	     solItems=solItems->next(1);
	   }
	 }
	 temp++;
	 items=items->next(1);
       }
       if (foundContact) {
	 if (sval!=-1) {
	   if (solMen2) {
	     solMen2->remove(sval);
	     if (solMen2->size()<=1) sval=-1;
	     else {
	       if (uiRight.targetMenu->value()==tval) {
		 if (uiRight.solutionMenu->value()>uiRight.solutionMenu->size()-2) {
		   uiRight.solutionMenu->value(uiRight.solutionMenu->size()-2);
		 }
		 SolutionMenuItemSelected(uiRight.solutionMenu);
	       }
	     }
	     numSol--;
	   }
	 }
	 if (sval==-1) {
	   uiRight.targetMenu->remove(tval);
	   numSol--;
	   if (uiRight.targetMenu->size()<=1) {
	     clearCurrentTarget();
	   } else {
	     if (uiRight.targetMenu->value()>uiRight.targetMenu->size()-2) {
	       uiRight.targetMenu->value(uiRight.targetMenu->size()-2);
	     }
	     TargetMenuItemSelected(uiRight.targetMenu);
	   }
	 }
       }
     }
   }


   void cleanUpAndFinish(int signal = 0) {
      if(talkWithMainApp != NULL) delete talkWithMainApp;
      printf("Execution of SubVE Bezel finished.\n");
      fflush(stdout);
      //jvm->DestroyJavaVM();
      //
      exit(1);
   }

   /* TargetMenuItemSelected -- when a target in the menu is been selected,
   	the new target information will be displayed on to the screen.  It
	also send a signal to subve to highlight the current selected target*/
   void TargetMenuItemSelected (Fl_Widget *w) {
      int temp = 0;
      const Fl_Menu_Item * items = ((Fl_Menu_Button*)w)->menu();
      while(temp <((Fl_Menu_Button*)w)->value())
      {
      	items=items->next(1);
	temp++;
      }

      uiRight.targetMenu->label(items->label());
      uiRight.targetMenu->redraw();

      if(!isMultiSolutionContact(items->label())) {
	uiRight.solutionMenu->menu(NULL);
	uiRight.solutionMenu->label("Solution");
      	uiRight.solutionMenu->redraw();
	talkWithMainApp->selectedTarget = (((Fl_Menu_Button*)w)->menu()+((Fl_Menu_Button*)w)->value())->user_data_;
 	talkWithMainApp->askMainApp(MainAppCommand::SELECT_TARGET,talkWithMainApp->selectedTarget);
      }
      else{
	uiRight.solutionMenu->menu(((Fl_Menu_Button*)(items->user_data_))->menu());
	/*talkWithMainApp->selectedTarget = uiRight.solutionMenu->menu()->user_data_;
	talkWithMainApp->askMainApp(MainAppCommand::SELECT_TARGET,talkWithMainApp->selectedTarget);
	uiRight.solutionMenu->label(uiRight.solutionMenu->menu()->label());
	uiRight.solutionMenu->redraw();
	*/
	int val=((Fl_Menu_Button*)(items->user_data_))->value();
	uiRight.solutionMenu->value(val);
	SolutionMenuItemSelected(uiRight.solutionMenu);
      }
      
      uiRight.targetMenu->label(items->label());
      uiRight.targetMenu->redraw();
      
   }

   /* SolutionMenuItemSelected -- when a solution of a target in the menu is been selected,
   	the new solution information will be displayed on to the screen.  It
	also send a signal to subve to highlight the current selected solution*/
   void SolutionMenuItemSelected (Fl_Widget *w) {
      int temp = 0;
      const Fl_Menu_Item * items = ((Fl_Menu_Button*)w)->menu();
      while(temp <((Fl_Menu_Button*)w)->value())
      {
      	items=items->next(1);
	temp++;
      }
      talkWithMainApp->selectedTarget = (((Fl_Menu_Button*)w)->menu()+((Fl_Menu_Button*)w)->value())->user_data_;
      talkWithMainApp->askMainApp(MainAppCommand::SELECT_TARGET,talkWithMainApp->selectedTarget);
      uiRight.solutionMenu->label(items->label());
      uiRight.solutionMenu->redraw();

   }

   /* WnpMenuItemSelected -- when a weapon in the menu is been selected,
   	the new weapon information will be displayed on to the screen.  It
	also send a signal to subve to highlight the current selected weapon*/
   void WnpMenuItemSelected (Fl_Widget *w) {
      int temp = 0;
      const Fl_Menu_Item * items = ((Fl_Menu_Button*)w)->menu();
      while(temp <((Fl_Menu_Button*)w)->value())
      {
      	items=items->next(1);
	temp++;
      }
      talkWithMainApp->selectedWeapon = (((Fl_Menu_Button*)w)->menu()+((Fl_Menu_Button*)w)->value())->user_data_;
      talkWithMainApp->askMainApp(MainAppCommand::SELECT_WEAPON,talkWithMainApp->selectedWeapon);
      uiRight.wnpMenu->label(items->label());
      uiRight.wnpMenu->redraw();
   }


   /* UuvMenuItemSelected -- when a UUV in the menu is been selected,
   	the new weapon information will be displayed on to the screen.  It
	also send a signal to subve to highlight the current selected weapon*/
   void UuvMenuItemSelected (Fl_Widget *w) {
      talkWithMainApp->selectedWeapon = (((Fl_Menu_Button*)w)->menu()+((Fl_Menu_Button*)w)->value())->user_data_;
      talkWithMainApp->askMainApp(MainAppCommand::SELECT_WEAPON,talkWithMainApp->selectedWeapon);
      int temp = 0;
      const Fl_Menu_Item * items = ((Fl_Menu_Button*)w)->menu();
      while(temp <((Fl_Menu_Button*)w)->value())
      {
      	items=items->next(1);
	temp++;
      }
      uiRight.uuvMenu->label(items->label());
      uiRight.uuvMenu->redraw();
   }

   /* MisMenuItemSelected -- when a Mis weapon in the menu is been selected,
   	the new weapon information will be displayed on to the screen.  It
	also send a signal to subve to highlight the current selected weapon*/
   void MisMenuItemSelected (Fl_Widget *w) {
      talkWithMainApp->selectedWeapon = (((Fl_Menu_Button*)w)->menu()+((Fl_Menu_Button*)w)->value())->user_data_;
      talkWithMainApp->askMainApp(MainAppCommand::SELECT_WEAPON,talkWithMainApp->selectedWeapon);
      int temp = 0;
      const Fl_Menu_Item * items = ((Fl_Menu_Button*)w)->menu();
      while(temp <((Fl_Menu_Button*)w)->value())
      {
      	items=items->next(1);
	temp++;
      }
      uiRight.misMenu->label(items->label());
      uiRight.misMenu->redraw();
   }

   /* SaMenuItemSelected -- when a SA weapon in the menu is been selected,
   	the new SA information will be displayed on to the screen.  It
	also send a signal to subve to highlight the current selected weapon*/
   void SaMenuItemSelected (Fl_Widget *w) {
      talkWithMainApp->selectedWeapon = (((Fl_Menu_Button*)w)->menu()+((Fl_Menu_Button*)w)->value())->user_data_;
      talkWithMainApp->askMainApp(MainAppCommand::SELECT_WEAPON,talkWithMainApp->selectedWeapon);
      int temp = 0;
      const Fl_Menu_Item * items = ((Fl_Menu_Button*)w)->menu();
      while(temp <((Fl_Menu_Button*)w)->value())
      {
      	items=items->next(1);
	temp++;
      }
      uiRight.saMenu->label(items->label());
      uiRight.saMenu->redraw();
   }

   /* TaMenuItemSelected -- when a TA weapon in the menu is been selected,
   	the new TA information will be displayed on to the screen.  It
	also send a signal to subve to highlight the current selected weapon*/
   void TaMenuItemSelected (Fl_Widget *w) {
      talkWithMainApp->selectedWeapon = (((Fl_Menu_Button*)w)->menu()+((Fl_Menu_Button*)w)->value())->user_data_;
      talkWithMainApp->askMainApp(MainAppCommand::SELECT_WEAPON,talkWithMainApp->selectedWeapon);
      int temp = 0;
      const Fl_Menu_Item * items = ((Fl_Menu_Button*)w)->menu();
      while(temp <((Fl_Menu_Button*)w)->value())
      {
      	items=items->next(1);
	temp++;
      }
      uiRight.taMenu->label(items->label());
      uiRight.taMenu->redraw();
   }

   /* targetSelectedInMainApp-- this function is ussed to display target
    	informations for the target that is selected in subve, */
   void targetSelectedInMainApp(OID::Type contactID) {
      if(contactID == BAD_OID)
	clearCurrentTarget();
      else {
	bool foundContact=false,foundWeap=false;
	int temp = 0;
	int tval=0,sval=-1;
	Fl_Menu_Button* solMen2;
	const Fl_Menu_Item * items = uiRight.targetMenu->menu();
	while(temp<uiRight.targetMenu->size()-1) {
	  if(!isMultiSolutionContact(items->label())) {
	    if (items->user_data_==contactID) {
	      tval=temp;
	      sval=-1;
	      foundContact=true;
	    }
	  }
	  else{
	    Fl_Menu_Button* solMen = (Fl_Menu_Button *)items->user_data_;
	    const Fl_Menu_Item * solItems = solMen->menu();
	    int temp2=0;
	    while(temp2<solMen->size()-1) {
	      if (solItems->user_data_==contactID) {
		tval=temp;
		sval=temp2;
		solMen2=solMen;
		foundContact=true;
	      }
	      temp2++;
	      solItems=solItems->next(1);
	    }
	  }
	  temp++;
	  items=items->next(1);
	}
	temp=0;
	items = uiRight.wnpMenu->menu();
	while(temp<uiRight.wnpMenu->size()-1) {
	    if (items->user_data_==contactID) {
	      tval=temp;
	      sval=-1;
	      foundWeap=true;
	    }
	    temp++;
	    items=items->next(1);
	}
	if (foundContact) {
	  uiRight.targetMenu->value(tval);
	  if (sval!=-1) {
	    if (solMen2) {
	      solMen2->value(sval);
	    }
	  }
	  TargetMenuItemSelected(uiRight.targetMenu);
	} else if (foundWeap) {
	  uiRight.wnpMenu->value(tval);
	  WnpMenuItemSelected(uiRight.wnpMenu);
	}
      }
     
   }

   /* updateCurrentWeapon -- this function updates the current precise contact
	weapon data displayed in the right bezel to current data passed
	over from subve through share memory.  These datas includes course,
	age, speed, depth, etc...*/
   void updateCurrentWeapon(const WeaponContact& weapon) {
      char number[UI_NUMFIELD_MAX_LENGTH+1];
      char number2[9];
      char subage[]={'\0','\0','\0','\0','\0','\0','\0'};
      int index;

      CON_LATLON(weapon.xyz[0],2,number2);
      uiRight.weaponLON->value(number2,strlen(number2));

      CON_LATLON(weapon.xyz[1],1,number2);
      uiRight.weaponLAT->value(number2,strlen(number2));

      sprintf(number,"%4.0f",ABS(METERS_TO_FEET(weapon.xyz[2])));
      uiRight.weaponDepth->value(number,strlen(number));

      sprintf(number,"%-5.1f",ABS(weapon.hpr[0]));
      uiRight.weaponCourse->value(number,strlen(number));

      CON_AGE(weapon.time.tv_sec,subage);
      uiRight.weaponAge->value(subage,strlen(subage));

      sprintf(number,"%-5.1f",FAKE_WEAPON_SPEED);
      uiRight.weaponSpeed->value(number,strlen(number));

      index = int(weapon.Mode);
      sprintf(number,WeaponCodes[index]);
      uiRight.weaponMode->value(number,strlen(number));

   // Get Target Range and Bearing:
   // Get Target Range and Bearing:
     range_bearing(xship, yship,weapon.xyz[0], weapon.xyz[1], hship); // Range and Bearing of Weapon from Ownship
      range = range * 1.936;

      sprintf(number,"%-6.1f",ABS(range));
      uiRight.weaponRange->value(number,strlen(number));

      sprintf(number,"%-3.0f",ABS(bearing));
      strcat(number, "T");
      uiRight.weaponBearing->value(number,strlen(number));

      uiRight.weaponLON->redraw();
      uiRight.weaponLAT->redraw();
      uiRight.weaponDepth->redraw();
      uiRight.weaponCourse->redraw();
      uiRight.weaponAge->redraw();
      uiRight.weaponRange->redraw();
      uiRight.weaponBearing->redraw();
      uiRight.weaponSpeed->redraw();
      uiRight.weaponMode->redraw();
   }

   /* addNewWeapon -- This function adds new weapon and information to the menu list
   	 of weapons, so it can later be displayed on the right bezel*/
   void addNewWeapon(OID::Type contactID, const char* weaponName, const WeaponContact& weapon) {
   	/* we store 'contactID' in the user_data field of the menu item,
   	   so when a menu item is selected, we can tell the main app
   	   what contact the action affects by passing its contactID as parameter. */
      int val=uiRight.wnpMenu->add(weaponName,(const char*)NULL,NULL,contactID);
      /* This displays the last Weapon added to the list. */
      /*uiRight.wnpMenu->label(weaponName);
	uiRight.wnpMenu->redraw();
	talkWithMainApp->selectedWeapon = contactID;
	talkWithMainApp->askMainApp(MainAppCommand::SELECT_WEAPON,contactID);*/
      updateCurrentWeapon(weapon);
      uiRight.wnpMenu->value(val);
      WnpMenuItemSelected(uiRight.wnpMenu);
   }

   /* clearCurrentWeapon -- this function clears all the data fields of the
   	current weapon information displayed on the right bezel. */
   void clearCurrentWeapon() {
      uiRight.weaponLON->value(emptyString,1);
      uiRight.weaponLAT->value(emptyString,1);
      uiRight.weaponDepth->value(emptyString,1);
      uiRight.weaponCourse->value(emptyString,1);
      uiRight.weaponAge->value(emptyString,1);
      uiRight.weaponRange->value(emptyString,1);
      uiRight.weaponBearing->value(emptyString,1);
      uiRight.weaponSpeed->value(emptyString,1);
      uiRight.weaponMode->value(emptyString,1);

      uiRight.wnpMenu->label("TRP");
      uiRight.wnpMenu->redraw();

      uiRight.weaponLON->redraw();
      uiRight.weaponLAT->redraw();
      uiRight.weaponDepth->redraw();
      uiRight.weaponCourse->redraw();
      uiRight.weaponAge->redraw();
      uiRight.weaponRange->redraw();
      uiRight.weaponBearing->redraw();
      uiRight.weaponSpeed->redraw();
      uiRight.weaponMode->redraw();
   }

   /* removeWeapon -- This function removes the current selected weapon
   	from the weapon list. */
   void removeWeapon(OID::Type contactID) {
     if(contactID != BAD_OID) {
       bool foundWeap=false;
       int temp = 0;
       int tval=0;
       const Fl_Menu_Item * items = uiRight.wnpMenu->menu();
       while(temp<uiRight.wnpMenu->size()-1) {
	 if (items->user_data_==contactID) {
	   tval=temp;
	   foundWeap=true;
	 }
	 temp++;
	 items=items->next(1);
       }
       if (foundWeap) {
	 uiRight.wnpMenu->remove(tval);
	 if (uiRight.wnpMenu->size()<=1) {
	   clearCurrentWeapon();
	 } else {
	   if (uiRight.wnpMenu->value()>uiRight.wnpMenu->size()-2) {
	     uiRight.wnpMenu->value(uiRight.wnpMenu->size()-2);
	   }
	   WnpMenuItemSelected(uiRight.wnpMenu);
	 }
       }
     }
   }



/*------------------- main --------------------*/

   int main(int argc, char **argv) {

      PreciseContact ownshipData;

   // first thing to do is to set signal handlers, to ensure cleaning up before finishing
      struct sigaction signalHandling;
      signalHandling.sa_handler = cleanUpAndFinish;
      signalHandling.sa_flags = 0;
      sigaction(SIGINT,&signalHandling,NULL);
      sigaction(SIGQUIT,&signalHandling,NULL);
      sigaction(SIGTERM,&signalHandling,NULL);

      ownshipData.reset();

      printf("SubVE Bezel is waiting for a connection to main app...\n");
      fflush(stdout);


      // spin waiting when the shared mem segment is not available
	      while(talkWithMainApp == NULL) {
		               talkWithMainApp = new(UI_SHMEM_ID) UiMediator ;
		                usleep(100000);
	      }


   /************************* Start JAVA stuff ****************************/
    JavaVMOption options[2];
    JavaVMInitArgs vm_args;
        //  JavaVM *jvm;
        //  JNIEnv *env;


  long result;
  jmethodID mid;
  jfieldID fid;
  jobject jobj;
       //  jclass cls;

  int i, asize;

  options[0].optionString = ".";
  options[1].optionString = "-Djava.compiler=NONE";

          vm_args.version = JNI_VERSION_1_4;
	        JNI_GetDefaultJavaVMInitArgs(&vm_args);
		  vm_args.options = options;
		    vm_args.nOptions = 2;
		      vm_args.ignoreUnrecognized = JNI_TRUE;

		        result = JNI_CreateJavaVM(&jvm,(void **)&env, &vm_args);
			  if(result == JNI_ERR ) {
				      printf("Error invoking the JVM");
				          exit (-1);
					    }

			    cls = (env)->FindClass("TimeBearing");
			      if( cls == NULL ) {
				          printf("can't find class ArrayHandler\n");
					      exit (-1);
					        }
			        (env)->ExceptionClear();
				  mid=(env)->GetStaticMethodID(cls,"main","([Ljava/lang/String;)V");

   jobjectArray applicationArgs;
   jstring applicationArg0;

	        //mainMethod = (*env)->GetStaticMethodID(env, helloWorldClass, "main", "([Ljava/lang/String;)V");

    applicationArgs = env->NewObjectArray( 1, (env)->FindClass( "java/lang/String"), NULL);
    applicationArg0 = env->NewStringUTF( "From-C-program");
    env->SetObjectArrayElement( applicationArgs, 0, applicationArg0);

		        //env->CallStaticVoidMethod(cls, mid, applicationArgs);
		
  /************************* End JAVA stuff ****************************/

      Fl_Image *imageMap = new Fl_Image((const unsigned char*)talkWithMainApp->image+(MAP_IMAGE_X*(MAP_IMAGE_Y-1))*3, MAP_IMAGE_X,MAP_IMAGE_Y,3,-MAP_IMAGE_X*3);

      Fl_Group* allWins = create_all();
      imageMap->label(uiRight.mapBox);

      allWins->show();

      // More JAVA
      Fl::wait();
             env->CallStaticVoidMethod(cls, mid, applicationArgs);
	     

      while(running && !talkWithMainApp->readyToQuit() & Fl::wait(0.1f) >= 0) {
         if(talkWithMainApp->isNewMapAvailable()) {
         /* FLTK does not have a way to update an image object; the only thing
              we can do is destroy it and create a new one */
            delete imageMap;
            imageMap = new Fl_Image((const unsigned char*)talkWithMainApp->image+(MAP_IMAGE_X*(MAP_IMAGE_Y-1))*3, MAP_IMAGE_X,MAP_IMAGE_Y,3,-MAP_IMAGE_X*3);
            imageMap->label(uiRight.mapBox);
            uiRight.mapBox->redraw();
            talkWithMainApp->signalMapWasRead();
         }
         if(talkWithMainApp->isContactDataAvailable()) {
            switch(talkWithMainApp->actionFlag & BezelCommand::CONTACT_DATA_AVAILABLE) {
               case BezelCommand::NEW_CONTACT:
                  switch(talkWithMainApp->actionData.contactType) {
                     case CONTACT_IS_OWNSHIP:
                     // get the OID used by the main app to refer to the ownship
                        ownshipID = talkWithMainApp->actionData.contactID;
                     // update bezel
                        ownshipData = talkWithMainApp->actionData.precise;
                        talkWithMainApp->signalContactDataWasRead();
                        updateOwnship(ownshipData);
                        break;
                     case CONTACT_IS_VEHICLE:
		        addNewTarget(talkWithMainApp->actionData.contactID,talkWithMainApp->actionData.name,talkWithMainApp->actionData.precise);
                        talkWithMainApp->signalContactDataWasRead();
                        break;
                     case HYPOTHETICAL_CONTACT:
		        addNewTarget(talkWithMainApp->actionData.contactID,talkWithMainApp->actionData.name,talkWithMainApp->actionData.solution,talkWithMainApp->actionData.hypothetical);
                        talkWithMainApp->signalContactDataWasRead();
			break;
                     case CONTACT_FROM_SENSOR:
		        addNewTarget(talkWithMainApp->actionData.contactID,talkWithMainApp->actionData.name,talkWithMainApp->actionData.solution,talkWithMainApp->actionData.fromSensor);
                        talkWithMainApp->signalContactDataWasRead();
                        break;
                     case CONTACT_IS_WEAPON:
		     	addNewWeapon(talkWithMainApp->actionData.contactID,talkWithMainApp->actionData.name,talkWithMainApp->actionData.weapon);
                        talkWithMainApp->signalContactDataWasRead();
                        break;
                  }
                  break;

            // when updating a contact, we're assuming all info we receive comes from the curretly selected contact
               case BezelCommand::NEW_CONTACT_POSITION:
               case BezelCommand::RESENDING_CONTACT_POSITION:
                  switch(talkWithMainApp->actionData.contactType) {
                     case CONTACT_IS_OWNSHIP:
                        ownshipData = talkWithMainApp->actionData.precise;
                        updateOwnship(ownshipData);
                        break;
                     case CONTACT_IS_VEHICLE:
		        updateCurrentTarget(talkWithMainApp->actionData.precise);
                        break;
                     case HYPOTHETICAL_CONTACT:
		     	updateCurrentTarget(talkWithMainApp->actionData.hypothetical);
			break;
                     case CONTACT_FROM_SENSOR:
		     	updateCurrentTarget(talkWithMainApp->actionData.fromSensor);
                        break;
                     case CONTACT_IS_WEAPON:
		     	updateCurrentWeapon(talkWithMainApp->actionData.weapon);
                        break;
                  }
                  talkWithMainApp->signalContactDataWasRead();
                  break;

               case BezelCommand::REMOVE_CONTACT:
                  switch(talkWithMainApp->actionData.contactType) {
                     case CONTACT_IS_VEHICLE:
                     case CONTACT_FROM_SENSOR:
                     case HYPOTHETICAL_CONTACT:
		     	removeTarget(talkWithMainApp->actionData.contactID);
                        break;
                     case CONTACT_IS_WEAPON:
		     	removeWeapon(talkWithMainApp->actionData.contactID);
                        break;
                  }
                  talkWithMainApp->signalContactDataWasRead();
                  break;
            }

         }
         else if(talkWithMainApp->actionFlag & BezelCommand::SELECT_CONTACT)
	 {
	    targetSelectedInMainApp(talkWithMainApp->actionData.contactID);
            talkWithMainApp->clearFlag(BezelCommand::SELECT_CONTACT);
         }
      }

      delete imageMap;

      if(!talkWithMainApp->readyToQuit()) talkWithMainApp->signalQuit();

      cleanUpAndFinish();

      return 0;
   }



