#ifndef COORD_H
#define COORD_H

/// 2002, Fernando Das-Neves, Virginia Tech UVAG Group

#include <sys/time.h>

   typedef enum { POS = 0 , LOB = 1 } RegionType;

   /*! Coord is a position+orientation in 3D space */
   class Coord {
   public:
      ///xyz position data
      float xyz[3];
      ///hpr is head,pitch and rotation
      float hpr[3];
      ///reset initializes xyz and hpr arrays to 0
      void reset() {
         xyz[0] = xyz[1] = xyz[2] = 0; hpr[0] = hpr[1] = hpr[2] = 0; }
   };

   /*!CoordAndTime is [(x,y,z),(h,p,r)] plus a time value represented as a double   */
   class CoordAndTime: public Coord {
   public:
      timeval time;
      	/// setTime
      void setTime(time_t secs, long usecs = 0) {
         time.tv_sec = secs; time.tv_usec = usecs;}
	/// reset() initializes all time values
      void reset() {
         Coord::reset(); time.tv_sec = 0; time.tv_usec = 0; }
   };

   /*! PreciseContact holds information about a contact with a known (X,Y,Z) position   */
   class PreciseContact: public CoordAndTime {
   public:
   	///known course heading
      float course;
      	///known speed in knots
      float speed;
      	///known speed in knots
      float speed_water;
   };

   typedef enum{
	GO		= 0,
	NO_GO		= 1,
	LAUNCH		= 2,
	RUN_TO_ENABLE 	= 3,
	BBCM_LOCALIZE	= 4,
	BBCM_SEARCH	= 5,
	CLOSE_IN_SEARCH	= 6,
	COVER_SEARCH	= 7,
	DIRECTED_SEARCH = 8,
	INVESTIGATE	= 9,
	TERMINAL_HOMING	= 10,
	PURSUIT		= 11,
	REACQUIRE_SEARCH= 12,
	REATTACK	= 13,
	UNDIRECTED_SEARCH= 14,
        FENCE_DEPLOYMENT = 15,
	SHUTDOWN = 99
   }ModeType;
   
   /*! Weaponcontact holds information about a contact with a known (X,Y,Z) position   */
   class WeaponContact: public CoordAndTime {
   public:
   	///known course heading
      float course;
      	///known speed in knots
      float speed;
      	///known speed in knots
      float speed_water;
      ModeType Mode;
   };

    /*! ContactFromSensor holds information about a contact with a position limited by an ellipse or a bearing wedge.  */
   class ContactFromSensor: public CoordAndTime {
   public:
   	///contact speed returned from sensor
      float speed;
      	///contact course returned from sensor
      float course;
      	///contact speed returned from sensor
      float speed_water;
      	///contact intiation time
      timeval startTime;
      	///contact region type
      RegionType type;
      	///contact ellipse semi-major axis
      float semi_major_axis;
      	///contact ellipse semi-minor axis
      float semi_minor_axis;
      	///contact ellipse angle
      float ellipse_angle;
      	///own-ship latitude
      float os_lat;
      	///own-ship longitude
      float os_lon;
      	///contact bearing
      float bearing;
      	///contact bearing error
      float bearing_err;
	///contact range
      float range;
      	///contact range error
      float range_err;
   };

    /*! HypotheticalContact holds a list of contacts which it is merging  */
   class HypotheticalContact: public ContactFromSensor {
   public:
   	///contact list array is 10x64
     char contactList[10][64];
     	///number of contacts within the array
     int numContacts;
   };

   //! An enum.
   /*! Constants to tag a contact datum with the source it came from.*/
   typedef enum {
   	CONTACT_UNKNOWN      = 0xFF,	/*!Contact unknown*/
	CONTACT_IS_OWNSHIP   = 0,	/*!Contact is OS*/
	CONTACT_IS_VEHICLE   = 1,	/*!Contact is a vehicle*/
	CONTACT_IS_WEAPON    = 2,	/*!Contact is a weapon*/
	CONTACT_FROM_SENSOR  = 3,	/*!Contact from sensor*/
	HYPOTHETICAL_CONTACT = 4,	/*!Hypothetical contact*/
	CONTACT_FROM_WEAPON  = 5
	}
	ContactType;


#endif
