#include<string>
#include <iostream>
#include <fstream>
#include <list>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>
#include <limits.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <signal.h>
#include <string.h>
#include "ContactEvent.h"
#include "FeederConnection.h"
#include "FeederCommand.h"
#include "timeval_ops.h"

   using namespace std;

   /// All steps from one track file
   typedef list<ContactEvent*> FeederTrack;

   /// All tracks from all track files
   typedef list<FeederTrack*> FeederTrackList;

   /// structure used to keep info about a track and a current position in the track
   struct FeederListInfo {
      FeederTrackList::iterator track_it;
      FeederTrack::iterator iter,end;
   };

   int NO_TRACKS = -9999;



   int readTracks(const char* const feederFile, FeederTrackList& tracks) {
      #define STEP_BUFFER_SIZE 1024
      char step[STEP_BUFFER_SIZE];
      const char *separators = " \t,;";
      ifstream files(feederFile);
      if(!files) {
         cout << "ERROR: Can't open or find \'" << feederFile << "\' containing the list of tracks." << endl;
         return NO_TRACKS;
      }
      int track_nro = 1, line_nro = 0,
         failed_tracks = 0;
      string trackname;
      ifstream trackFile;
      // read each track, from filenames in feederFile
      files >>  trackname;
      do {
         ContactEvent* evt;
         //files >>  trackname;
         string::iterator sit = trackname.begin();
        // skip leading spaces in line containing file name
         while(sit != trackname.end() && (*sit == ' ' || *sit == '\t')) ++sit;
         if(sit != trackname.end()) {
            trackFile.open(trackname.c_str() + (trackname.begin() - sit));
            if(trackFile) {
               char *token;
               FeederTrack *track = new FeederTrack;
               while(trackFile) {
                  ++line_nro;
                  trackFile.getline(step,STEP_BUFFER_SIZE);
                  evt = new ContactEvent;
                  token = strtok(step, separators);
                  if(token ==NULL) // this is an empty line, skip it
                     continue;
                  evt->name = token;
                  //token = strtok(NULL,separators);
                  //evt->iconID = atoi(token);
                  if(strncasecmp(evt->name.c_str(),"OS",2) == 0 || strncasecmp(evt->name.c_str(),"OWNSHIP",7) == 0)
                     evt->contactType = CONTACT_IS_OWNSHIP;
                  else if(toupper(*evt->name.c_str()) == 'W')
                     evt->contactType = CONTACT_IS_WEAPON;
                  else if(toupper(*evt->name.c_str()) == 'S')
                     evt->contactType = CONTACT_FROM_SENSOR;
                  else if(toupper(*evt->name.c_str()) == 'H')
                     evt->contactType = HYPOTHETICAL_CONTACT;
                  else
                     evt->contactType = CONTACT_IS_VEHICLE;
                  switch(evt->contactType) {
		     // here starts a long, boring sequence of item reading and error checking...
                     case CONTACT_IS_OWNSHIP:
                     case CONTACT_IS_VEHICLE:
		       //no such thing for new sensor
		       token = strtok(NULL,separators);
		       //evt->iconID = atoi(token);
                        if(token == NULL)
                           goto skipLine;
			if (!strncasecmp(token,"HOS",3)) evt->threat=HOSTILE;
			else if (!strncasecmp(token,"FRD",3)) evt->threat=FRIENDLY;
			else evt->threat=UNKNOWN;
			//category
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
			if (!strncasecmp(token,"SUB",3)) evt->category=SUB;
			else if (!strncasecmp(token,"NAV",3)) evt->category=NAV;
			else if (!strncasecmp(token,"FSH",3)) evt->category=FSH;
			else if (!strncasecmp(token,"MER",3)) evt->category=MER;
			else if (!strncasecmp(token,"SUR",3)) evt->category=SUR;
			else if (!strncasecmp(token,"LND",3)) evt->category=LND;
			else if (!strncasecmp(token,"AIR",3)) evt->category=AIR;
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.time.tv_sec = atol(token);
                        evt->coord.precise.time.tv_usec = 0;
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.xyz[1] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.xyz[0] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.xyz[2] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.course = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.speed = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.hpr[0] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.speed_water = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.hpr[1] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.precise.hpr[2] = atof(token);
			break;
                     case CONTACT_IS_WEAPON:
		       //no such thing for new sensor
		       token = strtok(NULL,separators);
		       //evt->iconID = atoi(token);
                        if(token == NULL)
                           goto skipLine;
			if (!strncasecmp(token,"HOS",3)) evt->threat=HOSTILE;
			else if (!strncasecmp(token,"FRD",3)) evt->threat=FRIENDLY;
			else evt->threat=UNKNOWN;
			//category
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
			if (!strncasecmp(token,"SUB",3)) evt->category=SUB;
			else if (!strncasecmp(token,"NAV",3)) evt->category=NAV;
			else if (!strncasecmp(token,"FSH",3)) evt->category=FSH;
			else if (!strncasecmp(token,"MER",3)) evt->category=MER;
			else if (!strncasecmp(token,"SUR",3)) evt->category=SUR;
			else if (!strncasecmp(token,"LND",3)) evt->category=LND;
			else if (!strncasecmp(token,"AIR",3)) evt->category=AIR;
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.time.tv_sec = atol(token);
                        evt->coord.weapon.time.tv_usec = 0;
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.xyz[1] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.xyz[0] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.xyz[2] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.course = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.speed = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.hpr[0] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.speed_water = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.hpr[1] = atof(token);
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.weapon.hpr[2] = atof(token);
                        token = strtok(NULL,separators);
			if(token != NULL)
				evt->coord.weapon.Mode = (ModeType)atoi(token);
			break;
                     case CONTACT_FROM_SENSOR:
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
			evt->solution = token;
			//evt->name.append(evt->solution);
			//threat
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
			if (!strncasecmp(token,"HOS",3)) evt->threat=HOSTILE;
			else if (!strncasecmp(token,"FRD",3)) evt->threat=FRIENDLY;
			else evt->threat=UNKNOWN;
			//category
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
			if (!strncasecmp(token,"SUB",3)) evt->category=SUB;
			else if (!strncasecmp(token,"NAV",3)) evt->category=NAV;
			else if (!strncasecmp(token,"FSH",3)) evt->category=FSH;
			else if (!strncasecmp(token,"MER",3)) evt->category=MER;
			else if (!strncasecmp(token,"SUR",3)) evt->category=SUR;
			else if (!strncasecmp(token,"LND",3)) evt->category=LND;
			else if (!strncasecmp(token,"AIR",3)) evt->category=AIR;
			//time
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
                        evt->coord.fromSensor.time.tv_sec = atol(token);
                        evt->coord.fromSensor.time.tv_usec = 0;
                        evt->coord.fromSensor.startTime.tv_sec = atol(token);
                        evt->coord.fromSensor.startTime.tv_usec = 0;
			// POS/LOB
                        token = strtok(NULL,separators);
                        if(token == NULL)
			  goto skipLine;
			if((toupper(*token) == 'L')) {
			  evt->coord.fromSensor.type=LOB;
			  evt->coord.fromSensor.xyz[0] =evt->coord.fromSensor.xyz[1] =0.0;
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.os_lat = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.os_lon = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.xyz[2] = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.course = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.speed = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.hpr[0] = atof(token);
			  evt->coord.fromSensor.hpr[1] = evt->coord.fromSensor.hpr[2] = 0;
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.speed_water = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.bearing = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.bearing_err = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.range = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.range_err = atof(token);

			} else if((toupper(*token) == 'P')) {
			  evt->coord.fromSensor.type=POS;
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.xyz[1] = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.xyz[0] = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.xyz[2] = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.course = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.speed = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.hpr[0] = atof(token);
			  evt->coord.fromSensor.hpr[1] = evt->coord.fromSensor.hpr[2] = 0;
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.speed_water = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.semi_major_axis = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.semi_minor_axis = atof(token);
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  evt->coord.fromSensor.ellipse_angle = atof(token);


			} else {
			  goto skipLine;
			}
			break;
		      case HYPOTHETICAL_CONTACT:
                        token = strtok(NULL,separators);
                        if(token == NULL)
                           goto skipLine;
			evt->solution = token;
                        token = strtok(NULL,separators);
                        if(token == NULL)
			  goto skipLine;
                        evt->coord.hypothetical.time.tv_sec = atol(token);
                        evt->coord.hypothetical.time.tv_usec = 0;
                        evt->coord.fromSensor.startTime.tv_sec = atol(token);
                        evt->coord.fromSensor.startTime.tv_usec = 0;
                        token = strtok(NULL,separators);
                        if(token == NULL)
			  goto skipLine;
                        evt->coord.hypothetical.numContacts = atoi(token);
			for (int k=0;k<evt->coord.hypothetical.numContacts;k++) {
			  token = strtok(NULL,separators);
			  if(token == NULL)
			    goto skipLine;
			  strncpy(evt->coord.hypothetical.contactList[k],token,100);
			}
			break;
                  }
                  track->push_back(evt);
                  continue;
               skipLine:
                  cout << trackname <<": Wrong or imcomplete data, line " << line_nro << " skipped" << endl;
               }
               // add track only if there was something in it
               if(track->size() != 0)
                  tracks.push_back(track);
               else
                  delete track;
               trackFile.close();
               trackFile.clear();
            }
            else {
               failed_tracks++;
               cout << "ERROR: Can't open track #" << track_nro << " named '" << trackname << "' in " << feederFile << ". Skipping..." << endl;
            }
         }

         track_nro++;
	 files >>  trackname;
      } while(files);
      files.close();
      return failed_tracks;
   }


   bool makeMeADaemon() {
      pid_t pid;
      if((pid = fork()) < 0)
         return false;
      else if(pid != 0) {
         exit(0); // parent goes bye-bye
      }
   /* child continues */
      setsid();
      chdir("/");
      umask(0);
      close(STDIN_FILENO);
      close(STDOUT_FILENO);
      close(STDERR_FILENO);
      return true;
   }

   void mergeAllTracks(FeederTrackList& tracks, FeederTrack& unifiedTrack) {
      ContactEvent evt;
      timeval minTime;
      FeederListInfo li;
      list<FeederListInfo> itlist; // list of {iterator, end, contact name, contact type}
      list<FeederListInfo>::iterator min_list, itlist_it;

      for(FeederTrackList::iterator list_it = tracks.begin(); list_it != tracks.end(); ++list_it) {
         li.track_it = list_it;
         li.iter = (*list_it)->begin();
         li.end  = (*list_it)->end();
         itlist.push_back(li);
      }

      while(!itlist.empty()) {
      // find event with earliest time from next event in each track
         minTime.tv_sec = INT_MAX;
         minTime.tv_usec = INT_MAX;
         for(itlist_it = itlist.begin(); itlist_it != itlist.end(); ++itlist_it) {
            if((*(itlist_it->iter))->contactType == CONTACT_FROM_SENSOR) {
               if((*(itlist_it->iter))->coord.fromSensor.time < minTime) {
                  min_list = itlist_it;
                  minTime = (*(itlist_it->iter))->coord.fromSensor.time;
               }
            }
            else {
               if((*(itlist_it->iter))->coord.precise.time < minTime) {
                  min_list = itlist_it;
                  minTime = (*(itlist_it->iter))->coord.precise.time;
               }
            }

         }
         unifiedTrack.push_back(*(min_list->iter));
         ++(min_list->iter);
      /* if this was the last event of track track,
       remove the track & the track info from the track list */
         if(min_list->iter == min_list->end) {
            tracks.erase(min_list->track_it);
            itlist.erase(min_list);
         }
      }
   }

   FeederConnection connection;

   void sigHandler(int) {
      connection.closeConnection();
      exit(1);
   }

   int main(int argc, char * argv[]) {
      FeederTrackList tracks;
      FeederTrack evtList;
      char *config;
      int currentArg = 1;
      float accelFactor = 1.0f;
      while(argv[currentArg] != NULL && *argv[currentArg] == '-') {
         switch(argv[currentArg][1]) {
            case 'a':
               currentArg++;
               if(*argv[currentArg])
                  accelFactor = atof(argv[currentArg]);
               break;

         }
         currentArg++;
      }
   	// if there is one last argument, it has to be the list file name
      if(argv[currentArg] == NULL) {
         config = "feeder.list";
      }
      else {
         config = argv[currentArg];
      }
      int failed_tracks = readTracks(config,tracks);
      if(failed_tracks  == NO_TRACKS) {
         cout << "Exiting." << endl;
         exit(-1);
      }
      if(failed_tracks != 0) {
         char answer;
         cout << "WARNING: " << failed_tracks <<  ' ';
         if(failed_tracks == 1)
            cout << "track";
         else
            cout << "tracks";
         cout << " could not be read from '" << config << "' continue (y/n)? ";
         cin >> answer;
         if(tolower(answer) != 'y') {
            exit(2);
         }
      }

   // set signal handlers to ensure a nice cleanup
      signal(SIGTERM,sigHandler);
      signal(SIGINT,sigHandler);
   // SIGPIPE will be sent by the OS when the other end of the socket dies (broken pipe)
      signal(SIGPIPE,sigHandler);

      if(connection.startFeeder() < 0) {
         cout << "\nFeeder ERROR: Can't start connection, maybe some other application is using a file\n called  " << FeederConnection::SOCK_ADDRESS << ". Exiting.\n" << endl;
         exit(2);
      }
      mergeAllTracks(tracks,evtList);
      cout << "Total events to feed: " << evtList.size() << endl;
      if(accelFactor != 1.0f) cout << "Playing events with time accelerated " << accelFactor << " times" << endl;
      cout.flush();
   /*** NOTE: Uncomment this if you want the feeder to work as a Unix daemon ***
      makeMeADaemon();
   ****/
   /* If this program is running now as a daemon,
      from here on it won't be able to write
      to stdin, stdout and stderr */
      FeederTrack::iterator nextEvent = evtList.begin();
      timeval startTime, currentTime, timeOfFirstEvent, timeOfNextEvent;
      timeval stopTime, // time when the feeder was stopped
              delayWhileStopped(TIME_ZERO); // total time accumulated while the feeder has been stopped
      // get time of earliest event
      if((*nextEvent)->contactType == CONTACT_FROM_SENSOR)
         timeOfFirstEvent = (*nextEvent)->coord.fromSensor.time;
      else
         timeOfFirstEvent = (*nextEvent)->coord.precise.time;
      bool feederIsRunning = true; // flag that indicates if the feeder is not stopped, at the beginning the feeder is not stopped
      bool feederIsAlive = true; // and will stay true until somebody sends the QUIT command to the feeder
      unsigned char command; // will be used to receive commands, like stop, resume and quit
      gettimeofday(&startTime,NULL);
      timeOfNextEvent = startTime; // cheat, so first event plays immediately
      while(feederIsAlive) {
	 if(feederIsRunning) {
            gettimeofday(&currentTime,NULL);
	    currentTime-=delayWhileStopped;
            while(nextEvent != evtList.end() && currentTime >= timeOfNextEvent) {
              connection.sendEvent(**nextEvent);
              ++nextEvent;
              if(nextEvent != evtList.end()) {
                 if((*nextEvent)->contactType == CONTACT_FROM_SENSOR)
                    timeOfNextEvent = (*nextEvent)->coord.fromSensor.time;
                 else
                    timeOfNextEvent = (*nextEvent)->coord.precise.time;
            	  // scale time and slide it appropiately with respect to current time
                 timeOfNextEvent-=timeOfFirstEvent;
                 timeOfNextEvent/=accelFactor;
                 timeOfNextEvent+=startTime;
              }
           }
	 }
         if(connection.isDataAvailable()) {
	     connection.readByte(command);
	     switch(command) {
  	       case FeederCommand::STOP_FEEDER:
	         if(feederIsRunning) { // this is to ignore more than one consecutive STOP_FEEDER commands
                   gettimeofday(&stopTime,NULL);
		   feederIsRunning = false;
	         }
	         break;
	       case FeederCommand::RESUME_FEEDER:
	         if(!feederIsRunning) { // this is to ignore more than one consecutive RESUME_FEEDER commands
                   gettimeofday(&currentTime,NULL);
                   currentTime-=stopTime;
		   delayWhileStopped+=currentTime;
	           feederIsRunning = true;
	         }
	         break;
	       case FeederCommand::QUIT:
                 feederIsAlive = false;
	         break;
	     }
	 }
         usleep(50000); // up to 20 times per sec.
      }
      connection.closeConnection();
   }
