// This program takes an iv file and changes all colors to gray //
// Fernando Das-Neves, Virginia Tech, 2002                      //

#include <Inventor/SoInput.h>
#include <Inventor/SoOutput.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/SoDB.h>
#include <stdio.h>

void makeMFColorGray(SoMFColor& mfcolor) {
  int i;
  float *clr, gray;
  SbColor *colors = mfcolor.startEditing();
  for(i=0; i<mfcolor.getNum(); i++) {
    clr = (float*)colors->getValue();
    gray = (clr[0]+clr[1]+clr[2])/3.0f;
    colors->setValue(gray,gray,gray);
    colors++;
  }
  mfcolor.finishEditing();
}


SoCallbackAction::Response changeMaterialColorToColor(void*,SoCallbackAction*, const SoNode* mat) {
  makeMFColorGray(((SoMaterial*)mat)->ambientColor);
  makeMFColorGray(((SoMaterial*)mat)->diffuseColor);
  makeMFColorGray(((SoMaterial*)mat)->specularColor);
  makeMFColorGray(((SoMaterial*)mat)->emissiveColor);
  return SoCallbackAction::CONTINUE;
}

SoCallbackAction::Response changeBaseColorToColor(void*,SoCallbackAction*, const SoNode* baseColor) {
  makeMFColorGray(((SoBaseColor*)baseColor)->rgb);
  return SoCallbackAction::CONTINUE;
}

void printUsage() {
  printf("USAGE: ivingray inventor_file output_name\n");
}

int main(int argc, char** argv) {
  SoDB::init();
  SoSeparator *root;  
  SoInput loader;
  SoWriteAction writer;
  SoCallbackAction traversingAction;

  if(argc != 3) {
    printUsage();
    return 1;
  }
  traversingAction.addPostCallback(SoBaseColor::getClassTypeId(),changeBaseColorToColor,NULL);
  traversingAction.addPostCallback(SoMaterial::getClassTypeId(),changeMaterialColorToColor,NULL);
  
   if(loader.openFile(argv[1]) == FALSE) {
    printf("Couldn't open '%s'\n",argv[1]);
    return 2;
  }

  root = SoDB::readAll(&loader);
  loader.closeFile();
  if(root == NULL) {
    printf("Couldn't read a valid inventor object from file '%s'\n",argv[1]);
    return 3;
  }

  root->ref();
  traversingAction.apply(root);

  if(writer.getOutput()->openFile(argv[2]) == FALSE) {
    printf("Couldn't write result to '%s'\n",argv[1]);
    return 4;
  }
  writer.apply(root);
  writer.getOutput()->closeFile();
}
