#ifndef _OBSERVER_H
#define _OBSERVER_H

// 2002, Virginia Tech UVAG Group //


/*! Here you will find two base classes to simple implement of the Observer pattern.
   For more infomation regarding the Observer pattern, read "Design Patterns" by
   Gamma et al, or go to http://www.inf.fu-berlin.de/~bokowski/Observer.html .
*/

class Observable;


/*! An Observer is an object that can be notified of changes in an Observable object,
and when that happens it will expect its update() method to be called. 
Take a look at the Observer pattern at http://www.inf.fu-berlin.de/~bokowski/Observer.html . */
class Observer {
public:
  virtual void setModel(Observable* aModel) =0;
  virtual void update(void *info) =0;
};

/*! An observable is an object that can be watched by one or more Observers.
When an observable changes, it tells all its observers of the change by calling
its notify() method, which has an optional info parameter that can
be used to pass data to the Observables.
Take a look at the Observer pattern at http://www.inf.fu-berlin.de/~bokowski/Observer.html  . */
class Observable {
public:
  virtual void addView(Observer *anObserver) { anObserver->setModel(this); }
  virtual void notify(void *info) =0;
  virtual void removeView(Observer *) { }
};

#endif
