#include <Inventor/Xt/SoXt.h>
#include <signal.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/IntrinsicP.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtRenderArea.h>
#include <Xm/Form.h>
#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/MainW.h>
#include <Inventor/nodes/SoLightModel.h>
#define USE_LIGHT
#ifdef USE_LIGHT
	#include <Inventor/nodes/SoDirectionalLight.h>
	#include <Inventor/nodes/SoEnvironment.h>
	#include <Inventor/nodes/SoPointLight.h>
#endif
#include <Inventor/nodes/SoMatrixTransform.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/nodes/SoScale.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoLOD.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/SoInput.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/actions/SoGetBoundingBoxAction.h>
#include <Inventor/nodes/SoCoordinate3.h>

// Used to add a texture on top of the map, currently the grid
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTextureCoordinatePlane.h>
#include <Inventor/nodes/SoTexture2Transform.h>

// Used to render the little map in the bezel
#include <Inventor/SoOffscreenRenderer.h>
#include <Inventor/nodes/SoOrthographicCamera.h>

// Used to keep map and icon movement coordinated
#include <Inventor/engines/SoCalculator.h>

// object-picking headers
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/SoPath.h>
#include <Inventor/SoPickedPoint.h>
#include <Inventor/actions/SoRayPickAction.h>

#include "VoidPtrList.h"
#include <map>

#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoPickStyle.h>

#ifdef __linux__
  #include <values.h>
#else
  #include <limits.h>
#endif

// next 2 only used to debug the minimap
#if DBG_SAVE_MAP_IMAGE_TO_DISK
  #include <Inventor/SoOutput.h>
#endif

/* X11 stuff + keyboard handling */
#include <X11/Intrinsic.h>
#include <X11/Xlib.h>
#define XK_LATIN1 // get Latin-1 (iso 8859-1) keysyms
#define XK_MISCELLANY // get keypad keysyms
#include <X11/keysymdef.h>

// TALOSS headers
#include "FeederConnection.h"
#include "FeederCommand.h"
#include "ContactEvent.h"
#include "timeval_ops.h"
#include "track.h"
#include "trackview.h"
#include "UiMediator.h"
#include "oid.h"
#include "TrackUiView.h"
#include "bezel/ui_area_sizes.h"
#include "DiverseHUD.h"
#include "IvColorScale.h"
#include "IvCompass.h"
#include "IvTiledBathymetry.h"
#include "IvHighlightSelection2D.h"
#include "IvTransformEngine.h"
#include "IvContactLabeler2D.h"
#include "NameToTrackMapping.h"
#include "ContactIconCollection3D.h"
#include "IvContactLabeler2D.h"
#include "degree_conv.h"
#include "bezel/ui_constants.h"
#include "degree_conv.h"

// DGL headers
#include <eventmgr/devents.h>
#include <dgl/dglcore/DGL.h>
#include <dgl/libdgl/cdgl.h>
#include <displaymgr/DisplayManager.h>
#include <displaymgr/DDisplay.h>
#include <navmgr/NavManager.h>
#include <dmath/dMatrix.h>
#include <dmath/dQuat.h>
#include <dmath/Rect.h>
#include <Producer/RenderSurface>
#include <displaymgr/DWindow.h>
#include <displaymgr/DisplayConfig.h>
#include <eventmgr/EventManager.h>

/* xwand headers - control X11 cursor with a wand - 
   only makes sense in a CAVE, idesk or something like that. */
#include <X11/cursorfont.h>
#include "xwand/invisibleCursor.h"
#include "xwand/XWandFakeMouse.h"



using namespace std;


// Object used to talk to the data feeder
static FeederConnection feeder;


// Object to talk to the GUI
static UiMediator *talkWithBezel = NULL;


// These are the colors used for contact labels
static const float OWNSHIP_LABEL_COLOR[3] = { 0.5   , 0.5,    1.0f };
static const float CONTACT_LABEL_COLOR[3] = { 1.0f,   0.5,     0.5 };
static const float WEAPON_LABEL_COLOR[3]  = { 0.6   , 1.0f,    0.6 };


/* This is the name of the file that contains the list of filenames for 3D icons. 
   The first icon in the list is icon #0, next is icon #1 and so forth. */
static const char* const CONTACT_ICONLIST_FILE = "contactIcon.list";

// if FAST_TRANSPARENCY is defined, then we use Inventor's faster but not-so-nice way way to render translucient object
//#define FAST_TRANSPARENCY

// maximum number of contact events sent by the feeder that we check for in one pass
#define MAX_SIMULTANEOUS_CONTACT_EVENTS 3

/* Boundaries of the total area possibly covered by the tileset. This is NOT
   the area covered by the current tileset, but the area for which the
   lowest left tile will be tile #0. */
#define MAP_MIN_LAT  -90.0f
#define MAP_MIN_LON -180.0f
#define MAP_MAX_LAT   90.0f
#define MAP_MAX_LON  180.0f

// width and length of a map tile in degrees - all tiles are assumed to be of equal size, except the tiles at the edge of the map
#define TILE_SIZE_X 0.5f
#define TILE_SIZE_Y 0.5f

// minimum distance between onwship and an edge in the visible area of the map before we 'slide' the map to show more tiles
#define EDGE_THRESHOLD (TILE_SIZE_X/5.0f)

#define TILE_SCALING_TO_METERS_X DEGREES_TO_METERS(1.0f)
#define TILE_SCALING_TO_METERS_Y DEGREES_TO_METERS(1.0f)

// This is the exaggeration applied to map heights . Make it a dynamically changeable variable
float TILE_EXAGGERATION_Z= 16.0f;


// this is the deepest point we expect contacts (depth of AOU/wedges) in feet
#define BATHYMETRY_MAX_DEPTH -1000.0f
#define AOU_MAX_DEPTH -300.0f

#define VERYSHALLOW_THRESHOLD 400.0f
#define SHALLOW_THRESHOLD 1000.0f
#define MIDRANGE_THRESHOLD 8000.0f
#define DEEP_THRESHOLD 16000.0f

// This is the  default radius in X,Y of the map area to show around the ownship
#define DEFAULT_MAP_RADIUS_AROUND_X (TILE_SIZE_X*0.5f)
#define DEFAULT_MAP_RADIUS_AROUND_Y (TILE_SIZE_Y*0.5f)


/* give a more explicit name to xwand application events: */
// event 8 = print current nav position
#define APP_ACTION_IS_PRINT_CURRENT_POS XW_APP_EVENT_8
// event 9 = quit this program
#define APP_ACTION_IS_QUIT XW_APP_EVENT_9



/* This type defines the ways the visible area of bathymetry can be updated: 
   FREE_MAP_BROWSING = wherever the user wants to go,
   MAP_FOLLOWS_OWNSHIP = bathymetry changes to keep the onwship on the map
*/
   typedef enum { FREE_MAP_BROWSING, MAP_FOLLOWS_OWNSHIP } MapUpdatingMode;

// This is the height of the isometric camera over the minimap; this value tells how much of the minimap is visible
#define CAMERA_HEIGHT_IN_MINIMAP 18200.0f

//determins whether or not region growth is on
   bool GROW_REGIONS=false;

/* These are  global objects so that the vertical exaggeration slider can use them to change the Z scale,
   and the wireframe drawing mode can be turned on or off */
   static   SoDrawStyle *bathymetryStyle;

   NameToTrackMapping trackDirectory; // creates tracks, and keeps track of mapping from from track names to track objects



/// a struct used in subve.c++ to pass all selection objects to callbacks
   struct SelectionBoxes {
      /// currently selected target in render area
      IvHighlightSelection2D *targetSelection;
      /// currently selected weapon in render area
      IvHighlightSelection2D *weaponSelection;
   };


/*! This class is a queue of ContactEvents. We use it to keep and read events 
    generated in this program, instead of coming from an outside source. */
   class ContactEventQueue {
       bool eventAvailable;
       ContactEvent theEvent;
   public:
       void addEvent(const ContactEvent& newEvent) { theEvent = newEvent; eventAvailable=true; }
       void readEvent(ContactEvent& anEvent) { if(eventAvailable) { anEvent=theEvent; eventAvailable=false; } }
       ContactEventQueue():eventAvailable(false) { }
       bool isDataAvailable() const { return eventAvailable; }
   };


/*! This class slightly modifies the behavior of IvHighlightSelection2D in updateBbox(), 
    to handle the case where the desired higlighted object is marked 'invisible' */
   class IvContactViewHighlighter2D: public IvHighlightSelection2D {
   public:
     /// Creates a  selection object, that will work with a certain DiverseHUD's render area
      IvContactViewHighlighter2D(DiverseHUD* theHUD):IvHighlightSelection2D(theHUD) {
      }
      virtual void updateBbox();
   };

   void IvContactViewHighlighter2D::updateBbox() {
      IvContactView* cview = (IvContactView*)getSelectedObject();
      if(cview != NULL) {
         if(cview->isVisible()) {
            show();
            IvHighlightSelection2D::updateBbox();
         } 
         else {
            hide();
         }
      }
   }

   void saveSceneGraph(SoNode* rootNode,const char* prefixFilename=NULL) {
       const int MAX_LEN_FILENAME=256;
       char filename[MAX_LEN_FILENAME];
       SoWriteAction action;
       time_t now;
       struct tm *timeval;
       time(&now);
       timeval = gmtime(&now);
       if(prefixFilename==NULL)
           strftime(filename,MAX_LEN_FILENAME-1,"%Y-%m-%d_%H%M%S.iv",timeval);
       else {
           unsigned int lenprefix = strlen(prefixFilename);
           strncpy(filename,prefixFilename,lenprefix);
           strftime(filename+lenprefix,MAX_LEN_FILENAME-1-lenprefix,"%Y-%m-%d_%H%M%S.iv",timeval);
       }
       SoSeparator* falseRoot = new SoSeparator;
       falseRoot->ref();
       SoSeparator* camSep = new SoSeparator;
       falseRoot->addChild(camSep);
       camSep->setName("Camera_Aim");
       SoTranslation *moveCam = new SoTranslation;
       camSep->addChild(moveCam);
       dMatrix NavMatrix = NavManager::getMatrix();
       dVec3 vec = NavMatrix.getTrans();
       moveCam->translation.setValue(vec[0],vec[1],vec[2]);
       //vec = NavManager::getDirection();
       SoCoordinate3 *pts = new SoCoordinate3;
       camSep->addChild(pts);
       pts->point.set1Value(0,0,0,0);
       pts->point.set1Value(1,0,0,-1000);
       SoLineSet *line = new SoLineSet;
       camSep->addChild(line);
       line->numVertices.setValue(2);
       line->startIndex.setValue(0);
       falseRoot->addChild(rootNode);
       action.getOutput()->openFile(filename);
       action.getOutput()->setBinary(FALSE);
       action.apply(falseRoot);
       action.getOutput()->closeFile();
       falseRoot->removeChild(rootNode);
       falseRoot->unref();
   }


   void turnWindowBorderOffPreCallback() {
       vector<DDisplay*> dglDisplays = DisplayManager::getDisplays();
       vector<DWindow*> dglWindows = dglDisplays[0]->getWindows();
       dglWindows[0]->setBorder(false);
   }

   typedef 
      struct {
         SbViewportRegion viewport;
         SoNode * rootNode;
	 SoGLRenderAction *renderer;
         float cameraAspectRatio; // display's field of view, in radians
         unsigned int windowLeft,windowBottom;
      } RenderingData;

   /// This is a global variable that keeps all the data needed by the rendering callback
   RenderingData mainWindowRenderingData;


   SoSeparator* xxxroot;


void drawSceneGraphCallback()
{
      glEnable(GL_DEPTH_TEST);
      glMatrixMode(GL_MODELVIEW);
      glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
      glPushMatrix();
      glRotatef(90.0f,1.0f,0.0f,0.0f);     
      GLint vp[4];
      glGetIntegerv(GL_VIEWPORT, vp);
      SbViewportRegion viewport;
      viewport.setWindowSize(vp[2], vp[3]);
      SoGLRenderAction renderer(viewport);
      renderer.apply(xxxroot);
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();
}


   // This is the function that draws the scene - called by Diverse in dglFrame()
/*   void drawSceneGraphCallback() {

      glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
      glRotatef(90.0f,1.0f,0.0f,0.0f);     
      GLint vp[4];
      glGetIntegerv(GL_VIEWPORT, vp);
      SbViewportRegion viewport;
      viewport.setWindowSize(vp[2], vp[3]);
     mainWindowRenderingData.renderer->setViewportRegion(viewport);
      mainWindowRenderingData.renderer->apply(mainWindowRenderingData.rootNode);
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();
   }
*/
   void changeColorscaleDepth(IvTiledBathymetry *b, IvColorScale* colorscale, float d) {
      b->setColorscaleDepth(0,FT_TO_M(d),64);
      colorscale->setRange(-d,0,9);
   }

   void setBathymetryToWireframe(SoDrawStyle* wireframe) {
      wireframe->style = SoDrawStyle::LINES;
   }

   /* This function is used to change the bathymetry to solid mode */
   void setBathymetryToSolid(SoDrawStyle* wireframe) {
      wireframe->style = SoDrawStyle::FILLED;
   }

   typedef enum { LATITUDE,LONGITUDE } GeoCoordinateType;
   void makeLatLonString(char *charArray, float latLonValue, GeoCoordinateType coordType) {
     char dir;
     if (latLonValue>=0) {
       dir=(coordType==LATITUDE?'N':'E');
     } else {
       latLonValue=-latLonValue;
       dir=(coordType==LATITUDE?'S':'W');
     }
     sprintf(charArray,"%1.2f%c\0\0",latLonValue,dir);     
   }

   /*this routine adds placement markers to the map*/
   SoGroup* createLocationMarkers(const SbBox3f& bbox) {
     register unsigned int i,j,xsteps,ysteps;
     SoText2 *labelObject;
     SoTranslation *trans;
     SoTransformSeparator *sep;
     SoLOD *details;
     SoGroup *detailGroup;
     const float steps[] = {0.166666667,0.333333333,0.5};
     const unsigned int numsteps = 3;
     SoGroup* markerGroup = new SoGroup;
     
     //make the outer markings
     SoFont *font = new SoFont();
     font->name.setValue("defaultFont");
     font->size.setValue(18);
     markerGroup->addChild(font);
     SoMaterial *mat = new SoMaterial;
     mat->diffuseColor.setValue(0,0,0);
     markerGroup->addChild(mat);

     sep = new SoTransformSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     trans->translation.setValue(bbox.getMin().getValue()[0],(bbox.getMin().getValue()[1]+bbox.getMax().getValue()[1])*0.5,20);
     labelObject = new SoText2;
     sep->addChild(labelObject);
     labelObject->justification = SoText2::RIGHT;

     sep = new SoTransformSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     labelObject = new SoText2;
     sep->addChild(labelObject);
     labelObject->justification = SoText2::LEFT;

     sep = new SoTransformSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     labelObject = new SoText2;
     sep->addChild(labelObject);
     labelObject->justification = SoText2::CENTER;

     sep = new SoTransformSeparator;
     markerGroup->addChild(sep);
     trans = new SoTranslation;
     sep->addChild(trans);
     labelObject = new SoText2;
     sep->addChild(labelObject);
     labelObject->justification = SoText2::CENTER;

     details=new SoLOD;
     markerGroup->addChild(details);
     details->center.setValue((bbox.getMin().getValue()[0]+bbox.getMax().getValue()[0])*0.5,(bbox.getMin().getValue()[1]+bbox.getMax().getValue()[1])*0.5,20);
     details->range.set1Value(0,10000);
     details->range.set1Value(1,80000);
     for (unsigned int stepIndex = 0; stepIndex<numsteps; stepIndex++) {
       float step = steps[stepIndex];
       detailGroup = new SoGroup;
       details->addChild(detailGroup);
       xsteps=int((bbox.getMax().getValue()[0]-bbox.getMin().getValue()[0])/step-0.1);
       ysteps=int((bbox.getMax().getValue()[1]-bbox.getMin().getValue()[1])/step-0.1);

       for (i=0;i<xsteps; i++) {
	     sep = new SoTransformSeparator;
	     detailGroup->addChild(sep);
	     trans = new SoTranslation;
	     sep->addChild(trans);     
	     labelObject = new SoText2;
	     sep->addChild(labelObject);
	     labelObject->justification = SoText2::CENTER;
       }

       for (j=0;j<ysteps; j++) {
	     sep = new SoTransformSeparator;
	     detailGroup->addChild(sep);
	     trans = new SoTranslation;
	     sep->addChild(trans);     
	     labelObject = new SoText2;
	     sep->addChild(labelObject);
	     labelObject->justification = SoText2::CENTER;
       }

       for (i=0; i<xsteps; i++) {
           for (j=0; j<ysteps; j++) {
	       sep = new SoTransformSeparator;
	       detailGroup->addChild(sep);
	       trans = new SoTranslation;
	       sep->addChild(trans);     
	       labelObject = new SoText2;
	       sep->addChild(labelObject);
	       labelObject->justification = SoText2::CENTER;
	       
	       sep = new SoTransformSeparator;
	       detailGroup->addChild(sep);
	       trans = new SoTranslation;
	       sep->addChild(trans);     
	       labelObject = new SoText2;
	       sep->addChild(labelObject);
	       labelObject->justification = SoText2::CENTER;
           }
       }
     }
     return markerGroup;
   }



/*this routine assign values to the  placement markers on the map*/
   void updateLocationMarkers(SoGroup* markerGroup, const SbBox3f& bbox) {
     register unsigned int i, j, xsteps, ysteps, childIndex;
     float x, y, step;
     const float steps[] = {0.166666667,0.333333333,0.5};
     const unsigned int numsteps = 3;
     char label[100];
     SoText2 *labelObject;
     SoTranslation *trans;
     SoTransformSeparator *sep;
     SoGroup *details, *detailGroup;

     if(markerGroup==NULL) return;

     sep = (SoTransformSeparator*)markerGroup->getChild(2);

     trans = (SoTranslation*)sep->getChild(0);

     trans->translation.setValue(bbox.getMin().getValue()[0],(bbox.getMin().getValue()[1]+bbox.getMax().getValue()[1])*0.5,20);

     labelObject =  (SoText2*)sep->getChild(1);

     sep->addChild(labelObject);

     makeLatLonString(label,bbox.getMin().getValue()[0],LONGITUDE);

     labelObject->string.setValue(label);

     sep = (SoTransformSeparator*)markerGroup->getChild(3);
     trans = (SoTranslation*)sep->getChild(0);
     trans->translation.setValue(bbox.getMax().getValue()[0],(bbox.getMax().getValue()[1]+bbox.getMin().getValue()[1])*0.5,20);
     labelObject =  (SoText2*)sep->getChild(1);
     makeLatLonString(label,bbox.getMax().getValue()[0],LONGITUDE);
     labelObject->string.setValue(label);

     sep = (SoTransformSeparator*)markerGroup->getChild(4);
     trans = (SoTranslation*)sep->getChild(0);
     trans->translation.setValue((bbox.getMin().getValue()[0]+bbox.getMax().getValue()[0])*0.5,bbox.getMin().getValue()[1],20);
     labelObject =  (SoText2*)sep->getChild(1);
     makeLatLonString(label,bbox.getMin().getValue()[1],LATITUDE);
     labelObject->string.setValue(label);

     sep = (SoTransformSeparator*)markerGroup->getChild(5);
     trans = (SoTranslation*)sep->getChild(0);
     trans->translation.setValue((bbox.getMin().getValue()[0]+bbox.getMax().getValue()[0])*0.5,bbox.getMax().getValue()[1],20);
     labelObject =  (SoText2*)sep->getChild(1);
     makeLatLonString(label,bbox.getMax().getValue()[1],LATITUDE);
     labelObject->string.setValue(label);

     details = (SoGroup*)markerGroup->getChild(6);
     for (int stepIndex = 0; stepIndex<numsteps; stepIndex++) {
       step = steps[stepIndex];
       childIndex=0;
       detailGroup = (SoGroup*)details->getChild(stepIndex);
       xsteps=int((bbox.getMax().getValue()[0]-bbox.getMin().getValue()[0])/step-0.1);
       ysteps=int((bbox.getMax().getValue()[1]-bbox.getMin().getValue()[1])/step-0.1);
       
       x=bbox.getMin().getValue()[0]+step;
       y=bbox.getMin().getValue()[1]; 
       for (i=0;i<xsteps; i++) {
	     sep = (SoTransformSeparator*)detailGroup->getChild(childIndex);
	     trans = (SoTranslation*)sep->getChild(0);            
	     trans->translation.setValue(x,y+step*0.5,20);
             labelObject = (SoText2*)sep->getChild(1);
	     makeLatLonString(label,i,LONGITUDE);
	     labelObject->string.setValue(label);
	     childIndex++;
             x+=step;
       }

       x=bbox.getMin().getValue()[0];    
       y=bbox.getMin().getValue()[1]+step;
       for (j=0; j<ysteps; j++) {
	     sep = (SoTransformSeparator*)detailGroup->getChild(childIndex);
	     trans = (SoTranslation*)sep->getChild(0);            
	     trans->translation.setValue(x+step*0.5,y,20);
             labelObject = (SoText2*)sep->getChild(1);
	     makeLatLonString(label,i,LATITUDE);
	     labelObject->string.setValue(label);
	     childIndex++;
             y+=step;
       }

       x=bbox.getMin().getValue()[0]+step;    
       y=bbox.getMin().getValue()[1]+step;
       for (i=0;i<xsteps; i++) {
           for (j=0; j<ysteps; j++) {
	     sep = (SoTransformSeparator*)detailGroup->getChild(childIndex);
	     trans = (SoTranslation*)sep->getChild(0);            
	     trans->translation.setValue(x,y+step*0.5,20);
             labelObject = (SoText2*)sep->getChild(1);
	     makeLatLonString(label,i,LATITUDE);
	     labelObject->string.setValue(label);
	     childIndex++;

	     sep = (SoTransformSeparator*)detailGroup->getChild(childIndex);
	     trans =  (SoTranslation*)sep->getChild(0);            
	     trans->translation.setValue(x+step*0.5,y,20);
             labelObject = (SoText2*)sep->getChild(1);
	     makeLatLonString(label,i,LONGITUDE);
	     labelObject->string.setValue(label);
	     childIndex++;
             y+=step;
          }
          x+=step;
       }
    }
  }

 
   // IvSeaSurface is a translucent rectangle that can be shown and hidden
   class IvSeaSurface: public SoSwitch {
      protected:
         ~IvSeaSurface() { };
      public:
         IvSeaSurface(float seaRed, float seaGreen, float seaBlue);
         /* if a viewport and a map are provided, then the bounding box is recalculated to encompass the map.
         Otherwise use the values already in bbox */
         void updateSurfaceBox(SbBox3f& bbox, const SbViewportRegion *vp=NULL,  SoNode* map=NULL);
         inline void hide() { whichChild.setValue(SO_SWITCH_NONE); }
         inline void show() { whichChild.setValue(SO_SWITCH_ALL);  }
         inline bool isVisible() const { return whichChild.getValue() != SO_SWITCH_NONE; }
         void setColor(float seaRed, float seaGreen, float seaBlue);
   };

   IvSeaSurface::IvSeaSurface(float seaRed, float seaGreen, float seaBlue):SoSwitch(4) {
      whichChild=SO_SWITCH_ALL;
      SoMaterialBinding *mbinding=new SoMaterialBinding;
      mbinding->value=SoMaterialBinding::OVERALL;
      addChild(mbinding);
      SoMaterial *boxMat = new SoMaterial;
      addChild(boxMat);
      setColor(seaRed, seaGreen, seaBlue);
      SoCoordinate3 *boundcoord = new SoCoordinate3;
      boundcoord->point.setNum(5);
      addChild(boundcoord);
      SoFaceSet *boundary = new SoFaceSet;
      boundary->numVertices.setNum(1);
      boundary->numVertices.set1Value(0,5);
      addChild(boundary);
   }

   void IvSeaSurface::updateSurfaceBox(SbBox3f& bbox, const SbViewportRegion *vp,  SoNode* map) {
      /* if we're provided viewport and a map, then recalculate the bounding box around the map.
         Otherwise we use the values already in bbox */
      if (vp!=NULL && map!=NULL) {
         SoGetBoundingBoxAction bboxAction(*vp);
         bboxAction.apply(map);
         bbox = bboxAction.getBoundingBox();
      }
      SbVec3f *boundcoord = ((SoCoordinate3*)this->getChild(2))->point.startEditing();
      boundcoord[0].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[1].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[2].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[3].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[4].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      ((SoCoordinate3*)this->getChild(0))->point.finishEditing();
   }

   void IvSeaSurface::setColor(float r, float g, float b){
       SoMaterial *boxMat = (SoMaterial*)getChild(1);
       boxMat->ambientColor.set1Value(0,r,g,b);
       boxMat->diffuseColor.set1Value(0,r,g,b);
       boxMat->specularColor.set1Value(0,r,g,b);
       boxMat->emissiveColor.set1Value(0,r,g,b);
       boxMat->transparency.set1Value(0,0.8);
   }


   /* This function will update a box around an object created with a call to createBoundingBoxAround()
     to the current coordinates of the object's bounding box. */
   void updateBoundingBoxAround(SoGroup* map, SoGroup* mapBox, SbBox3f& bbox, const SbViewportRegion *vp=NULL) {
      printf("updateBoundingBoxAround:begin\n");
      if (vp) {
         SoGetBoundingBoxAction bboxAction =(*vp);
         bboxAction.apply(map);
         bbox = bboxAction.getBoundingBox();
      }
      // IMPORTANT: Getting boundcoord right depends on the bounding box geometry having been created with a call to createBoundingBoxAround()
      SbVec3f *boundcoord = ((SoCoordinate3*)mapBox->getChild(2))->point.startEditing();
      boundcoord[0].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[1].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[2].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[3].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[4].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
   
      boundcoord[5].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[6].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[7].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],0);
      boundcoord[8].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],0);
      boundcoord[9].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
   
      boundcoord[10].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[11].setValue(bbox.getMin().getValue()[0],bbox.getMin().getValue()[1],0);
   
      boundcoord[12].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[13].setValue(bbox.getMin().getValue()[0],bbox.getMax().getValue()[1],0);
   
      boundcoord[14].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[15].setValue(bbox.getMax().getValue()[0],bbox.getMax().getValue()[1],0);
   
      boundcoord[16].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],bbox.getMin().getValue()[2]);
      boundcoord[17].setValue(bbox.getMax().getValue()[0],bbox.getMin().getValue()[1],0);
      ((SoCoordinate3*)mapBox->getChild(0))->point.finishEditing();
      cout << "bathymetry bbox min = [" << bbox.getMin().getValue()[0] << ',' << bbox.getMin().getValue()[1] << ',' << bbox.getMin().getValue()[2] << "],\n" << endl;
      cout << "                max = [" << bbox.getMax().getValue()[0] << ',' << bbox.getMax().getValue()[1] << ',' << bbox.getMax().getValue()[2] << "]\n" << endl;
   }

   /* This function will create an Inventor group that draws a box around an object's bounding box. */
   SoGroup* createBoundingBoxAround(SoGroup* parent, SoGroup* map, SbBox3f& bbox, const SbViewportRegion *vp) {
      printf("Entering createBoundingBoxAround()\n");
      SoGroup *boxroot = new SoGroup;
      parent->addChild(boxroot);
      boxroot->setName("map box");
      SoMaterialBinding *mbinding=new SoMaterialBinding;
      mbinding->value=SoMaterialBinding::OVERALL;
      boxroot->addChild(mbinding);
      SoBaseColor *color= new SoBaseColor;
      // make the box light blue
      color->rgb.setValue(0.0f,0.235f,1.0f);
      boxroot->addChild(color);
      SoCoordinate3 *boundcoord = new SoCoordinate3;
      boundcoord->point.setNum(18); // A pairs of 5 pts for the top and bottom, plus 4 pairs to join the corners
      boxroot->addChild(boundcoord);
      printf("Calling updateBoundingBoxAround()\n");
      updateBoundingBoxAround(map,boxroot,bbox,vp);
      printf("Back to createBoundingBoxAround()\n");
      SoLineSet *boundary = new SoLineSet;
      boundary->numVertices.setNum(6);
      boundary->numVertices.set1Value(0,5);
      boundary->numVertices.set1Value(1,5);
      boundary->numVertices.set1Value(2,2);
      boundary->numVertices.set1Value(3,2);
      boundary->numVertices.set1Value(4,2);
      boundary->numVertices.set1Value(5,2);
      boxroot->addChild(boundary);
      return boxroot;
   }

   // Called by the viewer when the slider dynamically changes Z scaling
   bool setVerticalExaggeration(SoScale *mapScaling,unsigned int val) {
      TILE_EXAGGERATION_Z=val;
     //change the exaggeration of the bathymetry
      mapScaling->scaleFactor.setValue(mapScaling->scaleFactor.getValue()[0],mapScaling->scaleFactor.getValue()[1],TILE_EXAGGERATION_Z);
     //change the scale of the tracks and contacts
      trackDirectory.applyScaleToTracks(mapScaling->scaleFactor.getValue()[0],mapScaling->scaleFactor.getValue()[1],mapScaling->scaleFactor.getValue()[2]);
     // we always accept the scaling factor sent, so we always return true
      return true;
   }


   // Called by the viewer when the user requested to toggle wireframe rendering on/off
   bool  toggleWireframeMode(bool drawAsWireframe) {
      if(drawAsWireframe)
         setBathymetryToWireframe(bathymetryStyle);
      else
         setBathymetryToSolid(bathymetryStyle);
    // we always accept request to toggle drawing style, so we return true
      return true;
   }



   SoGroup* createSceneGraph(SoGroup* root,
                     SoScale*& mapScaling,   						
                     SoTranslation*& mapCentering,
                     IvTiledBathymetry*& bathymetry,
                     SoDrawStyle*& bathymetryInWireframe,
                     SoSeparator*& ownshipIcon, SoSeparator*& ownshipMarker,
                     SoSeparator*& targetMarker,
                     SoSeparator*& weaponIcon,  SoSeparator*& weaponMarker,
                     SoSwitch*& colormapSwitch, SoSwitch *& textureSwitch,
                     SoTexture2Transform *& rangeRingCenter) {
      SoInput loader;
      SoGroup *group;
      SoSeparator *mapRoot;
      SoNode *node;

      /* add a light close to the horizon, lighting the map from the side,
         to add contrast to the map features. */
      SoDirectionalLight *spotLight = new SoDirectionalLight;
      spotLight->intensity.setValue(1.0);
      spotLight->direction.setValue(0.57735,0.57735,-0.57735);
      root->addChild(spotLight);
     
      mapRoot = new SoSeparator;
      root->addChild(mapRoot);

      mapScaling = new SoScale;
      // set scaling factor so that everything is in meters (bathymetry tiles are in degrees in X and Y, and in meters in Z), plus some vertical exaggeration
      mapScaling->scaleFactor.setValue(TILE_SCALING_TO_METERS_X,TILE_SCALING_TO_METERS_Y,TILE_EXAGGERATION_Z);
      mapRoot->addChild(mapScaling);   

      mapCentering = new SoTranslation;
      mapRoot->addChild(mapCentering);   
   
   // load colormaps
   
      colormapSwitch = new SoSwitch();
      mapRoot->addChild(colormapSwitch);
   
      loader.openFile("color.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
   // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
   /* Now remove parent node added by loader.
      IMPORTANT:This works only if the colormap was the only child,
      which we know is true because we made the colormaps */
      group->removeChild(node);
      group->unref();
   
      loader.openFile("sandscale.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
   // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
      group->removeChild(node);
      group->unref();
   
      loader.openFile("bluescale.colormap.iv");
      group = SoDB::readAll(&loader);
      loader.closeFile();
      // SoInput adds a separator at the top if there wasn't one, so we skip it
      node = group->getChild(0);
      colormapSwitch->addChild(node);
      group->removeChild(node);
      group->unref();
   

      // set default colormap
      colormapSwitch->whichChild.setValue(0);

      // set style to be able to render bathymetry as wireframe when required
      bathymetryInWireframe = new SoDrawStyle;
      bathymetryInWireframe->style = SoDrawStyle::FILLED;
      mapRoot->addChild(bathymetryInWireframe);
   
      SoMaterialBinding *mapBinding = new SoMaterialBinding;
      mapBinding->value.setValue(SoMaterialBinding::PER_VERTEX_INDEXED);
      mapRoot->addChild(mapBinding);
   
      SoSeparator *mapTexRoot = new SoSeparator;
      mapRoot->addChild(mapTexRoot);

      textureSwitch=new SoSwitch;
      mapTexRoot->addChild(textureSwitch);
 
   
      // add a square texture on top of the map; cheap way to overlap a grid
      SoGroup *gridSep=new SoGroup;
      SoTexture2 *bottomgrid = new SoTexture2;
      gridSep->addChild(bottomgrid);
      bottomgrid->filename.setValue("square.rgb");
      bottomgrid->wrapS=SoTexture2::REPEAT;
      bottomgrid->wrapT=SoTexture2::REPEAT;
      SoTextureCoordinatePlane *texPlane1 = new SoTextureCoordinatePlane;
      texPlane1->directionS.setValue(SbVec3f(12,0,0));
      texPlane1->directionT.setValue(SbVec3f(0,12,0));
      gridSep->addChild(texPlane1);
      textureSwitch->addChild(gridSep);
   
      // add a range ring  texture on top of the map
      SoGroup *ringSep=new SoGroup;
      SoTexture2 *bottomcircle = new SoTexture2;
      ringSep->addChild(bottomcircle);
      bottomcircle->filename.setValue("circles.rgb");
      bottomcircle->wrapS=SoTexture2::CLAMP;
      bottomcircle->wrapT=SoTexture2::CLAMP;
      SoTexture2Transform *scale = new SoTexture2Transform;
      rangeRingCenter = new SoTexture2Transform;
      rangeRingCenter->translation.setValue(0,0);
      scale->translation.setValue(0.5,0.5);
      //1 km range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=111km*20/512
      //scale->scaleFactor.setValue(4.3359375,4.3359375);
      //1 kyd range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=121.36kyd*20/512
      //scale->scaleFactor.setValue(4.740625,4.740625);
      //2 kyd range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=121.36kyd*20/512 /2
      scale->scaleFactor.setValue(2.3703125,2.3703125);
      ringSep->addChild(scale);
      ringSep->addChild(rangeRingCenter);
      SoTextureCoordinatePlane *texPlane2 = new SoTextureCoordinatePlane;
      //1 km range rings: radius of inner circle is 20 pixels of a 512x512 pixels.  1 deg=111km*20/512
      texPlane2->directionS.setValue(SbVec3f(1,0,0));
      texPlane2->directionT.setValue(SbVec3f(0,1,0));
      ringSep->addChild(texPlane2);
      textureSwitch->addChild(ringSep);
   
   // start with range rings
      textureSwitch->whichChild.setValue(1);
   
   // create tiled bathymetry object
bathymetry = NULL; 
/****      bathymetry = new IvTiledBathymetry();
      mapRoot->addChild(bathymetry); ****/
   
   // load icon for ownship
      loader.openFile("goodsub.iv");
      ownshipIcon = SoDB::readAll(&loader);
      loader.closeFile();
   
   // load marker icon for ownship's track
      loader.openFile("goodsub_marker.iv");
      ownshipMarker = SoDB::readAll(&loader);
      loader.closeFile();
   
   
   // load marker icon for target contact's track
      loader.openFile("track_marker.iv");
      targetMarker = SoDB::readAll(&loader);
      loader.closeFile();
   
   
   // load icon for weapon
      loader.openFile("weapon.iv");
      weaponIcon = SoDB::readAll(&loader);
      loader.closeFile();
   
   // load marker icon for weapon's track
      loader.openFile("weapon_marker.iv");
      weaponMarker = SoDB::readAll(&loader);
      loader.closeFile();
   
      return mapRoot;
   } 

   DiverseHUD* createHUD(SoGroup* root, SbViewportRegion* viewport, SoMFColor& colormap,
                   IvColorScale*& colorscale , SelectionBoxes& selectionBoxes, IvContactLabeler2D*& contactLabeler, IvCompass*& compass) {
      DiverseHUD *hud = new DiverseHUD(viewport);
      root->addChild(hud);
      // makes the HUD objects not shiny
      SoLightModel *lm = new SoLightModel;
      lm->model.setValue(SoLightModel::BASE_COLOR);
      hud->addChild(lm);
      colorscale = new IvColorScale;
      colorscale->setSize(30,200);
      colorscale->setPosition(8,16);
      colorscale->setColormap(colormap);
      colorscale->setRange(-DEEP_THRESHOLD,0,9);
      hud->addChild(colorscale);
   
      selectionBoxes.targetSelection = new IvContactViewHighlighter2D(hud);
      selectionBoxes.targetSelection->setColor(CONTACT_LABEL_COLOR[0],CONTACT_LABEL_COLOR[1],CONTACT_LABEL_COLOR[2]);
      hud->addChild(selectionBoxes.targetSelection);

      selectionBoxes.weaponSelection = new IvContactViewHighlighter2D(hud);
      selectionBoxes.weaponSelection->setColor(WEAPON_LABEL_COLOR[0],WEAPON_LABEL_COLOR[1],WEAPON_LABEL_COLOR[2]);
      hud->addChild(selectionBoxes.weaponSelection);

      contactLabeler = new IvContactLabeler2D(hud);
      hud->addChild(contactLabeler);     
   
      compass = new IvCompass;
      compass->setSize(150,150);
      compass->setPosition(mainWindowRenderingData.viewport.getWindowSize().getValue()[0]-300,20);
      hud->addChild(compass);

      return hud;
   }


   /* This function calculates how far the camera should be from the scene to have the are in 'box' completely visible */
   void calculateDistanceToViewAll(float& neededDistance, float& newNearClipDistance, float& newFarClipDistance, 
               const SbBox3f& box, float slack = 0.0f, float cameraAspectRatio = 1.0f) {
      // calculate box diagonal, that's the maximum possible length of the object bouded by the box
      float x=box.getMax()[0]-box.getMin()[0];
      float y=box.getMax()[1]-box.getMin()[1];
      float z=box.getMax()[2]-box.getMin()[2];
      float radius=sqrt(x*x+y*y+z*z);
   
      neededDistance = radius / cameraAspectRatio;
   
      newNearClipDistance  = neededDistance - radius;
      if (newNearClipDistance < 0.0 )
         newNearClipDistance = 0;
      newFarClipDistance   = neededDistance + radius;
      printf("viewAll() distance = %f\n",neededDistance);
   }

   /* This is a helper function that does all the work of figuring where to put the scene so 'aNode' is visible, with an inclination of 'tiltAngle' on the X axis */
   void overviewObject(SoNode *aNode, const SbViewportRegion& vp, const float tiltAngle, const float cameraAspectRatio, SoTranslation* sceneCenter, const SoScale *scaling, SoRotation *rot, SoTranslation* sceneOffset,float& nearClip, float& farClip) {
      float distance;
      float xmin,ymin,zmin,xmax,ymax,zmax, center[3];
      /* Clear translations - since overviewObject() "pushes away" the object to make it visible,
         if we don't reset Z every time we call overviewObject(), it would get farther and farther away of the origin. */
      sceneCenter->translation.setValue(0,0,0);
      sceneOffset->translation.setValue(0,0,0);
      SoGetBoundingBoxAction bboxAction(vp);
      bboxAction.apply(aNode);
      bboxAction.getXfBoundingBox().getBounds(xmin,ymin,zmin,xmax,ymax,zmax);
      SbBox3f bbox = bboxAction.getBoundingBox();
      calculateDistanceToViewAll(distance,nearClip,farClip,bbox,1.0f,cameraAspectRatio);
      printf("viewAll() distance = %f\n",distance);
      center[0]=(xmin+xmax)/2;
      center[1]=(ymin+ymax)/2;
      center[2]=(zmin+zmax)/2;
      printf("overviewObject():\n\tnon-transformed map bbox min = [%f, %f, %f], max = [%f,%f,%f]\n",xmin,ymin,zmin,xmax,ymax,zmax);
      printf("overviewObject():\n\ttransformed map bbox min = [%f, %f, %f], max = [%f,%f,%f]\n",bbox.getMin()[0],bbox.getMin()[1],bbox.getMin()[2],bbox.getMax()[0],bbox.getMax()[1],bbox.getMax()[2]);
      rot->rotation.setValue(SbVec3f(1,0,0),tiltAngle);
      sceneCenter->translation.setValue(-center[0],-center[1],-center[2]);
      sceneOffset->translation.setValue(0,0,-distance-(bbox.getMin()[2]+bbox.getMax()[2])/2);
      /* IMPORTANT! note that here we're pivoting over the Y axis, even though we moved the scene away on Z.
           We have to do this because our drawing callback rotates the scene 90 degrees, so that Z becomes "front" instead of "up" */
	float T1, T2, T3, T4;
	rot->rotation.getValue().getValue(T1, T2, T3, T4);
   }


   /* Moves the scene so that the camera is pointing at the center of the bounding box of the Inventor object 'aNode', an the object is completely visible.
      The object will be visible from the top.
      Theis function is not completely generic, scene manipulation is dependent upong the shape of the scene built in main() */
   void setTopDownOverview(SoNode* aNode, const SbViewportRegion& vp, const float cameraAspectRatio,  SoTranslation* sceneCenter, const SoScale *scaling, SoRotation *rot, SoTranslation* sceneOffset) {
      float nearClip, farClip;
      // reset DGL navigator (if any) to put camera in a known position
      dMatrix Temp;
      Temp.makeIdentity();
      NavManager::setMatrix(Temp);
      overviewObject(aNode,vp,0.0f,cameraAspectRatio,sceneCenter,scaling,rot,sceneOffset,nearClip,farClip);
   }

   /* Moves the scene so that the camera is pointing at the center of the bounding box of the Inventor object 'aNode', an the object is completely visible.
      The scene tilted will be tilted 'mapTiltAngleInRadians'  on the X axis.
      Theis function is not completely generic, scene manipulation is dependent upong the shape of the scene built in main() */
   void setTopDownPerspectiveOverview(SoNode* aNode, float  mapTiltAngleInRadians, const SbViewportRegion& vp, const float cameraAspectRatio, SoTranslation * sceneCenter, const SoScale *scaling, SoRotation *rot, SoTranslation *sceneOffset) {
      float nearClip, farClip;
      printf("begin setTopDownPerspectiveOverview()\n");
      // reset DGL navigator (if any) to put camera in a known position
      dMatrix Temp;
      Temp.makeIdentity();
      NavManager::setMatrix(Temp);
      overviewObject(aNode,vp, mapTiltAngleInRadians,cameraAspectRatio,sceneCenter,scaling,rot,sceneOffset,nearClip,farClip);
      printf("end setTopDownPerspectiveOverview()\n");
   }
 
   /* Moves the scene so that the camera is pointing at the point 'centerPos', with the scene tilted 'mapTiltAngleInRadians'  on the X axis.
      Theis function is not completely generic, scene manipulation is dependent upong the shape of the scene built in main() */
   void setViewCenteredOn(const SbVec3f& centerPos, const SbViewportRegion& vp, const float cameraAspectRatio, SoTranslation *sceneCenter, float mapTiltAngleInRadians, SoRotation* mapRotation, SoTranslation *sceneOffset) {
   #define ENCLOSING_VIEW_X 5000.0f
   #define ENCLOSING_VIEW_Y 5000.0f
   #define ENCLOSING_VIEW_Z 250.0f
      float distance,nearClip,farClip;
      const float *xyz = centerPos.getValue();
      printf("setViewCenteredOn(%f,%f,%f)\n",xyz[0],xyz[1],xyz[2]);
      SbBox3f enclosingBox;
      enclosingBox.setBounds(xyz[0]-ENCLOSING_VIEW_X,xyz[1]-ENCLOSING_VIEW_Y,xyz[2]-ENCLOSING_VIEW_Z,
                           xyz[0]+ENCLOSING_VIEW_X,xyz[1]+ENCLOSING_VIEW_Y,xyz[2]+ENCLOSING_VIEW_Z);;
      calculateDistanceToViewAll(distance,nearClip,farClip,enclosingBox,cameraAspectRatio);
      sceneCenter->translation.setValue(-centerPos[0],-centerPos[1],-centerPos[2]);
      mapRotation->rotation.setValue(SbVec3f(1,0,0),mapTiltAngleInRadians);
      sceneOffset->translation.setValue(0,0,-distance);
      dMatrix Temp;
      Temp.makeIdentity();
      NavManager::setMatrix(Temp);
   }

   SbBox3f fillMapCenteredOn(IvTiledBathymetry* bathymetry, SbViewportRegion* renderArea, float x, float y) {
      bathymetry->fillVisibleAreaWithTiles(Rectangle2dFloat(x-DEFAULT_MAP_RADIUS_AROUND_X,y-DEFAULT_MAP_RADIUS_AROUND_Y, x+DEFAULT_MAP_RADIUS_AROUND_X,y+DEFAULT_MAP_RADIUS_AROUND_Y));
      SoGetBoundingBoxAction action(*renderArea);
      action.apply(bathymetry);
      return action.getBoundingBox();
   }

   void cleanUpAndFinish(int signal = 0) {
   // IMPORTANT! clean-up shared memory
      if(talkWithBezel != NULL) delete talkWithBezel;
   // close socket connection to feeder before finishing
      feeder.closeConnection();
      exit(1);
   }


/*** feeder functions ****/

   void stopFeeder() {
      feeder.sendByte(FeederCommand::STOP_FEEDER);
   }

   void resumeFeeder() {
      feeder.sendByte(FeederCommand::RESUME_FEEDER);
   }

   void tellFeederToDie() {
      feeder.sendByte(FeederCommand::QUIT);
   }

   // Pick only if it's one of the contact icons. Called by a SoSelection node
   SoPath *pickFilter(void * selectionObject, const SoPickedPoint *pick)
   {
   // Find out which child of selection got picked
      SoNode *n;
      SoPath *path = pick->getPath();
      int i =0;
      while (i < path->getLength()) {
         n = path->getNode(i);
         if (n->isOfType(IvContactView::getClassTypeId())) // is it a contact view?
            break;
         i++;
      }
   	// was a IvContactView picked?
      if(i != path->getLength()) {
         // we found an IvContactView; discard the path after it, we don't care about the rest
         path->truncate(i+1);
         SelectionBoxes *selectionBoxes = (SelectionBoxes*)selectionObject;
         DiscreteTrack *track = dynamic_cast<DiscreteTrack*>( ((IvContactView*)(n))->getModel() );
         switch(track->getContactType()) {
            case CONTACT_IS_WEAPON:
               if(talkWithBezel->selectedWeapon != BAD_OID)
                  ((IvHighlightSelection2D*)(selectionBoxes->weaponSelection))->deselect();
               ((IvHighlightSelection2D*)(selectionBoxes->weaponSelection))->setSelection(path);
               talkWithBezel->selectedWeapon = track;
               talkWithBezel->askBezel(BezelCommand::SELECT_CONTACT,OID::fromObject(track));
               break;
            case CONTACT_IS_VEHICLE:
            case CONTACT_FROM_SENSOR:
               if(talkWithBezel->selectedTarget != BAD_OID)
                  ((IvHighlightSelection2D*)(selectionBoxes->targetSelection))->deselect();
               ((IvHighlightSelection2D*)(selectionBoxes->targetSelection))->setSelection(path);
               talkWithBezel->selectedTarget = track;
               talkWithBezel->askBezel(BezelCommand::SELECT_CONTACT,OID::fromObject(track));
         }
       	 // resend the track's current position to the views, so the UI gets the current position of the new selection
         track->changed(TrackEvent::RESENDING_TRACK_POS);
         // we need to return the path to the SoSelection object for the SoSelection to force a repaint - strange...
         return path;
      }
   /* fool the SoSelection into thinking it failed to pick; we don't need it to actually
      keep any path info in the SoSelection, since we're keeping that information
      by ourselves in the selectionObject */
      return NULL;
   }


   SoPath* makePath(SoNode* root, SoNode* finalNode) {
      SoSearchAction nodeFinder;
      nodeFinder.setNode(finalNode);
      nodeFinder.setInterest(SoSearchAction::FIRST);
      nodeFinder.apply(root);
      SoPath* pathToNode = nodeFinder.getPath();
      pathToNode->ref();
      return pathToNode;
   }

   bool connectToFeeder(FeederConnection& feeder) {
      unsigned int attempts = 0;
      #define MAX_ATTEMPTS_TO_CONNECT 5
      while(attempts < MAX_ATTEMPTS_TO_CONNECT && feeder.connectToFeeder() < 0) {
         attempts++;
         usleep(500000); // wait for 1/2 second
      }
      return attempts != MAX_ATTEMPTS_TO_CONNECT;
   }


   SoSeparator* get3dIconFor(const ContactIconCollection3D& icons, Category vehicleID) {
      SoSeparator *icon;
      if(vehicleID >= icons.getSize()) {
         return (SoSeparator*)icons.getDefaultIcon()->copy();
      }
      else {
         icon = (SoSeparator*)icons[(unsigned int)vehicleID];
         if(icon != NULL)
            return (SoSeparator*)icon->copy();
         else
            return (SoSeparator*)icons.getDefaultIcon()->copy();
      }
   }

   /* This function must setup the 2D interface, also called "the bezel". Since till
      now the UI is a separate program, this function executes the bezel program */
   void setup2dUserInterface(const char* xdisplay) {	   
       static char* environ[] = { "DISPLAY=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx", NULL }; 
       static char* argv[] = { "bezel", NULL };
       if(fork()==0) { // we're the child
            if(xdisplay==NULL) {
                xdisplay=getenv("DISPLAY");
	        if(xdisplay==NULL)
		    xdisplay=":0.0";
            }
	    strcpy(environ[0]+8,xdisplay); // put value of DISPLAY env. var. immediately after DISPLAY= in environ[0]
            printf("bezel will appear on DISPLAY %s\n",xdisplay);
	    if(execve("./bezel/bezel",argv,environ) == -1) {
              printf("ERROR: Can't find bezel program\n");
	      exit(-1);
            }
       }
   } 

   void printHelp(const char* thisProgramName) {
     printf("USAGE: %s [-w | --xwand] [-a | --angle <pitch>][-h | --help]\n",thisProgramName);
     printf("\t-w, --xwand\t Tells this program it should xwand to control cursor in a VR display.\n");
     printf("\t-a, --angle <pitch>\t Map pitch. Inclination in degrees to show the map.\n");
     printf("\t-h, --help\t Prints this help and exits.\n");
   }

   void setWindowSize(unsigned int left, unsigned int bottom, unsigned int width, unsigned int height) {
       vector<DDisplay*> dglDisplays = DisplayManager::getDisplays();
       vector<DWindow*> dglWindows = dglDisplays[0]->getWindows();
       dglWindows[0]->setWindowGeometry(Rect(left,bottom,width,height));
   }

   
   // Finds the X Display named 'xdisplayName' among the displays being used by Diverse
   Display* findXDisplayInDglNamed(const char* xdisplayName) {
       Display* dpy = NULL;
       vector<DDisplay*> dglDisplays = DisplayManager::getDisplays();
       // try to find which a dgl display with the same string
       for(vector<DDisplay*>:: iterator iter = dglDisplays.begin(); iter !=  dglDisplays.end(); ++iter) {
          vector<DWindow*> dglWindows = (*iter)->getWindows();
          Producer::RenderSurface *mySurface = (Producer::RenderSurface*)dglWindows[0]->getRenderSurface();
          if(mySurface != NULL) {
              dpy = mySurface->getDisplay();
              if(strcmp(xdisplayName,DisplayString(dpy)) ) {
                   break;
              }
          }
       }
       return dpy;
   }


   // Returns the X Display and X Window of the first display, as defined by the currently-used display DSO
   void getX11OutputDisplayAndWindow(Display*& dpy, Window& window) {
         vector<DDisplay*> dglDisplays = DisplayManager::getDisplays();
        //Get the dgl window data out
	vector<DWindow*> dglWindows = dglDisplays[0]->getWindows();
        Producer::RenderSurface* mySurface = (Producer::RenderSurface*)dglWindows[0]->getRenderSurface();
	if(mySurface == NULL) {
	    dpy = NULL;
	    window = NULL;
        } else {
            dpy = mySurface->getDisplay();
            window = mySurface->getWindow();
        }
    }


    // this function is used mainly during debugging
    void printCurrentHeadPosition() {                     
        printf("------------------------ CURRENT HEAD TRACKER -------------------------\n");
        dVec3 vec3;
	dMatrix navmatrix=  NavManager::getMatrix();
	vec3=navmatrix.getTrans();
        printf("Current Position: XYZ=[%f,%f,%f],", vec3[0],vec3[1],vec3[2]);
        DTrackerEvent *evt=EventManager::getTrackerState("headtracker");
        vec3=evt->getDirection();                     
        printf(" HPR=[%f,%f,%f]\n",vec3[0],vec3[1],vec3[2]);
        printf("-----------------------------------------------------------------------\n");

    }

//////////////////////////////////////////////////////
//////////                MAIN             ///////////
//////////////////////////////////////////////////////

   int main(int argc, char**argv) {      
      int i; // just a generic loop var
      float mapTiltAngle=-45.0f; // the inclination angle for the map - here's in degrees, but throughout the program it will be in radians (see further below)

      bool seaSurfaceIsVisible=false; // this flags memorizes the current visibility state of the sea surface
      bool useXwand=false;
      bool xwandIsActive=false;
      Cursor wandCursor;
      Cursor invisibleCursor; // this cursor (a blank one) is only used if we have to use xwand
      Display *renderingDisplay, *xwandDisplay; // if we're going to use xwand, this holds which X11 Display to use
      Window xwandWindow;      // window where the xwand will control the cursor (only if xwand is used)
      XWandFakeMouse fakeMouse; // if we're going to use xwand, this object IS the xwand

      SoXt::init(argv[0]);
   
      // Initialize DGL - It reads the config file, sets up navigators and renderers, etc      
      fflush(stdout);
      printf("Calling dglInit...\n");      
      fflush(stdout);
      dglInit(argc,argv);
      printf("Calling dglStart...\n");      
      fflush(stdout);
      dglStart(); 

      // get DGL renderer
      fflush(stdout);
      printf("getting display list...\n");
      vector<DDisplay*> displayList = DisplayManager::getDisplays();
      printf("getting display #1...\n");
      DDisplay* dglDisplay = displayList[0];
      printf("Display = %s\n",dglDisplay->getDisplayName().c_str());
      displayList.clear();

      if (!dglDisplay) {
         printf("ERROR: Could not find a valid DGL display object. Exiting.\n");
         return EXIT_FAILURE;
      }

      getX11OutputDisplayAndWindow(renderingDisplay,xwandWindow);
      if (renderingDisplay == NULL) {
           printf("ERROR: Unable to get Window IDs from DGL output DSO. Exiting.\n");
          return EXIT_FAILURE;
      } 

      // calculate area left for the rendering window, after we substract what's taken by the bezel
      mainWindowRenderingData.windowLeft = 0;
      mainWindowRenderingData.windowBottom = BOTTOM_AREA_HEIGHT;
      mainWindowRenderingData.viewport.setWindowSize(min(1280,WidthOfScreen(DefaultScreenOfDisplay(renderingDisplay)))-RIGHT_AREA_WIDTH,
                                               min(1024,HeightOfScreen(DefaultScreenOfDisplay(renderingDisplay)))-TOP_AREA_HEIGHT-BOTTOM_AREA_HEIGHT);

      setWindowSize(mainWindowRenderingData.windowLeft,
          mainWindowRenderingData.windowBottom,mainWindowRenderingData.viewport.getWindowSize().getValue()[0],mainWindowRenderingData.viewport.getWindowSize().getValue()[1]);

      SoInput xxxloader;
      xxxloader.openFile("cube.iv");
      xxxroot = SoDB::readAll(&xxxloader);
      xxxroot->ref();
      xxxloader.closeFile();


      dglDisplayCallback( drawSceneGraphCallback );
 
      // setup bezel (the 2D inteface around the rendering window) on the same display the rendering window is going to appear
/****      setup2dUserInterface(DisplayString(renderingDisplay)); ****/

   // Set signal handlers, to ensure cleaning up after anything goes wrong
      struct sigaction signalHandling;
      signalHandling.sa_handler = cleanUpAndFinish;
      signalHandling.sa_flags = 0;
      sigaction(SIGINT,&signalHandling,NULL);
      sigaction(SIGQUIT,&signalHandling,NULL);
      sigaction(SIGTERM,&signalHandling,NULL);


   /* create object that will communitate with bezel through shared memory,
      the object in itself is in shared memory */
/****      talkWithBezel = new(ObjectInSharedMemory::CREATE_WITH_KEY,UI_SHMEM_ID) UiMediator; ****/
   
   // Establish connnection with contact data source
      cout << " Waiting for connection from Contact Data Feeder...\n";
      cout.flush();
      if(!connectToFeeder(feeder)) {
         cerr << endl << "Could not connect to data source. Exiting." << endl;
         cleanUpAndFinish();
      }

      cout << "feeder connected." << endl;
      cout.flush();
   
   /* Here initialize Inventor Types that are not built-in. This needs to
      be done after SoDB::init() is called, which is done by SoXt::init() */ 

      // Set the near and far clipping plane in all viewports so we don't have to fight with them later.
      DisplayManager::setNearClip(0.0001);
      DisplayManager::setFarClip(FLT_MAX);

   // Flag used to tell when we need to render a new minimap
      bool updateMiniMap = false;
      bool updateCamera = false;
   
   // Flag used to tell when we need to recenter the minimap
      bool recenterMiniMap = false;
   
   // This is a flag that tells how to update the bathymetry: following the ownship, or wherever the user wants to go
      MapUpdatingMode mapUpdatingMode = MAP_FOLLOWS_OWNSHIP;
   
   // Flag used to tell when we need to recenter the rangeRings
      bool showRings = true;
      	
      Timeline timeline; // object that manages all tracks
      // variables to create, keep track and display ships
      ContactEvent contactEvent;
      ObservableTrack *track, *ownshipTrack;
      TrackTraceDecorator *trackTrace;
      IvTrackView *traceView;
      TrackUiView *trackUiView;
      CoordAndTime *coordAndTime;

      // this is a local contact event queue; right now we only use to to deal with Hypothetical Contacts created by the user
      ContactEventQueue localEventQueue; 
      bool newContactEventAvailable; // true if a new event is available from the feeder or from the local contact event queue

      int xtiles=0, ytiles=0; // how many map tiles to slide in X and Y when we get close to the edge of the map's visible area   
   
      int bytes_read; // number of bytes read from the feeder when a new contact data is available
      int contactEventsRead = 0; //  number of contact events received in one pass of the contact event loop

      // enable depth testing before starting to render, otherwise Inventor rendering doesn't do depth testing correctly
      glEnable(GL_DEPTH_TEST);
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);

      bool quitRenderingNow = false;     
/****      while(!talkWithBezel->readyToQuit() && !quitRenderingNow && dglIsRunning()) { ****/
      while(!quitRenderingNow && dglIsRunning()) {

         /* process UI pending command -- these commands don't come from the 3D view, but from the 2D user interface */
/****         if(talkWithBezel->isCommandAvailable()) {
         talkWithBezel->signalCommandDone();
         } ****/

         dglSync(); // we need to be sure that DGL is not rendering while we're here, otherwise the camera change may also affect the main view - This is a problem with Inventor
         dglFrame();

      }

      // wait for all rendering threads to finish their job...
      dglSync();

      // ...now we can exit.
   
      return EXIT_SUCCESS;
   }
	
