#ifndef TIMEVAL_DOUBLE_H
#define TIMEVAL_DOUBLE_H

// 2002, Fernando Das-Neves, Virgina Tech UVAG Group //


#include <sys/time.h>
#include <iostream>
#include <limits.h>

using namespace std;


/*! TIME_MIN is the minimum value a timeval struct can take */
const timeval TIME_MIN = { 0, 0 };

/*! TIME_MAX is the maximum value a timeval struct can take */
const timeval TIME_MAX = { LONG_MAX, 999999 };

/*! TIME_ZERO is a timeval struct set to 0 seconds, 0 millisecs */
const timeval TIME_ZERO = { 0, 0 };

/*! BAD_TIME is a value a valid timeval can never take */
const timeval BAD_TIME = { 0, LONG_MIN };

double timevalToDouble(const timeval& time);
timeval doubleToTimeval(double dbltime);

inline bool operator<(const timeval& t1, const timeval& t2) {
  return (t1.tv_sec < t2.tv_sec) || ( t1.tv_sec == t2.tv_sec && t1.tv_usec < t2.tv_usec);
}

inline bool operator>(const timeval& t1, const timeval& t2) {
  return (t1.tv_sec > t2.tv_sec) || ( t1.tv_sec == t2.tv_sec && t1.tv_usec > t2.tv_usec);
}

inline bool operator<=(const timeval& t1, const timeval& t2) {
  return ( t1.tv_sec == t2.tv_sec && t1.tv_usec <= t2.tv_usec) || (t1.tv_sec <= t2.tv_sec);
}

inline bool operator>=(const timeval& t1, const timeval& t2) {
  return ( t1.tv_sec == t2.tv_sec && t1.tv_usec >= t2.tv_usec) || (t1.tv_sec >= t2.tv_sec);
}


void operator-=(timeval& t1, const timeval& t2);
void operator+=(timeval& t1, const timeval& t2);



timeval operator-(const timeval& t1, const timeval& t2);
timeval operator+(const timeval& t1, const timeval& t2);
void operator*=(timeval& t1, float factor);
void operator/=(timeval& t1, float factor);

ostream& operator<<(ostream& os,timeval& tm);

inline bool operator!=(const timeval& t1, const timeval& t2) {
  return (t1.tv_sec != t2.tv_sec || t1.tv_usec != t2.tv_usec);
}

inline bool operator==(const timeval& t1, const timeval& t2) {
  return (t1.tv_sec == t2.tv_sec && t1.tv_usec == t2.tv_usec);
}

#endif
