#include "XWandFakeMouse.h" 
#include <eventmgr/EventManager.h>
#include <eventmgr/devents.h>

const char* XWandFakeMouse::XWandConfigDefaultFilename = "xwand.config";

void XWandFakeMouse::initXWandData() {
	//Set the default values for data
	xwandData.numButtons = 0;
	xwandData.buttons = NULL;
	xwandData.display = ":0.0";
	xwandData.numArgs = 0;
	xwandData.app = NULL;
	xwandData.numActions = 0;
}

XWandFakeMouse::XWandFakeMouse():buttons(0) {
    initXWandData();
    useDisplay(xwandData.display);
};

bool XWandFakeMouse::initFromFile(const char* XwandConfigFileName) {
    buttons = 0;
    // read wand configuration file
    bool successInReadingFile = xwandData.readXWandConfigFile(XwandConfigFileName, false);
    if (successInReadingFile)
        useDisplay(xwandData.display);
    return successInReadingFile;
}

// Take input from the wand joystick and move the mouse based on this
void XWandFakeMouse::updateScreenPointerWithJoyStick() {
	//Where I am at currently (has to be gotten from Mouse)
	int currPos[2];
	//Where the joystick is going to be
	float joyPos[2];

	//Get the current mouses position
	getMousePosition(currPos[0],currPos[1]);
	//Get where the joystick is pointing
        DWandEvent* wandEvent = EventManager::getWandState();
  	joyPos[0]=wandEvent->getJoystickX();
  	joyPos[1]=wandEvent->getJoystickY();
  	//Update the mouse position
	if (joyPos[0] !=0 && joyPos[1] !=0) {
	    //The updating factor
	    int ox = (int)(joyPos[0]*joyPos[0]*joyPos[0]*25.f) ;
	    int oy = -(int)(joyPos[1]*joyPos[1]*joyPos[1]*25.f) ;
	    
	    //Basically figure out what needs to be added
	    currPos[0] += ox ;
	    currPos[1] += oy ;
	    
	    //Set the mouse pointer position
	    setPosition(currPos[0], currPos[1]);
        }
};


// returns the want button press state as an 8-bit array (bit i=1 => button #i pressed)
unsigned char XWandFakeMouse::rawReadMouseButtonState() {
     unsigned char buttonsState=0;
     DWandEvent* wandEvent = EventManager::getWandState();
     for(int btnIndex=0; btnIndex<4; btnIndex++) {
         if (wandEvent->getButton(btnIndex)) {
             buttonsState |= (1<<btnIndex);
	 }
     }
     return buttonsState;
}

// checks which buttons are pressed/released, and find which actions to execute
int XWandFakeMouse::performButtonAction() {
    unsigned char bitButton_copy, bitButton, buttonstate = rawReadMouseButtonState();
    // the purpose of the next lines is to differentiate button press from button release
    bitButton = buttons ^ buttonstate;
    bitButton_copy = bitButton;
    unsigned short int btnIndex;
    if(bitButton_copy != 0) {
       // get button index from bit index, but a lot cheaper than taking log2
       btnIndex=0;
       bitButton_copy >>= 1;
       while(bitButton_copy!=0) {
          btnIndex++;       
          bitButton_copy >>= 1;
       }
       // find associated action, and perform it
       if(btnIndex < xwandData.numButtons) {
           switch(xwandData.buttons[btnIndex]) {
               case XW_RUN_APP:
                    break; // ignore request to run app
               case XW_LEFTBUTTON:
		    if((buttonstate & bitButton) != 0) {
  		        press(1);
		    } else {
			release(1);
                    }
                    break;
               case XW_RIGHTBUTTON:
		    if(buttonstate & bitButton != 0)
  		        press(3);
		    else
			release(3);
                    break;
           }
           buttons = buttonstate;
           if ((buttonstate & bitButton) != 0) {
		return xwandData.buttons[btnIndex];
	   }
        }
    }
    return XW_NONE;
}	


