#ifndef ICON_COLLECTION_H
#define ICON_COLLECTION_H

#include <list>
#include <Inventor/nodes/SoNode.h>

   using namespace std;

   /*!  Keeps a collection of inventor objects and returns them by index number.
      It can also return a default object is the requested object is missing.
      Use the load3dIconsFrom() method to load icons in the order given by the file passed as parameter. */
   class ContactIconCollection3D {
   protected:
      typedef list<void*> IconList;
      bool IamTheOwner;
   
   public:
      SoNode **icons;
      unsigned int size, index_default_icon;   	
      typedef enum { NO_ERROR = 0, CANT_FIND_FILE = 1 } FILE_RESULT;
      ContactIconCollection3D():size(0),index_default_icon(1),icons(NULL),IamTheOwner(false) { 
      }
      ContactIconCollection3D(unsigned int numberOfIcons);
      void setSize(unsigned int numberOfIcons);
      unsigned int getSize() const { 
         return size; }
      SoNode* operator[](unsigned int iconNo) const;
      SoNode* operator[](unsigned int iconNo)  { 
         return icons[iconNo]; }
      /*! Set the index of the default icon to return when an icon is missing */
      void setDefaultIconTo(unsigned int index);
      SoNode* getDefaultIcon() const { 
         return index_default_icon<size ? icons[index_default_icon] : NULL; }
      /*! Load objects in the order given by a text file. Each line in the text file must be the path to an Inventor object.
         After load3dIconsFrom() is called, the [] operator will return NULL for a missing icon,
         unless a default icon is designated using setDefaultIconTo(). */
      FILE_RESULT load3dIconsFrom(const char* const filename, bool ownership = true);
      /* When this method is called, NULL will be returned by the [] operator if an icon is missing */
      void dontKeepADefaultIcon();
      ~ContactIconCollection3D();
   
   };


#endif
