#ifndef IVHUD_H
#define IVHUD_H

/// 2002, Fernando Das-Neves, Virginia Tech UVAG Group ///

#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/nodes/SoSubNode.h>
#include <GL/gl.h>


/*! This class is very interesting: If added as the **LAST** children to 
the scene, it will make any children node to be drawn in 2D on top 
of anything on the screen (hence the IvHUD name). 
The coordinate system will be 1 unit = 1 pixel, (0,0) will be
the left bottom corner of the window, with X and Y axis increasing
to the right and to the top.
Note this object resets OpenGL modelview matrix to the identity matrix while drawing.
IMPORTANT: This HUD trick will work ONLY if the IvHUD node is the last
node added after the scene in itself. It does not matter how you build the rest of
the scene graph; the HUD only depends on being drawn after the rest of the scene.
*/
   class IvHUD: public SoGroup {
      SO_NODE_HEADER(IvHUD);
   // sets OpenGL in 2D orthographic mode, 1 unit = 1 pixel
      static void preDraw(int h, int w, bool& lights);
   // restores OpenGL to 3D rendering
      static void postDraw(bool lights);
   protected:
   /// true if, when asked to compute the BBox, this node does not ignore the request
      bool hasBBox;
   /// true if we render the HUD + children, false if don't render them
      bool isVisible;
   // These implement supported actions.
      virtual void  GLRender(SoGLRenderAction *action);
      void getBoundingBox(SoGetBoundingBoxAction *action);
      void initInstance(SbViewportRegion* vp);
   /// This is an inventor shared object, so the destructor must never be called explicitly
      virtual ~IvHUD();
   public:
      /// This variable holds a copy of the OpenGL modelView and projection matrices, before the HUD changed the projection to 2D pixel orthographic coords
      GLdouble glModelViewMatrix[16],glProjMatrix[16];
      /// viewport contains the OpenGL viewport x,y,width and height, in that order
      GLint viewport[4];
      static void initClass();
   
      IvHUD(SbViewportRegion* vp = NULL);
   /// Constructor that takes approximate number of children as argument.
      IvHUD(int numChildren, SbViewportRegion* vp = NULL);
   
   /// hides the HUD and all its children
      void hide() { 
         isVisible = false; }
   /// makes the HUD and all its children visible (default)
      void show() { 
         isVisible = true; }
   /// set to true if the HUD will reports its bounding box (default: false)
      void hasBoudingBox(bool yesOrNo) { 
         hasBBox = yesOrNo; }
   
      /// The viewport is calculated only once, when the constructor is called. If the viewport changes, call this function so the HUD finds out about the new size.
      void updateViewport(SbViewportRegion* vp);	
   
   /// The viewport is calculated only once, when the constructor is called. If the viewport changes, call this function so the HUD finds out about the new size.
      void updateViewportFromGL();	
      const SbVec2s getWindowSize() const { 
         return  SbVec2s(viewport[2],viewport[3]); }
   };

#endif
