#ifndef IV_HL_SEL_H
#define IV_HL_SEL_H

#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/Xt/SoXtRenderArea.h>
#include <Inventor/SoPath.h>
#include "DiverseHUD.h"



/*! IvHighlightSelection2D objects are drawn in 2D, and so they
    assume that they are a child of an IvHUD object in the scene graph.
    An IvHighlightSelection2D will draw a 2D bounding box around the
    screen projection of an object's 3D bounding box, in the color
    and thickness specified. */
   class IvHighlightSelection2D: public SoCallback  {
   protected:
      float xmin,ymin,xmax,ymax;
      SoPath* selectionPath;
      float boxColor[3];
      float lineThickness;
      /// this is a flag to disable/enable drawing without lchanging the selection
      bool drawFlag;
      // This is the HUD that does the 2D coordinate setting
      DiverseHUD *hud;

      static void drawBbox(void*, SoAction*);
      ~IvHighlightSelection2D() {
         if(selectionPath) selectionPath->unref();  }
   public:
      /// Creates a  selection object, that will work with a certain IvHUD's render area and 'aCamera'. 
      IvHighlightSelection2D(DiverseHUD* theHUD);
      /// Receives a path to an object in the scene graph, and selects the object at end of the path
      void setSelection(SoPath* p);
      void deselect() {
         if(selectionPath) selectionPath->unref(); selectionPath = NULL; }
      void setColor(float r, float g, float b) {
         boxColor[0]=r; boxColor[1]=g; boxColor[2]=b; }
      void setLineThickness(float thickness) {
         lineThickness = thickness; }
   /*! Updates the bounding box around the selected object. The selection object will automatically update the bounding box every time
      it is drawn, so you don't normally need to call this method. */
      virtual void updateBbox();
   /// Returns the Inventor object that is being highlighted, or NULL if no object is selected
      SoNode* getSelectedObject() const { 
         if(selectionPath != NULL) 
            return selectionPath->getTail(); 
         else 
            return NULL; }
      void hide() { 
         drawFlag = false; }
      void show() { 
         drawFlag = true; }
      void setVisible(bool yesOrNo) { 
         drawFlag = yesOrNo; }
      bool isVisible() const { 
         return drawFlag; } 	    
   };

#endif
