#include "IvTransformEngine.h"
#include <Inventor/SbLinear.h>

   SO_ENGINE_SOURCE(IvTransformEngine);

// Initializes the IvTransformEngine class.

   void IvTransformEngine::initClass() {
      SO_ENGINE_INIT_CLASS(IvTransformEngine, SoEngine, "IvTransformEngine");
   }

// Constructor
   IvTransformEngine::IvTransformEngine() {
   // Do standard constructor tasks
      SO_ENGINE_CONSTRUCTOR(IvTransformEngine);
   
   // Define input fields and their default values
      SO_ENGINE_ADD_INPUT(inputMatrix, (1.0f,0.0f,0.0f,0.0f,
					0.0f,1.0f,0.0f,0.0f,
					0.0f,0.0f,1.0f,0.0f,
					0.0f,0.0f,0.0f,1.0f));
      SO_ENGINE_ADD_INPUT(rotation,  (0.0f,0.0f,0.0f,1.0f));
      SO_ENGINE_ADD_INPUT(scaleFactor,  (1.0f,1.0f,1.0f));
      SO_ENGINE_ADD_INPUT(translation,  (0.0f,0.0f,0.0f));
   
   // Define the output, specifying its type
      SO_ENGINE_ADD_OUTPUT(transfMatrix, SoSFMatrix);  

      outputMatrix.makeIdentity();
   }


// This is the evaluation routine.
   void IvTransformEngine::evaluate() {

      const SbVec3f& t = translation.getValue();
      const SbVec3f& s = scaleFactor.getValue();
      SbMatrix auxMatrix;
      SbMat mx;
      mx[0][1]=mx[0][2]=mx[0][3]=mx[1][0]=mx[1][2]=mx[1][3]=mx[2][0]=mx[2][1]=mx[2][3]=0.0f;
      mx[0][0]=s[0];
      mx[1][1]=s[1];
      mx[2][2]=s[2];
      mx[3][0]=t[0];
      mx[3][1]=t[1];
      mx[3][2]=t[2];
      mx[3][3]=1.0f;
      outputMatrix=inputMatrix.getValue();
      outputMatrix.multRight(mx);
      rotation.getValue().getValue(auxMatrix);
      outputMatrix.multRight(auxMatrix);
   
      // Set the matrix output value 
      SO_ENGINE_OUTPUT(transfMatrix, SoSFMatrix,setValue(outputMatrix));
   }


// Destructor. Does nothing.
   IvTransformEngine::~IvTransformEngine(){  
   }
