#ifndef IVTRANSFENGINE_H
#define IVTRANSFENGINE_H

#include <Inventor/engines/SoSubEngine.h>
#include <Inventor/fields/SoSFVec3f.h>
#include <Inventor/fields/SoSFRotation.h>
#include <Inventor/fields/SoSFMatrix.h>

/*! This engine takes a translation, scaling and rotation
   and outputs a 4x4 matrix representing the transformation
   resulting from applying the transformations in that order.
   Since Inventor pre-multiplies, the equivalent code in inventor it would be:
   ...
   parent->addChild(rotation);
   parent->addChild(scaleFactor);
   parent->addChild(translation);
   ...
   rotation->setValue(...);
   scaleFactor->setValue(...);
   translation->setValue(...);
*/
   class IvTransformEngine : public SoEngine {
   
      SO_ENGINE_HEADER(IvTransformEngine);
   
   public:
   

   /// engine input: a 4x4 matrix -- allows concatenating the output of an IvTransformEngine to the input of another IvTransformEngine
      SoSFMatrix   inputMatrix;   
   // The Ooher Engine Inputs:
      SoSFRotation rotation;
      SoSFVec3f	   scaleFactor;
      SoSFVec3f    translation;
   
   /// Engine Output: the scaled coordinate as a field
      SoEngineOutput  transfMatrix;  // (SoSFMatrix)
   
   /// This matrix contains the same value in transfMatrix, but available as a matrix to be used directly
      SbMatrix outputMatrix;

   // Initialization
      static void initClass();
   
   // Constructor
      IvTransformEngine();
   
   private:
   // Destructor
      virtual ~IvTransformEngine();
   
   // Evaluation method
      virtual void evaluate();
   };


#endif
