// sharedMemObj.cpp		A Shared Memory Object base class implementation


#include "SharedMemObj.h"
#include <errno.h>
#include <stdio.h>


extern int errno;

void* ObjectInSharedMemory::operator new(size_t blocksize, CREATE_SEGMENT, key_t shmkey) throw() {
        ObjectInSharedMemory *rtn;
	int shmsize, shmid;

	shmsize = blocksize;
     	shmid = shmget( shmkey , shmsize, (IPC_CREAT | 0600) );

     	if ( shmid == -1 ) {
          #ifndef DEBUG
            fprintf(stderr,"ObjectInSharedMemory::new(size_t,CREATE_SEGMENT,key_t) shared memory get (shmget) failed (errno = %i)\n",errno);
          #endif
		return NULL;
	}

     	if ( (int)(rtn = (ObjectInSharedMemory*)shmat( shmid, 0, SHM_RND) ) == -1 ) {
            fprintf(stderr,"ObjectInSharedMemory::new(size_t,CREATE_SEGMENT,key_t) shared memory attach (shmat) error (errno = %i)\n",errno);
            return NULL;
	}
	/* Here we're using rtn->shmid as a mutex, where -1 means 'NO ACCESS'.
	   This is to ensure that this constructor (which creates the segment),
	   and the constructor who only maps the segment do not try to access
	   rtn->ref_count at  the same time. rtn->ref_count has to be initialized
	   by this constructor before being incremented by the other constructor */
	rtn->shmid = -1;
	rtn->ref_count = 1;
	rtn->shmid = shmid;

	return rtn;
}

void* ObjectInSharedMemory::operator new(size_t blocksize)  throw() {
        ObjectInSharedMemory *rtn;
	int shmsize, shmid;
	shmsize = blocksize;
     	shmid = shmget( IPC_PRIVATE , shmsize, (IPC_CREAT | 0600) );
     	if ( shmid == -1 ) {
                #ifdef DEBUG
	      	  fprintf(stderr,"ObjectInSharedMemory::new(size_t) shared memory get (shmget) failed (errno = %i\n",errno);
                #endif
	      	return NULL;
	} else 	if ( (int)(rtn = (ObjectInSharedMemory*)shmat( shmid, 0, SHM_RND) ) == -1 ) {
            #ifdef DEBUG
              fprintf(stderr,"ObjectInSharedMemory::new(size_t)  shared memory attach (shmat) error (errno = %i)\n",errno);
            #endif
	    return NULL;
	}

	/* Here we're using rtn->shmid as a mutex, where -1 means 'NO ACCESS'.
	   This is to ensure that this constructor (which creates the segment),
	   and the constructor who only maps the segment do not try to access
	   rtn->ref_count at  the same time. rtn->ref_count has to be initialized
	   by this constructor before being incremented by the other constructor */
	rtn->shmid = -1;
	rtn->ref_count = 1;
	rtn->shmid = shmid;

	return rtn;
}


void* ObjectInSharedMemory::operator new(size_t, key_t existing_segment_shmkey)  throw() {
        ObjectInSharedMemory *rtn;
        int master_id = shmget(existing_segment_shmkey,0,0600);
        if(master_id == -1) {
            #ifdef DEBUG
              fprintf(stderr, "ObjectInSharedMemory::new(size_t,int) shared memory get (shget) error (errno =%i)\n",errno);
            #endif
	    return NULL;
        }
     	if ( (int)(rtn = (ObjectInSharedMemory*)shmat( master_id, 0, SHM_RND) ) == -1 ) {
            #ifdef DEBUG
              fprintf(stderr, "ObjectInSharedMemory::new(size_t,int) shared memory attach (shmat) error (errno =%i)\n",errno);
            #endif
	    return NULL;
	}

        /* we will use rtn->shmid as a mutex, were -1 means 'DO NOT ACCESS'.
	   The constructor that creates the shmem segment will set rtn->shmid to
	   the id value AFTER it initialized rtn->ref_count. We do this
	   to avoid a race condition between the two constructors */
	while(rtn->shmid == -1);
	// ok, now increment the referece count for this segment
	++rtn->ref_count;
	return rtn;
}

void ObjectInSharedMemory::operator delete(void *p) {
	ObjectInSharedMemory* ps = (ObjectInSharedMemory*)p;
	/* This is not safe, there is a potential race condition
	   in here, but it won't be fatal. In the worst case the segment
	   will be marked twice for deletion */
        if ( ps->ref_count > 0 )
	  --ps->ref_count;
	if(ps->ref_count <= 0)
	  shmctl(ps->shmid, IPC_RMID, NULL);
	shmdt( p );
}
