#ifndef OID_H
#define OID_H

#include <stdlib.h> // get definition for NULL

/*! An OID is a unique identifier for an object. It is possible to transform
  from OIDs to objects, and from objects to OIDs.
  The purpose of this class is to give an abstraction to work instead of pointers,
  particularly when some of the objects involved do not need to know wether an
  OID is a pointer, or a more complex structure. */
	struct OID {
	   /// Type of OID unique identifier
		typedef void* Type;
		/// A constant that defines an invalid value for an OID
   	#define BAD_OID NULL
		/*! Returns the object associated to an OID, that previously was obtained
			 by calling OID::fromObject(). */
		inline static void* toObject(Type anOID) { return (void*)anOID; }
		/*! Returns a unique identifier associated to an object. */
		inline static Type fromObject(void* anObject) { return (Type)anObject; } 
	};
	
#endif
