#ifndef _DISPLAYCONFIGTREE_H_
#define _DISPLAYCONFIGTREE_H_
#include <qlistview.h>
#include <string>
#include <vector>

char* format( char *s, const char *fmt, ... );

using namespace std;

class PerspectiveData
{
    public:
        PerspectiveData();
        
        void setFOV( float x, float y ) { m_fovx = x; m_fovy = y; };
        void setFOVX( float x ) { m_fovx = x; };
        void setFOVY( float y ) { m_fovy = y; };
        float getFOVX() { return m_fovx; };
        float getFOVY() { return m_fovy; };
        string type() { return "PERSPECTIVE"; };

    private:
        float m_fovx;
        float m_fovy;
};

class ScreenData
{
    public:
        ScreenData();

        void setName( string name ) { m_name = name; };
        string getName() { return m_name; };
        void setCenter( float x, float y, float z ) { m_centerx = x; m_centery = y; m_centerz = z; };
        void setCenterX( float x ) { m_centerx = x; };
        float getCenterX() { return m_centerx; };
        void setCenterY( float y ) { m_centerx = y; };
        float getCenterY() { return m_centery; };
        void setCenterZ( float z ) { m_centerx = z; };
        float getCenterZ() { return m_centerz; };
        void setHPR( float h, float p, float r ) { m_heading = h; m_pitch = p; m_roll = r; };
        void setHeading( float h ) { m_heading = h; };
        float getHeading() { return m_heading; };
        void setPitch( float p ) { m_pitch = p; };
        float getPitch() { return m_pitch; };
        void setRoll( float r ) { m_roll = r; };
        float getRoll() { return m_roll; };
        void setDisplayScale( float w, float h, float d ) { m_width = w; m_height = h; m_depth = d; };
        void setWidth( float w ) { m_width = w; };
        float getWidth() { return m_width; };
        void setHeight( float h ) { m_height = h; };
        float getHeight() { return m_height; };
        void setDepth( float d ) { m_depth = d; }
        float getDepth() { return m_depth; };
        string type() { return "SCREEN"; };

    private:
		string m_name;
        float m_centerx;
        float m_centery;
        float m_centerz;
        float m_heading;
        float m_pitch;
        float m_roll;
        float m_width;
        float m_height;
        float m_depth;
};

class CameraData
{
    public:
        enum PROJECTION_TYPE
        {
            SCREEN,
            PERSPECTIVE
        };
        enum EYE
		{
			MONO,
			LEFT,
			RIGHT,
			STEREO
		};

        CameraData();
        ~CameraData();

        void setTrackerName( string name ) { m_trackername = name; };
        string getTrackerName() { return m_trackername; };
        void setEye( int eye ) { m_eye = eye; };
        int getEye() { return m_eye; };
        void setProjectionType( int type ) { m_projectiontype = type; };
        int getProjectionType() { return m_projectiontype; };
        PerspectiveData* getPerspective() { return m_perspective; };
		ScreenData* getScreen() { return m_screen; };

        string type() { return "CAMERA"; };
    
    private:
        string m_trackername;
        int m_eye;
		int m_projectiontype;
        PerspectiveData* m_perspective;
		ScreenData* m_screen;
};

class DataListViewItem;

class Data
{
	public:
		Data( Data* parent, const string& name = "Root" );
        virtual ~Data() {};
        void setName( string name ) { m_name = name; };
        string getName() { return m_name; };
		virtual string type() { return "ROOT"; };
		Data* getParent() { return m_parent; };
		void setItem( DataListViewItem* item ) { m_item = item; };
		DataListViewItem* getItem() { return m_item; };

    private:
		Data* m_parent;
		DataListViewItem* m_item;
        string m_name;
};

class DataListViewItem : public QListViewItem
{
	public:
		DataListViewItem( QListView* parent, Data* data );
		DataListViewItem( DataListViewItem* parent, Data* data );
		~DataListViewItem();
		void setData( Data* data ) { m_data = data; };
		Data* getData() { return m_data; };

	private:
		Data* m_data;
};

class ViewportData : public Data
{
    public:
        ViewportData( Data* parent, const string& name = "Viewport0" );
		~ViewportData();

        void setLeft( int val ) { m_left = val; };
        int getLeft() { return m_left; };
        void setBottom( int val ) { m_bottom = val; };
        int getBottom() { return m_bottom; };
        void setWidth( int val ) { m_width = val; };
        int getWidth() { return m_width; };
        void setHeight( int val ) { m_height = val; };
        int getHeight() { return m_height; };
        void setCamera( CameraData* camera ) { m_camera = camera; };
        CameraData* getCamera() { return m_camera; };
        string type() { return "VIEWPORT"; };

    private:
        int m_left;
        int m_bottom;
        int m_width;
        int m_height;
        CameraData* m_camera;
};

class WindowData : public Data
{
    public:
        WindowData( Data* parent, const string& name = "Window0" );
		~WindowData();

        void setLeft( int val ) { m_left = val; };
        int getLeft() { return m_left; };
        void setBottom( int val ) { m_bottom = val; };
        int getBottom() { return m_bottom; };
        void setWidth( int val ) { m_width = val; };
        int getWidth() { return m_width; };
        void setHeight( int val ) { m_height = val; };
        int getHeight() { return m_height; };
        void setXScreen( int val ) { m_xscreen = val; };
        int getXScreen() { return m_xscreen; };
        void setQuadBuffer( bool buffered ) { m_quadbuffer = buffered; };
        bool getQuadBuffer() { return m_quadbuffer; };
        void setBorder( bool bordered ) { m_border = bordered; };
        bool getBorder() { return m_border; };
		void setFullScreen( bool fullscreen ) { m_fullscreen = fullscreen; };
		bool getFullScreen() { return m_fullscreen; };
        void addViewport( ViewportData* viewport ) { m_viewports.push_back( viewport ); };
        void removeViewport( ViewportData* viewport )
		{
			vector<ViewportData*>::iterator itr;
			for ( itr = m_viewports.begin(); itr != m_viewports.end(); itr++ )
				if ( *itr == viewport )
				{
					m_viewports.erase( itr );
					return;
				}
		};
        vector<ViewportData*> getViewports() { return m_viewports; };
        string type() { return "WINDOW"; };
		int getViewportNameNum() { return m_viewportnamenum++; };
		void setSharedContextWindow( string sharedcontextwindow ) { m_sharedcontextwindow = sharedcontextwindow; };
		string getSharedContextWIndow() { return m_sharedcontextwindow; };

    private:
		int m_viewportnamenum;
        int m_left;
        int m_bottom;
        int m_width;
        int m_height;
        int m_xscreen;
        bool m_quadbuffer;
        bool m_border;
		bool m_fullscreen;
		string m_sharedcontextwindow;
        vector<ViewportData*> m_viewports;
};

class DisplayData : public Data
{
    public:
        DisplayData( Data* parent, const string& name = "Display" );
		~DisplayData();

		void setFileName( const string& filename ) { m_filename = filename; };
		string getFileName() { return m_filename; };
        bool isChanged() { return m_changed; };
        void setChanged( bool changed ) { m_changed = changed; };
        void setThreaded( bool threaded ) { m_threaded = threaded; };
        bool getThreaded() { return m_threaded; };
        void addWindow( WindowData* window ) { m_windows.push_back( window ); };
        void removeWindow( WindowData* window )
		{
			vector<WindowData*>::iterator itr;
			for ( itr = m_windows.begin(); itr != m_windows.end(); itr++ )
				if ( *itr == window )
				{
					m_windows.erase( itr );
					return;
				}
		};
        vector<WindowData*> getWindows() { return m_windows; };
        string type() { return "DISPLAY"; };
		int getWindowNameNum() { return m_windownamenum++; };

    private:
		int m_windownamenum;
		string m_filename;
        bool m_changed;
        bool m_threaded;
        vector<WindowData*> m_windows;
};

class DisplayConfigTree : public QListView
{
    Q_OBJECT
    public:
        DisplayConfigTree( QWidget* parent );

        void newDisplay();
        void openDisplay();
        void saveDisplay( const string& filename = "" );
        void saveAsDisplay();
        void closeDisplay();
        void closeAllDisplays();
		void newWindow();
		void removeWindow();
		void newViewport();
		void removeViewport();

    private:
        vector<DisplayData*> m_displays;
        DataListViewItem* m_treeroot;
};

#endif
