#include "DisplayConfigWidget.h"
#include "DisplayConfigTree.h"
#include "DisplayWidget.h"
#include "WindowWidget.h"
#include "ViewportWidget.h"
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qstring.h>
#include <qcheckbox.h>
#include <qwidgetstack.h>
#include <qvalidator.h>
#include <qfont.h>
#include <qcombobox.h>
#include <qvgroupbox.h>
#include <qpopupmenu.h>

DisplayConfigWidget::DisplayConfigWidget( QWidget* parent )
    :QWidget( parent )
{
    init();
}

bool DisplayConfigWidget::init()
{
    QHBoxLayout* mainlayout = new QHBoxLayout( this, 5 );

    m_tree = new DisplayConfigTree( this );
    mainlayout->addWidget( m_tree );

    m_mainwidgetstack = new QWidgetStack( this );
    mainlayout->addWidget( m_mainwidgetstack );
    
	int stackorder = 0;

    QWidget* widget = new QWidget( this );
    m_mainwidgetstack->addWidget( widget, stackorder );
    QVBoxLayout* vlayout = new QVBoxLayout( widget, 5 );
    QLabel* label = new QLabel( "Choose New from the File menu", widget );
    label->setAlignment( Qt::AlignHCenter | Qt::AlignBottom );
    vlayout->addWidget( label );
    label = new QLabel( "to create a new display", widget );
    label->setAlignment( Qt::AlignHCenter | Qt::AlignTop );
    vlayout->addWidget( label );

	stackorder++;
	m_displaywidget = new DisplayWidget( this, stackorder, m_tree );
    m_mainwidgetstack->addWidget( m_displaywidget, stackorder );

	stackorder++;
    m_windowwidget = new WindowWidget( this, stackorder, m_tree );
	m_mainwidgetstack->addWidget( m_windowwidget, stackorder );

	stackorder++;
    m_viewportwidget = new ViewportWidget( this, stackorder, m_tree );
	m_mainwidgetstack->addWidget( m_viewportwidget, stackorder );

	m_rootmenu = new QPopupMenu( m_tree );
	m_rootmenu->insertItem( "Add Display", this, SLOT(newDisplay()) );
	m_rootmenu->insertItem( "Close All Displays", this, SLOT(closeAllDisplays()) );

	m_displaymenu = new QPopupMenu( m_tree );
	m_displaymenu->insertItem( "Save", this, SLOT(saveDisplay()) );
	m_displaymenu->insertItem( "Save As", this, SLOT(saveAsDisplay()) );
	m_displaymenu->insertItem( "Add Window", this, SLOT(newWindow()) );
	m_displaymenu->insertItem( "Close Display", this, SLOT(closeDisplay()) );

	m_windowmenu = new QPopupMenu( m_tree );
	m_windowmenu->insertItem( "Add Viewport", this, SLOT(newViewport()) );
	m_windowmenu->insertItem( "Remove Window", this, SLOT(removeWindow()) );

	m_viewportmenu = new QPopupMenu( m_tree );
	m_viewportmenu->insertItem( "Remove Viewport", this, SLOT(removeViewport()) );

    m_mainwidgetstack->raiseWidget( 0 );
    return true;
}

void DisplayConfigWidget::newDisplay()
{
    m_tree->newDisplay();
}

void DisplayConfigWidget::openDisplay()
{
	m_tree->openDisplay();
}

void DisplayConfigWidget::saveDisplay()
{
	m_tree->saveDisplay();
}

void DisplayConfigWidget::saveAsDisplay()
{
	m_tree->saveAsDisplay();
}

void DisplayConfigWidget::closeDisplay()
{
	m_tree->closeDisplay();
}

void DisplayConfigWidget::closeAllDisplays()
{
	m_tree->closeAllDisplays();
}

void DisplayConfigWidget::newWindow()
{
	m_tree->newWindow();
}

void DisplayConfigWidget::removeWindow()
{
	m_tree->removeWindow();
}

void DisplayConfigWidget::newViewport()
{
	m_tree->newViewport();
}

void DisplayConfigWidget::removeViewport()
{
	m_tree->removeViewport();
}

void DisplayConfigWidget::updateSelection( QListViewItem* item )
{
	if ( !item )
		return;
	string type = ((DataListViewItem*)item)->getData()->type();
	if ( type == "ROOT" )
		m_mainwidgetstack->raiseWidget( 0 );
	else if ( type == "DISPLAY" )
		m_displaywidget->update( (DisplayData*)((DataListViewItem*)item)->getData() );
	else if ( type == "WINDOW" )
		m_windowwidget->update( (WindowData*)((DataListViewItem*)item)->getData() );
	else if ( type == "VIEWPORT" )
		m_viewportwidget->update( (ViewportData*)((DataListViewItem*)item)->getData() );
}

void DisplayConfigWidget::itemContextMenu( QListViewItem* item, const QPoint& point, int column )
{
	if ( !item || column < 0 )
		return;
	string type = ((DataListViewItem*)item)->getData()->type();
	if ( type == "ROOT" )
	{
		m_rootmenu->popup( point, 0 );
	}
	else if ( type == "DISPLAY" )
	{
		m_displaymenu->popup( point, 0 );
	}
	else if ( type == "WINDOW" )
	{
		m_windowmenu->popup( point, 0 );
	}
	else if ( type == "VIEWPORT" )
	{
		m_viewportmenu->popup( point, 0 );
	}
}
