#include "DisplayWidget.h"
#include "DisplayConfigTree.h"
#include "DisplayConfigWidget.h"
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qfont.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qwidgetstack.h>
#include <qstring.h>

DisplayWidget::DisplayWidget( QWidget* parent, int stackorder, DisplayConfigTree* tree )
:QWidget( parent )
{
	m_parent = parent;
	m_stackorder = stackorder;
	m_tree = tree;

    QVBoxLayout* vlayout = new QVBoxLayout( this, 5 );
    QVGroupBox* groupbox = new QVGroupBox( "Display", this );
    QFont font = groupbox->font();
    QFont bigfont( font );
    bigfont.setPointSize( (int)(bigfont.pointSize() * 1.5) );
    groupbox->setFont( bigfont );
    vlayout->addWidget( groupbox );
    QWidget* widget = new QWidget( groupbox );
    widget->setFont( font );
    vlayout = new QVBoxLayout( widget, 5 );
    QHBoxLayout* layout = new QHBoxLayout( vlayout, 5 );

    QLabel* label = new QLabel( "Display Name:", widget );
    layout->addWidget( label );

    m_name = new QLineEdit( "Display", widget, "name" );
    layout->addWidget( m_name );
    connect( m_name, SIGNAL( textChanged( const QString& ) ),
            this, SLOT( nameChanged( const QString& ) ) );

    layout = new QHBoxLayout( vlayout, 5 );
    m_threaded = new QCheckBox( "Threaded", widget, "threaded" );
    layout->addWidget( m_threaded );
    connect( m_threaded, SIGNAL( toggled( bool ) ),
            this, SLOT( threadedToggled( bool ) ) );
}

void DisplayWidget::update( DisplayData* display )
{
	if ( !display )
		return;

	m_display = display;
	((DisplayConfigWidget*)m_parent)->getMainWidgetStack()->raiseWidget( this );
	m_name->setText( m_display->getName().c_str() );
	m_threaded->setChecked( m_display->getThreaded() );
}

void DisplayWidget::nameChanged( const QString& name )
{
	m_display->setName( name.latin1() );
    m_display->getItem()->setText( 1, name );
    m_display->setChanged( true );
}

void DisplayWidget::threadedToggled( bool threaded )
{
	m_display->setThreaded( threaded );
    m_display->setChanged( true );
}
