#include "ViewportWidget.h"
#include "DisplayConfigTree.h"
#include "DisplayConfigWidget.h"
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qfont.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qcombobox.h>
#include <qwidgetstack.h>

ViewportWidget::ViewportWidget( QWidget* parent, int stackorder, DisplayConfigTree* tree )
{
	m_parent = parent;
	m_stackorder = stackorder;
	m_tree = tree;

    QVBoxLayout* vlayout = new QVBoxLayout( this, 5 );
    QVGroupBox* groupbox = new QVGroupBox( "Viewport", this );
    QFont font = groupbox->font();
    QFont bigfont( font );
    bigfont.setPointSize( (int)(bigfont.pointSize() * 1.5) );
    groupbox->setFont( bigfont );
    vlayout->addWidget( groupbox );
    QWidget* widget = new QWidget( groupbox );
    widget->setFont( font );
    vlayout = new QVBoxLayout( widget, 5 );
    QHBoxLayout* hlayout = new QHBoxLayout( vlayout );

    QLabel* label = new QLabel( "Name:", widget );
    hlayout->addWidget( label );
    m_name = new QLineEdit( "Viewport", widget );
    hlayout->addWidget( m_name );
    connect( m_name, SIGNAL( textChanged( const QString& ) ),
            this, SLOT( nameChanged( const QString& ) ) );

    hlayout = new QHBoxLayout( vlayout );
    groupbox = new QVGroupBox( "Size", widget );
    hlayout->addWidget( groupbox );
    QWidget* subwidget = new QWidget( groupbox );

    QGridLayout* gridlayout = new QGridLayout( subwidget, 2, 4, 5 );
    label = new QLabel( "Left:", subwidget );
    label->setAlignment( Qt::AlignRight );
    gridlayout->addWidget( label, 0, 0 );
    m_sizeleft = new QLineEdit( "0", subwidget );
    QIntValidator* validator = new QIntValidator( m_sizeleft );
	m_sizeleft->setValidator( validator );
    gridlayout->addWidget( m_sizeleft, 0, 1 );
	connect( m_sizeleft, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( sizeleftChanged( const QString& ) ) );

    label = new QLabel( "Width:", subwidget );
    label->setAlignment( Qt::AlignRight );
    gridlayout->addWidget( label, 0, 2 );
    m_sizewidth = new QLineEdit( "640", subwidget );
    validator = new QIntValidator( m_sizewidth );
	m_sizewidth->setValidator( validator );
    gridlayout->addWidget( m_sizewidth, 0, 3 );
	connect( m_sizewidth, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( sizewidthChanged( const QString& ) ) );

    label = new QLabel( "Bottom:", subwidget );
    label->setAlignment( Qt::AlignRight );
    gridlayout->addWidget( label, 1, 0 );
    m_sizebottom = new QLineEdit( "0", subwidget );
    validator = new QIntValidator( m_sizebottom );
	m_sizebottom->setValidator( validator );
    gridlayout->addWidget( m_sizebottom, 1, 1 );
	connect( m_sizebottom, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( sizebottomChanged( const QString& ) ) );

    label = new QLabel( "Height:", subwidget );
    label->setAlignment( Qt::AlignRight );
    gridlayout->addWidget( label, 1, 2 );
    m_sizeheight = new QLineEdit( "480", subwidget );
    validator = new QIntValidator( m_sizeheight );
	m_sizeheight->setValidator( validator );
    gridlayout->addWidget( m_sizeheight, 1, 3 );
	connect( m_sizeheight, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( sizeheightChanged( const QString& ) ) );

    groupbox = new QVGroupBox( "Camera", widget );
    groupbox->setFont( bigfont );
    vlayout->addWidget( groupbox );
    widget = new QWidget( groupbox );
    widget->setFont( font );
    vlayout = new QVBoxLayout( widget, 5 );
    hlayout = new QHBoxLayout( vlayout );

    label = new QLabel( "Tracker Name:", widget );
    hlayout->addWidget( label );
    m_trackername = new QLineEdit( "headtracker", widget );
    hlayout->addWidget( m_trackername );
	connect( m_trackername, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( trackernameChanged( const QString& ) ) );

    hlayout = new QHBoxLayout( vlayout );
    label = new QLabel( "Type:", widget );
    label->setAlignment( Qt::AlignRight );
    hlayout->addWidget( label );
    m_projectiontype = new QComboBox( widget );
    hlayout->addWidget( m_projectiontype );
	connect( m_projectiontype, SIGNAL( activated( int ) ),
			this, SLOT( projectiontypeChanged( int ) ) );

    m_projectiontype->insertItem( "SCREEN" );
    m_projectiontype->insertItem( "PROJECTION" );

    label = new QLabel( "Eye:", widget );
    label->setAlignment( Qt::AlignRight );
    hlayout->addWidget( label );
    m_eye = new QComboBox( widget );
    hlayout->addWidget( m_eye );
	connect( m_eye, SIGNAL( activated( int ) ),
			this, SLOT( eyeChanged( int ) ) );

    m_eye->insertItem( "MONO" );
    m_eye->insertItem( "LEFT" );
    m_eye->insertItem( "RIGHT" );
    m_eye->insertItem( "STEREO" );

    hlayout = new QHBoxLayout( vlayout );
    m_projectionwidgetstack = new QWidgetStack( widget );
    hlayout->addWidget( m_projectionwidgetstack );

    m_screenwidget = new QWidget( this );
    m_projectionwidgetstack->addWidget( m_screenwidget, 0 );
    vlayout = new QVBoxLayout( m_screenwidget, 5 );
    groupbox = new QVGroupBox( "Screen", m_screenwidget );
    groupbox->setFont( bigfont );
    vlayout->addWidget( groupbox );
    widget = new QWidget( groupbox );
    widget->setFont( font );
    vlayout = new QVBoxLayout( widget, 5 );

    hlayout = new QHBoxLayout( vlayout );
    label = new QLabel( "Screen Name:", widget );
    hlayout->addWidget( label );
    m_screenname = new QLineEdit( "Screen", widget );
    hlayout->addWidget( m_screenname );
    connect( m_screenname, SIGNAL( textChanged( const QString& ) ),
            this, SLOT( screenNameChanged( const QString& ) ) );

    hlayout = new QHBoxLayout( vlayout );
    groupbox = new QVGroupBox( "Center", widget );
    hlayout->addWidget( groupbox );
    subwidget = new QWidget( groupbox );

    gridlayout = new QGridLayout( subwidget, 2, 3, 5 );
    label = new QLabel( "x:", subwidget );
    label->setAlignment( Qt::AlignHCenter );
    gridlayout->addWidget( label, 0, 0 );
    m_centerx = new QLineEdit( "0.0", subwidget );
    QDoubleValidator* doublevalidator = new QDoubleValidator( m_centerx );
	m_centerx->setValidator( doublevalidator );
    gridlayout->addWidget( m_centerx, 1, 0 );
	connect( m_centerx, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( centerxChanged( const QString& ) ) );

    label = new QLabel( "y:", subwidget );
    label->setAlignment( Qt::AlignHCenter );
    gridlayout->addWidget( label, 0, 1 );
    m_centery = new QLineEdit( "1.0", subwidget );
    doublevalidator = new QDoubleValidator( m_centery );
	m_centery->setValidator( doublevalidator );
    gridlayout->addWidget( m_centery, 1, 1 );
	connect( m_centery, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( centeryChanged( const QString& ) ) );

    label = new QLabel( "z:", subwidget );
    label->setAlignment( Qt::AlignHCenter );
    gridlayout->addWidget( label, 0, 2 );
    m_centerz = new QLineEdit( "0.0", subwidget );
    doublevalidator = new QDoubleValidator( m_centerz );
	m_centerz->setValidator( doublevalidator );
    gridlayout->addWidget( m_centerz, 1, 2 );
	connect( m_centerz, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( centerzChanged( const QString& ) ) );

    hlayout = new QHBoxLayout( vlayout );
    groupbox = new QVGroupBox( "Orientation", widget );
    hlayout->addWidget( groupbox );
    subwidget = new QWidget( groupbox );

    gridlayout = new QGridLayout( subwidget, 2, 3, 5 );
    label = new QLabel( "Heading:", subwidget );
    label->setAlignment( Qt::AlignHCenter );
    gridlayout->addWidget( label, 0, 0 );
    m_heading = new QLineEdit( "0.0", subwidget );
    doublevalidator = new QDoubleValidator( -180.0, 180.0, 2, m_heading );
	m_heading->setValidator( doublevalidator );
    gridlayout->addWidget( m_heading, 1, 0 );
	connect( m_heading, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( headingChanged( const QString& ) ) );

    label = new QLabel( "Pitch:", subwidget );
    label->setAlignment( Qt::AlignHCenter );
    gridlayout->addWidget( label, 0, 1 );
    m_pitch = new QLineEdit( "0.0", subwidget );
    doublevalidator = new QDoubleValidator( -90.0, 90.0, 2, m_pitch );
	m_pitch->setValidator( doublevalidator );
    gridlayout->addWidget( m_pitch, 1, 1 );
	connect( m_pitch, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( pitchChanged( const QString& ) ) );

    label = new QLabel( "Roll:", subwidget );
    label->setAlignment( Qt::AlignHCenter );
    gridlayout->addWidget( label, 0, 2 );
    m_roll = new QLineEdit( "0.0", subwidget );
    doublevalidator = new QDoubleValidator( -180.0, 180.0, 2, m_roll );
	m_roll->setValidator( doublevalidator );
    gridlayout->addWidget( m_roll, 1, 2 );
	connect( m_roll, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( rollChanged( const QString& ) ) );

    hlayout = new QHBoxLayout( vlayout );
    groupbox = new QVGroupBox( "Meters", widget );
    hlayout->addWidget( groupbox );
    subwidget = new QWidget( groupbox );

    gridlayout = new QGridLayout( subwidget, 2, 3, 5 );
    label = new QLabel( "Width:", subwidget );
    label->setAlignment( Qt::AlignHCenter );
    gridlayout->addWidget( label, 0, 0 );
    m_width = new QLineEdit( "1.0", subwidget );
    doublevalidator = new QDoubleValidator( m_roll );
	m_width->setValidator( doublevalidator );
    gridlayout->addWidget( m_width, 1, 0 );
	connect( m_width, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( widthChanged( const QString& ) ) );

    label = new QLabel( "Height:", subwidget );
    label->setAlignment( Qt::AlignHCenter );
    gridlayout->addWidget( label, 0, 1 );
    m_height = new QLineEdit( "1.0", subwidget );
    doublevalidator = new QDoubleValidator( m_roll );
	m_height->setValidator( doublevalidator );
    gridlayout->addWidget( m_height, 1, 1 );
	connect( m_height, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( heightChanged( const QString& ) ) );

    label = new QLabel( "Depth:", subwidget );
    label->setAlignment( Qt::AlignHCenter );
    gridlayout->addWidget( label, 0, 2 );
    m_depth = new QLineEdit( "1.0", subwidget );
    doublevalidator = new QDoubleValidator( m_roll );
	m_depth->setValidator( doublevalidator );
    gridlayout->addWidget( m_depth, 1, 2 );
	connect( m_depth, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( depthChanged( const QString& ) ) );

    m_perspectivewidget = new QWidget( this );
    m_projectionwidgetstack->addWidget( m_perspectivewidget, 1 );
    vlayout = new QVBoxLayout( m_perspectivewidget, 5 );
    groupbox = new QVGroupBox( "Projection", m_perspectivewidget );
    groupbox->setFont( bigfont );
    vlayout->addWidget( groupbox );
    widget = new QWidget( groupbox );
    widget->setFont( font );
    vlayout = new QVBoxLayout( widget, 5 );
    hlayout = new QHBoxLayout( vlayout );

	label = new QLabel( "FOVx", widget );
	hlayout->addWidget( label );
	m_fovx = new QLineEdit( "35.0", widget );
	hlayout->addWidget( m_fovx );
	connect( m_fovx, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( fovxChanged( const QString& ) ) );

    hlayout = new QHBoxLayout( vlayout );
	label = new QLabel( "FOVy", widget );
	hlayout->addWidget( label );
	m_fovy = new QLineEdit( "26.25", widget );
	hlayout->addWidget( m_fovy );
	connect( m_fovy, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( fovyChanged( const QString& ) ) );
}

void ViewportWidget::update( ViewportData* viewport )
{
	if ( !viewport )
		return;

	m_viewport = viewport;
	char temp[32];
	CameraData* camera = m_viewport->getCamera();
	ScreenData* screen = camera->getScreen();
	PerspectiveData* perspective = camera->getPerspective();
	int type = camera->getProjectionType();
	int eye = camera->getEye();

	m_name->setText( m_viewport->getName().c_str() );
	m_sizeleft->setText( format( temp, "%d", m_viewport->getLeft() ) );
	m_sizebottom->setText( format( temp, "%d", m_viewport->getBottom() ) );
	m_sizewidth->setText( format( temp, "%d", m_viewport->getWidth() ) );
	m_sizeheight->setText( format( temp, "%d", m_viewport->getHeight() ) );

	m_trackername->setText( camera->getTrackerName().c_str() );
	m_eye->setCurrentItem( camera->getEye() );
	m_projectiontype->setCurrentItem( type );
	m_centerx->setText( format( temp, "%-12.10f", screen->getCenterX() ) );
	m_centery->setText( format( temp, "%-12.10f", screen->getCenterY() ) );
	m_centerz->setText( format( temp, "%-12.10f", screen->getCenterZ() ) );
	m_heading->setText( format( temp, "%-12.10f", screen->getHeading() ) );
	m_pitch->setText( format( temp, "%-12.10f", screen->getPitch() ) );
	m_roll->setText( format( temp, "%-12.10f", screen->getRoll() ) );
	m_width->setText( format( temp, "%-12.10f", screen->getWidth() ) );
	m_height->setText( format( temp, "%-12.10f", screen->getHeight() ) );
	m_depth->setText( format( temp, "%-12.10f", screen->getDepth() ) );
	m_fovx->setText( format( temp, "%-12.10f", perspective->getFOVX() ) );
	m_fovy->setText( format( temp, "%-12.10f", perspective->getFOVY() ) );

	((DisplayConfigWidget*)m_parent)->getMainWidgetStack()->raiseWidget( this );
	m_projectionwidgetstack->raiseWidget( type );
}

void ViewportWidget::nameChanged( const QString& name )
{
	vector<ViewportData*> viewports = ((WindowData*)m_viewport->getParent())->getViewports();
	for ( int i = 0; i < viewports.size(); i++ )
	{
		if ( viewports[i] == m_viewport )
			continue;
		if ( viewports[i]->getName() == name.latin1() )
		{
			m_name->setText( m_viewport->getName().c_str() );
			return;
		}
	}
	m_viewport->setName( name.latin1() );
    m_viewport->getItem()->setText( 1, name );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::sizeleftChanged( const QString& left )
{
	m_viewport->setLeft( atoi( left ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::sizebottomChanged( const QString& bottom )
{
	m_viewport->setBottom( atoi( bottom ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::sizewidthChanged( const QString& width )
{
	m_viewport->setWidth( atoi( width ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::sizeheightChanged( const QString& height )
{
	m_viewport->setHeight( atoi( height ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::trackernameChanged( const QString& trackername )
{
	m_viewport->getCamera()->setTrackerName( trackername.latin1() );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::projectiontypeChanged( int projectiontype )
{
	m_viewport->getCamera()->setProjectionType( projectiontype );
	m_projectionwidgetstack->raiseWidget( projectiontype );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::eyeChanged( int eye )
{
	m_viewport->getCamera()->setEye( eye );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::screenNameChanged( const QString& screenname )
{
	m_viewport->getCamera()->getScreen()->setName( screenname.latin1() );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::centerxChanged( const QString& centerx )
{
	m_viewport->getCamera()->getScreen()->setCenterX( atof( centerx.latin1() ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::centeryChanged( const QString& centery )
{
	m_viewport->getCamera()->getScreen()->setCenterY( atof( centery.latin1() ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::centerzChanged( const QString& centerz )
{
	m_viewport->getCamera()->getScreen()->setCenterZ( atof( centerz.latin1() ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::headingChanged( const QString& heading )
{
	m_viewport->getCamera()->getScreen()->setCenterY( atof( heading ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::pitchChanged( const QString& pitch )
{
	m_viewport->getCamera()->getScreen()->setCenterY( atof( pitch ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::rollChanged( const QString& roll )
{
	m_viewport->getCamera()->getScreen()->setCenterY( atof( roll ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::widthChanged( const QString& width )
{
	m_viewport->getCamera()->getScreen()->setCenterY( atof( width ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::heightChanged( const QString& height )
{
	m_viewport->getCamera()->getScreen()->setCenterY( atof( height ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::depthChanged( const QString& depth )
{
	m_viewport->getCamera()->getScreen()->setCenterY( atof( depth ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::fovxChanged( const QString& fovx )
{
	m_viewport->getCamera()->getPerspective()->setFOVX( atof( fovx ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}

void ViewportWidget::fovyChanged( const QString& fovy )
{
	m_viewport->getCamera()->getPerspective()->setFOVX( atof( fovy ) );
    ((DisplayData*)m_viewport->getParent()->getParent())->setChanged( true );
}
