#include "WindowWidget.h"
#include "DisplayConfigTree.h"
#include "DisplayConfigWidget.h"
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qfont.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qcheckbox.h>
#include <qwidgetstack.h>
#include <qcombobox.h>

WindowWidget::WindowWidget( QWidget* parent, int stackorder, DisplayConfigTree* tree )
{
	m_parent = parent;
	m_stackorder = stackorder;
	m_tree = tree;

    QVBoxLayout* vlayout = new QVBoxLayout( this, 5 );
    QVGroupBox* groupbox = new QVGroupBox( "Window", this );
    QFont font = groupbox->font();
    QFont bigfont( font );
    bigfont.setPointSize( (int)(bigfont.pointSize() * 1.5) );
    groupbox->setFont( bigfont );
    vlayout->addWidget( groupbox );
    QWidget* widget = new QWidget( groupbox );
    widget->setFont( font );
    vlayout = new QVBoxLayout( widget, 5 );
    QHBoxLayout* hlayout = new QHBoxLayout( vlayout );

    QLabel* label = new QLabel( "Name:", widget );
    hlayout->addWidget( label );
    m_name = new QLineEdit( "Window", widget, "name" );
    hlayout->addWidget( m_name );
    connect( m_name, SIGNAL( textChanged( const QString& ) ),
            this, SLOT( nameChanged( const QString& ) ) );

    hlayout = new QHBoxLayout( vlayout );
    groupbox = new QVGroupBox( "Size", widget );
    hlayout->addWidget( groupbox );
    QWidget* subwidget = new QWidget( groupbox );

    QGridLayout* gridlayout = new QGridLayout( subwidget, 2, 4, 5 );
    label = new QLabel( "Left:", subwidget );
    label->setAlignment( Qt::AlignRight );
    gridlayout->addWidget( label, 0, 0 );
    m_sizeleft = new QLineEdit( "0", subwidget, "left" );
    QIntValidator* validator = new QIntValidator( m_sizeleft );
	m_sizeleft->setValidator( validator );
    gridlayout->addWidget( m_sizeleft, 0, 1 );
	connect( m_sizeleft, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( sizeleftChanged( const QString& ) ) );

    label = new QLabel( "Width:", subwidget );
    label->setAlignment( Qt::AlignRight );
    gridlayout->addWidget( label, 0, 2 );
    m_sizewidth = new QLineEdit( "640", subwidget, "width" );
    validator = new QIntValidator( m_sizewidth );
	m_sizewidth->setValidator( validator );
    gridlayout->addWidget( m_sizewidth, 0, 3 );
	connect( m_sizewidth, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( sizewidthChanged( const QString& ) ) );

    label = new QLabel( "Bottom:", subwidget );
    label->setAlignment( Qt::AlignRight );
    gridlayout->addWidget( label, 1, 0 );
    m_sizebottom = new QLineEdit( "0", subwidget, "bottom" );
    validator = new QIntValidator( m_sizebottom );
	m_sizebottom->setValidator( validator );
    gridlayout->addWidget( m_sizebottom, 1, 1 );
	connect( m_sizebottom, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( sizebottomChanged( const QString& ) ) );

    label = new QLabel( "Height:", subwidget );
    label->setAlignment( Qt::AlignRight );
    gridlayout->addWidget( label, 1, 2 );
    m_sizeheight = new QLineEdit( "480", subwidget, "height" );
    validator = new QIntValidator( m_sizeheight );
	m_sizeheight->setValidator( validator );
    gridlayout->addWidget( m_sizeheight, 1, 3 );
	connect( m_sizeheight, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( sizeheightChanged( const QString& ) ) );

    hlayout = new QHBoxLayout( vlayout );
    label = new QLabel( "XScreen:", widget );
    hlayout->addWidget( label );
    m_xscreen = new QLineEdit( "0", widget );
    validator = new QIntValidator( 0, 256, m_xscreen, "xscreen" );
	m_xscreen->setValidator( validator );
    hlayout->addWidget( m_xscreen );
	connect( m_xscreen, SIGNAL( textChanged( const QString& ) ),
			this, SLOT( xscreenChanged( const QString& ) ) );

    hlayout = new QHBoxLayout( vlayout );
    m_quadbuffer = new QCheckBox( "Quadbuffer ( video card support for stereo ):", widget, "quadbuffer" );
    hlayout->addWidget( m_quadbuffer );
	connect( m_quadbuffer, SIGNAL( toggled( bool ) ),
			this, SLOT( quadbufferChanged( bool ) ) );

    hlayout = new QHBoxLayout( vlayout );
    m_border = new QCheckBox( "Border:", widget, "border" );
    hlayout->addWidget( m_border );
	connect( m_border, SIGNAL( toggled( bool ) ),
			this, SLOT( borderChanged( bool ) ) );

    hlayout = new QHBoxLayout( vlayout );
    m_fullscreen = new QCheckBox( "Fullscreen:", widget, "border" );
    hlayout->addWidget( m_fullscreen );
	connect( m_border, SIGNAL( toggled( bool ) ),
			this, SLOT( fullScreenChanged( bool ) ) );

    hlayout = new QHBoxLayout( vlayout );
    label = new QLabel( "Shared Context Parent:", widget );
    hlayout->addWidget( label );
    m_sharedcontextwindow = new QComboBox( widget, "sharedcontextwindow" );
    hlayout->addWidget( m_sharedcontextwindow );
	connect( m_sharedcontextwindow, SIGNAL( activated( const QString& ) ),
			this, SLOT( sharedContextWindowChanged( const QString& ) ) );
}

void WindowWidget::update( WindowData* window )
{
	if ( !window )
		return;

	m_window = window;
	char temp[32];
	m_name->setText( m_window->getName().c_str() );
	m_sizeleft->setText( format( temp, "%d", m_window->getLeft() ) );
	m_sizebottom->setText( format( temp, "%d", m_window->getBottom() ) );
	m_sizewidth->setText( format( temp, "%d", m_window->getWidth() ) );
	m_sizeheight->setText( format( temp, "%d", m_window->getHeight() ) );
	m_xscreen->setText( format( temp, "%d", m_window->getXScreen() ) );
	m_quadbuffer->setChecked( m_window->getQuadBuffer() );
	m_border->setChecked( m_window->getBorder() );
	m_fullscreen->setChecked( m_window->getFullScreen() );
	m_sharedcontextwindow->clear();
	m_sharedcontextwindow->insertItem( "" );
	int index = 0, curindex = 1;
	vector<WindowData*> windows = ((DisplayData*)m_window->getParent())->getWindows();
	for ( int i = 0; i < windows.size(); i++ )
	{
		if ( windows[i] == m_window )
			continue;
		if ( windows[i]->getName() == m_window->getName() )
			index = curindex;
		m_sharedcontextwindow->insertItem( windows[i]->getName().c_str() );
		curindex++;
	}
	if ( index == 0 )
		m_window->setSharedContextWindow( "" );
	m_sharedcontextwindow->setCurrentItem( index );
	((DisplayConfigWidget*)m_parent)->getMainWidgetStack()->raiseWidget( this );
}

void WindowWidget::nameChanged( const QString& name )
{
	vector<WindowData*> windows = ((DisplayData*)m_window->getParent())->getWindows();
	for ( int i = 0; i < windows.size(); i++ )
	{
		if ( windows[i] == m_window )
			continue;
		if ( windows[i]->getName() == name.latin1() )
		{
			m_name->setText( m_window->getName().c_str() );
			return;
		}
	}
	m_window->setName( name.latin1() );
    m_window->getItem()->setText( 1, name );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}

void WindowWidget::sizeleftChanged( const QString& left )
{
	m_window->setLeft( atoi( left ) );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}

void WindowWidget::sizebottomChanged( const QString& bottom )
{
	m_window->setBottom( atoi( bottom ) );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}

void WindowWidget::sizewidthChanged( const QString& width )
{
	m_window->setWidth( atoi( width ) );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}

void WindowWidget::sizeheightChanged( const QString& height )
{
	m_window->setHeight( atoi( height ) );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}

void WindowWidget::xscreenChanged( const QString& xscreen )
{
	m_window->setXScreen( atoi( xscreen ) );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}

void WindowWidget::quadbufferChanged( bool quadbuffer )
{
	m_window->setQuadBuffer( quadbuffer );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}

void WindowWidget::borderChanged( bool border )
{
	m_window->setBorder( border );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}

void WindowWidget::fullScreenChanged( bool fullscreen )
{
	m_window->setFullScreen( fullscreen );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}

void WindowWidget::sharedContextWindowChanged( const QString& sharedcontextwindow )
{
	m_window->setSharedContextWindow( sharedcontextwindow.latin1() );
    ((DisplayData*)m_window->getParent())->setChanged( true );
}
