//=============================================================================
// diverse-config.cpp
// - this file defines an application that can be used to access the 
// configured state of diverse including:
// 
//
// Created 5/12/03 By Daniel Larimer modified by Andrew A. Ray
//=============================================================================
#include<iostream>
#include<string>
using namespace std;

#define INSTALLDIR "/usr/local/diverse"
#define CXX "g++"
#define CXXFLAGS "-Wall -O2"
#define INCDIRS ""
#define LIBDIRS ""
#define LIBS "-ldiverse -ldmath -ldutil -ldreg -lservicemgr -ltracer -lpluginmgr -ldisplaymgr -leventmgr -lfontmgr -lnavmgr"
#define LINKLINE "g++ -shared -Wl,-rpath -Wl,/usr/local/diverse/lib -Wl,-rpath -Wl,/usr/local/lib "
#define LINKPLUG "g++ -shared"
#define DEFINES "-D_LINUX_ -DQT_CLEAN_NAMESPACE"
#define VERSION "3.0Beta"
#define PATH "/usr/local/diverse/bin"
#define QTLIBDIR ""
#define QTINC ""
#ifdef __APPLE__
#define DGLLIB "-ldgl -ldglcore -lOpenThreads -lProducer -ldxml -framework OpenGL"
#else
#define DGLLIB "-ldgl -lGL -lGLU"
#endif

#define DIVERSELIB "-ldiverse"
#define PRODUCERLIBS "-L/usr/local/lib"
#define VTKLIBS ""
#define VTKLIBDIR ""
#define VTKINCDIR ""
#define COINLIBS ""
#define COINLIBDIR ""
#define COININCDIR ""
#define DOSGLIBS ""
#define DOSGINCDIR ""
#define LIBFLAGS "-Wl,-rpath -Wl,/usr/local/diverse/lib -Wl,-rpath -Wl,/usr/local/lib "
#define LIBDIR "-L/usr/local/diverse/lib"
#define INCDIR  "-I/usr/local/diverse/include"
#define LIBFLAGS "-Wl,-rpath -Wl,/usr/local/diverse/lib -Wl,-rpath -Wl,/usr/local/lib "
#define SOUNDINC "-I/usr/local/diverse/include/fmod-3.63"
#define SOUNDLIB "-lfmod-3.63"

void print_usage();
void print_val( const string& val );
int main( int argc, char** argv)
{
    if( argc <= 1 )
    {
        print_usage();
        return 0;
    }
        if (string(argv[1]) == "--test")
        {
            return 1;
        }
    for( int i = 1; i < argc; i++ )
    {
            print_val( argv[i] ); 
    }
    return 0;
}   

void print_val( const string& val )
{
    if( val == "--about" )
    {
        cout<<"DIVERSE is an open source API for Device Independent\n"
            <<"Virtual Environments, Reusable, Scalable, and Extensible.\n\n";
        cout<<"http://diverse.sourceforge.net/\n\n";
    }
    else if (val == "--compiler" )
    {
        cout<<CXX<<endl;
    }
    else if (val == "--cxxflags" )
    {
        cout<<CXXFLAGS<<endl;
    }
    else if (val =="--sound-include")
    {
	    cout << SOUNDINC << endl;
    }
    else if (val =="--sound-libs")
    {
	    cout << SOUNDLIB<< endl;
    }
    else if (val == "--defines" )
    {
        cout<<DEFINES<<endl;
    }
    else if (val == "--install-dir" )
    {
        cout<<INSTALLDIR<<endl;
    }
    else if (val == "--lib-dirs" )
    {
        cout<<LIBDIRS<<endl;
    }
    else if (val == "--version" )
    {
        cout<<VERSION<<endl;
    }
    else if (val == "--help")
    {
        print_usage();
    }
    else if(val =="--coin-include")
    {
            cout << COININCDIR << endl;
    }
    else if (val =="--coin-libs")
    {
	    cout << COINLIBDIR << " " << COINLIBS << endl;
    }
    else if (val == "--vtk-libs")
    {
	    cout << VTKLIBS << " " << VTKLIBDIR << endl;
    }
    else if (val == "--vtk-include")
    {
	    cout << VTKINCDIR << endl;
    }
    else if (val == "--dosg-libs")
    {
	    cout << DOSGLIBS << endl;
    }
    else if (val == "--dosg-include")
    {
	    cout << DOSGINCDIR << endl;
    }
    else if (val == "--include")
    {
	    cout << INCDIR << endl;
    }
    else if (val == "--libs")
    {
	    cout << LIBDIR <<" " <<LIBS<< " "<<  LIBFLAGS<< endl;
    }
    else if (val == "--dgl-libs")
    {
	    cout << DGLLIB << endl;
    }
    else if (val == "--qt-libs")
    {
	    cout << QTLIBDIR << endl;
    }
    else if (val == "--qt-include")
    {
	    cout << QTINC << endl;
    }
    else
    {
        cout<<"Unknown Augument: "<<val<<endl;
    }
}

void print_usage()
{
    cout<<"usage: diverse-config [args]"  " [--about|--compiler|--cxxflags\n"
	    << "\t--cxxflags|--defines|--sound-include|--coin-include|--vtk-include\n"
	    << "\t--dosg-include|--qt-include |--sound-libs|--coin-libs|--vtk-libs\n"
	    <<"\t--dosg-libs|--dgl-libs|--qt-libs|--include|--install-dir\n"
	    <<"\t--libs|--version|--help\n\n";

    cout<<"\t--about\t\t\tprint information about DIVERSE.\n";
    cout<<"\t--compiler\t\tprint the compiler to use.\n";    
    cout<<"\t--cxxflags\t\tprint the flags to pass to the compiler.\n";    
    cout<<"\t--defines\t\tprint defines that need to be passed to\n"
        <<"\t\t\t\tthe compiler.\n";
    cout<<"\t--sound-include\t\tSound include directories\n";
    cout<<"\t--coin-include\t\tCoin include directories\n";
    cout<<"\t--vtk-include\t\tVTK include directories\n";
    cout<<"\t--dosg-include\t\tDOSG include directories\n";
    cout<<"\t--qt-include\t\tQT include directories\n";
    cout<<"\t--sound-libs\t\tSound library link information\n";
    cout<<"\t--coin-libs\t\tCoin lib  directories\n";
    cout<<"\t--vtk-libs\t\tVTK library link information\n";
    cout<<"\t--dosg-libs\t\tDOSG library link information\n";
    cout<<"\t--dgl-libs\t\tDGL library link information\n";
    cout<<"\t--qt-libs\t\tQT library link information\n";
    cout<<"\t--include\t\tprint the directorys to look for include files.\n";
    cout<<"\t--install-dir\t\tprint the directory dgl is installed in.\n";    
    cout<<"\t--libs\t\t\tprint the libraries to link against.\n";
    cout<<"\t--version\t\tprint the diverse version number.\n";
    cout<<"\t--help\t\t\tprint usage options.\n";
    cout<<endl; 
}
