#ifndef FLUID
#include "Fluid.h"
#endif

#ifndef NODE
#include "Node.h"
#endif

#ifndef SPRING
#include "Spring.h"
#endif

#ifdef WIN32
#include <windows.h>
#endif
#include "math.h"
#include <GL/gl.h>

FluidObject::FluidObject()
{
	nodeCount = 0;

	for (int i=0; i<10; i+=3)
		for (int j=0; j<10; j+=3)
			for (int k=0; k<10; k+=3)
			{
				mesh[nodeCount] = new Node(0,0,0,0);
				mesh[nodeCount]->setCenter(i, j, k);

				nodeCount++;
			}


	// Create the springs

	// Check each node in the 6 directions around it. If there is a node at one of those
	// points, create a spring connecting it to the node pointer that has been located

	// For each node in the list
	// Check for corresponding nodes (based on spacing) - brute force

	// Note: This is an inefficient brute force algo that gets the job done.

	int spacing = 3;
	int outside = 0;

	Node* base;

	float x, y, z;

	for (int node=0; node < nodeCount; node++)
	{
		base = mesh[node];
		base->getCenter(x, y, z);

		for (int i=0; i<nodeCount; i++)
		{
			if (mesh[i]->containsPoint(x, y, z+spacing))
			{
				base->leftSpring = new Spring(base, mesh[i]);
				base->leftNode = mesh[i];
			}

			if (mesh[i]->containsPoint(x, y, z-spacing))
			{
				base->rightSpring = new Spring(base, mesh[i]);
				base->rightNode = mesh[i];
			}

			if (mesh[i]->containsPoint(x, y+spacing,z))
			{
				base->upSpring = new Spring(base, mesh[i]);
				base->upNode = mesh[i];
			}

			if (mesh[i]->containsPoint(x, y-spacing,z))
			{
				base->downSpring = new Spring(base, mesh[i]);
				base->downNode = mesh[i];
			}

			if (mesh[i]->containsPoint(x+spacing,y,z))
			{
				base->inSpring = new Spring(base, mesh[i]);
				base->inNode = mesh[i];
			}

			if (mesh[i]->containsPoint(x-spacing,y,z))
			{
				base->outSpring = new Spring(base, mesh[i]);
				base->outNode = mesh[i];
			}
		}

		if ( !(base->inSpring) || !(base->outSpring) || !(base->leftSpring) || !(base->rightSpring) || !(base->upSpring) || !(base->downSpring)  )
		{
			outside = 1;

			base->setColor(1,1,1);
		}
		else
		{
			outside = 0;
		}

		base->setOutside(outside);	
	}	

	asSurface = 0;
	asPoints = 1;
}

FluidObject::~FluidObject()
{
	for (int i=0; i<nodeCount; i++)
	{
		delete mesh[i];
	}
}

void FluidObject::initialize()
{
	// Determine the initial configutation of the mesh

}




void FluidObject::applyForce(int magnitude2)
{
	// Move to a new position
	Node* node = mesh[6];

	node->setCenter(-5,-5,-5);
	node->setActingForce(0.5);
	node->setFirm(0);

	for (int i=0; i<nodeCount; i++)
	{
		mesh[i]->applyForce();
	}
}

void FluidObject::generateSurface()
{
	for (int i=0; i<nodeCount; i++)
	{
		// Check the current node. 
	}

}

void FluidObject::draw()
{
	// Draw as a surface
	if (asSurface)
		for (int i=0; i<nodeCount; i++)
		{
			glDisable(GL_CULL_FACE);
			mesh[i]->setDrawSprings(0);
			mesh[i]->draw();

			glPolygonMode(GL_FRONT, GL_FILL);
			glPolygonMode(GL_BACK, GL_FILL);
			mesh[i]->makeSurface();
		}


	// Draw as points
	if (asPoints)
		for (int i=0; i<nodeCount; i++)
		{
			mesh[i]->setDrawSprings(1);
			mesh[i]->draw();
		}	

}

void FluidObject::enableDrawSprings()
{
	for (int i=0; i<nodeCount; i++)
	{
		mesh[i]->setDrawSprings(1);
	}
}

void FluidObject::disableDrawSprings()
{
	for (int i=0; i<nodeCount; i++)
	{
		mesh[i]->setDrawSprings(0);
	}
}

void FluidObject::applyTexture()
{
}

void FluidObject::loadMesh()
{
}

void FluidObject::saveMesh()
{
}

void FluidObject::setDrawingMode(int val)
{
	if (val)
	{
		asSurface = 1;
		asPoints = 0;
	}
	else
	{
		asSurface = 0;
		asPoints = 1;
	}
}
