#ifndef NODE
#include "Node.h"
#endif

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>

#include <math.h>
#include <stdio.h>

// Constructors
Node::Node()
{
}

Node::Node(float x, float y, float z, float size)
{
	centerX = x;
	centerY = y;
	centerZ = z;

	nodeSize = 4.0;

	outsideNode = 0;

	actingForce = 0.0;
	attenuation = 1.8;
	firm = 0;

	rColor = 1.0;
	gColor = 1.0;
	bColor = 0.0;

	drawSprings = 1;
	drawPoints = 1;

	leftNode = NULL;
	rightNode = NULL;
	upNode = NULL;
	downNode = NULL;
	inNode = NULL;
	outNode = NULL;
	
	leftSpring = NULL;
	rightSpring = NULL;
	upSpring = NULL;
	downSpring = NULL;
	inSpring = NULL;
	outSpring = NULL;
}


Node::~Node()
{
	if (leftSpring)
		delete leftSpring;

	if (rightSpring)
		delete rightSpring;

	if (upSpring)
		delete upSpring;

	if (downSpring)
		delete downSpring;
	
	if (inSpring)
		delete inSpring;
	
	if (outSpring)
		delete outSpring;
}


void Node::calculateNormal(float a, float b, float c, float x, float y, float z)
{
	// Calculate the cross product of the view and up vectors
	float vx = (b * z) - (y * c);
	float vy = (c * x) - (z * a);
	float vz = (a * y) - (x * b);

	// Set to be a unit normal
	double magnitude = sqrt(vx * vx + vy * vy + vz * vz);
	vx /= magnitude;
	vy /= magnitude;
	vz /= magnitude;

	glNormal3f(vx, vy, vz);
}

void Node::drawSurface(Node* node1, Node* node2)
{
	float x1, y1, z1;
	float x2, y2, z2;
	float x3, y3, z3;
	
	glColor3f(0.0, 0.0, 0.5);
	glBegin(GL_TRIANGLES);

	this->getCenter(x1,y1,z1);
	node1->getCenter(x2,y2,z2);
	node2->getCenter(x3,y3,z3);

	calculateNormal((x1-x2),(y1-y2),(z1-z2),(x1-x3),(y1-y3),(z1-z3));
	glVertex3f(x1,y1,z1);
	glVertex3f(x2,y2,z2);
	glVertex3f(x3,y3,z3);

	glEnd();
}

void Node::drawSurface(Node* node1, Node* node2, Node* node3)
{
	float x1, y1, z1;
	float x2, y2, z2;
	float x3, y3, z3;
	
	glColor3f(0.0, 0.0, 0.5);
	glBegin(GL_TRIANGLES);

	node1->getCenter(x1,y1,z1);
	node2->getCenter(x2,y2,z2);
	node3->getCenter(x3,y3,z3);

	calculateNormal((x1-x2),(y1-y2),(z1-z2),(x1-x3),(y1-y3),(z1-z3));
	glVertex3f(x1,y1,z1);
	glVertex3f(x2,y2,z2);
	glVertex3f(x3,y3,z3);

	glEnd();
}

void Node::drawSurface(Node* node1, Node* node2, Node* node3, Node* node4)
{
	float x1, y1, z1;
	float x2, y2, z2;
	float x3, y3, z3;
	float x4, y4, z4;

	glColor3f(0.0, 0.0, 0.5);

	glBegin(GL_QUADS);

	node1->getCenter(x1,y1,z1);
	node2->getCenter(x2,y2,z2);
	node3->getCenter(x3,y3,z3);
	node4->getCenter(x4,y4,z4);

	calculateNormal((x1-x2),(y1-y2),(z1-z2),(x1-x3),(y1-y3),(z1-z3));
	glVertex3f(x1,y1,z1);
	glVertex3f(x2,y2,z2);
	glVertex3f(x3,y3,z3);
	glVertex3f(x4,y4,z4);

	glEnd();
}


void Node::makeSurface()
{
	if (!(this->isOutside()))
		return;

	
	glPolygonMode(GL_FRONT, GL_FILL);
	glPolygonMode(GL_BACK, GL_FILL);

		// 2 possible square cases
	if ((upNode) && (downNode) && (leftNode) && (rightNode))
	{
		if ((upNode->isOutside()) && (downNode->isOutside()) && (leftNode->isOutside()) && (rightNode->isOutside()))
		{
			drawSurface(upNode, rightNode, downNode, leftNode);
		}
	}

	if ((inNode) && (outNode) && (leftNode) && (rightNode))
	{
		if ((inNode->isOutside()) && (outNode->isOutside()) && (leftNode->isOutside()) && (rightNode->isOutside()))
		{
			drawSurface(inNode, leftNode, outNode, rightNode);
		}
	}
	
	// 8 Basic Cases
	if ((upNode) && (leftNode))
	{
		if ( (upNode->isOutside()) && (leftNode->isOutside()) ) 
		{
			drawSurface(upNode, leftNode);
		}
	}

	if ((upNode) && (rightNode))
	{
		if ( (upNode->isOutside()) && (rightNode->isOutside()) )
		{
			drawSurface(upNode, rightNode);
		}
	}

	if ((upNode) && (inNode))
	{
		if ( (upNode->isOutside()) && (inNode->isOutside()) )
		{
			drawSurface(upNode, inNode);
		}
	}

	if ((upNode) && (outNode))
	{
		if ( (upNode->isOutside()) && (outNode->isOutside()) )
		{
			drawSurface(upNode, outNode);
		}
	}

	if ((downNode) && (leftNode))
	{
		if ( (downNode->isOutside()) && (leftNode->isOutside()) )
		{
			drawSurface(downNode, leftNode);
		}
	}

	if ((downNode) && (rightNode))
	{
		if ( (downNode->isOutside()) && (rightNode->isOutside()) )
		{
			drawSurface(downNode, rightNode);
		}
	}

	if ((downNode) && (inNode))
	{
		if ( (downNode->isOutside()) && (inNode->isOutside()) )
		{
			drawSurface(downNode, inNode);
		}
	}

	if ((downNode) && (outNode))
	{
		if ( (downNode->isOutside()) && (outNode->isOutside()) )
		{
			drawSurface(downNode, outNode);
		}
	}

	// Diamond format cases

	// Up, Right, Out
	if ((upNode) && (rightNode) && (outNode))
	{
		if ( (upNode->isOutside()) && (rightNode->isOutside()) && (outNode->isOutside()) )
		{
			drawSurface(upNode, rightNode, outNode);
		}
	}

	// Up, Right, In
	if ((upNode) && (rightNode) && (inNode))
	{
		if ( (upNode->isOutside()) && (rightNode->isOutside()) && (inNode->isOutside()) )
		{
			drawSurface(upNode, rightNode, inNode);
		}
	}

	// Up, Left, In
	if ((upNode) && (leftNode) && (outNode))
	{
		if ( (upNode->isOutside()) && (leftNode->isOutside()) && (outNode->isOutside()) )
		{
			drawSurface(upNode, leftNode, outNode);
		}
	}

	// Up, Left, Out
	if ((upNode) && (leftNode) && (outNode))
	{
		if ( (upNode->isOutside()) && (leftNode->isOutside()) && (outNode->isOutside()) )
		{
			drawSurface(upNode, leftNode, outNode);
		}
	}

	// Down, Left, out
	if ((downNode) && (leftNode) && (outNode))
	{
		if ( (downNode->isOutside()) && (leftNode->isOutside()) && (outNode->isOutside()) )
		{
			drawSurface(downNode, leftNode, outNode);
		}
	}

	// Down, Left, in
	if ((downNode) && (leftNode) && (inNode))
	{
		if ( (downNode->isOutside()) && (leftNode->isOutside()) && (inNode->isOutside()) )
		{
			drawSurface(downNode, leftNode, inNode);
		}
	}

	// Down, Right, out
	if ((downNode) && (rightNode) && (outNode))
	{
		if ( (downNode->isOutside()) && (rightNode->isOutside()) && (outNode->isOutside()) )
		{
			drawSurface(downNode, rightNode, outNode);
		}
	}

	// Down, right, in
	if ((downNode) && (rightNode) && (inNode))
	{
		if ( (downNode->isOutside()) && (rightNode->isOutside()) && (inNode->isOutside()) )
		{
			drawSurface(downNode, rightNode, inNode);
		}
	}
}

void Node::setColor(float r, float g, float b)
{
	rColor = r;
	gColor = g;
	bColor = b;
}

int Node::containsPoint(float x, float y, float z)
{
	if ((x == centerX) && (y == centerY) && (z == centerZ))
		return 1;
	else 
		return 0;
}

void Node::setOutside(int flag)
{
	outsideNode = flag;
}

int Node::isOutside()
{
	return outsideNode;
}

void Node::calculateNewPosition(Node* base, Node* move)
{
	float x, y, z;
	float x1, y1, z1;
	float xdir, ydir, zdir;
	float magnitude;

	// Calculate vector between the new position and this one
	base->getCenter(x, y, z);
	move->getCenter(x1, y1, z1);

	xdir = x - x1;
	ydir = y - y1;
	zdir = z - z1;

	magnitude = sqrt(xdir * xdir + ydir + ydir + zdir * zdir);
	magnitude = actingForce;

	x = x1 + (xdir * magnitude);
	y = y1 + (ydir * magnitude);
	z = z1 + (zdir * magnitude);

	if (!firm)
		move->setCenter(x,y,z);
}
// Actions
void Node::draw()
{
	float x, y, z;
	float x2, y2, z2;

	if (drawSprings)
	{
		if (leftSpring)
			leftSpring->draw();

		if (rightSpring)
			rightSpring->draw();
		
		if (upSpring)
			upSpring->draw();

		if (downSpring)
			downSpring->draw();

		if (inSpring)
			inSpring->draw();

		if (outSpring)
			outSpring->draw();
	}

	if (drawPoints)
	{
		glColor3f(rColor, gColor, bColor);
		glPointSize(nodeSize);

		glBegin(GL_POINTS);
			glVertex3f(centerX, centerY, centerZ);
		glEnd();
	}

}

void Node::setDrawSprings(int flag)
{
	drawSprings = flag;
}

void Node::setDrawPoints(int flag)
{
	drawPoints = flag;
}

void Node::setCenter(float x, float y, float z)
{
	centerX = x;
	centerY = y;
	centerZ = z;
}

void Node::setFirm(int val)
{
	firm = val;
}

void Node::getCenter(float &x, float &y, float &z)
{
	x = centerX;
	y = centerY;
	z = centerZ;
}

// Force management - calculate effect, attenuate, and pass to other nodes
void Node::applyForce()
{
	if (actingForce == 0.0)
		return;

	// For each of the none null connecting nodes, run the calculate new
	// position method, then attenuate the acting force and pass on to
	// each of those nodes.

	// Calculate vectors to non-null attached nodes
	if (this->leftNode != NULL)
	{
		calculateNewPosition(this, this->leftNode);		
		leftNode->setActingForce(actingForce/attenuation);
	}
	
	if (this->rightNode != NULL)
	{
		calculateNewPosition(this, this->rightNode);	
		rightNode->setActingForce(actingForce/attenuation);
	}
	
	if (this->inNode != NULL)
	{
		calculateNewPosition(this, this->inNode);	
		inNode->setActingForce(actingForce/attenuation);
	}
	
	if (this->outNode != NULL)
	{
		calculateNewPosition(this, this->outNode);	
		outNode->setActingForce(actingForce/attenuation);
	}
	
	if (this->upNode != NULL)
	{
		calculateNewPosition(this, this->upNode);	
		upNode->setActingForce(actingForce/attenuation);
	}

	if (this->downNode != NULL)
	{
		calculateNewPosition(this, this->downNode);	
		downNode->setActingForce(actingForce/attenuation);
	}

	actingForce = 0.0;
}

void Node::setAttenuation(float val)
{
	attenuation = val;
}

void Node::setActingForce(float val)
{
	actingForce = val;
}

float Node::getActingForce()
{
	return actingForce;
}
