// OpenGL Fluid Node Class
// 
// Author: Stephen Manley - smanley@nyx.net
// Date: Oct 12th 1999
// 
// Description:
// 
// This class implements a particle used in a fluid model in opengl.
//
// Each particle is allowed to have six connected nodes.  
//
// When a spring compresses, the other springs must move and sum their forces
// appropriately. This compression will also have the effect of physically
// moving the location of the current node.
// 
//////////////////////////////////////////////////////////////////////////////////////

class Spring;
class Node;


#ifndef NODE
#define NODE

#ifndef SPRING
#include "Spring.h"
#endif

class Node
{
	public:

		// Constructors
		Node();
		Node(float x, float y, float z, float size);
		~Node();

		// Actions
		void draw();

		// Force management - calculate effect, attenuate, and pass to other nodes
		void applyForce();

		void calculateNormal(float a, float b, float c, float x, float y, float z);

		void makeSurface();
		void drawSurface(Node* node1, Node* node2, Node* node3, Node* node4);
		void drawSurface(Node* node1, Node* node2, Node* node3);
		void drawSurface(Node* node1, Node* node2);

		int isOutside();
		int containsPoint(float x, float y, float z);

		void setCenter(float x, float y, float z);
		void setColor(float r, float g, float b);
		void setOutside(int flag);
		void setDrawSprings(int flag);
		void setDrawPoints(int flag);
		void setActingForce(float val);
		void setAttenuation(float val);
		void setFirm(int val);

		void getCenter(float &x, float &y, float &z);
		float getActingForce();


		// Connecting nodes
		Node*	leftNode;
		Node*	rightNode;
		Node*	upNode;
		Node* 	downNode;
		Node*   inNode;
		Node* 	outNode;

		// Connecting springs
		Spring*	leftSpring;
		Spring* rightSpring;
		Spring* upSpring;
		Spring* downSpring;
		Spring* inSpring;
		Spring* outSpring;

	private:

		void calculateNewPosition(Node* base, Node* move);

		// Draw springs
		int drawSprings;
		int drawPoints;

		// Outside node
		int	outsideNode;

		// Unmoveble node
		int firm;

		// Node center
		float centerX, centerY, centerZ;

		float rColor, gColor, bColor;

		// Node up vector
		float upX, upY, upZ;

		// Node size
		float nodeSize;

		// Current force acting on the node
		float actingForce;

		// Node "inertia"
		float attenuation;
};
#endif
