// OpenGL Fluid Spring Class
// 
// Author: Stephen Manley - smanley@nyx.net
// Date: Oct 12th 1999
// 
// Description:
// 
// This class implements a spring node for fluid (soft object) simulation.
// 
// Springs are two dimensional (although it is quite possible and probably
// quite nifty to create spring systems) so they have both a beginning and
// ending node.
// 
// Forces: An applied force is an input to the system, and is positive. Since
// every action has an equal and opposite reaction, then the reactionary forces
// elsewhere in the spring matrix are negative.
//
// 
//////////////////////////////////////////////////////////////////////////////////////


class Spring;
class Node;

#ifndef SPRING
#define SPRING

#ifndef NODE
#include "Node.h"
#endif


class Spring
{
	public:
		// Constructors
		Spring();
		~Spring();

		Spring(Node* base1, Node* base2, double min=0, double max=0, double k=0, double attenuation=0);

		// Actions
		void draw();
		void applyForce(double force);

		// Modifiers
		double getLength();
		double getReaction();

		void setAttenuation(float value);

	private:

		void calculateLength();

		Node *baseNode1;
		Node* baseNode2;

		float rColor, gColor, bColor;

		double maxLength;
		double minLength;
		double springConstant;

		double currentLength;
		double appliedForce;		
		double reactionForce;

		float attenuation;
};
#endif
