// Module Name: fluid.cpp
// Author:      Stephen Manley (smanley@nyx.net)
//
// Description:
// 
// This is the main program for the fluid (soft) surface simulation. The 
// objective is to allow a flexible, scalable and simple model to render
// and manipulate soft objects with something approaching realtime 
// preformance. 
//
// The deformed object supports specular lighting and texture mapping in
// this demonstration.
//  
// Usage:
//
//
// Todos:
//
// Force modelling - move a node and propagate the change
// Animation of motion - move discreet amounts and then change
// Turning external nodes into surface
// Texture mapping the surface
// Isosurface generation from the specified object
// 
////////////////////////////////////////////////////////////////////////////

// Eeeewwwww

// System includes
#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
//#include <GL/glut.h>
//#include <glut.h>
#include <iostream>
#include <fstream>
#include <dgl/libdgl/cdgl.h>
using namespace std;

// Object Includes

#ifndef FLUID
#include "Fluid.h"
#endif


// Height and width of the active window
int windowHeight, windowWidth;

// Mouse status variables
int mouseClickX, mouseClickY;
int leftMouseDown = 0;

// Object pointers
//Camera* camera;
FluidObject* mesh;

// Configure projection information
void init(void) 
{
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glShadeModel (GL_SMOOTH);

   //camera->on();

   // Enable lighting
   GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat mat_shininess[] = { 50.0 };
   GLfloat light_position[] = { 5.0, 5.0, 5.0, 0.0 };

   GLfloat spot_direction[] = { -1.0, -1.0, 0 };
 
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glShadeModel (GL_SMOOTH);

   glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
 
   glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 45.0);
   glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spot_direction);
   glLightfv(GL_LIGHT0, GL_POSITION, light_position);

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);

   // Make sure we clip hidden surfaces!
   glEnable(GL_DEPTH_TEST);
glLoadIdentity();
}

// Redraw the whole scene
void display(void)
{
	int xSize = 40;
	int ySize = 40;
	float rGridColor = .0;
	float gGridColor = .4;
	float bGridColor = .0;
	int gridSpacing = 2;

	// Set the active projection

	glMatrixMode(GL_MODELVIEW);
	//gluLookAt(0, 2, -15, 0,0,0, 0,1,0);
	// Rotate so that we're looking into the scene
	glRotated(180,1,0,0);

	// Draw whatever mesh we're working with
	

	// Draw the grid out over the tiles to show subdivisions
	glColor3f(rGridColor, gGridColor, bGridColor);

	double z = 15;

	glBegin(GL_LINES);

    for (int x=0; x<=xSize; x+=gridSpacing)
	{
        glVertex3d(x-15, 0-15, z);
		glVertex3d(x-15, xSize-15, z);
    }

    for (int y=0; y<=ySize; y+=gridSpacing)
	{
	    glVertex3d(0-15, y-15, z);
		glVertex3d(ySize-15, y-15, z);
	}
    
	glEnd();


	
	// Draw the object
	mesh->draw();

	// Swap and flush the buffers
	//glutSwapBuffers();

}

// Reshape the window - save the parameters
void reshape (int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);

   windowWidth = w;
   windowHeight = h; 
}


// Keyboard handler
bool keyboard(DKeyEvent* DEvent)
{
	double xR, yR, zR;
	double xS, yS, zS;

   switch (DEvent->ascii()) 
   {
      case 27:
         exit(0);
         break;

      case 'l':
         display(); // redraw
         break;   

      case '2':
//		 camera->moveBackward(0.2);
         display();
         break;

      case '6':
 //		 camera->moveRight(0.2);
         display();
         break;

      case '4':
//		 camera->moveLeft(0.2);
         display();
         break;

      case '+':
 //		 camera->moveUp(0.2);
         display();
         break;

      case '-':
//		 camera->moveDown(0.2);
         display();
         break;     

      case '*':
 //		 camera->rotateEye(5.,0.0);
         display();
         break;

      case '/':
//		 camera->rotateEye(-5.,0.0);
         display();
         break; 

      case 'z':
//		  camera->rotatePoint(0,0,0,+5,0);
		  display();
		  break;      
	  
	  case 'x':
		  //camera->rotatePoint(0,0,0,-5,0);
		  display();
		  break;

	  case 'a':
		  mesh->applyForce(1);
		  display();
		  break;

	  case 'r':
		  delete mesh;
		  mesh = new FluidObject();
		  display();
		  break;
	

	  case '8':
		  mesh->setDrawingMode(1);
		  break;

  	  case '9':
		  mesh->setDrawingMode(0);
		  break; 
   }
   return false;
}

// Mouse motion callback
void mouseMove(int x, int y)
{
	// Not used
}

// Snags mouse clicks for the mouse move callback
void mouse(int button, int state, int mouseX, int mouseY)
{
	/*
	// Snag the origin point for the control drag
    if (button == GLUT_LEFT_BUTTON)
	{
   		if (state == GLUT_DOWN)
   		{
			if (!leftMouseDown)
			{
				mouseClickX = mouseX;
	   			mouseClickY = mouseY;
			}
   		
   		    leftMouseDown = 1;
   		}
		
		if (state == GLUT_UP)
		{
			leftMouseDown = 0;
		}
   }
   */
}

void special(int key, int x, int y)
{
	/*
	if (key == GLUT_KEY_UP)
	{
		camera->rotatePoint(0,0,0,0,5);
		display();
	}
	
	if (key == GLUT_KEY_DOWN)
	{
		camera->rotatePoint(0,0,0,0,-5);
		display();
	}
	
	if (key == GLUT_KEY_LEFT)
	{
		camera->rotatePoint(0,0,0,5,0);
		display();
	}

	if (key == GLUT_KEY_RIGHT)
	{
		camera->rotatePoint(0,0,0,-5,0);
		display();
	}
	*/
}

int main(int argc, char** argv)
{
	// Create scene objects
	//camera = new Camera();
	mesh = new FluidObject();


	// Glut init
	/*
	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize (500, 500); 
	glutInitWindowPosition (100, 100);
	glutCreateWindow ("Fluid Demo");
	*/

	// Turn on camera


	//camera->moveLeft(10);
	//camera->moveForward(10);
	//camera->moveUp(6);

	dglInit(argc, argv);
	dglKeyCallback(keyboard);
	dglDisplayCallback(display);
	dglRun();
	// Callbacks
	/*
	glutSpecialFunc(special);
	glutDisplayFunc(display); 
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
	glutMotionFunc(mouseMove);
	glutMainLoop();
	*/
   
	return 0;
}




