/***************************************************************************
                          extract.cpp  -  description
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Michael Shore
    email                : mshore@sv.vt.edu
 ***************************************************************************/

/** port of Examples/VisualizationAlgorithms/Tcl/ExtractGeometry.tcl **/

#include <dgl/dvtk/DVtkRenderer.h>
#include <dgl/dvtk/DVtkRenderWindow.h>

#include <stdlib.h>
#include <signal.h>

#include <vtk/vtkQuadric.h>
#include <vtk/vtkSampleFunction.h>
#include <vtk/vtkTransform.h>
#include <vtk/vtkSphere.h>
#include <vtk/vtkImplicitBoolean.h>
#include <vtk/vtkExtractGeometry.h>
#include <vtk/vtkShrinkFilter.h>
#include <vtk/vtkDataSetMapper.h>
#include <vtk/vtkActor.h>
#include <vtk/vtkOutlineFilter.h>
#include <vtk/vtkPolyDataMapper.h>
#include <vtk/vtkProperty.h>

#include <vtk/vtkImageData.h>

int main (int argc, char** argv)
{
	
/** Begin VTK Actor Code **/

   // Here we create two ellipsoidal implicit functions and boolean them
   // together tto form a "cross" shaped implicit function. 
   vtkQuadric *quadric = vtkQuadric::New();
      quadric->SetCoefficients(.5, 1, .2, 0, .1, 0, 0, .2, 0, 0);
   vtkSampleFunction *sample = vtkSampleFunction::New();
      sample->SetSampleDimensions(50, 50, 50);
      sample->SetImplicitFunction(quadric);
      sample->ComputeNormalsOff();
   vtkTransform *trans = vtkTransform::New();
      trans->Scale(1, .5, .333);
   vtkSphere *sphere = vtkSphere::New();
      sphere->SetRadius(0.25);
      sphere->SetTransform(trans);
   vtkTransform *trans2 = vtkTransform::New();
      trans2->Scale(.25, .5, 1.0);
   vtkSphere *sphere2 = vtkSphere::New();
      sphere2->SetRadius(0.25);
      sphere2->SetTransform(trans2);
   vtkImplicitBoolean *onion = vtkImplicitBoolean::New();
      onion->AddFunction(sphere);
      onion->AddFunction(sphere2);
      onion->SetOperationType(0); //onion


   // Here is where it gets interesting. The implicit function is used to
   // extract those cells completely inside the function. They are then 
   // shrunk to help show what was extracted.
   vtkExtractGeometry *extract = vtkExtractGeometry::New();
      extract->SetInput(sample->GetOutput());
      extract->SetImplicitFunction(onion);
   vtkShrinkFilter *shrink = vtkShrinkFilter::New();
      shrink->SetInput((vtkDataSet*) extract->GetOutput());
      shrink->SetShrinkFactor(0.5);
   vtkDataSetMapper *dataMapper = vtkDataSetMapper::New();
      dataMapper->SetInput((vtkDataSet*) shrink->GetOutput());
   vtkActor *dataActor = vtkActor::New();
      dataActor->SetMapper(dataMapper);
		//dataActor->SetPosition(0,0,-5);

   // The outline gives context to the original data.
   vtkOutlineFilter *outline = vtkOutlineFilter::New();
      outline->SetInput(sample->GetOutput());
   vtkPolyDataMapper *outlineMapper = vtkPolyDataMapper::New();
      outlineMapper->SetInput(outline->GetOutput());
   vtkActor *outlineActor = vtkActor::New();
      outlineActor->SetMapper(outlineMapper);
      outlineActor->GetProperty()->SetColor(0,0,0);
		//outlineActor->SetPosition(0,0,-5);

/** End VTK Actor Code **/
   
	// Create Renderer and Render Window
   DVtkRenderer *ren;
   DVtkRenderWindow *renwin;

   ren = DVtkRenderer::New(argc,argv);
   renwin = DVtkRenderWindow::New();

	// Add Renderer to RenderWindow (imitating VTK code)
   renwin -> AddRenderer(ren);

	// Add Actors to Renderer
   ren -> AddActor(outlineActor);
   ren -> AddActor(dataActor);

	// Set Background Color
   ren -> SetBackground(1,1,1);

	ren -> RotateSceneX(90);

	// Run application
	ren -> run();
	
	// Cleanup
   ren->Delete();

   return 0;
}
