/***************************************************************************
                          finance.cpp  -  description
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Michael Shore
    email                : mshore@sv.vt.edu
 ***************************************************************************/

/** port from Examples/VisualizationAlgorithms/Tcl/ColorIsosurface.tcl **/

#include <dgl/dvtk/DVtkRenderer.h>
#include <dgl/dvtk/DVtkRenderWindow.h>

#include <stdlib.h>
#include <signal.h>

#include <vtk/vtkPLOT3DReader.h>
#include <vtk/vtkContourFilter.h>
#include <vtk/vtkPolyDataNormals.h>
#include <vtk/vtkPolyDataMapper.h>
#include <vtk/vtkLODActor.h>
#include <vtk/vtkStructuredGridOutlineFilter.h>
#include <vtk/vtkActor.h>

#include <vtk/vtkStructuredGrid.h>


int main (int argc, char** argv)
{

	const char *temp=getenv("VTK_DATA_ROOT");
   char filename[2048], filename2[2048];
  
   if ( ! temp )
   {
		cerr << "ERROR: Please define environment variable VTK_DATA_ROOT\n";
		return NULL;
   }
	
	strcpy(filename,temp);
	strcat(filename,"/Data/combxyz.bin");
	strcpy(filename2,temp);
	strcat(filename2,"/Data/combq.bin");

   /** Start VTK Code **/

   // Read some data. The important thing here is to read a function as a data
   // array as well as the scalar and vector.  (here function 153 is named
   // "Velocity Magnitude").Later this data array will be used to color the
   // isosurface.  
   //
   vtkPLOT3DReader *pl3d = vtkPLOT3DReader::New();
      pl3d->SetXYZFileName(filename);
      pl3d->SetQFileName(filename2);
      pl3d->SetScalarFunctionNumber(100);
      pl3d->SetVectorFunctionNumber(202);
      pl3d->AddFunction(153);
      pl3d->Update();
      pl3d->DebugOn();
    
   // The contoru filter uses the labeled scalar (function number 100
   // above to generate the contour surface; all other data is interpolated
   // during the contouring process.
   //
   vtkContourFilter *iso = vtkContourFilter::New();
      iso->SetInput(pl3d->GetOutput());
      iso->SetValue(0, .24);

   vtkPolyDataNormals *normals = vtkPolyDataNormals::New();
      normals->SetInput(iso->GetOutput());
      normals->SetFeatureAngle(45);

   // We indicate to the mapper to use the velcoity magnitude, which is a 
   // vtkDataArray that makes up part of the point attribute data.
   //
   vtkPolyDataMapper *isoMapper = vtkPolyDataMapper::New();
      isoMapper->SetInput(normals->GetOutput());
      isoMapper->ScalarVisibilityOn();
      isoMapper->SetScalarRange(0, 1500);
      isoMapper->SetScalarModeToUsePointFieldData();
      isoMapper->ColorByArrayComponent("VelocityMagnitude", 0);

   vtkLODActor *isoActor = vtkLODActor::New();
      isoActor->SetMapper(isoMapper);
      isoActor->SetNumberOfCloudPoints(1000);
      isoActor->SetPosition(-5,10,-30);

   vtkStructuredGridOutlineFilter *outline = vtkStructuredGridOutlineFilter::New();
      outline->SetInput(pl3d->GetOutput());
   vtkPolyDataMapper *outlineMapper = vtkPolyDataMapper::New();
      outlineMapper->SetInput(outline->GetOutput());
   vtkActor *outlineActor = vtkActor::New();
      outlineActor->SetMapper(outlineMapper);
      outlineActor->SetPosition(-5,10,-30);

   /** End VTK Code **/
   
	//Create renderer and render window
   DVtkRenderer *ren;
   DVtkRenderWindow *renwin;

   ren = DVtkRenderer::New(argc,argv);
   renwin = DVtkRenderWindow::New();

	//Add Renderer to RenderWindow (imitating VTK code)
   renwin -> AddRenderer(ren);

	//Add actors to render
   ren -> AddActor(outlineActor);
   ren -> AddActor(isoActor);

	//Set background color
   ren -> SetBackground(0.1,0.2,0.4);

	//Run application
	ren -> run();

	//Cleanup
   ren->Delete();

   return 0;
}
