/** port from Examples/DataManipulation/Tcl/marching.tcl **/

#include <dgl/dvtk/DVtkRenderer.h>
#include <dgl/dvtk/DVtkRenderWindow.h>

#include <vtk/vtkProperty.h>
#include <vtk/vtkActor.h>
#include <vtk/vtkPolyDataMapper.h>
#include <vtk/vtkFloatArray.h>
#include <vtk/vtkPoints.h>
#include <vtk/vtkIdList.h>
#include <vtk/vtkUnstructuredGrid.h>
#include <vtk/vtkContourFilter.h>
#include <vtk/vtkExtractEdges.h>
#include <vtk/vtkTubeFilter.h>
#include <vtk/vtkShrinkPolyData.h>
#include <vtk/vtkCubeSource.h>
#include <vtk/vtkSphereSource.h>
#include <vtk/vtkThresholdPoints.h>
#include <vtk/vtkGlyph3D.h>
#include <vtk/vtkVectorText.h>
#include <vtk/vtkTransform.h>
#include <vtk/vtkTransformPolyDataFilter.h>
#include <vtk/vtkPointData.h>
#include <vtk/vtkPolyData.h>

void case12(vtkFloatArray *Scalars, vtkVectorText *caseLabel, int In, int Out);

int main(int argc, char** argv)
{
	vtkFloatArray *Scalars = vtkFloatArray::New();
	   Scalars->InsertNextValue(1.0);
		Scalars->InsertNextValue(0.0);
		Scalars->InsertNextValue(0.0);
		Scalars->InsertNextValue(1.0);
		Scalars->InsertNextValue(0.0);
		Scalars->InsertNextValue(0.0);
		Scalars->InsertNextValue(0.0);
		Scalars->InsertNextValue(0.0);

	vtkPoints *Points = vtkPoints::New();
		Points->InsertNextPoint(0,0,0);
		Points->InsertNextPoint(1,0,0);
		Points->InsertNextPoint(1,1,0);
		Points->InsertNextPoint(0,1,0);
		Points->InsertNextPoint(0,0,1);
		Points->InsertNextPoint(1,0,1);
		Points->InsertNextPoint(1,1,1);
		Points->InsertNextPoint(0,1,1);

	vtkIdList *Ids = vtkIdList::New();
		Ids->InsertNextId(0);
		Ids->InsertNextId(1);
		Ids->InsertNextId(2);
		Ids->InsertNextId(3);
		Ids->InsertNextId(4);
		Ids->InsertNextId(5);
		Ids->InsertNextId(6);
		Ids->InsertNextId(7);

	vtkUnstructuredGrid *Grid = vtkUnstructuredGrid::New();
		Grid->Allocate(10,10);
		Grid->InsertNextCell(12,Ids);
		Grid->SetPoints(Points);
		Grid->GetPointData()->SetScalars(Scalars);

	// Find the triangles that lie along the 0.5 contour in this cube.
	vtkContourFilter *Marching = vtkContourFilter::New();
		Marching->SetInput(Grid);
		Marching->SetValue(0,0.5);
		Marching->Update();

	// Extract the edges of the triangles just found.
	vtkExtractEdges *triangleEdges = vtkExtractEdges::New();
		triangleEdges->SetInput(Marching->GetOutput());

	// Draw the edges as tubes instead of lines.  Also create the associated
	// mapper and actor to display the tubes.
	vtkTubeFilter *triangleEdgeTubes = vtkTubeFilter::New();
		triangleEdgeTubes->SetInput(triangleEdges->GetOutput());
		triangleEdgeTubes->SetRadius(0.005);
		triangleEdgeTubes->SetNumberOfSides(6);
		triangleEdgeTubes->UseDefaultNormalOn();
		triangleEdgeTubes->SetDefaultNormal(.577,.577,.577);
	vtkPolyDataMapper *triangleEdgeMapper = vtkPolyDataMapper::New();
		triangleEdgeMapper->SetInput(triangleEdgeTubes->GetOutput());
		triangleEdgeMapper->ScalarVisibilityOff();
	vtkActor *triangleEdgeActor = vtkActor::New();
		triangleEdgeActor->SetMapper(triangleEdgeMapper);
		triangleEdgeActor->GetProperty()->SetDiffuseColor(0.1800, 0.2800, 0.2300);
		triangleEdgeActor->GetProperty()->SetSpecular(.4);
		triangleEdgeActor->GetProperty()->SetSpecularPower(10);

	// Shrink the triangles we found earlier.  Create the associated mapper
	// and actor.  Set the opacity of the shrunken triangles.
	
	vtkShrinkPolyData *aShrinker = vtkShrinkPolyData::New();
		aShrinker->SetShrinkFactor(1);
		aShrinker->SetInput(Marching->GetOutput());
	vtkPolyDataMapper *aMapper = vtkPolyDataMapper::New();
		aMapper->ScalarVisibilityOff();
		aMapper->SetInput(aShrinker->GetOutput());
	vtkActor *Triangles = vtkActor::New();
		Triangles->SetMapper(aMapper);
		Triangles->GetProperty()->SetDiffuseColor(0.8900, 0.8100, 0.3400);
		Triangles->GetProperty()->SetOpacity(.6);

	// Draw a cube the same size and at the same position as the one created
	// previously.  Extract the edges because we only want to see the outline
	// of the cube.  Pass the edges through a vtkTubeFilter so they are displayed
	// as tubes rather than lines.
	
	vtkCubeSource *CubeModel = vtkCubeSource::New();
		CubeModel->SetCenter(.5,.5,.5);
	vtkExtractEdges *Edges = vtkExtractEdges::New();
		Edges->SetInput(CubeModel->GetOutput());
	vtkTubeFilter *Tubes = vtkTubeFilter::New();
		Tubes->SetInput(Edges->GetOutput());
		Tubes->SetRadius(.01);
		Tubes->SetNumberOfSides(6);
		Tubes->UseDefaultNormalOn();
		Tubes->SetDefaultNormal(.577,.577,.577);
	// Create the mapper and actor to display the cube edges.
	vtkPolyDataMapper *TubeMapper = vtkPolyDataMapper::New();
		TubeMapper->SetInput(Tubes->GetOutput());
	vtkActor *CubeEdges = vtkActor::New();
		CubeEdges->SetMapper(TubeMapper);
		CubeEdges->GetProperty()->SetDiffuseColor(0.9412, 0.9020, 0.5490);
		CubeEdges->GetProperty()->SetSpecular(.4);
		CubeEdges->GetProperty()->SetSpecularPower(10);

	// Create a sphere to use as a glyph source for vtkGlyph3D.
	vtkSphereSource *Sphere = vtkSphereSource::New();
		Sphere->SetRadius(0.04);
		Sphere->SetPhiResolution(20);
		Sphere->SetThetaResolution(20);
	// Remove the part of the cube with data values below 0.5.
	vtkThresholdPoints *ThresholdIn = vtkThresholdPoints::New();
		ThresholdIn->SetInput(Grid);
		ThresholdIn->ThresholdByUpper(.5);
	// Display spheres at the vertices remaining in the cube data set after
	// it was passed through vtkThresholdPoints.
	vtkGlyph3D *Vertices = vtkGlyph3D::New();
		Vertices->SetInput(ThresholdIn->GetOutput());
		Vertices->SetSource(Sphere->GetOutput());
	// Create a mapper and actor to display the glyphs.
	vtkPolyDataMapper *SphereMapper = vtkPolyDataMapper::New();
		SphereMapper->SetInput(Vertices->GetOutput());
		SphereMapper->ScalarVisibilityOff();
	vtkActor *CubeVertices = vtkActor::New();
		CubeVertices->SetMapper(SphereMapper);
		CubeVertices->GetProperty()->SetDiffuseColor(1.0000, 0.3882, 0.2784);

	// Define the text for the label
	vtkVectorText *caseLabel = vtkVectorText::New();
		caseLabel->SetText("Case 1");
	
	// Set up a transform to move the label to a new position.
	vtkTransform *aLabelTransform = vtkTransform::New();
		aLabelTransform->Identity();
		aLabelTransform->Translate(-.2, 0, 1.25);
		aLabelTransform->Scale(.05, .05, .05);

	// Move the label to a new position.
	vtkTransformPolyDataFilter *labelTransform = vtkTransformPolyDataFilter::New();
		labelTransform->SetTransform(aLabelTransform);
		labelTransform->SetInput(caseLabel->GetOutput());
	
	// Create a mapper and actor to display the text.
	vtkPolyDataMapper *labelMapper = vtkPolyDataMapper::New();
		labelMapper->SetInput(labelTransform->GetOutput());
	
	vtkActor *labelActor = vtkActor::New();
		labelActor->SetMapper(labelMapper);

	// Define the base that the cube sits on.  Create its associated mapper
	// and actor.  Set the position of the actor.
	vtkCubeSource *baseModel = vtkCubeSource::New();
		baseModel->SetXLength(1.5);
		baseModel->SetYLength(.01);
		baseModel->SetZLength(1.5);
	vtkPolyDataMapper *baseMapper = vtkPolyDataMapper::New();
		baseMapper->SetInput(baseModel->GetOutput());
	vtkActor *base = vtkActor::New();
		base->SetMapper(baseMapper);
		base->SetPosition(.5,-.09,.5);

	DVtkRenderer *ren1 = DVtkRenderer::New(argc,argv);
	DVtkRenderWindow *renWin = DVtkRenderWindow::New();
	renWin->AddRenderer(ren1);

	ren1->AddActor(triangleEdgeActor);
	ren1->AddActor(base);
	ren1->AddActor(labelActor);
	ren1->AddActor(CubeEdges);
	ren1->AddActor(CubeVertices);
	ren1->AddActor(Triangles);

	ren1->SetBackground(0.4392, 0.5020, 0.5647);
	ren1->RotateSceneX(90);

	case12(Scalars,caseLabel,0,1);	

	Grid->Modified();

	ren1->run();

	return 0;
}

void case12(vtkFloatArray *Scalars, vtkVectorText *caseLabel, int In, int Out)
{
	Scalars->InsertValue(0,Out);
	Scalars->InsertValue(1,In);
	Scalars->InsertValue(2,Out);
	Scalars->InsertValue(3,In);
	Scalars->InsertValue(4,In);
	Scalars->InsertValue(5,In);
	Scalars->InsertValue(6,Out);
	Scalars->InsertValue(7,Out);
	
	if (In == 1)
	{
		caseLabel->SetText("Case 12 - 00111010");
	}
	else
	{
		caseLabel->SetText("Case 12c - 11000101");
	}
}
