/***************************************************************************
                          office.cpp  -  description
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Michael Shore
    email                : mshore@sv.vt.edu
 ***************************************************************************/

/** port from Examples/VisualizationAlgorithms/Tcl/OfficeTubes.tcl **/

#include <dgl/dvtk/DVtkRenderer.h>
#include <dgl/dvtk/DVtkRenderWindow.h>

#include <stdlib.h>
#include <signal.h>

#include <vtk/vtkStructuredGridReader.h>
#include <vtk/vtkPointSource.h>
#include <vtk/vtkRungeKutta4.h>
#include <vtk/vtkStreamLine.h>
#include <vtk/vtkTubeFilter.h>
#include <vtk/vtkPolyDataMapper.h>
#include <vtk/vtkActor.h>
#include <vtk/vtkStructuredGridGeometryFilter.h>
#include <vtk/vtkStructuredGridOutlineFilter.h>
#include <vtk/vtkProperty.h>
#include <vtk/vtkStructuredGrid.h>

#include <vtk/vtkPolyData.h>
#include <vtk/vtkPointData.h>


int main (int argc, char** argv)
{
	const char *temp=getenv("VTK_DATA_ROOT");
   char filename[2048];
  
   if ( ! temp )
   {
		cerr << "ERROR: Please define environment variable VTK_DATA_ROOT\n";
		return NULL;
	}
	
	strcpy(filename,temp);
	strcat(filename,"/Data/office.binary.vtk");

   /** Start VTK Code **/

   // We read a data file the is a CFD analysis of airflow in an office (with
   // ventilation and a burning cigarette). We force an update so that we
   // can query the output for its length, i.e., the length of the diagonal
   // of the bounding box. This is useful for normalizing the data.
   //
   vtkStructuredGridReader *reader = vtkStructuredGridReader::New();
      reader->SetFileName(filename);
      reader->Update();//force a read to occur
		
   // Now we will generate multiple streamlines in the data. We create a random
   // cloud of points and then use those as integration seeds. We select the
   // integration order to use (RungeKutta order 4) and associate it with the
   // streamer. The start position is the position in world space where we want
   // to begin streamline integration; and we integrate in both directions. The
   // step length is the length of the line segments that make up the streamline
   // (i.e., related to display). The IntegrationStepLength specifies the
   // integration step length as a fraction of the cell size that the streamline
   // is in.
   // Create source for streamtubes
   vtkPointSource *seeds = vtkPointSource::New();
      seeds->SetRadius(0.15);
      seeds->SetCenter(0.1, 2.1, 0.5);
      seeds->SetNumberOfPoints(6);
   vtkRungeKutta4 *integ = vtkRungeKutta4::New();
   vtkStreamLine *streamer = vtkStreamLine::New();
      streamer->SetInput(reader->GetOutput());
      streamer->SetSource(seeds->GetOutput());
      streamer->SetMaximumPropagationTime(500);
      streamer->SetStepLength(0.5);
      streamer->SetIntegrationStepLength(0.05);
      streamer->SetIntegrationDirectionToIntegrateBothDirections();
      streamer->SetIntegrator(integ);

   // The tube is wrapped around the generated streamline. By varying the radius
   // by the inverse of vector magnitude, we are creating a tube whose radius is
   // proportional to mass flux (in incompressible flow).
   vtkTubeFilter *streamTube = vtkTubeFilter::New();
      streamTube->SetInput(streamer->GetOutput());
      streamTube->SetRadius(0.02);
      streamTube->SetNumberOfSides(12);
      streamTube->SetVaryRadiusToVaryRadiusByVector();
   vtkPolyDataMapper *mapStreamTube = vtkPolyDataMapper::New();
      mapStreamTube->SetInput(streamTube->GetOutput());
      mapStreamTube->SetScalarRange(reader->GetOutput()->GetPointData()->GetScalars()->GetRange());
   vtkActor *streamTubeActor = vtkActor::New();
      streamTubeActor->SetMapper(mapStreamTube);
      streamTubeActor->GetProperty()->BackfaceCullingOn();

   // From here on we generate a whole bunch of planes which correspond to
   // the geometry in the analysis; tables, bookshelves and so on.
   vtkStructuredGridGeometryFilter *table1 = vtkStructuredGridGeometryFilter::New();
      table1->SetInput(reader->GetOutput());
      table1->SetExtent(11, 15, 7, 9, 8, 8);
   vtkPolyDataMapper *mapTable1 = vtkPolyDataMapper::New();
      mapTable1->SetInput(table1->GetOutput());
      mapTable1->ScalarVisibilityOff();
   vtkActor *table1Actor = vtkActor::New();
      table1Actor->SetMapper(mapTable1);
      table1Actor->GetProperty()->SetColor(.59, .427, .392);

   vtkStructuredGridGeometryFilter *table2 = vtkStructuredGridGeometryFilter::New();
      table2->SetInput(reader->GetOutput());
      table2->SetExtent(11, 15, 10, 12, 8, 8);
   vtkPolyDataMapper *mapTable2 = vtkPolyDataMapper::New();
      mapTable2->SetInput(table2->GetOutput());
      mapTable2->ScalarVisibilityOff();
   vtkActor *table2Actor = vtkActor::New();
      table2Actor->SetMapper(mapTable2);
      table2Actor->GetProperty()->SetColor(.59, .427, .392);

   vtkStructuredGridGeometryFilter *FilingCabinet1 = vtkStructuredGridGeometryFilter::New();
      FilingCabinet1->SetInput(reader->GetOutput());
      FilingCabinet1->SetExtent(15, 15, 7, 9, 0, 8);
   vtkPolyDataMapper *mapFilingCabinet1 = vtkPolyDataMapper::New();
      mapFilingCabinet1->SetInput(FilingCabinet1->GetOutput());
      mapFilingCabinet1->ScalarVisibilityOff();
   vtkActor *FilingCabinet1Actor = vtkActor::New();
      FilingCabinet1Actor->SetMapper(mapFilingCabinet1);
      FilingCabinet1Actor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *FilingCabinet2 = vtkStructuredGridGeometryFilter::New();
      FilingCabinet2->SetInput(reader->GetOutput());
      FilingCabinet2->SetExtent(15, 15, 10, 12, 0, 8);
   vtkPolyDataMapper *mapFilingCabinet2 = vtkPolyDataMapper::New();
      mapFilingCabinet2->SetInput(FilingCabinet2->GetOutput());
      mapFilingCabinet2->ScalarVisibilityOff();
   vtkActor *FilingCabinet2Actor = vtkActor::New();
      FilingCabinet2Actor->SetMapper(mapFilingCabinet2);
      FilingCabinet2Actor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf1Top = vtkStructuredGridGeometryFilter::New();
      bookshelf1Top->SetInput(reader->GetOutput());
      bookshelf1Top->SetExtent(13, 13, 0, 4, 0, 11);
   vtkPolyDataMapper *mapBookshelf1Top = vtkPolyDataMapper::New();
      mapBookshelf1Top->SetInput(bookshelf1Top->GetOutput());
      mapBookshelf1Top->ScalarVisibilityOff();
   vtkActor *bookshelf1TopActor = vtkActor::New();
      bookshelf1TopActor->SetMapper(mapBookshelf1Top);
      bookshelf1TopActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf1Bottom = vtkStructuredGridGeometryFilter::New();
      bookshelf1Bottom->SetInput(reader->GetOutput());
      bookshelf1Bottom->SetExtent(20, 20, 0, 4, 0, 11);
   vtkPolyDataMapper *mapBookshelf1Bottom = vtkPolyDataMapper::New();
      mapBookshelf1Bottom->SetInput(bookshelf1Bottom->GetOutput());
      mapBookshelf1Bottom->ScalarVisibilityOff();
   vtkActor *bookshelf1BottomActor = vtkActor::New();
      bookshelf1BottomActor->SetMapper(mapBookshelf1Bottom);
      bookshelf1BottomActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf1Front = vtkStructuredGridGeometryFilter::New();
      bookshelf1Front->SetInput(reader->GetOutput());
      bookshelf1Front->SetExtent(13, 20, 0, 0, 0, 11);
   vtkPolyDataMapper *mapBookshelf1Front = vtkPolyDataMapper::New();
      mapBookshelf1Front->SetInput(bookshelf1Front->GetOutput());
      mapBookshelf1Front->ScalarVisibilityOff();
   vtkActor *bookshelf1FrontActor = vtkActor::New();
      bookshelf1FrontActor->SetMapper(mapBookshelf1Front);
      bookshelf1FrontActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf1Back = vtkStructuredGridGeometryFilter::New();
      bookshelf1Back->SetInput(reader->GetOutput());
      bookshelf1Back->SetExtent(13, 20, 4, 4, 0, 11);
   vtkPolyDataMapper *mapBookshelf1Back = vtkPolyDataMapper::New();
      mapBookshelf1Back->SetInput(bookshelf1Back->GetOutput());
      mapBookshelf1Back->ScalarVisibilityOff();
   vtkActor *bookshelf1BackActor = vtkActor::New();
      bookshelf1BackActor->SetMapper(mapBookshelf1Back);
      bookshelf1BackActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf1LHS = vtkStructuredGridGeometryFilter::New();
      bookshelf1LHS->SetInput(reader->GetOutput());
      bookshelf1LHS->SetExtent(13, 20, 0, 4, 0, 0);
   vtkPolyDataMapper *mapBookshelf1LHS = vtkPolyDataMapper::New();
      mapBookshelf1LHS->SetInput(bookshelf1LHS->GetOutput());
      mapBookshelf1LHS->ScalarVisibilityOff();
   vtkActor *bookshelf1LHSActor = vtkActor::New();
      bookshelf1LHSActor->SetMapper(mapBookshelf1LHS);
      bookshelf1LHSActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf1RHS = vtkStructuredGridGeometryFilter::New();
      bookshelf1RHS->SetInput(reader->GetOutput());
      bookshelf1RHS->SetExtent(13, 20, 0, 4, 11, 11);
   vtkPolyDataMapper *mapBookshelf1RHS = vtkPolyDataMapper::New();
      mapBookshelf1RHS->SetInput(bookshelf1RHS->GetOutput());
      mapBookshelf1RHS->ScalarVisibilityOff();
   vtkActor *bookshelf1RHSActor = vtkActor::New();
      bookshelf1RHSActor->SetMapper(mapBookshelf1RHS);
      bookshelf1RHSActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf2Top = vtkStructuredGridGeometryFilter::New();
      bookshelf2Top->SetInput(reader->GetOutput());
      bookshelf2Top->SetExtent(13, 13, 15, 19, 0, 11);
   vtkPolyDataMapper *mapBookshelf2Top = vtkPolyDataMapper::New();
      mapBookshelf2Top->SetInput(bookshelf2Top->GetOutput());
      mapBookshelf2Top->ScalarVisibilityOff();
   vtkActor *bookshelf2TopActor = vtkActor::New();
      bookshelf2TopActor->SetMapper(mapBookshelf2Top);
      bookshelf2TopActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf2Bottom = vtkStructuredGridGeometryFilter::New();
      bookshelf2Bottom->SetInput(reader->GetOutput());
      bookshelf2Bottom->SetExtent(20, 20, 15, 19, 0, 11);
   vtkPolyDataMapper *mapBookshelf2Bottom = vtkPolyDataMapper::New();
      mapBookshelf2Bottom->SetInput(bookshelf2Bottom->GetOutput());
      mapBookshelf2Bottom->ScalarVisibilityOff();
   vtkActor *bookshelf2BottomActor = vtkActor::New();
      bookshelf2BottomActor->SetMapper(mapBookshelf2Bottom);
      bookshelf2BottomActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf2Front = vtkStructuredGridGeometryFilter::New();
      bookshelf2Front->SetInput(reader->GetOutput());
      bookshelf2Front->SetExtent(13, 20, 15, 15, 0, 11);
   vtkPolyDataMapper *mapBookshelf2Front = vtkPolyDataMapper::New();
      mapBookshelf2Front->SetInput(bookshelf2Front->GetOutput());
      mapBookshelf2Front->ScalarVisibilityOff();
   vtkActor *bookshelf2FrontActor = vtkActor::New();
      bookshelf2FrontActor->SetMapper(mapBookshelf2Front);
      bookshelf2FrontActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf2Back = vtkStructuredGridGeometryFilter::New();
      bookshelf2Back->SetInput(reader->GetOutput());
      bookshelf2Back->SetExtent(13, 20, 19, 19, 0, 11);
   vtkPolyDataMapper *mapBookshelf2Back = vtkPolyDataMapper::New();
      mapBookshelf2Back->SetInput(bookshelf2Back->GetOutput());
      mapBookshelf2Back->ScalarVisibilityOff();
   vtkActor *bookshelf2BackActor = vtkActor::New();
      bookshelf2BackActor->SetMapper(mapBookshelf2Back);
      bookshelf2BackActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf2LHS = vtkStructuredGridGeometryFilter::New();
      bookshelf2LHS->SetInput(reader->GetOutput());
      bookshelf2LHS->SetExtent(13, 20, 15, 19, 0, 0);
   vtkPolyDataMapper *mapBookshelf2LHS = vtkPolyDataMapper::New();
      mapBookshelf2LHS->SetInput(bookshelf2LHS->GetOutput());
      mapBookshelf2LHS->ScalarVisibilityOff();
   vtkActor *bookshelf2LHSActor = vtkActor::New();
      bookshelf2LHSActor->SetMapper(mapBookshelf2LHS);
      bookshelf2LHSActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *bookshelf2RHS = vtkStructuredGridGeometryFilter::New();
      bookshelf2RHS->SetInput(reader->GetOutput());
      bookshelf2RHS->SetExtent(13, 20, 15, 19, 11, 11);
   vtkPolyDataMapper *mapBookshelf2RHS = vtkPolyDataMapper::New();
      mapBookshelf2RHS->SetInput(bookshelf2RHS->GetOutput());
      mapBookshelf2RHS->ScalarVisibilityOff();
   vtkActor *bookshelf2RHSActor = vtkActor::New();
      bookshelf2RHSActor->SetMapper(mapBookshelf2RHS);
      bookshelf2RHSActor->GetProperty()->SetColor(.8, .8, .6);

   vtkStructuredGridGeometryFilter *window = vtkStructuredGridGeometryFilter::New();
      window->SetInput(reader->GetOutput());
      window->SetExtent(20, 20, 6, 13, 10, 13);
   vtkPolyDataMapper *mapWindow = vtkPolyDataMapper::New();
      mapWindow->SetInput(window->GetOutput());
      mapWindow->ScalarVisibilityOff();
   vtkActor *windowActor = vtkActor::New();
      windowActor->SetMapper(mapWindow);
      windowActor->GetProperty()->SetColor(.3, .3, .5);

   vtkStructuredGridGeometryFilter *outlet = vtkStructuredGridGeometryFilter::New();
      outlet->SetInput(reader->GetOutput());
      outlet->SetExtent(0, 0, 9, 10, 14, 16);
   vtkPolyDataMapper *mapOutlet = vtkPolyDataMapper::New();
      mapOutlet->SetInput(outlet->GetOutput());
      mapOutlet->ScalarVisibilityOff();
   vtkActor *outletActor = vtkActor::New();
      outletActor->SetMapper(mapOutlet);
      outletActor->GetProperty()->SetColor(0, 0, 0);

   vtkStructuredGridGeometryFilter *inlet = vtkStructuredGridGeometryFilter::New();
      inlet->SetInput(reader->GetOutput());
      inlet->SetExtent(0, 0, 9, 10, 0, 6);
   vtkPolyDataMapper *mapInlet = vtkPolyDataMapper::New();
      mapInlet->SetInput(inlet->GetOutput());
      mapInlet->ScalarVisibilityOff();
   vtkActor *inletActor = vtkActor::New();
      inletActor->SetMapper(mapInlet);
      inletActor->GetProperty()->SetColor(0, 0, 0);

   vtkStructuredGridOutlineFilter *outline = vtkStructuredGridOutlineFilter::New();
      outline->SetInput(reader->GetOutput());
   vtkPolyDataMapper *mapOutline = vtkPolyDataMapper::New();
      mapOutline->SetInput(outline->GetOutput());
   vtkActor *outlineActor = vtkActor::New();
      outlineActor->SetMapper(mapOutline);
      outlineActor->GetProperty()->SetColor(0, 0, 0);
   
   /** End VTK Code **/

	//Create Renderer and RenderWindow
   DVtkRenderer *ren;
   DVtkRenderWindow *renwin;

   ren = DVtkRenderer::New(argc,argv);
   renwin = DVtkRenderWindow::New();

	//Add Renderer to RenderWindow (imitating VTK code)
   renwin -> AddRenderer(ren);

	//Add actors to renderer
   ren -> AddActor(table1Actor);
   ren -> AddActor(table2Actor);
   ren -> AddActor(FilingCabinet1Actor);
   ren -> AddActor(FilingCabinet2Actor);
   ren -> AddActor(bookshelf1TopActor);
   ren -> AddActor(bookshelf1BottomActor);
   ren -> AddActor(bookshelf1FrontActor);
   ren -> AddActor(bookshelf1BackActor);
   ren -> AddActor(bookshelf1LHSActor);
   ren -> AddActor(bookshelf1RHSActor);
   ren -> AddActor(bookshelf2TopActor);
   ren -> AddActor(bookshelf2BottomActor);
   ren -> AddActor(bookshelf2FrontActor);
   ren -> AddActor(bookshelf2BackActor);
   ren -> AddActor(bookshelf2LHSActor);
   ren -> AddActor(bookshelf2RHSActor);
   ren -> AddActor(windowActor);
   ren -> AddActor(outletActor);
   ren -> AddActor(inletActor);
   ren -> AddActor(outlineActor);
   ren -> AddActor(streamTubeActor);

	//Set background color
   ren -> SetBackground(0.5,0.6,0.7);

	//Run application
	ren -> run();	
	
	//Cleanup
	table1Actor->Delete();
	table2Actor->Delete();
	FilingCabinet1Actor->Delete();
	FilingCabinet2Actor->Delete();
	bookshelf1TopActor->Delete();
	bookshelf1BottomActor->Delete();
	bookshelf1FrontActor->Delete();
	bookshelf1BackActor->Delete();
	bookshelf1LHSActor->Delete();
	bookshelf1RHSActor->Delete();
	bookshelf2TopActor->Delete();
	bookshelf2BottomActor->Delete();
	bookshelf2FrontActor->Delete();
	bookshelf2BackActor->Delete();
	bookshelf2LHSActor->Delete();
	bookshelf2RHSActor->Delete();
	windowActor->Delete();
	outletActor->Delete();
	inletActor->Delete();
	outlineActor->Delete();
	streamTubeActor->Delete();
	
   ren->Delete();

   return 0;
}
