/***************************************************************************
                          simple2.cpp  -  description
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Michael Shore
    email                : mshore@sv.vt.edu
 ***************************************************************************/

/** port of example from vtk2CAVE **/

#include "dgl/dvtk/DVtkRenderer.h"
#include "dgl/dvtk/DVtkRenderWindow.h"

#include <stdlib.h>
#include <signal.h>

#include "vtk/vtkTransform.h"
#include "vtk/vtkTransformFilter.h"
#include "vtk/vtkDataSetMapper.h"
#include "vtk/vtkElevationFilter.h"
#include "vtk/vtkExtractEdges.h"
#include "vtk/vtkSphereSource.h"
#include "vtk/vtkCylinderSource.h"

#include "vtk/vtkPolyData.h"
#include "vtk/vtkPointData.h"

bool go;
void signal_catcher(int sig)
{
   go = false;
}

int main (int argc, char** argv)
{

//Begin VTK Actor Code
	
    //Create a new sphere source
    vtkSphereSource *src = vtkSphereSource::New();
    src->SetThetaResolution(25);
    src->SetPhiResolution(25);

    //Create a wireframe, elevation mapped sphere
    vtkTransform * aTransform = vtkTransform::New();
    aTransform->Translate(0., 1.0, 0.7);
    aTransform->RotateY(1.);

    vtkElevationFilter *colorIt = vtkElevationFilter::New();
    colorIt->SetInput(src->GetOutput());
    colorIt->SetLowPoint(0,0,-1);
    colorIt->SetHighPoint(0,0,1);

    vtkExtractEdges *edg = vtkExtractEdges::New();
    edg->SetInput(colorIt->GetOutput());

    vtkTransformFilter * aTF = vtkTransformFilter::New();
    aTF->SetInput(edg->GetOutput());
    aTF->SetTransform(aTransform);

    vtkDataSetMapper *mapper = vtkDataSetMapper::New();
    mapper->GlobalImmediateModeRenderingOn();
    mapper->SetInput(aTF->GetOutput());

    vtkActor *actor = vtkActor::New();
    actor->SetMapper(mapper);


    // Create a solid sphere, 3 feet above
    vtkTransform * aTransform2 = vtkTransform::New();
    aTransform2->Translate(0., 1.0, -0.5);

    vtkTransformFilter * aTF2 = vtkTransformFilter::New();
    aTF2->SetInput(src->GetOutput());
    aTF2->SetTransform(aTransform2);

    vtkDataSetMapper *mapper2 = vtkDataSetMapper::New();
    mapper2->SetInput(aTF2->GetOutput());

    vtkActor *actor2 = vtkActor::New();
    actor2->SetMapper(mapper2);

//End VTK Actor Code

   //Create Signal Catcher
   signal(SIGINT, signal_catcher);
   signal(SIGTERM, signal_catcher);

   go=true;

   //Create Renderer and RenderWindow
   DVtkRenderer *ren;
   DVtkRenderWindow *renwin;
   
   ren = DVtkRenderer::New(argc,argv);
   renwin = DVtkRenderWindow::New();

   //Add Renderer to RenderWindow (imitating VTK code)
   renwin -> AddRenderer(ren);

   //Add Actors to Renderer
   ren -> AddActor(actor);
   ren -> AddActor(actor2);

   int count = ren -> GetActors() -> GetNumberOfItems();

   if (count == 0)
      go = false;

   //Render Loop
	while (ren ->isRunning() && go)
   {
		//rotate top sphere
      aTransform->RotateZ(1);
		
      //render
      ren -> Render();
   }

   //Cleanup
	actor->Delete();
	actor2->Delete();
   ren->Delete();
   //renwin -> Delete();

	

   return 0;
}
