/** port from Example Code on www.vtk.org **/

#include <dgl/dvtk/DVtkRenderer.h>
#include <dgl/dvtk/DVtkRenderWindow.h>

#ifdef WIN32
#include "vtk/vtkProperty.h"
#include "vtk/vtkSphereSource.h"
#include "vtk/vtkPolyDataMapper.h"
#include "vtk/vtkActor.h"
#else
#include "vtkProperty.h"
#include "vtkSphereSource.h"
#include "vtkPolyDataMapper.h"
#include "vtkActor.h"
#endif

int main (int argc, char** argv)
{
  
  /** Begin VTK Actor Code **/

  // create sphere geometry
  vtkSphereSource *sphere = vtkSphereSource::New();
  sphere->SetRadius(1.0);
  sphere->SetThetaResolution(18);
  sphere->SetPhiResolution(18);

  // map to graphics library
  vtkPolyDataMapper *map = vtkPolyDataMapper::New();
  map->SetInput(sphere->GetOutput());

  // actor coordinates geometry, properties, transformation
  vtkActor *aSphere = vtkActor::New();
  aSphere->SetMapper(map);
  aSphere->GetProperty()->SetColor(0,0,1); // sphere color blue

  /** End VTK Actor Code **/

  // a renderer and render window
  DVtkRenderer *ren1 = DVtkRenderer::New(argc,argv);
  DVtkRenderWindow *renWin = DVtkRenderWindow::New();

  //Add renderer to render window
  renWin->AddRenderer(ren1);

  // add the actor to the scene
  ren1->AddActor(aSphere);
  ren1->SetBackground(1,1,1); // Background color white

  // run the application
  ren1->run();

  return 0;
  
}
