#ifndef _COMMAND_H_
#define _COMMAND_H_
#include<vector>
#include <string>

//Windows import / export stuff
#ifdef WIN32
#ifdef COMMANDER_EXPORTS
#define COMMANDER_API __declspec(dllexport)
#else
#define COMMANDER_API __declspec(dllimport)
#endif /* COMMANDER_EXPORTS */
#else
#define COMMANDER_API
#endif /* WIN32 */

using namespace std;

//=============================================================================
/// @class Command
/// @author Daniel Larimer
/// @brief Encapsulates a command.
//=============================================================================
class COMMANDER_API Command
{
    public:
        Command( const string& line );
        Command( const Command& cpy );

        /// Takes the args, with args[0] being the command itself
        Command( const vector<string>& args );

		/// @brief Set the arguments for the command.
		/// @param args - a vector containing the arguments.
        void           setArgs( const vector<string>& args ) { m_args = args;        }
		/// @brief Get the number of arugments.
		/// @return the number of arguments.
        int            numArgs()const                        { return m_args.size(); }
		/// @brief Get the arguments for the command.
		/// @return a vector containing the arguments.
        vector<string> getArgs()const                        { return m_args;        }
        string         getLine()const;

    private:
        vector<string> m_args;
};

#endif
