#include"Commander.h"
#include"CommandOut.h"
#include"CommandHandler.h"
#include<tracer/Tracer.h>
#include<iostream>
using namespace std;

//=============================================================================
/// @brief Constructor.
/// @param name - the name of the Commander object.
//=============================================================================
Commander::Commander( const string& name )
:m_name(name)
{
}

//=============================================================================
// Function Name: getCommandList
/// @brief Get the list of commands that match the input parameter.
/// @param startchars - the string of characters to search the command handlers with.
/// @returns a list of the commands that match the input paramater.
//=============================================================================
vector<string> Commander::getCommandList( const string& startchars )
{
    vector<string> cmds;
    map<string, CommandHandler*>::iterator itr = m_handlers.begin();
    while( itr != m_handlers.end() )
    {
        if( itr->first.substr( 0, startchars.size() ) == startchars )
        {
            cmds.push_back( itr->first );
        }
        itr++;
    }
    return cmds;
}

//=============================================================================
// Function Name: 
/// @brief Add a command handler to the Commander.
/// @param cmd - the command to be associated with the CommandHandler.
/// @param hndler - the CommandHandler object to add.
/// @return true if the CommandHandler was added successfully, else false.
//=============================================================================
bool Commander::addCommandHandler( const string& cmd, CommandHandler* hndler )
{
    if( m_handlers.find(cmd) != m_handlers.end() )
    {
        Tracer::error( "Command %s already known can\'t add another "
                       "handler for this command.", cmd.c_str() );
        return false;
    }
    m_handlers[cmd] = hndler;
    return true;
}

//=============================================================================
// Function Name: removeCommandHandler
/// @brief Removes the command from the list.
/// @param cmd - the command to remove.
/// @param hndler - the CommandHandler object to remove.
/// @return true if the command was removed successfully, otherwise false.
//=============================================================================
bool Commander::removeCommandHandler( const string& cmd, CommandHandler* hndler )
{
    if( m_handlers.find(cmd) == m_handlers.end() )
    {
        Tracer::error( "Command %s doesn't have any handlers, can't remove handler!",
                       cmd.c_str() );
        return false;
    }
    m_handlers.erase( m_handlers.find(cmd) );
    return true;
}

//=============================================================================
/// @brief Executes a Commander command, allowing Commander to load and unload
/// plugins dynamically.
/// @param cmd - the command to execute.
/// @return a pointer to a CommandOut object.
//=============================================================================
CommandOut* Commander::execute( const Command& cmd )
{
    cout<<"Commander::execute( "<<cmd.getLine()<<" )\n";    
    CommandOut* out = new CommandOut( cmd );
    if( cmd.numArgs() == 0 ) return out;
    map<string, CommandHandler*>::iterator itr = m_handlers.find( cmd.getArgs()[0] );
    if( itr == m_handlers.end() )
    {
        out->print( "commander: %s: command not found\n", cmd.getArgs()[0].c_str() );
        Tracer::error( "commander: %s: command not found\n", cmd.getArgs()[0].c_str() );
        return out;
    }
    delete out;
    return itr->second->execute( cmd );
}
