#ifndef _COMMANDER_H_
#define _COMMANDER_H_
#include<commander/Command.h>

#include<map>
#include<string>
using namespace std;

class CommandHandler;
class CommandOut;
// used for callbacks
//typedef ((CommandOut*)(void)) execfunction;


//=============================================================================
/// @class Commander
/// @author Daniel Larimer
/// @brief The command engine; Takes commands and dispatches them.
//=============================================================================
class COMMANDER_API Commander 
{
//    Q_OBJECT
    public:
        Commander(const string& name);

        string getCommanderName()const                { return m_name; }
        void   setCommanderName( const string& name ) { m_name = name; }

        /// Returns a list of commands that start with the specified string
        vector<string> getCommandList( const string& startchars = "" );

        //=============================================================================
        /// Adds a command handler class for the specified command
        /// @return false - if the command already has a handler 
        /// @return true - on success
        //=============================================================================
        bool        addCommandHandler( const string& cmd, CommandHandler* hndler );

        //=============================================================================
        /// Adds a call custom call back function for the specified command
        //=============================================================================
//        bool        addCommandHandler( const string& cmd, void* obj, execfunction* exec );

        //==============================================================================
        /// Adds a slot to be called when a the command is executed.  The
        /// slot should have the form  SLOT( function( const Command&, CommandOut*& ) )
        /// and should create and modify the CommandOut 
        //==============================================================================
//        bool        addCommandHandler( const string& cmd, 
//                                       const QObject* receiver, const char* member );

        bool        removeCommandHandler( const string& cmd, CommandHandler* hndler );   

//    public slots:
        CommandOut* execute( const Command& cmd );

//    signals:
//        void executed( const Command& cmd, const CmdOut& out );
//        void commandAdded( const string& cmd );
//        void commandRemoved( const string& cmd );

    private:
        map<string, CommandHandler*>   m_handlers;
        string                         m_name;
};

#endif
