#include <pexpr/POMessage.h>
#include <pexpr/POTextMsg.h>
#include <pexpr/POBox.h>
#include <tracer/Tracer.h>
#include <msgmgr/MessageManager.h>
#ifdef WIN32
#include <winsock2.h>
#else
#include<unistd.h>
#endif
#include <iostream>
using namespace std;
#include "cdefines.h"
#include "communicator.h"


Communicator::Communicator()
{
	MessageManager::init();
	m_box = (POBox*)MessageManager::registerPOBox(COLLABBOX);
	m_serveraddr = POAddress(SERVERNAME, COLLABBOX);
}

void Communicator::sendMessage(POMessage* AMessage)
{
	m_box->sendMessage(AMessage, m_serveraddr);
}

void Communicator::connectToServer(const string& Info)
{
	m_serveraddr = POAddress(SERVERNAME, COLLABBOX);
	MessageManager::connectTo(Info, COLLABSERVERPORT);
	char buf[128];
	gethostname(buf, 128);
	string avatar;
	//cout << "Client:  " << buf << " trying to connect to collab system" << endl;
	if (getenv("DIVERSE_AVATAR") != NULL)
		avatar = string(getenv("DIVERSE_AVATAR"));
	else
	{
		Tracer::error("Could not find avatar definition\n");
		avatar="BADAVATARFILE";
	}

	string Message = string(ADDUSER) +string(":") +   string(buf) + string(":")+avatar;
	POTextMsg Msg;
	Msg.setText(Message);
	printf("Sending message\n");
	if (!m_box->sendMessage(&Msg, m_serveraddr))
		cout << "Hrm, no message\n";
}

void Communicator::disconnectFromServer()
{
	char buf[128];
	gethostname(buf, 128);

	string Message = REMOVEUSER + string(":") + string(buf);
	POTextMsg Msg;
	Msg.setText(Message);

	m_box->sendMessage(&Msg, m_serveraddr);
	MessageManager::closePOBox(COLLABBOX);
}

POMessage* Communicator::getMessage()
{
	if (m_box->hasMessages())
	{
		const POMessage* Temp = m_box->popMessage();
		POMessage* tmp = Temp->clone();
		delete Temp;
		return tmp;
	}
	else
	{
		return NULL;
	}
}

vector<POMessage*> Communicator::getMessages()
{
	vector <POMessage*> Return;
	while (m_box->hasMessages())
	{
		Return.push_back(m_box->popMessage()->clone());
	}
	return Return;
}
/*
void Communicator::setTo(const string& Name)
{
	POAddress Temp(Name, COLLABBOX);
	m_serveraddr = Temp;
}
*/
void Communicator::setTo(POAddress Name)
{
	m_serveraddr = Name;
}
void Communicator::startListen(const int& PORT)
{
	MessageManager::servicePort(PORT);
}
