#include <Inventor/actions/SoGetMatrixAction.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/SoDB.h>
#include <Inventor/SoInteraction.h>
#include <Inventor/SoSceneManager.h>
#include <Inventor/nodekits/SoNodeKit.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoRotor.h>
#include <Inventor/nodes/SoShuttle.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/fields/SoSFVec3f.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoTransformSeparator.h>
#include <Inventor/nodes/SoMatrixTransform.h>
#include <tracer/Tracer.h>
#include <dmath/dQuat.h>
#include <dmath/dMatrix.h>
#include <string>
using namespace std;
#include <dgl/dcoin/dcoin.h>
#include "coinavatar.h"

CoinAvatar::CoinAvatar()
{
	//DCoin::init();
	DCoin::init(false);
	m_node    = new SoSeparator;
	m_posnode = new SoTransform;
	m_node->ref();
	m_posnode->ref();
}

CoinAvatar::~CoinAvatar()
{
	m_node->unref();
	m_posnode->unref();
	m_model->unref();
}

void CoinAvatar::loadModel(const string& FileName)
{
	m_model = DCoin::loadFile(FileName.c_str());
	if (m_model == NULL)
	{
		Tracer::error("Could not load coin model");
	}
	float Temp[7];
	memcpy(Temp,Object::getPosition(), sizeof(float)*7);
	m_posnode->translation.setValue(Temp[0], Temp[1], Temp[2]);
	m_posnode->rotation.setValue(Temp[3], Temp[4], Temp[5], Temp[6]);

	//Now flip it 90 degrees so it is at the right position
	SoTransform* m_flip = new SoTransform;
	dMatrix test90;
	test90.makeIdentity();
	test90.makeRotate(DEG2RAD(-90), dVec3(1,0,0));
	dQuat rot = test90.getQuat();
	//Set the rotation and then the translation of the navigation node
	m_posnode->rotation.setValue(rot.x(), rot.y(), rot.z(), rot.w());

	m_node->addChild(m_posnode);
	m_node->addChild(m_flip);
	m_node->addChild(m_model);
	DCoin::getWorld()->addChild(m_node);
}

void CoinAvatar::removeModel()
{
	DCoin::getWorld()->removeChild(m_node);
}

void CoinAvatar::setScale(const float& NewScale)
{
	Object::setScale(NewScale);
	m_posnode->scaleFactor.setValue(NewScale, NewScale, NewScale);
}

void CoinAvatar::setPosition(float* Temp)
{
	m_posnode->translation.setValue(Temp[0], Temp[1], Temp[2]);
	m_posnode->rotation.setValue(Temp[3], Temp[4], Temp[5], Temp[6]);
}
