#ifndef _D_COIN_H_
#define _D_COIN_H_

#include <dgl/dglcore/DGLAugmentInterface.h>

//Windows import / export stuff
#ifdef WIN32
#ifdef DCOIN_EXPORTS
#define DCOIN_API __declspec(dllexport)
#else
#define DCOIN_API __declspec(dllimport)
#endif /* DCOIN_EXPORTS */
#else
#define DCOIN_API
#endif /* WIN32 */

//Forward declarations of class names 
class SoSceneManager;
class SoGroup;
class SoTransform;

//=============================================================================
/// dcoin.h
/// @class Dcoin dcoin.h
/// @author Andrew A. Ray
/// @brief  The interface to coin for DIVERSE.
///
/// This class basically consists of a bare bones
/// scenegraph setup with a navigated node and a non navigated node.
/// You can access the navigated node by using getWorld and the non navigated
/// version by using getEther.
///
/// CAUTION:  before you use any functions in this class make sure you call
/// Dcoin::init() because this is a static class.  What this means is you need
/// to call all the functions by doing Dcoin::functionName.
//=============================================================================
class DCOIN_API DCoin : public DGLAugmentInterface
{
	public:
	//For virtual functions
	virtual ~DCoin();
	//Initalize the class
	//Pass in false if you do not want the world flipped
	//90 degrees 
	static void init(const bool& Flip=true);
	//Get the navigated world node
	static SoGroup* getWorld();
	//Get the base node
	static SoGroup* getScene();
	//Get the non-navigated ether node
	static SoGroup* getEther();
	//Load a model file
	static SoSeparator* loadFile(const string& filename);
	//Draw the scene graph (do not call this or any other functions below this
	//unless you know exactly what you are doing and why.
	virtual void draw() const;
	virtual int postConfig();
	virtual void postProcess();
	//Filp the world properly
	static void flipWorld(const bool& Flip);

	private:
	//Private constructor so we guarantee a static class
	DCoin();
	//Flip our axis properly
	static SoTransform* m_flip;
	//Pointer to self for a static class
	static DCoin* m_self;
	//The non navigated node
	static SoGroup* m_ether;
	//The navigated node
	static SoGroup* m_world;
	//The very root of the scene graph
	static SoGroup* m_base;
	//Flip the world or not?
	static bool     m_flipval;
};
#endif
