//=============================================================================
// DGL.h
//   - This class defines the DGL framework, including: call backs, screen 
//      registration, etc.
//
// Created 5/11/03 By Daniel Larimer
//=============================================================================
#ifndef _DGL_H_
#define _DGL_H_
#include<dmath/dmath.h>
#include<vector>
#include<map>
#include<string>
#include<dmath/dVec3.h>
#include<displaymgr/DisplayCallback.h>
#include<servicemgr/GService.h>
using namespace std;

#ifdef WIN32
#ifdef DGLCORE_EXPORTS
#define DGLCORE_API __declspec(dllexport)
#else
#define DGLCORE_API __declspec(dllimport)
#endif /* DGLCORE_EXPORTS */
#else
#define DGLCORE_API
#endif /* WIN32 */

class DGLAugmentInterface;
class DCamera;

//=============================================================================
/// @class DGL DGL.h
/// @author Daniel Larimer
/// @brief The DGL engine, manages drawing, augments, navigation and input
/// 
/// This class is designed to be the center of the DGL world, but not 
/// necessarily the entire application.  It provides functions for the
/// normal DGL applications including a default event loop.  
//=============================================================================
class DGLCORE_API DGL : public GService, public DisplayCallback
{
    public:
		/// @brief Destructor.
        virtual ~DGL(){}
        static void init( int argc = 0, char** argv = NULL );

		/// @brief Sets up the registry using a configuration file. Not implemented.
		/// @param filename - the name of the configuration file to use.
        static void setupRegistry( const string& filename = "diverse.xml" ){}
        
        void preConfig();
        void postConfig();
        
        static DGL* instance();        

        static void addAugment( DGLAugmentInterface* augment );
        static void removeAugment( DGLAugmentInterface* augment );

        virtual void clear( DCamera* cam );
        virtual void draw ( DCamera* cam );
        virtual void cull ( DCamera* cam );
       
        //====== RUN LOOP======//
        void preProcess();
        void postProcess();
        
    private:
        //===========================================
        // Ensure constructor is only called once
        //===========================================
        DGL( int argc = 0, char** argv = NULL );
        static bool                m_initialized;
        static bool                m_configured;
        static DGL*                m_app;
       
        static vector<DGLAugmentInterface*> m_augments;
        
        static bool                m_setup;
};

#endif
