//=============================================================================
// DGLAugment.h
// - Defines the base class for augments to DGL.
//
// Created 5/11/03 By Daniel Larimer
//=============================================================================
#ifndef _DGLAUGMENT_H_
#define _DGLAUGMENT_H_
#include "DGLAugmentInterface.h"
#include<pluginmgr/Plugin.h>
#include<string>

#ifdef WIN32
#ifdef DGLCORE_EXPORTS
#define DGLCORE_API __declspec(dllexport)
#else
#define DGLCORE_API __declspec(dllimport)
#endif /* DGLCORE_EXPORTS */
#else
#define DGLCORE_API
#endif /* WIN32 */

using namespace std;

//=============================================================================
/// @class DGLAugment
/// @brief Defines the interface to extend DGL.
///
/// This class defines the interface for receiving events and 
/// adding new drawing functions to DGL applications.  Every DGL app
/// should derive their own class from DGLAugment where it would define
/// how to draw each frame, process, and respond to events.
//=============================================================================
class DGLCORE_API DGLAugment : public Plugin , public DGLAugmentInterface
{
    public:
        DGLAugment( const string& name = "DGLAugment" );
        virtual ~DGLAugment();
       
        /// @brief All OpenGL code goes here.  This function may be called
        /// by multiple threads at the same time
        virtual void draw()const{};

        /// @brief OpenGL code which must be displayed last goes here.  This function may be called
        /// by multiple threads at the same time
        virtual void postdraw()const{};

        /// @brief This function is called before the navigation transformation 
        /// and may be called by multiple threads at the same time
        virtual void preNavDraw()const{};

        /// @brief Called after Sync and before DrawFrame, fast code only
        virtual void preProcess(){};
        
        /// @brief Called after FrawFrame before the next sync, all other processing
        virtual void postProcess(){};
        
        /// @brief Called before all DGLDisplay::Config() functions
        virtual int preConfig(){ return 0; };

        /// @brief Called after all displays have been configured
        virtual int postConfig(){ return 0; };
       
        virtual void load();
        virtual void unload();

        /// @brief Called when the GL context is initialized
        /// should set the GL state
        virtual void initContext(){};

        /// @brief Called when the GL context is initialized
        /// should initialize textures and display lists
        virtual void initContextLists(){};
};

#endif
