#ifndef _DGL_AUGMENT_INTERFACE_H_
#define _DGL_AUGMENT_INTERFACE_H_

//=============================================================================
/// @class DGLAugmentInterface
/// @brief This is the base class for all DGL augments.
///
/// All DGL augments should be derived from this class. Only the stages of the
/// application necessary for your augment need to be overridden.
//=============================================================================
class DGLAugmentInterface
{
    public:
       
        /// @brief All OpenGL code goes here.  This function may be called
        /// by multiple threads at the same time
        virtual void draw()const{};

        /// @brief OpenGL code which must be displayed last goes here.  This function may be called
        /// by multiple threads at the same time
        virtual void postdraw()const{};

        /// @brief This function is called before the navigation transformation 
        /// and may be called by multiple threads at the same time
        virtual void preNavDraw()const{};

        /// @brief Called after Sync and before DrawFrame, fast code only
        virtual void preProcess(){};
        
        /// @brief Called after FrawFrame before the next sync, all other processing
        virtual void postProcess(){};
        
        /// @brief Called before all DGLDisplay::Config() functions
        virtual int preConfig(){ return 0; };

        /// @brief Called after all displays have been configured
        virtual int postConfig(){ return 0; };

        /// @brief Called when the GL context is initialized
        /// should set the GL state
        virtual void initContext(){};

        /// @brief Called when the GL context is initialized
        /// should initialize textures and display lists
        virtual void initContextLists(){};
};
#endif
