/***************************************************************************
                          DVtkRenderWindow.h  -  Version 0.3.2
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Michael Shore
    email                : mshore@vt.edu
 ***************************************************************************/

#ifndef VTKDGLRENDERWINDOW_H
#define VTKDGLRENDERWINDOW_H

#include <vtk/vtkOpenGLRenderWindow.h>

#ifdef WIN32
#ifdef DVTK_EXPORTS
#define DVTK_API __declspec(dllexport)
#else
#define DVTK_API __declspec(dllimport)
#endif /* DVTK_EXPORTS */
#else
#define DVTK_API
#endif /* WIN32 */

class DVtkRenderer;


//=============================================================================
/// @class DVtkRenderWindow
/// @author Michael Shore
/// @brief This class provides a neutered vtkRenderWindow class
/// for DVtkRenderer to interact with.
///
/// Since DGL handles all our windowing needs, we don't want VTK's RenderWindows
/// to get in our way!
//=============================================================================
class DVTK_API DVtkRenderWindow : public vtkOpenGLRenderWindow
{
   public:

		static DVtkRenderWindow *New();

		void AddRenderer(DVtkRenderer *newRen);

		void Render();

		void MakeCurrent();

		//Definition of unused virtual methods
		/// @brief Definition of unused virtual method. Not implemented.
		void SetDisplayId(void*){}
		/// @brief Definition of unused virtual method. Not implemented.
		void SetWindowId(void*){}
		/// @brief Definition of unused virtual method. Not implemented.
		void SetParentId(void*){}
		/// @brief Definition of unused virtual method. Not implemented.
		void* GetGenericDisplayId(){return NULL;}
		/// @brief Definition of unused virtual method. Not implemented.
		void* GetGenericWindowId(){return NULL;}
		/// @brief Definition of unused virtual method. Not implemented.
		void* GetGenericParentId(){return NULL;}
		/// @brief Definition of unused virtual method. Not implemented.
		void* GetGenericContext(){return NULL;}
		/// @brief Definition of unused virtual method. Not implemented.
		void* GetGenericDrawable(){return NULL;}
		/// @brief Definition of unused virtual method. Not implemented.
		void SetWindowInfo(char*){}
		/// @brief Definition of unused virtual method. Not implemented.
		void SetParentInfo(char*){}
		/// @brief Definition of unused virtual method. Not implemented.
		void Start(){}
		/// @brief Definition of unused virtual method. Not implemented.
		void Frame(){}
		/// @brief Definition of unused virtual method. Not implemented.
		void HideCursor(){}
		/// @brief Definition of unused virtual method. Not implemented.
		void ShowCursor(){}
		/// @brief Definition of unused virtual method. Not implemented.
		void SetFullScreen(int){}
		/// @brief Definition of unused virtual method. Not implemented.
		void WindowRemap(){}
		/// @brief Definition of unused virtual method. Not implemented.
		int  GetEventPending(){return 0;}

   protected:
      DVtkRenderWindow();
      ~DVtkRenderWindow();

      DVtkRenderer *ren;

   private:
      DVtkRenderWindow(const DVtkRenderWindow*); // not implemented
      void operator= (const DVtkRenderWindow*); // not implemented

};

#endif
