/***************************************************************************
                          DVtkRenderer.h  -  Version 0.2.1
                             -------------------
    begin                : Fri Apr 26 2002
    copyright            : (C) 2002 by Michael Shore
    email                : mshore@sv.vt.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VTKDGLRENDERER_H
#define VTKDGLRENDERER_H

#include <vtk/vtkOpenGLRenderer.h>

#include <dgl/libdgl/DglApp.h>

#include <pthread.h>

#ifdef WIN32
#ifdef DVTK_EXPORTS
#define DVTK_API __declspec(dllexport)
#else
#define DVTK_API __declspec(dllimport)
#endif /* DVTK_EXPORTS */
#else
#define DVTK_API
#endif /* WIN32 */

//=============================================================================
/// @class DVtkRenderer
/// @author Michael Shore
/// @brief 
//=============================================================================
class DVTK_API DVtkRenderer : public vtkOpenGLRenderer, public DglApp
{
   public:

	static DVtkRenderer *New(int argc, char** argv); //create a new instance of DVtkRenderer

	void AddActor(vtkProp *p);

	void RotateSceneX(float deg);

	void Render(); //Render the scene

   protected:
      DVtkRenderer(int argc, char** argv);	   //constructor
      ~DVtkRenderer();   //destructor

      /** Callback Methods **/
	void preProcess();
	void initContext();
	void draw() const;

		/** lock and unlock the mutex for thread safety **/
      void lock() {pthread_mutex_lock(&mutex);}
      void unlock() {pthread_mutex_unlock(&mutex);}

	friend void MyUpdateGeometry(void *ren); // friend function to get around draw() const problem

   private:
      DVtkRenderer(const DVtkRenderer&);   // not implemented
      void operator= (const DVtkRenderer&);  // not implemented

		float degX,degY,degZ;

      pthread_mutex_t mutex;

};

#endif
