#include"CDglApp.h"
#include<iostream>
#include<dgl/dglcore/DGL.h>
#include<servicemgr/GServiceManager.h>
using namespace std;

bool                    CDglApp::m_init             = false;
dglPostConfigCB         CDglApp::m_postconfig       = NULL;
dglPreConfigCB          CDglApp::m_preconfig        = NULL;
dglDisplayCB            CDglApp::m_display          = NULL;
dglPreNavDisplayCB      CDglApp::m_pndsp            = NULL;
dglPostProcessCB        CDglApp::m_preproc          = NULL;
dglPreProcessCB         CDglApp::m_postproc         = NULL;
dglInitContextCB        CDglApp::m_contextproc      = NULL;
dglInitContextListsCB   CDglApp::m_contextlistsproc = NULL;
dglWandCB               CDglApp::m_cwand            = NULL;
dglKeyCB                CDglApp::m_ckey             = NULL;
dglMouseCB              CDglApp::m_cmouse           = NULL;
dglTrackerCB            CDglApp::m_ctracker         = NULL;
void*                   CDglApp::m_data             = NULL;
CDglApp*                CDglApp::m_app              = NULL;

//=============================================================================
/// @brief Initializes the CDglApp.
/// @param argc - number of command line arguments.
/// @param argv - the aurguments.
//=============================================================================
void CDglApp::init(int argc, char** argv)
{
    if( !m_init )
    {
        m_app = new CDglApp( argc, argv);
    }
}

//=============================================================================
/// @brief Constructor.
/// @param argc - number of command line arguments.
/// @param argv - the aurguments.
/// @param name - the name of the application.
//=============================================================================
CDglApp::CDglApp( int argc, char** argv, const string& name )
    :DglApp(argc, argv, "diverse.xml", true)
{
    setupRegistry();
    m_init = true;
}

//=============================================================================
/// @brief Configure and run DIVERSE applications in an endless loop
/// until all displays are closed.
//=============================================================================
void CDglApp::runDgl()
{
    m_app->configApp();
    m_app->run();
}

//=============================================================================
/// @brief Start the service manager and passes control back to the the calling function.
///
/// Should only be used if more control over the application loop is needed.
//=============================================================================
void CDglApp::startDgl()
{
    GServiceManager::start();
}

//=============================================================================
/// @brief Calls the display callback if one was set.
//=============================================================================
void CDglApp::draw()const
{
    if( m_display )
        (m_display)();
}

//=============================================================================
/// @brief Calls the preNav draw callback if one was set.
//=============================================================================
void CDglApp::preNavDraw()const
{
    if( m_pndsp )
        (m_pndsp)();
}

//=============================================================================
/// @brief Calls the init context callback if one was set.
//=============================================================================
void CDglApp::initContext()
{
    if ( m_contextproc )
        (m_contextproc)();
}

//=============================================================================
/// @brief Calls the init context lists callback if one was set.
//=============================================================================
void CDglApp::initContextLists()
{
    if ( m_contextlistsproc )
        (m_contextlistsproc)();
}

//=============================================================================
/// @brief Calls the post process callback if one was set.
//=============================================================================
void CDglApp::postProcess()
{
    if( m_postproc )
        (m_postproc)();
}

//=============================================================================
/// @brief Calls the pre process callback if one was set.
//=============================================================================
void CDglApp::preProcess()
{
    if( m_preproc )
        (m_preproc)();
}

//=============================================================================
/// @brief Calls the post config callback if one was set.
//=============================================================================
int CDglApp::postConfig()
{
    if( m_postconfig )
        (m_postconfig)();
    return 0;
}

//=============================================================================
/// @brief Calls the pre config callback if one was set.
//=============================================================================
int CDglApp::preConfig()
{
    if( m_preconfig )
        (m_preconfig)();
    return 0;
}

//=============================================================================
/// @brief Calls the wand event callback if one was set.
//=============================================================================
bool CDglApp::onWandEvent(DWandEvent* evt)
{
	if (m_cwand)
		return (m_cwand)(evt);
	return false;
}

//=============================================================================
/// @brief Calls the tracker event callback if one was set.
//=============================================================================
bool CDglApp::onTrackerEvent(DTrackerEvent* evt)
{
	if (m_ctracker)
	{
		return (m_ctracker)(evt);
	}
	return false;
}

//=============================================================================
/// @brief Calls the mouse event callback if one was set.
//=============================================================================
bool CDglApp::onMouseEvent(DMouseEvent* evt)
{
	if (m_cmouse)
		return (m_cmouse)(evt);
	return false;
}

//=============================================================================
/// @brief Calls the keyboard event callback if one was set.
//=============================================================================
bool CDglApp::onKeyEvent(DKeyEvent* evt)
{
	if (m_ckey)
		return (m_ckey)(evt);
	return false;
}
