#include<dgl/libdgl/CDglApp.h>
#include<servicemgr/GServiceManager.h>
#include<dgl/dglcore/DGL.h>
#include"CDglApp.h"
#include"cdgl.h"


extern "C"
{
	//=============================================================================
	/// @brief Causes frame to be called for all services, thereby rendering each
	/// available display
	//=============================================================================
	void dglFrame()
	{
		GServiceManager::frame();
	}

	//=============================================================================
	/// @brief Tells DGL to configure itself and run through 1 iteration.
	///
	/// Should be used when the programmer needs extra control over the application.
	/// Used in conjunction with DGL::frame() and DGL::sync().
	//=============================================================================
	void dglStart()
	{
        CDglApp::instance()->configApp();
		CDglApp::startDgl();
	}

	//=============================================================================
	/// @brief Returns true if DGL is running, otherwise returns false.
	//=============================================================================
	bool dglIsRunning()
	{
		return GServiceManager::isRunning();
	}

	//=============================================================================
	/// @brief Gets the pointer to CDglApp::m_data whch is a void pointer. Can be
	/// used anytime.
	//=============================================================================
	void* dglGetData()
	{
		return CDglApp::m_data;
	}

	//=============================================================================
	/// @brief Set the CDglApp::m_data to point to Data.
	/// @param Data - a void pointer which can be used to store a pointer to any object.
	//=============================================================================
	void dglSetData(void* Data)
	{
		CDglApp::m_data = Data;

	}

	//=============================================================================
	/// @brief Initializes DGL. Should be called prior to any other DGL function.
	/// @param argc - number of command line arguments.
	/// @param argv - the aurguments.
	//=============================================================================
    void dglInit( int argc, char** argv )
    {
        CDglApp::init(argc, argv);
    }

	//=============================================================================
	/// @brief Starts DGL and runs continuously until all displays are closed.
	//=============================================================================
    void dglRun()
    {
        CDglApp::runDgl();
    }

	//=============================================================================
	/// @brief Set the preConfig callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application during preConfig which occurs prior
	/// to creation of any display. Called only once prior to any
	/// rendering.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglPreConfigCallback( dglPreConfigCB cb )
    {
        CDglApp::m_preconfig = cb;
    }
    
	//=============================================================================
	/// @brief Set the postConfig callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application during postConfig which occurs after
	/// the displays have been created and the OpenGL contexts have
	/// been created for each display. Called only once prior to any 
	/// rendering.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglPostConfigCallback( dglPostConfigCB cb )
    {
        CDglApp::m_postconfig = cb;
    }
    
	//=============================================================================
	/// @brief Set the display callback function.
	///
	/// A callback function to be used to have DGL call a function for drawing within
	/// your application.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglDisplayCallback( dglDisplayCB cb)
    {
        CDglApp::m_display = cb;
    }
    
	//=============================================================================
	/// @brief Set the preProcess callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application during preProcess which occurs before
	/// each frame is rendered. Called once every loop.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglPreProcessCallback( dglPreProcessCB cb )
    {
        CDglApp::m_preproc = cb;
    }

	//=============================================================================
	/// @brief Set the postProcess callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application during postProcess which occurs after
	/// each frame is rendered. Called once every loop.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglPostProcessCallback( dglPostProcessCB cb )
    {
        CDglApp::m_postproc = cb;
    }

	//=============================================================================
	/// @brief Set the initContext callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application prior to the first frame being rendered.
	/// This should be used to set up the OpenGL state machine by
	/// enabling and disabling states. It will be called only once 
	/// for each display.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglInitContextCallback( dglInitContextCB cb )
    {
        CDglApp::m_contextproc = cb;
    }

	//=============================================================================
	/// @brief Set the initContextLists callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application prior to the first frame being rendered
	/// and after the dglInitContextCallback is called (if defined).
	/// This should be used to set up display lists and textures prior
	/// to rendering. It will be called for only once for each display.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglInitContextListsCallback( dglInitContextListsCB cb )
    {
        CDglApp::m_contextlistsproc = cb;
    }

	//=============================================================================
	/// @brief Set the preNav callback function.
	///
	/// Allows a callback to be called prior to any navigation in the application 
	/// cycle.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglPreNavDisplayCallback( dglPreNavDisplayCB cb)
    {
	    CDglApp::m_pndsp = cb;
    }

	//=============================================================================
	/// @brief Set the onWandEvent callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application to access wand events. Useful if you
	/// want customized control of wand input.
	//=============================================================================
    void dglWandCallback( dglWandCB cb)
    {
	    CDglApp::m_cwand = cb;
    }

	//=============================================================================
	/// @brief Set the onTrackerEvent callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application to access tracker events. Useful if you
	/// want customized control of tracker (i.e. headtracker) input.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglTrackerCallback( dglTrackerCB cb)
    {
	    CDglApp::m_ctracker = cb;
    }

	//=============================================================================
	/// @brief Set the onMouseEvent callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application to access mouse events. Useful if you
	/// want customized control of mouse input.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglMouseCallback( dglMouseCB cb)
    {
	    CDglApp::m_cmouse = cb;
    }

	//=============================================================================
	/// @brief Set the onKeyEvent callback function.
	///
	/// A callback function to be used to have DGL call a function
	/// within your application to access key events. Useful if you
	/// want customized control of keyboard input.
	/// @param cb - the callback function to be set.
	//=============================================================================
    void dglKeyCallback( dglKeyCB cb)
    {
	    CDglApp::m_ckey = cb;
    }

	//=============================================================================
	/// @brief A callback function to be used to have DGL call a function
	/// within your application to perform customized sync behaviour.
	//=============================================================================
    void dglSync()
    {
	    GServiceManager::sync();
    }
}    
