#ifndef _DCAMERA_H_
#define _DCAMERA_H_
#include<dmath/Rect.h>
#include<dmath/dMatrix.h>

//=============================================================================
/// @class DCamera
/// @author Daniel Larimer
/// @brief The DCamera class describes the camera's view.
///
/// It maintains projection and model view matrices as well
/// as the viewport rectangle used with the camera which describe the camera's
/// position, orientation, and viewport within the DIVERSE world.
//=============================================================================
class DCamera
{
    public:
		/// @brief Constructor.
		/// @param proj - the projection rectangle
		/// @param view - the projection matrix.
		/// @param model - the model view matrix.
        DCamera( const Rect& proj, const float* view, const float* model )
        :m_projectionrect(proj), m_viewmatrix(view), m_modelmatrix(model){}

        /// @brief Get the camera's projection matrix.
		/// @return the matrix that describes the fustrum
        float* getProjectionMatrix()const{ return m_viewmatrix.ptr(); }

		/// @brief Get the camera's model view matrix.
        /// @return the matrix that describes the camera's position
        /// and orientation
        float* getModelViewMatrix()const { return m_modelmatrix.ptr(); }

		/// @brief Get the viewport rectangle.
        /// @return the rectangle describing the viewport
        Rect  getProjectionRect()const { return m_projectionrect; }
        
    private:
        Rect    m_projectionrect;
        dMatrix m_viewmatrix;
        dMatrix m_modelmatrix;
        
};


#endif
