#include"DDisplay.h"
#include"DisplayConfig.h"
#include"DWindow.h"
#include"DViewport.h"
#include<Producer/CameraConfig>
#include<Producer/CameraGroup>
#include <fontmgr/FontManager.h>

using namespace Producer;

//=============================================================================
/// @brief Constructor.
///
/// Builds a display based on the configuration file.
/// @param cfgfile - the properties for this display,
//=============================================================================
DDisplay::DDisplay( DisplayProps* cfgfile )
{
    m_config = cfgfile;
    CameraConfig* cfg = new CameraConfig();
    for( unsigned int i = 0; i < m_config->windows.size(); i++ )
    {
        DWindow* win = new DWindow( m_config->windows[i] );
        m_windows.push_back( win );
        vector<DViewport*> vps = win->getViewports();
        for( unsigned int x = 0; x < vps.size(); x++ )
        {
            cfg->addCamera( vps[x]->getViewportName(), vps[x]->getCamera() );
        }
    }
    m_group = new Producer::CameraGroup( cfg );
}

//=============================================================================
/// @brief Destructor.
///
/// Cleans up and deletes the configuration passed in the constructor.
//=============================================================================
DDisplay::~DDisplay()
{
    delete m_config;
}

//=============================================================================
/// @brief Get the name of the display.
/// @return the name of the display.
//=============================================================================
string DDisplay::getDisplayName()const
{
    return m_config->name;
}


//=============================================================================
/// @brief Get whether or not the display is theaded.
/// @return true if the display is threaded, else false.
//=============================================================================
bool DDisplay::isThreaded()const
{
    return m_config->threaded;
}

//=============================================================================
/// @brief Provides access to the entire display configuration.
//=============================================================================
DisplayProps* DDisplay::getConfig()const
{
    return m_config;
}

//=============================================================================
/// @brief Builds the windows, creates the pipes, threads, and other 
/// display components
//=============================================================================
void DDisplay::config()
{
    if( m_config->threaded )
        m_group->realize( CameraGroup::ThreadPerCamera );
    else
        m_group->realize();
}

//=============================================================================
/// @brief Calls preFrame for all windows.
//=============================================================================
void DDisplay::preFrame()
{
    for( unsigned int i = 0 ; i < m_windows.size(); i++ )
        m_windows[i]->preFrame();
}

//=============================================================================
/// @brief Calls frame for all windows.
//=============================================================================
void DDisplay::frame()
{
    m_group->frame();
}

//=============================================================================
/// @brief Calls postFrame for all windows
//=============================================================================
void DDisplay::postFrame()
{
    for( unsigned int i = 0; i < m_windows.size(); i++ )
        m_windows[i]->postFrame();
}

//=============================================================================
/// @brief Called to sync all drawing threads
//=============================================================================
void DDisplay::sync()
{
    m_group->sync();
}
