#ifndef _DDISPLAY_H_
#define _DDISPLAY_H_
#include<Producer/CameraGroup>
#include<string>

#ifdef WIN32
#ifdef DISPLAYMGR_EXPORTS
#define DISPLAYMGR_API __declspec(dllexport)
#else
#define DISPLAYMGR_API __declspec(dllimport)
#endif /* DISPLAYMGR_EXPORTS */
#else
#define DISPLAYMGR_API
#endif /* WIN32 */

using namespace std;
using namespace Producer;

class DisplayProps;
class DWindow;

//=============================================================================
/// @class DDisplay
/// @author Daniel Larimer
/// @brief Provides a generic display based on a configuration file.
///
/// A display is a collection of windows each of which may have one
/// or more viewports.  The windows and viewports are configured to provide
/// mappings for HMD's CAVEs or any other screen based view.
///
/// The display maintains the list of windows ( DWindow pointers ) used by the
/// display.
//=============================================================================
class DISPLAYMGR_API DDisplay 
{
    public:
        DDisplay( DisplayProps* cfgfile );
        virtual ~DDisplay();

		/// @brief Get the display type.
		/// @return string containing the name of the display type.
        string getDisplayType()const { return "DDisplay"; }
        string getDisplayName()const;
        bool   isThreaded()const;

        DisplayProps* getConfig()const;

		/// @brief Called but not used.
        void preConfig(){};
        void config();
		/// @brief Called but not used.
        void postConfig(){};

        void preFrame();
        void frame();
        void postFrame();
        void sync();

		/// @brief Get a list of all windows associated with the display.
		/// @return a vector containing pointers to each of the DWindows.
        vector<DWindow*>  getWindows()const { return m_windows; }
        
    private:
        DisplayProps*            m_config;
        vector<DWindow*>         m_windows;
        Producer::CameraGroup*   m_group;
};

#endif
