#include <eventmgr/devents.h>
#include <eventmgr/EventManager.h>
#include <servicemgr/GServiceManager.h>
#include <eventmgr/keyConverter.h>
#include "DProducerInputCallback.h"

void DMKCallback::setRS(RenderSurface* rs)
{
	m_rs = rs;
}
void DMKCallback::setKM(KeyboardMouse* km)
{
	m_km = km;
}
void DMKCallback::mouseMotion(float A, float B)
{
	//Temp vars
	float C,D;
	//Put the mouse data into pixel format stored in the temp vars
	m_km->computePixelCoords(A,B, m_rs, C,D);
	//Make the window based on the 0,0 origin
	C = C-m_rs->getWindowOriginX();
	D = D-m_rs->getWindowOriginY();
	//Now flip the value for the y axis because 0 is at the bottom for them
	D = fabs(m_rs->getWindowHeight() - D);
	//If the mouse is off the screen do nothing
	if (C < 0 || D < 0)
		return;
	//Make the event
	DMouseEvent* evt = new DMouseEvent( DMouseEvent::MOVE );
	//Populate the event
        evt->setPosition( (int)C,(int)D);
	//Send the event to the system
	EventManager::pushEvent( evt );
}
void DMKCallback::passiveMouseMotion(float A, float B)
{
	//cout <<"Mouse in window moved\n";
}
void DMKCallback::buttonPress(float A, float B, unsigned int Button)
{
	DMouseEvent* evt = new DMouseEvent( DMouseEvent::PRESS);
	if (Button == 1)
		evt->setButton( DMouseEvent::LEFT, true );
	if (Button == 2)
		evt->setButton( DMouseEvent::MIDDLE, true );
	if (Button == 3)
		evt->setButton( DMouseEvent::RIGHT, true );
	float C,D;
	//Turn it into pixels
	m_km->computePixelCoords(A,B, m_rs, C,D);
	//Make the window based on the 0,0 origin
	C = C-m_rs->getWindowOriginX();
	D = D-m_rs->getWindowOriginY();
	//Now flip the value for the y axis because 0 is at the bottom for them
	D = fabs(m_rs->getWindowHeight() - D);
	//Populate event
	evt->setPosition( (int)C,(int)D);
	//Send event off to system
	EventManager::pushEvent( evt );
}
void DMKCallback::buttonRelease(float A, float B, unsigned int Button)
{
	DMouseEvent* evt = new DMouseEvent( DMouseEvent::RELEASE);
	if (Button == 0)
		evt->setButton( DMouseEvent::LEFT, true );
	if (Button == 1)
		evt->setButton( DMouseEvent::MIDDLE, true );
	if (Button == 2)
		evt->setButton( DMouseEvent::RIGHT, true );
	evt->setPosition( (int)A,(int)B);
	EventManager::pushEvent( evt );
}
void DMKCallback::keyPress(KeyCharacter A)
{
	DKeyEvent* evt = new DKeyEvent( DKeyEvent::DOWN, A, A );
	EventManager::pushEvent( evt );
}
void DMKCallback::keyRelease(KeyCharacter A)
{
	DKeyEvent* evt = new DKeyEvent( DKeyEvent::UP, A, A );
	EventManager::pushEvent( evt );
}
void DMKCallback::specialKeyRelease(KeyCharacter A)
{
	DKeyEvent* evt = new DKeyEvent( DKeyEvent::UP, A, A );
	EventManager::pushEvent( evt );
}
void DMKCallback::specialKeyPress(KeyCharacter A)
{
	if (A == KeyChar_Escape)
		GServiceManager::quit();
	DKeyEvent* evt = new DKeyEvent( DKeyEvent::DOWN, A, A );
	EventManager::pushEvent( evt );
}

void DMKCallback::shutdown()
{
	GServiceManager::quit();
}

