#ifndef _DProducerInputCallback_h_
#define _DProducerInputCallback_h_
#include <Producer/KeyboardMouse>

#ifdef WIN32
#ifdef DISPLAYMGR_EXPORTS
#define DISPLAYMGR_API __declspec(dllexport)
#else
#define DISPLAYMGR_API __declspec(dllimport)
#endif /* DISPLAYMGR_EXPORTS */
#else
#define DISPLAYMGR_API
#endif /* WIN32 */

using namespace Producer;

//=============================================================================
/// @class DMKCallback
/// @brief This class is used to push events from producer render surfaces to the 
/// DIVERSE event system.
///
/// You need to set the render surface and the KeyboardMouse object for this
/// class to function properly. If you do not set them it will not work and will
/// probably crash.
//=============================================================================
class DISPLAYMGR_API DMKCallback : public KeyboardMouseCallback
{
	public:
	//Set functions because I need to be able
	//To compute pixel coords for the system
	void setRS(RenderSurface* rs);
	void setKM(KeyboardMouse* km);
	//Standard callback functions
        virtual void mouseScroll( ScrollingMotion ){};
        virtual void mouseMotion( float A, float B) ;
        virtual void passiveMouseMotion( float A, float B);
        virtual void buttonPress( float A, float B, unsigned int Button);
        virtual void doubleButtonPress( float, float, unsigned int ){};
        virtual void buttonRelease( float A, float B, unsigned int Button);
        virtual void keyPress( KeyCharacter A);
        virtual void keyRelease( KeyCharacter A);
        virtual void specialKeyPress( KeyCharacter );
        virtual void specialKeyRelease( KeyCharacter );
        virtual void shutdown();
        virtual bool idle(){return false;};
	private:
	RenderSurface* m_rs;
	KeyboardMouse* m_km;

};
#endif
