#include"DViewport.h"
#include<tracer/Tracer.h>
#include"DWindow.h"
#include"DisplayConfig.h"
#include"DisplayManager.h"
#include"ScreenHandler.h"
#include"PerspectiveHandler.h"
#include"dgl/dglcore/DGL.h"
#include<iostream>
using namespace std;

//=============================================================================
/// @brief Constructor.
///
/// Constructs a window based on the WindowProps object, the DWindow
/// takes responsibility for the config and will delete it in the destructor.
/// @param cfg - the configuration for the window.
/// @param parent - a pointer to the parent DWindow object.
//=============================================================================
DViewport::DViewport( ViewportProps* cfg, DWindow* parent )
{
    m_parent = parent;
    m_config = cfg;
    
    m_camera = new Camera();
    m_camera->setRenderSurface( parent->getRenderSurface() );
//    m_camera->setProjectionRectangle( int(cfg->left), int(cfg->bottom), 
//                                      int(cfg->width), int(cfg->height) );    
//    m_camera->setProjectionRectangle( 0.0f, 0.0f, 1.0f, 1.0f );
    m_viewtype = cfg->camera->type;
    if( m_viewtype == CameraProps::SCREEN )
    {
        m_screen = new ScreenHandler( cfg->camera->screen, cfg->camera->eye, 
                                      cfg->camera->tracker );
        m_camera->setSceneHandler( m_screen );
    }
    else if ( m_viewtype == CameraProps::PERSPECTIVE)
    {
        m_perspective = new PerspectiveHandler( cfg->camera->perspective);
        m_perspective = new PerspectiveHandler( cfg->camera->perspective);
        m_camera->setSceneHandler( m_perspective);
		if (cfg->camera->perspective != NULL)
		{
			m_camera->setLensPerspective(  (float)cfg->camera->perspective->fovx,
										   (float)cfg->camera->perspective->fovy,
										   DisplayManager::getNearClip(), 
										   DisplayManager::getFarClip());
		}
		else
		{
			m_camera->setLensPerspective( 35,35 ,
										   DisplayManager::getNearClip(), 
										   DisplayManager::getFarClip());

		}
    }
}

//=============================================================================
/// @brief Get the name of the viewport.
/// @return a string containing the name of the viewport.
//=============================================================================
string DViewport::getViewportName()const
{
    return m_config->name;
}

//=============================================================================
/// @brief Set the name of the viewport.
///
/// The name must be unique for all children of the parent DWindow object.
/// @param name - a string containing the name of the viewport.
//=============================================================================
void DViewport::setViewportName( const string& name )
{
    m_config->name = name;
}

//=============================================================================
/// @brief Set the geometry of the viewport.
/// @param geom - a Rect containing the left and bottom coordinate and the width
/// and height of the viewport.
//=============================================================================
void DViewport::setGeometry( const Rect& geom )
{
//    m_camera->setProjectionRectangle( geom.x(), geom.y(), geom.width(), geom.height() );
    m_config->left   = geom.x();
    m_config->bottom = geom.y();
    m_config->width  = geom.width();
    m_config->height = geom.height();
}

//=============================================================================
/// @brief Get the geometry of the viewport.
/// @return a Rect containing the left and bottom coordinate and the width
/// and height of the viewport.
//=============================================================================
Rect DViewport::getGeometry()const
{
    return Rect(m_config->left, m_config->bottom, 
                m_config->width, m_config->height);
}

//=============================================================================
/// @brief Called for each camera during the preProcess stage. Does nothing.
//=============================================================================
void DViewport::preFrame()
{
}

//=============================================================================
/// @brief Called for each camera during the process stage.
//=============================================================================
void DViewport::frame()
{
    m_camera->frame();
}

//=============================================================================
/// @brief Called for each camera during the postProcess stage. Does nothing.
//=============================================================================
void DViewport::postFrame()
{
}

//=============================================================================
/// @brief Called for each camera during the sync stage.
//=============================================================================
void DViewport::sync()
{
    m_camera->sync();
}
