#ifndef _DVIEWPORT_H_
#define _DVIEWPORT_H_
#include<string>
#include<vector>
#include<dmath/Rect.h>
#include<Producer/Camera>

using namespace std;
using namespace Producer;

#ifdef WIN32
#ifdef DISPLAYMGR_EXPORTS
#define DISPLAYMGR_API __declspec(dllexport)
#else
#define DISPLAYMGR_API __declspec(dllimport)
#endif /* DISPLAYMGR_EXPORTS */
#else
#define DISPLAYMGR_API
#endif /* WIN32 */


class DWindow;
class ViewportProps;
class PerspectiveHandler;
class SceneHandler;
class ScreenHandler;

//=============================================================================
/// @class DViewport
/// @brief Provides a generic viewport based on a configuration file.
///
/// A viewport is a section of a window to which drawing will be clipped.
/// There can be multiple viewports for each window, and each viewport
/// can display a different view or perspective of the frame.
///
/// The viewport can have only one camera. Depending on the type of camera,
/// it also maintains a pointer to the respective handler ( ScreenHandler or
/// PerspectiveHandler ).
//=============================================================================
class DISPLAYMGR_API DViewport
{
    public:
        DViewport( ViewportProps* cfg, DWindow* parent );
        
        virtual string  getViewportName()const;
        virtual void    setViewportName( const string& name );
       
        virtual void    setGeometry(const Rect& newGeom);
        virtual Rect    getGeometry()const;
        
        Camera*         getCamera()const { return m_camera; }

        virtual void preFrame();
        virtual void frame();
        virtual void postFrame();
        virtual void sync();

		/// @brief Get the viewport properties.
		/// @return a pointer to the ViewportProps object.
		ViewportProps* getViewportConfig()const{return m_config;};

    private:
        ViewportProps* m_config;
        DWindow*       m_parent;
        Camera*        m_camera;
        int                m_viewtype;
        PerspectiveHandler* m_perspective;
        ScreenHandler*     m_screen;
};
#endif
