#ifndef _DWINDOW_H_
#define _DWINDOW_H_
#include<string>
#include<vector>
#include<dmath/Rect.h>
#include"DisplayConfig.h"
#include<Producer/RenderSurface>
#include<Producer/KeyboardMouse>
#include"DProducerInputCallback.h"

#ifdef WIN32
#ifdef DISPLAYMGR_EXPORTS
#define DISPLAYMGR_API __declspec(dllexport)
#else
#define DISPLAYMGR_API __declspec(dllimport)
#endif /* DISPLAYMGR_EXPORTS */
#else
#define DISPLAYMGR_API
#endif /* WIN32 */

class DViewport;

using namespace std;

//=============================================================================
/// @class DWindow
/// @brief Provides a generic window based on a configuration file.
///
/// Each window has a unique context and may contain 1 or more viewports.
///
/// The window maintains the list of viewports ( DViewport pointers ) used by the
/// window.
//=============================================================================
class DISPLAYMGR_API DWindow
{
    public:
        DWindow( WindowProps* cfg );
        ~DWindow();
    
        string             getWindowName()const;
        void               setWindowName( const string& name );
                           
        void               setWindowGeometry(const Rect& newGeom);
        Rect               getWindowGeometry()const;
        vector<DViewport*> getViewports()const;

        bool               hasBorder()const;
        void               setBorder( bool state );

		/// @brief Get the Producer render surface.
        /// @return a pointer to the render surface 
        Producer::RenderSurface*  getRenderSurface()const { return m_surface; }

        void preFrame();
        void frame();
        void postFrame();
        void sync();

		/// @brief Get the window properties.
		/// @return a pointer to the WindowProps object.
        WindowProps* getWindowConfig()const { return m_config; }

    private:
        WindowProps*                m_config;
        Producer::RenderSurface*    m_surface;
        vector<DViewport*>          m_viewports;
	    KeyboardMouse*              m_keymouse;
	    DMKCallback*                m_input;
};
#endif
