#ifndef _DISPLAYCALLBACK_H_
#define _DISPLAYCALLBACK_H_
class DCamera;

//=============================================================================
/// @class DisplayCallback
/// @author Daniel Larimer
/// @brief Defines functions that will be called each frame.
///
/// This is the base class for any display callbacks that will be used by DIVERSE.
/// You should derive your display callback from DisplayCallback and call one
/// of the DisplayManager add callback functions to register the display callback
/// so that it will be called each frame. Use the associated remove callback
/// function to stop the callback from being called each frame.
///
/// Example: See DebugHeadsUp or GnomonHeadsUp for examples of using the
/// DisplayCallback class and the add and remove callback functions.
//=============================================================================
class DisplayCallback
{
    public:
        virtual void overlay(){}
        virtual void draw ( DCamera* cam ){};
        virtual void cull ( DCamera* cam ){};
        virtual void clear( DCamera* cam ){};
        virtual void frame(){};
};

#endif
