//=============================================================================
// PerspectiveHandler.h
//  - Defines the class used by Camera to display the scene.
// 
// Created 5/12/03 By Daniel Larimer
//=============================================================================
#ifndef _PROJECTIONHANDLER_H_
#define _PROJECTIONHANDLER_H_
#include<dmath/dmath.h>
#include<Producer/Camera>
using namespace Producer;

#ifdef WIN32
#ifdef DISPLAYMGR_EXPORTS
#define DISPLAYMGR_API __declspec(dllexport)
#else
#define DISPLAYMGR_API __declspec(dllimport)
#endif /* DISPLAYMGR_EXPORTS */
#else
#define DISPLAYMGR_API
#endif /* WIN32 */

class PerspectiveProps;

//=============================================================================
/// @class PerspectiveHandler
/// @brief used by Camera to draw the scene based on PerspectiveProps.
///
/// Calls DisplayManager::draw(Camera&) to draw the scene.
/// Calls DisplayManager::clear(Camera&) to draw the scene.
/// Calls DisplayManager::cull(Camera&) to cull the scene.
//=============================================================================
class DISPLAYMGR_API PerspectiveHandler : public Producer::Camera::SceneHandler
{
    public:
        PerspectiveHandler( PerspectiveProps* cfg );
        virtual void clear( Camera& cam );
        virtual void draw ( Camera& cam );
        virtual void cull ( Camera& cam );
        virtual bool frame( Camera& cam );

		/// @brief Not implemented.
		/// @return always returns false.
        bool useAutoView() { return false; }

    private:
        PerspectiveProps* m_config;
};

#endif
