//=============================================================================
// GVector.h
// - Impliments a vector class that supports all common vector calculations
//
// Created 11/7/02 By Daniel Larimer
//=============================================================================
#ifndef GVECTOR_H_
#define GVECTOR_H_
#include<dmath/dVec3.h>
#include<iostream>

#ifdef WIN32
#ifdef DMATH_EXPORTS
#define DMATH_API __declspec(dllexport)
#else
#define DMATH_API __declspec(dllimport)
#endif /* DMATH_EXPORTS */
#else
#define DMATH_API
#endif /* WIN32 */

using namespace std;

//=============================================================================
/// @class GVector
/// @author Daniel Larimer modified by Andrew A. Ray
/// @brief A vector class with an origin and direction.
//=============================================================================
class DMATH_API GVector
{   
    public:
        GVector( const dVec3& origin, const dVec3& tip );
        GVector( float dx = 0, float dy = 0, float dz = 0);
        GVector( const GVector& cpy );

        dVec3 GetTip()const;
        dVec3 GetOrigin()const;

        void SetTip( const dVec3& tip );
        void SetOrigin( const dVec3& origin );

        GVector ToUnit()const;

        float GetMagSquared()const;
        float GetMagnitude()const;
        void   SetMagnitude( const float& mag );

		/// @brief Get the x coordinate value of the direction vector.
        inline float x()const{ return m_direction.x(); }
		/// @brief Get the y coordinate value of the direction vector.
        inline float y()const{ return m_direction.y(); }
		/// @brief Get the z coordinate value of the direction vector.
        inline float z()const{ return m_direction.z(); }
        
		/// @brief Get a reference to the x coordinate value of the direction vector.
        inline float& x(){ return m_direction.x(); }
		/// @brief Get a reference to the y coordinate value of the direction vector.
        inline float& y(){ return m_direction.y(); }
		/// @brief Get a reference to the z coordinate value of the direction vector.
        inline float& z(){ return m_direction.z(); }

        GVector  operator+ ( const GVector& vec )const;
        GVector& operator+=( const GVector& vec );
        GVector  operator- ( const GVector& vec )const;
        GVector& operator-=( const GVector& vec );
        GVector  operator* ( const float& scale )const;
        GVector& operator*=( const float& scale );
        GVector  operator^ ( const GVector& vec )const; // Cross Product
        GVector& operator^=( const GVector& vec ); // Cross Product
        float   operator* ( const GVector& vec )const; // Dot Product
        GVector& operator= ( const GVector& vec ); // Assignment
       
     private:
        dVec3  m_origin;
        dVec3  m_direction;
};

ostream& operator<<( ostream& os, const GVector& out );

#endif
