/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
 *
 * This library is open source and may be redistributed and/or modified under  
 * the terms of the OpenSceneGraph Public License (DIVERSEPL) version 0.0 or 
 * (at your option) any later version.  The full license is in LICENSE file
 * included with this distribution, and on the openscenegraph.org website.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * OpenSceneGraph Public License for more details.
*/

#ifndef __DIVERSE_MATH
#define __DIVERSE_MATH

#ifdef _DARWIN_
#ifndef __DARWIN_OSX__
#define __DARWIN_OSX__
#endif
#endif

#include <math.h>
#include<stdlib.h>

#ifdef WIN32
#include <float.h>
#ifndef isnan
#define isnan _isnan
#endif
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif
#ifdef DMATH_EXPORTS
#define DMATH_API __declspec(dllexport)
#else
#define DMATH_API __declspec(dllimport)
#endif /* DMATH_EXPORTS */
#else
#define DMATH_API
#endif /* WIN32 */

#if defined(__GNUC__) && defined(WIN32)
#  include <iostream>
#endif

// for OSX users :
// comment in if you want backwards compatibility with 10.1.x versions
// otherwise you'll have problems with missing floorf and __isnan*()
//#define DARWIN_OSX_PRE_10_2

#if defined(_MSC_VER) && (_MSC_VER >= 1300)
    #include <float.h>
#endif


#if (defined(WIN32) && !(defined(_MSC_VER) && (_MSC_VER >= 1300)) && !defined(__MINGW32__) ) || \
    defined (sun) || \
    defined (__DARWIN_OSX__)
    

    #include <float.h>


    // PJA MAC OSX
    // This appears to be the simplest way to get these defined under MACOSX
    // where they arent in math.h

    #ifndef acosf
    #define acosf (float)acos
    #endif

    #ifndef asinf
    #define asinf (float)asin
    #endif

    #ifndef cosf
    #define cosf (float)cos
    #endif

    #ifndef sinf
    #define sinf (float)sin
    #endif

    #ifndef logf
    #define logf (float)log
    #endif

    #ifndef powf
    #define powf (float)pow
    #endif

    #ifndef sqrtf
    #define sqrtf (float)sqrt
    #endif
    
    #ifndef fabsf
    #define fabsf (float)fabs
    #endif

#endif
#ifndef isnan
#define isnan
#endif
#ifndef isnanf
#define isnanf (float)isnan
#endif




// define the stand trig values
#ifdef PI
#undef PI
#undef PI_2
#undef PI_4
#endif
const double PI   = 3.14159265358979323846;
const double PI_2 = 1.57079632679489661923;
const double PI_4 = 0.78539816339744830962;

/** return the minimum of two values, equivilant to std::min.
  * std::min not used because of STL implementation under IRIX contains no std::min.*/
template<typename T>
inline T absolute(T v) { return v<(T)0?-v:v; }

/** return the minimum of two values, equivilant to std::min.
  * std::min not used because of STL implementation under IRIX contains no std::min.*/
template<typename T>
inline T minimum(T lhs,T rhs) { return lhs<rhs?lhs:rhs; }

/** return the maximum of two values, equivilant to std::max.
  * std::max not used because of STL implementation under IRIX contains no std::max.*/
template<typename T>
inline T maximum(T lhs,T rhs) { return lhs>rhs?lhs:rhs; }

template<typename T>
inline T clampTo(T v,T minimum,T maximum) { return v<minimum?minimum:v>maximum?maximum:v; }

template<typename T>
inline T clampAbove(T v,T minimum) { return v<minimum?minimum:v; }

template<typename T>
inline T clampBelow(T v,T maximum) { return v>maximum?maximum:v; }

template<typename T>
inline T sign(T v) { return v<(T)0?(T)-1:(T)1; }

template<typename T>
inline T square(T v) { return v*v; }

template<typename T>
inline T signedSquare(T v) { return v<(T)0?-v*v:v*v;; }

inline float inDegrees(float angle) { return angle*(float)PI/180.0f; }
inline double inDegrees(double angle) { return angle*PI/180.0; }

template<typename T>
inline T inRadians(T angle) { return angle; }

inline float DegreesToRadians(float angle) { return angle*(float)PI/180.0f; }
inline double DegreesToRadians(double angle) { return angle*PI/180.0; }

inline float RadiansToDegrees(float angle) { return angle*180.0f/(float)PI; }
inline double RadiansToDegrees(double angle) { return angle*180.0/PI; }

inline bool isNaN(float v) { return isnan(v) != 0; }


/** compute the volume of tetrahedron */
template<typename T>
inline float computeVolume(const T& a,const T& b,const T& c,const T& d)
{
    return fabsf(((b-c)^(a-b))*(d-b));
}

/** compute the volume of prism */
template<typename T>
inline float computeVolume(const T& f1,const T& f2,const T& f3,
                           const T& b1,const T& b2,const T& b3)
{
    return computeVolume(f1,f2,f3,b1)+
           computeVolume(b1,b2,b3,f2)+
           computeVolume(b1,b3,f2,f3);
}


#endif  // __DIVERSE_MATH
