#include"DOSG.h"
#include<iostream>
#include<string>
#include<vector>
#include<signal.h>
#ifdef WIN32
#include <windows.h>
#else
#include<dirent.h>
#endif /* WIN32 */
#include<GL/gl.h>

#ifndef __sgi
 #include<cstdio>
#else
 #include<stdio.h>
#endif

#include<navmgr/NavManager.h>
#include"diverseconfig.h"
#include<displaymgr/DisplayManager.h>
#include<displaymgr/DCamera.h>
#include<servicemgr/GServiceManager.h>
#include<dmath/dMatrix.h>
#include<dmath/Rect.h>
#include<tracer/Tracer.h>
#include<pluginmgr/PluginManager.h>
#include<dreg/Registry.h>
#include<dreg/Entry.h>
#include<navmgr/NavManager.h>
#include<dmath/dVec3.h>
#include<dmath/dMatrix.h>
#include<osg/Vec3>
#include <osgUtil/UpdateVisitor>
#include<osgDB/ReadFile>
#include<osg/Matrix>
#include<dgl/dglcore/DGL.h>

using namespace std;

#ifdef WIN32
#define DSEPARATOR ';'
#else
#define DSEPARATOR ':'
#endif

bool                              DOSG::m_done        = false;
osg::MatrixTransform*             DOSG::m_root        = NULL;
osg::MatrixTransform*             DOSG::m_ether       = NULL;
osg::MatrixTransform*             DOSG::m_world       = NULL;
osg::FrameStamp*                  DOSG::m_framestamp  = NULL;
ContextVar<osgUtil::SceneView*>   DOSG::m_sceneview;

//=============================================================================
// Function Name: init
/// @brief Creates the one and only DOSG object.
//=============================================================================
void DOSG::init()
{
    // Make sure this function is only called once
    static bool initialized = false;
    if( initialized ) return;
    initialized = true;

    setupRegistry();    
    // Initialize Everything that DOSG depends on
    Tracer::init();
    Registry::init();
    NavManager::init();
    DisplayManager::init();
    GServiceManager::init();
    
    DOSG *dosg = new DOSG();
    DisplayManager::addCullCallback( dosg );
    DisplayManager::addClearCallback( dosg );
    DisplayManager::addDrawCallback( dosg );

    string val = Registry::getValue("/DIVERSE/DOSG/ServicePriority", "9" );
    
    dosg->setPriority( atoi(val.c_str()), false );
    GServiceManager::addService( dosg );
}

//=============================================================================
// Function Name: setupRegistry
/// @brief Populates the registry with the default values if they don't exist.
//=============================================================================
void DOSG::setupRegistry()
{
    static bool setup = false;
    if( setup ) return;
    
    Registry::getValue("/DIVERSE/DOSG/ServicePriority", "9" );
    
    setup = true;
}

//=============================================================================
/// @brief Draw the scene.
/// @param cam - the camera to used for the scene.
//=============================================================================
void DOSG::draw( DCamera* cam )
{
    initSceneView();
//    Rect vp = cam->getProjectionRect();
//    (*m_sceneview)->setViewport( vp.x(), vp.y(), vp.width(), vp.height());
//    (*m_sceneview)->setProjectionMatrix( osg::Matrix(cam->getProjectionMatrix()) );
//    (*m_sceneview)->setViewMatrix( osg::Matrix(cam->getModelViewMatrix()) );
//    (*m_sceneview)->cull();
    (*m_sceneview)->draw();
}

//=============================================================================
/// @brief Cull the scene.
/// @param cam - the camera to used for the scene.
//=============================================================================
void DOSG::cull( DCamera* cam )
{
//    Tracer t("DOSG::cull()");
    initSceneView();
    Rect vp = cam->getProjectionRect();
    (*m_sceneview)->setViewport( vp.x(), vp.y(), vp.width(), vp.height());
    (*m_sceneview)->setProjectionMatrix( osg::Matrix(cam->getProjectionMatrix()) );
    (*m_sceneview)->setViewMatrix( osg::Matrix(cam->getModelViewMatrix()) );
    (*m_sceneview)->cull();
}

//=============================================================================
/// @brief Clear the scene.
/// @param cam - the camera to used for the scene.
//=============================================================================
void DOSG::clear( DCamera* cam )
{
}

//=============================================================================
// Function Name: initSceneView
/// @brief For each context we create a new scene view and pass it the root
//=============================================================================
void DOSG::initSceneView()
{
    static OpenThreads::Mutex mutex;
    static int contextcount = 0;
    static ContextVar<int> cid;
    if( !m_sceneview.isDefined() )
    {
    	mutex.lock();
        cid = contextcount++;
        cout<<"initSceneView("<<contextcount<<")\n";

        if( contextcount == 1 )
            osg::DisplaySettings::instance()->setMaxNumberOfGraphicsContexts( 10 );
        m_sceneview.set(new osgUtil::SceneView());
        if( !m_framestamp ) m_framestamp = new osg::FrameStamp();
        (*m_sceneview)->setDisplaySettings( osg::DisplaySettings::instance() );
    
        (*m_sceneview)->setDefaults();
        (*m_sceneview)->init();
        (*m_sceneview)->setFrameStamp( m_framestamp ); 
        (*m_sceneview)->setSceneData( m_root );
        (*m_sceneview)->getState()->setContextID( *cid );
    	mutex.unlock();
    }
}

//=============================================================================
// Function Name: DOSG
/// @brief Default constructor, calls all config functions.
//=============================================================================
DOSG::DOSG()
    :GService( "DOSG Service" )
{
    m_root  = new osg::MatrixTransform;
    m_ether = new osg::MatrixTransform;
    m_world = new osg::MatrixTransform;
    m_root->addChild( m_ether );
    m_root->addChild( m_world );
    
    // Put everything into DIVERSE coordinates
//    m_root->setMatrix( osg::Matrix::rotate( DEG2RAD(-90), 1, 0, 0 ) );
    
    Tracer::stat( 3, "DOSG Initialized");    
}

//=============================================================================
// Function Name: config
/// @brief Runs preConfig, config, and postConfig for all augments and calls
/// Init if DOSG hasn't been initialized
//=============================================================================
void DOSG::preConfig()
{
}

//=============================================================================
// Function Name: postConfig 
/// @brief Runs preConfig, config, and postConfig for all augments and calls
/// Init if DOSG hasn't been initialized.
//=============================================================================
void DOSG::postConfig()
{
}

//=============================================================================
// Function Name: PostProcess
/// @brief This function does all of the "heavy lifting" between frames and is
/// where all code that doesn't make changes to the scene may run.
///
/// If the display is multi-threaded postProcess() will run at the same time
/// as cull() and draw().
//=============================================================================
void DOSG::postProcess()
{
//    (*m_sceneview)->update();
}

//=============================================================================
// Function Name: PreProcess
/// @brief This function does all of the "heavy lifting" between frames and is
/// where all code that may make changes is called.
//=============================================================================
void DOSG::preProcess()
{
    static int      frame_num =0;
    static bool     init = true;
    static DTime    timer;
    if( init ) 
    {   
        timer.start();
        init = false;
    }
    if( !m_framestamp ) 
        m_framestamp = new osg::FrameStamp();
    
    m_framestamp->setFrameNumber( frame_num );
    m_framestamp->setReferenceTime( timer.elapsed().sec() );
    ++frame_num; 
    
    // Update the navigation
    m_world->setMatrix(osg::Matrix( NavManager::getMatrix().ptr() ));
    
    static osgUtil::UpdateVisitor* _updateVisitor = new osgUtil::UpdateVisitor;
    if ( m_root )
    { 
        _updateVisitor->reset();

        _updateVisitor->setFrameStamp(m_framestamp);

        _updateVisitor->setTraversalNumber(m_framestamp->getFrameNumber());
        
        m_root->accept(*_updateVisitor);
        
        // now force a recompute of the bounding volume while we are still in
        // the read/write app phase, this should prevent the need to recompute
        // the bounding volumes from within the cull traversal which may be
        // multi-threaded.
        m_root->getBound();
    }
}
