#ifndef _DOSG_H_
#define _DOSG_H_
#include<dmath/dmath.h>
#include<dmath/osdep.h>
#include<vector>
#include<displaymgr/DisplayCallback.h>
#include<servicemgr/GService.h>
#include<dutil/ContextVar.h>
#include<osg/MatrixTransform>
#include<osgUtil/SceneView>
#include<osg/FrameStamp>

using namespace std;

#ifdef WIN32
#ifdef DOSG_EXPORTS
#define DOSG_API __declspec(dllexport)
#else
#define DOSG_API __declspec(dllimport)
#endif /* DOSG_EXPORTS */
#else
#define DOSG_API
#endif /* WIN32 */

//=============================================================================
/// @class DOSG
/// @author Daniel Larimer
/// @brief The DIVERSE interface to Open Scene Graph.
//=============================================================================
class DOSG_API DOSG : public GService, public DisplayCallback
{
    public:
        static void init();
        static void setupRegistry();
        
        void preConfig();
        void postConfig();

        void cull ( DCamera* cam );
        void draw ( DCamera* cam );
        void clear( DCamera* cam );
        
        //====== RUN LOOP======//
        void preProcess();
        void postProcess();
        
		/// @brief Get the root node of the scene graph.
		/// @return a pointer to an osg::Group object.
        static osg::Group* getRoot()   { return m_root;  }
		/// @brief Get the ether node of the scene graph.
		/// @return a pointer to an osg::Group object.
        static osg::Group* getEther() { return m_ether; }
		/// @brief Get the world node of the scene graph.
		/// @return a pointer to an osg::Group object.
        static osg::Group* getWorld()  { return m_world; }

    private:
        DOSG();
        
        static void initSceneView();
        static bool m_done;
        static osg::FrameStamp*                m_framestamp;

        static osg::MatrixTransform*           m_root;
        static osg::MatrixTransform*           m_ether;
        static osg::MatrixTransform*           m_world;
        static ContextVar<osgUtil::SceneView*> m_sceneview;
};

#endif
