#include "DOsgApp.h"
#include<eventmgr/EventManager.h>
#include<navmgr/NavManager.h>
#include<tracer/Tracer.h>
#include<dreg/Registry.h>
#include<dreg/Entry.h>
#include<servicemgr/GServiceManager.h>
#include<string>
#include<pluginmgr/PluginManager.h>
#include<dgl/dglcore/DGL.h>
#include<dosg/DOSG.h>
using namespace std;

//=============================================================================
// Function Name: Constructor
/// @brief Constructor.
///
/// This function does all of the initalization that is necessary for the DOsgApp.
/// @param argc - the number of command line arguments.
/// @param argv - the command line arguments.
/// @param cfgfile - the configuration file to be used for registry settings.
/// @param init - Always passes false. Not implemented.
//=============================================================================
DOsgApp::DOsgApp(int argc, char** argv,  const string& cfgfile, bool init )
    :DiverseApp(argc, argv, cfgfile, false)
{
    m_setup = false;
    setupRegistry( cfgfile );
    if( init ) initModules();
}

//=============================================================================
/// @brief Initializes the various DIVERSE managers.
//=============================================================================
void DOsgApp::initModules()
{
    DiverseApp::initModules();
    DGL::init();
    DOSG::init();
    
    //Set the class up to get all mouse, keyboard, wand, and tracker events
    EventManager::addMouseListener(this);
    EventManager::addKeyListener(this);
    EventManager::addWandListener(this);
}

//=============================================================================
/// @brief Sets up the registry using the specified configuration file.
/// @param cfgfile - the configuration file to be used for registry settings.
//=============================================================================
void DOsgApp::setupRegistry( const string& cfgfile )
{
    if( m_setup ) return;
    Registry::init(cfgfile);

    m_setup = true;
}

//=============================================================================
// Function Name: Deconstructor
/// @brief Destructor.
//=============================================================================
DOsgApp::~DOsgApp()
{
}

//=============================================================================
/// @brief Get the world node of the scene graph.
/// @return a pointer to an osg::Group object.
//=============================================================================
osg::Group* DOsgApp::getWorld()const
{
    return DOSG::getWorld();
}

//=============================================================================
/// @brief Get the ether node of the scene graph.
/// @return a pointer to an osg::Group object.
//=============================================================================
osg::Group* DOsgApp::getEther()const
{
    return DOSG::getEther();
}

//=============================================================================
/// @brief Get the root node of the scene graph.
/// @return a pointer to an osg::Group object.
//=============================================================================
osg::Group* DOsgApp::getRoot()const
{
    return DOSG::getRoot();
}
