//=============================================================================
// Entry.h
// - Defines an entry in the registry
//
// Created 5/29/03 By Daniel Larimer
//=============================================================================
#ifndef _ENTRY_H_
#define _ENTRY_H_
#include<dreg/RegLexer.h>
#include<fstream>
#include<map>
#include<string>
using namespace std;

//=============================================================================
/// @class Entry
/// @brief An entry in the registry containing the name of the entry, a value, and
/// a comment and possible sub-entries.
///
/// Used in conjunction with the Registry class to create XML based registry files
/// for internal or external use.
//=============================================================================
class DREG_API Entry
{
    public:
        Entry( const string& key, const string& val = "__UNDEFINED__", 
                                  const string& cmt = "__UNDEFINED__" );
        string getValue( const string& path, 
                         const string& defval = "__UNDEFINED__",
                         const string& cmt = "__UNDEFINED__" );
        void   setValue( const string& path, 
                         const string& defval = "__UNDEFINED__" );
        string getComment( const string& path, 
                         const string& defval = "__UNDEFINED__" );
        void   setComment( const string& path, 
                         const string& defval = "__UNDEFINED__" );
        Entry* getEntry( const string& path );
        void   remove( const string& path );
        
       void save( ofstream& out ); 
       bool read( RegLexer& lex );

	   /// @brief Return the root key for the Entry.
	   /// @return the root key for the Entry.
       string key()const { return m_key; }
	   /// @brief Return the value for the root key of the Entry.
	   /// @return the value for the root key of the Entry.
       string val()const { return m_val; }
	   /// @brief Return a map to the subentries.
	   /// @return a map to the subentries.
       map<string, Entry*> getEntries()const { return m_subentries; }

    private:
        void m_Save( ofstream& out, int depth );
        map<string, Entry*> m_subentries;
        string              m_key;
        string              m_val;
        string              m_comment;
    
};

#endif
