//=============================================================================
// Registry
// - This class provides a way to set, get, and save global settings.  The
// settings are read from one or more config files when the program is loaded
// and then the settings are saved when the program exits.
//=============================================================================
#ifndef _REGISTERY_H_
#define _REGISTERY_H_
#include<string>
#include<dreg/Entry.h>
using namespace std;

//=============================================================================
/// @class Registry
/// @brief A static class which handles the registry.
///
/// The Registry provides a means to set, get, save, and load global settings. The
/// settings are read from one or more configuration files when the application
/// is loaded. The configuration may be a physical file on the hard drive or
/// created and maintained internally by DIVERSE using default registry settings.
//=============================================================================
class DREG_API Registry
{
    public:
        Registry( const string& filename = "registry.cfg" );
        static void init( const string& filename = "registry.cfg" );

        static string getValue( const string& path, 
                                const string& defaultval = "__UNDEFINED__",
                                const string& cmt        = "__UNDEFINED__" );
        static void   setValue( const string& path, const string& value );
        static void   setComment( const string& path, const string& cmt );
        static string getComment( const string& path );
        static void   remove  ( const string& path );
        static Entry* getEntry( const string& path );

        static void save();
        static void saveAs( const string& fname );
    private:
        static Entry* m_root;
        
//        static string m_globalfile;
//        static string m_userfile;
        static string m_appfile;
        static bool   m_initialized;
};
#endif
