#include"ColonList.h"
#include<tracer/Tracer.h>

//===================================================
// For Windows the colon cannot be used as the colon
// is used in the path name just after the drive
// letter ( i.e. C:/DIVERSE/plugins )
//===================================================
#ifdef WIN32
#define DSEPARATOR ';'
#else
#define DSEPARATOR ':'
#endif


//=============================================================================
/// @brief Constructor.
/// @param liststr - the string to be initialized as a colon list.
//=============================================================================
ColonList::ColonList( const string& liststr )
{
    string nextitem;
    for( unsigned int i = 0; i < liststr.size(); i++ )
    {
        if( liststr[i] == DSEPARATOR )
        {
           if( nextitem.size() > 0 )
           {
               m_list.push_back( nextitem );
               nextitem = string();
           }
        }
        else
        {
            nextitem += liststr[i];
        }
    }
    if( nextitem.size() )
        m_list.push_back(nextitem);
}

//=============================================================================
/// @brief Add an item to a colon list.
/// @param val - a string to be added to the colon list.
/// @param unique - set to true if the item should be unique within the list,
/// else set to false.
//=============================================================================
void ColonList::addItem( const string& val, bool unique )
{
    if( unique && findItem(val) == -1 )
        m_list.push_back( val );
    else if( !unique )
        m_list.push_back( val );
}

//=============================================================================
/// @brief Remove an item from the colon list.
/// @param index - the index of the item in the list.
//=============================================================================
void ColonList::removeItem( int index )
{
    vector<string>::iterator itr = m_list.begin() + index;
    if( itr != m_list.end() )
        m_list.erase( itr );
}

//=============================================================================
/// @brief Remove an item from the colon list.
/// @param str - the string to be removed from the colon list.
//=============================================================================
void ColonList::removeItem( const string& str )
{
    vector<string>::iterator itr = m_list.begin();
    while( itr != m_list.end() )
    {
        if( *itr == str )
        {
            m_list.erase( itr );
            removeItem( str );
            return;
        }
        itr++;
    }
}

//=============================================================================
/// @brief Retrieve an item from the list.
/// @param index - the index of the item to retrieve from the list.
//=============================================================================
string ColonList::getItem( int index )const
{
    return m_list[index];
}

//=============================================================================
/// @brief Returns the colon list as a string.
/// @return a colon seperated list.
//=============================================================================
string ColonList::getString()const
{
    string tmp;
    for( unsigned int i = 0; i < m_list.size(); i++ )
    {
        tmp += m_list[i];
        if( i < m_list.size() - 1 )
            tmp+=DSEPARATOR;
    }
    return tmp;
}

//=============================================================================
/// @brief Find a an item within the colon list.
/// @param str - the string to find in the list.
/// @return the index of the first item that matches str, -1 if not found.
//=============================================================================
int ColonList::findItem( const string& str )const
{
    for( unsigned int i = 0; i < m_list.size() ; i++ )
        if( m_list[i] == str ) return i;
    return -1;
}

//=============================================================================
/// @brief Returns a vector containing the list items.
/// @return a vector containing the list items.
//=============================================================================
vector<string> ColonList::getList()const
{
    return m_list;
}
//=============================================================================
/// @brief Returns a vector containing the list items from the input string.
/// @param str - a string to convert to a colon list.
/// @return a vector containing the list items from the input string.
//=============================================================================
vector<string> ColonList::toList(const string& str)
{
    ColonList l(str);
    return l.getList();
}
