#ifndef _COLONLIST_H_
#define _COLONLIST_H_
#include<string>
#include<vector>
using namespace std;

#ifdef WIN32
#ifdef DUTIL_EXPORTS
#define DUTIL_API __declspec(dllexport)
#else
#define DUTIL_API __declspec(dllimport)
#endif /* DUTIL_EXPORTS */
#else
#define DUTIL_API
#endif /* WIN32 */

//=============================================================================
/// @brief Provides a means of using a colon separated string and extracting
/// the items from the list.
///
/// The ColonList class can be used to extract information from a colon list or
/// it can be used to build a colon list.
/// 
/// IMPORTANT: (Windows specific) - In Windows, the colon separated list uses
/// semi-colons instead of colons. This is a result of the ColonList being used
/// for paths which contain colons (i.e. C:/DIVERSE/plugins/). For the
/// DIVERSE_PLUGINS environment variable it is necessary to use semi-colons to
/// separate the list. If using Cygwin or another shell it is also necessary to
/// use quotes around the list.
//=============================================================================
class DUTIL_API ColonList
{
    public:
        ColonList( const string& liststr = "" );
        void           addItem( const string&, bool unique = false );
        void           removeItem( int index );
        void           removeItem( const string& );
        string         getString()const;
        string         getItem( int index )const;
        int            findItem( const string& )const;
        int            size()const{ return m_list.size(); }
        vector<string> getList()const;
		/// @brief Returns the character indexed by x.
		/// @param x - the index of the character to be returned.
		/// @return a string contaning the character indexed by x.
        string         operator[](int x)const { return getItem(x); }

        static vector<string> toList( const string& list );

    private:
        vector<string>  m_list;
};

#endif
