#include"CommandArgs.h"
#include<string>
#include<iostream>
using namespace std;

int                          CommandArgs::m_argc = 0;
char**                       CommandArgs::m_argv = 0;
map<string, vector<string> > CommandArgs::m_argmap;
map<string, Usage>           CommandArgs::m_usage;


//=============================================================================
/// @brief Initialize the static CommandArgs object.
/// @param argc - the number of arguments passed to the application.
/// @param argv - a pointer to the command line arguments character string
/// passed to the application.
//=============================================================================
void CommandArgs::init( int& argc, char** argv )
{
    m_argc = argc;
    m_argv = argv;

    for( int i = 1; i < argc; i++ )
    {
        string arg;
        string val;
        int place = 0;
        while( argv[i][place] != '\0' && argv[i][place] != '=' )
        {
            arg += argv[i][place];
            place++;
        }
        if( argv[i][place] == '=' ) place++;
        while( argv[i][place] != '\0' )
        {
            val += argv[i][place];
            place++;
        }
        m_argmap[string(arg)].push_back(string(val)); 
    }
}

//=============================================================================
/// @brief Get the list of all values for an argument.
/// @return the list of all values for an argument.
//=============================================================================
vector<string> CommandArgs::getValues( const string& arg )
{
    map<string, vector<string> >::iterator itr = m_argmap.find(arg);
    if( itr == m_argmap.end() ) return vector<string>();
    return itr->second;
}

//=============================================================================
/// @brief Get the last value for an argument.
/// @return the last value for an argument.
//=============================================================================
string CommandArgs::getValue( const string& arg )
{
    map<string, vector<string> >::iterator itr = m_argmap.find(arg);
    if( itr == m_argmap.end() ) return "";
    return itr->second.back();
}

//=============================================================================
/// @brief Finds an argument in the list.
/// @return true if argument was found with or without a value.
//=============================================================================
bool CommandArgs::findArg( const string& arg )
{
    map<string, vector<string> >::iterator itr = m_argmap.find(arg);
    if( itr == m_argmap.end() ) return false;
    return true;
}

//=============================================================================
/// @brief Add usage information for the application.
/// @param arg - the name of the argument.
/// @param values - the accepted values for the argument.
/// @param description - a description of the argument.
/// @param require - true if the argument is required, else false.
//=============================================================================
void CommandArgs::addUsage( const string& arg, const string& values,
                                   const string& description, bool require )
{
    Usage u;
    u.argname = arg;
    u.require = require;
    u.description = description;
    u.values = values;
    m_usage[arg] = u;
}

//=============================================================================
/// @brief Prints the usage for the application.
//=============================================================================
void CommandArgs::printUsage()
{
    if( m_argc )
        cout<<m_argv[0]<<" ";
    map<string, Usage>::iterator itr = m_usage.begin();
    while( itr != m_usage.end() )
    {
        if( !itr->second.require )
            cout<<"[";
        cout<<itr->first;//second.argname;
        if( itr->second.values != "" )
            cout<<"=["<<itr->second.values<<"]";
        if( !itr->second.require )
            cout<<"]";
        cout<<" ";
        itr++;
    }
    cout<<"\n\n";
    itr = m_usage.begin();
    while( itr != m_usage.end() )
    {
        cout<<"\t"<<itr->second.argname<<"\t"<<itr->second.description<<"\n\n";
        itr++;
    }
}
