#ifndef _COMMANDARGS_H_
#define _COMMANDARGS_H_
#include<map>
#include<vector>
#include<string>
using namespace std;

#ifdef WIN32
#ifdef DUTIL_EXPORTS
#define DUTIL_API __declspec(dllexport)
#else
#define DUTIL_API __declspec(dllimport)
#endif /* DUTIL_EXPORTS */
#else
#define DUTIL_API
#endif /* WIN32 */

//=============================================================================
/// @class Usage
/// @brief Used by CommandArgs to store the parameters passed to applications.
//=============================================================================
class Usage
{
    public:
        Usage& operator=( const Usage& c )
        {
            argname = c.argname;
            description = c.description;
            values = c.values;
            require = c.require;
            return *this;
        }
        string argname;
        string description;
        string values;
        bool   require;
};

//=============================================================================
/// @class CommandArgs
/// @brief Can be used to store and retrieve items of the argument list passed
/// to a DIVERSE application.
//=============================================================================
class DUTIL_API CommandArgs
{
    public:
        static void init( int& argc, char** argv );

		/// @brief Returns the number of arguments passed to the application.
		/// @return the number of arguments passed to the application.
        static int    getArgc()    { return m_argc; }
		/// @brief Returns a pointer to the command line arguments character string
		/// passed to the application.
		/// @return a pointer to the command line arguments character string
		/// passed to the application.
        static char** getArgv() { return m_argv; }

        static vector<string> getValues( const string& arg );
        
        static string         getValue ( const string& arg );
        
        static bool           findArg  ( const string& arg );

        static void   addUsage( const string& arg, 
                                const string& values, 
                                const string& description, 
                                bool require = false );
                                
        static void   printUsage();
 
    private:
        static int                          m_argc;
        static char**                       m_argv;
        /// maps an argument to one or more values
        static map<string, vector<string> > m_argmap; 
        static map<string, Usage>           m_usage;
};

#endif
