#include"DTime.h"
#ifndef WIN32
#include<sys/time.h>
#endif /* !WIN32 */
#ifdef LINUX
#include <cstdio>
#endif
#ifdef __sgi
	#include <stdio.h>
#else
	#include <cstdio>
    using namespace std;
#endif

//=============================================================================
/// @brief Constructor.
/// Sets the m_usec value to usec.
/// @param usec - the time in microseconds
//=============================================================================
DTime::DTime( int usec )
{
    m_start = 0;
    m_usec = usec;
}

//=============================================================================
/// @brief Constructor.
/// Sets the m_usec value based on the seconds, and microseconds passed to the
/// constructor.
/// @param sec - the number of seconds
/// @param usec - the number of microseconds
//=============================================================================
DTime::DTime( int sec, int usec )
{
    m_start = 0;
    m_usec = 1000000 * sec + usec;
}

//=============================================================================
/// @brief Constructor.
/// Sets the m_usec value based on the minutes, seconds, and microseconds passed
/// to the constructor.
/// @param min - the number of minutes
/// @param sec - the number of seconds
/// @param usec - the number of microseconds
//=============================================================================
DTime::DTime( int min, int sec, int usec )
{
    m_start = 0;
    m_usec = 60000*min + 1000 * sec + usec;
}

//=============================================================================
/// @brief Constructor.
/// Sets the m_usec value based on the hours, minutes, seconds, and microseconds
/// passed to the constructor.
/// @param hr - the number of hours
/// @param min - the number of minutes
/// @param sec - the number of seconds
/// @param usec - the number of microseconds
//=============================================================================
DTime::DTime( int hr, int min, int sec, int usec )
{
    m_start = 0;
    m_usec = 3600000*hr + 60000*min + 1000 * sec + usec;
}

//=============================================================================
/// @brief Sets the DTime object value to the current time since January 1, 1970.
//=============================================================================
DTime DTime::current()
{
    timeval tv;
    gettimeofday( &tv, NULL );
    return DTime( tv.tv_sec, tv.tv_usec );
}

//=============================================================================
/// @brief Starts the timer, setting the time to the current time.
//=============================================================================
void DTime::start()
{
    m_start = current().m_usec;
}

//=============================================================================
/// @brief Resets the timer to the current time and returns the time since the
/// last reset or start.
/// @return time since last reset or start.
//=============================================================================
DTime DTime::reset()
{
    DTime tmp = DTime(m_start);
    m_start = current().usec();
    return DTime(m_start - tmp.usec());
}

//=============================================================================
/// @brief Gets the time since the last reset or start.
/// \return time since last reset or start.
//=============================================================================
DTime DTime::elapsed()const
{
    return current() - m_start;
}
