#include"byteswap.h"

/// @fn bool std2host( void* d, int size )
/// @brief Converts standard byte order to the host format.
/// @param d - pointer to buffer.
/// @param size - size of buffer.
/// @return true - if swap was needed.
bool std2host( void* d, int size )
{
    #ifdef WIN32
        byteswap( d, size );
        return true;
    #endif
    #ifdef _LINUX_
        byteswap( d, size );
        return true;
    #endif
    return false;
}

/// @fn bool host2std( void* d, int size )
/// @brief Converts the host byte order to the standard order.
/// @param d - pointer to buffer.
/// @param size - size of buffer.
/// @return true if swap was needed.
bool host2std( void* d, int size )
{
    #ifdef WIN32
        byteswap( d, size );
        return true;
    #endif
    #ifdef _LINUX_
        byteswap( d, size );
        return true;
    #endif
    // Mac OS X & IRIX are in the standard format
    return false;
}

/// @fn void swap( char& a, char& b)
/// @brief Swap individual bytes.
/// @param a - byte to swap.
/// @param b - byte to swap.
/// @brief Swaps two bytes.
void swap( char& a, char& b)
{
    if( &a == &b ) return;
    char tmp = a;
    a = b;
    b = tmp;
}

/// @fn void byteswap( void* d, int size )
/// @param d - pointer to buffer.
/// @param size - size of buffer.
/// @brief Swaps the order of bytes.
void byteswap( void* d, int size )
{
    char* data = (char*)d;
    int tmp = 0;
    int hsize = size / 2;
    while( tmp < hsize )
    {
        swap( data[tmp], data[size-tmp-1] );
        tmp++;
    }
}
