//=============================================================================
// Token.h
// - A Token is the basic unit generated by the Lexer and is used by the parser
//
// Created 9/19/02 By Daniel Larimer
//=============================================================================
#ifndef _TOKEN_H_
#define _TOKEN_H_

#include<string>
#include<ctype.h>

using namespace std;

//=============================================================================
/// @struct Token
/// @brief A token used during parsing.
//=============================================================================
struct Token
{
	/// @brief Constructor.
	/// @param tp - the type of token.
	/// @param lex - the token value.
	/// @param ln - the line number.
	/// @param c - the column number within the line.
    Token( int tp, string lex = "", int ln = 0, int c = 0 )
    {
        type = tp;
        line = ln;
        col = c;
        source = lex;
        lexem = lex;
    }

	/// @brief Constructor.
    Token(){}
    
	/// @brief Get the length of the token.
	/// @return the length of the token.
    int    length() { return lexem.length(); }
	/// @brief Append the specified string to the token.
	/// @param txt - the string to append to the token.
    void   append( const string& txt ) { lexem += txt; }
	/// @brief equal relational operator.
    bool   operator==( const Token& tok )
    {
        return (tok.type) == type;// || (lexem == tok.lexem); 
    }
	/// @brief not equal relational operator.
    bool   operator!=( const Token& tok )
    {
        return (tok.type != type);// && lexem != tok.lexem; 
    }

    int    type;
    int    line;
    int    col;
    string lexem;
    string source;
};

#endif
