#ifndef _XMLNODE_H_
#define _XMLNODE_H_
#include<map>
#include<list>
#include<string>
#include<iostream>

#ifdef WIN32
#ifdef DXML_EXPORTS
#define DXML_API __declspec(dllexport)
#else
#define DXML_API __declspec(dllimport)
#endif /* DXML_EXPORTS */
#else
#define DXML_API
#endif /* WIN32 */

using namespace std;

//=============================================================================
/// @class XmlNode
/// @author Daniel Larimer
/// @brief A node in the XML tree.
///
/// A node has only one name, but can have zero or more key/value pair parameters.
//=============================================================================
class DXML_API XmlNode
{
    public:
        XmlNode( const string& tagname = "" );
        ~XmlNode();

		/// @brief Get the parent node of the XML tree.
        XmlNode* getParent()const { return m_parent; }

        bool   isTextNode()const;
        string getText()const;
        void   setText( const string& text );

        string getTag()const;
        void   setTag( const string& name );
        
        bool   hasParameter( const string& param )const;
        string getParameter( const string& param )const;
        void   setParameter( const string& param, const string& val );
		/// @brief Get a map of the parameters for the node.
        /// @return a map from name to value 
        map<string, string> getParams()const { return m_params; }

		/// @brief Get a list of children nodes.
        /// @return the list of children.
        list<XmlNode*> getChildren()const { return m_children; }
        
        void   appendNode( XmlNode* node );
        void   prependNode( XmlNode* node );

        void   write( ostream& out, int indent );
        
    private:
        void   printIndent( ostream& out, int indent );
        void   setParent( XmlNode* parent ) { m_parent = parent; }

        XmlNode*            m_parent;
        map<string, string> m_params;
        string              m_text;
        list<XmlNode*>      m_children;
        string              m_tagname; 
};

#endif
