#ifndef _XMLPARSER_H_
#define _XMLPARSER_H_
#include<string>

#ifdef WIN32
#ifdef DXML_EXPORTS
#define DXML_API __declspec(dllexport)
#else
#define DXML_API __declspec(dllimport)
#endif /* DXML_EXPORTS */
#else
#define DXML_API
#endif /* WIN32 */

using namespace std;

class XmlLexer;
class XmlNode;

//=============================================================================
/// @class XmlParser
/// @author Daniel Larimer
/// @brief Builds an Xml tree from a file.
///
/// This class parses an XML type file, separates each node and links each node
/// to their parent and children nodes.
//=============================================================================
class DXML_API XmlParser
{
    public:
       XmlParser( const string& fname = "" );
       ~XmlParser();
       XmlNode* parse( const string& fname );
	   /// @brief Get the root node which is the top level node in the tree.
	   /// @return the root node.
       XmlNode* getRoot()const { return m_root; }

    private:
       XmlNode*  parsehelper();

       string    m_filename;
       XmlNode*  m_root;
       XmlLexer* m_lex;
};

#endif
