#include"fsm.h"
#include<iostream>
using namespace std;


//=============================================================================
/// @brief Constructor.
//=============================================================================
FSM::FSM()
{
    addState( -1 ); // Error State
    gotoState( -1 );
}


//=============================================================================
/// @brief Jumps to the specified state, returns -1 if state doesn't exist
/// @param state - the state to goto
/// @return -1 on error, 0 on success
//=============================================================================
int FSM::gotoState( int state )
{
//    cout<<"Enter State: "<<state<<endl;
    m_state = m_table.find( state );
    if( m_state == m_table.end() )
    {
        gotoState( -1 );
        return -1;
    }
    return 0;
}


//=============================================================================
/// @brief Get the next state for the FSM.
///
/// Using the given character goes through the possible Transition in 
/// the current state and when a Transition is found it sets the next state
/// and action accordingly and returns 0 for success and -1 if an error
/// occured.
/// @param next_ch - the next input character.
/// @param nextstate - a pointer to an int inj which to store the next state.
/// @param action - the action to take.
/// @return -1 on error or 0 for success.
//=============================================================================
int FSM::getNextState( char next_ch, int* nextstate, int* action )
{
//    cout<<"FSM::getNextState( "<<next_ch<<" )";
//    cout<<"\tcurrent state: "<<m_state->first<<endl;
    for( unsigned int i = 0; i < m_state->second.size(); i++ )
    {
        if( m_state->second[i].m_key == next_ch )
        {
            *action = m_state->second[i].m_action;
            gotoState( m_state->second[i].m_next_state );
            *nextstate = getCurrentState();
            return 0;
        }
    }
    cout<<"\'"<<next_ch<<"\' doesn't have transition from state "<<m_state->first<<endl;
    gotoState( -1 ); 
    return -1;
}


//=============================================================================
/// @brief Adds a new state to the table with no Transitions.
/// @param state - the state to add.
/// @return -1 if error, 0 for success.
//=============================================================================
int FSM::addState( int state )
{
    if( m_table.find( state ) == m_table.end() )
    {
        m_table[state] = vector<Transition>();
        return 0;
    }else{
        cout<<"Error state: "<<state<<" already exists\n";
        return -1;
    }
}

//=============================================================================
/// @brief Add a transition.
/// @param state - the state for which to add a Transition.
/// @param trans - the Transition to add.
//=============================================================================
void FSM::addTransition( int state, Transition trans )
{
    map<int, vector<Transition> >::iterator itr = m_table.find( state );
    if( itr == m_table.end() )
    {
        cout<<"State: "<<state<<" doesn't exist\n";
        return;
    }
    itr->second.push_back( trans );
}

