#include"DKeyboard.h"
#include"devents.h"

//========================================================================
/// @brief Handles the event by mapping the KeyState with the event's 
/// Key and ascii codes.
/// @param evt - the DKeyEvent to be processed.
//========================================================================
void DKeyboard::handleEvent( const DKeyEvent* evt )
{
   m_keys [evt->key()]   = KeyState(evt->key(), 
                                    evt->ascii(), 
                                    evt->action() ); 
   m_asciimap[evt->ascii()] = KeyState(evt->key(), 
                                    evt->ascii(), 
                                    evt->action() ); 
}   

//========================================================================
/// @brief Determines if the key is down using the Key code.
/// @param keycode - a value representing the Key code as defined in DKeyEvent.
/// @return true if the key is down, false if it is up.
//========================================================================
bool DKeyboard::isDown( int keycode )const
{
    map<int,KeyState>::const_iterator itr = m_keys.find(keycode);
    if( itr == m_keys.end() )return false;
    return itr->second.state;
}

//========================================================================
/// @brief Determines if the key is down using the ascii character code.
/// @param ascii - a value representing the ascii character code.
/// @return true if the key is down, false if it is up.
//========================================================================
bool DKeyboard::isDown( char ascii )const
{
    map<char,KeyState>::const_iterator itr = m_asciimap.find(ascii);
    if( itr == m_asciimap.end() )return false;
    return itr->second.state;
}
