#ifndef _KEYBOARD_H_
#define _KEYBOARD_H_
#include<map>
#include<vector>

#ifdef WIN32
#ifdef EVENTMGR_EXPORTS
#define EVENTMGR_API __declspec(dllexport)
#else
#define EVENTMGR_API __declspec(dllimport)
#endif /* EVENTMGR_EXPORTS */
#else
#define EVENTMGR_API
#endif /* WIN32 */

using namespace std;
class DKeyEvent;

//===============================================================================
/// @struct KeyState
/// @brief A struct which holds the state of a keyboard key.
//===============================================================================
struct EVENTMGR_API KeyState
{
	//===========================================================================
	/// @brief Constructor.
	/// @param c - the Key code as defined in DKeyEvent.
	/// @param a - the ascii character code.
	/// @param s - the state of the keyboard key.
	//===========================================================================
    KeyState( int c = 0, char a = 0, bool s = false )
    {
        code  = c;
        ascii = a;
        state = s;
    }
    int  code;
    char ascii;
    bool state;
};

//===============================================================================
/// @class DKeyboard
/// @brief Holds the state of the keyboard.
///
/// As keyboard events occur, they are mapped into a DKeyboard object by the
/// EventManager. This object tracks the state of the keyboard and all jeys that
/// have been pressed since the start of the application. So, over time, this
/// object will grow as keyboard keys are pressed.
//===============================================================================
class EVENTMGR_API DKeyboard
{
    public:
        void handleEvent( const DKeyEvent* evt );
		/// @brief Returns a map of the Key codes to KeyState.
		/// @return a map of the Key codes to KeyState.
        map<int,KeyState>  getKeys()const { return m_keys; }
		/// @brief Returns a map of ascii character codes to KeyState.
		/// @return a map of ascii character codes to KeyState.
        map<char,KeyState> getAsciiKeys()const { return m_asciimap; }

        bool    isDown(int keycode)const;
        bool    isDown(char ascii )const;

		/// @brief Determines if the key is up using the Key code.
		/// @param keycode - a value representing the Key code as defined in DKeyEvent.
		/// @return True if the key is up, false if it is down.
        bool    isUp(int keycode)const  { return !isDown(keycode); }
		/// @brief Determines if the key is up using the ascii code.
		/// @param ascii - a value representing the ascii character code.
		/// @return True if the key is up, false if it is down.
        bool    isUp(char ascii )const  { return !isDown(ascii);   }
    
    private:
        map<int,KeyState>    m_keys;
        map<char,KeyState>   m_asciimap;
};

#endif
