#include"EventFactory.h"
#include"devents.h"
#include<iostream>
#include<tracer/Tracer.h>
using namespace std;

map<string, DEvent*>*  EventFactory::m_events = NULL;

//=============================================================================
/// @brief Initializes the EventFactory.
//=============================================================================
void EventFactory::init()
{
    EventFactory factory; // make sure it is loaded first
    if( factory.m_events == NULL ){
        factory.m_events = new map<string, DEvent*>;
    }
}

//=============================================================================
/// @brief Used to add a new event type to the factory. Msg is a pointer to a 
/// event that the factory will keep around and use to create new events
/// as needed.
/// @param evt - the event to use for a factory
/// @return true if no conflicts, false if type is registered
//=============================================================================
bool EventFactory::Register( DEvent* evt )
{
    init();
    #ifdef DEBUG
    cerr <<"EventFactory::Register( "<<evt->type()<<" )\n";
    #endif
    if( m_events->find( evt->type() ) != m_events->end() )
    {   
        cerr<<"Event Type Conflict: "<<evt->type()
            <<" is already registered with the factory.\n";
        return false;
    }
    (*m_events)[ evt->type() ] = evt;
    return true;
}

//=============================================================================
/// @brief Removes a event type from the factory. Useful if you are loading
/// and unloading DSOs
/// @param type - the event type
/// @return true if event could be removed, false if unknown type
//=============================================================================
bool EventFactory::UnRegister( const string& type )
{
    Tracer::debug( 6, "EventFactory::UnRegister( %s )", type.c_str() );
    map<string, DEvent*>::iterator itr = m_events->find(type);
    if( itr == m_events->end() )
    {
        cerr<<"Couldn't remove "<<type<<" event from factory.\n";
        return false;
    }
    delete itr->second;
    m_events->erase( itr );
    return true;
}

//=============================================================================
/// @brief Creates a new event based on a packed buffer
/// @param buffer - the packed buffer
/// @param size - a pointer to store the number of bytes unpacked
/// @return the event
//=============================================================================
DEvent* EventFactory::buildFromBuffer( char* buffer, int* size )
{
    //Tracer t("EventFactory::buildFromBuffer");

    DEvent evt;
    evt.unpack(buffer);
    DEvent* newevt = build( evt.type() );
    if( !newevt ) return NULL;
    *size = newevt->unpack(buffer);
    return newevt;
}

//=============================================================================
/// @brief Builds a new event based on type
/// @param type - the EventType
/// @return a copy of the event
//=============================================================================
DEvent* EventFactory::build( const string& type )
{
    map<string, DEvent*>::iterator itr = m_events->find(type);
    if( itr == m_events->end() )
    {
        Tracer::error( "Couldn't create %s event with factory.", type.c_str() );
        return NULL;
    }
    DEvent* tmp = itr->second->clone();
    return tmp; 
}
