#ifndef _DEVENTFACTORY_H_
#define _DEVENTFACTORY_H_
#include<string>
#include<map>

#ifdef WIN32
#ifdef EVENTMGR_EXPORTS
#define EVENTMGR_API __declspec(dllexport)
#else
#define EVENTMGR_API __declspec(dllimport)
#endif /* EVENTMGR_EXPORTS */
#else
#define EVENTMGR_API
#endif /* WIN32 */

using namespace std;

class DEvent;

//=============================================================================
/// @class EventFactory
/// @brief The EventFactory is used by revent (the Remote Event manager ) to 
/// unpack and build remote events.
//=============================================================================
class EVENTMGR_API EventFactory
{
    public:
        static void init();
        static bool Register( DEvent* msg );
        static bool UnRegister( const string& type );

        static DEvent* buildFromBuffer( char* buffer, int* size );
        static DEvent* build( const string& type );
		/// @brief Get an event map which shows the possible events registered
		/// with this application.
		/// @return map of event type to an event object.
        static map<string, DEvent*> getEventMap() { return *m_events; };

    private:
        static map<string, DEvent*>  *m_events;
        static bool                      m_init;
};

//=============================================================================
/// @class RegisterEvent
/// @brief Used to register events with the EventFactory static class.
/// @param msgtype - the event class to register with the EventFactory.
//=============================================================================
template<class msgtype>
class RegisterEvent
{
    public:
    RegisterEvent()
    {   
        msg = new msgtype();
        if( !EventFactory::Register( msg ) )
        delete msg;
        msg = NULL;
    }
    ~RegisterEvent()
    {
//        if( msg );
//        EventFactory::UnRegister( msg->MessageType() );
    }
    msgtype* msg;
};

#endif
