#ifndef _FONTMANAGER_H_
#define _FONTMANAGER_H_
#ifdef WIN32
#include <windows.h>
#else
#include <X11/Xlib.h>
#endif
#include <map>
#include <dutil/ContextVar.h>

#ifdef WIN32
#ifdef FONTMGR_EXPORTS
#define FONTMANAGER_API __declspec(dllexport)
#else
#define FONTMANAGER_API __declspec(dllimport)
#endif /* FONTMGR_EXPORTS */
#else
#define FONTMANAGER_API
#endif

class dglfont;

//=============================================================================
/// @class FontManager 
/// @author Patrick Shinpaugh
/// @brief FontManager manages the OpenGL display lists generated for requested bitmap fonts
//=============================================================================
class FONTMANAGER_API FontManager
{
    public:
        ~FontManager();

#ifdef WIN32
        static void init( HWND hWnd = NULL );
        static dglfont* requestFont( int stockfont, unsigned char start = 0, unsigned char count = 255 );
#else
        static void init( Display* dpy = NULL );
        static dglfont* requestFont( char* fontname, unsigned char start = 0, unsigned char count = 255 );
#endif

        static bool deleteFont( dglfont* font);
        
#ifdef WIN32
        static HWND* getDisplay();
#else
        static Display* getDisplay();
#endif
    private:
        FontManager();
        
        static FontManager* m_fontmgr;
#ifdef WIN32
        ContextVar<HWND> m_hwnd;
        static map<int, ContextVar<dglfont*> > m_fontlist;
#else
        Display* m_display;
        static map<string, ContextVar<dglfont*> > m_fontlist;
#endif
};

#endif
