#ifndef _DIVERSEAPP_H_
#define _DIVERSEAPP_H_
#include<eventmgr/EventListener.h>
#include<string>
using namespace std;

#ifdef WIN32
#ifdef LIBDIVERSE_EXPORTS
#define LIBDIVERSE_API __declspec(dllexport)
#else
#define LIBDIVERSE_API __declspec(dllimport)
#endif /* LIBDIVERSE_EXPORTS */
#else
#define LIBDIVERSE_API
#endif /* WIN32 */

//=============================================================================
/// @class DiverseApp
/// @brief Provides a class that sets up the default DIVERSE environment.
///
/// This environment includes the following plugins, services, and navigations.
/// <br>
/// Modules                          <br>
///     Tracer                       <br>
///     Registry - diverse.xml       <br>
///     EventManager                 <br>
///     NavManager                   <br>
///     ServiceManager               <br>
///     PluginManager                <br>
/// Services:                        <br>
///     EventService                 <br>
///     QtService                    <br>
///     NavService                   <br>
///     DtkWandService               <br>
///     DtkTrackerService            <br>
/// Navigations:                     <br>
///     MouseFlyNav                  <br>
///     WandJoystickNav              <br>
///     MouseLookNav                 <br>
///     KeyboardNav                  <br>
//=============================================================================
class LIBDIVERSE_API DiverseApp : public EventListener
{
    public:
        DiverseApp( int argc=0, char** argv=NULL, const string& cfgfile = "diverse.xml", bool init = true );
        virtual ~DiverseApp();

        virtual bool onMouseEvent(DMouseEvent* evt);
        virtual bool onKeyEvent(DKeyEvent* evt);
        virtual bool onTrackerEvent(DTrackerEvent* evt);
        virtual bool onWandEvent(DWandEvent* evt);

        void configApp();
        void run();
        
        void preConfig();
        void config();
        void postConfig();
        void start();
        void preProcess();
        void process();
        void frame();
        void postProcess();
        
        bool isRunning();
        void quit();
        
    protected:
        void setupRegistry( const string& cfgfile = "diverse.xml" );
        void initModules();

    private:
        bool m_setup;
};

#endif
