#include"MessageManager.h"
#include<pexpr/PostOffice.h>
#include<pexpr/POBox.h>
#include<stdlib.h>
#include<tracer/Tracer.h>
#include<dreg/Registry.h>
#include<dutil/ColonList.h>
#include<qstring.h>
#ifdef WIN32
#include <winsock2.h>
#define getpid GetCurrentProcessId
#else
#include<unistd.h>
#endif

PostOffice* MessageManager::m_po = NULL;
bool        MessageManager::m_setup = false;


//=============================================================================
// Function Name: init
/// @brief Initializes the MessageManager, opens all ports based on the registry 
/// settings
//=============================================================================
void MessageManager::init()
{
    Tracer t("MessageManager::init()");
    if( m_po ) return;
    Registry::init();
    char* chname = new char[255];
    gethostname( chname, 255 );
    string hostname = chname; // getenv("HOSTNAME");
    delete[] chname;
    string poname = Registry::getValue("/DIVERSE/MessageManager/POName",
                           hostname +'_' 
                           + string(QString::number(getpid()).latin1()), 
                           "The name used to identify the post"
                           "office on the network" );
    m_po = new PostOffice( poname  );

    ColonList serv_ports( 
              Registry::getValue( "/DIVERSE/MessageManager/ServerPortList", 
                                  "", 
                                  "The colon seperated list of ports to "
                                  "listen on." ) );
    vector<string> plist = serv_ports.getList();
  //  for( unsigned int i = 0; i < plist.size(); i++ )
   //     m_po->servicePort( atoi( plist[i].c_str() ) );
}

//=============================================================================
/// @brief Sets up the registry for the displaymgr.
/// @param cfg - the configuration file to be used for the registry settings.
//=============================================================================
void MessageManager::setupRegistry( const string& cfg )
{
    if( m_setup ) return;
    Registry::init(cfg);
    char* chname = new char[255];
    gethostname( chname, 255 );
    string hostname = chname; // getenv("HOSTNAME");
    delete[] chname;
    Registry::getValue("/DIVERSE/MessageManager/POName",
                           hostname,  
                           "The name used to identify the post"
                           "office on the network" );
    Registry::getValue( "/DIVERSE/MessageManager/ServerPortList", 
                        "8000:8001", 
                        "The colon seperated list of ports to "
                        "listen on." );
    m_setup = true;
}

//=============================================================================
/// @brief Connect to the specified server and port.
/// @param adr - the host address of the machine to connect to.
/// @param port - the port to connect to.
/// @return true if the connection was successful, else false.
//=============================================================================
string MessageManager::connectTo( const string& adr, int port )
{
#ifdef _DEBUG_
    if( !m_po )
        Tracer::error( "MessageManager::init() wasn't"
                       " called before first use" );
#endif
    return m_po->connectTo( adr, port, true );
}

//=============================================================================
/// @brief Creates a new POServer that listens for connections on the specified
/// port.
/// @param port - the port to listen on.
/// @return true if successful, else returns false.
//=============================================================================
bool MessageManager::servicePort( int port )
{
#ifdef _DEBUG_
    if( !m_po )
        Tracer::error( "MessageManager::init() wasn't"
                       " called before first use" );
#endif
    return m_po->servicePort( port );
}

//=============================================================================
/// @brief Closes the specified port.
/// @param port - the port to close.
/// @return always true regardless of success or failure.
//=============================================================================
bool MessageManager::closePort( int port )
{
#ifdef _DEBUG_
    if( !m_po )
        Tracer::error( "MessageManager::init() wasn't"
                       " called before first use" );
#endif
    m_po->closePort( port );
    Tracer::warn( 6, "MessageManager::closePort( %d )\n"
                     "always returns true.", port );
    return true;
}

//=============================================================================
/// @brief Creates a new post office box and updates all connections.
/// @param boxname - the name for the POBox.
/// @return a pointer to the new POBox.
//=============================================================================
POBox* MessageManager::registerPOBox( const string& boxname )
{
#ifdef _DEBUG_
    if( !m_po )
        Tracer::error( "MessageManager::init() wasn't"
                       " called before first use" );
#endif
    return m_po->registerPOBox( boxname );
}   

//=============================================================================
/// @brief Removes the box from the post office. This should only be called by 
/// the destructor ~POBox() when you delete it.
/// @param boxname - the name of the box to remove.
/// @return true if successful, otherwise false.
//=============================================================================
bool MessageManager::closePOBox( const string& boxname )
{
#ifdef _DEBUG_
    if( !m_po )
        Tracer::error( "MessageManager::init() wasn't"
                       " called before first use" );
#endif
    return m_po->closePOBox( boxname );
}
