#include"POAddress.h"

//=============================================================================
/// @brief Equal relational operator.
//=============================================================================
bool operator==( const POAddress& a, const POAddress& b )
{
    return a.getPO() == b.getPO() && a.getBox() == b.getBox();
}

//=============================================================================
/// @brief Not equal relational operator.
//=============================================================================
bool operator!=( const POAddress& a, const POAddress& b )
{
    return a.getPO() != b.getPO() || a.getBox() != b.getBox();
}

//=============================================================================
/// @brief Constructor.
/// @param po - a string containing the name of the PostOffice.
/// @param box - a string containing the name of the POBox.
//=============================================================================
POAddress::POAddress( const string& po, const string& box )
:Packable(),m_po(po), m_box(box)
{
}

//=============================================================================
/// @brief Copy constructor.
/// @param cpy - the POAddress object to be copied.
//=============================================================================
POAddress::POAddress( const POAddress& cpy )
:Packable()
{
    m_po = cpy.m_po;
    m_box = cpy.m_box;
}

//=============================================================================
/// @brief returns the pack size of the event.
/// @return the number of bytes required to pack the event.
//=============================================================================
int POAddress::packsize()const
{
    return m_po.packsize() + m_box.packsize();
}

//=============================================================================
/// @brief packs the event into the buffer.
/// @param buffer - the buffer into which the event will
/// be packed. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes packed into the buffer.
//=============================================================================
int POAddress::pack( char* buffer )const
{
    char* tmp = buffer;
    tmp += m_po.pack( tmp );
    tmp += m_box.pack( tmp );
    return tmp - buffer;
}

//=============================================================================
/// @brief unpacks the buffer into the event.
/// @param buffer - the buffer from which the event will
/// be unpacked. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes unpacked from the buffer.
//=============================================================================
int POAddress::unpack( char* buffer )
{
    char* tmp = buffer;
    tmp += m_po.unpack(tmp);
    tmp += m_box.unpack(tmp);
    return tmp - buffer;
}
