#ifndef PO_ADDRESS_H_
#define PO_ADDRESS_H_
#include<pexpr/Packable.h>
#include<pexpr/POTypes.h>
#include <iostream>
using namespace std;

//=============================================================================
/// @class POAddress
/// @brief A packable object to storing addresses ( the name of the PostOffice
/// and the name of the POBox ).
//=============================================================================
class PEXPR_API POAddress : impliments Packable
{
    public:
        POAddress( const string& po = "undefined", const string& box ="undefined" );
        POAddress( const POAddress& cpy );
		/// @brief Destructor.
        virtual ~POAddress(){};
		/// @brief Print debug output to stdout.
		void debug(){cout << "PO = " << m_po.val() << endl; cout << "BOX = " << m_box.val() << endl;};
        
		/// @brief Get a string containing the PostOffice name.
		/// @return a string containing the PostOffice name.
        string getPO()const { return m_po.val(); }
		/// @brief Get a string containing the POBox name.
		/// @return a string containing the POBox name.
        string getBox()const{ return m_box.val(); }
		/// @brief Get a string containing the dot separated address.
		/// @return a string containing the dot separated address.
        string toString()const { return m_po.val() + '.' + m_box.val(); }
       
        int packsize()const;
        int pack( char* buf )const;
        int unpack( char* buf );

    private:
        POString m_po;
        POString m_box;
};

bool PEXPR_API operator==( const POAddress& a, const POAddress& b );
bool PEXPR_API operator!=( const POAddress& a, const POAddress& b );

#endif
