#include"POBox.h"
#include"POMessage.h"
#include"PostOffice.h"
#include<iostream>
#include<tracer/Tracer.h>
#include<qapplication.h>
using namespace std;

//=============================================================================
// Function Name: POBox
/// @brief Constructor.
/// @param name - the name of the inbox.
/// @param po - a pointer to the post office.
//=============================================================================
POBox::POBox( const string& name, PostOffice* po )
:QObject(), m_name(name), m_po(po), m_queuestate(true)
{
}

//=============================================================================
// Function Name: ~POBox
/// @brief Destructor. Delete all unread messages
//=============================================================================
POBox::~POBox()
{
    list<const POMessage*>::iterator itr = m_inbox.begin();
    while( itr != m_inbox.end() )
    {
        delete *itr;
        itr++;
    }
}

//=============================================================================
// Function Name: popMessage
/// @brief Returns the oldest message on the stack, you are responsible for 
/// deleting it.
/// @return a pointer to a POMessage.
//=============================================================================
const POMessage* POBox::popMessage()
{
    const POMessage* back = m_inbox.back();
    m_inbox.pop_back();
    return back;
}

//=============================================================================
// Function Name: HasMessages
/// @brief Returns true if the mail box contains messages
/// @param check_now - true if you want Pony Express to check sockets before
/// before checking messages.  This will happen automatically
/// next time qApp->processEvents() is called, but for the 
/// lowest latency we can call it now.
/// @return true if there are messages, otherwise false
//=============================================================================
bool POBox::hasMessages( bool check_now )const
{
    // if check_now, it will update the socket for the latest messages
    if( check_now ) qApp->processEvents();
    return m_inbox.size() > 0;
}

//=============================================================================
// Function Name: SendMessage
/// @brief Sets the message from field to POBox::GetName().
/// @param msg - the message to send.
/// @param adr - the address to send it to.
/// @param now - whether or not to write it out the socket now, or wait until
/// the next cycle.
/// @return true if the message could be sent, otherwise false.
//=============================================================================
bool POBox::sendMessage( POMessage* msg, const POAddress& adr, bool now )
{
    Tracer::debug( "POBox<%s>::sendMessage( %s, %s.%s )", 
                    m_name.c_str(),
                    msg->messageType().c_str(),
                    adr.getPO().c_str(),
                    adr.getBox().c_str() );
    msg->setDestination( adr );
    msg->setSource( POAddress( m_po->getName(), m_name ) );
    emit messageSent( msg );
    bool rtn = m_po->sendMessage( msg );
    
    // send the message out the socket now
    if( now ) 
        qApp->processEvents();
    return rtn;
}

//=============================================================================
// Function Name: AddMessage
/// @brief Used by the PostOffice to add incomming messages to the box and then
/// emits MessageArrived().
/// @param msg - the incoming message.
//=============================================================================
void POBox::addMessage( const POMessage* msg )
{
    Tracer::debug( 7, "POBox<%s>::addMessage( %s, %s.%s )", 
                    getName().c_str(),
                    msg->messageType().c_str(),
                    msg->getSource().getPO().c_str(),
                    msg->getSource().getBox().c_str() );
    if( m_queuestate )
    {
        m_inbox.push_back( msg );
        emit messageArrived();
    }
    emit messageArrived(msg);

    if( !m_queuestate ) delete msg;
}
