//=============================================================================
// POBox.h
//
// Created 5/13/03 By Daniel Larimer
//=============================================================================
#ifndef _POBOX_H_
#define _POBOX_H_
#include<qobject.h>
#include<string>
#include<list>
#include<pexpr/POAddress.h>
#include<pexpr/POBoxIF.h>

using namespace std;
class PostOffice;
class POMessage;

//=============================================================================
/// @class POBox POBox.h
/// @author Daniel Larimer
/// @brief A receiving queue for messages.
//=============================================================================
class PEXPR_API POBox : public QObject, public POBoxIF
{
    Q_OBJECT
    public:
        POBox( const string& name, PostOffice* po );
        ~POBox();

        const POMessage* popMessage();
        
		/// @brief Set the queue state.
		/// @param state - true if the queue should is accepting messages. Set it
		/// to false to prevent the queue from accpting messages.
        void   setQueueState( bool state = true )   { m_queuestate = state; }
		/// @brief Get the state of the queue.
		/// @return true if the queue is accepting messages, false if it is not.
        bool   getQueueState()const                 { return m_queuestate;  }

		/// @brief Get the name of the POBox.
		/// @return the name of the POBox.
        string getName()const { return m_name; }
        bool   hasMessages( bool check_now = false )const;

		/// @brief Get the PostOffice to which the POBox belongs.
		/// @return a pointer to the PostOffice to which the POBox belongs.
        PostOffice* getPostOffice()const{ return m_po; }
    
        bool sendMessage( POMessage* msg, const POAddress& adr, bool send_now = false );
    
    public slots:
        void addMessage( const POMessage* msg );

    signals:
        /// @brief Emitted when a new message arives if QueueState is true.
		///
        /// Slots hooked to this signal shouldn't set the QueueState to false
        /// unless they want the POBox to delete the message for them.
        void messageArrived();

        /// @brief Emitted for every message that arrives.
        void messageArrived( const POMessage* msg );
        void messageSent   ( const POMessage* msg );
        
    private:
        list<const POMessage*>    m_inbox;
        string                    m_name;
        PostOffice*               m_po;
        bool                      m_queuestate;
};

#endif
