#include"POBoxListMsg.h"
#include"POMessageFactory.h"

RegisterMessage<POBoxListMsg> poboxlistmsg;

//=============================================================================
/// @brief Constructor.
/// @param cmd - the command to be associated with the message.
//=============================================================================
POBoxListMsg::POBoxListMsg( int cmd )
:m_cmd(cmd)
{
}

//=============================================================================
/// @brief Copy constructor.
/// @param cpy - the POBoxListMsg object to be copied.
//=============================================================================
POBoxListMsg::POBoxListMsg( const POBoxListMsg& cpy )
:POMessage((POMessage)cpy), m_cmd(cpy.m_cmd), m_boxes(cpy.m_boxes)
{
}

//=============================================================================
/// @brief Add a box to the list of boxes associated with the message.
/// @param box - the name of the POBox to be added to the list.
//=============================================================================
void POBoxListMsg::addBox( const string& box )
{
    m_boxes.push_back(box);
}

//=============================================================================
/// @brief Clone the POBoxListMsg.
/// @return a POMessage pointer of the cloned POBoxListMsg. 
//=============================================================================
POMessage* POBoxListMsg::clone()const
{
    return new POBoxListMsg( *this );
}

//=============================================================================
/// @brief Returns the pack size of the event.
/// @return the number of bytes required to pack the event.
//=============================================================================
int POBoxListMsg::packsize()const
{
    int size = POMessage::packsize();
    size += m_cmd.packsize();
    size += sizeof(int); // number of boxes
    
    for( unsigned int i = 0; i < m_boxes.size(); i++ )
    {
        POString str(m_boxes[i]);
        size += str.packsize();
    }
    return size;
}

//=============================================================================
/// @brief Packs the event into the buffer.
/// @param buffer - the buffer into which the event will
/// be packed. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes packed into the buffer.
//=============================================================================
int POBoxListMsg::pack( char* buffer )const
{
    char* tmp = buffer;
    tmp += POMessage::pack(tmp);
    tmp += m_cmd.pack(tmp);
    tmp += POInt(m_boxes.size()).pack(tmp);
    
    for( unsigned int i = 0; i < m_boxes.size(); i++ )
    {
//        POString str(m_boxes[i]);
        tmp += POString(m_boxes[i]).pack(tmp);
    }
    return tmp - buffer;
}

//=============================================================================
/// @brief Unpacks the buffer into the event.
/// @param buffer - the buffer from which the event will
/// be unpacked. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes unpacked from the buffer.
//=============================================================================
int POBoxListMsg::unpack( char* buffer )
{
    char* tmp = buffer;
    tmp += POMessage::unpack(tmp);
    tmp += m_cmd.unpack(tmp);

    POInt size;
    tmp += size.unpack(tmp);

    POString box;
    for( int i = 0; i < size.val(); i++ )
    {
        tmp += box.unpack(tmp);
        m_boxes.push_back( box );
    }
    return tmp - buffer;
}
