#ifndef _POBOXLISTMSG_H_
#define _POBOXLISTMSG_H_
#include<pexpr/POMessage.h>
#include<vector> 

//=============================================================================
/// @class POBoxListMsg
/// @brief Message used to pass commands to POBox objects.
//=============================================================================
class PEXPR_API POBoxListMsg : public POMessage
{
    public:
		/// @brief POBox commands.
        enum BOXCMD
		{
			/// @brief Request new boxes be created.
			NEWBOXES,
			/// @brief Request boxes be closed.
			CLOSEBOXES
		};
        POBoxListMsg( int cmd = NEWBOXES );
        POBoxListMsg( const POBoxListMsg& cpy );
		/// @brief Destructor.
        virtual ~POBoxListMsg(){}

		/// @brief Get the type of message.
		/// @return a string containing the type of message.
        string messageType()const { return "POBoxListMsg"; }

        void addBox( const string& box );

		/// @brief Get the list of boxes the command is for.
		/// @return a vector containing the names of the boxes.
        vector<string> getBoxes()const { return m_boxes;      }
		/// @brief Get the command associated with the message.
		/// @return the BOXCMD value representing the command.
        int getCommand()const          { return m_cmd.val();  }

        POMessage* clone()const;

        int packsize()const;
        int pack( char* buffer )const;
        int unpack( char* buffer );
        
    private:
        POInt               m_cmd;
        vector<string>      m_boxes;
};

#endif
