//=============================================================================
// POClient.h
//
// Created 5/13/03 By Daniel Larimer
//=============================================================================
#ifndef _POCLIENT_H_
#define _POCLIENT_H_
#include<qobject.h>
#include<string>
#include<vector>
#include<qstring.h>
#include<pexpr/POAddress.h>

// the maximum size of a message before
// it is broken up into multiple messages
#define MAX_MSG_SIZE (1024*1024*4) // 4MB

using namespace std;

class QSocket;
class POMessage;
class POBoxListMsg;
class POPartialMsg;

//=============================================================================
/// @class POClient
/// @brief Interfaces with
//=============================================================================
class PEXPR_API POClient : public QObject
{
    Q_OBJECT
    public:
		/// @brief The state of the POClient.
        enum CLIENT_STATE
		{
			/// @brief POClient is connected to the server.
			CONNECTED,
			/// @brief POClient is waiting to connect.
			WAIT_CLIST,
			/// @brief POClient is not connected to the server.
			DISCONNECTED
		};
        POClient( int fd );
        POClient( const string& host, int port );
        ~POClient();
        
		/// @brief Get the name of the client.
		/// @return a string containing the name of the client.
        string getClientName()const { return m_clientname; }
		/// @brief Get the state of the client.
		/// @return a CLIENT_STATE value representing the state of the client.
        int    getState()const { return m_state; }
    
		/// @brief Get a list of all boxes associated with the client.
		/// @return a vector containing the names of all the POBoxes associated
		/// with the client.
        vector<string>  getAllBoxes()const { return m_clientboxes; }

    public slots:
        bool sendMessage( const POAddress& adr, const POMessage* msg );
        bool sendMessage( const POMessage* msg );

    private slots:
        void error( int );
        void socketConnected();
        void socketClosed();
        void read();
        
    signals:
        void socketConnected( POClient* client );
        void connectionEstablished( POClient* client );
        void receivedMessage( const POMessage* msg );
        void connectionLost( POClient* );
        void newPOBox( const string& clientname, const string& boxname );
    
    private:
        void handleSegment( POPartialMsg* msg );
        void handleBoxListMsg( const POBoxListMsg* msg );
        void removeClientBox( const string& box );
    
        int             m_state;
        string          m_clientname;
        vector<string>  m_clientboxes;
        QSocket*        m_socket;
        char*           m_buffer;
        int             m_usedsize;
        string          m_peeradr;
        int             m_peerport;

        POInt                   m_incommingsize;
        vector<POPartialMsg*>   m_msgsegments;
};

#endif
