#include"POErrorMsg.h"
#include"POMessageFactory.h"

RegisterMessage<POErrorMsg> poerrormsg;

//=============================================================================
// Function Name: POErrorMsg
/// @brief Default constructor.
/// @param txt - the text for the message.
//=============================================================================
POErrorMsg::POErrorMsg( const string& txt )
:m_txt(txt)
{
}

//=============================================================================
/// @brief Copy constructor.
/// @param cpy - the POErrorMsg to be copied.
//=============================================================================
POErrorMsg::POErrorMsg( const POErrorMsg& cpy )
:POMessage((POMessage)cpy), m_txt(cpy.m_txt)
{
}

//=============================================================================
/// @brief Destructor.
//=============================================================================
POErrorMsg::~POErrorMsg()
{
}

//=============================================================================
/// @brief Set the text for the message.
/// @param txt - the text for the message.
//=============================================================================
void POErrorMsg::setText( const string& txt )
{
    m_txt = txt;
}


//=============================================================================
// Function Name: clone
/// @brief Creates an identical copy of this message. This function must be 
/// overloaded in derived classes.
/// @return a copy of this message.
//=============================================================================
POMessage* POErrorMsg::clone()const
{
    return new POErrorMsg( *this );
}

//=============================================================================
// Function Name: packsize
/// @brief Calculates the number of bytes needed to pack this message.
/// @return size in bytes.
//=============================================================================
int POErrorMsg::packsize()const
{
    int size = POMessage::packsize();
    size += m_txt.packsize();
    return size;
}

//=============================================================================
// Function Name: pack
/// @brief Packs the message into a pre-alocated buffer of size >= packsize().
/// @param buffer - the buffer to pack into.
/// @return the number of bytes packed.
//=============================================================================
int POErrorMsg::pack( char* buffer )const
{
    char* tmp = buffer;
    tmp += POMessage::pack(tmp);
    tmp += m_txt.pack(tmp); 
    
    return tmp - buffer;
}

//=============================================================================
// Function Name: unpack
/// @brief Unpacks the message, setting source, destination, and derived type.
/// @param buffer the buffer to unpack, must be valid, no error checking.
/// @return the number of bytes unpacked.
//=============================================================================
int POErrorMsg::unpack( char* buffer )
{
    char* tmp = buffer;
    tmp += POMessage::unpack(tmp);
   
    tmp += m_txt.unpack( tmp );

    return tmp - buffer;
}
