#include"POMessage.h"
#include"POMessageFactory.h"
#include<iostream>
#include<tracer/Tracer.h>

using namespace std;

RegisterMessage<POMessage> pomessage;

//=============================================================================
// Function Name: POMessage
/// @brief Default constructor.
//=============================================================================
POMessage::POMessage()
:m_derivedtype("POMessage")
{
}

//=============================================================================
/// @brief Copy constructor.
/// @param cpy - the POMessage object to be copied.
//=============================================================================
POMessage::POMessage( const POMessage& cpy )
:Packable(),
 m_source( cpy.m_source ), 
 m_destination( cpy.m_destination ),
 m_derivedtype(cpy.m_derivedtype)
{
    
}

//=============================================================================
/// @brief Destructor.
//=============================================================================
POMessage::~POMessage()
{
}

//=============================================================================
/// @brief Get the address of the source.
/// @return a POAddress object containing information about the source of the
/// message.
//=============================================================================
POAddress POMessage::getSource()const
{
    return m_source;
}

//=============================================================================
/// @brief Get the address of the destination.
/// @return a POAddress object containing information about the destination of
/// the message.
//=============================================================================
POAddress POMessage::getDestination()const
{
    return m_destination;
}   

//=============================================================================
/// @brief Set the address of the source.
/// param source - a POAddress object containing information about the source
/// of the message.
//=============================================================================
void POMessage::setSource( const POAddress& source )
{
    m_source = source;
}

//=============================================================================
/// @brief Get the address of the destination.
/// @return dest - a POAddress object containing information about the destination
/// of the message.
//=============================================================================
void POMessage::setDestination( const POAddress& dest )
{
    m_destination = dest;
}


//=============================================================================
// Function Name: clone
/// @brief Creates an identical copy of this message. This function must be 
/// overloaded in derived classes.
/// @return a copy of this message.
//=============================================================================
POMessage* POMessage::clone()const
{
    return new POMessage( *this );
}

//=============================================================================
// Function Name: packsize
/// @brief Calculates the number of bytes needed to pack this message.
/// @return size in bytes.
//=============================================================================
int POMessage::packsize()const
{
    int size = 0;
    size += sizeof(int); // message size
    size += POString( messageType() ).packsize();
    size += m_source.packsize();
    size += m_destination.packsize();
    return size;
}

//=============================================================================
// Function Name: pack
/// @brief Packs the message into a pre-alocated buffer of size >= packsize().
/// @param buffer - the buffer to pack into.
/// @return the number of bytes packed.
//=============================================================================
int POMessage::pack( char* buffer )const
{
    char* tmp  = buffer;
    
    POInt size(packsize());
    tmp += size.pack(tmp);

    POString type(messageType());
    tmp += type.pack( tmp );
    tmp += m_source.pack( tmp );
    tmp += m_destination.pack( tmp );
   /* 
    if( (tmp-buffer) != packsize() )
    {
        Tracer::error( "POMessage::pack() != POMessage::packsize()\n"
                       "\tMessageType = %s\n"
                       "\tpack() = %d vs. packsize = %d", 
                       messageType().c_str(), (tmp-buffer), packsize() );
        exit(1);
    }
    */
    return tmp - buffer;
}

//=============================================================================
// Function Name: unpack
/// @brief Unpacks the message, setting source, destination, and derived type.
/// @param buffer the buffer to unpack, must be valid, no error checking.
/// @return the number of bytes unpacked.
//=============================================================================
int POMessage::unpack( char* buffer )
{
    char* tmp = buffer;
   
    POInt size;
    tmp += size.unpack(tmp);
    
    tmp += m_derivedtype.unpack( tmp );
    tmp += m_source.unpack( tmp );
    tmp += m_destination.unpack( tmp );

    return tmp - buffer;
}
