//=============================================================================
// POPartialMsg.h
//
// Created 5/13/03 By Daniel Larimer
//=============================================================================
#ifndef _POPARTIALMSG_H_
#define _POPARTIALMSG_H_
#include<pexpr/POMessage.h>
#include<pexpr/POTypes.h>
#include<string>
using namespace std;


//=============================================================================
/// @class POPartialMsg POPartialMsg.h
/// @brief Can only be used internally to send large messages in many parts.
//=============================================================================
class PEXPR_API POPartialMsg : public POMessage
{
    public:
        POPartialMsg( int num = 0, 
                      int of = 0, 
                      char* buffer = NULL, int size = 0 );
        POPartialMsg( const POPartialMsg& cpy );
        virtual ~POPartialMsg();
        virtual string messageType()const { return "POPartialMsg"; };

        virtual POMessage* clone()const;

		/// @brief Get the size of the buffer.
		/// @return the size of the buffer in bytes.
        int getBufferSize()const { return m_size.val();   }
		/// @brief Get the index of the partial message out of the whole.
		/// @return the index of partial message out of the whole.
        int getNumber()const     { return m_num.val();    }
		/// @brief Get the total number of partial messages making up the whole message.
		/// @return the total number of partial messages making up the whole message.
        int getMaxNum()const     { return m_maxnum.val(); }
		/// @brief Get the data for the partial message.
		/// @return a pointer to the data for the partial message.
        char* getData()const     { return m_buffer;       }
        
        //==========================
        // Packable
        //==========================
        int packsize()const;
        int pack( char* buf )const;
        int unpack( char* buf );

    private:
        char*         m_buffer;
        POInt         m_size;
        POInt         m_num;
        POInt         m_maxnum;
};

#endif
