#ifndef _POSMALLFILEMSG_H_
#define _POSMALLFILEMSG_H_
#include<pexpr/POMessage.h>
#include<pexpr/POTypes.h>

//=============================================================================
/// @class POSmallFileMsg
/// @brief Used to send small files as messages.
//=============================================================================
class POSmallFileMsg : public POMessage
{
    public:
        POSmallFileMsg( const string& srcname = "",
                        const string& destfile = "");
        POSmallFileMsg( const POSmallFileMsg& cpy );
        virtual ~POSmallFileMsg();
        
		/// @brief Get the type of message.
		/// @return a string containing the type of message.
        string messageType()const { return "POSmallFileMsg"; }
        POMessage* clone()const;
       
        int packsize()const;
        int pack( char* buffer )const;
        int unpack( char* buffer );
       
        // mutators
        bool    loadFile( const string& fname );
        
        // accessors
		/// @brief Get the file data.
		/// @return a pointer to the file data.
        char*   getFileData()const { return m_filedata;       }
		/// @brief Get the size of the file.
		/// @return the size of the file.
        int     getFileSize()const { return m_filesize.val(); }
		/// @brief Get the name of the source file.
		/// @return a string containing the name of the source file.
        string  getSrcFile()const  { return m_filename.val(); }
		/// @brief Get the name of the destination file.
		/// @return a string containing the name of the destination file.
        string  getDestFile()const { return m_destfile.val(); }

    private:
        char*           m_filedata;
        POInt           m_filesize;
        POString        m_filename; 
        POString        m_destfile;
};
#endif
