#include"POTextMsg.h"
#include"POMessageFactory.h"

#include<iostream>
using namespace std;
static RegisterMessage<POTextMsg> potextmsg;

//=============================================================================
// Function Name: POTextMsg
/// @brief Default constructor.
//=============================================================================
POTextMsg::POTextMsg( const string& txt )
:m_txt(txt)
{
}

//=============================================================================
/// @brief Copy constructor.
/// @param cpy - the POTextMsg object to be copied.
//=============================================================================
POTextMsg::POTextMsg( const POTextMsg& cpy )
:POMessage((POMessage)cpy), m_txt(cpy.m_txt)
{
}

//=============================================================================
/// @brief Destructor.
//=============================================================================
POTextMsg::~POTextMsg()
{
}

//=============================================================================
/// @brief Set the text of the message.
/// @param txt - the text being set.
//=============================================================================
void POTextMsg::setText( const string& txt )
{
    m_txt = txt;
}


//=============================================================================
// Function Name: clone
/// @brief Creates an identical copy of this message.  This function must be 
/// overloaded in derived classes.
/// @return a copy of this message.
//=============================================================================
POMessage* POTextMsg::clone()const
{
    return new POTextMsg( *this );
}

//=============================================================================
// Function Name: packsize
/// @brief Calculates the number of bytes needed to pack this message.
/// @return the size in bytes.
//=============================================================================
int POTextMsg::packsize()const
{
    int size = POMessage::packsize();
    size += m_txt.packsize();
    return size;
}

//=============================================================================
// Function Name: pack
/// @brief Packs the message into a pre-alocated buffer of size >= packsize().
/// @param buffer - the buffer to pack into.
/// @return the number of bytes packed.
//=============================================================================
int POTextMsg::pack( char* buffer )const
{
    char* tmp = buffer;
    tmp += POMessage::pack(tmp);
    tmp += m_txt.pack(tmp); 
    
    return tmp - buffer;
}

//=============================================================================
// Function Name: unpack
/// @brief Unpacks the message, setting source, destination, and derived type.
/// @param buffer - the buffer to unpack, must be valid, no error checking.
/// @return the number of bytes unpacked.
//=============================================================================
int POTextMsg::unpack( char* buffer )
{
    char* tmp = buffer;
    tmp += POMessage::unpack(tmp);
   
    tmp += m_txt.unpack( tmp );

    return tmp - buffer;
}
