#include"POTypes.h"
#ifdef WIN32
#include <winsock2.h>
#else
#include<netinet/in.h>
#endif /* WIN 32 */

#include"byteswap.h"

//=============================================================================
// Function Name: packsize
/// @brief Calculates the number of bytes needed to pack this message.
/// @return the size in bytes.
//=============================================================================
int POString::packsize()const
{
    return sizeof(int) + m_str.size();
}

//=============================================================================
// Function Name: pack
/// @brief Packs the message into a pre-alocated buffer of size >= packsize().
/// @param buffer - the buffer to pack into.
/// @return the number of bytes packed.
//=============================================================================
int POString::pack( char* buffer )const
{
    char* tmp = buffer;
    int size = m_str.size();
    memcpy( tmp, &size, sizeof(size) );
    tmp += sizeof(int); // support longer strings

    memcpy( tmp, m_str.c_str(), m_str.size() );
    tmp += m_str.size();
    
    return tmp - buffer;
}

//=============================================================================
// Function Name: unpack
/// @brief Unpacks the message, setting source, destination, and derived type.
/// @param buffer - the buffer to unpack, must be valid, no error checking.
/// @return the number of bytes unpacked.
//=============================================================================
int POString::unpack( char* buffer )
{
    char* tmp = buffer;
    int size;
    memcpy( &size, tmp, sizeof(size) );
    tmp += sizeof(size);
    
    m_str = string( tmp, size );
    tmp += size;
    
    return tmp - buffer;
}

//=============================================================================
// Function Name: packsize
/// @brief Calculates the number of bytes needed to pack this message.
/// @return the size in bytes.
//=============================================================================
int POInt::packsize()const { return sizeof(int); }

//=============================================================================
// Function Name: pack
/// @brief Packs the message into a pre-alocated buffer of size >= packsize().
/// @param buffer - the buffer to pack into.
/// @return the number of bytes packed.
//=============================================================================
int POInt::pack( char* buffer )const
{
    int tmp = htonl(m_int);
    memcpy( buffer, &tmp, sizeof(int) );
    return sizeof(int);
}

//=============================================================================
// Function Name: unpack
/// @brief Unpacks the message, setting source, destination, and derived type.
/// @param buffer - the buffer to unpack, must be valid, no error checking.
/// @return the number of bytes unpacked.
//=============================================================================
int POInt::unpack( char* buffer )
{
    memcpy( &m_int, buffer, sizeof(int) );
    m_int = ntohl(m_int);
    return sizeof(m_int);
}

//=============================================================================
// Function Name: packsize
/// @brief Calculates the number of bytes needed to pack this message.
/// @return the size in bytes.
//=============================================================================
int POFloat::packsize()const { return sizeof(m_float); }

//=============================================================================
// Function Name: pack
/// @brief Packs the message into a pre-alocated buffer of size >= packsize().
/// @param buffer - the buffer to pack into.
/// @return the number of bytes packed.
//=============================================================================
int POFloat::pack( char* buffer )const
{
    float tmp = m_float;
    host2std((void*)&tmp, sizeof(tmp) );
    memcpy( buffer, &tmp, sizeof(tmp) );
    return sizeof(tmp);
}

//=============================================================================
// Function Name: unpack
/// @brief Unpacks the message, setting source, destination, and derived type.
/// @param buffer - the buffer to unpack, must be valid, no error checking.
/// @return the number of bytes unpacked.
//=============================================================================
int POFloat::unpack( char* buffer )
{
    memcpy( &m_float, buffer, sizeof(m_float) );
    std2host((void*)&m_float, sizeof(m_float) );
    return sizeof(m_float);
}

//=============================================================================
// Function Name: packsize
/// @brief Calculates the number of bytes needed to pack this message.
/// @return the size in bytes.
//=============================================================================
int PODouble::packsize()const { return sizeof(m_double); }

//=============================================================================
// Function Name: pack
/// @brief Packs the message into a pre-alocated buffer of size >= packsize().
/// @param buffer - the buffer to pack into.
/// @return the number of bytes packed.
//=============================================================================
int PODouble::pack( char* buffer )const
{
    double tmp = m_double;
    host2std((void*)&tmp, sizeof(tmp) );
    memcpy( buffer, &tmp, sizeof(tmp) );
    return sizeof(tmp);
}

//=============================================================================
// Function Name: unpack
/// @brief Unpacks the message, setting source, destination, and derived type.
/// @param buffer - the buffer to unpack, must be valid, no error checking.
/// @return the number of bytes unpacked.
//=============================================================================
int PODouble::unpack( char* buffer )
{
    memcpy( &m_double, buffer, sizeof(m_double) );
    std2host((void*)&m_double, sizeof(m_double) );
    return sizeof(m_double);
}
