#ifndef _POTYPES_H_
#define _POTYPES_H_
#include<pexpr/Packable.h>
#include<string>
using namespace std;

#ifdef WIN32
	#include <windows.h>
	#ifdef PEXPR_EXPORTS
		#define PEXPR_API __declspec(dllexport)
	#else
		#define PEXPR_API __declspec(dllimport)
	#endif
#else
	#define PEXPR_API
#endif          /*WIN32*/

//=============================================================================
/// @class POString
/// @brief A packable string.
//=============================================================================
class PEXPR_API POString : public Packable
{
    public:
        POString( const string& str = ""):m_str(str){};
        POString( const POString& str ):Packable(),m_str(str.val()){};
        virtual ~POString(){}
            
        int packsize()const;
        int pack( char* buf )const;
        int unpack( char* buf );

        operator string&() { return m_str; }
        string val()const  { return m_str; }
    private:
        string m_str;
};

//=============================================================================
/// @class POInt
/// @brief A packable integer.
//=============================================================================
class PEXPR_API POInt : public Packable
{
    public:
        POInt( const int& val = 0 ):m_int(val){};

        int packsize()const;
        int pack( char* buf )const;
        int unpack( char* buf );

		POInt& operator = ( int val ) { m_int = val; return *this; };
        operator int&() { return m_int; }
        int val()const  { return m_int; }

     private:
        int m_int;
};

//=============================================================================
/// @class POFloat
/// @brief A packable float.
//=============================================================================
class PEXPR_API POFloat : public Packable
{
    public:
        POFloat( const float& val = 0 ):m_float(val){};

        int packsize()const;
        int pack( char* buf )const;
        int unpack( char* buf );

		POFloat& operator = ( float val ) { m_float = val; return *this; };
        operator float&() { return m_float; }
        float val()const  { return m_float; }

     private:
        float m_float;
};

//=============================================================================
/// @class PODouble
/// @brief A packable double.
//=============================================================================
class PEXPR_API PODouble : public Packable
{
    public:
        PODouble( const double& val = 0 ):m_double(val){};

        int packsize()const;
        int pack( char* buf )const;
        int unpack( char* buf );

		PODouble& operator = ( double val ) { m_double = val; return *this; };
        operator double&() { return m_double; }
        double val()const  { return m_double; }

     private:
        double m_double;
};

#endif
