//=============================================================================
// PostOffice.h
//
// This code is released under the GPL
// Created 5/13/03 By Daniel Larimer
//=============================================================================
#ifndef _POSTOFFICE_H_
#define _POSTOFFICE_H_
#include<qobject.h>
#include<map>
#include<vector>
#include<string>
#include<pexpr/POAddress.h>

using namespace std;
class POServer;
class POClient;
class POBox;
class POMessage;

//=============================================================================
/// @class PostOffice
/// @brief Manages POClients and POBoxes.
//=============================================================================
class PEXPR_API PostOffice : public QObject
{
    Q_OBJECT
    public:
        PostOffice( const string& name );
        ~PostOffice();

        POClient*              getClient(const string& cname)const;
        POBox*                 getPOBox( const string& boxname )const;
        map<string, POClient*> getAllClients()const;
		/// @brief Get all of the POBox objects associated with this PostOffice.
        map<string, POBox*>    getAllBoxes()const { return m_localboxes; }

		/// @brief Get the name of the PostOffice.
        string           getName()const { return m_poname; }
        const POMessage* popMessage( const string& pobox_name );
        bool             hasMessage( const string& pobox_name );

        static map<string, PostOffice*> getAllPostOffices();
        static PostOffice*              getPostOffice( const string& name );
        
    public slots:
        string  connectTo( const string& host, int port, bool block = true );
        bool    servicePort( int port );
        void    closePort( int port );

        POBox*  registerPOBox( const string& name );
        bool    closePOBox( const string& name, bool dodelete = false );
        
        
        void connectClient     ( POClient* client );
        void addConnectedClient( POClient* client );
        bool sendMessage( const POMessage* msg );
        bool sendMessage( POMessage* msg, const string& from, 
                                          const POAddress& to );

        void newClientPOBox( const string& cname, const string& boxname );
    
    signals:
        void newClient( const string& poname, const string& cname );
        void newPOBox( const string& poname, const string& cname, const string& boxname );
        void newPOBox( const string& poname, const string& boxname );
        void clientClosed( const string& poname, const string& cname );
        void portClosed( int port );
        void portOpened( int port );

        
    private slots:
        bool deliverMessage( const POMessage* msg );
        void closeConnection( POClient* client );
        
    private:
        string                    m_poname;
        map<int, POServer*>       m_servers;
        map<string, POClient*>    m_clients;
        map<string, POBox*>       m_localboxes;

        static map<string, PostOffice*> m_postoffices;
};

#endif
