#include"Plugin.h"
#include <iostream>

using namespace std;

//=============================================================================
/// @brief Constructor.
/// @param name - the name of the plugin.
/// @param des - a description of the plugin.
//=============================================================================
Plugin::Plugin( const string& name, const string& des )
:m_name(name), m_description(des)
{
    m_loaded = false;
}

//=============================================================================
/// @brief Get the name of the plugin.
/// @return a string containing thew name of the plugin.
//=============================================================================
string Plugin::getName()const
{
	return m_name;
}

//=============================================================================
/// @brief Get the name of the file from which the plugin was loaded.
/// @return a string containing the file name.
//=============================================================================
string Plugin::getFileName()const
{
	return m_file;
}

//=============================================================================
/// @brief Get a handle to the plugin.
/// @return a handle to the plugin.
//=============================================================================
PLGHANDLE Plugin::getHandle()const
{
	return m_handle;
}

//=============================================================================
/// @brief Determine if the plugin is loaded.
/// @return true if the plugin is loaded, else false.
//=============================================================================
bool   Plugin::isLoaded()const
{
	return m_loaded;
}

//=============================================================================
/// @brief Get the description of the plugin.
/// @return a string containing a description of the plugin.
//=============================================================================
string Plugin::getDescription()const
{
	return m_description;
}

//=============================================================================
/// @brief Set the description for the plugin.
/// @param desc - a string containg the description of the plugin.
//=============================================================================
void Plugin::setDescription( const string& desc )
{
    m_description = desc;
}

//=============================================================================
/// @brief Set the file name.
/// @param filename - a string containing the name of the file.
//=============================================================================
void Plugin::setFileName( const string& filename )
{
    m_file = filename;
}

//=============================================================================
/// @brief Set the handle for the plugin.
/// @param handle - the handle for the plugin.
//=============================================================================
void Plugin::setHandle( PLGHANDLE handle )
{
    m_handle = handle;
}

//=============================================================================
/// @brief Sets the status of the plugin as loaded.
//=============================================================================
void Plugin::load()
{
    m_loaded = true;
}

//=============================================================================
/// @brief Sets the status of the plugin as not loaded.
//=============================================================================
void Plugin::unload()
{
    m_loaded = false;
}
