#ifndef _PLUGINMANAGER_H_
#define _PLUGINMANAGER_H_
#include<commander/CommandHandler.h>

#include<string>
#include<vector>
#include<map>

#ifdef WIN32
#ifdef PLUGINMGR_EXPORTS
#define PLUGINMGR_API __declspec(dllexport)
#else
#define PLUGINMGR_API __declspec(dllimport)
#endif /* PLUGINMGR_EXPORTS */
#else
#define PLUGINMGR_API
#endif /* WIN32 */

#include"dutil/ColonList.h"

using namespace std;

class Plugin;

//=============================================================================
/// @class PluginManager
/// @author Daniel Larimer
/// @brief Repository for loadable plugins.
///
/// The PluginManager is a static object which must be initialized prior to any
/// of its functions being called. The application framework initializes this 
/// object early in the application initialization phase. The PluginManager manages
/// all plugins used in DIVERSE. It keeps a list of all plugins which are currently
/// loaded and allows you to get information about the plugins as well as a pointer
/// to the plugin if necessary. It also allows you to load new plugins or unload
/// plugins which were already loaded.
//=============================================================================
class PLUGINMGR_API PluginManager : public CommandHandler
{
    public:
        static void           init();
        static void           exit();
        static void           setupRegistry( const string& filename = "diverse.xml" );
        static string         openPlugin( const string& filename, CommandOut* out = NULL ); 
        static bool           closePlugin( const string& plugin_name );

        static Plugin*        getPlugin( const string& plugin_name );
        static Plugin*        loadPlugin( const string& plugin_name, CommandOut* out=NULL );
        static bool           unloadPlugin( const string& plugin_name );
        static map<string, Plugin*> getPlugins();
        static void           registerPlugin( Plugin* plg );

        static void           loadLibrary( const string& libname );
        static vector<string> getSearchDirectories();
        static void           addSearchDirectory( const string& path );
        static void           removeSearchDirectory( const string& dir );
        static void           setPath( const string& path );
        static string         getPath();
        
        //====================================================
        // Executes commands from the command manager
        //====================================================
        CommandOut*           execute( const Command& cmd );
    private:
        PluginManager();
        static PluginManager*       m_instance;

        static string m_OpenPlugin( const string& filename, CommandOut* out ); 

        static map<string, Plugin*> m_plugins;
        static ColonList            m_path;
        static bool                 m_setup;
        static bool                 m_initialized;
};

#endif
