#include"CommandWidget.h"
#include<commander/CommandOut.h>
#include<iostream>
using namespace std;

#if ( QT_VERSION < 300 )
CommandWidget::CommandWidget( Commander* cmdr, QWidget* parent )
:QMultiLineEdit( parent )
{
    m_prompttxt = "[commander]$ ";
    m_prompt = m_prompttxt;
    m_firsttab = true;
    m_historyindex = 0;
    m_history.push_back("");
    m_commander = cmdr;
	m_line = 0;
	m_column = 0;
    connect( this, SIGNAL( returnPressed() ), this, SLOT( sendCommand() ) );
    resize( 500, 300 );
    append( m_prompt );
    setCursorPosition( m_line, lineLength( m_line ) );
}

void CommandWidget::sendCommand()
{
    QString cmd = textLine( m_line );
    CommandOut* out = m_commander->execute( Command( cmd.latin1() + m_prompttxt.length()) );
    setDefaultTabStop( 20 );
    vector<string> lines = out->getOutput();
    for( unsigned int i = 0; i < lines.size(); i++ )
	{
        append( lines[i].c_str() );
		m_line++;
	}
    append( m_prompt );
	m_line++;
    setCursorPosition( m_line, lineLength( m_line ) );
    if( cmd.length() > m_prompttxt.length() )
    {
        m_history.back() = (cmd.latin1()+m_prompttxt.length());
        m_historyindex = m_history.size();
        m_history.push_back("");
    }
    delete out;
}

void CommandWidget::keyPressEvent( QKeyEvent* e )
{
	unsigned int x;
    int para, index;
    int lastpara = m_line;
    getCursorPosition( &para, &index );
    // Keep things in the last paragraph
    if( para != lastpara )
        setCursorPosition( lastpara, lineLength( lastpara ) );
    getCursorPosition( &para, &index );
	setWrapColumnOrWidth( width() );
    
    if( e->key() == Key_Tab )
    {
        string firstch = textLine( lastpara ).latin1() + m_prompttxt.length();
        firstch = firstch.substr( 0, firstch.size()-1);
        unsigned int i = 0;
        for( ; i < firstch.size(); i++ )
            if( firstch[i] != ' ' ) break;
        vector<string> options = 
                m_commander->getCommandList( firstch.substr( i, firstch.size() ) );
        if( options.size() > 1 && !m_firsttab )
        {
            unsigned int maxlen = 0;
            string maxstr;
            for( x = 0; x < options.size(); x++ )
            {   
                if( options[x].size() > maxlen )
                {
                    maxlen = options[x].size();
                    maxstr = options[x];
                }
            }   
            //QFontMetrics fm( currentFont() );
            QFontMetrics fm( font() );
            int fmwidth = fm.width( maxstr.c_str() );
            setDefaultTabStop( (fmwidth + 20) / fm.width( 'x' ) );
            
            unsigned int cols = width() / (fmwidth+20);
            unsigned int rows = options.size() / cols;
            if( cols % options.size() ) rows++;
            vector<string> sorted;
            // sort based on cols
            for( x = 0; x < rows; x++ )
            {
                for( unsigned int y = 0; y < cols; y++ )
                {
                    unsigned int pos = rows*y + x;
                    if( pos >= options.size() )
                        sorted.push_back( "" );
                    else
                        sorted.push_back( options[pos] );
                }
            }
            options = sorted;
            int colcount = 0; 
            string tmp;
            for( x = 0; x < options.size(); x++ )
            {
                tmp += options[x] + '\t';
                if( (x % cols) >= (cols - 1) )
				{
					append( tmp.c_str() );
					m_line++;
					tmp = "";
				}
            }
			if ( tmp != "" )
			{
	            append( tmp.c_str() );
				m_line++;
			}
        }
        if( options.size() == 1 )
        {
            removeLine( m_line );
			m_line--;
            append( m_prompt + options[0].c_str() + " " );
			m_line++;
        }
        else if( options.size() != 0 )
        {
            //===========================
            // calculate most incommon
            //===========================
            string incommon = firstch;
            bool good = true;
            unsigned int len = firstch.size();
            do
            {
                for( unsigned int x = 0; x < options.size(); x++ )
                {
                    if( options[x].size() <= len || options[x][len] != options[0][len] )
                    {
                        good = false;
                        break;
                    }
                }
                if( good )
                    incommon += options[0][len];
                len++;
            }while( good );
            if( m_firsttab )
			{
				removeLine( m_line );
				m_line--;
			}
            append( m_prompt + incommon.c_str() );
			m_line++;
            cerr<<"\a";
            m_firsttab = false;
        }
        setCursorPosition( m_line, lineLength( m_line ) );
    }
    else if( e->key() == Key_Backspace || e->key() == Key_Left )
    {
        m_firsttab = true;
        if( (unsigned int)index > m_prompttxt.length() + 1 )
            QMultiLineEdit::keyPressEvent( e );
    }
    else if( e->key() == Key_Home )
    {
        setCursorPosition( lastpara, m_prompttxt.length() + 1 );
    }
    else if( e->key() == Key_Up )
    {
        if( (unsigned)m_historyindex == m_history.size() -1 )
            m_current = textLine( lastpara ).latin1();
        if( m_historyindex > 0 )
        {
            m_historyindex--;
            removeLine( lastpara );
			m_line--;
            append( m_prompt + m_history[m_historyindex].c_str() );
			m_line++;
        }
        setCursorPosition( m_line, lineLength( m_line ) );
    }
    else if( e->key() == Key_Down )
    {
        if( (unsigned)m_historyindex < m_history.size() - 1 )
        {
            m_historyindex++;
            removeLine( lastpara );
			m_line--;
            append( m_prompt + m_history[m_historyindex].c_str() );
			m_line++;
        }
        else if( (unsigned) m_historyindex == m_history.size() )
        {
            removeLine( lastpara );
			m_line--;
            append( m_current.c_str() );
			m_line++;
        }
        setCursorPosition( m_line, lineLength( m_line ) );
    }
    else if( e->key() == Key_Enter || e->key() == Key_Return )
    {
        sendCommand();
    }
    else
    {
        m_firsttab = true;
        QMultiLineEdit::keyPressEvent( e );
    }
}

#else

CommandWidget::CommandWidget( Commander* cmdr, QWidget* parent )
:QTextEdit( parent )
{
   // setTextFormat(RichText);
    m_prompttxt = "[commander]$";
    m_prompt = "<tt><b>" + m_prompttxt + "</b> ";
    m_firsttab = true;
    m_historyindex = 0;
    m_history.push_back("");
    m_commander = cmdr;
    connect( this, SIGNAL( returnPressed() ), this, SLOT( sendCommand() ) );
    resize( 500, 300 );
    append( m_prompt );
    setCursorPosition( paragraphs()-1, paragraphLength( paragraphs()-1) );
}

void CommandWidget::sendCommand()
{
    QString cmd = text( paragraphs() - 1 );
    CommandOut* out = m_commander->execute( Command( cmd.latin1() + m_prompttxt.length()) );
    setTabStopWidth( 20 );
    vector<string> lines = out->getOutput();
    for( unsigned int i = 0; i < lines.size(); i++ )
        append( lines[i].c_str() );
    append( m_prompt );
    setCursorPosition( paragraphs()-1, paragraphLength( paragraphs()-1) );
    if( cmd.length() > m_prompttxt.length() )
    {
        m_history.back() = (cmd.latin1()+m_prompttxt.length());
        m_historyindex = m_history.size();
        m_history.push_back("");
    }
    delete out;
}

void CommandWidget::keyPressEvent( QKeyEvent* e )
{
    int para, index;
    int lastpara = paragraphs() - 1;
    getCursorPosition( &para, &index );
    // Keep things in the last paragraph
    if( para != lastpara )
        setCursorPosition( lastpara, paragraphLength( lastpara ) );
    getCursorPosition( &para, &index );
    
    if( e->key() == Key_Tab )
    {
        string firstch = text( lastpara ).latin1() + m_prompttxt.length();
        firstch = firstch.substr( 0, firstch.size()-1);
        unsigned int i = 0;
        for( ; i < firstch.size(); i++ )
            if( firstch[i] != ' ' ) break;
        vector<string> options = 
                m_commander->getCommandList( firstch.substr( i, firstch.size() ) );
        if( options.size() > 1 && !m_firsttab )
        {
            unsigned int maxlen = 0;
            string maxstr;
            for( unsigned int x = 0; x < options.size(); x++ )
            {   
                if( options[x].size() > maxlen )
                {
                    maxlen = options[x].size();
                    maxstr = options[x];
                }
            }   
            //QFontMetrics fm( currentFont() );
            QFontMetrics fm( font() );
            int fmwidth = fm.width( maxstr.c_str() );
            setTabStopWidth( fmwidth + 20 );
            
            int cols = width() / (fmwidth+20);
            int rows = options.size() / cols;
            if( cols % options.size() ) rows++;
            vector<string> sorted;
            // sort based on cols
            for( int x = 0; x < rows; x++ )
            {
                for( int y = 0; y < cols; y++ )
                {
                    int pos = rows*y + x;
                    if( pos >= options.size() )
                        sorted.push_back( "" );
                    else
                        sorted.push_back( options[pos] );
                }
            }
            options = sorted;
            int colcount = 0; 
            string tmp;
            for( unsigned int x = 0; x < options.size(); x++ )
            {
                tmp += options[x] + '\t';
                if( (x % cols) >= (cols - 1) ) tmp += '\n';
            }   
            append( tmp.c_str() );
        }
        if( options.size() == 1 )
        {
            removeParagraph( paragraphs() - 1 );
            append( m_prompt + options[0].c_str() + " " );
        }
        else if( options.size() != 0 )
        {
            //===========================
            // calculate most incommon
            //===========================
            string incommon = firstch;
            bool good = true;
            unsigned int len = firstch.size();
            do
            {
                for( unsigned int x = 0; x < options.size(); x++ )
                {
                    if( options[x].size() <= len || options[x][len] != options[0][len] )
                    {
                        good = false;
                        break;
                    }
                }
                if( good )
                    incommon += options[0][len];
                len++;
            }while( good );
            if( m_firsttab ) removeParagraph( paragraphs()-1);
            append( m_prompt + incommon.c_str() );
            cerr<<"\a";
            m_firsttab = false;
        }
        setCursorPosition( paragraphs()-1, paragraphLength( paragraphs()-1) );
    }
    else if( e->key() == Key_Backspace || e->key() == Key_Left )
    {
        m_firsttab = true;
        if( index > m_prompttxt.length() + 1 )
            QTextEdit::keyPressEvent( e );
    }
    else if( e->key() == Key_Home )
    {
        setCursorPosition( lastpara, m_prompttxt.length() + 1 );
    }
    else if( e->key() == Key_Up )
    {
        if( (unsigned)m_historyindex == m_history.size() -1 )
            m_current = text( lastpara ).latin1();
        if( m_historyindex > 0 )
        {
            m_historyindex--;
            removeParagraph( lastpara );
            append( m_prompt + m_history[m_historyindex].c_str() );
        }
        setCursorPosition( paragraphs()-1, paragraphLength( paragraphs()-1) );
    }
    else if( e->key() == Key_Down )
    {
        if( (unsigned)m_historyindex < m_history.size() - 1 )
        {
            m_historyindex++;
            removeParagraph( lastpara );
            append( m_prompt + m_history[m_historyindex].c_str() );
        }
        else if( (unsigned) m_historyindex == m_history.size() )
        {
            removeParagraph( lastpara );
            append( m_current.c_str() );
        }
        setCursorPosition( paragraphs()-1, paragraphLength( paragraphs()-1) );
    }
    else if( e->key() == Key_Enter || e->key() == Key_Return )
    {
        sendCommand();
    }
    else
    {
        m_firsttab = true;
        QTextEdit::keyPressEvent( e );
    }
}

#endif
