//=============================================
// EventMonitorWidget.h
// Author: Patrick Shinpaugh
// Date: July 1, 2003
//=============================================
#ifndef _EVENTMONITORWIDGET_H_
#define _EVENTMONITORWIDGET_H_
#include <eventmgr/EventListener.h>
#include <eventmgr/devents.h>
#include <eventmgr/DKeyboard.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qcheckbox.h>

#ifdef WIN32
#ifdef QTUTIL_EXPORTS
#define QTUTIL_API __declspec(dllexport)
#else
#define QTUTIL_API __declspec(dllimport)
#endif /* QTUTIL_EXPORTS */
#else
#define QTUTIL_API
#endif /* WIN32 */

//=============================================================================
// Provides a check box class which needed additional functionality for its tasks.
//=============================================================================
class QTUTIL_API RemoteEventCheckBox : public QCheckBox
{
    Q_OBJECT
    public:
        RemoteEventCheckBox( QWidget *parent, const char* name = 0 );

    public slots:
        void eventImport( bool state );
        void eventExport( bool state );

    signals:
        void importEvent( const char* name, bool state );
        void exportEvent( const char* name, bool state );
};

//=============================================================================
// The main widget which holds all of the other widgets required for the
// EventMonitor.
//=============================================================================
class QTUTIL_API EventMonitorWidget:public QWidget, public EventListener
{
    Q_OBJECT
    public:
        EventMonitorWidget( QWidget* parent );
        ~EventMonitorWidget();

        virtual bool onMouseEvent( DMouseEvent* evt );
        virtual bool onKeyEvent( DKeyEvent* evt );
        virtual bool onWandEvent( DWandEvent* evt );
        virtual bool onTrackerEvent( DTrackerEvent* evt );
        virtual bool onOtherEvent( const string& etype, DEvent* evt );
        
    public slots:
        void location();
        void eventImport( const char* name, bool state );
        void eventExport( const char* name, bool state );

    private:
        string m_HSegment;

        QLabel* m_titles[4];
        QLabel* m_labels[7][12];
        QLabel* m_keys[3];

        bool m_wandbuttons[4];
        bool m_mousebuttons[3];
        map<char,KeyState> m_asciimap;
};

#endif
