#include"POMonitor.h"
#include<qlistview.h>
#include<qlayout.h>
#include<qwidgetstack.h>
#include<pexpr/PostOffice.h>
#include<pexpr/POBox.h>
#include<pexpr/POClient.h>
#include<tracer/Tracer.h>
#include"BoxMonitor.h"
#include"POStatus.h"
#if ( QT_VERSION < 300 )
#include "qt23wrappers.h"
#endif /* WIN32 */

POMonitor::POMonitor( QWidget* parent )
:QWidget( parent )
{
    QGridLayout* glayout = new QGridLayout( this, 2, 2, 3 );
    
    m_listview = new QListView( this );
    m_listview->setMaximumWidth( 200 );
    m_listview->setMinimumWidth( 200 );
    m_listview->addColumn( "Post Offices", 180 );

    connect( m_listview, SIGNAL( clicked( QListViewItem* ) ),
             this, SLOT( itemClicked( QListViewItem* ) ) );
    
    glayout->addMultiCellWidget( m_listview, 0, 1, 0, 0, AlignLeft );

    m_postack = new QWidgetStack( this );
    m_postack->setMaximumSize( 600, 120 );
    glayout->addWidget( m_postack, 0, 1 );
    glayout->setColStretch( 1, 3 );

    m_boxstack = new QWidgetStack( this );
    glayout->addWidget( m_boxstack, 1, 1 );

    m_defaultpostat = new POStatus( m_postack, NULL );
    m_defaultboxmon = new BoxMonitor( m_boxstack, NULL );
    
    resize( 800, 400 );

    // TODO Init Stack
    map<string, PostOffice*> plist = PostOffice::getAllPostOffices();
    map<string, PostOffice*>::iterator itr = plist.begin();
    while( itr != plist.end() )
    {
        addPostOffice( itr->first );
        itr++;
    }
}

void POMonitor::addClient( const string& po, const string& client )
{
#if ( QT_VERSION < 300 )
    QListViewItem* poitem = findItem( m_listview, po.c_str() );
#else
    QListViewItem* poitem = m_listview->findItem( po.c_str(), 0 );
#endif
    if( poitem == NULL )
    {
        addPostOffice( po );
        return;
    }
    QListViewItem* conlist = findItemIn(poitem, "Clients", 0 );
    QListViewItem* citem = new QListViewItem( conlist, client.c_str() );
    citem->setExpandable(true);

    vector<string> boxes = PostOffice::getPostOffice( po )->getClient( client )->getAllBoxes();
    for( unsigned int i = 0; i < boxes.size(); i++ )
    {
        new QListViewItem( citem, boxes[i].c_str() );
    }
}

QListViewItem* POMonitor::findItemIn( QListViewItem* item, const string& txt, int col )
{
    QListViewItem* chld = item->firstChild();
    while( chld )
    {
        if( chld->text(col) == txt.c_str() )
            return chld;
        chld = chld->nextSibling();
    }
    return NULL;
}

void POMonitor::addPostOffice( const string& po )
{
    map<string, PostOffice*> plist = PostOffice::getAllPostOffices();
    map<string, PostOffice*>::iterator itr = plist.find(po);
    if( itr != plist.end() )
    {
        connect( itr->second, 
                 SIGNAL( newClient( const string&, const string&  ) ),
                 this,
                 SLOT( addClient( const string&, const string& ) ) );
        connect( itr->second,
                 SIGNAL( newPOBox( const string&, const string&, const string& ) ),
                 this, SLOT( addPOBox( const string&, const string&, const string& ) ) );
        
        connect( itr->second,
                 SIGNAL( newPOBox( const string&, const string& ) ),
                 this, SLOT( addPOBox( const string&, const string& ) ) );

        connect( itr->second, 
                 SIGNAL( clientClosed( const string&, const string& ) ), 
                 this,
                 SLOT( removeClient( const string&, const string& ) ) );

        QListViewItem* poitem = new QListViewItem( m_listview, po.c_str() );
        poitem->setExpandable(true);
        QListViewItem* clist = new QListViewItem( poitem, "Clients" );
        QListViewItem* blist = new QListViewItem( poitem, "PO Boxes" );
        clist->setExpandable(true);
        blist->setExpandable(true);

        map<string, POBox*> boxes = itr->second->getAllBoxes();
        map<string, POBox*>::iterator bitr = boxes.begin();
        while( bitr != boxes.end() )
        {
            new QListViewItem( blist, bitr->first.c_str() );
            bitr++;
        }   
        map<string, POClient*> clients = itr->second->getAllClients();
        map<string, POClient*>::iterator citr = clients.begin();
        while( citr != clients.end() )
        {
            new QListViewItem( clist, citr->first.c_str() );
            citr++;
        }   
        
        itr++;
    }else
    {
        Tracer::warn("POMonitor::addPostOffice( %s ) - Unknown Post Office", po.c_str() );
    }

}

void POMonitor::addPOBox( const string& po, const string& client, const string& box )
{
#if ( QT_VERSION < 300 )
    QListViewItem* poitem = findItem( m_listview, po.c_str() );
#else
    QListViewItem* poitem = m_listview->findItem( po.c_str(), 0 );
#endif
    if( poitem == NULL )
    {
        addPostOffice( po );
        return;
    }
    QListViewItem* cli = findItemIn( findItemIn(poitem, "Clients", 0 ),  client.c_str(), 0 );
    if( !cli )
    {
        addClient( po, client );
        return;
    }
    new QListViewItem( cli, client.c_str() );
}

void POMonitor::addPOBox( const string& po, const string& box )
{
#if ( QT_VERSION < 300 )
    QListViewItem* poitem = findItem( m_listview, po.c_str() );
#else
    QListViewItem* poitem = m_listview->findItem( po.c_str(), 0 );
#endif
    if( poitem == NULL )
    {
        addPostOffice( po );
        return;
    }
    QListViewItem* boxlist = findItemIn( poitem, "PO Boxes", 0 );
    new QListViewItem( boxlist, box.c_str() );
}

void POMonitor::removeClient( const string& po, const string& client )
{
#if ( QT_VERSION < 300 )
    QListViewItem* poitem = findItem( m_listview, po.c_str() );
#else
    QListViewItem* poitem = m_listview->findItem( po.c_str(), 0 );
#endif
    if( poitem ) 
    {
        QListViewItem* citem = findItemIn(findItemIn( poitem, "Clients", 0), client.c_str(), 0 );
        if( citem )
            delete citem;
    }
}

void POMonitor::removePOBox( const string& po, const string& box )
{
#if ( QT_VERSION < 300 )
    QListViewItem* poitem = findItem( m_listview, po.c_str() );
#else
    QListViewItem* poitem = m_listview->findItem( po.c_str(), 0 );
#endif
    if( poitem ) 
    {
        QListViewItem* bitem = findItemIn( findItemIn(poitem, "PO Boxes", 0), box.c_str(), 0 );
        if( bitem )
            delete bitem;
    }
}
void POMonitor::removePOBox( const string& po, const string& client, const string& box )
{
#if ( QT_VERSION < 300 )
    QListViewItem* poitem = findItem( m_listview, po.c_str() );
#else
    QListViewItem* poitem = m_listview->findItem( po.c_str(), 0 );
#endif
    if( poitem ) 
    {
        QListViewItem* citem = findItemIn( findItemIn( poitem, "Clients", 0), client.c_str(), 0 );
        if( citem )
        {
            QListViewItem* bitem = findItemIn( poitem, box.c_str(), 0 );
            if( bitem )
                delete bitem;
        }
    }
}


void POMonitor::itemClicked( QListViewItem* item )
{
    if( !item ) return;
    if( item->parent() == 0 )
    {
        // post office clicked
        return;
    }

    if( item->parent()->parent() == 0 )
    {
        // PO Box List 


        // or Clients

        return;
    }
    if( item->parent()->parent()->parent() == 0 )
    {
        if( item->parent()->text(0) == "PO Boxes" )
        {
            POBox* box = PostOffice::getPostOffice( item->parent()->parent()->text(0).latin1() )
                        ->getPOBox(item->text(0).latin1() );
            if( m_boxes[box] == 0 )
            {
                BoxMonitor* bm = new BoxMonitor( m_boxstack, box );
                m_boxes[box] = bm;
                m_boxstack->addWidget( bm, int(box) );
            }
            else
            {

            }
            m_boxstack->raiseWidget( int(box) );
            return;
        }
        if( item->parent()->text(0) == "Clients" )
        {
            return;
        }
        return;
    }
    if( item->parent()->parent()->parent()->parent() == 0 )
    {
        // Remote PO Box
    }
}

