#include"PathEditWidget.h"
#include<qlistbox.h>
#include<qlabel.h>
#include<qpushbutton.h>
#include<qfiledialog.h>
#include<qlayout.h>
#include<dreg/Registry.h>
#include<dreg/Entry.h>
#include<tracer/Tracer.h>
#include<iostream>
#include<diverseconfig.h>
using namespace std;

PathEditWidget::PathEditWidget( QWidget* parent )
:QWidget( parent )
{
    QVBoxLayout* vlayout = new QVBoxLayout( this );
    m_label = new QLabel( "Path:", this );
    vlayout->addWidget( m_label );
    m_panelpathlist = new QListBox( this );
    vlayout->addWidget( m_panelpathlist );

    QHBoxLayout* hlayout = new QHBoxLayout( vlayout );
    
    QPushButton* selpath = new QPushButton( "Add", this, "Add Path" );
    connect( selpath, SIGNAL( clicked() ), 
             this,    SLOT  ( selectPath() ) );

    QPushButton* rem = new QPushButton( "Remove", this, "Remove Selected" );
    connect( rem, SIGNAL( clicked() ),
             this, SLOT ( removeSelectedPath() ) );

    hlayout->addWidget( rem );
    hlayout->addWidget( selpath );
}

void PathEditWidget::removeSelectedPath()
{
    if( m_panelpathlist->currentItem() != -1 )
    {
        QString txt = m_panelpathlist->currentText();
        if( txt == QString::null )
        {
            Tracer::error( "PathEditWidget::removeSelectedPath() - nothing selected" );
            return;
        }
        m_pathlist.removeItem( txt.latin1() );
        emit directoryRemoved( txt.latin1() );
        emit pathChanged( m_pathlist.getString() );
        Tracer::warn( "Path changed, emit pathChanged( %s )", m_pathlist.getString().c_str() );
        m_panelpathlist->removeItem( m_panelpathlist->currentItem() ); 
    }
}

void PathEditWidget::selectPath()
{
    QString dir = QFileDialog::getExistingDirectory( INSTALLDIR,
                                                     this,
           "Select a directory.",
           "Select the directory containing your preference panes.",
           true );
    if( dir != QString::null )
        addPath( dir );
}

void PathEditWidget::addPath( const QString& path )
{
    if( m_pathlist.findItem( path.latin1() ) != -1 )return;
    m_pathlist.addItem( path.latin1() );
    m_panelpathlist->insertItem( new QListBoxText( path ) );
    emit directoryAdded( path.latin1() );
    emit pathChanged( m_pathlist.getString() );
}

void PathEditWidget::setPath( const string& path )
{
    m_pathlist = ColonList( path ); 
    m_panelpathlist->clear();
    emit pathChanged( m_pathlist.getString() );
    vector<string> dirs = m_pathlist.getList();
    for( unsigned int i = 0; i < dirs.size(); i ++ )
        m_panelpathlist->insertItem( new QListBoxText( dirs[i].c_str() ) );
}

void PathEditWidget::setLabel( const QString& label )
{
    m_label->setText( label );
}
