#include"QDScreenWidget.h"
#include<dmath/dVec3.h>
#include <displaymgr/DisplayManager.h>
#include <displaymgr/DCamera.h>
#include<dutil/DTime.h>
#include<iostream>
#include<tracer/Tracer.h>
using namespace std;

//=============================================================================
// Function Name: QDScreenWidget
/// @brief Constructor.
/// @param parent - the parent widget.
//=============================================================================
QDScreenWidget::QDScreenWidget( QWidget* parent )
    :QDWidget( parent )
{
    m_left   = - 1;
    m_right  =   1;
    m_top    =   1;
    m_bottom = - 1;

    m_displayid = 0;
}

//=============================================================================
// Function Name: paintGL
/// @brief Sets up the frustum and model view and then displays the scene.
//=============================================================================
void QDScreenWidget::paintGL()
{
    if( !isVisible() )return;
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    //=============================
    // Calculate Position
    //=============================
    dVec3 eye = dVec3(0,0,0); //DGL::GetHeadPos();
    //Tracer::warn("Need to add eye code QDScreenWidget::paintGL()");
        
    // Translate eye to face screen
    dVec3 eyetx = m_inv_rot_mat * eye;
    
    dVec3 v = m_pos_orig - eye;
    // Distance from eye to screen
    float D = v.x()* m_norm.x() + v.y()*m_norm.y() + v.z()*m_norm.z();
    
    // Calculate "scale"
    float s = DisplayManager::getNearClip() / (D);
    
    float left   = (m_left   - eyetx.x() ) * s;
    float right  = (m_right  - eyetx.x() ) * s;
    float top    = (m_top    - eyetx.z() ) * s;
    float bottom = (m_bottom - eyetx.z() ) * s;
    
    glFrustum( left, right, bottom, top,
               DisplayManager::getNearClip(), DisplayManager::getFarClip() );
   
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    gluLookAt( eye.x(), eye.y(), eye.z(),
               eye.x() + m_norm.x(),
               eye.y() + m_norm.y(),
               eye.z() + m_norm.z(),
               m_up.x(), m_up.y(), m_up.z() );

    dMatrix proj;
    proj.makeFrustum( 
               left, right, bottom, top,
               DisplayManager::getNearClip(), DisplayManager::getFarClip() );
        
    dMatrix viewmat;
	int x = QDWidget::x(), y = QDWidget::y(), w = width(), h = height();
    viewmat.makeLookAt( 
               eye,
               dVec3(eye.x() + m_norm.x(),
               eye.y() + m_norm.y(),
               eye.z() + m_norm.z()),
               dVec3(m_up.x(), m_up.y(), m_up.z()) );
    DCamera dcam( Rect(x,y,w,h), proj.ptr(), 
                                 viewmat.ptr() );
               
    //=================================
    // Give DIVERSE plugins a chance to cull
    //=================================
    DisplayManager::cull(&dcam);
    DisplayManager::draw(&dcam);
}

//=============================================================================
/// @brief Resize the OpenGL viewport when the window is resized.
/// @param width - the width of the viewport.
/// @param height - the height of the viewport.
//=============================================================================
void QDScreenWidget::resizeGL( int width, int height )
{
    m_top = m_pos.z() + float(height)/width;
    m_bottom = m_pos.z() - float(height)/width;
    glViewport( 0, 0, width, height );
}

//=============================================================================
// Function Name: SetPosition
/// @brief Sets the position of the screen
/// @param pos - the position of the center of the screen
/// @param ang - the normal to the screen
//=============================================================================
void QDScreenWidget::setPosition( const dVec3& pos, const dVec3& ang )
{
    m_pos_orig = pos;

    m_rot_mat = dMatrix::rotate( -ang.h(), dVec3( 0, 0, 1 ),
                                      -ang.p(), dVec3( 1, 0, 0 ),
                                      -ang.r(), dVec3( 0, 1, 0 ) );
    m_inv_rot_mat = dMatrix::inverse(m_rot_mat);
    //===================================
    // Find Normal
    //===================================
    dVec4 n(0,1,0,0);
    m_norm = m_rot_mat * n;
    m_norm.normalize();
   
    cout<<"norm = "<<m_norm.x()<<", "<<m_norm.y()<<", "<<m_norm.z()<<endl;
    //===================================
    // Find Screen left, right, top, bottom
    //===================================
    m_pos = m_inv_rot_mat * dVec4( pos.x(), pos.y(), pos.z(), 0 );
    cout<<"pos = "<<m_pos.x()<<", "<<m_pos.y()<<", "<<m_pos.z()<<endl;
    
    m_left   = m_pos.x() - 1;
    m_right  = m_pos.x() + 1;
    m_top    = m_pos.z() + 1;
    m_bottom = m_pos.z() - 1;

    //==================================
    // Find the up vector
    //==================================
    m_up = dVec4( 0, 0, 1, 0 );
    m_up = m_rot_mat * m_up;
    m_up.normalize();

    cout<<"up = "<<m_up.x()<<", "<<m_up.y()<<", "<<m_up.z()<<endl;

    cout<<"===========================================\n";
}
