#ifndef _QTSCREENWIDGET_H_
#define _QTSCREENWIDGET_H_
#include<dmath/dMatrix.h>
#include<dmath/dVec3.h>
#include<dmath/dVec4.h>
#include "QDWidget.h"
#include<string>
using namespace std;

#ifdef WIN32
#ifdef QTUTIL_EXPORTS
#define QTUTIL_API __declspec(dllexport)
#else
#define QTUTIL_API __declspec(dllimport)
#endif /* QTUTIL_EXPORTS */
#else
#define QTUTIL_API
#endif /* WIN32 */

//=============================================================================
/// @class QDScreenWidget
/// @author Daniel Larimer
/// @brief Displays the contents of a DGL screen, used in conjuntions with
/// the QDWindow. It is the canvas where the graphical output actually takes place.
//=============================================================================
class QTUTIL_API QDScreenWidget : public QDWidget
{
    public: 
        QDScreenWidget( QWidget* parent );
        
        void setPosition( const dVec3& pos, const dVec3& ang );
        
        void paintGL();
        void resizeGL( int width, int height );
        void initializeGL(){};

    private:
        int    m_displayid;
        bool   m_threaded;
        float  m_left;
        float  m_right;
        float  m_bottom;
        float  m_top;
        
        dMatrix            m_rot_mat;
        dMatrix            m_inv_rot_mat;
        dVec4              m_pos;
        dVec3              m_pos_orig;
        dVec3              m_eyedelta;
        dVec4              m_up;
        dVec4              m_norm;
};


#endif
