#include"QDWindow.h"
#include<qpopupmenu.h>
#include<qmenubar.h>
#include<qapplication.h>
#include<signal.h>
#include<qtimer.h>
#include<qstatusbar.h>
#include "PreferenceDialog.h"
#include "EventMonitorDialog.h"
#include "caveDeviceSimulatorDialog.h"
#include "CommandWidget.h"
#include "POMonitor.h"
#include "PanelRegistry.h"
#include "QServiceManager.h"
#include "QDWidget.h"
#include<commander/CommandManager.h>
#include<commander/CommandOut.h>

#include <string>
#include <iostream>
#include <vector>
using namespace std;
#include <dmath/Rect.h>

QDWindow::QDWindow()
	:QMainWindow()
{
    PanelRegistry::init();
    resize( QSize( 640, 480 ) );
    m_fcount = 0;

    QPopupMenu* file = new QPopupMenu(this);
    file->insertItem( "&Settings", this, SLOT( editPrefs() ), CTRL + Key_E );
    file->insertItem( "&Quit", this, SLOT( quit() ), CTRL + Key_Q );

    menuBar()->insertItem("&File", file);

    QPopupMenu* tools = new QPopupMenu(this);
    tools->insertItem( "&Event Monitor", this, SLOT( eventMonitor() ), CTRL + Key_E );
    tools->insertItem( "&Device Simulator", this, SLOT( caveDeviceSimulator() ), CTRL + Key_D );
    tools->insertItem( "&Message Monitor", this, SLOT( messageMonitor() ), CTRL + Key_M );
    tools->insertItem( "&Command Line", this, SLOT( showCommandLine() ), CTRL + Key_L );
    menuBar()->insertItem("&Tools", tools);

    m_overlays = new QPopupMenu(this);
    m_debugol = 
        m_overlays->insertItem( "&Debug", this, SLOT( toggleDebugOverlay() ), CTRL + Key_Question );
    m_gnomonol = 
        m_overlays->insertItem( "&Gnomon", this, SLOT( toggleGnomonOverlay() ), CTRL + Key_G );
    m_overlays->setItemChecked( m_debugol,  false );
    m_overlays->setItemChecked( m_gnomonol, false );
    menuBar()->insertItem("&Overlays", m_overlays);


    QTimer* t = new QTimer( this );
    t->start(1000);
    connect( t, SIGNAL( timeout() ),
             this, SLOT(calcfps() ) );
    m_lastframe = 0;
    m_etime.start();

    m_dwidget = new QDWidget( this );
    m_dwidget->resize( 400, 400 );
    setCentralWidget( m_dwidget );
    statusBar()->message("Ready");
    /// Closing the window quits the program
    connect( qApp, SIGNAL( lastWindowClosed() ),
			this, SLOT( quit() ) );

    connect( QServiceManager::instance(), SIGNAL( processSignal() ),
			this, SLOT( frame() ) );
}

void QDWindow::toggleDebugOverlay()
{
    cout<<"toggleDebugOverlay()\n";
    bool checked = m_overlays->isItemChecked( m_debugol );
    CommandManager.execute( Command( "pluginmgr load DebugHeadsUp" ) );
    if( checked )
        delete CommandManager.execute( Command( "debugHeadsUp hide" ) );
    else
        delete CommandManager.execute( Command( "debugHeadsUp show" ) );
    m_overlays->setItemChecked( m_debugol, !checked );
    
}
void QDWindow::toggleGnomonOverlay()
{
    cout<<"toggleDebugOverlay()\n";
    bool checked = m_overlays->isItemChecked( m_gnomonol );
    if( checked )
        delete CommandManager.execute( Command( "pluginmgr unload GnomonHeadsUp" ) );
    else
        delete CommandManager.execute( Command( "pluginmgr load GnomonHeadsUp") );
    m_overlays->setItemChecked( m_gnomonol, !checked );
}

void QDWindow::quit()
{
    close();
    qApp->quit();
    ::raise(2);
}

void QDWindow::showCommandLine()
{
    CommandWidget* w = new CommandWidget( &CommandManager, NULL );
    w->show();
}
void QDWindow::editPrefs()
{
    PreferenceDialog::showDialog( this );
}

void QDWindow::eventMonitor()
{
    EventMonitorDialog::open( this );
}

void QDWindow::caveDeviceSimulator()
{
    caveDeviceSimulatorDialog::open( this );
}

void QDWindow::messageMonitor()
{
    POMonitor* mon = new POMonitor(NULL);
    mon->show();
}

void QDWindow::calcfps()
{
    int dframe = m_fcount - m_lastframe;
    float secs = m_etime.elapsed().sec();
    m_etime.start();
    float fps = dframe/secs;
    statusBar()->message( QString::number( fps ) + QString("fps") );
    m_lastframe = m_fcount;
}

void QDWindow::frame()
{
    m_fcount++;
    m_dwidget->frame();
}
