#include"QServiceManager.h"
#include<servicemgr/GServiceManager.h>
#include<dreg/Registry.h>
#include<qapplication.h>
#include<dutil/CommandArgs.h>
#include"EventMonitorDialog.h"
#include"caveDeviceSimulatorDialog.h"
#include"CommandWidget.h"
#include"POMonitor.h"
#include<commander/CommandOut.h>
#include<commander/CommandManager.h>

#include<iostream>
using namespace std;

QServiceManager* QServiceManager::m_instance = NULL;

//=============================================================================
// Function Name: init()
/// @brief Initialize the QServiceManager.
///
/// This will construct the service, add it to the GServiceManager.  This must
/// be called prior to calling any QServiceManager functions.
//=============================================================================
void QServiceManager::init()
{
    static bool initialized = false;
    if( initialized ) return;
    m_instance = new QServiceManager();
    initialized = true;
}

//=============================================================================
// Function Name: instance
/// @brief Get a pointer to the single instance of QServiceManager.
///
/// If QServiceManager::init() hasn't been called instance() will call it.
/// @return a pointer to the one and only instance of the QServiceManager.
//=============================================================================
QServiceManager* QServiceManager::instance()
{
    init();
    return m_instance;
}

//=============================================================================
// Function Name: QServiceManager()
/// @brief Constructor.
//=============================================================================
QServiceManager::QServiceManager()
    :QObject(), GService("QServiceManager")
{
    string priority = 
        Registry::getValue( "/DIVERSE/QServiceManager/ServicePriority", "6" );
    
    setPriority( atoi( priority.c_str() ), false );
    GServiceManager::addService( this );
    if( !qApp )
    {
        char** argv = CommandArgs::getArgv();
        int argc = CommandArgs::getArgc();
        qApp = new QApplication( argc, argv );
    }
    CommandManager.addCommandHandler( "caveDeviceSimulator", this );
    CommandManager.addCommandHandler( "eventMonitor", this );
    CommandManager.addCommandHandler( "messageMonitor", this );
    CommandManager.addCommandHandler( "command", this );
}

//=============================================================================
/// @brief Executes a Commander command, allowing Commander to load and unload
/// plugins dynamically.
/// @param cmd - the command to execute.
/// @return a pointer to a CommandOut object.
//=============================================================================
CommandOut* QServiceManager::execute( const Command& cmd )
{
    CommandOut* out = new CommandOut( cmd );
    vector<string> args = cmd.getArgs();
    if( args[0] == "caveDeviceSimulator" )
    {
        out->print( "Opening the CAVE Device Simulator..." );
        caveDeviceSimulatorDialog::open( NULL );
    }
    if( args[0] == "eventMonitor" )
    {
        out->print( "Opening the Event Monitor..." );
        EventMonitorDialog::open( NULL );
    }
    if( args[0] == "messageMonitor" )
    {
        out->print( "Opening the Message Monitor..." );
        POMonitor* mon = new POMonitor(NULL);
        mon->show();
    }
    if( args[0] == "command" )
    {
        out->print( "Opening a new command line window..." );
        CommandWidget* w = new CommandWidget( &CommandManager, NULL );
        w->show();
    }
    return out;
}

//=============================================================================
// Function Name: preProcess
/// @brief Gives Qt a chance to process events and then emits preProcessSignal.
//=============================================================================
void QServiceManager::preProcess()
{
    qApp->processEvents();
    emit preProcessSignal();
}

//=============================================================================
// Function Name: preProcess
/// @brief Emits processSignal.
//=============================================================================
void QServiceManager::process()
{
    emit processSignal();
}

//=============================================================================
// Function Name: preProcess
/// @brief Emits postProcessSignal.
//=============================================================================
void QServiceManager::postProcess()
{
    emit postProcessSignal();
}
