#ifndef _QSERVICEMANAGER_H_
#define _QSERVICEMANAGER_H_
#include<qobject.h>
#include<servicemgr/GService.h>
#include<commander/CommandHandler.h>

#ifdef WIN32
#ifdef QTUTIL_EXPORTS
#define QTUTIL_API __declspec(dllexport)
#else
#define QTUTIL_API __declspec(dllimport)
#endif /* QTUTIL_EXPORTS */
#else
#define QTUTIL_API
#endif /* WIN32 */

//=============================================================================
/// @class QServiceManager
/// @author Daniel Larimer
/// @brief Maps the service manager callbacks to signals.
///
/// This class provides an interface by which QObjects may connect to a
/// signal to get the preProcess, process, and postProcess call backs.  It 
/// calls qApp->processEvents() durring preProcess.  
///
/// Because QServiceManager is a service of the GServiceManager it gets
/// called along with all of the other services.  The default priority for this
/// service is 10; however, you may change this using the Registry
/// key "/DIVERSE/QServiceManager/ServicePriority". 
//=============================================================================
class QTUTIL_API QServiceManager : public QObject, public GService, public CommandHandler
{
    Q_OBJECT
    public:
        static void init();
        static QServiceManager* instance();

        void preProcess();
        void process();
        void postProcess();

        CommandOut* execute( const Command& cmd );
        
    signals:
        void preProcessSignal();
        void processSignal();
        void postProcessSignal();

    private:
        QServiceManager();
        static QServiceManager* m_instance;
};

#endif
