//=============================================
// caveDeviceSimulatorWidget.h
// Author: Patrick Shinpaugh
// Date: July 3, 2003
//=============================================
#include <stdarg.h>
#include <eventmgr/EventManager.h>
#include <eventmgr/devents.h>
#include <navmgr/NavManager.h>
#include <dreg/Registry.h>
#include <dreg/Entry.h>
#include <tracer/Tracer.h>
#include <qtabwidget.h>
#include <qlayout.h>
#include <qcanvas.h>
#include <qvalidator.h>
#ifdef WIN32
#include <winsock2.h>
#define getpid GetCurrentProcessId
#else
#include <unistd.h>
#endif
#include "caveDeviceSimulatorWidget.h"

// JOYSTICK_GRID_X is the grid size in the x direction
#define JOYSTICK_GRID_X 200
// JOYSTICK_GRID_Y is the grid size in the y direction
#define JOYSTICK_GRID_Y 200
// SLIDER_DIVISIONS is half the slider divisions for the head and wand tracker sliders
#define SLIDER_DIVISIONS 100

int caveDeviceSimulatorWidget::m_nextindex = 0;

static char* format( char *s, const char *fmt, ... )
{
    va_list args;
    va_start(args, fmt);
    vsprintf( s, fmt, args );
    va_end(args);
    return s;
}

//===================================
// class CaveDevSimLabel
//===================================
CaveDevSimButton::CaveDevSimButton( const QString & text, QWidget * parent, const char * name )
    :QPushButton( text, parent, name )
{
    m_headwand = -1;
    m_num = -1;
}

CaveDevSimButton::CaveDevSimButton( int num, const QString & text, QWidget * parent, const char * name )
    :QPushButton( text, parent, name )
{
    m_headwand = -1;
    m_num = num;
}

CaveDevSimButton::CaveDevSimButton( int headwand, int num, const QString & text, QWidget * parent, const char * name )
    :QPushButton( text, parent, name )
{
    m_headwand = headwand;
    m_num = num;
}

void CaveDevSimButton::resetSlider()
{
    emit reset( m_headwand, m_num );
}

void CaveDevSimButton::initSlider()
{
    emit init( m_headwand, m_num );
}

void CaveDevSimButton::resetButtons()
{
    emit reset( -1, m_num );
}

void CaveDevSimButton::initButtons()
{
    emit init( -1, m_num );
}

void CaveDevSimButton::updateButtons( bool on )
{
    emit buttonToggled( m_num, on );
}

//===================================
// class CaveDevSimLabel
//===================================
CaveDevSimLabel::CaveDevSimLabel( QWidget * parent, const char * name, WFlags f )
    :QLabel( parent, name, f )
{
}

void CaveDevSimLabel::sliderToXYZ( int val )
{
    char temp[10];
    setText( format( temp, "%-5.3f", val / (float)SLIDER_DIVISIONS ) );
}

void CaveDevSimLabel::sliderToHPR( int val )
{
    char temp[10];
    setText( format( temp, "%-5.1f", val * 180.0f / (float)SLIDER_DIVISIONS ) );
}

//===================================
// class CaveDevSimCanvasView
//===================================
CaveDevSimCanvasView::CaveDevSimCanvasView( QCanvas * canvas, QWidget * parent, const char * name, WFlags f )
    :QCanvasView( canvas, parent, name, f )
{
    m_canvas = canvas;
    m_canvas->resize( JOYSTICK_GRID_X + 1, JOYSTICK_GRID_Y + 1 );
    QPen pen( red );
    for ( int i=0;i<2;i++ )
    {
        m_joysticklines[i] = new QCanvasLine( canvas );
        m_joysticklines[i]->setPen( pen );
        if ( i == 0 )
        {
            m_joysticklines[i]->setPoints( 0, 0, 0, JOYSTICK_GRID_Y );
            m_joysticklines[i]->move( JOYSTICK_GRID_X / 2, 0 );
        }
        else
        {
            m_joysticklines[i]->setPoints( 0, 0, JOYSTICK_GRID_X, 0 );
            m_joysticklines[i]->move( 0, JOYSTICK_GRID_Y / 2 );
        }
        m_joysticklines[i]->show();
    }
    m_springstickval = false;
    m_joystickcanvaspassed = false;
    m_canvas->update();
}

void CaveDevSimCanvasView::updateXPosition( const QString &text )
{
    float val = atof( text );
    if ( val > 1.0f )
        val = 1.0f;
    else if ( val < -1.0f )
        val = -1.0f;
    updatePosition( (int)((float)JOYSTICK_GRID_Y / 2.0f * ( val + 1.0f) ), (int)m_joysticklines[1]->y() );
	emit joystickChanged();
}

void CaveDevSimCanvasView::updateYPosition( const QString &text )
{
    float val = atof( text );
    if ( val > 1.0f )
        val = 1.0f;
    else if ( val < -1.0f )
        val = -1.0f;
    updatePosition( (int)m_joysticklines[0]->x(), (int)((float)JOYSTICK_GRID_Y / 2.0f * (2.0f - ( val + 1.0f )) ));
	emit joystickChanged();
}

void CaveDevSimCanvasView::updatePosition( int x, int y )
{
    if ( x < 0 )
        x = 0;
    else if ( x > JOYSTICK_GRID_X )
        x = JOYSTICK_GRID_X;
    if ( y < 0 )
        y = 0;
    else if ( y > JOYSTICK_GRID_Y )
        y = JOYSTICK_GRID_Y;

    m_joysticklines[0]->move( x, 0 );
    m_joysticklines[0]->show();
    m_joysticklines[1]->move( 0, y );
    m_joysticklines[1]->show();
    m_canvas->update();
}

void CaveDevSimCanvasView::contentsMouseMoveEvent( QMouseEvent* evt )
{
    QPoint point = evt->pos();
    updatePosition( point.x(), point.y() );
    float x = (float)(2.0f * point.x() / JOYSTICK_GRID_X - 1.0f );
    float y = (float)(1.0f - 2.0f * point.y() / JOYSTICK_GRID_Y );
    if ( x > 1.0f )
        x = 1.0f;
    else if ( x < -1.0f )
        x = -1.0f;
    if ( y > 1.0f )
        y = 1.0f;
    else if ( y < -1.0f )
        y = -1.0f;
	m_joystickcanvaspassed = true;
    emit joystickChanged( x, y );
}

void CaveDevSimCanvasView::contentsMousePressEvent( QMouseEvent* evt )
{
    QPoint point = evt->pos();
    updatePosition( point.x(), point.y() );
    float x = (float)(2.0f * point.x() / JOYSTICK_GRID_X - 1.0f );
    float y = (float)(1.0f - 2.0f * point.y() / JOYSTICK_GRID_Y );
    if ( x > 1.0f )
        x = 1.0f;
    else if ( x < -1.0f )
        x = -1.0f;
    if ( y > 1.0f )
        y = 1.0f;
    else if ( y < -1.0f )
        y = -1.0f;
	m_joystickcanvaspassed = true;
    emit joystickChanged( x, y );
}

void CaveDevSimCanvasView::contentsMouseReleaseEvent( QMouseEvent* evt )
{
    if ( m_springstickval )
    {
        updatePosition( JOYSTICK_GRID_X / 2, JOYSTICK_GRID_Y / 2 );
		m_joystickcanvaspassed = true;
        emit joystickChanged( 0.0f, 0.0f );
    }
}

void CaveDevSimCanvasView::updateSpringStick( bool val )
{
    if ( val )
    {
        m_springstickval = true;
        updatePosition( JOYSTICK_GRID_X / 2, JOYSTICK_GRID_Y / 2 );
		m_joystickcanvaspassed = true;
        emit joystickChanged( 0.0f, 0.0f );
    }
    else
        m_springstickval = false;
}

//===================================
// class caveDeviceSimulatorWidget
//===================================
caveDeviceSimulatorWidget::caveDeviceSimulatorWidget( QWidget* parent )
    :QWidget( parent, 0, Qt::WDestructiveClose )
{
    Tracer t("caveDeviceSimulatorWidget::constructor\n");

    char* name = new char[255];
    gethostname( name, 255 );
    m_sourcename = name;
	m_sourcename += format( name, "%d.%d", getpid(), caveDeviceSimulatorWidget::m_nextindex++ );
	delete name;

	int i;
    m_headeventpassed = false;
    m_wandeventpassed = false;
    m_HSegment = Registry::getValue( "/DGL/HeadTracker", "headtracker" );
    DEvent* evt = EventManager::getTrackerState( m_HSegment );
    if ( evt )
    {
        m_headval.pos = m_headinitval.pos = ((DTrackerEvent*)evt)->getPosition();
        m_headval.dir = m_headinitval.dir = ((DTrackerEvent*)evt)->getDirection();
        evt = NULL;
    }
    else
    {
        m_headval.pos = m_headinitval.pos = dVec3( 0.0f, 0.0f, 0.0f );
        m_headval.dir = m_headinitval.dir = dVec3( 0.0f, 0.0f, 0.0f );
    }
    m_headresetval.pos = dVec3( 0.0f, 0.0f, 0.0f );
    m_headresetval.dir = dVec3( 0.0f, 0.0f, 0.0f );
    
    evt = EventManager::getWandState();
    if ( evt )
    {
        m_wandval.pos = m_wandinitval.pos = ((DWandEvent*)evt)->getPosition();
        m_wandval.dir = m_wandinitval.dir = ((DWandEvent*)evt)->getDirection();
        m_joystickval[0] = m_joystickinitval[0] = ((DWandEvent*)evt)->getJoystickX();
        m_joystickval[1] = m_joystickinitval[1] = ((DWandEvent*)evt)->getJoystickY();
        m_joystickresetval[0] = m_joystickresetval[1] = 0.0f;
        for ( i=0;i<4;i++ )
        {
            m_buttonsval[i] = m_buttonsinitval[i] = ((DWandEvent*)evt)->getButton( i );
            m_buttonsresetval[i] = false;
        }
        evt = NULL;
    }
    else
    {
        m_wandval.pos = m_wandinitval.pos = dVec3( 0.0f, 0.2f, -0.2f );
        m_wandval.dir = m_wandinitval.dir = dVec3( 0.0f, 0.0f, 0.0f );
        m_joystickval[0] = m_joystickinitval[0] = m_joystickval[1] = m_joystickinitval[1] = 0.0f;
        m_joystickresetval[0] = m_joystickresetval[1] = 0.0f;
        for ( i=0;i<4;i++ )
            m_buttonsval[i] = m_buttonsinitval[i] = m_buttonsresetval[i] = false;
    }
    m_wandresetval.pos = dVec3( 0.0f, 0.2f, -0.2f );
    m_wandresetval.dir = dVec3( 0.0f, 0.0f, 0.0f );
    m_springstick = false;
    
    this->setMinimumSize( QSize( 700, 250 ) );

    QVBoxLayout* main = new QVBoxLayout( this );
    QHBoxLayout* top = new QHBoxLayout( main );
    QHBoxLayout* bottom = new QHBoxLayout( main );

    QPushButton* button = new QPushButton( "Quit", this, "quitbutton" );
    button->setAutoDefault( false );
    connect( button, SIGNAL( clicked() ), this, SLOT( close() ) );
    top->addWidget( button );
    
    button = new QPushButton( "Reset All", this, "resetallbutton" );
    button->setAutoDefault( false );
    connect( button, SIGNAL( clicked() ), this, SLOT( resetAll() ) );
    top->addWidget( button );
    
    button = new QPushButton( "Init All", this, "initallbutton" );
    button->setAutoDefault( false );
    connect( button, SIGNAL( clicked() ), this, SLOT( initAll() ) );
    top->addWidget( button );
    
    button = new QPushButton( "Update Mem", this, "updatemembutton" );
    button->setAutoDefault( false );
    connect( button, SIGNAL( clicked() ), this, SLOT( updateMem() ) );
    top->addWidget( button );
    
    button = new QPushButton( "Update Widget", this, "updatewidgetbutton" );
    button->setAutoDefault( false );
    connect( button, SIGNAL( clicked() ), this, SLOT( updateWidget() ) );
    top->addWidget( button );
    
    button = new QPushButton( "Help", this, "helpbutton" );
    button->setAutoDefault( false );
    connect( button, SIGNAL( clicked() ), this, SLOT( help() ) );
    top->addWidget( button );
    
    QTabWidget* tabWidget = new QTabWidget( this, "tabwidget" );
    bottom->addWidget( tabWidget );
    bottom->setStretchFactor( tabWidget, 2 );
    QWidget* headWidget = new QWidget( tabWidget, "headwidget" );
    QWidget* wandWidget = new QWidget( tabWidget, "wandwidget" );
    tabWidget->addTab( headWidget, "Head Tracker" );
    tabWidget->addTab( wandWidget, "Wand Tracker" );
    QGridLayout* headgrid = new QGridLayout( headWidget, 6, 5 );
    QGridLayout* wandgrid = new QGridLayout( wandWidget, 6, 5 );
    
    QDoubleValidator* validator;
    QLabel* label;
    dVec3 pos, dir;
    char temp[32];
    for ( i=0;i<2;i++ )
    {
        label= new QLabel( "X", i == 0 ? headWidget : wandWidget,
                format( temp, "%sXlabel", i == 0 ? "head" : "wand" ) );
        ( i == 0 ? headgrid : wandgrid )->addWidget( label, 0, 0 );
        label= new QLabel( "Y", i == 0 ? headWidget : wandWidget,
                format( temp, "%sYlabel", i == 0 ? "head" : "wand" ) );
        ( i == 0 ? headgrid : wandgrid )->addWidget( label, 1, 0 );
        label= new QLabel( "Z", i == 0 ? headWidget : wandWidget,
                format( temp, "%sZlabel", i == 0 ? "head" : "wand" ) );
        ( i == 0 ? headgrid : wandgrid )->addWidget( label, 2, 0 );
        label= new QLabel( "H", i == 0 ? headWidget : wandWidget,
                format( temp, "%sHlabel", i == 0 ? "head" : "wand" ) );
        ( i == 0 ? headgrid : wandgrid )->addWidget( label, 3, 0 );
        label= new QLabel( "P", i == 0 ? headWidget : wandWidget,
                format( temp, "%sPlabel", i == 0 ? "head" : "wand" ) );
        ( i == 0 ? headgrid : wandgrid )->addWidget( label, 4, 0 );
        label= new QLabel( "R", i == 0 ? headWidget : wandWidget,
                format( temp, "%sRlabel", i == 0 ? "head" : "wand" ) );
        ( i == 0 ? headgrid : wandgrid )->addWidget( label, 5, 0 );
        for ( int j=0;j<6;j++ )
        {
            m_trackvals[i][j] = new CaveDevSimLabel( i == 0 ? headWidget : wandWidget,
                    format( temp, "%strackval-%d", i == 0 ? "head" : "wand", j ) );
            m_trackvals[i][j]->setFixedSize( 40, 20 );
            m_trackvals[i][j]->setAutoResize( false );
            ( i == 0 ? headgrid : wandgrid )->addWidget( m_trackvals[i][j], j, 1 );
            Coord hwcoord = i == 0 ? m_headval : m_wandval;
            int val;
            if ( j < 3 )
                val = (int)( (float)SLIDER_DIVISIONS * hwcoord.pos[ j ] );
            else
                val = (int)( (float)SLIDER_DIVISIONS * hwcoord.dir[ j - 3 ] / 180.0f );
            m_sliders[i][j] = new QSlider( -SLIDER_DIVISIONS, SLIDER_DIVISIONS, 1, val,
                    Qt::Horizontal, i == 0 ? headWidget : wandWidget,
                    format( temp, "%sslider-%d", i == 0 ? "head" : "wand", j ) );
            ( i == 0 ? headgrid : wandgrid )->addWidget( m_sliders[i][j], j, 2 );
            for ( int k=0;k<2;k++ )
            {
                button = new CaveDevSimButton( i, j, k == 0 ? "Init" : "Reset",
                        i == 0 ? headWidget : wandWidget,
                        format( temp, "%s%sbutton-%d", i == 0 ? "head" : "wand",
                        k == 0 ? "Init" : "Reset", i ) );
                button->setMaximumSize( 40, 30 );
                button->setAutoDefault( false );
                ( i == 0 ? headgrid : wandgrid )->addWidget( button, j, 3 + k );
                connect( button, SIGNAL( pressed() ), button,
                        k == 0 ? SLOT( initSlider() ) : SLOT( resetSlider() ) );
                connect( button, k == 0 ? SIGNAL( init( int, int ) ) : SIGNAL( reset( int, int ) ),
                        this, k == 0 ? SLOT( initSlider( int, int ) ) : SLOT( resetSlider( int, int ) ) );
            }
            connect( m_sliders[i][j], SIGNAL( valueChanged( int ) ), m_trackvals[i][j],
                    ( j < 3 ? SLOT( sliderToXYZ( int ) ) : SLOT( sliderToHPR( int ) ) ) );
            connect( m_sliders[i][j], SIGNAL( valueChanged( int ) ), this,
                    ( i == 0 ? SLOT( updateHeadTrackEvent( int ) ) : SLOT( updateWandTrackEvent( int ) ) ) );
        }
        if ( i == 0 )
        {
            pos = m_headval.pos;
            dir = m_headval.dir;
        }
        else if ( i == 1 )
        {
            pos = m_wandval.pos;
            dir = m_wandval.dir;
        }
        m_trackvals[i][0]->setText( format( temp, "%-5.3f", pos.x() ) );
        m_trackvals[i][1]->setText( format( temp, "%-5.3f", pos.y() ) );
        m_trackvals[i][2]->setText( format( temp, "%-5.3f", pos.z() ) );
        m_trackvals[i][3]->setText( format( temp, "%-5.1f", dir.h() ) );
        m_trackvals[i][4]->setText( format( temp, "%-5.1f", dir.p() ) );
        m_trackvals[i][5]->setText( format( temp, "%-5.1f", dir.r() ) );
    }
    
    QVBoxLayout* joysticklayout = new QVBoxLayout( bottom );
    label = new QLabel( "Joystick", this, "joysticklabel" );
    label->setAlignment( AlignHCenter | AlignTop );
    joysticklayout->addWidget( label );
    QCanvas* joystickcanvas = new QCanvas( this, "joystickcanvas" );
    m_joystickview = new CaveDevSimCanvasView( joystickcanvas, this, "joystickview" );
    m_joystickview->setFixedSize( JOYSTICK_GRID_X + 5, JOYSTICK_GRID_Y + 5 );
    joysticklayout->addWidget( m_joystickview );
    
    QVBoxLayout* buttonslayout = new QVBoxLayout( bottom );
    label = new QLabel( "Buttons", this, "buttonslabel" );
    label->setAlignment( AlignHCenter | AlignTop );
    buttonslayout->addWidget( label );
    for ( i=0;i<4;i++ )
    {
        m_buttons[i] = new CaveDevSimButton( i, format( temp, "%d", i + 1 ),
                this, format( temp, "button-%d", i ) );
        m_buttons[i]->setToggleButton( true );
        m_buttons[i]->setAutoDefault( false );
        m_buttons[i]->setMaximumSize( 50, 20 );
        buttonslayout->addWidget( m_buttons[i] );
        connect( m_buttons[i], SIGNAL( toggled( bool ) ),
                m_buttons[i], SLOT( updateButtons( bool ) ) );
        connect( m_buttons[i], SIGNAL( buttonToggled( int, bool ) ),
                this, SLOT( updateWandButtonEvent( int, bool ) ) );
    }

    QVBoxLayout* resetinitjoystickbtns = new QVBoxLayout( buttonslayout );
    button = new CaveDevSimButton( "Reset Joystick", this, "resetjoystickbutton" );
    button->setAutoDefault( false );
    resetinitjoystickbtns->addWidget( button );
    connect( button, SIGNAL( pressed() ), this, SLOT( resetJoystick() ) );
    button = new CaveDevSimButton( "Init Joystick", this, "initjoystickbutton" );
    button->setAutoDefault( false );
    resetinitjoystickbtns->addWidget( button );
    connect( button, SIGNAL( pressed() ), this, SLOT( initJoystick() ) );

    QHBoxLayout* joystickxlayout = new QHBoxLayout( buttonslayout );
    label = new QLabel( "X:", this, "joystickxlabel" );
    joystickxlayout->addWidget( label );
    m_joysticklineedit[0] = new QLineEdit( this, "joystickx" );
    validator = new QDoubleValidator( -1.0f, 1.0f, 3,
            m_joysticklineedit[0], "joystickxvalidator" );
    m_joysticklineedit[0]->setValidator( validator );
    m_joysticklineedit[0]->setMaximumSize( 50, 20 );
    joystickxlayout->addWidget( m_joysticklineedit[0] );

    connect( m_joysticklineedit[0], SIGNAL( textChanged( const QString & ) ),
            m_joystickview, SLOT( updateXPosition( const QString & ) ) );
    m_joysticklineedit[0]->setText( format( temp, "%-5.3f", m_joystickval[0] ) );

    QHBoxLayout* joystickylayout = new QHBoxLayout( buttonslayout );
    label = new QLabel( "Y:", this, "joystickylabel" );
    joystickylayout->addWidget( label );
    m_joysticklineedit[1] = new QLineEdit( this, "joysticky" );
    validator = new QDoubleValidator( -1.0f, 1.0f, 3,
            m_joysticklineedit[1], "joystickyvalidator" );
    m_joysticklineedit[1]->setValidator( validator );
    m_joysticklineedit[1]->setMaximumSize( 50, 20 );
    joystickylayout->addWidget( m_joysticklineedit[1] );

    connect( m_joysticklineedit[1], SIGNAL( textChanged( const QString & ) ),
            m_joystickview, SLOT( updateYPosition( const QString & ) ) );
    m_joysticklineedit[1]->setText( format( temp, "%-5.3f", m_joystickval[1] ) );

    connect( m_joystickview, SIGNAL( joystickChanged( float, float ) ),
            this, SLOT( updateJoystickLineEdit( float, float ) ) );
    connect( m_joystickview, SIGNAL( joystickChanged() ),
            this, SLOT( updateWandJoystickckEvent() ) );

    m_springstick = new QCheckBox( "Spring Stick", this, "springstick" );
    buttonslayout->addWidget( m_springstick );
    connect( m_springstick, SIGNAL( toggled( bool ) ),
            m_joystickview, SLOT( updateSpringStick( bool ) ) );

    EventManager::addTrackListener( m_HSegment, this );
    EventManager::addWandListener( this );
    Tracer::debug( "caveDeviceSimulatorWidget - constructed" );
}

caveDeviceSimulatorWidget::~caveDeviceSimulatorWidget()
{
    EventManager::removeTrackListener( m_HSegment, this );
    EventManager::removeWandListener( this );
    emit widgetClosed();
}

void caveDeviceSimulatorWidget::resetAll()
{
    resetJoystick();
    
	int i;
    for ( i=0;i<2;i++ )
    {
        for ( int j=0;j<6;j++ )
            resetSlider( i, j );
    }
    for ( i=0;i<4;i++ )
    {
        resetButton( -1, i );;
    }
}

void caveDeviceSimulatorWidget::initAll()
{
    initJoystick();
    
	int i;
    for ( i=0;i<2;i++ )
    {
        for ( int j=0;j<6;j++ )
            initSlider( i, j );
    }
    for ( i=0;i<4;i++ )
    {
        initButton( -1, i );
    }
}

void caveDeviceSimulatorWidget::resetSlider( int headwand, int num )
{
    if( headwand < 0 || headwand > 1 )
    {
        Tracer::debug( "Attempt to reset non-existent tracker" );
        return;
    }
    
    float val;
    switch ( num )
    {
        case 0:
            val = ( headwand == 0 ? m_headresetval : m_wandresetval ).pos.x();
            break;
        case 1:
            val = ( headwand == 0 ? m_headresetval : m_wandresetval ).pos.y();
            break;
        case 2:
            val = ( headwand == 0 ? m_headresetval : m_wandresetval ).pos.z();
            break;
        case 3:
            val = ( headwand == 0 ? m_headresetval : m_wandresetval ).dir.h();
            break;
        case 4:
            val = ( headwand == 0 ? m_headresetval : m_wandresetval ).dir.p();
            break;
        case 5:
            val = ( headwand == 0 ? m_headresetval : m_wandresetval ).dir.r();
            break;
        default:
            Tracer::debug( "Attempt to reset non-existent slider" );
            return;
    }
    int sliderval;
    if ( num < 3 )
        sliderval = (int)( (float)SLIDER_DIVISIONS * val );
    else
        sliderval = (int)( (float)SLIDER_DIVISIONS * val / 180.0f );
    m_sliders[headwand][num]->setValue( sliderval );
}

void caveDeviceSimulatorWidget::initSlider( int headwand, int num )
{
    if( headwand < 0 || headwand > 1 )
    {
        Tracer::debug( "Attempt to init non-existent tracker" );
        return;
    }
    
    float val;
    switch ( num )
    {
        case 0:
            val = ( headwand == 0 ? m_headinitval : m_wandinitval ).pos.x();
            break;
        case 1:
            val = ( headwand == 0 ? m_headinitval : m_wandinitval ).pos.y();
            break;
        case 2:
            val = ( headwand == 0 ? m_headinitval : m_wandinitval ).pos.z();
            break;
        case 3:
            val = ( headwand == 0 ? m_headinitval : m_wandinitval ).dir.h();
            break;
        case 4:
            val = ( headwand == 0 ? m_headinitval : m_wandinitval ).dir.p();
            break;
        case 5:
            val = ( headwand == 0 ? m_headinitval : m_wandinitval ).dir.r();
            break;
        default:
            Tracer::debug( "Attempt to init non-existent slider" );
            return;
    }
    int sliderval;
    if ( num < 3 )
        sliderval = (int)( (float)SLIDER_DIVISIONS * val );
    else
        sliderval = (int)( (float)SLIDER_DIVISIONS * val / 180.0f );
    m_sliders[headwand][num]->setValue( sliderval );
}

void caveDeviceSimulatorWidget::resetJoystick()
{
    char temp[20];
    for ( int i=0;i<2;i++ )
    {
        m_joystickval[i] = m_joystickresetval[i];
        m_joysticklineedit[i]->setText( format( temp, "%-5.3f", m_joystickval[i] ) );
    }
}

void caveDeviceSimulatorWidget::initJoystick()
{
    char temp[20];
    for ( int i=0;i<2;i++ )
    {
        m_joystickval[i] = m_joystickinitval[i];
        m_joysticklineedit[i]->setText( format( temp, "%-5.3f", m_joystickval[i] ) );
    }
}

void caveDeviceSimulatorWidget::resetButton( int headwand, int num )
{
    if ( headwand != -1 )
    {
        Tracer::debug( "Attempt to reset non-existent button" );
        return;
    }
    m_buttonsval[num] = m_buttonsresetval[num];
    m_buttons[num]->setOn( m_buttonsval[num] );
}

void caveDeviceSimulatorWidget::initButton( int headwand, int num )
{
    if ( headwand != -1 )
    {
        Tracer::debug( "Attempt to reset non-existent button" );
        return;
    }
    m_buttonsval[num] = m_buttonsinitval[num];
    m_buttons[num]->setOn( m_buttonsval[num] );
}

void caveDeviceSimulatorWidget::updateMem()
{
    Tracer::debug( "Update Mem is not currently implemented" );
}
                        
void caveDeviceSimulatorWidget::updateWidget()
{
    Tracer::debug( "Update Widget is not currently implemented" );
}

void caveDeviceSimulatorWidget::help()
{
    Tracer::debug( "Help is not currently implemented" );
}

void caveDeviceSimulatorWidget::updateHeadTrackEvent( int val )
{
    if( m_headeventpassed )
        return;
    m_headval.pos.x() = m_sliders[0][0]->value() / (float)SLIDER_DIVISIONS;
    m_headval.pos.y() = m_sliders[0][1]->value() / (float)SLIDER_DIVISIONS;
    m_headval.pos.z() = m_sliders[0][2]->value() / (float)SLIDER_DIVISIONS;
    m_headval.dir.h() = m_sliders[0][3]->value() * 180.0f / (float)SLIDER_DIVISIONS;
    m_headval.dir.p() = m_sliders[0][4]->value() * 180.0f / (float)SLIDER_DIVISIONS;
    m_headval.dir.r() = m_sliders[0][5]->value() * 180.0f / (float)SLIDER_DIVISIONS;
    DTrackerEvent* evt = new DTrackerEvent( m_HSegment , m_headval.pos, m_headval.dir );
	evt->setSource( m_sourcename );
    EventManager::pushEvent( evt );
}

void caveDeviceSimulatorWidget::updateWandTrackEvent( int val )
{
    if( m_wandeventpassed )
        return;
    m_wandval.pos.x() = m_sliders[1][0]->value() / (float)SLIDER_DIVISIONS;
    m_wandval.pos.y() = m_sliders[1][1]->value() / (float)SLIDER_DIVISIONS;
    m_wandval.pos.z() = m_sliders[1][2]->value() / (float)SLIDER_DIVISIONS;
    m_wandval.dir.h() = m_sliders[1][3]->value() * 180.0f / (float)SLIDER_DIVISIONS;
    m_wandval.dir.p() = m_sliders[1][4]->value() * 180.0f / (float)SLIDER_DIVISIONS;
    m_wandval.dir.r() = m_sliders[1][5]->value() * 180.0f / (float)SLIDER_DIVISIONS;
    DWandEvent* evt = new DWandEvent( DWandEvent::TRACK, "wandtracker", m_wandval.pos, m_wandval.dir );
    evt->setNumButtons(4);
	evt->setSource( m_sourcename );
    EventManager::pushEvent( evt );
}

void caveDeviceSimulatorWidget::updateWandJoystickckEvent()
{
    if( m_wandeventpassed )
        return;
    m_joystickval[0] = m_joysticklineedit[0]->text().toFloat();
    m_joystickval[1] = m_joysticklineedit[1]->text().toFloat();
    DWandEvent* evt = new DWandEvent( DWandEvent::JOYSTICKMOVE, "wandtracker", m_wandval.pos, m_wandval.dir );
    evt->setNumButtons(4);
    evt->setJoystickPos( m_joystickval[0], m_joystickval[1] );
	evt->setSource( m_sourcename );
    EventManager::pushEvent( evt );
}

void caveDeviceSimulatorWidget::updateWandButtonEvent( int num, bool on )
{
    if( m_wandeventpassed )
        return;
    if ( num < 0 || num > 3 )
    {
        Tracer::debug( "Attempt to update non-existent wand button" );
        return;
    }
    m_buttonsval[num] = on;
    int action = on ? DWandEvent::BUTTONPRESS : DWandEvent::BUTTONRELEASE;
    DWandEvent* evt = new DWandEvent( action, "wandtracker", m_wandval.pos, m_wandval.dir );
    evt->setNumButtons(4);
    evt->setJoystickPos( m_joystickval[0], m_joystickval[1] );
    //evt->setButton( num, true );
    for (int i=0; i<4; i++)
    {
	evt->setButton(i, m_buttonsval[i]);
    }
	evt->setSource( m_sourcename );
    EventManager::pushEvent( evt );
}

void caveDeviceSimulatorWidget::updateJoystickLineEdit( float x, float y )
{
	if ( m_joystickview->getJoystickCanvasPassed() )
	{
		char temp[10];
		m_joysticklineedit[0]->setText( format( temp, "%-5.3f", x ) );
		m_joysticklineedit[1]->setText( format( temp, "%-5.3f", y ) );
		m_joystickview->setJoystickCanvasPassed( false );
	}
}

bool caveDeviceSimulatorWidget::onTrackerEvent( DTrackerEvent* evt )
{
	if ( m_sourcename == evt->getSource() )
		return false;
    if ( evt->getName() == m_HSegment )
    {
        dVec3 pos = evt->getPosition(), dir = evt->getDirection();

        m_headval.pos.x() = pos.x();
        m_headval.pos.y() = pos.y();
        m_headval.pos.z() = pos.z();
        m_headval.dir.h() = dir.h();
        m_headval.dir.p() = dir.p();
        m_headval.dir.r() = dir.r();
        for ( int i=0;i<6;i++ )
        {
            float val;
            int sliderval;
            if ( i < 3 )
            {
                val = m_headval.pos[i];
                sliderval = (int)( (float)SLIDER_DIVISIONS * val );
            }
            else
            {
                val = m_headval.dir[i-3];
                sliderval = (int)( (float)SLIDER_DIVISIONS * val / 180.0f );
            }
            m_headeventpassed = true;
            m_sliders[0][i]->setValue( sliderval );
            m_headeventpassed = false;
        }
    }
    return true;
}

bool caveDeviceSimulatorWidget::onWandEvent( DWandEvent* evt )
{
	if ( m_sourcename == evt->getSource() )
		return false;
    dVec3 pos = evt->getPosition(), dir = evt->getDirection();
	int i;
    switch ( evt->action() )
    {
        case DWandEvent::TRACK:

            m_wandval.pos.x() = pos.x();
            m_wandval.pos.y() = pos.y();
            m_wandval.pos.z() = pos.z();
            m_wandval.dir.h() = dir.h();
            m_wandval.dir.p() = dir.p();
            m_wandval.dir.r() = dir.r();
            for ( i=0;i<6;i++ )
            {
                float val;
                int sliderval;
                if ( i < 3 )
                {
                    val = m_wandval.pos[i];
                    sliderval = (int)( (float)SLIDER_DIVISIONS * val );
                }
                else
                {
                    val = m_wandval.dir[i-3];
                    sliderval = (int)( (float)SLIDER_DIVISIONS * val / 180.0f );
                }
                m_wandeventpassed = true;
                m_sliders[1][i]->setValue( sliderval );
                m_wandeventpassed = false;
            }
            break;
        case DWandEvent::JOYSTICKMOVE:
            m_joystickval[0] = evt->getJoystickX();
            m_joystickval[1] = evt->getJoystickY();
            m_wandeventpassed = true;
            updateJoystickLineEdit( m_joystickval[0], m_joystickval[1] );
            m_wandeventpassed = false;
            break;
        case DWandEvent::BUTTONPRESS:
            for ( i=0;i<4;i++ )
            {
                if ( evt->getButton( i ) )
                {
                    m_buttonsval[i] = true;
                    m_wandeventpassed = true;
                    m_buttons[i]->setDown( true );
                    m_wandeventpassed = false;
                }
            }
            break;
        case DWandEvent::BUTTONRELEASE:
            for ( i=0;i<4;i++ )
            {
                if ( evt->getButton( i ) )
                {
                    m_buttonsval[i] = false;
                    m_wandeventpassed = true;
                    m_buttons[i]->setDown( false );
                    m_wandeventpassed = false;
                }
            }
            break;
    }
    return false;
}
