#include"EventMsg.h"
#include<eventmgr/devents.h>
#include<eventmgr/EventFactory.h>
#include<pexpr/POMessageFactory.h>
#include<tracer/Tracer.h>

RegisterMessage<EventMsg> eventmsg;

//=============================================================================
/// @brief Constructor.
/// @param evt - the event to be sent as a message.
//=============================================================================
EventMsg::EventMsg( const DEvent* evt )
    :POMessage()
{
    if( evt )
        m_evt = evt->clone();
    else
        m_evt = NULL;
}

//=============================================================================
/// @brief Destructor.
//=============================================================================
EventMsg::~EventMsg()
{
    if( m_evt ) delete m_evt;
}

//=============================================================================
/// @brief Copy constructor.
/// @param cpy - the vent message being copied.
//=============================================================================
EventMsg::EventMsg( const EventMsg& cpy )
    :POMessage( (POMessage)cpy )
{
    if( cpy.m_evt )
        m_evt = cpy.m_evt->clone();
    else
        m_evt = NULL;
}

//=============================================================================
/// @brief returns the packsize of the event.
/// @return the number of bytes required to pack the event.
//=============================================================================
int EventMsg::packsize()const
{
    int size = POMessage::packsize();
    if( !m_evt )
    {
        Tracer::error( "EventMsg::packsize() - No valid event, can't pack this message" );
//        int* tmp = NULL;
//        int x = *tmp;
        exit(1);
    }
    size += m_evt->packsize();
    return size;
}

//=============================================================================
/// @brief packs the event into the buffer.
/// @param buffer - the buffer into which the event will
/// be packed. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes packed into the buffer.
//=============================================================================
int EventMsg::pack( char* buffer )const
{
    char* tmp = buffer;
    tmp += POMessage::pack(tmp);
    tmp += m_evt->pack(tmp);
    return tmp - buffer;
}

//=============================================================================
/// @brief unpacks the buffer into the event.
/// @param buffer - the buffer from which the event will
/// be unpacked. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes unpacked from the buffer.
//=============================================================================
int EventMsg::unpack( char* buffer )
{
    if( m_evt ) delete m_evt;
    char* tmp = buffer;
    tmp += POMessage::unpack(tmp);
    int size;
    m_evt = EventFactory::buildFromBuffer( tmp, &size );
    if( m_evt == NULL ) Tracer::error("EventMsg::unpack() - NULL event");
    tmp += size;
    return tmp - buffer;
}

//=============================================================================
/// @brief clones the EventMsg.
/// @return pointer to a POMessage where the cloned EventMsg resides.
//=============================================================================
POMessage* EventMsg::clone()const
{
    return (POMessage*)new EventMsg(*this);
}

//=============================================================================
/// @brief Get the event object associated with the event message.
/// @return a pointer to the event.
//=============================================================================
DEvent* EventMsg::getEvent()const
{
    return m_evt;
}
