#ifndef _EVENTSERVER_H_
#define _EVENTSERVER_H_
#include<servicemgr/GService.h>
#include<eventmgr/EventListener.h>
#include<pexpr/POAddress.h>
#include<dutil/ColonList.h>
#include<map>
#include<vector>

#ifdef WIN32
#ifdef REVENT_EXPORTS
#define REVENT_API __declspec(dllexport)
#else
#define REVENT_API __declspec(dllimport)
#endif /* REVENT_EXPORTS */
#else
#define REVENT_API
#endif /* WIN32 */

using namespace std;

class POBox;
class PostOffice;
class POMessage;
class EventMsg;

//=============================================================================
/// @class EventServer
/// @author Daniel Larimer
/// @brief Runs in the DIVERSE Daemon.
//=============================================================================
class REVENT_API EventServer : public GService, public EventListener
{
    public:
        EventServer();

        bool onAnyEvent( DEvent* evt );
        
        void addConnection( const POAddress&, const ColonList& param );
        void removeConnection( const POAddress&, const ColonList& param );
        void addRemoteConnection( const ColonList& param );
        void removeRemoteConnection( const ColonList& param );
        
        void postProcess();

        void handleMessage( const POMessage* msg );
        void handleEventMsg( const EventMsg* msg );

        // Turn off warnings
        void preConfig(){};
        void config(){};
        void postConfig(){};

    private:
        PostOffice*         m_po;
        POBox*              m_box;  // receives messages 

        int                 m_servicept;

        /// Maps event type to the list of people connected 
        /// to receive that kind of event
        map<string, vector<POAddress> > m_exports;

        /// Maps the host name to the PostOffice name
        map<string, string>             m_hostmap;

};


#endif
