#ifndef _REVENTCLIENT_H_
#define _REVENTCLIENT_H_
#include<servicemgr/GService.h>
#include<eventmgr/EventListener.h>
#include<pexpr/POAddress.h>
#include<string>

#ifdef WIN32
#ifdef REVENT_EXPORTS
#define REVENT_API __declspec(dllexport)
#else
#define REVENT_API __declspec(dllimport)
#endif /* REVENT_EXPORTS */
#else
#define REVENT_API
#endif /* WIN32 */

using namespace std;

class POBox;
class PostOffice;
class POMessage;
class EventMsg;

//=============================================================================
/// @class REventClient
/// @author Daniel Larimer
/// @brief Connects to the event daemon and sends/receives events.
///
/// A static object which handles remote events. To initialize, call
/// REventClient::instance.
//=============================================================================
class REVENT_API REventClient : public GService, public EventListener
{
    public:
        // Get Local Events
        bool onAnyEvent( DEvent* );

        // Check for messages
        void postProcess();

        // parse messages
        void handleMessage( const POMessage* msg );

        // Push Events from message
        void handleEventMsg( const EventMsg* msg );

        static REventClient* instance(); 

        static void  exportEvent( const string& type, bool state );

        static void  importEvent( const string& type, bool state );

        static void  importRemoteEvents( const string& host,
                                         const string& etype,
                                         bool state );

        static void  exportRemoteEvents( const string& host,
                                         const string& etype,
                                         bool state );

    private:
        REventClient();
        static REventClient*       m_instance;

        static PostOffice*         m_po;
        static POAddress           m_server; // the address of the DAEMON
        static POBox*              m_box;

        static map<string, bool>   m_exports; // Events we are exporting
        static map<string, bool>   m_imports; // Events we are importing

        static int                 m_servicept; // The port the EventServer is running on

};


#endif
