#include"GService.h"
#include"GServiceManager.h"

//=============================================================================
/// @brief Constructor.
/// @param name - the name os the service.
//=============================================================================
GService::GService( const string& name )
:m_servicename(name), m_running(false)
{
    m_priority = 1;
}

//=============================================================================
/// @brief Called during the preConfig stage.
//=============================================================================
void GService::preConfig()
{
    Tracer::warn(10, "GService::preConfig() wasn't overloaded for %s",
                  getServiceName().c_str());
}

//=============================================================================
/// @brief Called during the config stage.
//=============================================================================
void GService::config()
{
    Tracer::warn(10, "GService::config() wasn't overloaded for %s",
                  getServiceName().c_str());
}

//=============================================================================
/// @brief Called during the postConfig stage.
//=============================================================================
void GService::postConfig()
{
    Tracer::warn(10, "GService::postConfig() wasn't overloaded for %s",
                  getServiceName().c_str());
}

//=============================================================================
/// @brief Called during the postProcess stage.
//=============================================================================
void GService::postProcess()
{
}

//=============================================================================
/// @brief Starts the service by setting the priority to a positive value.
/// @return always returns true.
//=============================================================================
bool GService::start()
{
    setPriority( abs( getPriority() ) );
    m_running = true;
    return true;
}

//=============================================================================
/// @brief Stops the service by setting the priority to a negative value.
/// @return always returns true.
//=============================================================================
bool GService::stop()
{
    setPriority( -abs( getPriority() ) );
    m_running = false;
    return true;
}

//=============================================================================
/// @brief Return the state of the service.
/// @return true if the service is running, else false.
//=============================================================================
bool GService::isRunning()const
{
    return getPriority() >= 1;
}

//=============================================================================
/// @brief Get the name of the service
//=============================================================================
string GService::getServiceName()const
{
    return m_servicename;
}

//=============================================================================
/// @brief Sets the priority for the service.
/// @param p - the priority level.
/// @param tell_servicemgr - set to true if the servicemgr should change the
/// level at which the service is running, or set to false to ignore the new
/// priority. The default value is true.
//=============================================================================
void GService::setPriority( int p, bool tell_servicemgr )
{
    m_priority = p;
    if( tell_servicemgr )
        GServiceManager::updateSchedule();
}
