#ifndef _CCLIENT_H_
#define _CCLIENT_H_
#include<pexpr/POAddress.h>

#include<qobject.h>
#include<string>
#include<vector>
#include<map>
using namespace std;

#ifdef WIN32
#ifdef SHMMGR_EXPORTS
#define SHMMGR_API __declspec(dllexport)
#else
#define SHMMGR_API __declspec(dllimport)
#endif /* SHMMGR_EXPORTS */
#else
#define SHMMGR_API
#endif /* WIN32 */

class CSSegmentListMsg;
class PostOffice;
class ShMem;
class MemorySegment;
class POBox;
class POMessage;
class POErrorMsg;
class POTextMsg;

//=============================================================================
/// @class CClient CClient.h
/// @author Daniel Larimer
/// @brief Interfaces with CommuneServer.
//=============================================================================
class SHMMGR_API CClient : public QObject
{
    Q_OBJECT
    public:
        CClient( const string& cname, PostOffice* po, const POAddress& server );
        CClient( const string& cname, const string& hostname = "localhost", 
                                      int port = 9000 );

        ShMem*              getSegment( const string& name, 
                                        bool create = false,
                                        int usec_timeout = 100000 );
        
        int  mapSegment( const string& host, int port, 
                         const string& rsegment, const string& lsegment,
                         int usec_timeout = 100000 );
        int  unmapSegment( const string& host, const string& rsegment,
                           const string& lsegment, int usec_timeout = 100000 );

        vector<string>      getSegmentNames();
        map<string, ShMem*> getConnectedSegments();

        bool           createSegment( const string& name, int size, 
                                      int usec_timeout = 100000 );
        bool           removeSegment( const string& name,
                                      int usec_timeout = 100000 );

    public slots:
        void           handleMessage();
        void           handleSegmentListMsg( CSSegmentListMsg* );
        void           handleErrorMsg( POErrorMsg* );
        void           handleTextMsg( POTextMsg* );

    signals:
        void newSegment( const string& name );

    private:
        vector<string>      m_segnames;
        map<string, ShMem*> m_segs;
        POAddress           m_serveradr;
        PostOffice*         m_po;
        POBox*              m_mbox;

        enum WAITSTAT{ SM_INACTIVE, SM_WAIT, SM_OK, SM_ERROR };
        int                 m_getstat;
        int                 m_createstat;
        int                 m_mapstat;
        int                 m_remstat;
};

#endif
