#include"CSCreateSegmentMsg.h"
#include<pexpr/POMessageFactory.h>

RegisterMessage<CSCreateSegmentMsg> cscreatesegmentmsg;

//=============================================================================
/// @brief Constructor.
/// @param name - the name of the segment to be created.
/// @param size - the size of the segment to be created.
//=============================================================================
CSCreateSegmentMsg::CSCreateSegmentMsg( const string& name, int size )
:POMessage(), m_sname(name)
{
    m_size  = size;
}

//=============================================================================
/// @brief Copy constructor.
/// @param cpy - the CSCreateSegmentMsg to be copied.
//=============================================================================
CSCreateSegmentMsg::CSCreateSegmentMsg( const CSCreateSegmentMsg& cpy )
:POMessage( (POMessage)cpy ), m_sname(cpy.m_sname), m_size(cpy.m_size)
{
    
}

//=============================================================================
/// @brief Destructor.
//=============================================================================
CSCreateSegmentMsg::~CSCreateSegmentMsg()
{
}

//=============================================================================
/// @brief Clone the CSCreateSegmentMsg.
/// @return a POMessage pointer of the cloned CSCreateSegmentMsg. 
//=============================================================================
POMessage* CSCreateSegmentMsg::clone()const
{
    return (POMessage*) new CSCreateSegmentMsg(*this);
}

//=============================================================================
/// @brief returns the pack size of the event.
/// @return the number of bytes required to pack the event.
//=============================================================================
int CSCreateSegmentMsg::packsize()const
{
    int size = POMessage::packsize();
    size += m_sname.packsize();
    size += m_size.packsize();
    return size;
}

//=============================================================================
/// @brief packs the event into the buffer.
/// @param buffer - the buffer into which the event will
/// be packed. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes packed into the buffer.
//=============================================================================
int CSCreateSegmentMsg::pack( char* buffer )const
{
    char* tmp = buffer;
    tmp += POMessage::pack(tmp);

    tmp += m_sname.pack(tmp);
    tmp += m_size.pack(tmp);

    return tmp - buffer;
}

//=============================================================================
/// @brief unpacks the buffer into the event.
/// @param buffer - the buffer from which the event will
/// be unpacked. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes unpacked from the buffer.
//=============================================================================
int CSCreateSegmentMsg::unpack( char* buffer )
{
    char* tmp = buffer;
    tmp += POMessage::unpack(tmp);

    tmp += m_sname.unpack(tmp);
    tmp += m_size.unpack(tmp);

    return tmp - buffer;
}
