#include"CSSegmentListMsg.h"
#include<pexpr/POMessageFactory.h>
#include<iostream>
using namespace std;

RegisterMessage<CSSegmentListMsg> cssegmentlistmsg;

//=============================================================================
/// @brief Constructor.
/// @param cmd - the SEGMENTCMD value representing the command. Default is STATUS.
//=============================================================================
CSSegmentListMsg::CSSegmentListMsg( int cmd )
:m_cmd(cmd)
{
}

//=============================================================================
/// @brief Copy constructor.
/// @param cpy - the CSSegmentListMsg to be copied.
//=============================================================================
CSSegmentListMsg::CSSegmentListMsg( const CSSegmentListMsg& cpy )
:POMessage((POMessage)cpy), m_cmd(cpy.m_cmd), m_segments(cpy.m_segments)
{
}

//=============================================================================
/// @brief Adds a segment to the CSSegmentListMsg segment list.
/// @param segment - the segment to add to the list.
//=============================================================================
void CSSegmentListMsg::addSegment( const string& segment )
{
    m_segments.push_back(segment);
}

//=============================================================================
/// @brief Clone the CSSegmentListMsg.
/// @return a POMessage pointer of the cloned CSSegmentListMsg. 
//=============================================================================
POMessage* CSSegmentListMsg::clone()const
{
    return new CSSegmentListMsg( *this );
}

//=============================================================================
/// @brief returns the pack size of the event.
/// @return the number of bytes required to pack the event.
//=============================================================================
int CSSegmentListMsg::packsize()const
{
    int size = POMessage::packsize();
    size += m_cmd.packsize();
    size += sizeof(int); // number of segments
    
    for( unsigned int i = 0; i < m_segments.size(); i++ )
    {
        POString str(m_segments[i]);
        size += str.packsize();
    }
    return size;
}

//=============================================================================
/// @brief packs the event into the buffer.
/// @param buffer - the buffer into which the event will
/// be packed. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes packed into the buffer.
//=============================================================================
int CSSegmentListMsg::pack( char* buffer )const
{
    char* tmp = buffer;
    tmp += POMessage::pack(tmp);
    tmp += m_cmd.pack(tmp);
    tmp += POInt(m_segments.size()).pack(tmp);
    
    for( unsigned int i = 0; i < m_segments.size(); i++ )
    {
        tmp += POString(m_segments[i]).pack(tmp);
    }
    return tmp - buffer;
}

//=============================================================================
/// @brief unpacks the buffer into the event.
/// @param buffer - the buffer from which the event will
/// be unpacked. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes unpacked from the buffer.
//=============================================================================
int CSSegmentListMsg::unpack( char* buffer )
{
    char* tmp = buffer;
    tmp += POMessage::unpack(tmp);
    tmp += m_cmd.unpack(tmp);
    POInt size;
    tmp += size.unpack(tmp);

    POString segment;
    for( int i = 0; i < size.val(); i++ )
    {
        tmp += segment.unpack(tmp);
        m_segments.push_back( segment );
    }
    return tmp - buffer;
}
