#ifndef _POSEGMENTLISTMSG_H_
#define _POSEGMENTLISTMSG_H_
#include<pexpr/POMessage.h>
#include<vector>

#ifdef WIN32
#ifdef SHMMGR_EXPORTS
#define SHMMGR_API __declspec(dllexport)
#else
#define SHMMGR_API __declspec(dllimport)
#endif /* SHMMGR_EXPORTS */
#else
#define SHMMGR_API
#endif /* WIN32 */

//=============================================================================
/// @class CSSegmentListMsg
/// @brief CommuneServer segment list message.
//=============================================================================
class SHMMGR_API CSSegmentListMsg : public POMessage
{
    public:
		/// @brief Segment commands.
        enum SEGMENTCMD
		{
			/// @brief Request status.
			STATUS,
			/// @brief Request new segments.
			NEWSEGMENTS,
			/// @brief Request deletion of segments.
			DELETESEGMENTS
		};
        CSSegmentListMsg( int cmd = STATUS );
        CSSegmentListMsg( const CSSegmentListMsg& cpy );
		/// @brief Destructor.
        virtual ~CSSegmentListMsg(){}

		/// @brief Get the type of message.
		/// @return a string containing the type of message - "CSSegmentListMsg".
        string messageType()const { return "CSSegmentListMsg"; }

        void addSegment( const string& segment );

		/// @brief Get a list of names of all segments.
		/// @return a vector containing a list of the names of the segments.
        vector<string> getSegments()const { return m_segments; }
		/// @brief Get the command associated with the message.
		/// @return the value of the command.
        int getCommand()const             { return m_cmd.val(); }

        POMessage* clone()const;

        int packsize()const;
        int pack( char* buffer )const;
        int unpack( char* buffer );
        
    private:
        POInt               m_cmd;
        vector<string>      m_segments;
};

#endif
