#ifndef _COMMUNESERVER_H_
#define _COMMUNESERVER_H_
#include<qobject.h>
#include<string>
#include<map>
using namespace std;

#ifdef WIN32
#ifdef SHMMGR_EXPORTS
#define SHMMGR_API __declspec(dllexport)
#else
#define SHMMGR_API __declspec(dllimport)
#endif /* SHMMGR_EXPORTS */
#else
#define SHMMGR_API
#endif /* WIN32 */

class PostOffice;
class POBox;
class MemorySegment;
class CSCreateSegmentMsg;
class CSSegmentListMsg;
class ShMem;
class CClient;
class POTextMsg;

//=============================================================================
/// @class CommuneServer
/// @author Daniel Larimer
/// @brief Serves as a repository for the commune shared memory.
//=============================================================================
class SHMMGR_API CommuneServer : public QObject
{
    Q_OBJECT
    public:
        CommuneServer( const string& name, int port );

    public slots:
        void handleMessage();
        int  mapSegment( const string& host, int port, 
                         const string& rseg, const string& lseg);
        int  unmapSegment( const string& host, const string& rsegment,
                           const string& lsegment );

        int  createSegment( const string& segname, int size );

        void forwardLocalChange( const string& lsegname, char* buffer, 
                                 int start, int size );
        void forwardRemoteChange( const string& rsegname, char* buffer,
                                  int start, int size );

    private:
        void handleCreateSegment( const CSCreateSegmentMsg* msg );
        void handleListSegments( const CSSegmentListMsg* msg );
        void handleMapMessage  ( const POTextMsg* msg );
    
        map<string, MemorySegment*> m_memory;
        
        map<string, CClient*>       m_remoteshm; /* host to client */
        map<string, ShMem*>         m_local2remote;
        map<string, MemorySegment*> m_remote2local;
    
        PostOffice* m_po;
        POBox*      m_box;
};

#endif
