#include"MSWriteMsg.h"
#include<pexpr/POMessageFactory.h>

RegisterMessage<MSWriteMsg> mswritemsg;

//=============================================================================
/// @brief Constructor.
/// @param data - a pointer to the data to be used for the MSWriteMsg.
/// @param start - the index of the start of the message.
/// @param size - the size of the data in bytes.
//=============================================================================
MSWriteMsg::MSWriteMsg( char* data, int start, int size )
:POMessage()
{
    m_size  = size;
    m_start = start;
    m_data  = new char[m_size];
    memcpy( m_data, data, m_size );
}

//=============================================================================
/// @brief Copy constructor.
/// @param cpy - the MSWriteMsg to be copied.
//=============================================================================
MSWriteMsg::MSWriteMsg( const MSWriteMsg& cpy )
:POMessage( (POMessage)cpy )
{
    m_size  = cpy.m_size;
    m_start = cpy.m_start;
    m_data  = new char[m_size];
    memcpy( m_data, cpy.m_data, m_size );
}

//=============================================================================
/// @brief Destructor.
//=============================================================================
MSWriteMsg::~MSWriteMsg()
{
    if( m_data && m_size > 0 )
        delete[] m_data;
}

//=============================================================================
/// @brief Clone the MSWriteMsg.
/// @return a POMessage pointer of the cloned MSWriteMsg. 
//=============================================================================
POMessage* MSWriteMsg::clone()const
{
    return (POMessage*) new MSWriteMsg(*this);
}

//=============================================================================
/// @brief returns the pack size of the event.
/// @return the number of bytes required to pack the event.
//=============================================================================
int MSWriteMsg::packsize()const
{
    int size = POMessage::packsize();
    size += m_start.packsize();
    size += m_size.packsize();
    size += m_size.val();
    return size;
}

//=============================================================================
/// @brief packs the event into the buffer.
/// @param buffer - the buffer into which the event will
/// be packed. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes packed into the buffer.
//=============================================================================
int MSWriteMsg::pack( char* buffer )const
{
    char* tmp = buffer;
    tmp += POMessage::pack(tmp);

    tmp += m_start.pack(tmp);
    tmp += m_size.pack(tmp);

    memcpy( tmp, m_data, m_size.val() );
    tmp += m_size.val();
    return tmp - buffer;
}

//=============================================================================
/// @brief unpacks the buffer into the event.
/// @param buffer - the buffer from which the event will
/// be unpacked. This is used for sending and receiving
/// events as messages.
/// @return the number of bytes unpacked from the buffer.
//=============================================================================
int MSWriteMsg::unpack( char* buffer )
{
    char* tmp = buffer;
    tmp += POMessage::unpack(tmp);

    tmp += m_start.unpack(tmp);
    tmp += m_size.unpack(tmp);

    if( m_size != 0 )
    {
        m_data = new char[m_size];
        memcpy( m_data, tmp, m_size );
        tmp += m_size;
    }
    else
        m_data = NULL;

    return tmp - buffer;
}
