#ifndef _MSWRITEMSG_H_
#define _MSWRITEMSG_H_
#include<pexpr/POMessage.h>
#include<pexpr/POTypes.h>

#ifdef WIN32
#ifdef SHMMGR_EXPORTS
#define SHMMGR_API __declspec(dllexport)
#else
#define SHMMGR_API __declspec(dllimport)
#endif /* SHMMGR_EXPORTS */
#else
#define SHMMGR_API
#endif /* WIN32 */

//=============================================================================
/// @class MSWriteMsg
/// @brief A memory segment write message for sending remotely.
//=============================================================================
class SHMMGR_API MSWriteMsg : public POMessage
{
    public:
        MSWriteMsg( char* data = NULL, int start = 0, int size = 0);
        MSWriteMsg( const MSWriteMsg& cpy );
        virtual ~MSWriteMsg();

		/// @brief Get the type of POMessage.
		/// @return a string containing the type of message - "MSWriteMsg".
        virtual string messageType()const { return "MSWriteMsg"; };
        virtual POMessage* clone()const;

		/// @brief Get a pointer to the message data.
		/// @return a pointer to the message data.
        char* getData()const       { return m_data;        }
		/// @brief Get the index of the start of the message.
		/// @return the index of the start of the message.
        int   getStartIndex()const { return m_start.val(); }
		/// @brief Get the size of the message.
		/// @return the size of the message.
        int   getSize()const       { return m_size.val();  }

        //==========================
        // Packable
        //==========================
        int packsize()const;
        int pack( char* buf )const;
        int unpack( char* buf );

    private:
        char*  m_data;
        POInt  m_start;
        POInt  m_size;
};


#endif
