#ifndef _SHMEM_H_
#define _SHMEM_H_
#include<qobject.h>
#include<vector>
#include<string>
#include<pexpr/POAddress.h>

using namespace std;

#ifdef WIN32
#ifdef SHMMGR_EXPORTS
#define SHMMGR_API __declspec(dllexport)
#else
#define SHMMGR_API __declspec(dllimport)
#endif /* SHMMGR_EXPORTS */
#else
#define SHMMGR_API
#endif /* WIN32 */

class PostOffice;
class POBox;
class POTextMsg;
class MSWriteMsg;

//=============================================================================
/// @brief Encapsulates the shared memory segment.
//=============================================================================
class SHMMGR_API ShMem : public QObject
{
    Q_OBJECT
    public:
        ShMem( PostOffice* po, const POAddress& segaddr );
        virtual ~ShMem();

        string name()const;
        int    size()const;
    public slots:
        void handleMessage();

        //=====================================
        // ShMem Interface
        //=====================================
        int  read ( void* );
        void write( void* );
        void read ( char* buffer, int start, int size )const;
        void write( char* buffer, int start, int size );
        void resize( int size );
        //=====================================
        
    signals:
        void changed( const string& name, char* buffer, int start, int size );
        void changed();
    
    private:
        void handleWrite( const MSWriteMsg* msg );
        POAddress m_segaddr;

        char*  m_memory;
        int    m_size;
        POBox* m_box;

        POAddress m_addr;
};

#endif
