#include"ShmManager.h"
#include"CClient.h"
#include"MemorySegment.h"

CClient* ShmManager::m_client = NULL;

//=============================================================================
/// @brief Initialize the ShmManager static object and creates a single CClient.
/// @param servername - the name of the host.
/// @param port - the port which should be used for shared memory.
//=============================================================================
void ShmManager::init( const string& servername, int port )
{
    if( m_client ) return;
    m_client = new CClient( "ShmManager", servername, port );
}

//=============================================================================
// Function Name: getSegment
/// @brief Returns a pointer to the memory segment.
/// @param name - the name of the memory segment.
/// @param usec_timeout - the number of milliseconds to wait before timing out.
/// @return the pointer to the segment, NULL if no segment was found.
//=============================================================================
ShMem* ShmManager::getSegment( const string& name, int usec_timeout )
{
    return (ShMem*)m_client->getSegment( name, usec_timeout );
}

//=============================================================================
// Function Name: createSegment
/// @brief Creates a new shared memory segment of the specified size and name
/// @param segname - the name of the memory segment.
/// @param size - the size of the segment to create.
/// @param usec_timeout - the number of milliseconds to wait before timing out.
/// @return true if the segment could be created.
//=============================================================================
bool ShmManager::createSegment( const string& segname, int size, 
                                int usec_timeout )
{
    return m_client->createSegment( segname, size, usec_timeout );
}

//=============================================================================
/// @brief Removes a previously created shared memory segment.
/// @param segname - the name of the memory segment.
/// @param usec_timeout - the number of milliseconds to wait before timing out.
/// @return true if the segment was removed.
//=============================================================================
bool ShmManager::removeSegment( const string& segname, int usec_timeout )
{
    return m_client->removeSegment( segname, usec_timeout );
}

//=============================================================================
/// @brief Sends a message to the CommuneServer telling it to map a segment
/// from a remote CommuneServer to the local one.
/// @param host - the name of the host.
/// @param port - the port  which the local CommuneServer is using.
/// @param rseg - the name of the remote memory segment.
/// @param lseg - the name of the local memory segment.
/// @param usec_timeout - the number of milliseconds to wait before timing out.
//=============================================================================
bool ShmManager::mapSegment( const string& host, int port, const string& rseg,
                             const string& lseg, int usec_timeout )
{
    return m_client->mapSegment( host, port, rseg, lseg, usec_timeout );
}
