#ifndef _SHMMANAGER_H_
#define _SHMMANAGER_H_
#include<string>
using namespace std;

#ifdef WIN32
#ifdef SHMMGR_EXPORTS
#define SHMMGR_API __declspec(dllexport)
#else
#define SHMMGR_API __declspec(dllimport)
#endif /* SHMMGR_EXPORTS */
#else
#define SHMMGR_API
#endif /* WIN32 */

class CClient;
class ShMem;
class POTextMsg;
class POErrorMsg;

//=============================================================================
/// @class ShmManager
/// @author Daniel Larimer
/// @brief Provides global access to shared memory.
///
/// The shared memeory manager is a static object. It should be initialized prior
/// to calling any of its functions by calling ShmManager::init.
//=============================================================================
class SHMMGR_API ShmManager
{
    public:
        static void init(const string& servername = "localhost", 
                         int port = 9200 );

        static ShMem* getSegment( const string& segname, 
                                  int usec_timeout = 100000 );
        static bool   createSegment( const string& segname, int size,
                                  int usec_timeout = 100000 );
        static bool   removeSegment( const string& segname, 
                                  int usec_timeout = 100000 );
        static bool   mapSegment( const string& host, int port, 
                                  const string& remoteseg, 
                                  const string& localseg,
                                  int usec_timeout = 100000 );
		/// @brief Get a pointer to a CClient object containing information about
		/// the client.
		/// @return a pointer to the client.
        static CClient* getClient(){ return m_client; }
    private:
		/// @brief Private constructor.
        ShmManager(){};
        static CClient*    m_client;
};

#endif
