#ifndef _DSOUNDNODE_H_
#define _DSOUNDNODE_H_
#include<string>
#include<fmod.h>
#include<fmod_errors.h>
#include<dutil/DTime.h>
#include<dmath/dMatrix.h>
#include<dmath/dVec3.h>

#ifdef WIN32
#ifdef SOUNDMGR_EXPORTS
#define SOUNDMGR_API __declspec(dllexport)
#else
#define SOUNDMGR_API __declspec(dllimport)
#endif /* SOUNDMGR_EXPORTS */
#else
#define SOUNDMGR_API
#endif /* WIN32 */

using namespace std;

//=============================================================================
/// @class DSoundNode
/// @author Daniel Larimer
/// @brief Encapsulates a sound in 3D space.
//=============================================================================
class SOUNDMGR_API DSoundNode
{   
    public:
        DSoundNode( const string& filename, int priority = 5 );
        ~DSoundNode();

        void update();

        int  getPriority()const { return m_prio; }
        void setPriority( int p );
        
        void setPositionMatrix( const dMatrix& mat, bool update = false );
        void setPosition( const dVec3& pos = dVec3(),
                          const dVec3& dir = dVec3(0,1,0),
                          const dVec3& up = dVec3(0,0,1), bool update = false);

        dVec3 getPosition()const { return m_curpos; }
        dVec3 getDirection()const{ return m_curdir; }
        dVec3 getUp()const       { return m_curup;  }
        
        string getFileName()const{ return m_fname;  }

        void play();
        void pause();
        bool isPlaying();
        void setVolume( int vol );

        void setMinMaxDistance( float min = 4.0f, float max = 1000.0 );

    private:
        dVec3 m_lastpos;
        dVec3 m_lastdir;
        dVec3 m_lastup;

        dVec3 m_curpos;
        dVec3 m_curdir;
        dVec3 m_curup;
        dVec3 m_vel;

        DTime m_lastupdate;

        int            m_volume;
        int            m_channel;
        FSOUND_SAMPLE* m_sndptr;

        float         m_mindist;
        float         m_maxdist;

        int           m_prio;
        string        m_fname;
};

#endif
