#include"SoundManager.h"
#include"DSoundNode.h"
#include<dreg/Registry.h>
#include<fmod/fmod.h>
#include<fmod/fmod_errors.h>
#include<tracer/Tracer.h>
#include<servicemgr/GServiceManager.h>
#include<navmgr/NavManager.h>

bool                SoundManager::m_setup = false;
bool                SoundManager::m_init  = false;
vector<DSoundNode*> SoundManager::m_sounds;
DTime               SoundManager::m_timer;
dVec3               SoundManager::m_listenvel;
dVec3               SoundManager::m_listendir;
dVec3               SoundManager::m_listenpos;
SoundManager*       SoundManager::m_self = NULL;

//=============================================================================
/// @brief Initialize and instantiate the SoundManager static object.
//=============================================================================
void SoundManager::init()
{
    if( m_init ) return;
    m_self = new SoundManager;
    m_init = true;
    m_self->setupRegistry();
}

//=============================================================================
/// @brief Private constructor.
//=============================================================================
SoundManager::SoundManager(): GService("SoundService")
{
	GServiceManager::addService(this);
    string priority = Registry::getValue("/DIVERSE/DSound/ServicePriority", "11");
    setPriority(atoi(priority.c_str()), false);
}

//=============================================================================
/// @brief Called during the config stage of the application cycle to configure
/// the sound manager.
//=============================================================================
void SoundManager::config()
{
    
    if( FSOUND_GetVersion() < FMOD_VERSION )
    {
        Tracer::error( "You are using fmod version %.02f, but SoundManager was\n"
                        "\tcompiled with version %.02f", FSOUND_GetVersion(), 
                        FMOD_VERSION );
        return;
    }

    string drv = Registry::getValue( "/DIVERSE/SoundManager/SoundSystem" );
    if( drv == "OSS" )
        FSOUND_SetOutput( FSOUND_OUTPUT_OSS );
    else if( drv == "ESD" )
        FSOUND_SetOutput( FSOUND_OUTPUT_ESD );
    else if( drv == "ALSA" )
        FSOUND_SetOutput( FSOUND_OUTPUT_ALSA );
    else if( drv == "WINMM" )
        FSOUND_SetOutput( FSOUND_OUTPUT_WINMM );
    else if( drv == "DSOUND" )
        FSOUND_SetOutput( FSOUND_OUTPUT_DSOUND );
    else
    {
        Tracer::warn("SoundManager - No Sound!");
        FSOUND_SetOutput( FSOUND_OUTPUT_NOSOUND );
    }
    // give fmod 1mb of memory and it will be happy, but we will give it 4
//    if( !FSOUND_SetMemorySystem( new char[4*1024*1024], 4*1024*1024, NULL, NULL, NULL ) )
//    {
//        Tracer::error("SoundManager::init()\n\t\t%s", FMOD_ErrorString( FSOUND_GetError() ) );
//    }

    // select the driver

    Tracer::debug(" Driver list");    
    Tracer::debug("---------------------------------------------------------");    

    for (int i=0; i < FSOUND_GetNumDrivers(); i++) 
    {
        Tracer::debug("%d - %s\n", i+1, FSOUND_GetDriverName(i));    /* print driver names */
    }
    Tracer::debug("---------------------------------------------------------");    
    
    // hard code the first driver, make this nicer later
    if( FSOUND_GetNumDrivers() <= 0 )
    {
        Tracer::error( "No sound drivers found, sound will not be available" );
        return;
    }
    FSOUND_SetDriver(0); 
    Tracer::stat( 3, "Using driver: %s", FSOUND_GetDriverName(0) );
    {
        unsigned int caps = 0;

        FSOUND_GetDriverCaps(FSOUND_GetDriver(), &caps);
        
        Tracer::stat( 3,"---------------------------------------------------------");    
        Tracer::stat( 3,"Driver capabilities");
        Tracer::stat( 3,"---------------------------------------------------------");    
        if (!caps)
            Tracer::stat( 3,"- This driver will support software mode only.\n  It does not properly support 3D sound hardware.");
        if (caps & FSOUND_CAPS_HARDWARE)
            Tracer::stat( 3,"- Driver supports hardware 3D sound!");
        if (caps & FSOUND_CAPS_EAX2)
            Tracer::stat( 3,"- Driver supports EAX 2 reverb!");
        if (caps & FSOUND_CAPS_EAX3)
            Tracer::stat( 3,"- Driver supports EAX 3 reverb!");
        Tracer::stat( 3,"---------------------------------------------------------");    
    }
    FSOUND_SetMixer(FSOUND_MIXER_AUTODETECT);
   
    //=============================
    // initialize
    //=============================
    if( !FSOUND_Init( 44100, 32, 0) )
    {
        Tracer::error( "SoundManager::init()\n\t\t%s", FMOD_ErrorString( FSOUND_GetError() ) );
        return;
    }

    switch (FSOUND_GetOutput())
    {
        case FSOUND_OUTPUT_NOSOUND: Tracer::stat( 4, "FSOUND Output Method :FSOUND_OUTPUT_NOSOUND"); break;
        case FSOUND_OUTPUT_WINMM:   Tracer::stat( 4, "FSOUND Output Method :FSOUND_OUTPUT_WINMM"); break;
        case FSOUND_OUTPUT_DSOUND:  Tracer::stat( 4, "FSOUND Output Method :FSOUND_OUTPUT_DSOUND"); break;
        case FSOUND_OUTPUT_ASIO:    Tracer::stat( 4, "FSOUND Output Method :FSOUND_OUTPUT_ASIO"); break;
        case FSOUND_OUTPUT_OSS:     Tracer::stat( 4, "FSOUND Output Method :FSOUND_OUTPUT_OSS"); break;
        case FSOUND_OUTPUT_ESD:     Tracer::stat( 4, "FSOUND Output Method :FSOUND_OUTPUT_ESD"); break;
        case FSOUND_OUTPUT_ALSA:    Tracer::stat( 4, "FSOUND Output Method :FSOUND_OUTPUT_ALSA"); break;       
    };

    switch (FSOUND_GetMixer())
    {
        case FSOUND_MIXER_BLENDMODE:     Tracer::stat( 4, "FSOUND Mixer	:FSOUND_MIXER_BLENDMODE"); break;
        case FSOUND_MIXER_MMXP5:         Tracer::stat( 4, "FSOUND Mixer	:FSOUND_MIXER_MMXP5"); break;
        case FSOUND_MIXER_MMXP6:         Tracer::stat( 4, "FSOUND Mixer	:FSOUND_MIXER_MMXP6"); break;
        case FSOUND_MIXER_QUALITY_FPU:   Tracer::stat( 4, "FSOUND Mixer	:FSOUND_MIXER_QUALITY_FPU"); break;
        case FSOUND_MIXER_QUALITY_MMXP5: Tracer::stat( 4, "FSOUND Mixer	:FSOUND_MIXER_QUALITY_MMXP5"); break;
        case FSOUND_MIXER_QUALITY_MMXP6: Tracer::stat( 4, "FSOUND Mixer	:FSOUND_MIXER_QUALITY_MMXP6"); break;
        default:
            Tracer::warn("NO known Mixer!");
    };
    Tracer::stat( 4, "FSOUND Driver        : %s", FSOUND_GetDriverName(FSOUND_GetDriver()));
    Tracer::stat( 4, "Hardware 3D channels : %d", FSOUND_GetNumHardwareChannels());

    Tracer::stat( 4, "Sound Manager Initialized" );
}

//=============================================================================
/// @brief Sets up the registry for the soundmgr.
/// @param cfg - the configuration file to be used for the registry settings.
//=============================================================================
void SoundManager::setupRegistry( const string& cfg )
{
    if( m_setup ) return;
    Registry::init(cfg);
#ifdef WIN32
    Registry::getValue( "/DIVERSE/SoundManager/SoundSystem", "DSOUND", "ESD or OSS or ALSA or DSOUND or WINMM" );
#else
    Registry::getValue( "/DIVERSE/SoundManager/SoundSystem", "ESD", "ESD or OSS or ALSA or DSOUND or WINMM" );
#endif

    m_setup = true;
}

//=============================================================================
/// @brief Called during the postProcess stage of the application cycle to
/// update the position and orientation of the sound nodes and the listener.
//=============================================================================
void SoundManager::postProcess()
{
    // Update all sounds, see if they moved etc
    for( unsigned int i = 0; i < m_sounds.size(); i++ )
        m_sounds[i]->update();

    // Update the listeners position
    float delta_t = m_timer.reset().sec();
    
    dVec3 pos = NavManager::getMatrix().getTrans();// * dVec3( 0, 0, 0 );
    dVec3 dir = dMatrix::transform3x3(NavManager::getMatrix(), dVec3(0,1,0) );
    dVec3 up = NavManager::getMatrix() * dVec3( 0, 0, 1 );
    dir.normalize();

    m_listenvel = (pos - m_listenpos)/delta_t;
    m_listenpos = pos;
    m_listendir = dir;
    
    
    FSOUND_3D_Listener_SetAttributes( m_listenpos.ptr(), 
                                      m_listenvel.ptr(), 
                                      dir.x(), dir.y(), dir.z(),
                                      up.x(), up.y(), up.z()
                                      );

    
    FSOUND_Update();
    //Sleep(delta_t*.05);
}

//=============================================================================
/// @brief Add a sound node to be processed by the sound manager.
/// @param sound - a pointer to the sound node to add.
//=============================================================================
void SoundManager::addSound( DSoundNode* sound )
{
    m_sounds.push_back(sound);
}

//=============================================================================
/// @brief Remove a sound node from the processing of the sound manager.
/// @param sound - a pointer to the sound node to remove.
//=============================================================================
void SoundManager::removeSound( DSoundNode* sound )
{
    vector<DSoundNode*>::iterator itr = m_sounds.begin();
    while( itr != m_sounds.end() )
    {
        if( *itr == sound )
        {
            m_sounds.erase(itr);
            return;
        }
        itr++;
    }
}

//=============================================================================
/// @brief Not implemented.
//=============================================================================
void SoundManager::playSoundFromFile( const string& fname, float vol, const dVec3& pos )
{
}

//=============================================================================
/// @brief Get a pointer to the instance of the sound manager.
/// @return a pointer to the sound manager instance.
//=============================================================================
SoundManager* SoundManager::instance()
{
	return m_self;
}
