#ifndef _SOUNDMANAGER_H_
#define _SOUNDMANAGER_H_
#include<vector>
#include<string>
#include<dmath/dVec3.h>
#include<dutil/DTime.h>
#include<servicemgr/GService.h>
#include <soundmgr/DSoundNode.h>

#ifdef WIN32
#ifdef SOUNDMGR_EXPORTS
#define SOUNDMGR_API __declspec(dllexport)
#else
#define SOUNDMGR_API __declspec(dllimport)
#endif /* SOUNDMGR_EXPORTS */
#else
#define SOUNDMGR_API
#endif /* WIN32 */

using namespace std;

//=============================================================================
/// @class SoundManager
/// @author Daniel Larimer
/// @brief Provides an interface for adding 3D sounds to the DIVERSE environment.
///
/// SoundManager is a static object which runs as a service. Only one SoundManager
/// object can exist in an application. You should call SoundManager::init prior
/// to calling any SoundManager functions.
//=============================================================================
class SOUNDMGR_API SoundManager : public GService
{
    public:
        void setupRegistry( const string& cfg = "diverse.xml" );
        static void init();

	void config();
        void postProcess();

        static void addSound( DSoundNode* sound );
        static void removeSound( DSoundNode* sound );

        static void playSoundFromFile( const string& fname, float vol, const dVec3& pos );
	static SoundManager* instance();

    private:
		SoundManager();
        static bool   m_setup;
        static bool   m_init;
        static DTime  m_timer;

        static dVec3  m_listenvel;
        static dVec3  m_listenpos;
        static dVec3  m_listendir;

        static vector<DSoundNode*> m_sounds;
		static SoundManager*  m_self;
};


#endif
