//=============================================================================
// wandJoystickNav.h
// - Navigation plugin class for wand joystick controller
//
// Created 6/4/03 By Patrick Shinpaugh
//=============================================================================
#ifndef _WANDJOYSTICKNAV_H_
#define _WANDJOYSTICKNAV_H_
#include <dmath/dVec3.h>
#include <eventmgr/devents.h>
#include <eventmgr/EventListener.h>
#include <navmgr/NavInterface.h>
#include <navmgr/NavManager.h>
#include <dutil/DTime.h>
#include <dmath/dMatrix.h>

#ifdef WIN32
#ifdef WANDNAV_EXPORTS
#define WANDNAV_API __declspec(dllexport)
#else
#define WANDNAV_API __declspec(dllimport)
#endif /* WANDNAV_EXPORTS */
#else
#define WANDNAV_API
#endif /* WIN32 */

class WANDNAV_API wandJoystickNav : public NavInterface,public EventListener
{
    public:
        enum WJNAVTYPE
        {
            WJNAV,
            WJPITCHNAV,
            WJROLLNAV,
            WJHPRNAV
        };

        wandJoystickNav( WJNAVTYPE navtype );
        ~wandJoystickNav();

        virtual bool onWandEvent( DWandEvent* evt );

        virtual void process();
        virtual void start();
        virtual void stop();

        virtual dMatrix getNavMatrix()const;
        virtual void     setNavMatrix( const dMatrix& mat );

        virtual float getSpeedFactor()const;
        virtual void setSpeedFactor( float speedfactor );
        virtual void setRotateFactor( float rfactor );
		virtual dVec3 getVelocity()const { return m_velocity; };
		virtual dVec3 getPivot();

        virtual bool isRunning()const;
        virtual string getNavigationName()const;

private:
        WJNAVTYPE m_navtype;

        dMatrix m_navmat;
        dVec3 m_wandpos;
        dVec3 m_wanddir;
		dVec3 m_velocity;

        float m_speedfactor;
        float m_rfactor;
        float m_twist;
        float m_transdeadzone;
        float m_rotdeadzone;
        float m_deltapos;
        float m_deltadir;

        bool m_active;
        DTime m_time;
};

#endif
