#include<iostream>

#ifndef __sgi
 #include<cstdlib>
#else
 #include<stdlib.h>
#endif

#include <dtk.h>
#include<tracer/Tracer.h>
#include<servicemgr/GServiceManager.h>
#include<DTK/dtkmgr/DtkManagerService.h>
#include<navmgr/NavManager.h>
#include<eventmgr/EventManager.h>
#include<eventmgr/devents.h>
#include<dreg/Registry.h>
#include"DtkNav.h"
using namespace std;

//=============================================================================
// Function Name: DtkNav
///     Constructor, the name of the navigation, DtkNav... shouldn't change.
//=============================================================================
DtkNav::DtkNav( const string& name )
{
    m_active = false;
    m_navlist = NULL;
    m_nav     = NULL;
    NavManager::addNavigation(this);

    //GServiceManager::load("DtkManagerService");
    DtkManagerService* dtk_serv =  (DtkManagerService*)
                                    GServiceManager::getService("DtkManagerService");
    if (dtk_serv == NULL)
    {
        Tracer::error("DtkManagerService service wasn't loaded");
        return;
    }

    dtkManager* manager = dtk_serv->getManager();
    if (manager == NULL)
    {
        Tracer::error(1, "DtkManagerService did not return a "
                         "working dtk manager");
        exit(1);
    }

    dtkAugment* chain = manager->check("dtkNavList", DTKNAVLIST_TYPE);
    m_navlist = (dtkNavList*) chain;
    //m_navlist = (dtkNavList*) manager->check( "dtkNavList", DTKNAVLIST_TYPE);
    if( m_navlist == NULL )
    {
        Tracer::error( "No valid dtkNavList found\n\n");
        //exit(1);
    }
    string tmp = Registry::getValue("/DIVERSE/DTK/DtkNav/nextkey", "P");
    m_nextkey = tmp[0];
    //Tracer::warn( "m_nextkey = %c", m_nextkey );


    //Load navigations
    //Get the entries in the file
    Entry* NavData = Registry::getEntry("/DIVERSE/DTK/DtkNav/navigations");
    //C++ stuff 
    map<string, Entry*>NavAugments = NavData->getEntries();
    map<string, Entry*>::iterator itr = NavAugments.begin();
    //Loop through the entries
    while (itr != NavAugments.end())
    {
        //If we are supposed to turn one on do it
        if (itr->second->getValue( "", "yes") == "yes" )
        {
            //Load it into the manager
            manager->load(itr->first.c_str());
            //Add it to the navigation list
            m_navlist->add((dtkNav*)manager->get(itr->first.c_str(), DTKNAV_TYPE));
        }
        itr++;
    }
    m_nav = m_navlist->first();
    if (m_nav == NULL)
        Tracer::error( "No valid dtkNavigation found\n\n");

}

DtkNav::~DtkNav()
{
    stop();
}

float DtkNav::getSpeedFactor() const
{
    return m_speedfactor;
}
void DtkNav::setSpeedFactor(float speedfactor)
{
    m_speedfactor = speedfactor;
}
void DtkNav::process()
{
    //DtkManagerService* dtk_serv =  (DtkManagerService*)
     //                               GServiceManager::getService("DtkManagerService");
    //dtkManager* manager = dtk_serv->getManager();
    //manager->print();
    if( !m_active ) return;
    if( !m_nav ) return;
    /*
    m_pos[0] = m_nav->location.x; 
    m_pos[1] = m_nav->location.y; 
    m_pos[2] = m_nav->location.z; 
    m_dir[0] = m_nav->location.h;
    m_dir[1] = m_nav->location.p;
    m_dir[2] = m_nav->location.r;
*/
}

void DtkNav::setNavMatrix( const dMatrix& mat )
{
    cout<<"DtkNav is broken\n";
}

dMatrix DtkNav::getNavMatrix()const
{
    cout<<"DtkNav::getNavMatrix() - can't do, dtkMatrix is too limited\n";
    return dMatrix();
}


bool DtkNav::onKeyEvent( DKeyEvent* evt )
{
    if( !isRunning() ) return false; 
    if( evt->action() == DKeyEvent::DOWN && evt->ascii() == m_nextkey ) 
    {
        m_nav = m_navlist->next(m_nav);
//        if( m_nav )
//            m_nav->location.set( m_pos.x(), m_pos.y(), m_pos.z(),
//                                 m_dir.h(), m_dir.p(), m_dir.r() );
    }
    return false;
}

//=============================================================================
// Function Name: start
///     Starts the navigation updating
//=============================================================================
void DtkNav::start()
{
    EventManager::addKeyListener( this );
    m_speedfactor = NavManager::getSpeedFactor() ;
    m_active = true;
    m_prev_update.start();
    if( m_navlist )
        m_navlist->activate();
}

//=============================================================================
// Function Name: stop
///     stops the navigation from updating
//=============================================================================
void DtkNav::stop()
{
    EventManager::removeKeyListener( this );
    m_active = false;
    if( m_navlist )
        m_navlist->deactivate();
}
//=============================================================================
// Function Name: isRunning
///     Returns true when the navigation is running
/// \return  true if start() has been called and stop() hasn't
//=============================================================================
bool DtkNav::isRunning() const
{
    return m_active;
}
