//=============================================================================
// DtkNav.h
/// @class DtkNav
/// @brief Provides dtk navigation support
//  @author Andrew Ray
//=============================================================================
#ifndef _DTKNAV_H
#define _DTKNAV_H
#include <dmath/dVec3.h>
#include <eventmgr/EventListener.h>
#include <navmgr/NavInterface.h>

#ifdef WIN32
#ifdef DTKNAV_EXPORTS
#define DTKNAV_API __declspec(dllexport)
#else
#define DTKNAV_API __declspec(dllimport)
#endif /* DTKNAV_EXPORTS */
#else
#define DTKNAV_API
#endif /* WIN32 */

class dtkManager;
class dtkNav;
class dtkNavList;
class DKeyEvent;
class DMouseEvent;

class DTKNAV_API DtkNav : public NavInterface, public EventListener
{
    public:
        DtkNav( const string& name = "DtkNav" );
        ~DtkNav();
        
        virtual bool onKeyEvent( DKeyEvent* evt);
        virtual void process();

        bool isRunning()const ;
        virtual void  start();
        virtual void  stop();
        
        virtual dMatrix getNavMatrix()const;
        virtual void    setNavMatrix( const dMatrix& mat );

		virtual void setRotateFactor( float rfactor ) {};
        virtual void setSpeedFactor(float speedfactor);
        virtual float getSpeedFactor() const;
		virtual dVec3 getVelocity()const { return m_velocity; };

        virtual string   getNavigationName()const { return "DtkNav"; }
        dtkNavList* getNavList() const {return m_navlist;};
        dtkNav*     getCurNav() const {return m_nav;};

    private:
        bool            m_active;
        dtkNavList*     m_navlist;
        dtkNav*         m_nav;
        DTime           m_prev_update;
        char            m_nextkey;
        float           m_speedfactor;
		dVec3			m_velocity;
};

#endif
