#include<dmath/dmath.h>
#include<iostream>
#ifndef __sgi
 #include<cstdlib>
#else
 #include<stdlib.h>
#endif

#ifndef WIN32
#include<GL/gl.h>
#include<GL/glu.h>
#endif /* !WIN32 */

#include<tracer/Tracer.h>
#include<dmath/dMatrix.h>
#include<dreg/Registry.h>

#include <pluginmgr/Plugin.h>
#include"MouseNav.h"
using namespace std;


void MouseNav::setSpeedFactor( float speedfactor )
{
    m_speedfactor = speedfactor;
}
float MouseNav::getSpeedFactor() const
{
    return m_speedfactor;
}
void MouseNav::setRotateFactor( float rfactor )
{
    m_rfactor = rfactor;
}
MouseNav::MouseNav( const string& name )
{
    m_active = false;
    NavManager::init();
    NavManager::addNavigation(this);
    EventManager::init();
    //SetDescription("Inteprets DMouseEvents and navigates");
    
    m_navmat.makeIdentity();
    m_dmat = m_resetmat = m_navmat;

    m_moving = false;
    m_button = -1;
    //===============================
    // Speed and Twist are per second
    //===============================
    m_speedfactor = atof( Registry::getValue( "/mouseLookNav/Speed", "2.0").c_str() );
    m_twist = atof( Registry::getValue( "/mouseLookNav/Twist", "45.0").c_str() );

    m_rfactor = NavManager::getRotateFactor();

    //===============================
    // Trans and Rot dead zones are
    // tolerances within which no
    // movement will be registered
    //===============================
    m_transdeadzone = 
        atoi( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/TransDeadZone", "10").c_str() );
    m_rotdeadzone = 
        atoi( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/RotDeadZone", "10").c_str() );

    m_pix_per_unit = 
        atoi( Registry::getValue("/DIVERSE/NavManager/mouseLookNav/PixelsPerUnit", "30").c_str() );
    m_pix_per_deg  = 
        atoi( Registry::getValue("/DIVERSE/NavManager/mouseLookNav/PixelsPerDeg", "50").c_str() );

}

MouseNav::~MouseNav()
{
    stop();
}

bool MouseNav::onMouseEvent( DMouseEvent* evt )
{
    switch ( evt->action() )
    {
        case DMouseEvent::PRESS:
            m_mousex = evt->x();
            m_mousey = evt->y();
            m_moving = true;
            break;
        case DMouseEvent::RELEASE:
			m_velocity = dVec3();
            return m_moving = false;
            break;
    }

    int dx = m_mousex - evt->x();
    int dy = m_mousey - evt->y();

    if ( abs(dx) <= m_rotdeadzone )
        m_deltax = 0.0f;
    else
        m_deltax = (float)(dx > 0 ? dx - m_rotdeadzone : dx + m_rotdeadzone);

    if ( abs(dy) <= m_rotdeadzone )
        m_deltay = 0.0f;
    else
        m_deltay = (float)(dy > 0 ? dy - m_rotdeadzone : dy + m_rotdeadzone);

    if( evt->getButton( DMouseEvent::LEFT ) )
        m_button = DMouseEvent::LEFT;
    if( evt->getButton( DMouseEvent::RIGHT ) )
        m_button = DMouseEvent::RIGHT;
    if( evt->getButton( DMouseEvent::MIDDLE ) )
        m_button = DMouseEvent::MIDDLE;
    return true;
}

void MouseNav::process()
{
    if( !m_active ) return;
    if( !m_moving ) return;

    float delta_t = m_time.reset().sec();

	if ( delta_t )
		m_velocity = m_navmat.inverse( m_navmat ).getTrans();
//    cout<<"r_factor = "<<m_rfactor<<endl;
    if( m_button == DMouseEvent::LEFT )
    {
        // rotate it
        m_dmat.makeRotate(
               m_rfactor * -DEG2RAD(  delta_t * m_twist * m_deltax / m_pix_per_deg ), 0, 0, 1 );
        // translate it
        m_dmat = m_dmat * dMatrix::translate( 0, -delta_t * m_speedfactor * m_deltay / m_pix_per_unit, 0 );

//        m_deltapos.y() += m_speedfactor * m_deltay / m_pix_per_unit;
//        m_deltadir.h() += m_twist * m_deltax / m_pix_per_deg;
    }
    if( m_button == DMouseEvent::RIGHT )
    {
//        m_deltadir.h() += m_twist * m_deltax / m_pix_per_deg;
//        m_deltadir.p() += m_twist * m_deltay / m_pix_per_deg;
        // rotate it
        m_dmat.makeRotate( m_rfactor *
                  -DEG2RAD( delta_t * m_twist * m_deltax / m_pix_per_deg),
                  dVec3(0, 0, 1 ),
                  m_rfactor  * 
                 -DEG2RAD( delta_t * m_twist * m_deltay / m_pix_per_deg), 
                  dVec3(1, 0, 0 ),
               0, dVec3( 0, 1, 0 ) );
    }
    if( m_button == DMouseEvent::MIDDLE )
    {
        m_dmat.makeTranslate( delta_t * m_speedfactor * m_deltax / m_pix_per_unit,
                              0,
                              -delta_t * m_speedfactor * m_deltay / m_pix_per_unit );
//        m_deltapos.x() += m_speedfactor * m_deltax / m_pix_per_unit;
//        m_deltapos.z() -= m_speedfactor * m_deltay / m_pix_per_unit;
    }
    m_navmat = m_navmat * m_dmat/* * m_navmat*/;
	if ( delta_t )
		m_velocity = ( m_navmat.inverse( m_navmat ).getTrans() - m_velocity ) / delta_t;
}

bool MouseNav::onKeyEvent( DKeyEvent* evt )
{
    switch( evt->ascii() )
    {
        case '-':
            m_speedfactor *= 0.5;
            break;
        case '+':
            m_speedfactor *= 2;
            break;
        case '0':
            m_navmat = m_resetmat;
            break;
        case '9':
            m_dir.y() = m_dir.z() = 0;
            break;
        case '8':
            m_resetmat = m_navmat;
            break;
    }
    return true;
}

void MouseNav::start()
{
    m_active = true;
    EventManager::addMouseListener(this);
    EventManager::addKeyListener(this);
    m_speedfactor = NavManager::getSpeedFactor() ;
    m_time.start();
}

void MouseNav::stop()
{
    EventManager::removeMouseListener(this);
    EventManager::removeKeyListener(this);
    m_active = false;
}

bool MouseNav::isRunning() const
{
    return m_active;
}
