//=============================================================================
// MouseNav.h
// - Defines a class that will draw/rotate a teapot
//
// Created 5/12/03 By Daniel Larimer, modified by Andrew A. Ray
//=============================================================================
#ifndef _MOUSENAV_H_
#define _MOUSENAV_H_
#include <dmath/dVec3.h>
#include <eventmgr/devents.h>
#include <eventmgr/EventListener.h>
#include <navmgr/NavInterface.h>
#include <navmgr/NavManager.h>
#include <dutil/DTime.h>

#ifdef WIN32
#ifdef MOUSENAV_EXPORTS
#define MOUSENAV_API __declspec(dllexport)
#else
#define MOUSENAV_API __declspec(dllimport)
#endif /* MOUSENAV_EXPORTS */
#else
#define MOUSENAV_API
#endif /* WIN32 */

class MOUSENAV_API MouseNav:public NavInterface,public EventListener
{
    public:
        MouseNav( const string& name = "MouseNav" );
        ~MouseNav();
        
        virtual bool onMouseEvent( DMouseEvent* evt);
        virtual bool onKeyEvent( DKeyEvent* evt);
        virtual void process();

        bool isRunning()const ;
        virtual void start();
        virtual void stop();
        virtual void setSpeedFactor( float speedfactor );
        virtual void setRotateFactor( float rfactor );
        virtual float getSpeedFactor() const;
		virtual dVec3 getVelocity()const { return m_velocity; };

        virtual dMatrix getNavMatrix()const { return m_navmat; }
        virtual void setNavMatrix( const dMatrix& navmat ){ m_navmat = navmat; }

        virtual string getNavigationName()const { return "MouseNav"; }

    private:
        dVec3 m_pos;
        dVec3 m_dir;
        dVec3 m_scale;
        dVec3 m_velocity;
        dMatrix m_dmat;
        dMatrix m_navmat;
        dMatrix m_resetmat;

        float m_pix_per_unit;
        float m_pix_per_deg;
        float m_speedfactor;
        float m_twist;
        float m_rfactor;
        float m_deltax;
        float m_deltay;

        int m_transdeadzone;
        int m_rotdeadzone;
        int m_mousex;
        int m_mousey;
        int m_button;

        bool m_moving;
        bool m_active;

        DTime m_time;
};

#endif
