/**************************************************************************
DIVERSE version 3.0 Beta2
released: Friday July 25th 
The DIVERSE HOMPAGE is at: diverse.sourceforge.net

Copyright (C) 2000 - 2003  Daniel Larimer, Andrew A. Ray, Patrick Shinpaugh
			   John Kelso, and Ron Kriz

This is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License (GPL) as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This software is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this software, in the top level source directory in a file
named "COPYING.GPL"; if not, see it at:
http://www.gnu.org/copyleft/gpl.html or write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
USA.
**************************************************************************/
//=============================================================================
// mouseLookNav.cpp
// Navigation plugin class for mouse look and keypad control
//
// Created 6/9/03 By Patrick Shinpaugh
//=============================================================================
#include <tracer/Tracer.h>
#include <dmath/dMatrix.h>
#include <dreg/Registry.h>
#include <eventmgr/keyConverter.h>
#include "mouseLookNav.h"

using namespace std;

mouseLookNav::mouseLookNav()
{
    m_active = false;
    NavManager::addNavigation( this );

    //===============================
    // Speed and Twist are per second
    //===============================
    m_speedfactor = atof( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/Speed", "2.0").c_str() );
    m_twist = atof( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/Twist", "45.0").c_str() );

    //===============================
    // Trans and Rot dead zones are
    // tolerances within which no
    // movement will be registered
    //===============================
    m_transdeadzone = atoi( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/TransDeadZone", "10").c_str() );
    m_rotdeadzone = atoi( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/RotDeadZone", "10").c_str() );

    m_pix_per_unit = atoi( Registry::getValue("/DIVERSE/NavManager/mouseLookNav/PixelsPerUnit", "30").c_str() );
    m_pix_per_deg  = atoi( Registry::getValue("/DIVERSE/NavManager/mouseLookNav/PixelsPerDeg", "30").c_str() );

    //===============================
    // Set default keys
    //===============================
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/PITCH_DOWN", "Key_8" ) )] = PITCH_DOWN;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/PITCH_UP", "Key_2" ) )] = PITCH_UP;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/TURN_LEFT", "Key_4" ) )] = TURN_LEFT;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/MOVE_BACKWARD", "Key_Z" ) )] = MOVE_BACKWARD;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/TURN_RIGHT", "Key_6" ) )] = TURN_RIGHT;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/MOVE_LEFT", "Key_1" ) )] = MOVE_LEFT;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/MOVE_FORWARD", "Key_Shift" ) )] = MOVE_FORWARD;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/MOVE_RIGHT", "Key_3" ) )] = MOVE_RIGHT;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/SPEED_INC", "Key_I" ) )] = SPEED_INC;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/SPEED_DEC", "Key_D" ) )] = SPEED_DEC;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/ROLL_LEFT", "Key_7" ) )] = ROLL_LEFT;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/ROLL_RIGHT", "Key_9" ) )] = ROLL_RIGHT;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/MOVE_DOWN", "Key_Enter" ) )] = MOVE_DOWN;
    m_keys[keyConverter::translateStringToKey( Registry::getValue( "/DIVERSE/NavManager/mouseLookNav/MOVE_UP", "Key_Plus" ) )] = MOVE_UP;

}

mouseLookNav::~mouseLookNav()
{
    stop();
}


float mouseLookNav::getSpeedFactor() const
{
    return m_speedfactor;
}


void mouseLookNav::setSpeedFactor( float speedfactor )
{
    m_speedfactor = speedfactor;
}

bool mouseLookNav::onMouseEvent( DMouseEvent* evt )
{
    switch ( evt->action() )
    {
        case DMouseEvent::PRESS:
            m_mousex = evt->x();
            m_mousey = evt->y();
            m_moving = true;
            break;
        case DMouseEvent::RELEASE:
			m_velocity = dVec3();
            return m_moving = false;
            break;
    }

    int dx = m_mousex - evt->x();
    int dy = m_mousey - evt->y();

    if ( abs(dx) <= m_rotdeadzone )
        m_deltax = 0.0f;
    else
        m_deltax = (float)(dx > 0 ? dx - m_rotdeadzone : dx + m_rotdeadzone);

    if ( abs(dy) <= m_rotdeadzone )
        m_deltay = 0.0f;
    else
        m_deltay = (float)(dy > 0 ? dy - m_rotdeadzone : dy + m_rotdeadzone);

    if( evt->getButton( DMouseEvent::LEFT ) )
        m_button = DMouseEvent::LEFT;
    if( evt->getButton( DMouseEvent::RIGHT ) )
        m_button = DMouseEvent::RIGHT;
    if( evt->getButton( DMouseEvent::MIDDLE ) )
        m_button = DMouseEvent::MIDDLE;
    return true;
}

bool mouseLookNav::onKeyEvent( DKeyEvent* evt )
{
    switch (evt->action())
    {
        case DKeyEvent::UP:
            m_keyspressed[evt->key()] = false;
            break;
        case DKeyEvent::DOWN:
            m_keyspressed[evt->key()] = true;
            break;
    }
    if( evt->action() == DKeyEvent::UP )
        switch( m_keys.find(evt->key())->second )
        {
            case SPEED_INC:
                Tracer::debug( "mouseLookNav::SPEED_INC to %f from %f", m_speedfactor*2.0, m_speedfactor );
                m_speedfactor *= 2.0f;
                break;
            case SPEED_DEC:
                Tracer::debug( "mouseLookNav::SPEED_DEC to %f from %f", m_speedfactor*0.5, m_speedfactor );
                m_speedfactor *= 0.5;
                break;
        }
    return true;
}

void mouseLookNav::process()
{
    if ( !m_active )
        return;
    dVec3 m_deltapos;
    dVec3 m_deltadir;
    float delta_t = m_time.reset().sec();

    map<int, bool>::iterator keypress_itr;
    int keyaction;

    for ( keypress_itr = m_keyspressed.begin();
          keypress_itr != m_keyspressed.end(); keypress_itr++ )
    {
        if ( keypress_itr->second )
        {
            keyaction = m_keys.find( keypress_itr->first )->second;
        }
        else
            continue;

        switch ( keyaction )
        {
        case MOVE_FORWARD:
            m_deltapos.y() += m_speedfactor;
            break;
        case MOVE_BACKWARD:
            m_deltapos.y() -= m_speedfactor;
            break;
        case MOVE_LEFT:
            m_deltapos.x() -= m_speedfactor;
            break;
        case MOVE_RIGHT:
            m_deltapos.x() += m_speedfactor;
            break;
        case MOVE_UP:
            m_deltapos.z() += m_speedfactor;
            break;
        case MOVE_DOWN:
            m_deltapos.z() -= m_speedfactor;
            break;
        case TURN_LEFT:
            m_deltadir.h() += m_twist * m_rfactor;
            break;
        case TURN_RIGHT:
            m_deltadir.h() -= m_twist * m_rfactor;
            break;
        case PITCH_UP:
            m_deltadir.p() += m_twist * m_rfactor;
            break;
        case PITCH_DOWN:
            m_deltadir.p() -= m_twist * m_rfactor;
            break;
        case ROLL_LEFT:
            m_deltadir.r() -= m_twist * m_rfactor;
            break;
        case ROLL_RIGHT:
            m_deltadir.r() += m_twist * m_rfactor;
            break;
        default:
            Tracer::debug( 5, "mouseLookNav::unmapped key\n" );
        }
    }

    if (m_moving)
    {
        if( m_button == DMouseEvent::LEFT )
        {
            m_deltapos.y() += m_speedfactor * m_deltay / m_pix_per_unit;
            m_deltadir.h() += m_twist * m_rfactor * m_deltax / m_pix_per_deg;
        }
        if( m_button == DMouseEvent::RIGHT )
        {
            m_deltadir.h() += m_twist * m_rfactor * m_deltax / m_pix_per_deg;
            m_deltadir.p() += m_twist * m_rfactor * m_deltay / m_pix_per_deg;
        }
        if( m_button == DMouseEvent::MIDDLE )
        {
            m_deltapos.x() += m_speedfactor * m_deltax / m_pix_per_unit;
            m_deltapos.z() -= m_speedfactor * m_deltay / m_pix_per_unit;
        }
    }
    m_deltadir.h() = DEG2RAD( m_deltadir.h() );
    m_deltadir.p() = DEG2RAD( m_deltadir.p() );
    m_deltadir.r() = DEG2RAD( m_deltadir.r() );

    dMatrix mat;
    dVec3 delta_dir = m_deltadir * -delta_t;
    dVec3 delta_pos = m_deltapos * -delta_t;
    mat.makeEulerZXY( delta_dir );

    m_navmat = m_navmat * (mat * dMatrix::translate( delta_pos ));
	m_velocity = m_deltapos;
}

void mouseLookNav::start()
{
    EventManager::addMouseListener( this );
    EventManager::addKeyListener( this );
    m_speedfactor = NavManager::getSpeedFactor() ;
    m_active = true;
    m_time.reset();
}

void mouseLookNav::stop()
{
    EventManager::removeMouseListener( this );
    EventManager::removeKeyListener( this );
    m_active = false;
}

bool mouseLookNav::isRunning() const
{
    return m_active;
}

string mouseLookNav::getNavigationName()const
{
    return "mouseLookNav";
}
