//=============================================================================
// mouseLookNav.h
// - Navigation plugin class for mouse look and keypad control
//
// Created 6/9/03 By Patrick Shinpaugh
//=============================================================================
#ifndef _MOUSELOOKNAV_H_
#define _MOUSELOOKNAV_H_
#include <dmath/dVec3.h>
#include <eventmgr/devents.h>
#include <eventmgr/EventListener.h>
#include <navmgr/NavInterface.h>
#include <navmgr/NavManager.h>
#include <dutil/DTime.h>

#ifdef WIN32
#ifdef MOUSELOOKNAV_EXPORTS
#define MOUSELOOKNAV_API __declspec(dllexport)
#else
#define MOUSELOOKNAV_API __declspec(dllimport)
#endif /* MOUSELOOKNAV_EXPORTS */
#else
#define MOUSELOOKNAV_API
#endif /* WIN32 */

class MOUSELOOKNAV_API mouseLookNav:public NavInterface,public EventListener
{
    public:
        enum ACTIONS
        {
            NO_ACTION,
            MOVE_FORWARD,
            MOVE_BACKWARD,
            MOVE_LEFT,
            MOVE_RIGHT,
            MOVE_UP,
            MOVE_DOWN,
            TURN_LEFT,
            TURN_RIGHT,
            PITCH_UP,
            PITCH_DOWN,
            ROLL_LEFT,
            ROLL_RIGHT,
            SPEED_INC,
            SPEED_DEC
        };
        
        mouseLookNav();
        ~mouseLookNav();

        virtual bool onMouseEvent( DMouseEvent* evt );
        virtual bool onKeyEvent( DKeyEvent* evt );

        virtual void process();
        virtual void start();
        virtual void stop();

        dMatrix getNavMatrix()const                 { return m_navmat; }
        void    setNavMatrix( const dMatrix& mat )  { m_navmat = mat;  }

        virtual float getSpeedFactor() const;
        virtual void setSpeedFactor( float speedfactor );
        virtual void setRotateFactor( float rfactor ) { m_rfactor = rfactor; }
		virtual dVec3 getVelocity()const { return m_velocity; };

        virtual bool isRunning()const;
        virtual string getNavigationName()const;

private:
        dMatrix m_navmat;
        dMatrix m_deltamat;

		dVec3 m_velocity;

        float m_pix_per_unit;
        float m_pix_per_deg;
        float m_speedfactor;
        float m_twist;
        float m_deltax;
        float m_deltay;
        float m_rfactor;

        int m_transdeadzone;
        int m_rotdeadzone;
        int m_mousex;
        int m_mousey;
        int m_button;

        bool m_moving;
        bool m_active;

        DTime m_time;
        map<int, int> m_keys;
        map<int, bool> m_keyspressed;
};

#endif
