#include"NavMgrPanel.h"
#include<qlayout.h>
#include<qtabwidget.h>
#include<qlistview.h>
#include<qpushbutton.h>
#include<qdir.h>
#include<qtutil/PathEditWidget.h>
#include<navmgr/NavManager.h>
#include<diverseconfig.h>
#if ( QT_VERSION < 300 )
#include "qt23wrappers.h"
#endif
#include<dreg/Registry.h>
#include<tracer/Tracer.h>
#include<iostream>
using namespace std;

NavMgrPanel::NavMgrPanel( QWidget* parent )
    :PanelWidget( parent, "Navigation Manager" ) 
{
    QVBoxLayout* vbox = new QVBoxLayout( this );
    m_tabs = new QTabWidget( this );
    vbox->addWidget( m_tabs );
    
    //=========================
    // Create the tabs
    //=========================
    QWidget* selector = new QWidget( m_tabs ); 
    QWidget* browser = new QWidget( m_tabs );
    QWidget* about = new QWidget( m_tabs ); 
    m_tabs->addTab( selector, "Navigations" );
    m_tabs->addTab( browser, "Path" );
    m_tabs->addTab( about, "Help" );

    //=========================
    // Create the select list
    //=========================
    QVBoxLayout* selvbox = new QVBoxLayout( selector );
    m_pllist = new QListView( selector );
    m_pllist->addColumn( "Name" );
    m_pllist->addColumn( "Load" );
    selvbox->addWidget( m_pllist );
    updateNavigations(); 

    //=========================
    // Create files
    //=========================
    QVBoxLayout* filevbox = new QVBoxLayout( browser );
    m_pedit = new PathEditWidget( browser );
    filevbox->addWidget( m_pedit );

    string navpath = 
        Registry::getValue("/DIVERSE/NavManager/navpath", 
                            DEFAULT_NAV_PATH, 
                            "The path to search for navigations" );
    m_pedit->setPath( navpath );
    connect( m_pedit, SIGNAL( pathChanged( const string& ) ),
             this,    SLOT  ( updatePath( const string& ) ) );
    connect( m_pedit, SIGNAL( directoryAdded( const string& ) ),
             this,    SLOT  ( addDir( const string& ) ) );
    connect( m_pedit, SIGNAL( directoryRemoved( const string& ) ),
             this,    SLOT  ( removeDir( const string& ) ) );
    connect( m_pllist, SIGNAL( clicked( QListViewItem* ) ),
             this,     SLOT  ( itemClicked( QListViewItem* ) ) );
}

void NavMgrPanel::updatePath( const string& path )
{
    Registry::setValue( "/DIVERSE/NavManager/navpath", path );
}
void NavMgrPanel::addDir( const string& path )
{
	#ifdef _DARWIN_
    QDir dir( path.c_str(), "*.dylib" );
    #elif defined WIN32
	#ifdef _DEBUG
    QDir dir( path.c_str(), "*.plugd" );
	#else
	QDir dir( path.c_str(), "*.plug" );
	#endif
	#else
    QDir dir( path.c_str(), "*.so" );
	#endif
    for( unsigned int i = 0; i < dir.count(); i++ )
    {
        string pname( dir[i].latin1() );
        #ifdef _DARWIN_
        pname = pname.substr( 0, pname.size() - 6 );
		#elif defined WIN32
		#ifdef _DEBUG
        pname = pname.substr( 0, pname.size() - 6 );
		#else
        pname = pname.substr( 0, pname.size() - 5 );
		#endif
        #else
        pname = pname.substr( 0, pname.size() - 3 );
        #endif
        string load = Registry::getValue( 
                            "/DIVERSE/NavManager/navigations/"+pname,
                            "no", "none" );
        string cmt = Registry::getComment( 
                            "/DIVERSE/NavManager/navigations/"+pname);
        QCheckListItem* item = 
                        new QCheckListItem( m_pllist,
                            pname.c_str(), 
                            QCheckListItem::CheckBox );
        item->setText( 1, cmt.c_str() );
        item->setOn( load == "yes" );
    }
}
void NavMgrPanel::removeDir( const string& path )
{
	#ifdef _DARWIN_
    QDir dir( path.c_str(), "*.dylib" );
    #elif defined WIN32
	#ifdef _DEBUG
    QDir dir( path.c_str(), "*.plugd" );
	#else
	QDir dir( path.c_str(), "*.plug" );
	#endif
	#else
    QDir dir( path.c_str(), "*.so" );
	#endif
    for( unsigned int i = 0; i < dir.count(); i++ )
    {
        string pname( dir[i].latin1() );
        #ifdef _DARWIN_
        pname = pname.substr( 0, pname.size() - 6 );
		#elif defined WIN32
		#ifdef _DEBUG
        pname = pname.substr( 0, pname.size() - 6 );
		#else
        pname = pname.substr( 0, pname.size() - 5 );
		#endif
        #else
        pname = pname.substr( 0, pname.size() - 3 );
        #endif
        Registry::remove("/DIVERSE/NavManager/navigations/"+pname );
        NavManager::unload( pname );
#if ( QT_VERSION < 300 )
        QListViewItem* item = findItem( m_pllist, pname.c_str() );
#else
        QListViewItem* item = m_pllist->findItem( pname.c_str(), 0 );
#endif /* QT_VERSION < 3 */
        if( item )
            delete item;
    }
}

PanelWidget* NavMgrPanel::clone( QWidget* parent )const
{
    return new NavMgrPanel( parent );
}

//=============================================================================
// Function Name: updateNavigations
///     Adds entries for all navigations found in the nav path, defaults to
///     no
//=============================================================================
void NavMgrPanel::updateNavigations()
{
    string navpath = 
        Registry::getValue("/DIVERSE/NavManager/navpath", 
                            DEFAULT_NAV_PATH, 
                            "The path to search for navigations" );
    ColonList clist(navpath);
    vector<string> dlist = clist.getList();
    m_pllist->clear();
    for( unsigned int i = 0; i < dlist.size(); i++ )
    {
        #ifdef _DARWIN_
        QDir dir( dlist[i].c_str(), "*.dylib" );
		#elif defined WIN32
		#ifdef _DEBUG
		QDir dir( dlist[i].c_str(), "*.plugd" );
		#else
		QDir dir( dlist[i].c_str(), "*.plug" );
		#endif
        #else
        QDir dir( dlist[i].c_str(), "*.so" );
        #endif
        for( unsigned int i = 0; i < dir.count(); i++ )
        {
            string pname( dir[i].latin1());
            #ifdef _DARWIN_
            pname = pname.substr( 0, pname.size() - 6 );
			#elif defined WIN32
			#ifdef _DEBUG
            pname = pname.substr( 0, pname.size() - 6 );
			#else
            pname = pname.substr( 0, pname.size() - 5 );
			#endif
            #else
            pname = pname.substr( 0, pname.size() - 3 );
            #endif
            string load = Registry::getValue( 
                                "/DIVERSE/NavManager/navigations/"+pname,
                                "no", "none" );
            string cmt = Registry::getComment( 
                                "/DIVERSE/NavManager/navigations/"+pname);
            QCheckListItem* item = 
                            new QCheckListItem( m_pllist,
                                pname.c_str(), 
                                QCheckListItem::CheckBox );
            item->setText( 1, cmt.c_str() );
            item->setOn( load == "yes" );
        }
    }
}

void NavMgrPanel::itemClicked( QListViewItem* item )
{
    QCheckListItem* citem = dynamic_cast<QCheckListItem*>(item);
    if( citem == NULL )
    {
        Tracer::error( "NavMgrPanel::itemClicked()\n"
                       "\tDynamic Cast to QCheckListItem failed\n" );
        return;
    }
    if( citem->isOn() )
    {
        citem->setText( 1, "yes" );
        string pname( citem->text(0).latin1() );
        Registry::setValue( 
                            "/DIVERSE/NavManager/navigations/"+pname,
                            "yes" );
        NavManager::load( pname );
    }
    else
    {
        citem->setText( 1, "no" );
        string pname( citem->text(0).latin1() );
        Registry::setValue( 
                            "/DIVERSE/NavManager/navigations/"+pname,
                            "no" );
        if( NavManager::isLoaded(pname ) )
            NavManager::unload( pname );
    }
}
