#include"PluginMgrPanel.h"
#include<qlayout.h>
#include<qtabwidget.h>
#include<qlistview.h>
#include<qpushbutton.h>
#include<qtutil/PathEditWidget.h>
#include<pluginmgr/PluginManager.h>
#include<pluginmgr/Plugin.h>
#include<dreg/Registry.h>


PluginMgrPanel::PluginMgrPanel( QWidget* parent )
    :PanelWidget( parent, "Plugin Manager" ) 
{
    QVBoxLayout* vbox = new QVBoxLayout( this );
    m_tabs = new QTabWidget( this );
    vbox->addWidget( m_tabs );
    
    //=========================
    // Create the tabs
    //=========================
    QWidget* selector = new QWidget( m_tabs ); 
    QWidget* browser = new QWidget( m_tabs );
    QWidget* about = new QWidget( m_tabs ); 
    m_tabs->addTab( selector, "Select" );
    m_tabs->addTab( browser, "Files" );
    m_tabs->addTab( about, "About" );

    //=========================
    // Create the select list
    //=========================
    QVBoxLayout* selvbox = new QVBoxLayout( selector );
    m_pllist = new QListView( selector );
    m_pllist->addColumn( "Name" );
    m_pllist->addColumn( "Description" );
    selvbox->addWidget( m_pllist );
    
    map<string, Plugin*> plugins = PluginManager::getPlugins();
    map<string, Plugin*>::iterator itr = plugins.begin();
    while( itr != plugins.end() )
    {
        QCheckListItem* item = 
                        new QCheckListItem( m_pllist,
                            itr->second->getName().c_str(), 
                            QCheckListItem::CheckBox );
        item->setText( 1, itr->second->getDescription().c_str() );
        if( itr->second->isLoaded() )
            item->setOn( true );
        m_itemmap[item] = itr->second;
        itr++;
    }

    //=========================
    // Create files
    //=========================
    QVBoxLayout* filevbox = new QVBoxLayout( browser );
    m_pedit = new PathEditWidget( browser );
    filevbox->addWidget( m_pedit );

    m_pedit->setPath( PluginManager::getPath() );
    connect( m_pedit, SIGNAL( pathChanged( const string& ) ),
             this,    SLOT  ( updatePath( const string& ) ) );
    connect( m_pllist, SIGNAL( clicked( QListViewItem* ) ),
             this,     SLOT  ( itemClicked( QListViewItem* ) ) );
}

void PluginMgrPanel::updatePath( const string& path )
{
    PluginManager::setPath( path );
}

PanelWidget* PluginMgrPanel::clone( QWidget* parent )const
{
    return new PluginMgrPanel( parent );
}

void PluginMgrPanel::itemClicked( QListViewItem* item )
{
    QCheckListItem* citem = (QCheckListItem*)item;
    if( citem->isOn() )
    {
        citem->setText( 1, "yes" );
        string pname( citem->text(0).latin1() );
        Registry::setValue(
                            "/DIVERSE/NavManager/navigations/"+pname,
                            "yes" );
        PluginManager::loadPlugin( pname );
    }
    else
    {
        citem->setText( 1, "no" );
        string pname( citem->text(0).latin1() );
        Registry::setValue( 
                            "/DIVERSE/NavManager/navigations/"+pname,
                            "no" );
        PluginManager::unloadPlugin( pname );
    }
}
