#include"ServiceMgrPanel.h"
#include<qlayout.h>
#include<qtabwidget.h>
#include<qlistview.h>
#include<qpushbutton.h>
#include<qdir.h>
#include<qtutil/PathEditWidget.h>
#include<servicemgr/GServiceManager.h>
#include<pluginmgr/PluginManager.h>
#include <diverseconfig.h>
#if ( QT_VERSION < 300 )
#include "qt23wrappers.h"
#endif
#include<dreg/Registry.h>
#include<iostream>
using namespace std;

ServiceMgrPanel::ServiceMgrPanel( QWidget* parent )
    :PanelWidget( parent, "Service Manager" ) 
{
    QVBoxLayout* vbox = new QVBoxLayout( this );
    m_tabs = new QTabWidget( this );
    vbox->addWidget( m_tabs );
    
    //=========================
    // Create the tabs
    //=========================
    QWidget* selector = new QWidget( m_tabs ); 
    QWidget* browser = new QWidget( m_tabs );
    QWidget* about = new QWidget( m_tabs ); 
    m_tabs->addTab( selector, "Services" );
    m_tabs->addTab( browser, "Path" );
    m_tabs->addTab( about, "About" );

    //=========================
    // Create the select list
    //=========================
    QVBoxLayout* selvbox = new QVBoxLayout( selector );
    m_pllist = new QListView( selector );
    m_pllist->addColumn( "Name" );
    m_pllist->addColumn( "Load" );
    selvbox->addWidget( m_pllist );
    updateServices(); 

    //=========================
    // Create files
    //=========================
    QVBoxLayout* filevbox = new QVBoxLayout( browser );
    m_pedit = new PathEditWidget( browser );
    filevbox->addWidget( m_pedit );

    string servicepath = 
        Registry::getValue("/DIVERSE/GServiceManager/servicepath", 
                            DEFAULT_SERVICE_PATH, 
                            "The path to search for services" );
    m_pedit->setPath( servicepath );
    connect( m_pedit, SIGNAL( pathChanged( const string& ) ),
             this,    SLOT  ( updatePath( const string& ) ) );
    connect( m_pedit, SIGNAL( directoryAdded( const string& ) ),
             this,    SLOT  ( addDir( const string& ) ) );
    connect( m_pedit, SIGNAL( directoryRemoved( const string& ) ),
             this,    SLOT  ( removeDir( const string& ) ) );
    connect( m_pllist, SIGNAL( clicked( QListViewItem* ) ),
             this,     SLOT  ( itemClicked( QListViewItem* ) ) );
}

void ServiceMgrPanel::updatePath( const string& path )
{
    Registry::setValue( "/DIVERSE/GServiceManager/servicepath", path );
}
void ServiceMgrPanel::addDir( const string& path )
{
    #if defined WIN32
	    #ifdef _DEBUG
            QDir dir( path.c_str(), "*.plugd" );
	    #else
	        QDir dir( path.c_str(), "*.plug" );
	    #endif
    #else
	     QDir dir( path.c_str(), "*.plug" );
    #endif
    for( unsigned int i = 0; i < dir.count(); i++ )
    {
        string pname( dir[i].latin1() );
        pname = pname.substr( 0, pname.size() - 6 );
		#if defined WIN32
		    #ifdef _DEBUG
                pname = pname.substr( 0, pname.size() - 6 );
		    #else
                pname = pname.substr( 0, pname.size() - 5 );
		#endif
		#else
            pname = pname.substr( 0, pname.size() - 5 );
        #endif
        
        string load = Registry::getValue( 
                            "/DIVERSE/GServiceManager/services/"+pname,
                            "no", "none" );
        string cmt = Registry::getComment( 
                            "/DIVERSE/GServiceManager/services/"+pname);
        QCheckListItem* item = 
                        new QCheckListItem( m_pllist,
                            pname.c_str(), 
                            QCheckListItem::CheckBox );
        item->setText( 1, cmt.c_str() );
        item->setOn( load == "yes" );
    }
}
void ServiceMgrPanel::removeDir( const string& path )
{
	#ifdef _DEBUG
    QDir dir( path.c_str(), "*.plugd" );
	#else
	QDir dir( path.c_str(), "*.plug" );
	#endif
    for( unsigned int i = 0; i < dir.count(); i++ )
    {
        string pname( dir[i].latin1() );
		#ifdef _DEBUG
        pname = pname.substr( 0, pname.size() - 6 );
		#else
        pname = pname.substr( 0, pname.size() - 5 );
		#endif
        Registry::remove("/DIVERSE/GServiceManager/services/"+pname );
        GServiceManager::stopService( pname );
#if ( QT_VERSION < 300 )
        QListViewItem* item = findItem( m_pllist, pname.c_str() );
#else
        QListViewItem* item = m_pllist->findItem( pname.c_str(), 0 );
#endif /* QT_VERSION < 3 */
        if( item )
            delete item;
    }
}

PanelWidget* ServiceMgrPanel::clone( QWidget* parent )const
{
    return new ServiceMgrPanel( parent );
}

//=============================================================================
// Function Name: updateServices
///     Adds entries for all services found in the service path, defaults to
///     no
//=============================================================================
void ServiceMgrPanel::updateServices()
{
    string servicepath = 
        Registry::getValue("/DIVERSE/GServiceManager/servicepath", 
                            DEFAULT_SERVICE_PATH, 
                            "The path to search for services" );
    ColonList clist(servicepath);
    vector<string> dlist = clist.getList();
    m_pllist->clear();
    for( unsigned int i = 0; i < dlist.size(); i++ )
    {
		#if defined WIN32
		#ifdef _DEBUG
		QDir dir( servicepath.c_str(), "*.plugd" );
		#else
		QDir dir( servicepath.c_str(), "*.plug" );
		#endif
		#else
        QDir dir( servicepath.c_str(), "*.plug" );
        #endif
        for( unsigned int i = 0; i < dir.count(); i++ )
        {
            string pname( dir[i].latin1() );
			#ifdef _DEBUG
            pname = pname.substr( 0, pname.size() - 6 );
			#else
            pname = pname.substr( 0, pname.size() - 5 );
			#endif
            string load = Registry::getValue( 
                                "/DIVERSE/GServiceManager/services/"+pname,
                                "no", "none" );
            string cmt = Registry::getComment( 
                                "/DIVERSE/GServiceManager/services/"+pname);
            QCheckListItem* item = 
                            new QCheckListItem( m_pllist,
                                pname.c_str(), 
                                QCheckListItem::CheckBox );
            item->setText( 1, cmt.c_str() );
            item->setOn( load == "yes" );
        }
    }
}

void ServiceMgrPanel::itemClicked( QListViewItem* item )
{
    QCheckListItem* citem = (QCheckListItem*)item;
    if( citem->isOn() )
    {
        citem->setText( 1, "yes" );
        string pname( citem->text(0).latin1() );
        Registry::setValue( 
                            "/DIVERSE/GServiceManager/services/"+pname,
                            "yes" );
        PluginManager::loadPlugin( pname );
        GServiceManager::startService( pname );     
    }
    else
    {
        citem->setText( 1, "no" );
        string pname( citem->text(0).latin1() );
        Registry::setValue( 
                            "/DIVERSE/GServiceManager/services/"+pname,
                            "no" );
        GServiceManager::stopService( pname );     
        PluginManager::unloadPlugin( pname );
    }
}
