#include "TracerPanel.h"
#include <qlayout.h>
#include <qlabel.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <dreg/Registry.h>
#include <dutil/ColonList.h>
#include <tracer/Tracer.h>
#include <iostream>
#include <qhgroupbox.h>
using namespace std;

TracerPanel::TracerPanel( QWidget* parent )
    :PanelWidget( parent, "Tracer" ) 
{
    QHBoxLayout* lay = new QHBoxLayout( this, 5, 5 );
    QHGroupBox* gbox = new QHGroupBox( this, "Tracer Preferences" );
    gbox->setTitle( "Tracer Preferences" );
    lay->addWidget( gbox );

    QWidget* cont = new QWidget( gbox );
    
    QGridLayout * grid = new QGridLayout(cont, 6, 3, 5, 5);
    
    string warning = Registry::getValue("/DIVERSE/Tracer/warning","5");
    string error = Registry::getValue("/DIVERSE/Tracer/error", "5");
    string debug = Registry::getValue("/DIVERSE/Tracer/debug", "5");
    string stat = Registry::getValue("/DIVERSE/Tracer/stat", "5");
    string trace = Registry::getValue("/DIVERSE/Tracer/trace", "0");

    int t_warn = atoi( warning.c_str() );
    int t_err = atoi( error.c_str() );
    int t_debug = atoi( debug.c_str() );
    int t_stat = atoi( stat.c_str() );
    int t_trace = atoi( trace.c_str() );

    m_toggleWarn = new QCheckBox(cont);
    m_toggleError = new QCheckBox(cont);
    m_toggleDebug = new QCheckBox(cont);
    m_toggleStat = new QCheckBox(cont);
    m_Warning = new QSlider(0, 10, 1, t_warn, Qt::Horizontal, cont, "warning");
    m_Error = new QSlider(0, 10, 1, t_err, Qt::Horizontal, cont, "error");
    m_Debug = new QSlider(0, 10, 1, t_debug, Qt::Horizontal, cont, "debug");
    m_Stat = new QSlider(0, 10, 1, t_stat, Qt::Horizontal, cont, "stat");
    m_Trace = new QCheckBox("Trace", cont);
    m_Trace->setChecked(t_trace);
    
    char* prop = getenv("TRACER_SPEW");
    string properties;
    if( prop != NULL )
        properties = string(getenv("TRACER_SPEW"));
    ColonList parser(properties);
    
    // Tell the Tracer what the stored values were
/*
    editWarning(t_warn);
    editError(t_err);
    editDebug(t_debug);
    editStat(t_stat);
    enableTrace(t_trace);

    if (parser.findItem("WARN") != -1)
        enableWarning(true);
    else
        enableWarning(false);
        
    if (parser.findItem("NOERROR") == -1)
        enableError(false);
    else
        enableError(true);
        
    if (parser.findItem("DEBUG") != -1)
        enableDebug(true);
    else
        enableDebug(false);
        
    if (parser.findItem("STAT") != -1)
        enableStat(true);
    else
        enableStat(false);
*/
    QLabel *warnLabel = new QLabel(cont);
    warnLabel->setText("Warning Level");
    
    QLabel *errorLabel = new QLabel(cont);
    errorLabel->setText("Error Level");
    
    QLabel *debugLabel = new QLabel(cont);
    debugLabel->setText("Debug Level");

    QLabel *statLabel = new QLabel(cont);
    statLabel->setText("Status Level");

    m_Warning->setTickmarks(QSlider::Below);
    m_Error->setTickmarks(QSlider::Below);
    m_Debug->setTickmarks(QSlider::Below);
    m_Stat->setTickmarks(QSlider::Below);

    // You need to add the widgets to the grid
    grid->addWidget( warnLabel,     1, 0 );
    grid->addWidget( m_Warning,     1, 1 );
    grid->addWidget( m_toggleWarn,  1, 2 );
    grid->addWidget( errorLabel,    2, 0 );
    grid->addWidget( m_Error,       2, 1 );
    grid->addWidget( m_toggleError, 2, 2 );
    grid->addWidget( debugLabel,    3, 0 );
    grid->addWidget( m_Debug,       3, 1 );
    grid->addWidget( m_toggleDebug, 3, 2 );
    grid->addWidget( statLabel,     4, 0 );
    grid->addWidget( m_Stat,        4, 1 );
    grid->addWidget( m_toggleStat,  4, 2 );
    grid->addWidget( m_Trace,       5, 0 );
 
    connect(m_Warning, SIGNAL(valueChanged(int)), 
            this, SLOT(editWarning(int)));
    connect(m_Error, SIGNAL(valueChanged(int)),
            this, SLOT(editError(int)));
    connect(m_Debug, SIGNAL(valueChanged(int)),
            this, SLOT(editDebug(int)));
    connect(m_Stat, SIGNAL(valueChanged(int)),
            this, SLOT(editStat(int)));
    connect(m_Trace, SIGNAL(toggled(bool)),
            this, SLOT(enableTrace(bool)));
    connect(m_toggleWarn, SIGNAL(toggled(bool)),
            this, SLOT(enableWarning(bool)));
    connect(m_toggleError, SIGNAL(toggled(bool)),
            this, SLOT(enableError(bool)));
    connect(m_toggleDebug, SIGNAL(toggled(bool)),
            this, SLOT(enableDebug(bool)));
    connect(m_toggleStat, SIGNAL(toggled(bool)),
            this, SLOT(enableStat(bool)));
}

TracerPanel::~TracerPanel()
{

}


PanelWidget* TracerPanel::clone( QWidget* parent )const
{
    return new TracerPanel(parent);
}


void TracerPanel::editWarning(int value)
{
    char buff[3];
    
    sprintf(buff, "%d", value);
    Registry::setValue("/DIVERSE/Tracer/warning", buff);
    Tracer::printWarn(value);
}

void TracerPanel::editError(int value)
{
    char buff[3];
    
    sprintf(buff, "%d", value);
    Registry::setValue("/DIVERSE/Tracer/error", buff);
    Tracer::printError(value);
}

void TracerPanel::editDebug(int value)
{
    char buff[3];

    sprintf(buff, "%d", value);
    Registry::setValue("/DIVERSE/Tracer/debug", buff);
    Tracer::printDebug(value);
}

void TracerPanel::editStat(int value)
{
    char buff[3];

    sprintf(buff, "%d", value); 
    Registry::setValue("/DIVERSE/Tracer/stat", buff);
    Tracer::printStatus(value);
}

void TracerPanel::enableWarning(bool state)
{
    m_Warning->setEnabled(state);
    m_toggleWarn->setChecked(state);

    if (state)
    {
        string warning = Registry::getValue("/DIVERSE/Tracer/warning","5");
        int t_warn = atoi( warning.c_str() );
    
        Tracer::printWarn(t_warn);
    }
    else
    {
        Tracer::printWarn(0);
    }
}

void TracerPanel::enableError(bool state)
{
    m_Error->setEnabled(state);
    m_toggleError->setChecked(state);
    
    if (state)
    {
        string error = Registry::getValue("/DIVERSE/Tracer/error","5");
        int t_error = atoi( error.c_str() );
        
        Tracer::printError(t_error);    
    }
    else
    {
        Tracer::printError(0);
    }
}

void TracerPanel::enableDebug(bool state)
{
    m_Debug->setEnabled(state);
    m_toggleDebug->setChecked(state);
    
    if (state)
    {
        string debug = Registry::getValue("/DIVERSE/Tracer/debug","5");
        int t_debug = atoi( debug.c_str() );
        
        Tracer::printDebug(t_debug);    
    }
    else
    {
        Tracer::printDebug(0);
    }
}

void TracerPanel::enableStat(bool state)
{
    m_Stat->setEnabled(state);
    m_toggleStat->setChecked(state);
    
    if (state)
    {
        string stat = Registry::getValue("/DIVERSE/Tracer/stat","5");
        int t_stat = atoi( stat.c_str() );
        
        Tracer::printStatus(t_stat);    
    }
    else
    {
        Tracer::printStatus(0);
    }
}

void TracerPanel::enableTrace(bool state)
{
    string value;
    
    if (state)
        value = "1";
    else
        value = "0";
        
    Registry::setValue("/DIVERSE/Tracer/trace", value);
    Tracer::printTrace(state);
}



