//=============================================================================
/// @class TracerPanel
/// @brief Allows editing of the Debug print settings
/// @author Charles Keener
//=============================================================================
#ifndef _PLUGINMGRPANEL_H_
#define _PLUGINMGRPANEL_H_
#include<qtutil/PanelWidget.h>
#include<dutil/ColonList.h>

#ifdef WIN32
#ifdef TRACERPANEL_EXPORTS
#define TRACERPANEL_API __declspec(dllexport)
#else
#define TRACERPANEL_API __declspec(dllimport)
#endif /* TRACERPANEL_EXPORTS */
#else
#define TRACERPANEL_API
#endif /* WIN32 */

using namespace std;

class Plugin;
class QCheckBox;
class QSlider;
class QListViewItem;
class PathEditWidget;

class TRACERPANEL_API TracerPanel : public PanelWidget
{
    Q_OBJECT
    public:
        TracerPanel( QWidget* parent = NULL );
        // You forgot to add the destructor
        ~TracerPanel();

        PanelWidget* clone( QWidget* parent = NULL )const;
    
    public slots:
        void editWarning(int value);
        void editError(int value);
        void editDebug(int value);
        void editStat(int value);

        void enableWarning(bool state);
        void enableError(bool state);
        void enableDebug(bool state);
        void enableStat(bool state);
        void enableTrace(bool state);
        
    private:
        QSlider *m_Warning;
        QSlider *m_Error;
        QSlider *m_Debug;
        QSlider *m_Stat;
        QCheckBox *m_Trace;

        QCheckBox *m_toggleWarn;
        QCheckBox *m_toggleError;
        QCheckBox *m_toggleDebug;
        QCheckBox *m_toggleStat;

        int mWarningLevel;
        int mErrorLevel;
        int mDebugLevel;
        int mStatLevel;
        bool mTrace;
};

#endif
