//=============================================================================
// DebugHeadsUp.h
// - Navigation plugin class for wand joystick controller
//
// Created 6/19/03 By Patrick Shinpaugh
//=============================================================================

#ifndef _DEBUGHEADSUP_H_
#define _DEBUGHEADSUP_H_
#include <eventmgr/devents.h>
#include <commander/CommandHandler.h>
#include <eventmgr/EventListener.h>
#include <pluginmgr/Plugin.h>
#include <displaymgr/DWindow.h>
#include <displaymgr/DisplayCallback.h>
#include <dutil/ContextVar.h>

#ifdef WIN32
#ifdef DEBUGHEADSUP_EXPORTS
#define DEBUGHEADSUP_API __declspec(dllexport)
#else
#define DEBUGHEADSUP_API __declspec(dllimport)
#endif /* DEBUGHEADSUP_EXPORTS */
#else
#define DEBUGHEADSUP_API
#endif /* WIN32 */

class dglfont;
class dMatrix;

struct Coord
{
    dVec3 xyz;
    dVec3 hpr;
};

class DEBUGHEADSUP_API DebugHeadsUp: public Plugin,
                                     public DisplayCallback, 
                                     public CommandHandler,
                                     public EventListener
{
    public:
        DebugHeadsUp( const string& name = "DebugHeadsUp" );

        virtual void overlay();
        bool onKeyEvent( DKeyEvent* evt);
        bool initFont();

        void printDirection( dMatrix mat, int column, int row )const;
        void columntext( const char* str, int column, int row )const;
        void columntext( const string& str, int column, int row )const;
        Coord worldToLocal( dVec3 pos, dVec3 dir )const;
		dVec3 vecToDir( dVec3 vector );
    
        CommandOut* execute( const Command& cmd );

        void load();
        void unload();

    private:
        bool m_visible;
        ContextVar<dglfont*> m_curfont;
};

#endif
