//=============================================================================
// DGL_HUD_Display.cpp
// - Navigation plugin class for wand joystick controller
//
// Created 6/19/03 By Patrick Shinpaugh
//=============================================================================

#include "GnomonHeadsUp.h"
#include <tracer/Tracer.h>
#include <dmath/dMatrix.h>
#include <navmgr/NavManager.h>
#include <navmgr/NavInterface.h>
#include <displaymgr/DisplayManager.h>
#include <math.h>
#include <GL/gl.h>

REGISTER_PLUGIN(GnomonHeadsUp);

#define DEG2RAD(x)  ((float) ((x)*M_PI/180.0f))
#define SIZE 40.0f

using namespace std;

GnomonHeadsUp::GnomonHeadsUp( const string& name )
:Plugin( name )
{
    setDescription( "Displays the navigation orientation in the lower right corner." );
}

void GnomonHeadsUp::load()
{
    Plugin::load();
    DisplayManager::addOverlayCallback( this );
}

void GnomonHeadsUp::unload()
{
    DisplayManager::removeOverlayCallback( this );
    Plugin::unload();
}

void GnomonHeadsUp::overlay()
{
    int winsize[4];
    glGetIntegerv( GL_VIEWPORT, winsize );
    float width = winsize[2], height = winsize[3];

    glMatrixMode( GL_PROJECTION );
    glPushMatrix();
    glLoadIdentity();
    glOrtho( 0.0f, width, 0.0f, height, -1.0f, 1.0f );
    glMatrixMode( GL_MODELVIEW );
    glPushMatrix();
    glLoadIdentity();

    glPushAttrib( GL_ALL_ATTRIB_BITS );
    glDisable( GL_LIGHTING );
    glDisable( GL_BLEND );
	glDisable( GL_TEXTURE_1D );
	glDisable( GL_TEXTURE_2D );
#ifdef GL_TEXTURE_3D
	glDisable( GL_TEXTURE_3D );
#endif
    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
    glLineWidth( 2.0f );
    
    glScalef( SIZE, SIZE, 1.0f );
    glTranslatef( width/SIZE - 1.0f, 1.0f, 0.0f);
    glPushMatrix();

    dVec3 x(1.0f, 0.0f, 0.0f), y(0.0f, 0.0f, 1.0f), z(0.0f, 1.0f, 0.0f);

    dMatrix mat = NavManager::getMatrix();
    mat.setTrans( 0.0f, 0.0f, 0.0f );
	dMatrix rot;
	rot = rot.rotate( -DEG2RAD( 90.0f ), 1.0f, 0.0f, 0.0f );
	mat *= rot;
    float* p_mat = mat.inverse( mat ).ptr();
    glMultMatrixf( p_mat );

    glColor3f(1.0f, 0.0f, 0.0f) ;
    glBegin(GL_LINES) ;
    glVertex3f(0.f, 0.f, 0.f) ;
    glVertex3f(x.x(), x.z(), -x.y()) ;

    glColor3f(0.0f, 1.0f, 0.0f) ;
    glVertex3f(0.f, 0.f, 0.f) ;
    glVertex3f(y.x(), y.z(), -y.y()) ;

    glColor3f(0.0f, 0.0f, 1.0f) ;
    glVertex3f(0.f, 0.f, 0.f) ;
    glVertex3f(z.x(), z.z(), -z.y()) ;
    glEnd();

    glPopMatrix();

    float angle;
    glColor3f(1.0f, 1.0f, 1.0f) ;
    glBegin(GL_LINE_LOOP);
    for (angle = 0; angle <= 360; angle += 5)
    {
      glVertex2f(cosf(DEG2RAD(angle)), sinf(DEG2RAD(angle)));
    }
    glEnd();

    glEnable(GL_BLEND) ;
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;
    glColor4f(1.f, 1.f, 1.f, .5f) ;
    glBegin(GL_POLYGON);
    for (angle = 0; angle <= 360; angle += 5)
    {
      glVertex3f(cosf(DEG2RAD(angle)), sinf(DEG2RAD(angle)), 0.0f);
    }
    glEnd();

    glPopAttrib() ;
    glPopMatrix();
    glMatrixMode( GL_PROJECTION );
    glPopMatrix();
    glMatrixMode( GL_MODELVIEW );
}
