//=============================================================================
// mouseNavAid.cpp
// - Navigation plugin class to warp cursor to center of screen on mouse press
//     and display crosshair and pointer
//
// Created 6/29/03 By Patrick Shinpaugh
//=============================================================================

#include "mouseNavAid.h"
#include <tracer/Tracer.h>
#include <dmath/dMatrix.h>
#include <navmgr/NavManager.h>
#include <navmgr/NavInterface.h>
#include <eventmgr/devents.h>
#include <eventmgr/EventListener.h>
#include <fontmgr/FontManager.h>
#include <fontmgr/dglfont.h>
#include <dgl/dglcore/DGL.h>
#include <dreg/Registry.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#ifndef WIN32
#include <GL/glx.h>
#endif /* !WIN32 */

REGISTER_PLUGIN(mouseNavAid);

#define DEG2RAD(x)  ((float) ((x)*M_PI/180.0f))
#define SIZE 40.0f

using namespace std;

mouseNavAid::mouseNavAid( const string& name )
:DGLAugment( name )
{
    setDescription( "Mouse navigation aid - center cursor on window and display crosshair" );
    EventManager::addMouseListener( this );
    m_transdeadzone = atoi( Registry::getValue( "/mouseNavAid/TransDeadZone", "10").c_str()
);
    m_rotdeadzone = atoi( Registry::getValue( "/mouseNavAid/RotDeadZone", "10").c_str() );
    Tracer::debug( "mouseNavAid constructor completed" );
}

bool mouseNavAid::onMouseEvent( DMouseEvent* evt )
{
    int winsize[4];
    switch ( evt->action() )
    {
        case DMouseEvent::PRESS:
        {
            m_visible = true;
            glGetIntegerv( GL_VIEWPORT, winsize );
            GLenum err = glGetError();
            if ( err )
            {
                Tracer::error( "There was a problem retrieving GL context data"
                        "GL error - %s", gluErrorString( err ) );
                return false;
            }
            m_mousex = evt->x();
            m_mousey = evt->y();
//===============================================================
// The following was to be implemented to center the cursor in
// the window. Unfortunately, to implement could result in
// undefined bahavior. Not implemented currently.
// If implemented like this, will require GLX 1.3 or greater.
//===============================================================
/*            Display* display = glXGetCurrentDisplay();
            GLXDrawable drawable = glXGetCurrentDrawable();
            XWarpPointer( display, drawable, drawable, winsize[0], winsize[1], winsize[2],
                    winsize[3], winsize[0] + winsize[2] / 2, winsize[1] + winsize[3] / 2 );
            DMouseEvent newevt(  );*/
            break;
        }
        case DMouseEvent::RELEASE:
            m_visible = false;
            break;
        case DMouseEvent::MOVE:
            break;
    }
    return false;
}

void mouseNavAid::postdraw()const
{
    if ( !m_visible )
        return;
    glMatrixMode( GL_PROJECTION );
    glPushMatrix();
    glLoadIdentity();
    
    int winsize[4];
    glGetIntegerv( GL_VIEWPORT, winsize );
    float width = winsize[2], height = winsize[3];

    glOrtho( 0.0f, width, height, 0.0f, -100.0f, 100.0f );
    glMatrixMode( GL_MODELVIEW );
    glPushMatrix();
    glLoadIdentity();

    glPushAttrib( GL_ALL_ATTRIB_BITS );
    glDisable( GL_LIGHTING );
    glLineWidth( 1.0f );

    glColor3f( 1.0f, 1.0f, 1.0f ) ;
    glBegin( GL_LINE_STRIP );
    glVertex2f( m_mousex - m_rotdeadzone, m_mousey - m_transdeadzone );
    glVertex2f( m_mousex + m_rotdeadzone, m_mousey - m_transdeadzone );
    glVertex2f( m_mousex + m_rotdeadzone, m_mousey + m_transdeadzone );
    glVertex2f( m_mousex - m_rotdeadzone, m_mousey + m_transdeadzone );
    glVertex2f( m_mousex - m_rotdeadzone, m_mousey - m_transdeadzone );
    glEnd();

    glBegin( GL_LINES );
    glVertex2f( width / 2 + m_rotdeadzone, height / 2 );
    glVertex2f( width / 2 - m_rotdeadzone, height / 2 );
    glVertex2f( width / 2, height / 2 + m_transdeadzone );
    glVertex2f( width / 2, height / 2 - m_transdeadzone );
    glEnd();
    
    glPopAttrib() ;
    glPopMatrix();
    glMatrixMode( GL_PROJECTION );
    glPopMatrix();
    glMatrixMode( GL_MODELVIEW );
}

