//=============================================================================
// WandButtonNavControl.cpp
// - Navigation plugin class for wand joystick controller
//
// Created 6/30/03 By Patrick Shinpaugh
//=============================================================================
#include "WandButtonNavControl.h"
#include <tracer/Tracer.h>
#include <eventmgr/devents.h>
#include <navmgr/NavManager.h>
#include <dreg/Registry.h>
#include <dgl/dglcore/DGL.h>

REGISTER_PLUGIN(WandButtonNavControl);

using namespace std;

WandButtonNavControl::WandButtonNavControl( const string& name )
:DGLAugment( name )
{
    EventManager::addWandListener( this );
    setDescription( "Button control for wand joystick navigation" );
    m_speedfactor = atof( Registry::getValue( "/wandJoystickNav/Speed", "2.0").c_str() );

    Tracer::debug( "WandButtonNavControl constructor completed" );
}

bool WandButtonNavControl::onWandEvent( DWandEvent* evt)
{
    if ( evt->action() == DWandEvent::BUTTONPRESS )
    {
        bool button[4];
        for ( int i=0;i<4;i++ )
            button[i] = evt->getButton( i );
        if ( button[0] && !button[1] && !button[2] && !button[3])// button 1 pressed
            NavManager::setSpeedFactor( NavManager::getSpeedFactor() / 2.0f );
        else if ( !button[0] && !button[1] && button[2] && !button[3])// button 3 pressed
            NavManager::setSpeedFactor( NavManager::getSpeedFactor() * 2.0f );
        else if ( button[0] && button[1] && !button[2] && !button[3])// button 1 & 2 pressed
        {
            dMatrix m = NavManager::getMatrix();
            m.setTrans( 0,0,0 );
            NavManager::setMatrix( m );
        }
        else if ( !button[0] && button[1] && button[2] && !button[3])// button 2 & 3 pressed
            NavManager::nextNavigation();
        else if ( button[0] && !button[1] && button[2] && !button[3])// button 1 & 3 pressed
            NavManager::setSpeedFactor( m_speedfactor );
    }
    return false;
}
