#ifndef _cserverservice_h_
#define _cserverservice_h_

#include <servicemgr/GService.h>
#include <pexpr/POAddress.h>
#include <pexpr/POTextMsg.h>
#include <dutil/ColonList.h>
#include <pluginmgr/Plugin.h>
#include <dcollab/core/handler.h>
#include <dcollab/core/communicator.h>
#include <iostream>

//Windows import / export stuff
#ifdef WIN32
#ifdef CSERVERSERVICE_EXPORTS
#define CSERVERSERVICE_API __declspec(dllexport)
#else
#define CSERVERSERVICE_API __declspec(dllimport)
#endif /* CSERVERSERVICE_EXPORTS */
#else
#define CSERVERSERVICE_API
#endif /* WIN32 */

using namespace std;
class POMessage;
class MessageHandler;
class Communicator;
class CSERVERSERVICE_API CServerService : public Plugin, public GService
{
	public:
	CServerService();	
	virtual ~CServerService();	

	//Post process
	virtual void postProcess(const DTime&)
	{
		//Temp storage
		vector<POMessage*> TMessages;
		//Get messages
		TMessages = m_com->getMessages();
		//Go through the messages and see if I can handle it
		for (unsigned int i=0; i< TMessages.size(); i++)
		{
			if (m_handlers->canHandle(TMessages[i]))
				m_handlers->handleMessage(TMessages[i]);
			else
				forwardMessage(TMessages[i]);
		}
	};

	//Load and unload plugins
	virtual void load()
	{
		Plugin::load();
		m_com = new Communicator;
		m_handlers = new Handler;
	};
	virtual void unload()
	{
		Plugin::unload();
	};

	//Message handler functions
	void addMHandler(MessageHandler* newHandler){m_handlers->addHandler(newHandler);};
	void removeMHandler(MessageHandler* badHandler){	m_handlers->removeHandler(badHandler);};
	Handler* getHandler(){	return m_handlers;};

	//Generic function for forwarding a message to the clients
	void  forwardMessage(POMessage* AMessage)
	{
		//cout << "=========Begin forward=========" << endl;
		for (unsigned int i=0; i< m_list.size(); i++)
		{
			if (m_list[i] != AMessage->getSource())
			{
				//m_list[i].debug();
				m_com->setTo(m_list[i]);
				m_com->sendMessage(AMessage);
			}
		}
		//cout << "=========End forward=========" << endl;
	};

	//Get the communicator object to handle messages
	Communicator*  getCom(){return m_com;};

	//Address management functions
	void addAddress(POAddress Info)
	{
		m_list.push_back(Info);
	};
	void removeAddress(POAddress Info)
	{
		for (unsigned int i=0; i<m_list.size(); i++)
		{
			if (m_list[i] == Info)
			{
				m_list.erase(m_list.begin() + i);
			}
		}
	};
	vector<POAddress> getAddresses(){return m_list;};
	void addName(const string& NewName) { m_names.push_back(NewName);};
	void removeName(const string& Name)
	{
		for (unsigned int i=0; i< m_names.size(); i++)
		{
			if (m_names[i] == Name)
				m_names.erase(m_names.begin()  + i );
		}
	};

	void addPair(const string& Name , POAddress Loc)
	{
		//cout << "Server is adding " << Name << endl;
		//cout << "With location ";
		Loc.debug();
		m_names.push_back(Name);
		m_list.push_back(Loc);
	};

	private:
	vector<POAddress> m_list;
	vector<string>    m_names;
	Communicator* m_com;
	Handler*      m_handlers;
};
#endif
