#include <dtk.h>
#include <tracer/Tracer.h>
#include <eventmgr/EventManager.h>
#include <tracer/Tracer.h>
#include "dtkwt.h"
#include<dmath/dMatrix.h>
#include<servicemgr/GServiceManager.h>


dtkwt::dtkwt():GService("dtkwt")
{
	int i;
    Tracer::init();
    GServiceManager::addService(this);
    //Create the memory segments for the head, wand, buttons, and joystick
	m_wand_segment= new dtkSharedMem(sizeof(float) * 6, "wand");
	m_button_segment = new dtkSharedMem(sizeof(char) * 1, "buttons");
	m_joystick_segment= new dtkSharedMem(sizeof(float) * 2, "joystick");
    //Initalize the class arrays to zero
	for (i=0; i<6; i++)
	{
		m_tempw[i] = 0;
        m_wand[i] = 0;
	}
    for (i=0; i<4; i++)
    {
		m_tempb[i] = 0;
        m_buttons[i] = 0;
    }
    for (i=0; i<2; i++)
    {
		m_tempj[i] = 0;
        m_joystick[i] = 0;
    }
	if (m_wand_segment ==NULL || m_button_segment == NULL || m_joystick_segment ==NULL)
		Tracer::error("Could not connect to the head or the wand, do not trust program results");
}

void dtkwt::postProcess()
{
	int i;
    unsigned char READ=0;
	m_wand_segment->read(m_tempw);
	m_button_segment->read(&READ);
	m_joystick_segment->read(m_tempj);
    
    if( m_wand_segment->differentByteOrder() )
    {
        for( unsigned int i = 0 ; i < 6; ++i )
        {
            dtk_swapBytes( &m_tempw[i], sizeof(float) );
        }
    }
    if( m_joystick_segment->differentByteOrder() )
    {
        for( unsigned int i = 0 ; i < 2; ++i )
        {
            dtk_swapBytes( &m_tempj[i], sizeof(float) );
        }
    }
    
    //Clear out the first four bits of memory for the button
    for (i=0; i<4; i++)
    {
        m_tempb[i] = (&READ)[i/8] & (01 << (i %8)); 
    }

	bool Diff = false;
    //Check to see if the user moved the wand
	for (i=0; i<6; i++)
	{
		if  (m_tempw[i] != m_wand[i])
			Diff = true;
	}
    //The xyz and hpr of the wand 
	dVec3 XYZ;
	dVec3 HPR;
	for (i=0; i<3; i++)
	{
		XYZ[i] = m_tempw[i];
		HPR[i] = m_tempw[i+3];
	}
    //Check to see if a button was pressed
	for (i=0; i<4; i++)
	{
        if (m_buttons[i] != m_tempb[i])
        {
            if (m_buttons[i] == 0)
            {
                Tracer::stat("Button pressed");
                EventManager::pushEvent(MakeEvent(XYZ, HPR, DWandEvent::BUTTONPRESS, B_PRESS));
            }
            else
            {
                Tracer::stat("Button released");
                EventManager::pushEvent(MakeEvent(XYZ, HPR, DWandEvent::BUTTONRELEASE, B_RELEASE));
            }
        }
	}
    for (i=0; i<4; i++)
        m_buttons[i] = m_tempb[i];

    //The user moved the wand, report status
	if (Diff)
    {
//        Tracer::stat("Wand moved from previous position");
        EventManager::pushEvent(MakeEvent(XYZ, HPR, DWandEvent::TRACK, WAND_MOVEMENT));
    }
    //Copy the wand data over that we read
	for (i=0; i<6; i++)
		m_wand[i] = m_tempw[i];
    //Finally check the joystick
    for (i=0; i<2; i++)
        if (m_joystick[i] != m_tempj[i])
        {
//            Tracer::stat("Joystick moved from previous position");
            EventManager::pushEvent(MakeEvent(XYZ, HPR, DWandEvent::JOYSTICKMOVE, JOYSTICK_MOVE));
        }
    for (i=0; i<2; i++)
        m_joystick[i] = m_tempj[i];
}

DWandEvent* dtkwt::MakeEvent(dVec3& XYZ,  dVec3& HPR,  DWandEvent::ACTION Type,  const string& Description)
{
    //Construct the basic event
    dVec3 hpr = HPR; //dVec3( DEG2RAD(HPR.h()), DEG2RAD(HPR.p()), DEG2RAD(HPR.z()) );
    DWandEvent* Temp = new DWandEvent(Type, Description, XYZ, hpr);
    Temp->setNumButtons(4);
    //Set the button data for the event
    for (int i=0; i<4; i++)
    {
        if (m_tempb[i])
            Temp->setButton(i,true);
        else
            Temp->setButton(i,false); 
    }
    //Set the position for the joystick
    Temp->setJoystickPos(m_tempj[0], m_tempj[1]);
    //Return the event
    return Temp;
}
