/*
 * The DIVERSE Toolkit
 * Copyright (C) 2000, 2001, 2002  Lance Arsenault
 * 
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License (GPL) as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software, in the top level source directory in a file
 * named "COPYING.GPL"; if not, see it at:
 * http://www.gnu.org/copyleft/gpl.html or write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 *
 */

/********************************************************************/
/**************** EDIT TO CONFIGURE *********************************/
/********************************************************************/

/* DEVICE_FILE is the serial port device file that the IS900 control
 * box is plugged to your computer via RS232 serial line. */

#define DEVICE_FILE   "/dev/ttyd10"

/********************************************************************/
/**************** END --  EDIT TO CONFIGURE *************************/
/********************************************************************/

#ifdef Linux

#  define SWAPBYTES     0             /* 0 or 1 = no or yes */
#  define DEFAULT_DEVICE_FILE  "/dev/ttyS0"   /* change me */

#else  /* IRIX or other like. */

#  define SWAPBYTES     1             /* 0 or 1 = no or yes */
#  define DEFAULT_DEVICE_FILE  "/dev/ttyd10"  /* change me */

#endif

#ifndef DEVICE_FILE
#  define DEVICE_FILE DEFAULT_DEVICE_FILE
#endif

#define NUM_STATIONS  2   /* number of stations (trackers) read */
#define HEAD_RECORDLEN ((size_t) 29)
#define WAND_RECORDLEN ((size_t) 32)

static char setupstring[] =
"f\n"                  // binary output
"u\n"                  // metric units
"l1,1\n"               // tracker 1 on
"l2,1\n"               // tracker 2 on
"l3,0\n"               // tracker 3 off
"l4,0\n"               // tracker 4 off
"R1\n"
"R2\n"
// Alignment Reference Frame oriention = switch x to y and y to x
"A1,0,0,0,1,0,0,0,1,0\n" // Alignment Reference Frame oriention
"A2,0,0,0,1,0,0,0,1,0\n" // Alignment Reference Frame oriention
"G1,0,0,0\n"
"G2,0,0,0\n"
"O1,2,4,1\n"           // set tracker 1 "head" output configuration
"O2,2,4,22,23,1\n"     // set tracker 2 "wand" output configuration
"ML0\n"                // turn off blue LEDs
"SC\n";                // set continuous spew

#define BUFFER_SIZE   ((size_t) 1023) /* read buffer size */

/**********************************************************************
  read a record from the tracker.  returns 1 if data updated,
  0 if no change, and -1 if an error
 **********************************************************************/

#define CHUCK  ((size_t) 120)  /* about the largest buffer read size */

/* data record offsets */

#define X 3
#define Y 7
#define Z 11


#define H 15 /* heading */
#define P 19 /* pitch */
#define R 23 /* roll */

#define BUTTONS 27
#define JOYSTICK 28

